{% comment %}
SPDX-FileCopyrightText: 2025 Helmholtz-Zentrum hereon GmbH

SPDX-License-Identifier: EUPL-1.2
{% endcomment %}

<!DOCTYPE html>
<html>
<head>
    <title>DASF Broker demo page</title>
</head>
<body>
    {% block content %}{% endblock %}
    <footer>
      {% block footer %}{% endblock %}
    </footer>
</body>
</html>
