{% extends "base.html" %}
{% comment %}
SPDX-FileCopyrightText: 2025 Helmholtz-Zentrum hereon GmbH

SPDX-License-Identifier: EUPL-1.2
{% endcomment %}

{% load static %}
{% load dasf %}

{% block content %}

  {% if brokertopic.api_info %}
  {{ brokertopic.api_info|json_script:"api-info" }}
  {% endif %}

   <dasf-module
     websocket-url="{% dasf_ws_url %}" {% if brokertopic.api_info %}api-info-element="api-info"{% endif %}
     topic="{{ brokertopic.slug }}"{% for key, val in module_params.items %} {{ key }}="{{ val }}"{% endfor %}>

{% endblock content %}

{% block footer %}
  <script type="module" src="{% static 'dasf_frontend/dasf-web-component/dist/wc/dasf.js' %}"></script>
{% endblock footer %}
