# SPDX-FileCopyrightText: 2022-2025 Helmholtz-Zentrum hereon GmbH
#
# SPDX-License-Identifier: EUPL-1.2

"""Template tags
-------------

This module contains the :mod:`~dasf_broker.templatetags.dasf` module that
defines template tags that should be used within DASF.
"""
