var C5 = Object.defineProperty;
var x5 = (e, t, n) => t in e ? C5(e, t, { enumerable: !0, configurable: !0, writable: !0, value: n }) : e[t] = n;
var yu = (e, t, n) => x5(e, typeof t != "symbol" ? t + "" : t, n);
function E5(e, t) {
  for (var n = 0; n < t.length; n++) {
    const o = t[n];
    if (typeof o != "string" && !Array.isArray(o)) {
      for (const i in o)
        if (i !== "default" && !(i in e)) {
          const s = Object.getOwnPropertyDescriptor(o, i);
          s && Object.defineProperty(e, i, s.get ? s : {
            enumerable: !0,
            get: () => o[i]
          });
        }
    }
  }
  return Object.freeze(Object.defineProperty(e, Symbol.toStringTag, { value: "Module" }));
}
var Tu = typeof globalThis < "u" ? globalThis : typeof window < "u" ? window : typeof global < "u" ? global : typeof self < "u" ? self : {};
function Ft(e) {
  return e && e.__esModule && Object.prototype.hasOwnProperty.call(e, "default") ? e.default : e;
}
var u$ = { exports: {} }, xt = {};
/**
 * @license React
 * react.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var zb;
function _5() {
  if (zb) return xt;
  zb = 1;
  var e = Symbol.for("react.element"), t = Symbol.for("react.portal"), n = Symbol.for("react.fragment"), o = Symbol.for("react.strict_mode"), i = Symbol.for("react.profiler"), s = Symbol.for("react.provider"), l = Symbol.for("react.context"), u = Symbol.for("react.forward_ref"), c = Symbol.for("react.suspense"), f = Symbol.for("react.memo"), p = Symbol.for("react.lazy"), g = Symbol.iterator;
  function h(T) {
    return T === null || typeof T != "object" ? null : (T = g && T[g] || T["@@iterator"], typeof T == "function" ? T : null);
  }
  var y = { isMounted: function() {
    return !1;
  }, enqueueForceUpdate: function() {
  }, enqueueReplaceState: function() {
  }, enqueueSetState: function() {
  } }, v = Object.assign, C = {};
  function $(T, H, G) {
    this.props = T, this.context = H, this.refs = C, this.updater = G || y;
  }
  $.prototype.isReactComponent = {}, $.prototype.setState = function(T, H) {
    if (typeof T != "object" && typeof T != "function" && T != null) throw Error("setState(...): takes an object of state variables to update or a function which returns an object of state variables.");
    this.updater.enqueueSetState(this, T, H, "setState");
  }, $.prototype.forceUpdate = function(T) {
    this.updater.enqueueForceUpdate(this, T, "forceUpdate");
  };
  function S() {
  }
  S.prototype = $.prototype;
  function w(T, H, G) {
    this.props = T, this.context = H, this.refs = C, this.updater = G || y;
  }
  var _ = w.prototype = new S();
  _.constructor = w, v(_, $.prototype), _.isPureReactComponent = !0;
  var b = Array.isArray, k = Object.prototype.hasOwnProperty, E = { current: null }, j = { key: !0, ref: !0, __self: !0, __source: !0 };
  function L(T, H, G) {
    var N, U = {}, Z = null, me = null;
    if (H != null) for (N in H.ref !== void 0 && (me = H.ref), H.key !== void 0 && (Z = "" + H.key), H) k.call(H, N) && !j.hasOwnProperty(N) && (U[N] = H[N]);
    var ue = arguments.length - 2;
    if (ue === 1) U.children = G;
    else if (1 < ue) {
      for (var Ce = Array(ue), $e = 0; $e < ue; $e++) Ce[$e] = arguments[$e + 2];
      U.children = Ce;
    }
    if (T && T.defaultProps) for (N in ue = T.defaultProps, ue) U[N] === void 0 && (U[N] = ue[N]);
    return { $$typeof: e, type: T, key: Z, ref: me, props: U, _owner: E.current };
  }
  function B(T, H) {
    return { $$typeof: e, type: T.type, key: H, ref: T.ref, props: T.props, _owner: T._owner };
  }
  function O(T) {
    return typeof T == "object" && T !== null && T.$$typeof === e;
  }
  function V(T) {
    var H = { "=": "=0", ":": "=2" };
    return "$" + T.replace(/[=:]/g, function(G) {
      return H[G];
    });
  }
  var W = /\/+/g;
  function q(T, H) {
    return typeof T == "object" && T !== null && T.key != null ? V("" + T.key) : H.toString(36);
  }
  function Q(T, H, G, N, U) {
    var Z = typeof T;
    (Z === "undefined" || Z === "boolean") && (T = null);
    var me = !1;
    if (T === null) me = !0;
    else switch (Z) {
      case "string":
      case "number":
        me = !0;
        break;
      case "object":
        switch (T.$$typeof) {
          case e:
          case t:
            me = !0;
        }
    }
    if (me) return me = T, U = U(me), T = N === "" ? "." + q(me, 0) : N, b(U) ? (G = "", T != null && (G = T.replace(W, "$&/") + "/"), Q(U, H, G, "", function($e) {
      return $e;
    })) : U != null && (O(U) && (U = B(U, G + (!U.key || me && me.key === U.key ? "" : ("" + U.key).replace(W, "$&/") + "/") + T)), H.push(U)), 1;
    if (me = 0, N = N === "" ? "." : N + ":", b(T)) for (var ue = 0; ue < T.length; ue++) {
      Z = T[ue];
      var Ce = N + q(Z, ue);
      me += Q(Z, H, G, Ce, U);
    }
    else if (Ce = h(T), typeof Ce == "function") for (T = Ce.call(T), ue = 0; !(Z = T.next()).done; ) Z = Z.value, Ce = N + q(Z, ue++), me += Q(Z, H, G, Ce, U);
    else if (Z === "object") throw H = String(T), Error("Objects are not valid as a React child (found: " + (H === "[object Object]" ? "object with keys {" + Object.keys(T).join(", ") + "}" : H) + "). If you meant to render a collection of children, use an array instead.");
    return me;
  }
  function X(T, H, G) {
    if (T == null) return T;
    var N = [], U = 0;
    return Q(T, N, "", "", function(Z) {
      return H.call(G, Z, U++);
    }), N;
  }
  function ce(T) {
    if (T._status === -1) {
      var H = T._result;
      H = H(), H.then(function(G) {
        (T._status === 0 || T._status === -1) && (T._status = 1, T._result = G);
      }, function(G) {
        (T._status === 0 || T._status === -1) && (T._status = 2, T._result = G);
      }), T._status === -1 && (T._status = 0, T._result = H);
    }
    if (T._status === 1) return T._result.default;
    throw T._result;
  }
  var J = { current: null }, ie = { transition: null }, re = { ReactCurrentDispatcher: J, ReactCurrentBatchConfig: ie, ReactCurrentOwner: E };
  function Y() {
    throw Error("act(...) is not supported in production builds of React.");
  }
  return xt.Children = { map: X, forEach: function(T, H, G) {
    X(T, function() {
      H.apply(this, arguments);
    }, G);
  }, count: function(T) {
    var H = 0;
    return X(T, function() {
      H++;
    }), H;
  }, toArray: function(T) {
    return X(T, function(H) {
      return H;
    }) || [];
  }, only: function(T) {
    if (!O(T)) throw Error("React.Children.only expected to receive a single React element child.");
    return T;
  } }, xt.Component = $, xt.Fragment = n, xt.Profiler = i, xt.PureComponent = w, xt.StrictMode = o, xt.Suspense = c, xt.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED = re, xt.act = Y, xt.cloneElement = function(T, H, G) {
    if (T == null) throw Error("React.cloneElement(...): The argument must be a React element, but you passed " + T + ".");
    var N = v({}, T.props), U = T.key, Z = T.ref, me = T._owner;
    if (H != null) {
      if (H.ref !== void 0 && (Z = H.ref, me = E.current), H.key !== void 0 && (U = "" + H.key), T.type && T.type.defaultProps) var ue = T.type.defaultProps;
      for (Ce in H) k.call(H, Ce) && !j.hasOwnProperty(Ce) && (N[Ce] = H[Ce] === void 0 && ue !== void 0 ? ue[Ce] : H[Ce]);
    }
    var Ce = arguments.length - 2;
    if (Ce === 1) N.children = G;
    else if (1 < Ce) {
      ue = Array(Ce);
      for (var $e = 0; $e < Ce; $e++) ue[$e] = arguments[$e + 2];
      N.children = ue;
    }
    return { $$typeof: e, type: T.type, key: U, ref: Z, props: N, _owner: me };
  }, xt.createContext = function(T) {
    return T = { $$typeof: l, _currentValue: T, _currentValue2: T, _threadCount: 0, Provider: null, Consumer: null, _defaultValue: null, _globalName: null }, T.Provider = { $$typeof: s, _context: T }, T.Consumer = T;
  }, xt.createElement = L, xt.createFactory = function(T) {
    var H = L.bind(null, T);
    return H.type = T, H;
  }, xt.createRef = function() {
    return { current: null };
  }, xt.forwardRef = function(T) {
    return { $$typeof: u, render: T };
  }, xt.isValidElement = O, xt.lazy = function(T) {
    return { $$typeof: p, _payload: { _status: -1, _result: T }, _init: ce };
  }, xt.memo = function(T, H) {
    return { $$typeof: f, type: T, compare: H === void 0 ? null : H };
  }, xt.startTransition = function(T) {
    var H = ie.transition;
    ie.transition = {};
    try {
      T();
    } finally {
      ie.transition = H;
    }
  }, xt.unstable_act = Y, xt.useCallback = function(T, H) {
    return J.current.useCallback(T, H);
  }, xt.useContext = function(T) {
    return J.current.useContext(T);
  }, xt.useDebugValue = function() {
  }, xt.useDeferredValue = function(T) {
    return J.current.useDeferredValue(T);
  }, xt.useEffect = function(T, H) {
    return J.current.useEffect(T, H);
  }, xt.useId = function() {
    return J.current.useId();
  }, xt.useImperativeHandle = function(T, H, G) {
    return J.current.useImperativeHandle(T, H, G);
  }, xt.useInsertionEffect = function(T, H) {
    return J.current.useInsertionEffect(T, H);
  }, xt.useLayoutEffect = function(T, H) {
    return J.current.useLayoutEffect(T, H);
  }, xt.useMemo = function(T, H) {
    return J.current.useMemo(T, H);
  }, xt.useReducer = function(T, H, G) {
    return J.current.useReducer(T, H, G);
  }, xt.useRef = function(T) {
    return J.current.useRef(T);
  }, xt.useState = function(T) {
    return J.current.useState(T);
  }, xt.useSyncExternalStore = function(T, H, G) {
    return J.current.useSyncExternalStore(T, H, G);
  }, xt.useTransition = function() {
    return J.current.useTransition();
  }, xt.version = "18.3.1", xt;
}
u$.exports = _5();
var z = u$.exports;
const lo = /* @__PURE__ */ Ft(z), Zu = /* @__PURE__ */ E5({
  __proto__: null,
  default: lo
}, [z]);
var c$ = { exports: {} }, Ir = {}, Pm = { exports: {} }, Om = {};
/**
 * @license React
 * scheduler.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var Bb;
function k5() {
  return Bb || (Bb = 1, function(e) {
    function t(ie, re) {
      var Y = ie.length;
      ie.push(re);
      e: for (; 0 < Y; ) {
        var T = Y - 1 >>> 1, H = ie[T];
        if (0 < i(H, re)) ie[T] = re, ie[Y] = H, Y = T;
        else break e;
      }
    }
    function n(ie) {
      return ie.length === 0 ? null : ie[0];
    }
    function o(ie) {
      if (ie.length === 0) return null;
      var re = ie[0], Y = ie.pop();
      if (Y !== re) {
        ie[0] = Y;
        e: for (var T = 0, H = ie.length, G = H >>> 1; T < G; ) {
          var N = 2 * (T + 1) - 1, U = ie[N], Z = N + 1, me = ie[Z];
          if (0 > i(U, Y)) Z < H && 0 > i(me, U) ? (ie[T] = me, ie[Z] = Y, T = Z) : (ie[T] = U, ie[N] = Y, T = N);
          else if (Z < H && 0 > i(me, Y)) ie[T] = me, ie[Z] = Y, T = Z;
          else break e;
        }
      }
      return re;
    }
    function i(ie, re) {
      var Y = ie.sortIndex - re.sortIndex;
      return Y !== 0 ? Y : ie.id - re.id;
    }
    if (typeof performance == "object" && typeof performance.now == "function") {
      var s = performance;
      e.unstable_now = function() {
        return s.now();
      };
    } else {
      var l = Date, u = l.now();
      e.unstable_now = function() {
        return l.now() - u;
      };
    }
    var c = [], f = [], p = 1, g = null, h = 3, y = !1, v = !1, C = !1, $ = typeof setTimeout == "function" ? setTimeout : null, S = typeof clearTimeout == "function" ? clearTimeout : null, w = typeof setImmediate < "u" ? setImmediate : null;
    typeof navigator < "u" && navigator.scheduling !== void 0 && navigator.scheduling.isInputPending !== void 0 && navigator.scheduling.isInputPending.bind(navigator.scheduling);
    function _(ie) {
      for (var re = n(f); re !== null; ) {
        if (re.callback === null) o(f);
        else if (re.startTime <= ie) o(f), re.sortIndex = re.expirationTime, t(c, re);
        else break;
        re = n(f);
      }
    }
    function b(ie) {
      if (C = !1, _(ie), !v) if (n(c) !== null) v = !0, ce(k);
      else {
        var re = n(f);
        re !== null && J(b, re.startTime - ie);
      }
    }
    function k(ie, re) {
      v = !1, C && (C = !1, S(L), L = -1), y = !0;
      var Y = h;
      try {
        for (_(re), g = n(c); g !== null && (!(g.expirationTime > re) || ie && !V()); ) {
          var T = g.callback;
          if (typeof T == "function") {
            g.callback = null, h = g.priorityLevel;
            var H = T(g.expirationTime <= re);
            re = e.unstable_now(), typeof H == "function" ? g.callback = H : g === n(c) && o(c), _(re);
          } else o(c);
          g = n(c);
        }
        if (g !== null) var G = !0;
        else {
          var N = n(f);
          N !== null && J(b, N.startTime - re), G = !1;
        }
        return G;
      } finally {
        g = null, h = Y, y = !1;
      }
    }
    var E = !1, j = null, L = -1, B = 5, O = -1;
    function V() {
      return !(e.unstable_now() - O < B);
    }
    function W() {
      if (j !== null) {
        var ie = e.unstable_now();
        O = ie;
        var re = !0;
        try {
          re = j(!0, ie);
        } finally {
          re ? q() : (E = !1, j = null);
        }
      } else E = !1;
    }
    var q;
    if (typeof w == "function") q = function() {
      w(W);
    };
    else if (typeof MessageChannel < "u") {
      var Q = new MessageChannel(), X = Q.port2;
      Q.port1.onmessage = W, q = function() {
        X.postMessage(null);
      };
    } else q = function() {
      $(W, 0);
    };
    function ce(ie) {
      j = ie, E || (E = !0, q());
    }
    function J(ie, re) {
      L = $(function() {
        ie(e.unstable_now());
      }, re);
    }
    e.unstable_IdlePriority = 5, e.unstable_ImmediatePriority = 1, e.unstable_LowPriority = 4, e.unstable_NormalPriority = 3, e.unstable_Profiling = null, e.unstable_UserBlockingPriority = 2, e.unstable_cancelCallback = function(ie) {
      ie.callback = null;
    }, e.unstable_continueExecution = function() {
      v || y || (v = !0, ce(k));
    }, e.unstable_forceFrameRate = function(ie) {
      0 > ie || 125 < ie ? console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported") : B = 0 < ie ? Math.floor(1e3 / ie) : 5;
    }, e.unstable_getCurrentPriorityLevel = function() {
      return h;
    }, e.unstable_getFirstCallbackNode = function() {
      return n(c);
    }, e.unstable_next = function(ie) {
      switch (h) {
        case 1:
        case 2:
        case 3:
          var re = 3;
          break;
        default:
          re = h;
      }
      var Y = h;
      h = re;
      try {
        return ie();
      } finally {
        h = Y;
      }
    }, e.unstable_pauseExecution = function() {
    }, e.unstable_requestPaint = function() {
    }, e.unstable_runWithPriority = function(ie, re) {
      switch (ie) {
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
          break;
        default:
          ie = 3;
      }
      var Y = h;
      h = ie;
      try {
        return re();
      } finally {
        h = Y;
      }
    }, e.unstable_scheduleCallback = function(ie, re, Y) {
      var T = e.unstable_now();
      switch (typeof Y == "object" && Y !== null ? (Y = Y.delay, Y = typeof Y == "number" && 0 < Y ? T + Y : T) : Y = T, ie) {
        case 1:
          var H = -1;
          break;
        case 2:
          H = 250;
          break;
        case 5:
          H = 1073741823;
          break;
        case 4:
          H = 1e4;
          break;
        default:
          H = 5e3;
      }
      return H = Y + H, ie = { id: p++, callback: re, priorityLevel: ie, startTime: Y, expirationTime: H, sortIndex: -1 }, Y > T ? (ie.sortIndex = Y, t(f, ie), n(c) === null && ie === n(f) && (C ? (S(L), L = -1) : C = !0, J(b, Y - T))) : (ie.sortIndex = H, t(c, ie), v || y || (v = !0, ce(k))), ie;
    }, e.unstable_shouldYield = V, e.unstable_wrapCallback = function(ie) {
      var re = h;
      return function() {
        var Y = h;
        h = re;
        try {
          return ie.apply(this, arguments);
        } finally {
          h = Y;
        }
      };
    };
  }(Om)), Om;
}
var Ub;
function P5() {
  return Ub || (Ub = 1, Pm.exports = k5()), Pm.exports;
}
/**
 * @license React
 * react-dom.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var Vb;
function O5() {
  if (Vb) return Ir;
  Vb = 1;
  var e = z, t = P5();
  function n(r) {
    for (var a = "https://reactjs.org/docs/error-decoder.html?invariant=" + r, d = 1; d < arguments.length; d++) a += "&args[]=" + encodeURIComponent(arguments[d]);
    return "Minified React error #" + r + "; visit " + a + " for the full message or use the non-minified dev environment for full errors and additional helpful warnings.";
  }
  var o = /* @__PURE__ */ new Set(), i = {};
  function s(r, a) {
    l(r, a), l(r + "Capture", a);
  }
  function l(r, a) {
    for (i[r] = a, r = 0; r < a.length; r++) o.add(a[r]);
  }
  var u = !(typeof window > "u" || typeof window.document > "u" || typeof window.document.createElement > "u"), c = Object.prototype.hasOwnProperty, f = /^[:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD][:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD\-.0-9\u00B7\u0300-\u036F\u203F-\u2040]*$/, p = {}, g = {};
  function h(r) {
    return c.call(g, r) ? !0 : c.call(p, r) ? !1 : f.test(r) ? g[r] = !0 : (p[r] = !0, !1);
  }
  function y(r, a, d, m) {
    if (d !== null && d.type === 0) return !1;
    switch (typeof a) {
      case "function":
      case "symbol":
        return !0;
      case "boolean":
        return m ? !1 : d !== null ? !d.acceptsBooleans : (r = r.toLowerCase().slice(0, 5), r !== "data-" && r !== "aria-");
      default:
        return !1;
    }
  }
  function v(r, a, d, m) {
    if (a === null || typeof a > "u" || y(r, a, d, m)) return !0;
    if (m) return !1;
    if (d !== null) switch (d.type) {
      case 3:
        return !a;
      case 4:
        return a === !1;
      case 5:
        return isNaN(a);
      case 6:
        return isNaN(a) || 1 > a;
    }
    return !1;
  }
  function C(r, a, d, m, x, P, M) {
    this.acceptsBooleans = a === 2 || a === 3 || a === 4, this.attributeName = m, this.attributeNamespace = x, this.mustUseProperty = d, this.propertyName = r, this.type = a, this.sanitizeURL = P, this.removeEmptyString = M;
  }
  var $ = {};
  "children dangerouslySetInnerHTML defaultValue defaultChecked innerHTML suppressContentEditableWarning suppressHydrationWarning style".split(" ").forEach(function(r) {
    $[r] = new C(r, 0, !1, r, null, !1, !1);
  }), [["acceptCharset", "accept-charset"], ["className", "class"], ["htmlFor", "for"], ["httpEquiv", "http-equiv"]].forEach(function(r) {
    var a = r[0];
    $[a] = new C(a, 1, !1, r[1], null, !1, !1);
  }), ["contentEditable", "draggable", "spellCheck", "value"].forEach(function(r) {
    $[r] = new C(r, 2, !1, r.toLowerCase(), null, !1, !1);
  }), ["autoReverse", "externalResourcesRequired", "focusable", "preserveAlpha"].forEach(function(r) {
    $[r] = new C(r, 2, !1, r, null, !1, !1);
  }), "allowFullScreen async autoFocus autoPlay controls default defer disabled disablePictureInPicture disableRemotePlayback formNoValidate hidden loop noModule noValidate open playsInline readOnly required reversed scoped seamless itemScope".split(" ").forEach(function(r) {
    $[r] = new C(r, 3, !1, r.toLowerCase(), null, !1, !1);
  }), ["checked", "multiple", "muted", "selected"].forEach(function(r) {
    $[r] = new C(r, 3, !0, r, null, !1, !1);
  }), ["capture", "download"].forEach(function(r) {
    $[r] = new C(r, 4, !1, r, null, !1, !1);
  }), ["cols", "rows", "size", "span"].forEach(function(r) {
    $[r] = new C(r, 6, !1, r, null, !1, !1);
  }), ["rowSpan", "start"].forEach(function(r) {
    $[r] = new C(r, 5, !1, r.toLowerCase(), null, !1, !1);
  });
  var S = /[\-:]([a-z])/g;
  function w(r) {
    return r[1].toUpperCase();
  }
  "accent-height alignment-baseline arabic-form baseline-shift cap-height clip-path clip-rule color-interpolation color-interpolation-filters color-profile color-rendering dominant-baseline enable-background fill-opacity fill-rule flood-color flood-opacity font-family font-size font-size-adjust font-stretch font-style font-variant font-weight glyph-name glyph-orientation-horizontal glyph-orientation-vertical horiz-adv-x horiz-origin-x image-rendering letter-spacing lighting-color marker-end marker-mid marker-start overline-position overline-thickness paint-order panose-1 pointer-events rendering-intent shape-rendering stop-color stop-opacity strikethrough-position strikethrough-thickness stroke-dasharray stroke-dashoffset stroke-linecap stroke-linejoin stroke-miterlimit stroke-opacity stroke-width text-anchor text-decoration text-rendering underline-position underline-thickness unicode-bidi unicode-range units-per-em v-alphabetic v-hanging v-ideographic v-mathematical vector-effect vert-adv-y vert-origin-x vert-origin-y word-spacing writing-mode xmlns:xlink x-height".split(" ").forEach(function(r) {
    var a = r.replace(
      S,
      w
    );
    $[a] = new C(a, 1, !1, r, null, !1, !1);
  }), "xlink:actuate xlink:arcrole xlink:role xlink:show xlink:title xlink:type".split(" ").forEach(function(r) {
    var a = r.replace(S, w);
    $[a] = new C(a, 1, !1, r, "http://www.w3.org/1999/xlink", !1, !1);
  }), ["xml:base", "xml:lang", "xml:space"].forEach(function(r) {
    var a = r.replace(S, w);
    $[a] = new C(a, 1, !1, r, "http://www.w3.org/XML/1998/namespace", !1, !1);
  }), ["tabIndex", "crossOrigin"].forEach(function(r) {
    $[r] = new C(r, 1, !1, r.toLowerCase(), null, !1, !1);
  }), $.xlinkHref = new C("xlinkHref", 1, !1, "xlink:href", "http://www.w3.org/1999/xlink", !0, !1), ["src", "href", "action", "formAction"].forEach(function(r) {
    $[r] = new C(r, 1, !1, r.toLowerCase(), null, !0, !0);
  });
  function _(r, a, d, m) {
    var x = $.hasOwnProperty(a) ? $[a] : null;
    (x !== null ? x.type !== 0 : m || !(2 < a.length) || a[0] !== "o" && a[0] !== "O" || a[1] !== "n" && a[1] !== "N") && (v(a, d, x, m) && (d = null), m || x === null ? h(a) && (d === null ? r.removeAttribute(a) : r.setAttribute(a, "" + d)) : x.mustUseProperty ? r[x.propertyName] = d === null ? x.type === 3 ? !1 : "" : d : (a = x.attributeName, m = x.attributeNamespace, d === null ? r.removeAttribute(a) : (x = x.type, d = x === 3 || x === 4 && d === !0 ? "" : "" + d, m ? r.setAttributeNS(m, a, d) : r.setAttribute(a, d))));
  }
  var b = e.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED, k = Symbol.for("react.element"), E = Symbol.for("react.portal"), j = Symbol.for("react.fragment"), L = Symbol.for("react.strict_mode"), B = Symbol.for("react.profiler"), O = Symbol.for("react.provider"), V = Symbol.for("react.context"), W = Symbol.for("react.forward_ref"), q = Symbol.for("react.suspense"), Q = Symbol.for("react.suspense_list"), X = Symbol.for("react.memo"), ce = Symbol.for("react.lazy"), J = Symbol.for("react.offscreen"), ie = Symbol.iterator;
  function re(r) {
    return r === null || typeof r != "object" ? null : (r = ie && r[ie] || r["@@iterator"], typeof r == "function" ? r : null);
  }
  var Y = Object.assign, T;
  function H(r) {
    if (T === void 0) try {
      throw Error();
    } catch (d) {
      var a = d.stack.trim().match(/\n( *(at )?)/);
      T = a && a[1] || "";
    }
    return `
` + T + r;
  }
  var G = !1;
  function N(r, a) {
    if (!r || G) return "";
    G = !0;
    var d = Error.prepareStackTrace;
    Error.prepareStackTrace = void 0;
    try {
      if (a) if (a = function() {
        throw Error();
      }, Object.defineProperty(a.prototype, "props", { set: function() {
        throw Error();
      } }), typeof Reflect == "object" && Reflect.construct) {
        try {
          Reflect.construct(a, []);
        } catch (he) {
          var m = he;
        }
        Reflect.construct(r, [], a);
      } else {
        try {
          a.call();
        } catch (he) {
          m = he;
        }
        r.call(a.prototype);
      }
      else {
        try {
          throw Error();
        } catch (he) {
          m = he;
        }
        r();
      }
    } catch (he) {
      if (he && m && typeof he.stack == "string") {
        for (var x = he.stack.split(`
`), P = m.stack.split(`
`), M = x.length - 1, K = P.length - 1; 1 <= M && 0 <= K && x[M] !== P[K]; ) K--;
        for (; 1 <= M && 0 <= K; M--, K--) if (x[M] !== P[K]) {
          if (M !== 1 || K !== 1)
            do
              if (M--, K--, 0 > K || x[M] !== P[K]) {
                var te = `
` + x[M].replace(" at new ", " at ");
                return r.displayName && te.includes("<anonymous>") && (te = te.replace("<anonymous>", r.displayName)), te;
              }
            while (1 <= M && 0 <= K);
          break;
        }
      }
    } finally {
      G = !1, Error.prepareStackTrace = d;
    }
    return (r = r ? r.displayName || r.name : "") ? H(r) : "";
  }
  function U(r) {
    switch (r.tag) {
      case 5:
        return H(r.type);
      case 16:
        return H("Lazy");
      case 13:
        return H("Suspense");
      case 19:
        return H("SuspenseList");
      case 0:
      case 2:
      case 15:
        return r = N(r.type, !1), r;
      case 11:
        return r = N(r.type.render, !1), r;
      case 1:
        return r = N(r.type, !0), r;
      default:
        return "";
    }
  }
  function Z(r) {
    if (r == null) return null;
    if (typeof r == "function") return r.displayName || r.name || null;
    if (typeof r == "string") return r;
    switch (r) {
      case j:
        return "Fragment";
      case E:
        return "Portal";
      case B:
        return "Profiler";
      case L:
        return "StrictMode";
      case q:
        return "Suspense";
      case Q:
        return "SuspenseList";
    }
    if (typeof r == "object") switch (r.$$typeof) {
      case V:
        return (r.displayName || "Context") + ".Consumer";
      case O:
        return (r._context.displayName || "Context") + ".Provider";
      case W:
        var a = r.render;
        return r = r.displayName, r || (r = a.displayName || a.name || "", r = r !== "" ? "ForwardRef(" + r + ")" : "ForwardRef"), r;
      case X:
        return a = r.displayName || null, a !== null ? a : Z(r.type) || "Memo";
      case ce:
        a = r._payload, r = r._init;
        try {
          return Z(r(a));
        } catch {
        }
    }
    return null;
  }
  function me(r) {
    var a = r.type;
    switch (r.tag) {
      case 24:
        return "Cache";
      case 9:
        return (a.displayName || "Context") + ".Consumer";
      case 10:
        return (a._context.displayName || "Context") + ".Provider";
      case 18:
        return "DehydratedFragment";
      case 11:
        return r = a.render, r = r.displayName || r.name || "", a.displayName || (r !== "" ? "ForwardRef(" + r + ")" : "ForwardRef");
      case 7:
        return "Fragment";
      case 5:
        return a;
      case 4:
        return "Portal";
      case 3:
        return "Root";
      case 6:
        return "Text";
      case 16:
        return Z(a);
      case 8:
        return a === L ? "StrictMode" : "Mode";
      case 22:
        return "Offscreen";
      case 12:
        return "Profiler";
      case 21:
        return "Scope";
      case 13:
        return "Suspense";
      case 19:
        return "SuspenseList";
      case 25:
        return "TracingMarker";
      case 1:
      case 0:
      case 17:
      case 2:
      case 14:
      case 15:
        if (typeof a == "function") return a.displayName || a.name || null;
        if (typeof a == "string") return a;
    }
    return null;
  }
  function ue(r) {
    switch (typeof r) {
      case "boolean":
      case "number":
      case "string":
      case "undefined":
        return r;
      case "object":
        return r;
      default:
        return "";
    }
  }
  function Ce(r) {
    var a = r.type;
    return (r = r.nodeName) && r.toLowerCase() === "input" && (a === "checkbox" || a === "radio");
  }
  function $e(r) {
    var a = Ce(r) ? "checked" : "value", d = Object.getOwnPropertyDescriptor(r.constructor.prototype, a), m = "" + r[a];
    if (!r.hasOwnProperty(a) && typeof d < "u" && typeof d.get == "function" && typeof d.set == "function") {
      var x = d.get, P = d.set;
      return Object.defineProperty(r, a, { configurable: !0, get: function() {
        return x.call(this);
      }, set: function(M) {
        m = "" + M, P.call(this, M);
      } }), Object.defineProperty(r, a, { enumerable: d.enumerable }), { getValue: function() {
        return m;
      }, setValue: function(M) {
        m = "" + M;
      }, stopTracking: function() {
        r._valueTracker = null, delete r[a];
      } };
    }
  }
  function Te(r) {
    r._valueTracker || (r._valueTracker = $e(r));
  }
  function We(r) {
    if (!r) return !1;
    var a = r._valueTracker;
    if (!a) return !0;
    var d = a.getValue(), m = "";
    return r && (m = Ce(r) ? r.checked ? "true" : "false" : r.value), r = m, r !== d ? (a.setValue(r), !0) : !1;
  }
  function et(r) {
    if (r = r || (typeof document < "u" ? document : void 0), typeof r > "u") return null;
    try {
      return r.activeElement || r.body;
    } catch {
      return r.body;
    }
  }
  function Xe(r, a) {
    var d = a.checked;
    return Y({}, a, { defaultChecked: void 0, defaultValue: void 0, value: void 0, checked: d ?? r._wrapperState.initialChecked });
  }
  function ft(r, a) {
    var d = a.defaultValue == null ? "" : a.defaultValue, m = a.checked != null ? a.checked : a.defaultChecked;
    d = ue(a.value != null ? a.value : d), r._wrapperState = { initialChecked: m, initialValue: d, controlled: a.type === "checkbox" || a.type === "radio" ? a.checked != null : a.value != null };
  }
  function Lt(r, a) {
    a = a.checked, a != null && _(r, "checked", a, !1);
  }
  function st(r, a) {
    Lt(r, a);
    var d = ue(a.value), m = a.type;
    if (d != null) m === "number" ? (d === 0 && r.value === "" || r.value != d) && (r.value = "" + d) : r.value !== "" + d && (r.value = "" + d);
    else if (m === "submit" || m === "reset") {
      r.removeAttribute("value");
      return;
    }
    a.hasOwnProperty("value") ? Se(r, a.type, d) : a.hasOwnProperty("defaultValue") && Se(r, a.type, ue(a.defaultValue)), a.checked == null && a.defaultChecked != null && (r.defaultChecked = !!a.defaultChecked);
  }
  function pt(r, a, d) {
    if (a.hasOwnProperty("value") || a.hasOwnProperty("defaultValue")) {
      var m = a.type;
      if (!(m !== "submit" && m !== "reset" || a.value !== void 0 && a.value !== null)) return;
      a = "" + r._wrapperState.initialValue, d || a === r.value || (r.value = a), r.defaultValue = a;
    }
    d = r.name, d !== "" && (r.name = ""), r.defaultChecked = !!r._wrapperState.initialChecked, d !== "" && (r.name = d);
  }
  function Se(r, a, d) {
    (a !== "number" || et(r.ownerDocument) !== r) && (d == null ? r.defaultValue = "" + r._wrapperState.initialValue : r.defaultValue !== "" + d && (r.defaultValue = "" + d));
  }
  var gt = Array.isArray;
  function yt(r, a, d, m) {
    if (r = r.options, a) {
      a = {};
      for (var x = 0; x < d.length; x++) a["$" + d[x]] = !0;
      for (d = 0; d < r.length; d++) x = a.hasOwnProperty("$" + r[d].value), r[d].selected !== x && (r[d].selected = x), x && m && (r[d].defaultSelected = !0);
    } else {
      for (d = "" + ue(d), a = null, x = 0; x < r.length; x++) {
        if (r[x].value === d) {
          r[x].selected = !0, m && (r[x].defaultSelected = !0);
          return;
        }
        a !== null || r[x].disabled || (a = r[x]);
      }
      a !== null && (a.selected = !0);
    }
  }
  function Le(r, a) {
    if (a.dangerouslySetInnerHTML != null) throw Error(n(91));
    return Y({}, a, { value: void 0, defaultValue: void 0, children: "" + r._wrapperState.initialValue });
  }
  function cn(r, a) {
    var d = a.value;
    if (d == null) {
      if (d = a.children, a = a.defaultValue, d != null) {
        if (a != null) throw Error(n(92));
        if (gt(d)) {
          if (1 < d.length) throw Error(n(93));
          d = d[0];
        }
        a = d;
      }
      a == null && (a = ""), d = a;
    }
    r._wrapperState = { initialValue: ue(d) };
  }
  function ye(r, a) {
    var d = ue(a.value), m = ue(a.defaultValue);
    d != null && (d = "" + d, d !== r.value && (r.value = d), a.defaultValue == null && r.defaultValue !== d && (r.defaultValue = d)), m != null && (r.defaultValue = "" + m);
  }
  function Pe(r) {
    var a = r.textContent;
    a === r._wrapperState.initialValue && a !== "" && a !== null && (r.value = a);
  }
  function Me(r) {
    switch (r) {
      case "svg":
        return "http://www.w3.org/2000/svg";
      case "math":
        return "http://www.w3.org/1998/Math/MathML";
      default:
        return "http://www.w3.org/1999/xhtml";
    }
  }
  function Ye(r, a) {
    return r == null || r === "http://www.w3.org/1999/xhtml" ? Me(a) : r === "http://www.w3.org/2000/svg" && a === "foreignObject" ? "http://www.w3.org/1999/xhtml" : r;
  }
  var ot, se = function(r) {
    return typeof MSApp < "u" && MSApp.execUnsafeLocalFunction ? function(a, d, m, x) {
      MSApp.execUnsafeLocalFunction(function() {
        return r(a, d, m, x);
      });
    } : r;
  }(function(r, a) {
    if (r.namespaceURI !== "http://www.w3.org/2000/svg" || "innerHTML" in r) r.innerHTML = a;
    else {
      for (ot = ot || document.createElement("div"), ot.innerHTML = "<svg>" + a.valueOf().toString() + "</svg>", a = ot.firstChild; r.firstChild; ) r.removeChild(r.firstChild);
      for (; a.firstChild; ) r.appendChild(a.firstChild);
    }
  });
  function ae(r, a) {
    if (a) {
      var d = r.firstChild;
      if (d && d === r.lastChild && d.nodeType === 3) {
        d.nodeValue = a;
        return;
      }
    }
    r.textContent = a;
  }
  var be = {
    animationIterationCount: !0,
    aspectRatio: !0,
    borderImageOutset: !0,
    borderImageSlice: !0,
    borderImageWidth: !0,
    boxFlex: !0,
    boxFlexGroup: !0,
    boxOrdinalGroup: !0,
    columnCount: !0,
    columns: !0,
    flex: !0,
    flexGrow: !0,
    flexPositive: !0,
    flexShrink: !0,
    flexNegative: !0,
    flexOrder: !0,
    gridArea: !0,
    gridRow: !0,
    gridRowEnd: !0,
    gridRowSpan: !0,
    gridRowStart: !0,
    gridColumn: !0,
    gridColumnEnd: !0,
    gridColumnSpan: !0,
    gridColumnStart: !0,
    fontWeight: !0,
    lineClamp: !0,
    lineHeight: !0,
    opacity: !0,
    order: !0,
    orphans: !0,
    tabSize: !0,
    widows: !0,
    zIndex: !0,
    zoom: !0,
    fillOpacity: !0,
    floodOpacity: !0,
    stopOpacity: !0,
    strokeDasharray: !0,
    strokeDashoffset: !0,
    strokeMiterlimit: !0,
    strokeOpacity: !0,
    strokeWidth: !0
  }, Re = ["Webkit", "ms", "Moz", "O"];
  Object.keys(be).forEach(function(r) {
    Re.forEach(function(a) {
      a = a + r.charAt(0).toUpperCase() + r.substring(1), be[a] = be[r];
    });
  });
  function Ge(r, a, d) {
    return a == null || typeof a == "boolean" || a === "" ? "" : d || typeof a != "number" || a === 0 || be.hasOwnProperty(r) && be[r] ? ("" + a).trim() : a + "px";
  }
  function Rt(r, a) {
    r = r.style;
    for (var d in a) if (a.hasOwnProperty(d)) {
      var m = d.indexOf("--") === 0, x = Ge(d, a[d], m);
      d === "float" && (d = "cssFloat"), m ? r.setProperty(d, x) : r[d] = x;
    }
  }
  var Fe = Y({ menuitem: !0 }, { area: !0, base: !0, br: !0, col: !0, embed: !0, hr: !0, img: !0, input: !0, keygen: !0, link: !0, meta: !0, param: !0, source: !0, track: !0, wbr: !0 });
  function tt(r, a) {
    if (a) {
      if (Fe[r] && (a.children != null || a.dangerouslySetInnerHTML != null)) throw Error(n(137, r));
      if (a.dangerouslySetInnerHTML != null) {
        if (a.children != null) throw Error(n(60));
        if (typeof a.dangerouslySetInnerHTML != "object" || !("__html" in a.dangerouslySetInnerHTML)) throw Error(n(61));
      }
      if (a.style != null && typeof a.style != "object") throw Error(n(62));
    }
  }
  function wt(r, a) {
    if (r.indexOf("-") === -1) return typeof a.is == "string";
    switch (r) {
      case "annotation-xml":
      case "color-profile":
      case "font-face":
      case "font-face-src":
      case "font-face-uri":
      case "font-face-format":
      case "font-face-name":
      case "missing-glyph":
        return !1;
      default:
        return !0;
    }
  }
  var Gt = null;
  function dn(r) {
    return r = r.target || r.srcElement || window, r.correspondingUseElement && (r = r.correspondingUseElement), r.nodeType === 3 ? r.parentNode : r;
  }
  var jn = null, qt = null, $t = null;
  function At(r) {
    if (r = nu(r)) {
      if (typeof jn != "function") throw Error(n(280));
      var a = r.stateNode;
      a && (a = td(a), jn(r.stateNode, r.type, a));
    }
  }
  function Jt(r) {
    qt ? $t ? $t.push(r) : $t = [r] : qt = r;
  }
  function $n() {
    if (qt) {
      var r = qt, a = $t;
      if ($t = qt = null, At(r), a) for (r = 0; r < a.length; r++) At(a[r]);
    }
  }
  function Ut(r, a) {
    return r(a);
  }
  function Yt() {
  }
  var On = !1;
  function Zn(r, a, d) {
    if (On) return r(a, d);
    On = !0;
    try {
      return Ut(r, a, d);
    } finally {
      On = !1, (qt !== null || $t !== null) && (Yt(), $n());
    }
  }
  function cr(r, a) {
    var d = r.stateNode;
    if (d === null) return null;
    var m = td(d);
    if (m === null) return null;
    d = m[a];
    e: switch (a) {
      case "onClick":
      case "onClickCapture":
      case "onDoubleClick":
      case "onDoubleClickCapture":
      case "onMouseDown":
      case "onMouseDownCapture":
      case "onMouseMove":
      case "onMouseMoveCapture":
      case "onMouseUp":
      case "onMouseUpCapture":
      case "onMouseEnter":
        (m = !m.disabled) || (r = r.type, m = !(r === "button" || r === "input" || r === "select" || r === "textarea")), r = !m;
        break e;
      default:
        r = !1;
    }
    if (r) return null;
    if (d && typeof d != "function") throw Error(n(231, a, typeof d));
    return d;
  }
  var dr = !1;
  if (u) try {
    var Mt = {};
    Object.defineProperty(Mt, "passive", { get: function() {
      dr = !0;
    } }), window.addEventListener("test", Mt, Mt), window.removeEventListener("test", Mt, Mt);
  } catch {
    dr = !1;
  }
  function Ro(r, a, d, m, x, P, M, K, te) {
    var he = Array.prototype.slice.call(arguments, 3);
    try {
      a.apply(d, he);
    } catch (Ee) {
      this.onError(Ee);
    }
  }
  var Qt = !1, Rn = null, zn = !1, er = null, Bn = { onError: function(r) {
    Qt = !0, Rn = r;
  } };
  function Jr(r, a, d, m, x, P, M, K, te) {
    Qt = !1, Rn = null, Ro.apply(Bn, arguments);
  }
  function Yr(r, a, d, m, x, P, M, K, te) {
    if (Jr.apply(this, arguments), Qt) {
      if (Qt) {
        var he = Rn;
        Qt = !1, Rn = null;
      } else throw Error(n(198));
      zn || (zn = !0, er = he);
    }
  }
  function yn(r) {
    var a = r, d = r;
    if (r.alternate) for (; a.return; ) a = a.return;
    else {
      r = a;
      do
        a = r, a.flags & 4098 && (d = a.return), r = a.return;
      while (r);
    }
    return a.tag === 3 ? d : null;
  }
  function fr(r) {
    if (r.tag === 13) {
      var a = r.memoizedState;
      if (a === null && (r = r.alternate, r !== null && (a = r.memoizedState)), a !== null) return a.dehydrated;
    }
    return null;
  }
  function Lr(r) {
    if (yn(r) !== r) throw Error(n(188));
  }
  function yo(r) {
    var a = r.alternate;
    if (!a) {
      if (a = yn(r), a === null) throw Error(n(188));
      return a !== r ? null : r;
    }
    for (var d = r, m = a; ; ) {
      var x = d.return;
      if (x === null) break;
      var P = x.alternate;
      if (P === null) {
        if (m = x.return, m !== null) {
          d = m;
          continue;
        }
        break;
      }
      if (x.child === P.child) {
        for (P = x.child; P; ) {
          if (P === d) return Lr(x), r;
          if (P === m) return Lr(x), a;
          P = P.sibling;
        }
        throw Error(n(188));
      }
      if (d.return !== m.return) d = x, m = P;
      else {
        for (var M = !1, K = x.child; K; ) {
          if (K === d) {
            M = !0, d = x, m = P;
            break;
          }
          if (K === m) {
            M = !0, m = x, d = P;
            break;
          }
          K = K.sibling;
        }
        if (!M) {
          for (K = P.child; K; ) {
            if (K === d) {
              M = !0, d = P, m = x;
              break;
            }
            if (K === m) {
              M = !0, m = P, d = x;
              break;
            }
            K = K.sibling;
          }
          if (!M) throw Error(n(189));
        }
      }
      if (d.alternate !== m) throw Error(n(190));
    }
    if (d.tag !== 3) throw Error(n(188));
    return d.stateNode.current === d ? r : a;
  }
  function vo(r) {
    return r = yo(r), r !== null ? ns(r) : null;
  }
  function ns(r) {
    if (r.tag === 5 || r.tag === 6) return r;
    for (r = r.child; r !== null; ) {
      var a = ns(r);
      if (a !== null) return a;
      r = r.sibling;
    }
    return null;
  }
  var da = t.unstable_scheduleCallback, Lc = t.unstable_cancelCallback, Xp = t.unstable_shouldYield, Zp = t.unstable_requestPaint, fn = t.unstable_now, Dl = t.unstable_getCurrentPriorityLevel, zl = t.unstable_ImmediatePriority, Bl = t.unstable_UserBlockingPriority, rs = t.unstable_NormalPriority, Ul = t.unstable_LowPriority, pr = t.unstable_IdlePriority, fa = null, Qr = null;
  function Dc(r) {
    if (Qr && typeof Qr.onCommitFiberRoot == "function") try {
      Qr.onCommitFiberRoot(fa, r, void 0, (r.current.flags & 128) === 128);
    } catch {
    }
  }
  var xr = Math.clz32 ? Math.clz32 : eh, zc = Math.log, Bc = Math.LN2;
  function eh(r) {
    return r >>>= 0, r === 0 ? 32 : 31 - (zc(r) / Bc | 0) | 0;
  }
  var pa = 64, os = 4194304;
  function ri(r) {
    switch (r & -r) {
      case 1:
        return 1;
      case 2:
        return 2;
      case 4:
        return 4;
      case 8:
        return 8;
      case 16:
        return 16;
      case 32:
        return 32;
      case 64:
      case 128:
      case 256:
      case 512:
      case 1024:
      case 2048:
      case 4096:
      case 8192:
      case 16384:
      case 32768:
      case 65536:
      case 131072:
      case 262144:
      case 524288:
      case 1048576:
      case 2097152:
        return r & 4194240;
      case 4194304:
      case 8388608:
      case 16777216:
      case 33554432:
      case 67108864:
        return r & 130023424;
      case 134217728:
        return 134217728;
      case 268435456:
        return 268435456;
      case 536870912:
        return 536870912;
      case 1073741824:
        return 1073741824;
      default:
        return r;
    }
  }
  function is(r, a) {
    var d = r.pendingLanes;
    if (d === 0) return 0;
    var m = 0, x = r.suspendedLanes, P = r.pingedLanes, M = d & 268435455;
    if (M !== 0) {
      var K = M & ~x;
      K !== 0 ? m = ri(K) : (P &= M, P !== 0 && (m = ri(P)));
    } else M = d & ~x, M !== 0 ? m = ri(M) : P !== 0 && (m = ri(P));
    if (m === 0) return 0;
    if (a !== 0 && a !== m && !(a & x) && (x = m & -m, P = a & -a, x >= P || x === 16 && (P & 4194240) !== 0)) return a;
    if (m & 4 && (m |= d & 16), a = r.entangledLanes, a !== 0) for (r = r.entanglements, a &= m; 0 < a; ) d = 31 - xr(a), x = 1 << d, m |= r[d], a &= ~x;
    return m;
  }
  function as(r, a) {
    switch (r) {
      case 1:
      case 2:
      case 4:
        return a + 250;
      case 8:
      case 16:
      case 32:
      case 64:
      case 128:
      case 256:
      case 512:
      case 1024:
      case 2048:
      case 4096:
      case 8192:
      case 16384:
      case 32768:
      case 65536:
      case 131072:
      case 262144:
      case 524288:
      case 1048576:
      case 2097152:
        return a + 5e3;
      case 4194304:
      case 8388608:
      case 16777216:
      case 33554432:
      case 67108864:
        return -1;
      case 134217728:
      case 268435456:
      case 536870912:
      case 1073741824:
        return -1;
      default:
        return -1;
    }
  }
  function th(r, a) {
    for (var d = r.suspendedLanes, m = r.pingedLanes, x = r.expirationTimes, P = r.pendingLanes; 0 < P; ) {
      var M = 31 - xr(P), K = 1 << M, te = x[M];
      te === -1 ? (!(K & d) || K & m) && (x[M] = as(K, a)) : te <= a && (r.expiredLanes |= K), P &= ~K;
    }
  }
  function Vl(r) {
    return r = r.pendingLanes & -1073741825, r !== 0 ? r : r & 1073741824 ? 1073741824 : 0;
  }
  function Uc() {
    var r = pa;
    return pa <<= 1, !(pa & 4194240) && (pa = 64), r;
  }
  function Wl(r) {
    for (var a = [], d = 0; 31 > d; d++) a.push(r);
    return a;
  }
  function To(r, a, d) {
    r.pendingLanes |= a, a !== 536870912 && (r.suspendedLanes = 0, r.pingedLanes = 0), r = r.eventTimes, a = 31 - xr(a), r[a] = d;
  }
  function nh(r, a) {
    var d = r.pendingLanes & ~a;
    r.pendingLanes = a, r.suspendedLanes = 0, r.pingedLanes = 0, r.expiredLanes &= a, r.mutableReadLanes &= a, r.entangledLanes &= a, a = r.entanglements;
    var m = r.eventTimes;
    for (r = r.expirationTimes; 0 < d; ) {
      var x = 31 - xr(d), P = 1 << x;
      a[x] = 0, m[x] = -1, r[x] = -1, d &= ~P;
    }
  }
  function D(r, a) {
    var d = r.entangledLanes |= a;
    for (r = r.entanglements; d; ) {
      var m = 31 - xr(d), x = 1 << m;
      x & a | r[m] & a && (r[m] |= a), d &= ~x;
    }
  }
  var I = 0;
  function R(r) {
    return r &= -r, 1 < r ? 4 < r ? r & 268435455 ? 16 : 536870912 : 4 : 1;
  }
  var F, ee, pe, de, ne, ve = !1, we = [], Ae = null, je = null, Be = null, Ie = /* @__PURE__ */ new Map(), ut = /* @__PURE__ */ new Map(), Ct = [], Pt = "mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset submit".split(" ");
  function Kt(r, a) {
    switch (r) {
      case "focusin":
      case "focusout":
        Ae = null;
        break;
      case "dragenter":
      case "dragleave":
        je = null;
        break;
      case "mouseover":
      case "mouseout":
        Be = null;
        break;
      case "pointerover":
      case "pointerout":
        Ie.delete(a.pointerId);
        break;
      case "gotpointercapture":
      case "lostpointercapture":
        ut.delete(a.pointerId);
    }
  }
  function hr(r, a, d, m, x, P) {
    return r === null || r.nativeEvent !== P ? (r = { blockedOn: a, domEventName: d, eventSystemFlags: m, nativeEvent: P, targetContainers: [x] }, a !== null && (a = nu(a), a !== null && ee(a)), r) : (r.eventSystemFlags |= m, a = r.targetContainers, x !== null && a.indexOf(x) === -1 && a.push(x), r);
  }
  function Tn(r, a, d, m, x) {
    switch (a) {
      case "focusin":
        return Ae = hr(Ae, r, a, d, m, x), !0;
      case "dragenter":
        return je = hr(je, r, a, d, m, x), !0;
      case "mouseover":
        return Be = hr(Be, r, a, d, m, x), !0;
      case "pointerover":
        var P = x.pointerId;
        return Ie.set(P, hr(Ie.get(P) || null, r, a, d, m, x)), !0;
      case "gotpointercapture":
        return P = x.pointerId, ut.set(P, hr(ut.get(P) || null, r, a, d, m, x)), !0;
    }
    return !1;
  }
  function ha(r) {
    var a = ya(r.target);
    if (a !== null) {
      var d = yn(a);
      if (d !== null) {
        if (a = d.tag, a === 13) {
          if (a = fr(d), a !== null) {
            r.blockedOn = a, ne(r.priority, function() {
              pe(d);
            });
            return;
          }
        } else if (a === 3 && d.stateNode.current.memoizedState.isDehydrated) {
          r.blockedOn = d.tag === 3 ? d.stateNode.containerInfo : null;
          return;
        }
      }
    }
    r.blockedOn = null;
  }
  function Er(r) {
    if (r.blockedOn !== null) return !1;
    for (var a = r.targetContainers; 0 < a.length; ) {
      var d = ga(r.domEventName, r.eventSystemFlags, a[0], r.nativeEvent);
      if (d === null) {
        d = r.nativeEvent;
        var m = new d.constructor(d.type, d);
        Gt = m, d.target.dispatchEvent(m), Gt = null;
      } else return a = nu(d), a !== null && ee(a), r.blockedOn = d, !1;
      a.shift();
    }
    return !0;
  }
  function Mo(r, a, d) {
    Er(r) && d.delete(a);
  }
  function bo() {
    ve = !1, Ae !== null && Er(Ae) && (Ae = null), je !== null && Er(je) && (je = null), Be !== null && Er(Be) && (Be = null), Ie.forEach(Mo), ut.forEach(Mo);
  }
  function on(r, a) {
    r.blockedOn === a && (r.blockedOn = null, ve || (ve = !0, t.unstable_scheduleCallback(t.unstable_NormalPriority, bo)));
  }
  function Mn(r) {
    function a(x) {
      return on(x, r);
    }
    if (0 < we.length) {
      on(we[0], r);
      for (var d = 1; d < we.length; d++) {
        var m = we[d];
        m.blockedOn === r && (m.blockedOn = null);
      }
    }
    for (Ae !== null && on(Ae, r), je !== null && on(je, r), Be !== null && on(Be, r), Ie.forEach(a), ut.forEach(a), d = 0; d < Ct.length; d++) m = Ct[d], m.blockedOn === r && (m.blockedOn = null);
    for (; 0 < Ct.length && (d = Ct[0], d.blockedOn === null); ) ha(d), d.blockedOn === null && Ct.shift();
  }
  var Xr = b.ReactCurrentBatchConfig, So = !0;
  function Vc(r, a, d, m) {
    var x = I, P = Xr.transition;
    Xr.transition = null;
    try {
      I = 1, ma(r, a, d, m);
    } finally {
      I = x, Xr.transition = P;
    }
  }
  function ss(r, a, d, m) {
    var x = I, P = Xr.transition;
    Xr.transition = null;
    try {
      I = 4, ma(r, a, d, m);
    } finally {
      I = x, Xr.transition = P;
    }
  }
  function ma(r, a, d, m) {
    if (So) {
      var x = ga(r, a, d, m);
      if (x === null) vh(r, a, m, Oi, d), Kt(r, m);
      else if (Tn(x, r, a, d, m)) m.stopPropagation();
      else if (Kt(r, m), a & 4 && -1 < Pt.indexOf(r)) {
        for (; x !== null; ) {
          var P = nu(x);
          if (P !== null && F(P), P = ga(r, a, d, m), P === null && vh(r, a, m, Oi, d), P === x) break;
          x = P;
        }
        x !== null && m.stopPropagation();
      } else vh(r, a, m, null, d);
    }
  }
  var Oi = null;
  function ga(r, a, d, m) {
    if (Oi = null, r = dn(m), r = ya(r), r !== null) if (a = yn(r), a === null) r = null;
    else if (d = a.tag, d === 13) {
      if (r = fr(a), r !== null) return r;
      r = null;
    } else if (d === 3) {
      if (a.stateNode.current.memoizedState.isDehydrated) return a.tag === 3 ? a.stateNode.containerInfo : null;
      r = null;
    } else a !== r && (r = null);
    return Oi = r, null;
  }
  function ls(r) {
    switch (r) {
      case "cancel":
      case "click":
      case "close":
      case "contextmenu":
      case "copy":
      case "cut":
      case "auxclick":
      case "dblclick":
      case "dragend":
      case "dragstart":
      case "drop":
      case "focusin":
      case "focusout":
      case "input":
      case "invalid":
      case "keydown":
      case "keypress":
      case "keyup":
      case "mousedown":
      case "mouseup":
      case "paste":
      case "pause":
      case "play":
      case "pointercancel":
      case "pointerdown":
      case "pointerup":
      case "ratechange":
      case "reset":
      case "resize":
      case "seeked":
      case "submit":
      case "touchcancel":
      case "touchend":
      case "touchstart":
      case "volumechange":
      case "change":
      case "selectionchange":
      case "textInput":
      case "compositionstart":
      case "compositionend":
      case "compositionupdate":
      case "beforeblur":
      case "afterblur":
      case "beforeinput":
      case "blur":
      case "fullscreenchange":
      case "focus":
      case "hashchange":
      case "popstate":
      case "select":
      case "selectstart":
        return 1;
      case "drag":
      case "dragenter":
      case "dragexit":
      case "dragleave":
      case "dragover":
      case "mousemove":
      case "mouseout":
      case "mouseover":
      case "pointermove":
      case "pointerout":
      case "pointerover":
      case "scroll":
      case "toggle":
      case "touchmove":
      case "wheel":
      case "mouseenter":
      case "mouseleave":
      case "pointerenter":
      case "pointerleave":
        return 4;
      case "message":
        switch (Dl()) {
          case zl:
            return 1;
          case Bl:
            return 4;
          case rs:
          case Ul:
            return 16;
          case pr:
            return 536870912;
          default:
            return 16;
        }
      default:
        return 16;
    }
  }
  var mr = null, us = null, Wc = null;
  function kv() {
    if (Wc) return Wc;
    var r, a = us, d = a.length, m, x = "value" in mr ? mr.value : mr.textContent, P = x.length;
    for (r = 0; r < d && a[r] === x[r]; r++) ;
    var M = d - r;
    for (m = 1; m <= M && a[d - m] === x[P - m]; m++) ;
    return Wc = x.slice(r, 1 < m ? 1 - m : void 0);
  }
  function qc(r) {
    var a = r.keyCode;
    return "charCode" in r ? (r = r.charCode, r === 0 && a === 13 && (r = 13)) : r = a, r === 10 && (r = 13), 32 <= r || r === 13 ? r : 0;
  }
  function Kc() {
    return !0;
  }
  function Pv() {
    return !1;
  }
  function Dr(r) {
    function a(d, m, x, P, M) {
      this._reactName = d, this._targetInst = x, this.type = m, this.nativeEvent = P, this.target = M, this.currentTarget = null;
      for (var K in r) r.hasOwnProperty(K) && (d = r[K], this[K] = d ? d(P) : P[K]);
      return this.isDefaultPrevented = (P.defaultPrevented != null ? P.defaultPrevented : P.returnValue === !1) ? Kc : Pv, this.isPropagationStopped = Pv, this;
    }
    return Y(a.prototype, { preventDefault: function() {
      this.defaultPrevented = !0;
      var d = this.nativeEvent;
      d && (d.preventDefault ? d.preventDefault() : typeof d.returnValue != "unknown" && (d.returnValue = !1), this.isDefaultPrevented = Kc);
    }, stopPropagation: function() {
      var d = this.nativeEvent;
      d && (d.stopPropagation ? d.stopPropagation() : typeof d.cancelBubble != "unknown" && (d.cancelBubble = !0), this.isPropagationStopped = Kc);
    }, persist: function() {
    }, isPersistent: Kc }), a;
  }
  var cs = { eventPhase: 0, bubbles: 0, cancelable: 0, timeStamp: function(r) {
    return r.timeStamp || Date.now();
  }, defaultPrevented: 0, isTrusted: 0 }, rh = Dr(cs), ql = Y({}, cs, { view: 0, detail: 0 }), Q_ = Dr(ql), oh, ih, Kl, Hc = Y({}, ql, { screenX: 0, screenY: 0, clientX: 0, clientY: 0, pageX: 0, pageY: 0, ctrlKey: 0, shiftKey: 0, altKey: 0, metaKey: 0, getModifierState: sh, button: 0, buttons: 0, relatedTarget: function(r) {
    return r.relatedTarget === void 0 ? r.fromElement === r.srcElement ? r.toElement : r.fromElement : r.relatedTarget;
  }, movementX: function(r) {
    return "movementX" in r ? r.movementX : (r !== Kl && (Kl && r.type === "mousemove" ? (oh = r.screenX - Kl.screenX, ih = r.screenY - Kl.screenY) : ih = oh = 0, Kl = r), oh);
  }, movementY: function(r) {
    return "movementY" in r ? r.movementY : ih;
  } }), Ov = Dr(Hc), X_ = Y({}, Hc, { dataTransfer: 0 }), Z_ = Dr(X_), ek = Y({}, ql, { relatedTarget: 0 }), ah = Dr(ek), tk = Y({}, cs, { animationName: 0, elapsedTime: 0, pseudoElement: 0 }), nk = Dr(tk), rk = Y({}, cs, { clipboardData: function(r) {
    return "clipboardData" in r ? r.clipboardData : window.clipboardData;
  } }), ok = Dr(rk), ik = Y({}, cs, { data: 0 }), Av = Dr(ik), ak = {
    Esc: "Escape",
    Spacebar: " ",
    Left: "ArrowLeft",
    Up: "ArrowUp",
    Right: "ArrowRight",
    Down: "ArrowDown",
    Del: "Delete",
    Win: "OS",
    Menu: "ContextMenu",
    Apps: "ContextMenu",
    Scroll: "ScrollLock",
    MozPrintableKey: "Unidentified"
  }, sk = {
    8: "Backspace",
    9: "Tab",
    12: "Clear",
    13: "Enter",
    16: "Shift",
    17: "Control",
    18: "Alt",
    19: "Pause",
    20: "CapsLock",
    27: "Escape",
    32: " ",
    33: "PageUp",
    34: "PageDown",
    35: "End",
    36: "Home",
    37: "ArrowLeft",
    38: "ArrowUp",
    39: "ArrowRight",
    40: "ArrowDown",
    45: "Insert",
    46: "Delete",
    112: "F1",
    113: "F2",
    114: "F3",
    115: "F4",
    116: "F5",
    117: "F6",
    118: "F7",
    119: "F8",
    120: "F9",
    121: "F10",
    122: "F11",
    123: "F12",
    144: "NumLock",
    145: "ScrollLock",
    224: "Meta"
  }, lk = { Alt: "altKey", Control: "ctrlKey", Meta: "metaKey", Shift: "shiftKey" };
  function uk(r) {
    var a = this.nativeEvent;
    return a.getModifierState ? a.getModifierState(r) : (r = lk[r]) ? !!a[r] : !1;
  }
  function sh() {
    return uk;
  }
  var ck = Y({}, ql, { key: function(r) {
    if (r.key) {
      var a = ak[r.key] || r.key;
      if (a !== "Unidentified") return a;
    }
    return r.type === "keypress" ? (r = qc(r), r === 13 ? "Enter" : String.fromCharCode(r)) : r.type === "keydown" || r.type === "keyup" ? sk[r.keyCode] || "Unidentified" : "";
  }, code: 0, location: 0, ctrlKey: 0, shiftKey: 0, altKey: 0, metaKey: 0, repeat: 0, locale: 0, getModifierState: sh, charCode: function(r) {
    return r.type === "keypress" ? qc(r) : 0;
  }, keyCode: function(r) {
    return r.type === "keydown" || r.type === "keyup" ? r.keyCode : 0;
  }, which: function(r) {
    return r.type === "keypress" ? qc(r) : r.type === "keydown" || r.type === "keyup" ? r.keyCode : 0;
  } }), dk = Dr(ck), fk = Y({}, Hc, { pointerId: 0, width: 0, height: 0, pressure: 0, tangentialPressure: 0, tiltX: 0, tiltY: 0, twist: 0, pointerType: 0, isPrimary: 0 }), Iv = Dr(fk), pk = Y({}, ql, { touches: 0, targetTouches: 0, changedTouches: 0, altKey: 0, metaKey: 0, ctrlKey: 0, shiftKey: 0, getModifierState: sh }), hk = Dr(pk), mk = Y({}, cs, { propertyName: 0, elapsedTime: 0, pseudoElement: 0 }), gk = Dr(mk), yk = Y({}, Hc, {
    deltaX: function(r) {
      return "deltaX" in r ? r.deltaX : "wheelDeltaX" in r ? -r.wheelDeltaX : 0;
    },
    deltaY: function(r) {
      return "deltaY" in r ? r.deltaY : "wheelDeltaY" in r ? -r.wheelDeltaY : "wheelDelta" in r ? -r.wheelDelta : 0;
    },
    deltaZ: 0,
    deltaMode: 0
  }), vk = Dr(yk), bk = [9, 13, 27, 32], lh = u && "CompositionEvent" in window, Hl = null;
  u && "documentMode" in document && (Hl = document.documentMode);
  var Sk = u && "TextEvent" in window && !Hl, jv = u && (!lh || Hl && 8 < Hl && 11 >= Hl), Rv = " ", Tv = !1;
  function Mv(r, a) {
    switch (r) {
      case "keyup":
        return bk.indexOf(a.keyCode) !== -1;
      case "keydown":
        return a.keyCode !== 229;
      case "keypress":
      case "mousedown":
      case "focusout":
        return !0;
      default:
        return !1;
    }
  }
  function Nv(r) {
    return r = r.detail, typeof r == "object" && "data" in r ? r.data : null;
  }
  var ds = !1;
  function wk(r, a) {
    switch (r) {
      case "compositionend":
        return Nv(a);
      case "keypress":
        return a.which !== 32 ? null : (Tv = !0, Rv);
      case "textInput":
        return r = a.data, r === Rv && Tv ? null : r;
      default:
        return null;
    }
  }
  function $k(r, a) {
    if (ds) return r === "compositionend" || !lh && Mv(r, a) ? (r = kv(), Wc = us = mr = null, ds = !1, r) : null;
    switch (r) {
      case "paste":
        return null;
      case "keypress":
        if (!(a.ctrlKey || a.altKey || a.metaKey) || a.ctrlKey && a.altKey) {
          if (a.char && 1 < a.char.length) return a.char;
          if (a.which) return String.fromCharCode(a.which);
        }
        return null;
      case "compositionend":
        return jv && a.locale !== "ko" ? null : a.data;
      default:
        return null;
    }
  }
  var Ck = { color: !0, date: !0, datetime: !0, "datetime-local": !0, email: !0, month: !0, number: !0, password: !0, range: !0, search: !0, tel: !0, text: !0, time: !0, url: !0, week: !0 };
  function Fv(r) {
    var a = r && r.nodeName && r.nodeName.toLowerCase();
    return a === "input" ? !!Ck[r.type] : a === "textarea";
  }
  function Lv(r, a, d, m) {
    Jt(m), a = Xc(a, "onChange"), 0 < a.length && (d = new rh("onChange", "change", null, d, m), r.push({ event: d, listeners: a }));
  }
  var Gl = null, Jl = null;
  function xk(r) {
    n1(r, 0);
  }
  function Gc(r) {
    var a = gs(r);
    if (We(a)) return r;
  }
  function Ek(r, a) {
    if (r === "change") return a;
  }
  var Dv = !1;
  if (u) {
    var uh;
    if (u) {
      var ch = "oninput" in document;
      if (!ch) {
        var zv = document.createElement("div");
        zv.setAttribute("oninput", "return;"), ch = typeof zv.oninput == "function";
      }
      uh = ch;
    } else uh = !1;
    Dv = uh && (!document.documentMode || 9 < document.documentMode);
  }
  function Bv() {
    Gl && (Gl.detachEvent("onpropertychange", Uv), Jl = Gl = null);
  }
  function Uv(r) {
    if (r.propertyName === "value" && Gc(Jl)) {
      var a = [];
      Lv(a, Jl, r, dn(r)), Zn(xk, a);
    }
  }
  function _k(r, a, d) {
    r === "focusin" ? (Bv(), Gl = a, Jl = d, Gl.attachEvent("onpropertychange", Uv)) : r === "focusout" && Bv();
  }
  function kk(r) {
    if (r === "selectionchange" || r === "keyup" || r === "keydown") return Gc(Jl);
  }
  function Pk(r, a) {
    if (r === "click") return Gc(a);
  }
  function Ok(r, a) {
    if (r === "input" || r === "change") return Gc(a);
  }
  function Ak(r, a) {
    return r === a && (r !== 0 || 1 / r === 1 / a) || r !== r && a !== a;
  }
  var wo = typeof Object.is == "function" ? Object.is : Ak;
  function Yl(r, a) {
    if (wo(r, a)) return !0;
    if (typeof r != "object" || r === null || typeof a != "object" || a === null) return !1;
    var d = Object.keys(r), m = Object.keys(a);
    if (d.length !== m.length) return !1;
    for (m = 0; m < d.length; m++) {
      var x = d[m];
      if (!c.call(a, x) || !wo(r[x], a[x])) return !1;
    }
    return !0;
  }
  function Vv(r) {
    for (; r && r.firstChild; ) r = r.firstChild;
    return r;
  }
  function Wv(r, a) {
    var d = Vv(r);
    r = 0;
    for (var m; d; ) {
      if (d.nodeType === 3) {
        if (m = r + d.textContent.length, r <= a && m >= a) return { node: d, offset: a - r };
        r = m;
      }
      e: {
        for (; d; ) {
          if (d.nextSibling) {
            d = d.nextSibling;
            break e;
          }
          d = d.parentNode;
        }
        d = void 0;
      }
      d = Vv(d);
    }
  }
  function qv(r, a) {
    return r && a ? r === a ? !0 : r && r.nodeType === 3 ? !1 : a && a.nodeType === 3 ? qv(r, a.parentNode) : "contains" in r ? r.contains(a) : r.compareDocumentPosition ? !!(r.compareDocumentPosition(a) & 16) : !1 : !1;
  }
  function Kv() {
    for (var r = window, a = et(); a instanceof r.HTMLIFrameElement; ) {
      try {
        var d = typeof a.contentWindow.location.href == "string";
      } catch {
        d = !1;
      }
      if (d) r = a.contentWindow;
      else break;
      a = et(r.document);
    }
    return a;
  }
  function dh(r) {
    var a = r && r.nodeName && r.nodeName.toLowerCase();
    return a && (a === "input" && (r.type === "text" || r.type === "search" || r.type === "tel" || r.type === "url" || r.type === "password") || a === "textarea" || r.contentEditable === "true");
  }
  function Ik(r) {
    var a = Kv(), d = r.focusedElem, m = r.selectionRange;
    if (a !== d && d && d.ownerDocument && qv(d.ownerDocument.documentElement, d)) {
      if (m !== null && dh(d)) {
        if (a = m.start, r = m.end, r === void 0 && (r = a), "selectionStart" in d) d.selectionStart = a, d.selectionEnd = Math.min(r, d.value.length);
        else if (r = (a = d.ownerDocument || document) && a.defaultView || window, r.getSelection) {
          r = r.getSelection();
          var x = d.textContent.length, P = Math.min(m.start, x);
          m = m.end === void 0 ? P : Math.min(m.end, x), !r.extend && P > m && (x = m, m = P, P = x), x = Wv(d, P);
          var M = Wv(
            d,
            m
          );
          x && M && (r.rangeCount !== 1 || r.anchorNode !== x.node || r.anchorOffset !== x.offset || r.focusNode !== M.node || r.focusOffset !== M.offset) && (a = a.createRange(), a.setStart(x.node, x.offset), r.removeAllRanges(), P > m ? (r.addRange(a), r.extend(M.node, M.offset)) : (a.setEnd(M.node, M.offset), r.addRange(a)));
        }
      }
      for (a = [], r = d; r = r.parentNode; ) r.nodeType === 1 && a.push({ element: r, left: r.scrollLeft, top: r.scrollTop });
      for (typeof d.focus == "function" && d.focus(), d = 0; d < a.length; d++) r = a[d], r.element.scrollLeft = r.left, r.element.scrollTop = r.top;
    }
  }
  var jk = u && "documentMode" in document && 11 >= document.documentMode, fs = null, fh = null, Ql = null, ph = !1;
  function Hv(r, a, d) {
    var m = d.window === d ? d.document : d.nodeType === 9 ? d : d.ownerDocument;
    ph || fs == null || fs !== et(m) || (m = fs, "selectionStart" in m && dh(m) ? m = { start: m.selectionStart, end: m.selectionEnd } : (m = (m.ownerDocument && m.ownerDocument.defaultView || window).getSelection(), m = { anchorNode: m.anchorNode, anchorOffset: m.anchorOffset, focusNode: m.focusNode, focusOffset: m.focusOffset }), Ql && Yl(Ql, m) || (Ql = m, m = Xc(fh, "onSelect"), 0 < m.length && (a = new rh("onSelect", "select", null, a, d), r.push({ event: a, listeners: m }), a.target = fs)));
  }
  function Jc(r, a) {
    var d = {};
    return d[r.toLowerCase()] = a.toLowerCase(), d["Webkit" + r] = "webkit" + a, d["Moz" + r] = "moz" + a, d;
  }
  var ps = { animationend: Jc("Animation", "AnimationEnd"), animationiteration: Jc("Animation", "AnimationIteration"), animationstart: Jc("Animation", "AnimationStart"), transitionend: Jc("Transition", "TransitionEnd") }, hh = {}, Gv = {};
  u && (Gv = document.createElement("div").style, "AnimationEvent" in window || (delete ps.animationend.animation, delete ps.animationiteration.animation, delete ps.animationstart.animation), "TransitionEvent" in window || delete ps.transitionend.transition);
  function Yc(r) {
    if (hh[r]) return hh[r];
    if (!ps[r]) return r;
    var a = ps[r], d;
    for (d in a) if (a.hasOwnProperty(d) && d in Gv) return hh[r] = a[d];
    return r;
  }
  var Jv = Yc("animationend"), Yv = Yc("animationiteration"), Qv = Yc("animationstart"), Xv = Yc("transitionend"), Zv = /* @__PURE__ */ new Map(), e1 = "abort auxClick cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel".split(" ");
  function Ai(r, a) {
    Zv.set(r, a), s(a, [r]);
  }
  for (var mh = 0; mh < e1.length; mh++) {
    var gh = e1[mh], Rk = gh.toLowerCase(), Tk = gh[0].toUpperCase() + gh.slice(1);
    Ai(Rk, "on" + Tk);
  }
  Ai(Jv, "onAnimationEnd"), Ai(Yv, "onAnimationIteration"), Ai(Qv, "onAnimationStart"), Ai("dblclick", "onDoubleClick"), Ai("focusin", "onFocus"), Ai("focusout", "onBlur"), Ai(Xv, "onTransitionEnd"), l("onMouseEnter", ["mouseout", "mouseover"]), l("onMouseLeave", ["mouseout", "mouseover"]), l("onPointerEnter", ["pointerout", "pointerover"]), l("onPointerLeave", ["pointerout", "pointerover"]), s("onChange", "change click focusin focusout input keydown keyup selectionchange".split(" ")), s("onSelect", "focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")), s("onBeforeInput", ["compositionend", "keypress", "textInput", "paste"]), s("onCompositionEnd", "compositionend focusout keydown keypress keyup mousedown".split(" ")), s("onCompositionStart", "compositionstart focusout keydown keypress keyup mousedown".split(" ")), s("onCompositionUpdate", "compositionupdate focusout keydown keypress keyup mousedown".split(" "));
  var Xl = "abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "), Mk = new Set("cancel close invalid load scroll toggle".split(" ").concat(Xl));
  function t1(r, a, d) {
    var m = r.type || "unknown-event";
    r.currentTarget = d, Yr(m, a, void 0, r), r.currentTarget = null;
  }
  function n1(r, a) {
    a = (a & 4) !== 0;
    for (var d = 0; d < r.length; d++) {
      var m = r[d], x = m.event;
      m = m.listeners;
      e: {
        var P = void 0;
        if (a) for (var M = m.length - 1; 0 <= M; M--) {
          var K = m[M], te = K.instance, he = K.currentTarget;
          if (K = K.listener, te !== P && x.isPropagationStopped()) break e;
          t1(x, K, he), P = te;
        }
        else for (M = 0; M < m.length; M++) {
          if (K = m[M], te = K.instance, he = K.currentTarget, K = K.listener, te !== P && x.isPropagationStopped()) break e;
          t1(x, K, he), P = te;
        }
      }
    }
    if (zn) throw r = er, zn = !1, er = null, r;
  }
  function an(r, a) {
    var d = a[xh];
    d === void 0 && (d = a[xh] = /* @__PURE__ */ new Set());
    var m = r + "__bubble";
    d.has(m) || (r1(a, r, 2, !1), d.add(m));
  }
  function yh(r, a, d) {
    var m = 0;
    a && (m |= 4), r1(d, r, m, a);
  }
  var Qc = "_reactListening" + Math.random().toString(36).slice(2);
  function Zl(r) {
    if (!r[Qc]) {
      r[Qc] = !0, o.forEach(function(d) {
        d !== "selectionchange" && (Mk.has(d) || yh(d, !1, r), yh(d, !0, r));
      });
      var a = r.nodeType === 9 ? r : r.ownerDocument;
      a === null || a[Qc] || (a[Qc] = !0, yh("selectionchange", !1, a));
    }
  }
  function r1(r, a, d, m) {
    switch (ls(a)) {
      case 1:
        var x = Vc;
        break;
      case 4:
        x = ss;
        break;
      default:
        x = ma;
    }
    d = x.bind(null, a, d, r), x = void 0, !dr || a !== "touchstart" && a !== "touchmove" && a !== "wheel" || (x = !0), m ? x !== void 0 ? r.addEventListener(a, d, { capture: !0, passive: x }) : r.addEventListener(a, d, !0) : x !== void 0 ? r.addEventListener(a, d, { passive: x }) : r.addEventListener(a, d, !1);
  }
  function vh(r, a, d, m, x) {
    var P = m;
    if (!(a & 1) && !(a & 2) && m !== null) e: for (; ; ) {
      if (m === null) return;
      var M = m.tag;
      if (M === 3 || M === 4) {
        var K = m.stateNode.containerInfo;
        if (K === x || K.nodeType === 8 && K.parentNode === x) break;
        if (M === 4) for (M = m.return; M !== null; ) {
          var te = M.tag;
          if ((te === 3 || te === 4) && (te = M.stateNode.containerInfo, te === x || te.nodeType === 8 && te.parentNode === x)) return;
          M = M.return;
        }
        for (; K !== null; ) {
          if (M = ya(K), M === null) return;
          if (te = M.tag, te === 5 || te === 6) {
            m = P = M;
            continue e;
          }
          K = K.parentNode;
        }
      }
      m = m.return;
    }
    Zn(function() {
      var he = P, Ee = dn(d), _e = [];
      e: {
        var xe = Zv.get(r);
        if (xe !== void 0) {
          var De = rh, qe = r;
          switch (r) {
            case "keypress":
              if (qc(d) === 0) break e;
            case "keydown":
            case "keyup":
              De = dk;
              break;
            case "focusin":
              qe = "focus", De = ah;
              break;
            case "focusout":
              qe = "blur", De = ah;
              break;
            case "beforeblur":
            case "afterblur":
              De = ah;
              break;
            case "click":
              if (d.button === 2) break e;
            case "auxclick":
            case "dblclick":
            case "mousedown":
            case "mousemove":
            case "mouseup":
            case "mouseout":
            case "mouseover":
            case "contextmenu":
              De = Ov;
              break;
            case "drag":
            case "dragend":
            case "dragenter":
            case "dragexit":
            case "dragleave":
            case "dragover":
            case "dragstart":
            case "drop":
              De = Z_;
              break;
            case "touchcancel":
            case "touchend":
            case "touchmove":
            case "touchstart":
              De = hk;
              break;
            case Jv:
            case Yv:
            case Qv:
              De = nk;
              break;
            case Xv:
              De = gk;
              break;
            case "scroll":
              De = Q_;
              break;
            case "wheel":
              De = vk;
              break;
            case "copy":
            case "cut":
            case "paste":
              De = ok;
              break;
            case "gotpointercapture":
            case "lostpointercapture":
            case "pointercancel":
            case "pointerdown":
            case "pointermove":
            case "pointerout":
            case "pointerover":
            case "pointerup":
              De = Iv;
          }
          var Ke = (a & 4) !== 0, Cn = !Ke && r === "scroll", le = Ke ? xe !== null ? xe + "Capture" : null : xe;
          Ke = [];
          for (var oe = he, fe; oe !== null; ) {
            fe = oe;
            var ke = fe.stateNode;
            if (fe.tag === 5 && ke !== null && (fe = ke, le !== null && (ke = cr(oe, le), ke != null && Ke.push(eu(oe, ke, fe)))), Cn) break;
            oe = oe.return;
          }
          0 < Ke.length && (xe = new De(xe, qe, null, d, Ee), _e.push({ event: xe, listeners: Ke }));
        }
      }
      if (!(a & 7)) {
        e: {
          if (xe = r === "mouseover" || r === "pointerover", De = r === "mouseout" || r === "pointerout", xe && d !== Gt && (qe = d.relatedTarget || d.fromElement) && (ya(qe) || qe[oi])) break e;
          if ((De || xe) && (xe = Ee.window === Ee ? Ee : (xe = Ee.ownerDocument) ? xe.defaultView || xe.parentWindow : window, De ? (qe = d.relatedTarget || d.toElement, De = he, qe = qe ? ya(qe) : null, qe !== null && (Cn = yn(qe), qe !== Cn || qe.tag !== 5 && qe.tag !== 6) && (qe = null)) : (De = null, qe = he), De !== qe)) {
            if (Ke = Ov, ke = "onMouseLeave", le = "onMouseEnter", oe = "mouse", (r === "pointerout" || r === "pointerover") && (Ke = Iv, ke = "onPointerLeave", le = "onPointerEnter", oe = "pointer"), Cn = De == null ? xe : gs(De), fe = qe == null ? xe : gs(qe), xe = new Ke(ke, oe + "leave", De, d, Ee), xe.target = Cn, xe.relatedTarget = fe, ke = null, ya(Ee) === he && (Ke = new Ke(le, oe + "enter", qe, d, Ee), Ke.target = fe, Ke.relatedTarget = Cn, ke = Ke), Cn = ke, De && qe) t: {
              for (Ke = De, le = qe, oe = 0, fe = Ke; fe; fe = hs(fe)) oe++;
              for (fe = 0, ke = le; ke; ke = hs(ke)) fe++;
              for (; 0 < oe - fe; ) Ke = hs(Ke), oe--;
              for (; 0 < fe - oe; ) le = hs(le), fe--;
              for (; oe--; ) {
                if (Ke === le || le !== null && Ke === le.alternate) break t;
                Ke = hs(Ke), le = hs(le);
              }
              Ke = null;
            }
            else Ke = null;
            De !== null && o1(_e, xe, De, Ke, !1), qe !== null && Cn !== null && o1(_e, Cn, qe, Ke, !0);
          }
        }
        e: {
          if (xe = he ? gs(he) : window, De = xe.nodeName && xe.nodeName.toLowerCase(), De === "select" || De === "input" && xe.type === "file") var He = Ek;
          else if (Fv(xe)) if (Dv) He = Ok;
          else {
            He = kk;
            var nt = _k;
          }
          else (De = xe.nodeName) && De.toLowerCase() === "input" && (xe.type === "checkbox" || xe.type === "radio") && (He = Pk);
          if (He && (He = He(r, he))) {
            Lv(_e, He, d, Ee);
            break e;
          }
          nt && nt(r, xe, he), r === "focusout" && (nt = xe._wrapperState) && nt.controlled && xe.type === "number" && Se(xe, "number", xe.value);
        }
        switch (nt = he ? gs(he) : window, r) {
          case "focusin":
            (Fv(nt) || nt.contentEditable === "true") && (fs = nt, fh = he, Ql = null);
            break;
          case "focusout":
            Ql = fh = fs = null;
            break;
          case "mousedown":
            ph = !0;
            break;
          case "contextmenu":
          case "mouseup":
          case "dragend":
            ph = !1, Hv(_e, d, Ee);
            break;
          case "selectionchange":
            if (jk) break;
          case "keydown":
          case "keyup":
            Hv(_e, d, Ee);
        }
        var rt;
        if (lh) e: {
          switch (r) {
            case "compositionstart":
              var at = "onCompositionStart";
              break e;
            case "compositionend":
              at = "onCompositionEnd";
              break e;
            case "compositionupdate":
              at = "onCompositionUpdate";
              break e;
          }
          at = void 0;
        }
        else ds ? Mv(r, d) && (at = "onCompositionEnd") : r === "keydown" && d.keyCode === 229 && (at = "onCompositionStart");
        at && (jv && d.locale !== "ko" && (ds || at !== "onCompositionStart" ? at === "onCompositionEnd" && ds && (rt = kv()) : (mr = Ee, us = "value" in mr ? mr.value : mr.textContent, ds = !0)), nt = Xc(he, at), 0 < nt.length && (at = new Av(at, r, null, d, Ee), _e.push({ event: at, listeners: nt }), rt ? at.data = rt : (rt = Nv(d), rt !== null && (at.data = rt)))), (rt = Sk ? wk(r, d) : $k(r, d)) && (he = Xc(he, "onBeforeInput"), 0 < he.length && (Ee = new Av("onBeforeInput", "beforeinput", null, d, Ee), _e.push({ event: Ee, listeners: he }), Ee.data = rt));
      }
      n1(_e, a);
    });
  }
  function eu(r, a, d) {
    return { instance: r, listener: a, currentTarget: d };
  }
  function Xc(r, a) {
    for (var d = a + "Capture", m = []; r !== null; ) {
      var x = r, P = x.stateNode;
      x.tag === 5 && P !== null && (x = P, P = cr(r, d), P != null && m.unshift(eu(r, P, x)), P = cr(r, a), P != null && m.push(eu(r, P, x))), r = r.return;
    }
    return m;
  }
  function hs(r) {
    if (r === null) return null;
    do
      r = r.return;
    while (r && r.tag !== 5);
    return r || null;
  }
  function o1(r, a, d, m, x) {
    for (var P = a._reactName, M = []; d !== null && d !== m; ) {
      var K = d, te = K.alternate, he = K.stateNode;
      if (te !== null && te === m) break;
      K.tag === 5 && he !== null && (K = he, x ? (te = cr(d, P), te != null && M.unshift(eu(d, te, K))) : x || (te = cr(d, P), te != null && M.push(eu(d, te, K)))), d = d.return;
    }
    M.length !== 0 && r.push({ event: a, listeners: M });
  }
  var Nk = /\r\n?/g, Fk = /\u0000|\uFFFD/g;
  function i1(r) {
    return (typeof r == "string" ? r : "" + r).replace(Nk, `
`).replace(Fk, "");
  }
  function Zc(r, a, d) {
    if (a = i1(a), i1(r) !== a && d) throw Error(n(425));
  }
  function ed() {
  }
  var bh = null, Sh = null;
  function wh(r, a) {
    return r === "textarea" || r === "noscript" || typeof a.children == "string" || typeof a.children == "number" || typeof a.dangerouslySetInnerHTML == "object" && a.dangerouslySetInnerHTML !== null && a.dangerouslySetInnerHTML.__html != null;
  }
  var $h = typeof setTimeout == "function" ? setTimeout : void 0, Lk = typeof clearTimeout == "function" ? clearTimeout : void 0, a1 = typeof Promise == "function" ? Promise : void 0, Dk = typeof queueMicrotask == "function" ? queueMicrotask : typeof a1 < "u" ? function(r) {
    return a1.resolve(null).then(r).catch(zk);
  } : $h;
  function zk(r) {
    setTimeout(function() {
      throw r;
    });
  }
  function Ch(r, a) {
    var d = a, m = 0;
    do {
      var x = d.nextSibling;
      if (r.removeChild(d), x && x.nodeType === 8) if (d = x.data, d === "/$") {
        if (m === 0) {
          r.removeChild(x), Mn(a);
          return;
        }
        m--;
      } else d !== "$" && d !== "$?" && d !== "$!" || m++;
      d = x;
    } while (d);
    Mn(a);
  }
  function Ii(r) {
    for (; r != null; r = r.nextSibling) {
      var a = r.nodeType;
      if (a === 1 || a === 3) break;
      if (a === 8) {
        if (a = r.data, a === "$" || a === "$!" || a === "$?") break;
        if (a === "/$") return null;
      }
    }
    return r;
  }
  function s1(r) {
    r = r.previousSibling;
    for (var a = 0; r; ) {
      if (r.nodeType === 8) {
        var d = r.data;
        if (d === "$" || d === "$!" || d === "$?") {
          if (a === 0) return r;
          a--;
        } else d === "/$" && a++;
      }
      r = r.previousSibling;
    }
    return null;
  }
  var ms = Math.random().toString(36).slice(2), No = "__reactFiber$" + ms, tu = "__reactProps$" + ms, oi = "__reactContainer$" + ms, xh = "__reactEvents$" + ms, Bk = "__reactListeners$" + ms, Uk = "__reactHandles$" + ms;
  function ya(r) {
    var a = r[No];
    if (a) return a;
    for (var d = r.parentNode; d; ) {
      if (a = d[oi] || d[No]) {
        if (d = a.alternate, a.child !== null || d !== null && d.child !== null) for (r = s1(r); r !== null; ) {
          if (d = r[No]) return d;
          r = s1(r);
        }
        return a;
      }
      r = d, d = r.parentNode;
    }
    return null;
  }
  function nu(r) {
    return r = r[No] || r[oi], !r || r.tag !== 5 && r.tag !== 6 && r.tag !== 13 && r.tag !== 3 ? null : r;
  }
  function gs(r) {
    if (r.tag === 5 || r.tag === 6) return r.stateNode;
    throw Error(n(33));
  }
  function td(r) {
    return r[tu] || null;
  }
  var Eh = [], ys = -1;
  function ji(r) {
    return { current: r };
  }
  function sn(r) {
    0 > ys || (r.current = Eh[ys], Eh[ys] = null, ys--);
  }
  function nn(r, a) {
    ys++, Eh[ys] = r.current, r.current = a;
  }
  var Ri = {}, tr = ji(Ri), _r = ji(!1), va = Ri;
  function vs(r, a) {
    var d = r.type.contextTypes;
    if (!d) return Ri;
    var m = r.stateNode;
    if (m && m.__reactInternalMemoizedUnmaskedChildContext === a) return m.__reactInternalMemoizedMaskedChildContext;
    var x = {}, P;
    for (P in d) x[P] = a[P];
    return m && (r = r.stateNode, r.__reactInternalMemoizedUnmaskedChildContext = a, r.__reactInternalMemoizedMaskedChildContext = x), x;
  }
  function kr(r) {
    return r = r.childContextTypes, r != null;
  }
  function nd() {
    sn(_r), sn(tr);
  }
  function l1(r, a, d) {
    if (tr.current !== Ri) throw Error(n(168));
    nn(tr, a), nn(_r, d);
  }
  function u1(r, a, d) {
    var m = r.stateNode;
    if (a = a.childContextTypes, typeof m.getChildContext != "function") return d;
    m = m.getChildContext();
    for (var x in m) if (!(x in a)) throw Error(n(108, me(r) || "Unknown", x));
    return Y({}, d, m);
  }
  function rd(r) {
    return r = (r = r.stateNode) && r.__reactInternalMemoizedMergedChildContext || Ri, va = tr.current, nn(tr, r), nn(_r, _r.current), !0;
  }
  function c1(r, a, d) {
    var m = r.stateNode;
    if (!m) throw Error(n(169));
    d ? (r = u1(r, a, va), m.__reactInternalMemoizedMergedChildContext = r, sn(_r), sn(tr), nn(tr, r)) : sn(_r), nn(_r, d);
  }
  var ii = null, od = !1, _h = !1;
  function d1(r) {
    ii === null ? ii = [r] : ii.push(r);
  }
  function Vk(r) {
    od = !0, d1(r);
  }
  function Ti() {
    if (!_h && ii !== null) {
      _h = !0;
      var r = 0, a = I;
      try {
        var d = ii;
        for (I = 1; r < d.length; r++) {
          var m = d[r];
          do
            m = m(!0);
          while (m !== null);
        }
        ii = null, od = !1;
      } catch (x) {
        throw ii !== null && (ii = ii.slice(r + 1)), da(zl, Ti), x;
      } finally {
        I = a, _h = !1;
      }
    }
    return null;
  }
  var bs = [], Ss = 0, id = null, ad = 0, Zr = [], eo = 0, ba = null, ai = 1, si = "";
  function Sa(r, a) {
    bs[Ss++] = ad, bs[Ss++] = id, id = r, ad = a;
  }
  function f1(r, a, d) {
    Zr[eo++] = ai, Zr[eo++] = si, Zr[eo++] = ba, ba = r;
    var m = ai;
    r = si;
    var x = 32 - xr(m) - 1;
    m &= ~(1 << x), d += 1;
    var P = 32 - xr(a) + x;
    if (30 < P) {
      var M = x - x % 5;
      P = (m & (1 << M) - 1).toString(32), m >>= M, x -= M, ai = 1 << 32 - xr(a) + x | d << x | m, si = P + r;
    } else ai = 1 << P | d << x | m, si = r;
  }
  function kh(r) {
    r.return !== null && (Sa(r, 1), f1(r, 1, 0));
  }
  function Ph(r) {
    for (; r === id; ) id = bs[--Ss], bs[Ss] = null, ad = bs[--Ss], bs[Ss] = null;
    for (; r === ba; ) ba = Zr[--eo], Zr[eo] = null, si = Zr[--eo], Zr[eo] = null, ai = Zr[--eo], Zr[eo] = null;
  }
  var zr = null, Br = null, pn = !1, $o = null;
  function p1(r, a) {
    var d = oo(5, null, null, 0);
    d.elementType = "DELETED", d.stateNode = a, d.return = r, a = r.deletions, a === null ? (r.deletions = [d], r.flags |= 16) : a.push(d);
  }
  function h1(r, a) {
    switch (r.tag) {
      case 5:
        var d = r.type;
        return a = a.nodeType !== 1 || d.toLowerCase() !== a.nodeName.toLowerCase() ? null : a, a !== null ? (r.stateNode = a, zr = r, Br = Ii(a.firstChild), !0) : !1;
      case 6:
        return a = r.pendingProps === "" || a.nodeType !== 3 ? null : a, a !== null ? (r.stateNode = a, zr = r, Br = null, !0) : !1;
      case 13:
        return a = a.nodeType !== 8 ? null : a, a !== null ? (d = ba !== null ? { id: ai, overflow: si } : null, r.memoizedState = { dehydrated: a, treeContext: d, retryLane: 1073741824 }, d = oo(18, null, null, 0), d.stateNode = a, d.return = r, r.child = d, zr = r, Br = null, !0) : !1;
      default:
        return !1;
    }
  }
  function Oh(r) {
    return (r.mode & 1) !== 0 && (r.flags & 128) === 0;
  }
  function Ah(r) {
    if (pn) {
      var a = Br;
      if (a) {
        var d = a;
        if (!h1(r, a)) {
          if (Oh(r)) throw Error(n(418));
          a = Ii(d.nextSibling);
          var m = zr;
          a && h1(r, a) ? p1(m, d) : (r.flags = r.flags & -4097 | 2, pn = !1, zr = r);
        }
      } else {
        if (Oh(r)) throw Error(n(418));
        r.flags = r.flags & -4097 | 2, pn = !1, zr = r;
      }
    }
  }
  function m1(r) {
    for (r = r.return; r !== null && r.tag !== 5 && r.tag !== 3 && r.tag !== 13; ) r = r.return;
    zr = r;
  }
  function sd(r) {
    if (r !== zr) return !1;
    if (!pn) return m1(r), pn = !0, !1;
    var a;
    if ((a = r.tag !== 3) && !(a = r.tag !== 5) && (a = r.type, a = a !== "head" && a !== "body" && !wh(r.type, r.memoizedProps)), a && (a = Br)) {
      if (Oh(r)) throw g1(), Error(n(418));
      for (; a; ) p1(r, a), a = Ii(a.nextSibling);
    }
    if (m1(r), r.tag === 13) {
      if (r = r.memoizedState, r = r !== null ? r.dehydrated : null, !r) throw Error(n(317));
      e: {
        for (r = r.nextSibling, a = 0; r; ) {
          if (r.nodeType === 8) {
            var d = r.data;
            if (d === "/$") {
              if (a === 0) {
                Br = Ii(r.nextSibling);
                break e;
              }
              a--;
            } else d !== "$" && d !== "$!" && d !== "$?" || a++;
          }
          r = r.nextSibling;
        }
        Br = null;
      }
    } else Br = zr ? Ii(r.stateNode.nextSibling) : null;
    return !0;
  }
  function g1() {
    for (var r = Br; r; ) r = Ii(r.nextSibling);
  }
  function ws() {
    Br = zr = null, pn = !1;
  }
  function Ih(r) {
    $o === null ? $o = [r] : $o.push(r);
  }
  var Wk = b.ReactCurrentBatchConfig;
  function ru(r, a, d) {
    if (r = d.ref, r !== null && typeof r != "function" && typeof r != "object") {
      if (d._owner) {
        if (d = d._owner, d) {
          if (d.tag !== 1) throw Error(n(309));
          var m = d.stateNode;
        }
        if (!m) throw Error(n(147, r));
        var x = m, P = "" + r;
        return a !== null && a.ref !== null && typeof a.ref == "function" && a.ref._stringRef === P ? a.ref : (a = function(M) {
          var K = x.refs;
          M === null ? delete K[P] : K[P] = M;
        }, a._stringRef = P, a);
      }
      if (typeof r != "string") throw Error(n(284));
      if (!d._owner) throw Error(n(290, r));
    }
    return r;
  }
  function ld(r, a) {
    throw r = Object.prototype.toString.call(a), Error(n(31, r === "[object Object]" ? "object with keys {" + Object.keys(a).join(", ") + "}" : r));
  }
  function y1(r) {
    var a = r._init;
    return a(r._payload);
  }
  function v1(r) {
    function a(le, oe) {
      if (r) {
        var fe = le.deletions;
        fe === null ? (le.deletions = [oe], le.flags |= 16) : fe.push(oe);
      }
    }
    function d(le, oe) {
      if (!r) return null;
      for (; oe !== null; ) a(le, oe), oe = oe.sibling;
      return null;
    }
    function m(le, oe) {
      for (le = /* @__PURE__ */ new Map(); oe !== null; ) oe.key !== null ? le.set(oe.key, oe) : le.set(oe.index, oe), oe = oe.sibling;
      return le;
    }
    function x(le, oe) {
      return le = Ui(le, oe), le.index = 0, le.sibling = null, le;
    }
    function P(le, oe, fe) {
      return le.index = fe, r ? (fe = le.alternate, fe !== null ? (fe = fe.index, fe < oe ? (le.flags |= 2, oe) : fe) : (le.flags |= 2, oe)) : (le.flags |= 1048576, oe);
    }
    function M(le) {
      return r && le.alternate === null && (le.flags |= 2), le;
    }
    function K(le, oe, fe, ke) {
      return oe === null || oe.tag !== 6 ? (oe = $m(fe, le.mode, ke), oe.return = le, oe) : (oe = x(oe, fe), oe.return = le, oe);
    }
    function te(le, oe, fe, ke) {
      var He = fe.type;
      return He === j ? Ee(le, oe, fe.props.children, ke, fe.key) : oe !== null && (oe.elementType === He || typeof He == "object" && He !== null && He.$$typeof === ce && y1(He) === oe.type) ? (ke = x(oe, fe.props), ke.ref = ru(le, oe, fe), ke.return = le, ke) : (ke = jd(fe.type, fe.key, fe.props, null, le.mode, ke), ke.ref = ru(le, oe, fe), ke.return = le, ke);
    }
    function he(le, oe, fe, ke) {
      return oe === null || oe.tag !== 4 || oe.stateNode.containerInfo !== fe.containerInfo || oe.stateNode.implementation !== fe.implementation ? (oe = Cm(fe, le.mode, ke), oe.return = le, oe) : (oe = x(oe, fe.children || []), oe.return = le, oe);
    }
    function Ee(le, oe, fe, ke, He) {
      return oe === null || oe.tag !== 7 ? (oe = Pa(fe, le.mode, ke, He), oe.return = le, oe) : (oe = x(oe, fe), oe.return = le, oe);
    }
    function _e(le, oe, fe) {
      if (typeof oe == "string" && oe !== "" || typeof oe == "number") return oe = $m("" + oe, le.mode, fe), oe.return = le, oe;
      if (typeof oe == "object" && oe !== null) {
        switch (oe.$$typeof) {
          case k:
            return fe = jd(oe.type, oe.key, oe.props, null, le.mode, fe), fe.ref = ru(le, null, oe), fe.return = le, fe;
          case E:
            return oe = Cm(oe, le.mode, fe), oe.return = le, oe;
          case ce:
            var ke = oe._init;
            return _e(le, ke(oe._payload), fe);
        }
        if (gt(oe) || re(oe)) return oe = Pa(oe, le.mode, fe, null), oe.return = le, oe;
        ld(le, oe);
      }
      return null;
    }
    function xe(le, oe, fe, ke) {
      var He = oe !== null ? oe.key : null;
      if (typeof fe == "string" && fe !== "" || typeof fe == "number") return He !== null ? null : K(le, oe, "" + fe, ke);
      if (typeof fe == "object" && fe !== null) {
        switch (fe.$$typeof) {
          case k:
            return fe.key === He ? te(le, oe, fe, ke) : null;
          case E:
            return fe.key === He ? he(le, oe, fe, ke) : null;
          case ce:
            return He = fe._init, xe(
              le,
              oe,
              He(fe._payload),
              ke
            );
        }
        if (gt(fe) || re(fe)) return He !== null ? null : Ee(le, oe, fe, ke, null);
        ld(le, fe);
      }
      return null;
    }
    function De(le, oe, fe, ke, He) {
      if (typeof ke == "string" && ke !== "" || typeof ke == "number") return le = le.get(fe) || null, K(oe, le, "" + ke, He);
      if (typeof ke == "object" && ke !== null) {
        switch (ke.$$typeof) {
          case k:
            return le = le.get(ke.key === null ? fe : ke.key) || null, te(oe, le, ke, He);
          case E:
            return le = le.get(ke.key === null ? fe : ke.key) || null, he(oe, le, ke, He);
          case ce:
            var nt = ke._init;
            return De(le, oe, fe, nt(ke._payload), He);
        }
        if (gt(ke) || re(ke)) return le = le.get(fe) || null, Ee(oe, le, ke, He, null);
        ld(oe, ke);
      }
      return null;
    }
    function qe(le, oe, fe, ke) {
      for (var He = null, nt = null, rt = oe, at = oe = 0, Wn = null; rt !== null && at < fe.length; at++) {
        rt.index > at ? (Wn = rt, rt = null) : Wn = rt.sibling;
        var Nt = xe(le, rt, fe[at], ke);
        if (Nt === null) {
          rt === null && (rt = Wn);
          break;
        }
        r && rt && Nt.alternate === null && a(le, rt), oe = P(Nt, oe, at), nt === null ? He = Nt : nt.sibling = Nt, nt = Nt, rt = Wn;
      }
      if (at === fe.length) return d(le, rt), pn && Sa(le, at), He;
      if (rt === null) {
        for (; at < fe.length; at++) rt = _e(le, fe[at], ke), rt !== null && (oe = P(rt, oe, at), nt === null ? He = rt : nt.sibling = rt, nt = rt);
        return pn && Sa(le, at), He;
      }
      for (rt = m(le, rt); at < fe.length; at++) Wn = De(rt, le, at, fe[at], ke), Wn !== null && (r && Wn.alternate !== null && rt.delete(Wn.key === null ? at : Wn.key), oe = P(Wn, oe, at), nt === null ? He = Wn : nt.sibling = Wn, nt = Wn);
      return r && rt.forEach(function(Vi) {
        return a(le, Vi);
      }), pn && Sa(le, at), He;
    }
    function Ke(le, oe, fe, ke) {
      var He = re(fe);
      if (typeof He != "function") throw Error(n(150));
      if (fe = He.call(fe), fe == null) throw Error(n(151));
      for (var nt = He = null, rt = oe, at = oe = 0, Wn = null, Nt = fe.next(); rt !== null && !Nt.done; at++, Nt = fe.next()) {
        rt.index > at ? (Wn = rt, rt = null) : Wn = rt.sibling;
        var Vi = xe(le, rt, Nt.value, ke);
        if (Vi === null) {
          rt === null && (rt = Wn);
          break;
        }
        r && rt && Vi.alternate === null && a(le, rt), oe = P(Vi, oe, at), nt === null ? He = Vi : nt.sibling = Vi, nt = Vi, rt = Wn;
      }
      if (Nt.done) return d(
        le,
        rt
      ), pn && Sa(le, at), He;
      if (rt === null) {
        for (; !Nt.done; at++, Nt = fe.next()) Nt = _e(le, Nt.value, ke), Nt !== null && (oe = P(Nt, oe, at), nt === null ? He = Nt : nt.sibling = Nt, nt = Nt);
        return pn && Sa(le, at), He;
      }
      for (rt = m(le, rt); !Nt.done; at++, Nt = fe.next()) Nt = De(rt, le, at, Nt.value, ke), Nt !== null && (r && Nt.alternate !== null && rt.delete(Nt.key === null ? at : Nt.key), oe = P(Nt, oe, at), nt === null ? He = Nt : nt.sibling = Nt, nt = Nt);
      return r && rt.forEach(function($5) {
        return a(le, $5);
      }), pn && Sa(le, at), He;
    }
    function Cn(le, oe, fe, ke) {
      if (typeof fe == "object" && fe !== null && fe.type === j && fe.key === null && (fe = fe.props.children), typeof fe == "object" && fe !== null) {
        switch (fe.$$typeof) {
          case k:
            e: {
              for (var He = fe.key, nt = oe; nt !== null; ) {
                if (nt.key === He) {
                  if (He = fe.type, He === j) {
                    if (nt.tag === 7) {
                      d(le, nt.sibling), oe = x(nt, fe.props.children), oe.return = le, le = oe;
                      break e;
                    }
                  } else if (nt.elementType === He || typeof He == "object" && He !== null && He.$$typeof === ce && y1(He) === nt.type) {
                    d(le, nt.sibling), oe = x(nt, fe.props), oe.ref = ru(le, nt, fe), oe.return = le, le = oe;
                    break e;
                  }
                  d(le, nt);
                  break;
                } else a(le, nt);
                nt = nt.sibling;
              }
              fe.type === j ? (oe = Pa(fe.props.children, le.mode, ke, fe.key), oe.return = le, le = oe) : (ke = jd(fe.type, fe.key, fe.props, null, le.mode, ke), ke.ref = ru(le, oe, fe), ke.return = le, le = ke);
            }
            return M(le);
          case E:
            e: {
              for (nt = fe.key; oe !== null; ) {
                if (oe.key === nt) if (oe.tag === 4 && oe.stateNode.containerInfo === fe.containerInfo && oe.stateNode.implementation === fe.implementation) {
                  d(le, oe.sibling), oe = x(oe, fe.children || []), oe.return = le, le = oe;
                  break e;
                } else {
                  d(le, oe);
                  break;
                }
                else a(le, oe);
                oe = oe.sibling;
              }
              oe = Cm(fe, le.mode, ke), oe.return = le, le = oe;
            }
            return M(le);
          case ce:
            return nt = fe._init, Cn(le, oe, nt(fe._payload), ke);
        }
        if (gt(fe)) return qe(le, oe, fe, ke);
        if (re(fe)) return Ke(le, oe, fe, ke);
        ld(le, fe);
      }
      return typeof fe == "string" && fe !== "" || typeof fe == "number" ? (fe = "" + fe, oe !== null && oe.tag === 6 ? (d(le, oe.sibling), oe = x(oe, fe), oe.return = le, le = oe) : (d(le, oe), oe = $m(fe, le.mode, ke), oe.return = le, le = oe), M(le)) : d(le, oe);
    }
    return Cn;
  }
  var $s = v1(!0), b1 = v1(!1), ud = ji(null), cd = null, Cs = null, jh = null;
  function Rh() {
    jh = Cs = cd = null;
  }
  function Th(r) {
    var a = ud.current;
    sn(ud), r._currentValue = a;
  }
  function Mh(r, a, d) {
    for (; r !== null; ) {
      var m = r.alternate;
      if ((r.childLanes & a) !== a ? (r.childLanes |= a, m !== null && (m.childLanes |= a)) : m !== null && (m.childLanes & a) !== a && (m.childLanes |= a), r === d) break;
      r = r.return;
    }
  }
  function xs(r, a) {
    cd = r, jh = Cs = null, r = r.dependencies, r !== null && r.firstContext !== null && (r.lanes & a && (Pr = !0), r.firstContext = null);
  }
  function to(r) {
    var a = r._currentValue;
    if (jh !== r) if (r = { context: r, memoizedValue: a, next: null }, Cs === null) {
      if (cd === null) throw Error(n(308));
      Cs = r, cd.dependencies = { lanes: 0, firstContext: r };
    } else Cs = Cs.next = r;
    return a;
  }
  var wa = null;
  function Nh(r) {
    wa === null ? wa = [r] : wa.push(r);
  }
  function S1(r, a, d, m) {
    var x = a.interleaved;
    return x === null ? (d.next = d, Nh(a)) : (d.next = x.next, x.next = d), a.interleaved = d, li(r, m);
  }
  function li(r, a) {
    r.lanes |= a;
    var d = r.alternate;
    for (d !== null && (d.lanes |= a), d = r, r = r.return; r !== null; ) r.childLanes |= a, d = r.alternate, d !== null && (d.childLanes |= a), d = r, r = r.return;
    return d.tag === 3 ? d.stateNode : null;
  }
  var Mi = !1;
  function Fh(r) {
    r.updateQueue = { baseState: r.memoizedState, firstBaseUpdate: null, lastBaseUpdate: null, shared: { pending: null, interleaved: null, lanes: 0 }, effects: null };
  }
  function w1(r, a) {
    r = r.updateQueue, a.updateQueue === r && (a.updateQueue = { baseState: r.baseState, firstBaseUpdate: r.firstBaseUpdate, lastBaseUpdate: r.lastBaseUpdate, shared: r.shared, effects: r.effects });
  }
  function ui(r, a) {
    return { eventTime: r, lane: a, tag: 0, payload: null, callback: null, next: null };
  }
  function Ni(r, a, d) {
    var m = r.updateQueue;
    if (m === null) return null;
    if (m = m.shared, Tt & 2) {
      var x = m.pending;
      return x === null ? a.next = a : (a.next = x.next, x.next = a), m.pending = a, li(r, d);
    }
    return x = m.interleaved, x === null ? (a.next = a, Nh(m)) : (a.next = x.next, x.next = a), m.interleaved = a, li(r, d);
  }
  function dd(r, a, d) {
    if (a = a.updateQueue, a !== null && (a = a.shared, (d & 4194240) !== 0)) {
      var m = a.lanes;
      m &= r.pendingLanes, d |= m, a.lanes = d, D(r, d);
    }
  }
  function $1(r, a) {
    var d = r.updateQueue, m = r.alternate;
    if (m !== null && (m = m.updateQueue, d === m)) {
      var x = null, P = null;
      if (d = d.firstBaseUpdate, d !== null) {
        do {
          var M = { eventTime: d.eventTime, lane: d.lane, tag: d.tag, payload: d.payload, callback: d.callback, next: null };
          P === null ? x = P = M : P = P.next = M, d = d.next;
        } while (d !== null);
        P === null ? x = P = a : P = P.next = a;
      } else x = P = a;
      d = { baseState: m.baseState, firstBaseUpdate: x, lastBaseUpdate: P, shared: m.shared, effects: m.effects }, r.updateQueue = d;
      return;
    }
    r = d.lastBaseUpdate, r === null ? d.firstBaseUpdate = a : r.next = a, d.lastBaseUpdate = a;
  }
  function fd(r, a, d, m) {
    var x = r.updateQueue;
    Mi = !1;
    var P = x.firstBaseUpdate, M = x.lastBaseUpdate, K = x.shared.pending;
    if (K !== null) {
      x.shared.pending = null;
      var te = K, he = te.next;
      te.next = null, M === null ? P = he : M.next = he, M = te;
      var Ee = r.alternate;
      Ee !== null && (Ee = Ee.updateQueue, K = Ee.lastBaseUpdate, K !== M && (K === null ? Ee.firstBaseUpdate = he : K.next = he, Ee.lastBaseUpdate = te));
    }
    if (P !== null) {
      var _e = x.baseState;
      M = 0, Ee = he = te = null, K = P;
      do {
        var xe = K.lane, De = K.eventTime;
        if ((m & xe) === xe) {
          Ee !== null && (Ee = Ee.next = {
            eventTime: De,
            lane: 0,
            tag: K.tag,
            payload: K.payload,
            callback: K.callback,
            next: null
          });
          e: {
            var qe = r, Ke = K;
            switch (xe = a, De = d, Ke.tag) {
              case 1:
                if (qe = Ke.payload, typeof qe == "function") {
                  _e = qe.call(De, _e, xe);
                  break e;
                }
                _e = qe;
                break e;
              case 3:
                qe.flags = qe.flags & -65537 | 128;
              case 0:
                if (qe = Ke.payload, xe = typeof qe == "function" ? qe.call(De, _e, xe) : qe, xe == null) break e;
                _e = Y({}, _e, xe);
                break e;
              case 2:
                Mi = !0;
            }
          }
          K.callback !== null && K.lane !== 0 && (r.flags |= 64, xe = x.effects, xe === null ? x.effects = [K] : xe.push(K));
        } else De = { eventTime: De, lane: xe, tag: K.tag, payload: K.payload, callback: K.callback, next: null }, Ee === null ? (he = Ee = De, te = _e) : Ee = Ee.next = De, M |= xe;
        if (K = K.next, K === null) {
          if (K = x.shared.pending, K === null) break;
          xe = K, K = xe.next, xe.next = null, x.lastBaseUpdate = xe, x.shared.pending = null;
        }
      } while (!0);
      if (Ee === null && (te = _e), x.baseState = te, x.firstBaseUpdate = he, x.lastBaseUpdate = Ee, a = x.shared.interleaved, a !== null) {
        x = a;
        do
          M |= x.lane, x = x.next;
        while (x !== a);
      } else P === null && (x.shared.lanes = 0);
      xa |= M, r.lanes = M, r.memoizedState = _e;
    }
  }
  function C1(r, a, d) {
    if (r = a.effects, a.effects = null, r !== null) for (a = 0; a < r.length; a++) {
      var m = r[a], x = m.callback;
      if (x !== null) {
        if (m.callback = null, m = d, typeof x != "function") throw Error(n(191, x));
        x.call(m);
      }
    }
  }
  var ou = {}, Fo = ji(ou), iu = ji(ou), au = ji(ou);
  function $a(r) {
    if (r === ou) throw Error(n(174));
    return r;
  }
  function Lh(r, a) {
    switch (nn(au, a), nn(iu, r), nn(Fo, ou), r = a.nodeType, r) {
      case 9:
      case 11:
        a = (a = a.documentElement) ? a.namespaceURI : Ye(null, "");
        break;
      default:
        r = r === 8 ? a.parentNode : a, a = r.namespaceURI || null, r = r.tagName, a = Ye(a, r);
    }
    sn(Fo), nn(Fo, a);
  }
  function Es() {
    sn(Fo), sn(iu), sn(au);
  }
  function x1(r) {
    $a(au.current);
    var a = $a(Fo.current), d = Ye(a, r.type);
    a !== d && (nn(iu, r), nn(Fo, d));
  }
  function Dh(r) {
    iu.current === r && (sn(Fo), sn(iu));
  }
  var mn = ji(0);
  function pd(r) {
    for (var a = r; a !== null; ) {
      if (a.tag === 13) {
        var d = a.memoizedState;
        if (d !== null && (d = d.dehydrated, d === null || d.data === "$?" || d.data === "$!")) return a;
      } else if (a.tag === 19 && a.memoizedProps.revealOrder !== void 0) {
        if (a.flags & 128) return a;
      } else if (a.child !== null) {
        a.child.return = a, a = a.child;
        continue;
      }
      if (a === r) break;
      for (; a.sibling === null; ) {
        if (a.return === null || a.return === r) return null;
        a = a.return;
      }
      a.sibling.return = a.return, a = a.sibling;
    }
    return null;
  }
  var zh = [];
  function Bh() {
    for (var r = 0; r < zh.length; r++) zh[r]._workInProgressVersionPrimary = null;
    zh.length = 0;
  }
  var hd = b.ReactCurrentDispatcher, Uh = b.ReactCurrentBatchConfig, Ca = 0, gn = null, Nn = null, Un = null, md = !1, su = !1, lu = 0, qk = 0;
  function nr() {
    throw Error(n(321));
  }
  function Vh(r, a) {
    if (a === null) return !1;
    for (var d = 0; d < a.length && d < r.length; d++) if (!wo(r[d], a[d])) return !1;
    return !0;
  }
  function Wh(r, a, d, m, x, P) {
    if (Ca = P, gn = a, a.memoizedState = null, a.updateQueue = null, a.lanes = 0, hd.current = r === null || r.memoizedState === null ? Jk : Yk, r = d(m, x), su) {
      P = 0;
      do {
        if (su = !1, lu = 0, 25 <= P) throw Error(n(301));
        P += 1, Un = Nn = null, a.updateQueue = null, hd.current = Qk, r = d(m, x);
      } while (su);
    }
    if (hd.current = vd, a = Nn !== null && Nn.next !== null, Ca = 0, Un = Nn = gn = null, md = !1, a) throw Error(n(300));
    return r;
  }
  function qh() {
    var r = lu !== 0;
    return lu = 0, r;
  }
  function Lo() {
    var r = { memoizedState: null, baseState: null, baseQueue: null, queue: null, next: null };
    return Un === null ? gn.memoizedState = Un = r : Un = Un.next = r, Un;
  }
  function no() {
    if (Nn === null) {
      var r = gn.alternate;
      r = r !== null ? r.memoizedState : null;
    } else r = Nn.next;
    var a = Un === null ? gn.memoizedState : Un.next;
    if (a !== null) Un = a, Nn = r;
    else {
      if (r === null) throw Error(n(310));
      Nn = r, r = { memoizedState: Nn.memoizedState, baseState: Nn.baseState, baseQueue: Nn.baseQueue, queue: Nn.queue, next: null }, Un === null ? gn.memoizedState = Un = r : Un = Un.next = r;
    }
    return Un;
  }
  function uu(r, a) {
    return typeof a == "function" ? a(r) : a;
  }
  function Kh(r) {
    var a = no(), d = a.queue;
    if (d === null) throw Error(n(311));
    d.lastRenderedReducer = r;
    var m = Nn, x = m.baseQueue, P = d.pending;
    if (P !== null) {
      if (x !== null) {
        var M = x.next;
        x.next = P.next, P.next = M;
      }
      m.baseQueue = x = P, d.pending = null;
    }
    if (x !== null) {
      P = x.next, m = m.baseState;
      var K = M = null, te = null, he = P;
      do {
        var Ee = he.lane;
        if ((Ca & Ee) === Ee) te !== null && (te = te.next = { lane: 0, action: he.action, hasEagerState: he.hasEagerState, eagerState: he.eagerState, next: null }), m = he.hasEagerState ? he.eagerState : r(m, he.action);
        else {
          var _e = {
            lane: Ee,
            action: he.action,
            hasEagerState: he.hasEagerState,
            eagerState: he.eagerState,
            next: null
          };
          te === null ? (K = te = _e, M = m) : te = te.next = _e, gn.lanes |= Ee, xa |= Ee;
        }
        he = he.next;
      } while (he !== null && he !== P);
      te === null ? M = m : te.next = K, wo(m, a.memoizedState) || (Pr = !0), a.memoizedState = m, a.baseState = M, a.baseQueue = te, d.lastRenderedState = m;
    }
    if (r = d.interleaved, r !== null) {
      x = r;
      do
        P = x.lane, gn.lanes |= P, xa |= P, x = x.next;
      while (x !== r);
    } else x === null && (d.lanes = 0);
    return [a.memoizedState, d.dispatch];
  }
  function Hh(r) {
    var a = no(), d = a.queue;
    if (d === null) throw Error(n(311));
    d.lastRenderedReducer = r;
    var m = d.dispatch, x = d.pending, P = a.memoizedState;
    if (x !== null) {
      d.pending = null;
      var M = x = x.next;
      do
        P = r(P, M.action), M = M.next;
      while (M !== x);
      wo(P, a.memoizedState) || (Pr = !0), a.memoizedState = P, a.baseQueue === null && (a.baseState = P), d.lastRenderedState = P;
    }
    return [P, m];
  }
  function E1() {
  }
  function _1(r, a) {
    var d = gn, m = no(), x = a(), P = !wo(m.memoizedState, x);
    if (P && (m.memoizedState = x, Pr = !0), m = m.queue, Gh(O1.bind(null, d, m, r), [r]), m.getSnapshot !== a || P || Un !== null && Un.memoizedState.tag & 1) {
      if (d.flags |= 2048, cu(9, P1.bind(null, d, m, x, a), void 0, null), Vn === null) throw Error(n(349));
      Ca & 30 || k1(d, a, x);
    }
    return x;
  }
  function k1(r, a, d) {
    r.flags |= 16384, r = { getSnapshot: a, value: d }, a = gn.updateQueue, a === null ? (a = { lastEffect: null, stores: null }, gn.updateQueue = a, a.stores = [r]) : (d = a.stores, d === null ? a.stores = [r] : d.push(r));
  }
  function P1(r, a, d, m) {
    a.value = d, a.getSnapshot = m, A1(a) && I1(r);
  }
  function O1(r, a, d) {
    return d(function() {
      A1(a) && I1(r);
    });
  }
  function A1(r) {
    var a = r.getSnapshot;
    r = r.value;
    try {
      var d = a();
      return !wo(r, d);
    } catch {
      return !0;
    }
  }
  function I1(r) {
    var a = li(r, 1);
    a !== null && _o(a, r, 1, -1);
  }
  function j1(r) {
    var a = Lo();
    return typeof r == "function" && (r = r()), a.memoizedState = a.baseState = r, r = { pending: null, interleaved: null, lanes: 0, dispatch: null, lastRenderedReducer: uu, lastRenderedState: r }, a.queue = r, r = r.dispatch = Gk.bind(null, gn, r), [a.memoizedState, r];
  }
  function cu(r, a, d, m) {
    return r = { tag: r, create: a, destroy: d, deps: m, next: null }, a = gn.updateQueue, a === null ? (a = { lastEffect: null, stores: null }, gn.updateQueue = a, a.lastEffect = r.next = r) : (d = a.lastEffect, d === null ? a.lastEffect = r.next = r : (m = d.next, d.next = r, r.next = m, a.lastEffect = r)), r;
  }
  function R1() {
    return no().memoizedState;
  }
  function gd(r, a, d, m) {
    var x = Lo();
    gn.flags |= r, x.memoizedState = cu(1 | a, d, void 0, m === void 0 ? null : m);
  }
  function yd(r, a, d, m) {
    var x = no();
    m = m === void 0 ? null : m;
    var P = void 0;
    if (Nn !== null) {
      var M = Nn.memoizedState;
      if (P = M.destroy, m !== null && Vh(m, M.deps)) {
        x.memoizedState = cu(a, d, P, m);
        return;
      }
    }
    gn.flags |= r, x.memoizedState = cu(1 | a, d, P, m);
  }
  function T1(r, a) {
    return gd(8390656, 8, r, a);
  }
  function Gh(r, a) {
    return yd(2048, 8, r, a);
  }
  function M1(r, a) {
    return yd(4, 2, r, a);
  }
  function N1(r, a) {
    return yd(4, 4, r, a);
  }
  function F1(r, a) {
    if (typeof a == "function") return r = r(), a(r), function() {
      a(null);
    };
    if (a != null) return r = r(), a.current = r, function() {
      a.current = null;
    };
  }
  function L1(r, a, d) {
    return d = d != null ? d.concat([r]) : null, yd(4, 4, F1.bind(null, a, r), d);
  }
  function Jh() {
  }
  function D1(r, a) {
    var d = no();
    a = a === void 0 ? null : a;
    var m = d.memoizedState;
    return m !== null && a !== null && Vh(a, m[1]) ? m[0] : (d.memoizedState = [r, a], r);
  }
  function z1(r, a) {
    var d = no();
    a = a === void 0 ? null : a;
    var m = d.memoizedState;
    return m !== null && a !== null && Vh(a, m[1]) ? m[0] : (r = r(), d.memoizedState = [r, a], r);
  }
  function B1(r, a, d) {
    return Ca & 21 ? (wo(d, a) || (d = Uc(), gn.lanes |= d, xa |= d, r.baseState = !0), a) : (r.baseState && (r.baseState = !1, Pr = !0), r.memoizedState = d);
  }
  function Kk(r, a) {
    var d = I;
    I = d !== 0 && 4 > d ? d : 4, r(!0);
    var m = Uh.transition;
    Uh.transition = {};
    try {
      r(!1), a();
    } finally {
      I = d, Uh.transition = m;
    }
  }
  function U1() {
    return no().memoizedState;
  }
  function Hk(r, a, d) {
    var m = zi(r);
    if (d = { lane: m, action: d, hasEagerState: !1, eagerState: null, next: null }, V1(r)) W1(a, d);
    else if (d = S1(r, a, d, m), d !== null) {
      var x = yr();
      _o(d, r, m, x), q1(d, a, m);
    }
  }
  function Gk(r, a, d) {
    var m = zi(r), x = { lane: m, action: d, hasEagerState: !1, eagerState: null, next: null };
    if (V1(r)) W1(a, x);
    else {
      var P = r.alternate;
      if (r.lanes === 0 && (P === null || P.lanes === 0) && (P = a.lastRenderedReducer, P !== null)) try {
        var M = a.lastRenderedState, K = P(M, d);
        if (x.hasEagerState = !0, x.eagerState = K, wo(K, M)) {
          var te = a.interleaved;
          te === null ? (x.next = x, Nh(a)) : (x.next = te.next, te.next = x), a.interleaved = x;
          return;
        }
      } catch {
      } finally {
      }
      d = S1(r, a, x, m), d !== null && (x = yr(), _o(d, r, m, x), q1(d, a, m));
    }
  }
  function V1(r) {
    var a = r.alternate;
    return r === gn || a !== null && a === gn;
  }
  function W1(r, a) {
    su = md = !0;
    var d = r.pending;
    d === null ? a.next = a : (a.next = d.next, d.next = a), r.pending = a;
  }
  function q1(r, a, d) {
    if (d & 4194240) {
      var m = a.lanes;
      m &= r.pendingLanes, d |= m, a.lanes = d, D(r, d);
    }
  }
  var vd = { readContext: to, useCallback: nr, useContext: nr, useEffect: nr, useImperativeHandle: nr, useInsertionEffect: nr, useLayoutEffect: nr, useMemo: nr, useReducer: nr, useRef: nr, useState: nr, useDebugValue: nr, useDeferredValue: nr, useTransition: nr, useMutableSource: nr, useSyncExternalStore: nr, useId: nr, unstable_isNewReconciler: !1 }, Jk = { readContext: to, useCallback: function(r, a) {
    return Lo().memoizedState = [r, a === void 0 ? null : a], r;
  }, useContext: to, useEffect: T1, useImperativeHandle: function(r, a, d) {
    return d = d != null ? d.concat([r]) : null, gd(
      4194308,
      4,
      F1.bind(null, a, r),
      d
    );
  }, useLayoutEffect: function(r, a) {
    return gd(4194308, 4, r, a);
  }, useInsertionEffect: function(r, a) {
    return gd(4, 2, r, a);
  }, useMemo: function(r, a) {
    var d = Lo();
    return a = a === void 0 ? null : a, r = r(), d.memoizedState = [r, a], r;
  }, useReducer: function(r, a, d) {
    var m = Lo();
    return a = d !== void 0 ? d(a) : a, m.memoizedState = m.baseState = a, r = { pending: null, interleaved: null, lanes: 0, dispatch: null, lastRenderedReducer: r, lastRenderedState: a }, m.queue = r, r = r.dispatch = Hk.bind(null, gn, r), [m.memoizedState, r];
  }, useRef: function(r) {
    var a = Lo();
    return r = { current: r }, a.memoizedState = r;
  }, useState: j1, useDebugValue: Jh, useDeferredValue: function(r) {
    return Lo().memoizedState = r;
  }, useTransition: function() {
    var r = j1(!1), a = r[0];
    return r = Kk.bind(null, r[1]), Lo().memoizedState = r, [a, r];
  }, useMutableSource: function() {
  }, useSyncExternalStore: function(r, a, d) {
    var m = gn, x = Lo();
    if (pn) {
      if (d === void 0) throw Error(n(407));
      d = d();
    } else {
      if (d = a(), Vn === null) throw Error(n(349));
      Ca & 30 || k1(m, a, d);
    }
    x.memoizedState = d;
    var P = { value: d, getSnapshot: a };
    return x.queue = P, T1(O1.bind(
      null,
      m,
      P,
      r
    ), [r]), m.flags |= 2048, cu(9, P1.bind(null, m, P, d, a), void 0, null), d;
  }, useId: function() {
    var r = Lo(), a = Vn.identifierPrefix;
    if (pn) {
      var d = si, m = ai;
      d = (m & ~(1 << 32 - xr(m) - 1)).toString(32) + d, a = ":" + a + "R" + d, d = lu++, 0 < d && (a += "H" + d.toString(32)), a += ":";
    } else d = qk++, a = ":" + a + "r" + d.toString(32) + ":";
    return r.memoizedState = a;
  }, unstable_isNewReconciler: !1 }, Yk = {
    readContext: to,
    useCallback: D1,
    useContext: to,
    useEffect: Gh,
    useImperativeHandle: L1,
    useInsertionEffect: M1,
    useLayoutEffect: N1,
    useMemo: z1,
    useReducer: Kh,
    useRef: R1,
    useState: function() {
      return Kh(uu);
    },
    useDebugValue: Jh,
    useDeferredValue: function(r) {
      var a = no();
      return B1(a, Nn.memoizedState, r);
    },
    useTransition: function() {
      var r = Kh(uu)[0], a = no().memoizedState;
      return [r, a];
    },
    useMutableSource: E1,
    useSyncExternalStore: _1,
    useId: U1,
    unstable_isNewReconciler: !1
  }, Qk = { readContext: to, useCallback: D1, useContext: to, useEffect: Gh, useImperativeHandle: L1, useInsertionEffect: M1, useLayoutEffect: N1, useMemo: z1, useReducer: Hh, useRef: R1, useState: function() {
    return Hh(uu);
  }, useDebugValue: Jh, useDeferredValue: function(r) {
    var a = no();
    return Nn === null ? a.memoizedState = r : B1(a, Nn.memoizedState, r);
  }, useTransition: function() {
    var r = Hh(uu)[0], a = no().memoizedState;
    return [r, a];
  }, useMutableSource: E1, useSyncExternalStore: _1, useId: U1, unstable_isNewReconciler: !1 };
  function Co(r, a) {
    if (r && r.defaultProps) {
      a = Y({}, a), r = r.defaultProps;
      for (var d in r) a[d] === void 0 && (a[d] = r[d]);
      return a;
    }
    return a;
  }
  function Yh(r, a, d, m) {
    a = r.memoizedState, d = d(m, a), d = d == null ? a : Y({}, a, d), r.memoizedState = d, r.lanes === 0 && (r.updateQueue.baseState = d);
  }
  var bd = { isMounted: function(r) {
    return (r = r._reactInternals) ? yn(r) === r : !1;
  }, enqueueSetState: function(r, a, d) {
    r = r._reactInternals;
    var m = yr(), x = zi(r), P = ui(m, x);
    P.payload = a, d != null && (P.callback = d), a = Ni(r, P, x), a !== null && (_o(a, r, x, m), dd(a, r, x));
  }, enqueueReplaceState: function(r, a, d) {
    r = r._reactInternals;
    var m = yr(), x = zi(r), P = ui(m, x);
    P.tag = 1, P.payload = a, d != null && (P.callback = d), a = Ni(r, P, x), a !== null && (_o(a, r, x, m), dd(a, r, x));
  }, enqueueForceUpdate: function(r, a) {
    r = r._reactInternals;
    var d = yr(), m = zi(r), x = ui(d, m);
    x.tag = 2, a != null && (x.callback = a), a = Ni(r, x, m), a !== null && (_o(a, r, m, d), dd(a, r, m));
  } };
  function K1(r, a, d, m, x, P, M) {
    return r = r.stateNode, typeof r.shouldComponentUpdate == "function" ? r.shouldComponentUpdate(m, P, M) : a.prototype && a.prototype.isPureReactComponent ? !Yl(d, m) || !Yl(x, P) : !0;
  }
  function H1(r, a, d) {
    var m = !1, x = Ri, P = a.contextType;
    return typeof P == "object" && P !== null ? P = to(P) : (x = kr(a) ? va : tr.current, m = a.contextTypes, P = (m = m != null) ? vs(r, x) : Ri), a = new a(d, P), r.memoizedState = a.state !== null && a.state !== void 0 ? a.state : null, a.updater = bd, r.stateNode = a, a._reactInternals = r, m && (r = r.stateNode, r.__reactInternalMemoizedUnmaskedChildContext = x, r.__reactInternalMemoizedMaskedChildContext = P), a;
  }
  function G1(r, a, d, m) {
    r = a.state, typeof a.componentWillReceiveProps == "function" && a.componentWillReceiveProps(d, m), typeof a.UNSAFE_componentWillReceiveProps == "function" && a.UNSAFE_componentWillReceiveProps(d, m), a.state !== r && bd.enqueueReplaceState(a, a.state, null);
  }
  function Qh(r, a, d, m) {
    var x = r.stateNode;
    x.props = d, x.state = r.memoizedState, x.refs = {}, Fh(r);
    var P = a.contextType;
    typeof P == "object" && P !== null ? x.context = to(P) : (P = kr(a) ? va : tr.current, x.context = vs(r, P)), x.state = r.memoizedState, P = a.getDerivedStateFromProps, typeof P == "function" && (Yh(r, a, P, d), x.state = r.memoizedState), typeof a.getDerivedStateFromProps == "function" || typeof x.getSnapshotBeforeUpdate == "function" || typeof x.UNSAFE_componentWillMount != "function" && typeof x.componentWillMount != "function" || (a = x.state, typeof x.componentWillMount == "function" && x.componentWillMount(), typeof x.UNSAFE_componentWillMount == "function" && x.UNSAFE_componentWillMount(), a !== x.state && bd.enqueueReplaceState(x, x.state, null), fd(r, d, x, m), x.state = r.memoizedState), typeof x.componentDidMount == "function" && (r.flags |= 4194308);
  }
  function _s(r, a) {
    try {
      var d = "", m = a;
      do
        d += U(m), m = m.return;
      while (m);
      var x = d;
    } catch (P) {
      x = `
Error generating stack: ` + P.message + `
` + P.stack;
    }
    return { value: r, source: a, stack: x, digest: null };
  }
  function Xh(r, a, d) {
    return { value: r, source: null, stack: d ?? null, digest: a ?? null };
  }
  function Zh(r, a) {
    try {
      console.error(a.value);
    } catch (d) {
      setTimeout(function() {
        throw d;
      });
    }
  }
  var Xk = typeof WeakMap == "function" ? WeakMap : Map;
  function J1(r, a, d) {
    d = ui(-1, d), d.tag = 3, d.payload = { element: null };
    var m = a.value;
    return d.callback = function() {
      _d || (_d = !0, hm = m), Zh(r, a);
    }, d;
  }
  function Y1(r, a, d) {
    d = ui(-1, d), d.tag = 3;
    var m = r.type.getDerivedStateFromError;
    if (typeof m == "function") {
      var x = a.value;
      d.payload = function() {
        return m(x);
      }, d.callback = function() {
        Zh(r, a);
      };
    }
    var P = r.stateNode;
    return P !== null && typeof P.componentDidCatch == "function" && (d.callback = function() {
      Zh(r, a), typeof m != "function" && (Li === null ? Li = /* @__PURE__ */ new Set([this]) : Li.add(this));
      var M = a.stack;
      this.componentDidCatch(a.value, { componentStack: M !== null ? M : "" });
    }), d;
  }
  function Q1(r, a, d) {
    var m = r.pingCache;
    if (m === null) {
      m = r.pingCache = new Xk();
      var x = /* @__PURE__ */ new Set();
      m.set(a, x);
    } else x = m.get(a), x === void 0 && (x = /* @__PURE__ */ new Set(), m.set(a, x));
    x.has(d) || (x.add(d), r = f5.bind(null, r, a, d), a.then(r, r));
  }
  function X1(r) {
    do {
      var a;
      if ((a = r.tag === 13) && (a = r.memoizedState, a = a !== null ? a.dehydrated !== null : !0), a) return r;
      r = r.return;
    } while (r !== null);
    return null;
  }
  function Z1(r, a, d, m, x) {
    return r.mode & 1 ? (r.flags |= 65536, r.lanes = x, r) : (r === a ? r.flags |= 65536 : (r.flags |= 128, d.flags |= 131072, d.flags &= -52805, d.tag === 1 && (d.alternate === null ? d.tag = 17 : (a = ui(-1, 1), a.tag = 2, Ni(d, a, 1))), d.lanes |= 1), r);
  }
  var Zk = b.ReactCurrentOwner, Pr = !1;
  function gr(r, a, d, m) {
    a.child = r === null ? b1(a, null, d, m) : $s(a, r.child, d, m);
  }
  function eb(r, a, d, m, x) {
    d = d.render;
    var P = a.ref;
    return xs(a, x), m = Wh(r, a, d, m, P, x), d = qh(), r !== null && !Pr ? (a.updateQueue = r.updateQueue, a.flags &= -2053, r.lanes &= ~x, ci(r, a, x)) : (pn && d && kh(a), a.flags |= 1, gr(r, a, m, x), a.child);
  }
  function tb(r, a, d, m, x) {
    if (r === null) {
      var P = d.type;
      return typeof P == "function" && !wm(P) && P.defaultProps === void 0 && d.compare === null && d.defaultProps === void 0 ? (a.tag = 15, a.type = P, nb(r, a, P, m, x)) : (r = jd(d.type, null, m, a, a.mode, x), r.ref = a.ref, r.return = a, a.child = r);
    }
    if (P = r.child, !(r.lanes & x)) {
      var M = P.memoizedProps;
      if (d = d.compare, d = d !== null ? d : Yl, d(M, m) && r.ref === a.ref) return ci(r, a, x);
    }
    return a.flags |= 1, r = Ui(P, m), r.ref = a.ref, r.return = a, a.child = r;
  }
  function nb(r, a, d, m, x) {
    if (r !== null) {
      var P = r.memoizedProps;
      if (Yl(P, m) && r.ref === a.ref) if (Pr = !1, a.pendingProps = m = P, (r.lanes & x) !== 0) r.flags & 131072 && (Pr = !0);
      else return a.lanes = r.lanes, ci(r, a, x);
    }
    return em(r, a, d, m, x);
  }
  function rb(r, a, d) {
    var m = a.pendingProps, x = m.children, P = r !== null ? r.memoizedState : null;
    if (m.mode === "hidden") if (!(a.mode & 1)) a.memoizedState = { baseLanes: 0, cachePool: null, transitions: null }, nn(Ps, Ur), Ur |= d;
    else {
      if (!(d & 1073741824)) return r = P !== null ? P.baseLanes | d : d, a.lanes = a.childLanes = 1073741824, a.memoizedState = { baseLanes: r, cachePool: null, transitions: null }, a.updateQueue = null, nn(Ps, Ur), Ur |= r, null;
      a.memoizedState = { baseLanes: 0, cachePool: null, transitions: null }, m = P !== null ? P.baseLanes : d, nn(Ps, Ur), Ur |= m;
    }
    else P !== null ? (m = P.baseLanes | d, a.memoizedState = null) : m = d, nn(Ps, Ur), Ur |= m;
    return gr(r, a, x, d), a.child;
  }
  function ob(r, a) {
    var d = a.ref;
    (r === null && d !== null || r !== null && r.ref !== d) && (a.flags |= 512, a.flags |= 2097152);
  }
  function em(r, a, d, m, x) {
    var P = kr(d) ? va : tr.current;
    return P = vs(a, P), xs(a, x), d = Wh(r, a, d, m, P, x), m = qh(), r !== null && !Pr ? (a.updateQueue = r.updateQueue, a.flags &= -2053, r.lanes &= ~x, ci(r, a, x)) : (pn && m && kh(a), a.flags |= 1, gr(r, a, d, x), a.child);
  }
  function ib(r, a, d, m, x) {
    if (kr(d)) {
      var P = !0;
      rd(a);
    } else P = !1;
    if (xs(a, x), a.stateNode === null) wd(r, a), H1(a, d, m), Qh(a, d, m, x), m = !0;
    else if (r === null) {
      var M = a.stateNode, K = a.memoizedProps;
      M.props = K;
      var te = M.context, he = d.contextType;
      typeof he == "object" && he !== null ? he = to(he) : (he = kr(d) ? va : tr.current, he = vs(a, he));
      var Ee = d.getDerivedStateFromProps, _e = typeof Ee == "function" || typeof M.getSnapshotBeforeUpdate == "function";
      _e || typeof M.UNSAFE_componentWillReceiveProps != "function" && typeof M.componentWillReceiveProps != "function" || (K !== m || te !== he) && G1(a, M, m, he), Mi = !1;
      var xe = a.memoizedState;
      M.state = xe, fd(a, m, M, x), te = a.memoizedState, K !== m || xe !== te || _r.current || Mi ? (typeof Ee == "function" && (Yh(a, d, Ee, m), te = a.memoizedState), (K = Mi || K1(a, d, K, m, xe, te, he)) ? (_e || typeof M.UNSAFE_componentWillMount != "function" && typeof M.componentWillMount != "function" || (typeof M.componentWillMount == "function" && M.componentWillMount(), typeof M.UNSAFE_componentWillMount == "function" && M.UNSAFE_componentWillMount()), typeof M.componentDidMount == "function" && (a.flags |= 4194308)) : (typeof M.componentDidMount == "function" && (a.flags |= 4194308), a.memoizedProps = m, a.memoizedState = te), M.props = m, M.state = te, M.context = he, m = K) : (typeof M.componentDidMount == "function" && (a.flags |= 4194308), m = !1);
    } else {
      M = a.stateNode, w1(r, a), K = a.memoizedProps, he = a.type === a.elementType ? K : Co(a.type, K), M.props = he, _e = a.pendingProps, xe = M.context, te = d.contextType, typeof te == "object" && te !== null ? te = to(te) : (te = kr(d) ? va : tr.current, te = vs(a, te));
      var De = d.getDerivedStateFromProps;
      (Ee = typeof De == "function" || typeof M.getSnapshotBeforeUpdate == "function") || typeof M.UNSAFE_componentWillReceiveProps != "function" && typeof M.componentWillReceiveProps != "function" || (K !== _e || xe !== te) && G1(a, M, m, te), Mi = !1, xe = a.memoizedState, M.state = xe, fd(a, m, M, x);
      var qe = a.memoizedState;
      K !== _e || xe !== qe || _r.current || Mi ? (typeof De == "function" && (Yh(a, d, De, m), qe = a.memoizedState), (he = Mi || K1(a, d, he, m, xe, qe, te) || !1) ? (Ee || typeof M.UNSAFE_componentWillUpdate != "function" && typeof M.componentWillUpdate != "function" || (typeof M.componentWillUpdate == "function" && M.componentWillUpdate(m, qe, te), typeof M.UNSAFE_componentWillUpdate == "function" && M.UNSAFE_componentWillUpdate(m, qe, te)), typeof M.componentDidUpdate == "function" && (a.flags |= 4), typeof M.getSnapshotBeforeUpdate == "function" && (a.flags |= 1024)) : (typeof M.componentDidUpdate != "function" || K === r.memoizedProps && xe === r.memoizedState || (a.flags |= 4), typeof M.getSnapshotBeforeUpdate != "function" || K === r.memoizedProps && xe === r.memoizedState || (a.flags |= 1024), a.memoizedProps = m, a.memoizedState = qe), M.props = m, M.state = qe, M.context = te, m = he) : (typeof M.componentDidUpdate != "function" || K === r.memoizedProps && xe === r.memoizedState || (a.flags |= 4), typeof M.getSnapshotBeforeUpdate != "function" || K === r.memoizedProps && xe === r.memoizedState || (a.flags |= 1024), m = !1);
    }
    return tm(r, a, d, m, P, x);
  }
  function tm(r, a, d, m, x, P) {
    ob(r, a);
    var M = (a.flags & 128) !== 0;
    if (!m && !M) return x && c1(a, d, !1), ci(r, a, P);
    m = a.stateNode, Zk.current = a;
    var K = M && typeof d.getDerivedStateFromError != "function" ? null : m.render();
    return a.flags |= 1, r !== null && M ? (a.child = $s(a, r.child, null, P), a.child = $s(a, null, K, P)) : gr(r, a, K, P), a.memoizedState = m.state, x && c1(a, d, !0), a.child;
  }
  function ab(r) {
    var a = r.stateNode;
    a.pendingContext ? l1(r, a.pendingContext, a.pendingContext !== a.context) : a.context && l1(r, a.context, !1), Lh(r, a.containerInfo);
  }
  function sb(r, a, d, m, x) {
    return ws(), Ih(x), a.flags |= 256, gr(r, a, d, m), a.child;
  }
  var nm = { dehydrated: null, treeContext: null, retryLane: 0 };
  function rm(r) {
    return { baseLanes: r, cachePool: null, transitions: null };
  }
  function lb(r, a, d) {
    var m = a.pendingProps, x = mn.current, P = !1, M = (a.flags & 128) !== 0, K;
    if ((K = M) || (K = r !== null && r.memoizedState === null ? !1 : (x & 2) !== 0), K ? (P = !0, a.flags &= -129) : (r === null || r.memoizedState !== null) && (x |= 1), nn(mn, x & 1), r === null)
      return Ah(a), r = a.memoizedState, r !== null && (r = r.dehydrated, r !== null) ? (a.mode & 1 ? r.data === "$!" ? a.lanes = 8 : a.lanes = 1073741824 : a.lanes = 1, null) : (M = m.children, r = m.fallback, P ? (m = a.mode, P = a.child, M = { mode: "hidden", children: M }, !(m & 1) && P !== null ? (P.childLanes = 0, P.pendingProps = M) : P = Rd(M, m, 0, null), r = Pa(r, m, d, null), P.return = a, r.return = a, P.sibling = r, a.child = P, a.child.memoizedState = rm(d), a.memoizedState = nm, r) : om(a, M));
    if (x = r.memoizedState, x !== null && (K = x.dehydrated, K !== null)) return e5(r, a, M, m, K, x, d);
    if (P) {
      P = m.fallback, M = a.mode, x = r.child, K = x.sibling;
      var te = { mode: "hidden", children: m.children };
      return !(M & 1) && a.child !== x ? (m = a.child, m.childLanes = 0, m.pendingProps = te, a.deletions = null) : (m = Ui(x, te), m.subtreeFlags = x.subtreeFlags & 14680064), K !== null ? P = Ui(K, P) : (P = Pa(P, M, d, null), P.flags |= 2), P.return = a, m.return = a, m.sibling = P, a.child = m, m = P, P = a.child, M = r.child.memoizedState, M = M === null ? rm(d) : { baseLanes: M.baseLanes | d, cachePool: null, transitions: M.transitions }, P.memoizedState = M, P.childLanes = r.childLanes & ~d, a.memoizedState = nm, m;
    }
    return P = r.child, r = P.sibling, m = Ui(P, { mode: "visible", children: m.children }), !(a.mode & 1) && (m.lanes = d), m.return = a, m.sibling = null, r !== null && (d = a.deletions, d === null ? (a.deletions = [r], a.flags |= 16) : d.push(r)), a.child = m, a.memoizedState = null, m;
  }
  function om(r, a) {
    return a = Rd({ mode: "visible", children: a }, r.mode, 0, null), a.return = r, r.child = a;
  }
  function Sd(r, a, d, m) {
    return m !== null && Ih(m), $s(a, r.child, null, d), r = om(a, a.pendingProps.children), r.flags |= 2, a.memoizedState = null, r;
  }
  function e5(r, a, d, m, x, P, M) {
    if (d)
      return a.flags & 256 ? (a.flags &= -257, m = Xh(Error(n(422))), Sd(r, a, M, m)) : a.memoizedState !== null ? (a.child = r.child, a.flags |= 128, null) : (P = m.fallback, x = a.mode, m = Rd({ mode: "visible", children: m.children }, x, 0, null), P = Pa(P, x, M, null), P.flags |= 2, m.return = a, P.return = a, m.sibling = P, a.child = m, a.mode & 1 && $s(a, r.child, null, M), a.child.memoizedState = rm(M), a.memoizedState = nm, P);
    if (!(a.mode & 1)) return Sd(r, a, M, null);
    if (x.data === "$!") {
      if (m = x.nextSibling && x.nextSibling.dataset, m) var K = m.dgst;
      return m = K, P = Error(n(419)), m = Xh(P, m, void 0), Sd(r, a, M, m);
    }
    if (K = (M & r.childLanes) !== 0, Pr || K) {
      if (m = Vn, m !== null) {
        switch (M & -M) {
          case 4:
            x = 2;
            break;
          case 16:
            x = 8;
            break;
          case 64:
          case 128:
          case 256:
          case 512:
          case 1024:
          case 2048:
          case 4096:
          case 8192:
          case 16384:
          case 32768:
          case 65536:
          case 131072:
          case 262144:
          case 524288:
          case 1048576:
          case 2097152:
          case 4194304:
          case 8388608:
          case 16777216:
          case 33554432:
          case 67108864:
            x = 32;
            break;
          case 536870912:
            x = 268435456;
            break;
          default:
            x = 0;
        }
        x = x & (m.suspendedLanes | M) ? 0 : x, x !== 0 && x !== P.retryLane && (P.retryLane = x, li(r, x), _o(m, r, x, -1));
      }
      return Sm(), m = Xh(Error(n(421))), Sd(r, a, M, m);
    }
    return x.data === "$?" ? (a.flags |= 128, a.child = r.child, a = p5.bind(null, r), x._reactRetry = a, null) : (r = P.treeContext, Br = Ii(x.nextSibling), zr = a, pn = !0, $o = null, r !== null && (Zr[eo++] = ai, Zr[eo++] = si, Zr[eo++] = ba, ai = r.id, si = r.overflow, ba = a), a = om(a, m.children), a.flags |= 4096, a);
  }
  function ub(r, a, d) {
    r.lanes |= a;
    var m = r.alternate;
    m !== null && (m.lanes |= a), Mh(r.return, a, d);
  }
  function im(r, a, d, m, x) {
    var P = r.memoizedState;
    P === null ? r.memoizedState = { isBackwards: a, rendering: null, renderingStartTime: 0, last: m, tail: d, tailMode: x } : (P.isBackwards = a, P.rendering = null, P.renderingStartTime = 0, P.last = m, P.tail = d, P.tailMode = x);
  }
  function cb(r, a, d) {
    var m = a.pendingProps, x = m.revealOrder, P = m.tail;
    if (gr(r, a, m.children, d), m = mn.current, m & 2) m = m & 1 | 2, a.flags |= 128;
    else {
      if (r !== null && r.flags & 128) e: for (r = a.child; r !== null; ) {
        if (r.tag === 13) r.memoizedState !== null && ub(r, d, a);
        else if (r.tag === 19) ub(r, d, a);
        else if (r.child !== null) {
          r.child.return = r, r = r.child;
          continue;
        }
        if (r === a) break e;
        for (; r.sibling === null; ) {
          if (r.return === null || r.return === a) break e;
          r = r.return;
        }
        r.sibling.return = r.return, r = r.sibling;
      }
      m &= 1;
    }
    if (nn(mn, m), !(a.mode & 1)) a.memoizedState = null;
    else switch (x) {
      case "forwards":
        for (d = a.child, x = null; d !== null; ) r = d.alternate, r !== null && pd(r) === null && (x = d), d = d.sibling;
        d = x, d === null ? (x = a.child, a.child = null) : (x = d.sibling, d.sibling = null), im(a, !1, x, d, P);
        break;
      case "backwards":
        for (d = null, x = a.child, a.child = null; x !== null; ) {
          if (r = x.alternate, r !== null && pd(r) === null) {
            a.child = x;
            break;
          }
          r = x.sibling, x.sibling = d, d = x, x = r;
        }
        im(a, !0, d, null, P);
        break;
      case "together":
        im(a, !1, null, null, void 0);
        break;
      default:
        a.memoizedState = null;
    }
    return a.child;
  }
  function wd(r, a) {
    !(a.mode & 1) && r !== null && (r.alternate = null, a.alternate = null, a.flags |= 2);
  }
  function ci(r, a, d) {
    if (r !== null && (a.dependencies = r.dependencies), xa |= a.lanes, !(d & a.childLanes)) return null;
    if (r !== null && a.child !== r.child) throw Error(n(153));
    if (a.child !== null) {
      for (r = a.child, d = Ui(r, r.pendingProps), a.child = d, d.return = a; r.sibling !== null; ) r = r.sibling, d = d.sibling = Ui(r, r.pendingProps), d.return = a;
      d.sibling = null;
    }
    return a.child;
  }
  function t5(r, a, d) {
    switch (a.tag) {
      case 3:
        ab(a), ws();
        break;
      case 5:
        x1(a);
        break;
      case 1:
        kr(a.type) && rd(a);
        break;
      case 4:
        Lh(a, a.stateNode.containerInfo);
        break;
      case 10:
        var m = a.type._context, x = a.memoizedProps.value;
        nn(ud, m._currentValue), m._currentValue = x;
        break;
      case 13:
        if (m = a.memoizedState, m !== null)
          return m.dehydrated !== null ? (nn(mn, mn.current & 1), a.flags |= 128, null) : d & a.child.childLanes ? lb(r, a, d) : (nn(mn, mn.current & 1), r = ci(r, a, d), r !== null ? r.sibling : null);
        nn(mn, mn.current & 1);
        break;
      case 19:
        if (m = (d & a.childLanes) !== 0, r.flags & 128) {
          if (m) return cb(r, a, d);
          a.flags |= 128;
        }
        if (x = a.memoizedState, x !== null && (x.rendering = null, x.tail = null, x.lastEffect = null), nn(mn, mn.current), m) break;
        return null;
      case 22:
      case 23:
        return a.lanes = 0, rb(r, a, d);
    }
    return ci(r, a, d);
  }
  var db, am, fb, pb;
  db = function(r, a) {
    for (var d = a.child; d !== null; ) {
      if (d.tag === 5 || d.tag === 6) r.appendChild(d.stateNode);
      else if (d.tag !== 4 && d.child !== null) {
        d.child.return = d, d = d.child;
        continue;
      }
      if (d === a) break;
      for (; d.sibling === null; ) {
        if (d.return === null || d.return === a) return;
        d = d.return;
      }
      d.sibling.return = d.return, d = d.sibling;
    }
  }, am = function() {
  }, fb = function(r, a, d, m) {
    var x = r.memoizedProps;
    if (x !== m) {
      r = a.stateNode, $a(Fo.current);
      var P = null;
      switch (d) {
        case "input":
          x = Xe(r, x), m = Xe(r, m), P = [];
          break;
        case "select":
          x = Y({}, x, { value: void 0 }), m = Y({}, m, { value: void 0 }), P = [];
          break;
        case "textarea":
          x = Le(r, x), m = Le(r, m), P = [];
          break;
        default:
          typeof x.onClick != "function" && typeof m.onClick == "function" && (r.onclick = ed);
      }
      tt(d, m);
      var M;
      d = null;
      for (he in x) if (!m.hasOwnProperty(he) && x.hasOwnProperty(he) && x[he] != null) if (he === "style") {
        var K = x[he];
        for (M in K) K.hasOwnProperty(M) && (d || (d = {}), d[M] = "");
      } else he !== "dangerouslySetInnerHTML" && he !== "children" && he !== "suppressContentEditableWarning" && he !== "suppressHydrationWarning" && he !== "autoFocus" && (i.hasOwnProperty(he) ? P || (P = []) : (P = P || []).push(he, null));
      for (he in m) {
        var te = m[he];
        if (K = x != null ? x[he] : void 0, m.hasOwnProperty(he) && te !== K && (te != null || K != null)) if (he === "style") if (K) {
          for (M in K) !K.hasOwnProperty(M) || te && te.hasOwnProperty(M) || (d || (d = {}), d[M] = "");
          for (M in te) te.hasOwnProperty(M) && K[M] !== te[M] && (d || (d = {}), d[M] = te[M]);
        } else d || (P || (P = []), P.push(
          he,
          d
        )), d = te;
        else he === "dangerouslySetInnerHTML" ? (te = te ? te.__html : void 0, K = K ? K.__html : void 0, te != null && K !== te && (P = P || []).push(he, te)) : he === "children" ? typeof te != "string" && typeof te != "number" || (P = P || []).push(he, "" + te) : he !== "suppressContentEditableWarning" && he !== "suppressHydrationWarning" && (i.hasOwnProperty(he) ? (te != null && he === "onScroll" && an("scroll", r), P || K === te || (P = [])) : (P = P || []).push(he, te));
      }
      d && (P = P || []).push("style", d);
      var he = P;
      (a.updateQueue = he) && (a.flags |= 4);
    }
  }, pb = function(r, a, d, m) {
    d !== m && (a.flags |= 4);
  };
  function du(r, a) {
    if (!pn) switch (r.tailMode) {
      case "hidden":
        a = r.tail;
        for (var d = null; a !== null; ) a.alternate !== null && (d = a), a = a.sibling;
        d === null ? r.tail = null : d.sibling = null;
        break;
      case "collapsed":
        d = r.tail;
        for (var m = null; d !== null; ) d.alternate !== null && (m = d), d = d.sibling;
        m === null ? a || r.tail === null ? r.tail = null : r.tail.sibling = null : m.sibling = null;
    }
  }
  function rr(r) {
    var a = r.alternate !== null && r.alternate.child === r.child, d = 0, m = 0;
    if (a) for (var x = r.child; x !== null; ) d |= x.lanes | x.childLanes, m |= x.subtreeFlags & 14680064, m |= x.flags & 14680064, x.return = r, x = x.sibling;
    else for (x = r.child; x !== null; ) d |= x.lanes | x.childLanes, m |= x.subtreeFlags, m |= x.flags, x.return = r, x = x.sibling;
    return r.subtreeFlags |= m, r.childLanes = d, a;
  }
  function n5(r, a, d) {
    var m = a.pendingProps;
    switch (Ph(a), a.tag) {
      case 2:
      case 16:
      case 15:
      case 0:
      case 11:
      case 7:
      case 8:
      case 12:
      case 9:
      case 14:
        return rr(a), null;
      case 1:
        return kr(a.type) && nd(), rr(a), null;
      case 3:
        return m = a.stateNode, Es(), sn(_r), sn(tr), Bh(), m.pendingContext && (m.context = m.pendingContext, m.pendingContext = null), (r === null || r.child === null) && (sd(a) ? a.flags |= 4 : r === null || r.memoizedState.isDehydrated && !(a.flags & 256) || (a.flags |= 1024, $o !== null && (ym($o), $o = null))), am(r, a), rr(a), null;
      case 5:
        Dh(a);
        var x = $a(au.current);
        if (d = a.type, r !== null && a.stateNode != null) fb(r, a, d, m, x), r.ref !== a.ref && (a.flags |= 512, a.flags |= 2097152);
        else {
          if (!m) {
            if (a.stateNode === null) throw Error(n(166));
            return rr(a), null;
          }
          if (r = $a(Fo.current), sd(a)) {
            m = a.stateNode, d = a.type;
            var P = a.memoizedProps;
            switch (m[No] = a, m[tu] = P, r = (a.mode & 1) !== 0, d) {
              case "dialog":
                an("cancel", m), an("close", m);
                break;
              case "iframe":
              case "object":
              case "embed":
                an("load", m);
                break;
              case "video":
              case "audio":
                for (x = 0; x < Xl.length; x++) an(Xl[x], m);
                break;
              case "source":
                an("error", m);
                break;
              case "img":
              case "image":
              case "link":
                an(
                  "error",
                  m
                ), an("load", m);
                break;
              case "details":
                an("toggle", m);
                break;
              case "input":
                ft(m, P), an("invalid", m);
                break;
              case "select":
                m._wrapperState = { wasMultiple: !!P.multiple }, an("invalid", m);
                break;
              case "textarea":
                cn(m, P), an("invalid", m);
            }
            tt(d, P), x = null;
            for (var M in P) if (P.hasOwnProperty(M)) {
              var K = P[M];
              M === "children" ? typeof K == "string" ? m.textContent !== K && (P.suppressHydrationWarning !== !0 && Zc(m.textContent, K, r), x = ["children", K]) : typeof K == "number" && m.textContent !== "" + K && (P.suppressHydrationWarning !== !0 && Zc(
                m.textContent,
                K,
                r
              ), x = ["children", "" + K]) : i.hasOwnProperty(M) && K != null && M === "onScroll" && an("scroll", m);
            }
            switch (d) {
              case "input":
                Te(m), pt(m, P, !0);
                break;
              case "textarea":
                Te(m), Pe(m);
                break;
              case "select":
              case "option":
                break;
              default:
                typeof P.onClick == "function" && (m.onclick = ed);
            }
            m = x, a.updateQueue = m, m !== null && (a.flags |= 4);
          } else {
            M = x.nodeType === 9 ? x : x.ownerDocument, r === "http://www.w3.org/1999/xhtml" && (r = Me(d)), r === "http://www.w3.org/1999/xhtml" ? d === "script" ? (r = M.createElement("div"), r.innerHTML = "<script><\/script>", r = r.removeChild(r.firstChild)) : typeof m.is == "string" ? r = M.createElement(d, { is: m.is }) : (r = M.createElement(d), d === "select" && (M = r, m.multiple ? M.multiple = !0 : m.size && (M.size = m.size))) : r = M.createElementNS(r, d), r[No] = a, r[tu] = m, db(r, a, !1, !1), a.stateNode = r;
            e: {
              switch (M = wt(d, m), d) {
                case "dialog":
                  an("cancel", r), an("close", r), x = m;
                  break;
                case "iframe":
                case "object":
                case "embed":
                  an("load", r), x = m;
                  break;
                case "video":
                case "audio":
                  for (x = 0; x < Xl.length; x++) an(Xl[x], r);
                  x = m;
                  break;
                case "source":
                  an("error", r), x = m;
                  break;
                case "img":
                case "image":
                case "link":
                  an(
                    "error",
                    r
                  ), an("load", r), x = m;
                  break;
                case "details":
                  an("toggle", r), x = m;
                  break;
                case "input":
                  ft(r, m), x = Xe(r, m), an("invalid", r);
                  break;
                case "option":
                  x = m;
                  break;
                case "select":
                  r._wrapperState = { wasMultiple: !!m.multiple }, x = Y({}, m, { value: void 0 }), an("invalid", r);
                  break;
                case "textarea":
                  cn(r, m), x = Le(r, m), an("invalid", r);
                  break;
                default:
                  x = m;
              }
              tt(d, x), K = x;
              for (P in K) if (K.hasOwnProperty(P)) {
                var te = K[P];
                P === "style" ? Rt(r, te) : P === "dangerouslySetInnerHTML" ? (te = te ? te.__html : void 0, te != null && se(r, te)) : P === "children" ? typeof te == "string" ? (d !== "textarea" || te !== "") && ae(r, te) : typeof te == "number" && ae(r, "" + te) : P !== "suppressContentEditableWarning" && P !== "suppressHydrationWarning" && P !== "autoFocus" && (i.hasOwnProperty(P) ? te != null && P === "onScroll" && an("scroll", r) : te != null && _(r, P, te, M));
              }
              switch (d) {
                case "input":
                  Te(r), pt(r, m, !1);
                  break;
                case "textarea":
                  Te(r), Pe(r);
                  break;
                case "option":
                  m.value != null && r.setAttribute("value", "" + ue(m.value));
                  break;
                case "select":
                  r.multiple = !!m.multiple, P = m.value, P != null ? yt(r, !!m.multiple, P, !1) : m.defaultValue != null && yt(
                    r,
                    !!m.multiple,
                    m.defaultValue,
                    !0
                  );
                  break;
                default:
                  typeof x.onClick == "function" && (r.onclick = ed);
              }
              switch (d) {
                case "button":
                case "input":
                case "select":
                case "textarea":
                  m = !!m.autoFocus;
                  break e;
                case "img":
                  m = !0;
                  break e;
                default:
                  m = !1;
              }
            }
            m && (a.flags |= 4);
          }
          a.ref !== null && (a.flags |= 512, a.flags |= 2097152);
        }
        return rr(a), null;
      case 6:
        if (r && a.stateNode != null) pb(r, a, r.memoizedProps, m);
        else {
          if (typeof m != "string" && a.stateNode === null) throw Error(n(166));
          if (d = $a(au.current), $a(Fo.current), sd(a)) {
            if (m = a.stateNode, d = a.memoizedProps, m[No] = a, (P = m.nodeValue !== d) && (r = zr, r !== null)) switch (r.tag) {
              case 3:
                Zc(m.nodeValue, d, (r.mode & 1) !== 0);
                break;
              case 5:
                r.memoizedProps.suppressHydrationWarning !== !0 && Zc(m.nodeValue, d, (r.mode & 1) !== 0);
            }
            P && (a.flags |= 4);
          } else m = (d.nodeType === 9 ? d : d.ownerDocument).createTextNode(m), m[No] = a, a.stateNode = m;
        }
        return rr(a), null;
      case 13:
        if (sn(mn), m = a.memoizedState, r === null || r.memoizedState !== null && r.memoizedState.dehydrated !== null) {
          if (pn && Br !== null && a.mode & 1 && !(a.flags & 128)) g1(), ws(), a.flags |= 98560, P = !1;
          else if (P = sd(a), m !== null && m.dehydrated !== null) {
            if (r === null) {
              if (!P) throw Error(n(318));
              if (P = a.memoizedState, P = P !== null ? P.dehydrated : null, !P) throw Error(n(317));
              P[No] = a;
            } else ws(), !(a.flags & 128) && (a.memoizedState = null), a.flags |= 4;
            rr(a), P = !1;
          } else $o !== null && (ym($o), $o = null), P = !0;
          if (!P) return a.flags & 65536 ? a : null;
        }
        return a.flags & 128 ? (a.lanes = d, a) : (m = m !== null, m !== (r !== null && r.memoizedState !== null) && m && (a.child.flags |= 8192, a.mode & 1 && (r === null || mn.current & 1 ? Fn === 0 && (Fn = 3) : Sm())), a.updateQueue !== null && (a.flags |= 4), rr(a), null);
      case 4:
        return Es(), am(r, a), r === null && Zl(a.stateNode.containerInfo), rr(a), null;
      case 10:
        return Th(a.type._context), rr(a), null;
      case 17:
        return kr(a.type) && nd(), rr(a), null;
      case 19:
        if (sn(mn), P = a.memoizedState, P === null) return rr(a), null;
        if (m = (a.flags & 128) !== 0, M = P.rendering, M === null) if (m) du(P, !1);
        else {
          if (Fn !== 0 || r !== null && r.flags & 128) for (r = a.child; r !== null; ) {
            if (M = pd(r), M !== null) {
              for (a.flags |= 128, du(P, !1), m = M.updateQueue, m !== null && (a.updateQueue = m, a.flags |= 4), a.subtreeFlags = 0, m = d, d = a.child; d !== null; ) P = d, r = m, P.flags &= 14680066, M = P.alternate, M === null ? (P.childLanes = 0, P.lanes = r, P.child = null, P.subtreeFlags = 0, P.memoizedProps = null, P.memoizedState = null, P.updateQueue = null, P.dependencies = null, P.stateNode = null) : (P.childLanes = M.childLanes, P.lanes = M.lanes, P.child = M.child, P.subtreeFlags = 0, P.deletions = null, P.memoizedProps = M.memoizedProps, P.memoizedState = M.memoizedState, P.updateQueue = M.updateQueue, P.type = M.type, r = M.dependencies, P.dependencies = r === null ? null : { lanes: r.lanes, firstContext: r.firstContext }), d = d.sibling;
              return nn(mn, mn.current & 1 | 2), a.child;
            }
            r = r.sibling;
          }
          P.tail !== null && fn() > Os && (a.flags |= 128, m = !0, du(P, !1), a.lanes = 4194304);
        }
        else {
          if (!m) if (r = pd(M), r !== null) {
            if (a.flags |= 128, m = !0, d = r.updateQueue, d !== null && (a.updateQueue = d, a.flags |= 4), du(P, !0), P.tail === null && P.tailMode === "hidden" && !M.alternate && !pn) return rr(a), null;
          } else 2 * fn() - P.renderingStartTime > Os && d !== 1073741824 && (a.flags |= 128, m = !0, du(P, !1), a.lanes = 4194304);
          P.isBackwards ? (M.sibling = a.child, a.child = M) : (d = P.last, d !== null ? d.sibling = M : a.child = M, P.last = M);
        }
        return P.tail !== null ? (a = P.tail, P.rendering = a, P.tail = a.sibling, P.renderingStartTime = fn(), a.sibling = null, d = mn.current, nn(mn, m ? d & 1 | 2 : d & 1), a) : (rr(a), null);
      case 22:
      case 23:
        return bm(), m = a.memoizedState !== null, r !== null && r.memoizedState !== null !== m && (a.flags |= 8192), m && a.mode & 1 ? Ur & 1073741824 && (rr(a), a.subtreeFlags & 6 && (a.flags |= 8192)) : rr(a), null;
      case 24:
        return null;
      case 25:
        return null;
    }
    throw Error(n(156, a.tag));
  }
  function r5(r, a) {
    switch (Ph(a), a.tag) {
      case 1:
        return kr(a.type) && nd(), r = a.flags, r & 65536 ? (a.flags = r & -65537 | 128, a) : null;
      case 3:
        return Es(), sn(_r), sn(tr), Bh(), r = a.flags, r & 65536 && !(r & 128) ? (a.flags = r & -65537 | 128, a) : null;
      case 5:
        return Dh(a), null;
      case 13:
        if (sn(mn), r = a.memoizedState, r !== null && r.dehydrated !== null) {
          if (a.alternate === null) throw Error(n(340));
          ws();
        }
        return r = a.flags, r & 65536 ? (a.flags = r & -65537 | 128, a) : null;
      case 19:
        return sn(mn), null;
      case 4:
        return Es(), null;
      case 10:
        return Th(a.type._context), null;
      case 22:
      case 23:
        return bm(), null;
      case 24:
        return null;
      default:
        return null;
    }
  }
  var $d = !1, or = !1, o5 = typeof WeakSet == "function" ? WeakSet : Set, Ue = null;
  function ks(r, a) {
    var d = r.ref;
    if (d !== null) if (typeof d == "function") try {
      d(null);
    } catch (m) {
      vn(r, a, m);
    }
    else d.current = null;
  }
  function sm(r, a, d) {
    try {
      d();
    } catch (m) {
      vn(r, a, m);
    }
  }
  var hb = !1;
  function i5(r, a) {
    if (bh = So, r = Kv(), dh(r)) {
      if ("selectionStart" in r) var d = { start: r.selectionStart, end: r.selectionEnd };
      else e: {
        d = (d = r.ownerDocument) && d.defaultView || window;
        var m = d.getSelection && d.getSelection();
        if (m && m.rangeCount !== 0) {
          d = m.anchorNode;
          var x = m.anchorOffset, P = m.focusNode;
          m = m.focusOffset;
          try {
            d.nodeType, P.nodeType;
          } catch {
            d = null;
            break e;
          }
          var M = 0, K = -1, te = -1, he = 0, Ee = 0, _e = r, xe = null;
          t: for (; ; ) {
            for (var De; _e !== d || x !== 0 && _e.nodeType !== 3 || (K = M + x), _e !== P || m !== 0 && _e.nodeType !== 3 || (te = M + m), _e.nodeType === 3 && (M += _e.nodeValue.length), (De = _e.firstChild) !== null; )
              xe = _e, _e = De;
            for (; ; ) {
              if (_e === r) break t;
              if (xe === d && ++he === x && (K = M), xe === P && ++Ee === m && (te = M), (De = _e.nextSibling) !== null) break;
              _e = xe, xe = _e.parentNode;
            }
            _e = De;
          }
          d = K === -1 || te === -1 ? null : { start: K, end: te };
        } else d = null;
      }
      d = d || { start: 0, end: 0 };
    } else d = null;
    for (Sh = { focusedElem: r, selectionRange: d }, So = !1, Ue = a; Ue !== null; ) if (a = Ue, r = a.child, (a.subtreeFlags & 1028) !== 0 && r !== null) r.return = a, Ue = r;
    else for (; Ue !== null; ) {
      a = Ue;
      try {
        var qe = a.alternate;
        if (a.flags & 1024) switch (a.tag) {
          case 0:
          case 11:
          case 15:
            break;
          case 1:
            if (qe !== null) {
              var Ke = qe.memoizedProps, Cn = qe.memoizedState, le = a.stateNode, oe = le.getSnapshotBeforeUpdate(a.elementType === a.type ? Ke : Co(a.type, Ke), Cn);
              le.__reactInternalSnapshotBeforeUpdate = oe;
            }
            break;
          case 3:
            var fe = a.stateNode.containerInfo;
            fe.nodeType === 1 ? fe.textContent = "" : fe.nodeType === 9 && fe.documentElement && fe.removeChild(fe.documentElement);
            break;
          case 5:
          case 6:
          case 4:
          case 17:
            break;
          default:
            throw Error(n(163));
        }
      } catch (ke) {
        vn(a, a.return, ke);
      }
      if (r = a.sibling, r !== null) {
        r.return = a.return, Ue = r;
        break;
      }
      Ue = a.return;
    }
    return qe = hb, hb = !1, qe;
  }
  function fu(r, a, d) {
    var m = a.updateQueue;
    if (m = m !== null ? m.lastEffect : null, m !== null) {
      var x = m = m.next;
      do {
        if ((x.tag & r) === r) {
          var P = x.destroy;
          x.destroy = void 0, P !== void 0 && sm(a, d, P);
        }
        x = x.next;
      } while (x !== m);
    }
  }
  function Cd(r, a) {
    if (a = a.updateQueue, a = a !== null ? a.lastEffect : null, a !== null) {
      var d = a = a.next;
      do {
        if ((d.tag & r) === r) {
          var m = d.create;
          d.destroy = m();
        }
        d = d.next;
      } while (d !== a);
    }
  }
  function lm(r) {
    var a = r.ref;
    if (a !== null) {
      var d = r.stateNode;
      switch (r.tag) {
        case 5:
          r = d;
          break;
        default:
          r = d;
      }
      typeof a == "function" ? a(r) : a.current = r;
    }
  }
  function mb(r) {
    var a = r.alternate;
    a !== null && (r.alternate = null, mb(a)), r.child = null, r.deletions = null, r.sibling = null, r.tag === 5 && (a = r.stateNode, a !== null && (delete a[No], delete a[tu], delete a[xh], delete a[Bk], delete a[Uk])), r.stateNode = null, r.return = null, r.dependencies = null, r.memoizedProps = null, r.memoizedState = null, r.pendingProps = null, r.stateNode = null, r.updateQueue = null;
  }
  function gb(r) {
    return r.tag === 5 || r.tag === 3 || r.tag === 4;
  }
  function yb(r) {
    e: for (; ; ) {
      for (; r.sibling === null; ) {
        if (r.return === null || gb(r.return)) return null;
        r = r.return;
      }
      for (r.sibling.return = r.return, r = r.sibling; r.tag !== 5 && r.tag !== 6 && r.tag !== 18; ) {
        if (r.flags & 2 || r.child === null || r.tag === 4) continue e;
        r.child.return = r, r = r.child;
      }
      if (!(r.flags & 2)) return r.stateNode;
    }
  }
  function um(r, a, d) {
    var m = r.tag;
    if (m === 5 || m === 6) r = r.stateNode, a ? d.nodeType === 8 ? d.parentNode.insertBefore(r, a) : d.insertBefore(r, a) : (d.nodeType === 8 ? (a = d.parentNode, a.insertBefore(r, d)) : (a = d, a.appendChild(r)), d = d._reactRootContainer, d != null || a.onclick !== null || (a.onclick = ed));
    else if (m !== 4 && (r = r.child, r !== null)) for (um(r, a, d), r = r.sibling; r !== null; ) um(r, a, d), r = r.sibling;
  }
  function cm(r, a, d) {
    var m = r.tag;
    if (m === 5 || m === 6) r = r.stateNode, a ? d.insertBefore(r, a) : d.appendChild(r);
    else if (m !== 4 && (r = r.child, r !== null)) for (cm(r, a, d), r = r.sibling; r !== null; ) cm(r, a, d), r = r.sibling;
  }
  var Hn = null, xo = !1;
  function Fi(r, a, d) {
    for (d = d.child; d !== null; ) vb(r, a, d), d = d.sibling;
  }
  function vb(r, a, d) {
    if (Qr && typeof Qr.onCommitFiberUnmount == "function") try {
      Qr.onCommitFiberUnmount(fa, d);
    } catch {
    }
    switch (d.tag) {
      case 5:
        or || ks(d, a);
      case 6:
        var m = Hn, x = xo;
        Hn = null, Fi(r, a, d), Hn = m, xo = x, Hn !== null && (xo ? (r = Hn, d = d.stateNode, r.nodeType === 8 ? r.parentNode.removeChild(d) : r.removeChild(d)) : Hn.removeChild(d.stateNode));
        break;
      case 18:
        Hn !== null && (xo ? (r = Hn, d = d.stateNode, r.nodeType === 8 ? Ch(r.parentNode, d) : r.nodeType === 1 && Ch(r, d), Mn(r)) : Ch(Hn, d.stateNode));
        break;
      case 4:
        m = Hn, x = xo, Hn = d.stateNode.containerInfo, xo = !0, Fi(r, a, d), Hn = m, xo = x;
        break;
      case 0:
      case 11:
      case 14:
      case 15:
        if (!or && (m = d.updateQueue, m !== null && (m = m.lastEffect, m !== null))) {
          x = m = m.next;
          do {
            var P = x, M = P.destroy;
            P = P.tag, M !== void 0 && (P & 2 || P & 4) && sm(d, a, M), x = x.next;
          } while (x !== m);
        }
        Fi(r, a, d);
        break;
      case 1:
        if (!or && (ks(d, a), m = d.stateNode, typeof m.componentWillUnmount == "function")) try {
          m.props = d.memoizedProps, m.state = d.memoizedState, m.componentWillUnmount();
        } catch (K) {
          vn(d, a, K);
        }
        Fi(r, a, d);
        break;
      case 21:
        Fi(r, a, d);
        break;
      case 22:
        d.mode & 1 ? (or = (m = or) || d.memoizedState !== null, Fi(r, a, d), or = m) : Fi(r, a, d);
        break;
      default:
        Fi(r, a, d);
    }
  }
  function bb(r) {
    var a = r.updateQueue;
    if (a !== null) {
      r.updateQueue = null;
      var d = r.stateNode;
      d === null && (d = r.stateNode = new o5()), a.forEach(function(m) {
        var x = h5.bind(null, r, m);
        d.has(m) || (d.add(m), m.then(x, x));
      });
    }
  }
  function Eo(r, a) {
    var d = a.deletions;
    if (d !== null) for (var m = 0; m < d.length; m++) {
      var x = d[m];
      try {
        var P = r, M = a, K = M;
        e: for (; K !== null; ) {
          switch (K.tag) {
            case 5:
              Hn = K.stateNode, xo = !1;
              break e;
            case 3:
              Hn = K.stateNode.containerInfo, xo = !0;
              break e;
            case 4:
              Hn = K.stateNode.containerInfo, xo = !0;
              break e;
          }
          K = K.return;
        }
        if (Hn === null) throw Error(n(160));
        vb(P, M, x), Hn = null, xo = !1;
        var te = x.alternate;
        te !== null && (te.return = null), x.return = null;
      } catch (he) {
        vn(x, a, he);
      }
    }
    if (a.subtreeFlags & 12854) for (a = a.child; a !== null; ) Sb(a, r), a = a.sibling;
  }
  function Sb(r, a) {
    var d = r.alternate, m = r.flags;
    switch (r.tag) {
      case 0:
      case 11:
      case 14:
      case 15:
        if (Eo(a, r), Do(r), m & 4) {
          try {
            fu(3, r, r.return), Cd(3, r);
          } catch (Ke) {
            vn(r, r.return, Ke);
          }
          try {
            fu(5, r, r.return);
          } catch (Ke) {
            vn(r, r.return, Ke);
          }
        }
        break;
      case 1:
        Eo(a, r), Do(r), m & 512 && d !== null && ks(d, d.return);
        break;
      case 5:
        if (Eo(a, r), Do(r), m & 512 && d !== null && ks(d, d.return), r.flags & 32) {
          var x = r.stateNode;
          try {
            ae(x, "");
          } catch (Ke) {
            vn(r, r.return, Ke);
          }
        }
        if (m & 4 && (x = r.stateNode, x != null)) {
          var P = r.memoizedProps, M = d !== null ? d.memoizedProps : P, K = r.type, te = r.updateQueue;
          if (r.updateQueue = null, te !== null) try {
            K === "input" && P.type === "radio" && P.name != null && Lt(x, P), wt(K, M);
            var he = wt(K, P);
            for (M = 0; M < te.length; M += 2) {
              var Ee = te[M], _e = te[M + 1];
              Ee === "style" ? Rt(x, _e) : Ee === "dangerouslySetInnerHTML" ? se(x, _e) : Ee === "children" ? ae(x, _e) : _(x, Ee, _e, he);
            }
            switch (K) {
              case "input":
                st(x, P);
                break;
              case "textarea":
                ye(x, P);
                break;
              case "select":
                var xe = x._wrapperState.wasMultiple;
                x._wrapperState.wasMultiple = !!P.multiple;
                var De = P.value;
                De != null ? yt(x, !!P.multiple, De, !1) : xe !== !!P.multiple && (P.defaultValue != null ? yt(
                  x,
                  !!P.multiple,
                  P.defaultValue,
                  !0
                ) : yt(x, !!P.multiple, P.multiple ? [] : "", !1));
            }
            x[tu] = P;
          } catch (Ke) {
            vn(r, r.return, Ke);
          }
        }
        break;
      case 6:
        if (Eo(a, r), Do(r), m & 4) {
          if (r.stateNode === null) throw Error(n(162));
          x = r.stateNode, P = r.memoizedProps;
          try {
            x.nodeValue = P;
          } catch (Ke) {
            vn(r, r.return, Ke);
          }
        }
        break;
      case 3:
        if (Eo(a, r), Do(r), m & 4 && d !== null && d.memoizedState.isDehydrated) try {
          Mn(a.containerInfo);
        } catch (Ke) {
          vn(r, r.return, Ke);
        }
        break;
      case 4:
        Eo(a, r), Do(r);
        break;
      case 13:
        Eo(a, r), Do(r), x = r.child, x.flags & 8192 && (P = x.memoizedState !== null, x.stateNode.isHidden = P, !P || x.alternate !== null && x.alternate.memoizedState !== null || (pm = fn())), m & 4 && bb(r);
        break;
      case 22:
        if (Ee = d !== null && d.memoizedState !== null, r.mode & 1 ? (or = (he = or) || Ee, Eo(a, r), or = he) : Eo(a, r), Do(r), m & 8192) {
          if (he = r.memoizedState !== null, (r.stateNode.isHidden = he) && !Ee && r.mode & 1) for (Ue = r, Ee = r.child; Ee !== null; ) {
            for (_e = Ue = Ee; Ue !== null; ) {
              switch (xe = Ue, De = xe.child, xe.tag) {
                case 0:
                case 11:
                case 14:
                case 15:
                  fu(4, xe, xe.return);
                  break;
                case 1:
                  ks(xe, xe.return);
                  var qe = xe.stateNode;
                  if (typeof qe.componentWillUnmount == "function") {
                    m = xe, d = xe.return;
                    try {
                      a = m, qe.props = a.memoizedProps, qe.state = a.memoizedState, qe.componentWillUnmount();
                    } catch (Ke) {
                      vn(m, d, Ke);
                    }
                  }
                  break;
                case 5:
                  ks(xe, xe.return);
                  break;
                case 22:
                  if (xe.memoizedState !== null) {
                    Cb(_e);
                    continue;
                  }
              }
              De !== null ? (De.return = xe, Ue = De) : Cb(_e);
            }
            Ee = Ee.sibling;
          }
          e: for (Ee = null, _e = r; ; ) {
            if (_e.tag === 5) {
              if (Ee === null) {
                Ee = _e;
                try {
                  x = _e.stateNode, he ? (P = x.style, typeof P.setProperty == "function" ? P.setProperty("display", "none", "important") : P.display = "none") : (K = _e.stateNode, te = _e.memoizedProps.style, M = te != null && te.hasOwnProperty("display") ? te.display : null, K.style.display = Ge("display", M));
                } catch (Ke) {
                  vn(r, r.return, Ke);
                }
              }
            } else if (_e.tag === 6) {
              if (Ee === null) try {
                _e.stateNode.nodeValue = he ? "" : _e.memoizedProps;
              } catch (Ke) {
                vn(r, r.return, Ke);
              }
            } else if ((_e.tag !== 22 && _e.tag !== 23 || _e.memoizedState === null || _e === r) && _e.child !== null) {
              _e.child.return = _e, _e = _e.child;
              continue;
            }
            if (_e === r) break e;
            for (; _e.sibling === null; ) {
              if (_e.return === null || _e.return === r) break e;
              Ee === _e && (Ee = null), _e = _e.return;
            }
            Ee === _e && (Ee = null), _e.sibling.return = _e.return, _e = _e.sibling;
          }
        }
        break;
      case 19:
        Eo(a, r), Do(r), m & 4 && bb(r);
        break;
      case 21:
        break;
      default:
        Eo(
          a,
          r
        ), Do(r);
    }
  }
  function Do(r) {
    var a = r.flags;
    if (a & 2) {
      try {
        e: {
          for (var d = r.return; d !== null; ) {
            if (gb(d)) {
              var m = d;
              break e;
            }
            d = d.return;
          }
          throw Error(n(160));
        }
        switch (m.tag) {
          case 5:
            var x = m.stateNode;
            m.flags & 32 && (ae(x, ""), m.flags &= -33);
            var P = yb(r);
            cm(r, P, x);
            break;
          case 3:
          case 4:
            var M = m.stateNode.containerInfo, K = yb(r);
            um(r, K, M);
            break;
          default:
            throw Error(n(161));
        }
      } catch (te) {
        vn(r, r.return, te);
      }
      r.flags &= -3;
    }
    a & 4096 && (r.flags &= -4097);
  }
  function a5(r, a, d) {
    Ue = r, wb(r);
  }
  function wb(r, a, d) {
    for (var m = (r.mode & 1) !== 0; Ue !== null; ) {
      var x = Ue, P = x.child;
      if (x.tag === 22 && m) {
        var M = x.memoizedState !== null || $d;
        if (!M) {
          var K = x.alternate, te = K !== null && K.memoizedState !== null || or;
          K = $d;
          var he = or;
          if ($d = M, (or = te) && !he) for (Ue = x; Ue !== null; ) M = Ue, te = M.child, M.tag === 22 && M.memoizedState !== null ? xb(x) : te !== null ? (te.return = M, Ue = te) : xb(x);
          for (; P !== null; ) Ue = P, wb(P), P = P.sibling;
          Ue = x, $d = K, or = he;
        }
        $b(r);
      } else x.subtreeFlags & 8772 && P !== null ? (P.return = x, Ue = P) : $b(r);
    }
  }
  function $b(r) {
    for (; Ue !== null; ) {
      var a = Ue;
      if (a.flags & 8772) {
        var d = a.alternate;
        try {
          if (a.flags & 8772) switch (a.tag) {
            case 0:
            case 11:
            case 15:
              or || Cd(5, a);
              break;
            case 1:
              var m = a.stateNode;
              if (a.flags & 4 && !or) if (d === null) m.componentDidMount();
              else {
                var x = a.elementType === a.type ? d.memoizedProps : Co(a.type, d.memoizedProps);
                m.componentDidUpdate(x, d.memoizedState, m.__reactInternalSnapshotBeforeUpdate);
              }
              var P = a.updateQueue;
              P !== null && C1(a, P, m);
              break;
            case 3:
              var M = a.updateQueue;
              if (M !== null) {
                if (d = null, a.child !== null) switch (a.child.tag) {
                  case 5:
                    d = a.child.stateNode;
                    break;
                  case 1:
                    d = a.child.stateNode;
                }
                C1(a, M, d);
              }
              break;
            case 5:
              var K = a.stateNode;
              if (d === null && a.flags & 4) {
                d = K;
                var te = a.memoizedProps;
                switch (a.type) {
                  case "button":
                  case "input":
                  case "select":
                  case "textarea":
                    te.autoFocus && d.focus();
                    break;
                  case "img":
                    te.src && (d.src = te.src);
                }
              }
              break;
            case 6:
              break;
            case 4:
              break;
            case 12:
              break;
            case 13:
              if (a.memoizedState === null) {
                var he = a.alternate;
                if (he !== null) {
                  var Ee = he.memoizedState;
                  if (Ee !== null) {
                    var _e = Ee.dehydrated;
                    _e !== null && Mn(_e);
                  }
                }
              }
              break;
            case 19:
            case 17:
            case 21:
            case 22:
            case 23:
            case 25:
              break;
            default:
              throw Error(n(163));
          }
          or || a.flags & 512 && lm(a);
        } catch (xe) {
          vn(a, a.return, xe);
        }
      }
      if (a === r) {
        Ue = null;
        break;
      }
      if (d = a.sibling, d !== null) {
        d.return = a.return, Ue = d;
        break;
      }
      Ue = a.return;
    }
  }
  function Cb(r) {
    for (; Ue !== null; ) {
      var a = Ue;
      if (a === r) {
        Ue = null;
        break;
      }
      var d = a.sibling;
      if (d !== null) {
        d.return = a.return, Ue = d;
        break;
      }
      Ue = a.return;
    }
  }
  function xb(r) {
    for (; Ue !== null; ) {
      var a = Ue;
      try {
        switch (a.tag) {
          case 0:
          case 11:
          case 15:
            var d = a.return;
            try {
              Cd(4, a);
            } catch (te) {
              vn(a, d, te);
            }
            break;
          case 1:
            var m = a.stateNode;
            if (typeof m.componentDidMount == "function") {
              var x = a.return;
              try {
                m.componentDidMount();
              } catch (te) {
                vn(a, x, te);
              }
            }
            var P = a.return;
            try {
              lm(a);
            } catch (te) {
              vn(a, P, te);
            }
            break;
          case 5:
            var M = a.return;
            try {
              lm(a);
            } catch (te) {
              vn(a, M, te);
            }
        }
      } catch (te) {
        vn(a, a.return, te);
      }
      if (a === r) {
        Ue = null;
        break;
      }
      var K = a.sibling;
      if (K !== null) {
        K.return = a.return, Ue = K;
        break;
      }
      Ue = a.return;
    }
  }
  var s5 = Math.ceil, xd = b.ReactCurrentDispatcher, dm = b.ReactCurrentOwner, ro = b.ReactCurrentBatchConfig, Tt = 0, Vn = null, An = null, Gn = 0, Ur = 0, Ps = ji(0), Fn = 0, pu = null, xa = 0, Ed = 0, fm = 0, hu = null, Or = null, pm = 0, Os = 1 / 0, di = null, _d = !1, hm = null, Li = null, kd = !1, Di = null, Pd = 0, mu = 0, mm = null, Od = -1, Ad = 0;
  function yr() {
    return Tt & 6 ? fn() : Od !== -1 ? Od : Od = fn();
  }
  function zi(r) {
    return r.mode & 1 ? Tt & 2 && Gn !== 0 ? Gn & -Gn : Wk.transition !== null ? (Ad === 0 && (Ad = Uc()), Ad) : (r = I, r !== 0 || (r = window.event, r = r === void 0 ? 16 : ls(r.type)), r) : 1;
  }
  function _o(r, a, d, m) {
    if (50 < mu) throw mu = 0, mm = null, Error(n(185));
    To(r, d, m), (!(Tt & 2) || r !== Vn) && (r === Vn && (!(Tt & 2) && (Ed |= d), Fn === 4 && Bi(r, Gn)), Ar(r, m), d === 1 && Tt === 0 && !(a.mode & 1) && (Os = fn() + 500, od && Ti()));
  }
  function Ar(r, a) {
    var d = r.callbackNode;
    th(r, a);
    var m = is(r, r === Vn ? Gn : 0);
    if (m === 0) d !== null && Lc(d), r.callbackNode = null, r.callbackPriority = 0;
    else if (a = m & -m, r.callbackPriority !== a) {
      if (d != null && Lc(d), a === 1) r.tag === 0 ? Vk(_b.bind(null, r)) : d1(_b.bind(null, r)), Dk(function() {
        !(Tt & 6) && Ti();
      }), d = null;
      else {
        switch (R(m)) {
          case 1:
            d = zl;
            break;
          case 4:
            d = Bl;
            break;
          case 16:
            d = rs;
            break;
          case 536870912:
            d = pr;
            break;
          default:
            d = rs;
        }
        d = Tb(d, Eb.bind(null, r));
      }
      r.callbackPriority = a, r.callbackNode = d;
    }
  }
  function Eb(r, a) {
    if (Od = -1, Ad = 0, Tt & 6) throw Error(n(327));
    var d = r.callbackNode;
    if (As() && r.callbackNode !== d) return null;
    var m = is(r, r === Vn ? Gn : 0);
    if (m === 0) return null;
    if (m & 30 || m & r.expiredLanes || a) a = Id(r, m);
    else {
      a = m;
      var x = Tt;
      Tt |= 2;
      var P = Pb();
      (Vn !== r || Gn !== a) && (di = null, Os = fn() + 500, _a(r, a));
      do
        try {
          c5();
          break;
        } catch (K) {
          kb(r, K);
        }
      while (!0);
      Rh(), xd.current = P, Tt = x, An !== null ? a = 0 : (Vn = null, Gn = 0, a = Fn);
    }
    if (a !== 0) {
      if (a === 2 && (x = Vl(r), x !== 0 && (m = x, a = gm(r, x))), a === 1) throw d = pu, _a(r, 0), Bi(r, m), Ar(r, fn()), d;
      if (a === 6) Bi(r, m);
      else {
        if (x = r.current.alternate, !(m & 30) && !l5(x) && (a = Id(r, m), a === 2 && (P = Vl(r), P !== 0 && (m = P, a = gm(r, P))), a === 1)) throw d = pu, _a(r, 0), Bi(r, m), Ar(r, fn()), d;
        switch (r.finishedWork = x, r.finishedLanes = m, a) {
          case 0:
          case 1:
            throw Error(n(345));
          case 2:
            ka(r, Or, di);
            break;
          case 3:
            if (Bi(r, m), (m & 130023424) === m && (a = pm + 500 - fn(), 10 < a)) {
              if (is(r, 0) !== 0) break;
              if (x = r.suspendedLanes, (x & m) !== m) {
                yr(), r.pingedLanes |= r.suspendedLanes & x;
                break;
              }
              r.timeoutHandle = $h(ka.bind(null, r, Or, di), a);
              break;
            }
            ka(r, Or, di);
            break;
          case 4:
            if (Bi(r, m), (m & 4194240) === m) break;
            for (a = r.eventTimes, x = -1; 0 < m; ) {
              var M = 31 - xr(m);
              P = 1 << M, M = a[M], M > x && (x = M), m &= ~P;
            }
            if (m = x, m = fn() - m, m = (120 > m ? 120 : 480 > m ? 480 : 1080 > m ? 1080 : 1920 > m ? 1920 : 3e3 > m ? 3e3 : 4320 > m ? 4320 : 1960 * s5(m / 1960)) - m, 10 < m) {
              r.timeoutHandle = $h(ka.bind(null, r, Or, di), m);
              break;
            }
            ka(r, Or, di);
            break;
          case 5:
            ka(r, Or, di);
            break;
          default:
            throw Error(n(329));
        }
      }
    }
    return Ar(r, fn()), r.callbackNode === d ? Eb.bind(null, r) : null;
  }
  function gm(r, a) {
    var d = hu;
    return r.current.memoizedState.isDehydrated && (_a(r, a).flags |= 256), r = Id(r, a), r !== 2 && (a = Or, Or = d, a !== null && ym(a)), r;
  }
  function ym(r) {
    Or === null ? Or = r : Or.push.apply(Or, r);
  }
  function l5(r) {
    for (var a = r; ; ) {
      if (a.flags & 16384) {
        var d = a.updateQueue;
        if (d !== null && (d = d.stores, d !== null)) for (var m = 0; m < d.length; m++) {
          var x = d[m], P = x.getSnapshot;
          x = x.value;
          try {
            if (!wo(P(), x)) return !1;
          } catch {
            return !1;
          }
        }
      }
      if (d = a.child, a.subtreeFlags & 16384 && d !== null) d.return = a, a = d;
      else {
        if (a === r) break;
        for (; a.sibling === null; ) {
          if (a.return === null || a.return === r) return !0;
          a = a.return;
        }
        a.sibling.return = a.return, a = a.sibling;
      }
    }
    return !0;
  }
  function Bi(r, a) {
    for (a &= ~fm, a &= ~Ed, r.suspendedLanes |= a, r.pingedLanes &= ~a, r = r.expirationTimes; 0 < a; ) {
      var d = 31 - xr(a), m = 1 << d;
      r[d] = -1, a &= ~m;
    }
  }
  function _b(r) {
    if (Tt & 6) throw Error(n(327));
    As();
    var a = is(r, 0);
    if (!(a & 1)) return Ar(r, fn()), null;
    var d = Id(r, a);
    if (r.tag !== 0 && d === 2) {
      var m = Vl(r);
      m !== 0 && (a = m, d = gm(r, m));
    }
    if (d === 1) throw d = pu, _a(r, 0), Bi(r, a), Ar(r, fn()), d;
    if (d === 6) throw Error(n(345));
    return r.finishedWork = r.current.alternate, r.finishedLanes = a, ka(r, Or, di), Ar(r, fn()), null;
  }
  function vm(r, a) {
    var d = Tt;
    Tt |= 1;
    try {
      return r(a);
    } finally {
      Tt = d, Tt === 0 && (Os = fn() + 500, od && Ti());
    }
  }
  function Ea(r) {
    Di !== null && Di.tag === 0 && !(Tt & 6) && As();
    var a = Tt;
    Tt |= 1;
    var d = ro.transition, m = I;
    try {
      if (ro.transition = null, I = 1, r) return r();
    } finally {
      I = m, ro.transition = d, Tt = a, !(Tt & 6) && Ti();
    }
  }
  function bm() {
    Ur = Ps.current, sn(Ps);
  }
  function _a(r, a) {
    r.finishedWork = null, r.finishedLanes = 0;
    var d = r.timeoutHandle;
    if (d !== -1 && (r.timeoutHandle = -1, Lk(d)), An !== null) for (d = An.return; d !== null; ) {
      var m = d;
      switch (Ph(m), m.tag) {
        case 1:
          m = m.type.childContextTypes, m != null && nd();
          break;
        case 3:
          Es(), sn(_r), sn(tr), Bh();
          break;
        case 5:
          Dh(m);
          break;
        case 4:
          Es();
          break;
        case 13:
          sn(mn);
          break;
        case 19:
          sn(mn);
          break;
        case 10:
          Th(m.type._context);
          break;
        case 22:
        case 23:
          bm();
      }
      d = d.return;
    }
    if (Vn = r, An = r = Ui(r.current, null), Gn = Ur = a, Fn = 0, pu = null, fm = Ed = xa = 0, Or = hu = null, wa !== null) {
      for (a = 0; a < wa.length; a++) if (d = wa[a], m = d.interleaved, m !== null) {
        d.interleaved = null;
        var x = m.next, P = d.pending;
        if (P !== null) {
          var M = P.next;
          P.next = x, m.next = M;
        }
        d.pending = m;
      }
      wa = null;
    }
    return r;
  }
  function kb(r, a) {
    do {
      var d = An;
      try {
        if (Rh(), hd.current = vd, md) {
          for (var m = gn.memoizedState; m !== null; ) {
            var x = m.queue;
            x !== null && (x.pending = null), m = m.next;
          }
          md = !1;
        }
        if (Ca = 0, Un = Nn = gn = null, su = !1, lu = 0, dm.current = null, d === null || d.return === null) {
          Fn = 1, pu = a, An = null;
          break;
        }
        e: {
          var P = r, M = d.return, K = d, te = a;
          if (a = Gn, K.flags |= 32768, te !== null && typeof te == "object" && typeof te.then == "function") {
            var he = te, Ee = K, _e = Ee.tag;
            if (!(Ee.mode & 1) && (_e === 0 || _e === 11 || _e === 15)) {
              var xe = Ee.alternate;
              xe ? (Ee.updateQueue = xe.updateQueue, Ee.memoizedState = xe.memoizedState, Ee.lanes = xe.lanes) : (Ee.updateQueue = null, Ee.memoizedState = null);
            }
            var De = X1(M);
            if (De !== null) {
              De.flags &= -257, Z1(De, M, K, P, a), De.mode & 1 && Q1(P, he, a), a = De, te = he;
              var qe = a.updateQueue;
              if (qe === null) {
                var Ke = /* @__PURE__ */ new Set();
                Ke.add(te), a.updateQueue = Ke;
              } else qe.add(te);
              break e;
            } else {
              if (!(a & 1)) {
                Q1(P, he, a), Sm();
                break e;
              }
              te = Error(n(426));
            }
          } else if (pn && K.mode & 1) {
            var Cn = X1(M);
            if (Cn !== null) {
              !(Cn.flags & 65536) && (Cn.flags |= 256), Z1(Cn, M, K, P, a), Ih(_s(te, K));
              break e;
            }
          }
          P = te = _s(te, K), Fn !== 4 && (Fn = 2), hu === null ? hu = [P] : hu.push(P), P = M;
          do {
            switch (P.tag) {
              case 3:
                P.flags |= 65536, a &= -a, P.lanes |= a;
                var le = J1(P, te, a);
                $1(P, le);
                break e;
              case 1:
                K = te;
                var oe = P.type, fe = P.stateNode;
                if (!(P.flags & 128) && (typeof oe.getDerivedStateFromError == "function" || fe !== null && typeof fe.componentDidCatch == "function" && (Li === null || !Li.has(fe)))) {
                  P.flags |= 65536, a &= -a, P.lanes |= a;
                  var ke = Y1(P, K, a);
                  $1(P, ke);
                  break e;
                }
            }
            P = P.return;
          } while (P !== null);
        }
        Ab(d);
      } catch (He) {
        a = He, An === d && d !== null && (An = d = d.return);
        continue;
      }
      break;
    } while (!0);
  }
  function Pb() {
    var r = xd.current;
    return xd.current = vd, r === null ? vd : r;
  }
  function Sm() {
    (Fn === 0 || Fn === 3 || Fn === 2) && (Fn = 4), Vn === null || !(xa & 268435455) && !(Ed & 268435455) || Bi(Vn, Gn);
  }
  function Id(r, a) {
    var d = Tt;
    Tt |= 2;
    var m = Pb();
    (Vn !== r || Gn !== a) && (di = null, _a(r, a));
    do
      try {
        u5();
        break;
      } catch (x) {
        kb(r, x);
      }
    while (!0);
    if (Rh(), Tt = d, xd.current = m, An !== null) throw Error(n(261));
    return Vn = null, Gn = 0, Fn;
  }
  function u5() {
    for (; An !== null; ) Ob(An);
  }
  function c5() {
    for (; An !== null && !Xp(); ) Ob(An);
  }
  function Ob(r) {
    var a = Rb(r.alternate, r, Ur);
    r.memoizedProps = r.pendingProps, a === null ? Ab(r) : An = a, dm.current = null;
  }
  function Ab(r) {
    var a = r;
    do {
      var d = a.alternate;
      if (r = a.return, a.flags & 32768) {
        if (d = r5(d, a), d !== null) {
          d.flags &= 32767, An = d;
          return;
        }
        if (r !== null) r.flags |= 32768, r.subtreeFlags = 0, r.deletions = null;
        else {
          Fn = 6, An = null;
          return;
        }
      } else if (d = n5(d, a, Ur), d !== null) {
        An = d;
        return;
      }
      if (a = a.sibling, a !== null) {
        An = a;
        return;
      }
      An = a = r;
    } while (a !== null);
    Fn === 0 && (Fn = 5);
  }
  function ka(r, a, d) {
    var m = I, x = ro.transition;
    try {
      ro.transition = null, I = 1, d5(r, a, d, m);
    } finally {
      ro.transition = x, I = m;
    }
    return null;
  }
  function d5(r, a, d, m) {
    do
      As();
    while (Di !== null);
    if (Tt & 6) throw Error(n(327));
    d = r.finishedWork;
    var x = r.finishedLanes;
    if (d === null) return null;
    if (r.finishedWork = null, r.finishedLanes = 0, d === r.current) throw Error(n(177));
    r.callbackNode = null, r.callbackPriority = 0;
    var P = d.lanes | d.childLanes;
    if (nh(r, P), r === Vn && (An = Vn = null, Gn = 0), !(d.subtreeFlags & 2064) && !(d.flags & 2064) || kd || (kd = !0, Tb(rs, function() {
      return As(), null;
    })), P = (d.flags & 15990) !== 0, d.subtreeFlags & 15990 || P) {
      P = ro.transition, ro.transition = null;
      var M = I;
      I = 1;
      var K = Tt;
      Tt |= 4, dm.current = null, i5(r, d), Sb(d, r), Ik(Sh), So = !!bh, Sh = bh = null, r.current = d, a5(d), Zp(), Tt = K, I = M, ro.transition = P;
    } else r.current = d;
    if (kd && (kd = !1, Di = r, Pd = x), P = r.pendingLanes, P === 0 && (Li = null), Dc(d.stateNode), Ar(r, fn()), a !== null) for (m = r.onRecoverableError, d = 0; d < a.length; d++) x = a[d], m(x.value, { componentStack: x.stack, digest: x.digest });
    if (_d) throw _d = !1, r = hm, hm = null, r;
    return Pd & 1 && r.tag !== 0 && As(), P = r.pendingLanes, P & 1 ? r === mm ? mu++ : (mu = 0, mm = r) : mu = 0, Ti(), null;
  }
  function As() {
    if (Di !== null) {
      var r = R(Pd), a = ro.transition, d = I;
      try {
        if (ro.transition = null, I = 16 > r ? 16 : r, Di === null) var m = !1;
        else {
          if (r = Di, Di = null, Pd = 0, Tt & 6) throw Error(n(331));
          var x = Tt;
          for (Tt |= 4, Ue = r.current; Ue !== null; ) {
            var P = Ue, M = P.child;
            if (Ue.flags & 16) {
              var K = P.deletions;
              if (K !== null) {
                for (var te = 0; te < K.length; te++) {
                  var he = K[te];
                  for (Ue = he; Ue !== null; ) {
                    var Ee = Ue;
                    switch (Ee.tag) {
                      case 0:
                      case 11:
                      case 15:
                        fu(8, Ee, P);
                    }
                    var _e = Ee.child;
                    if (_e !== null) _e.return = Ee, Ue = _e;
                    else for (; Ue !== null; ) {
                      Ee = Ue;
                      var xe = Ee.sibling, De = Ee.return;
                      if (mb(Ee), Ee === he) {
                        Ue = null;
                        break;
                      }
                      if (xe !== null) {
                        xe.return = De, Ue = xe;
                        break;
                      }
                      Ue = De;
                    }
                  }
                }
                var qe = P.alternate;
                if (qe !== null) {
                  var Ke = qe.child;
                  if (Ke !== null) {
                    qe.child = null;
                    do {
                      var Cn = Ke.sibling;
                      Ke.sibling = null, Ke = Cn;
                    } while (Ke !== null);
                  }
                }
                Ue = P;
              }
            }
            if (P.subtreeFlags & 2064 && M !== null) M.return = P, Ue = M;
            else e: for (; Ue !== null; ) {
              if (P = Ue, P.flags & 2048) switch (P.tag) {
                case 0:
                case 11:
                case 15:
                  fu(9, P, P.return);
              }
              var le = P.sibling;
              if (le !== null) {
                le.return = P.return, Ue = le;
                break e;
              }
              Ue = P.return;
            }
          }
          var oe = r.current;
          for (Ue = oe; Ue !== null; ) {
            M = Ue;
            var fe = M.child;
            if (M.subtreeFlags & 2064 && fe !== null) fe.return = M, Ue = fe;
            else e: for (M = oe; Ue !== null; ) {
              if (K = Ue, K.flags & 2048) try {
                switch (K.tag) {
                  case 0:
                  case 11:
                  case 15:
                    Cd(9, K);
                }
              } catch (He) {
                vn(K, K.return, He);
              }
              if (K === M) {
                Ue = null;
                break e;
              }
              var ke = K.sibling;
              if (ke !== null) {
                ke.return = K.return, Ue = ke;
                break e;
              }
              Ue = K.return;
            }
          }
          if (Tt = x, Ti(), Qr && typeof Qr.onPostCommitFiberRoot == "function") try {
            Qr.onPostCommitFiberRoot(fa, r);
          } catch {
          }
          m = !0;
        }
        return m;
      } finally {
        I = d, ro.transition = a;
      }
    }
    return !1;
  }
  function Ib(r, a, d) {
    a = _s(d, a), a = J1(r, a, 1), r = Ni(r, a, 1), a = yr(), r !== null && (To(r, 1, a), Ar(r, a));
  }
  function vn(r, a, d) {
    if (r.tag === 3) Ib(r, r, d);
    else for (; a !== null; ) {
      if (a.tag === 3) {
        Ib(a, r, d);
        break;
      } else if (a.tag === 1) {
        var m = a.stateNode;
        if (typeof a.type.getDerivedStateFromError == "function" || typeof m.componentDidCatch == "function" && (Li === null || !Li.has(m))) {
          r = _s(d, r), r = Y1(a, r, 1), a = Ni(a, r, 1), r = yr(), a !== null && (To(a, 1, r), Ar(a, r));
          break;
        }
      }
      a = a.return;
    }
  }
  function f5(r, a, d) {
    var m = r.pingCache;
    m !== null && m.delete(a), a = yr(), r.pingedLanes |= r.suspendedLanes & d, Vn === r && (Gn & d) === d && (Fn === 4 || Fn === 3 && (Gn & 130023424) === Gn && 500 > fn() - pm ? _a(r, 0) : fm |= d), Ar(r, a);
  }
  function jb(r, a) {
    a === 0 && (r.mode & 1 ? (a = os, os <<= 1, !(os & 130023424) && (os = 4194304)) : a = 1);
    var d = yr();
    r = li(r, a), r !== null && (To(r, a, d), Ar(r, d));
  }
  function p5(r) {
    var a = r.memoizedState, d = 0;
    a !== null && (d = a.retryLane), jb(r, d);
  }
  function h5(r, a) {
    var d = 0;
    switch (r.tag) {
      case 13:
        var m = r.stateNode, x = r.memoizedState;
        x !== null && (d = x.retryLane);
        break;
      case 19:
        m = r.stateNode;
        break;
      default:
        throw Error(n(314));
    }
    m !== null && m.delete(a), jb(r, d);
  }
  var Rb;
  Rb = function(r, a, d) {
    if (r !== null) if (r.memoizedProps !== a.pendingProps || _r.current) Pr = !0;
    else {
      if (!(r.lanes & d) && !(a.flags & 128)) return Pr = !1, t5(r, a, d);
      Pr = !!(r.flags & 131072);
    }
    else Pr = !1, pn && a.flags & 1048576 && f1(a, ad, a.index);
    switch (a.lanes = 0, a.tag) {
      case 2:
        var m = a.type;
        wd(r, a), r = a.pendingProps;
        var x = vs(a, tr.current);
        xs(a, d), x = Wh(null, a, m, r, x, d);
        var P = qh();
        return a.flags |= 1, typeof x == "object" && x !== null && typeof x.render == "function" && x.$$typeof === void 0 ? (a.tag = 1, a.memoizedState = null, a.updateQueue = null, kr(m) ? (P = !0, rd(a)) : P = !1, a.memoizedState = x.state !== null && x.state !== void 0 ? x.state : null, Fh(a), x.updater = bd, a.stateNode = x, x._reactInternals = a, Qh(a, m, r, d), a = tm(null, a, m, !0, P, d)) : (a.tag = 0, pn && P && kh(a), gr(null, a, x, d), a = a.child), a;
      case 16:
        m = a.elementType;
        e: {
          switch (wd(r, a), r = a.pendingProps, x = m._init, m = x(m._payload), a.type = m, x = a.tag = g5(m), r = Co(m, r), x) {
            case 0:
              a = em(null, a, m, r, d);
              break e;
            case 1:
              a = ib(null, a, m, r, d);
              break e;
            case 11:
              a = eb(null, a, m, r, d);
              break e;
            case 14:
              a = tb(null, a, m, Co(m.type, r), d);
              break e;
          }
          throw Error(n(
            306,
            m,
            ""
          ));
        }
        return a;
      case 0:
        return m = a.type, x = a.pendingProps, x = a.elementType === m ? x : Co(m, x), em(r, a, m, x, d);
      case 1:
        return m = a.type, x = a.pendingProps, x = a.elementType === m ? x : Co(m, x), ib(r, a, m, x, d);
      case 3:
        e: {
          if (ab(a), r === null) throw Error(n(387));
          m = a.pendingProps, P = a.memoizedState, x = P.element, w1(r, a), fd(a, m, null, d);
          var M = a.memoizedState;
          if (m = M.element, P.isDehydrated) if (P = { element: m, isDehydrated: !1, cache: M.cache, pendingSuspenseBoundaries: M.pendingSuspenseBoundaries, transitions: M.transitions }, a.updateQueue.baseState = P, a.memoizedState = P, a.flags & 256) {
            x = _s(Error(n(423)), a), a = sb(r, a, m, d, x);
            break e;
          } else if (m !== x) {
            x = _s(Error(n(424)), a), a = sb(r, a, m, d, x);
            break e;
          } else for (Br = Ii(a.stateNode.containerInfo.firstChild), zr = a, pn = !0, $o = null, d = b1(a, null, m, d), a.child = d; d; ) d.flags = d.flags & -3 | 4096, d = d.sibling;
          else {
            if (ws(), m === x) {
              a = ci(r, a, d);
              break e;
            }
            gr(r, a, m, d);
          }
          a = a.child;
        }
        return a;
      case 5:
        return x1(a), r === null && Ah(a), m = a.type, x = a.pendingProps, P = r !== null ? r.memoizedProps : null, M = x.children, wh(m, x) ? M = null : P !== null && wh(m, P) && (a.flags |= 32), ob(r, a), gr(r, a, M, d), a.child;
      case 6:
        return r === null && Ah(a), null;
      case 13:
        return lb(r, a, d);
      case 4:
        return Lh(a, a.stateNode.containerInfo), m = a.pendingProps, r === null ? a.child = $s(a, null, m, d) : gr(r, a, m, d), a.child;
      case 11:
        return m = a.type, x = a.pendingProps, x = a.elementType === m ? x : Co(m, x), eb(r, a, m, x, d);
      case 7:
        return gr(r, a, a.pendingProps, d), a.child;
      case 8:
        return gr(r, a, a.pendingProps.children, d), a.child;
      case 12:
        return gr(r, a, a.pendingProps.children, d), a.child;
      case 10:
        e: {
          if (m = a.type._context, x = a.pendingProps, P = a.memoizedProps, M = x.value, nn(ud, m._currentValue), m._currentValue = M, P !== null) if (wo(P.value, M)) {
            if (P.children === x.children && !_r.current) {
              a = ci(r, a, d);
              break e;
            }
          } else for (P = a.child, P !== null && (P.return = a); P !== null; ) {
            var K = P.dependencies;
            if (K !== null) {
              M = P.child;
              for (var te = K.firstContext; te !== null; ) {
                if (te.context === m) {
                  if (P.tag === 1) {
                    te = ui(-1, d & -d), te.tag = 2;
                    var he = P.updateQueue;
                    if (he !== null) {
                      he = he.shared;
                      var Ee = he.pending;
                      Ee === null ? te.next = te : (te.next = Ee.next, Ee.next = te), he.pending = te;
                    }
                  }
                  P.lanes |= d, te = P.alternate, te !== null && (te.lanes |= d), Mh(
                    P.return,
                    d,
                    a
                  ), K.lanes |= d;
                  break;
                }
                te = te.next;
              }
            } else if (P.tag === 10) M = P.type === a.type ? null : P.child;
            else if (P.tag === 18) {
              if (M = P.return, M === null) throw Error(n(341));
              M.lanes |= d, K = M.alternate, K !== null && (K.lanes |= d), Mh(M, d, a), M = P.sibling;
            } else M = P.child;
            if (M !== null) M.return = P;
            else for (M = P; M !== null; ) {
              if (M === a) {
                M = null;
                break;
              }
              if (P = M.sibling, P !== null) {
                P.return = M.return, M = P;
                break;
              }
              M = M.return;
            }
            P = M;
          }
          gr(r, a, x.children, d), a = a.child;
        }
        return a;
      case 9:
        return x = a.type, m = a.pendingProps.children, xs(a, d), x = to(x), m = m(x), a.flags |= 1, gr(r, a, m, d), a.child;
      case 14:
        return m = a.type, x = Co(m, a.pendingProps), x = Co(m.type, x), tb(r, a, m, x, d);
      case 15:
        return nb(r, a, a.type, a.pendingProps, d);
      case 17:
        return m = a.type, x = a.pendingProps, x = a.elementType === m ? x : Co(m, x), wd(r, a), a.tag = 1, kr(m) ? (r = !0, rd(a)) : r = !1, xs(a, d), H1(a, m, x), Qh(a, m, x, d), tm(null, a, m, !0, r, d);
      case 19:
        return cb(r, a, d);
      case 22:
        return rb(r, a, d);
    }
    throw Error(n(156, a.tag));
  };
  function Tb(r, a) {
    return da(r, a);
  }
  function m5(r, a, d, m) {
    this.tag = r, this.key = d, this.sibling = this.child = this.return = this.stateNode = this.type = this.elementType = null, this.index = 0, this.ref = null, this.pendingProps = a, this.dependencies = this.memoizedState = this.updateQueue = this.memoizedProps = null, this.mode = m, this.subtreeFlags = this.flags = 0, this.deletions = null, this.childLanes = this.lanes = 0, this.alternate = null;
  }
  function oo(r, a, d, m) {
    return new m5(r, a, d, m);
  }
  function wm(r) {
    return r = r.prototype, !(!r || !r.isReactComponent);
  }
  function g5(r) {
    if (typeof r == "function") return wm(r) ? 1 : 0;
    if (r != null) {
      if (r = r.$$typeof, r === W) return 11;
      if (r === X) return 14;
    }
    return 2;
  }
  function Ui(r, a) {
    var d = r.alternate;
    return d === null ? (d = oo(r.tag, a, r.key, r.mode), d.elementType = r.elementType, d.type = r.type, d.stateNode = r.stateNode, d.alternate = r, r.alternate = d) : (d.pendingProps = a, d.type = r.type, d.flags = 0, d.subtreeFlags = 0, d.deletions = null), d.flags = r.flags & 14680064, d.childLanes = r.childLanes, d.lanes = r.lanes, d.child = r.child, d.memoizedProps = r.memoizedProps, d.memoizedState = r.memoizedState, d.updateQueue = r.updateQueue, a = r.dependencies, d.dependencies = a === null ? null : { lanes: a.lanes, firstContext: a.firstContext }, d.sibling = r.sibling, d.index = r.index, d.ref = r.ref, d;
  }
  function jd(r, a, d, m, x, P) {
    var M = 2;
    if (m = r, typeof r == "function") wm(r) && (M = 1);
    else if (typeof r == "string") M = 5;
    else e: switch (r) {
      case j:
        return Pa(d.children, x, P, a);
      case L:
        M = 8, x |= 8;
        break;
      case B:
        return r = oo(12, d, a, x | 2), r.elementType = B, r.lanes = P, r;
      case q:
        return r = oo(13, d, a, x), r.elementType = q, r.lanes = P, r;
      case Q:
        return r = oo(19, d, a, x), r.elementType = Q, r.lanes = P, r;
      case J:
        return Rd(d, x, P, a);
      default:
        if (typeof r == "object" && r !== null) switch (r.$$typeof) {
          case O:
            M = 10;
            break e;
          case V:
            M = 9;
            break e;
          case W:
            M = 11;
            break e;
          case X:
            M = 14;
            break e;
          case ce:
            M = 16, m = null;
            break e;
        }
        throw Error(n(130, r == null ? r : typeof r, ""));
    }
    return a = oo(M, d, a, x), a.elementType = r, a.type = m, a.lanes = P, a;
  }
  function Pa(r, a, d, m) {
    return r = oo(7, r, m, a), r.lanes = d, r;
  }
  function Rd(r, a, d, m) {
    return r = oo(22, r, m, a), r.elementType = J, r.lanes = d, r.stateNode = { isHidden: !1 }, r;
  }
  function $m(r, a, d) {
    return r = oo(6, r, null, a), r.lanes = d, r;
  }
  function Cm(r, a, d) {
    return a = oo(4, r.children !== null ? r.children : [], r.key, a), a.lanes = d, a.stateNode = { containerInfo: r.containerInfo, pendingChildren: null, implementation: r.implementation }, a;
  }
  function y5(r, a, d, m, x) {
    this.tag = a, this.containerInfo = r, this.finishedWork = this.pingCache = this.current = this.pendingChildren = null, this.timeoutHandle = -1, this.callbackNode = this.pendingContext = this.context = null, this.callbackPriority = 0, this.eventTimes = Wl(0), this.expirationTimes = Wl(-1), this.entangledLanes = this.finishedLanes = this.mutableReadLanes = this.expiredLanes = this.pingedLanes = this.suspendedLanes = this.pendingLanes = 0, this.entanglements = Wl(0), this.identifierPrefix = m, this.onRecoverableError = x, this.mutableSourceEagerHydrationData = null;
  }
  function xm(r, a, d, m, x, P, M, K, te) {
    return r = new y5(r, a, d, K, te), a === 1 ? (a = 1, P === !0 && (a |= 8)) : a = 0, P = oo(3, null, null, a), r.current = P, P.stateNode = r, P.memoizedState = { element: m, isDehydrated: d, cache: null, transitions: null, pendingSuspenseBoundaries: null }, Fh(P), r;
  }
  function v5(r, a, d) {
    var m = 3 < arguments.length && arguments[3] !== void 0 ? arguments[3] : null;
    return { $$typeof: E, key: m == null ? null : "" + m, children: r, containerInfo: a, implementation: d };
  }
  function Mb(r) {
    if (!r) return Ri;
    r = r._reactInternals;
    e: {
      if (yn(r) !== r || r.tag !== 1) throw Error(n(170));
      var a = r;
      do {
        switch (a.tag) {
          case 3:
            a = a.stateNode.context;
            break e;
          case 1:
            if (kr(a.type)) {
              a = a.stateNode.__reactInternalMemoizedMergedChildContext;
              break e;
            }
        }
        a = a.return;
      } while (a !== null);
      throw Error(n(171));
    }
    if (r.tag === 1) {
      var d = r.type;
      if (kr(d)) return u1(r, d, a);
    }
    return a;
  }
  function Nb(r, a, d, m, x, P, M, K, te) {
    return r = xm(d, m, !0, r, x, P, M, K, te), r.context = Mb(null), d = r.current, m = yr(), x = zi(d), P = ui(m, x), P.callback = a ?? null, Ni(d, P, x), r.current.lanes = x, To(r, x, m), Ar(r, m), r;
  }
  function Td(r, a, d, m) {
    var x = a.current, P = yr(), M = zi(x);
    return d = Mb(d), a.context === null ? a.context = d : a.pendingContext = d, a = ui(P, M), a.payload = { element: r }, m = m === void 0 ? null : m, m !== null && (a.callback = m), r = Ni(x, a, M), r !== null && (_o(r, x, M, P), dd(r, x, M)), M;
  }
  function Md(r) {
    if (r = r.current, !r.child) return null;
    switch (r.child.tag) {
      case 5:
        return r.child.stateNode;
      default:
        return r.child.stateNode;
    }
  }
  function Fb(r, a) {
    if (r = r.memoizedState, r !== null && r.dehydrated !== null) {
      var d = r.retryLane;
      r.retryLane = d !== 0 && d < a ? d : a;
    }
  }
  function Em(r, a) {
    Fb(r, a), (r = r.alternate) && Fb(r, a);
  }
  var Lb = typeof reportError == "function" ? reportError : function(r) {
    console.error(r);
  };
  function _m(r) {
    this._internalRoot = r;
  }
  Nd.prototype.render = _m.prototype.render = function(r) {
    var a = this._internalRoot;
    if (a === null) throw Error(n(409));
    Td(r, a, null, null);
  }, Nd.prototype.unmount = _m.prototype.unmount = function() {
    var r = this._internalRoot;
    if (r !== null) {
      this._internalRoot = null;
      var a = r.containerInfo;
      Ea(function() {
        Td(null, r, null, null);
      }), a[oi] = null;
    }
  };
  function Nd(r) {
    this._internalRoot = r;
  }
  Nd.prototype.unstable_scheduleHydration = function(r) {
    if (r) {
      var a = de();
      r = { blockedOn: null, target: r, priority: a };
      for (var d = 0; d < Ct.length && a !== 0 && a < Ct[d].priority; d++) ;
      Ct.splice(d, 0, r), d === 0 && ha(r);
    }
  };
  function km(r) {
    return !(!r || r.nodeType !== 1 && r.nodeType !== 9 && r.nodeType !== 11);
  }
  function Fd(r) {
    return !(!r || r.nodeType !== 1 && r.nodeType !== 9 && r.nodeType !== 11 && (r.nodeType !== 8 || r.nodeValue !== " react-mount-point-unstable "));
  }
  function Db() {
  }
  function b5(r, a, d, m, x) {
    if (x) {
      if (typeof m == "function") {
        var P = m;
        m = function() {
          var he = Md(M);
          P.call(he);
        };
      }
      var M = Nb(a, m, r, 0, null, !1, !1, "", Db);
      return r._reactRootContainer = M, r[oi] = M.current, Zl(r.nodeType === 8 ? r.parentNode : r), Ea(), M;
    }
    for (; x = r.lastChild; ) r.removeChild(x);
    if (typeof m == "function") {
      var K = m;
      m = function() {
        var he = Md(te);
        K.call(he);
      };
    }
    var te = xm(r, 0, !1, null, null, !1, !1, "", Db);
    return r._reactRootContainer = te, r[oi] = te.current, Zl(r.nodeType === 8 ? r.parentNode : r), Ea(function() {
      Td(a, te, d, m);
    }), te;
  }
  function Ld(r, a, d, m, x) {
    var P = d._reactRootContainer;
    if (P) {
      var M = P;
      if (typeof x == "function") {
        var K = x;
        x = function() {
          var te = Md(M);
          K.call(te);
        };
      }
      Td(a, M, r, x);
    } else M = b5(d, a, r, x, m);
    return Md(M);
  }
  F = function(r) {
    switch (r.tag) {
      case 3:
        var a = r.stateNode;
        if (a.current.memoizedState.isDehydrated) {
          var d = ri(a.pendingLanes);
          d !== 0 && (D(a, d | 1), Ar(a, fn()), !(Tt & 6) && (Os = fn() + 500, Ti()));
        }
        break;
      case 13:
        Ea(function() {
          var m = li(r, 1);
          if (m !== null) {
            var x = yr();
            _o(m, r, 1, x);
          }
        }), Em(r, 1);
    }
  }, ee = function(r) {
    if (r.tag === 13) {
      var a = li(r, 134217728);
      if (a !== null) {
        var d = yr();
        _o(a, r, 134217728, d);
      }
      Em(r, 134217728);
    }
  }, pe = function(r) {
    if (r.tag === 13) {
      var a = zi(r), d = li(r, a);
      if (d !== null) {
        var m = yr();
        _o(d, r, a, m);
      }
      Em(r, a);
    }
  }, de = function() {
    return I;
  }, ne = function(r, a) {
    var d = I;
    try {
      return I = r, a();
    } finally {
      I = d;
    }
  }, jn = function(r, a, d) {
    switch (a) {
      case "input":
        if (st(r, d), a = d.name, d.type === "radio" && a != null) {
          for (d = r; d.parentNode; ) d = d.parentNode;
          for (d = d.querySelectorAll("input[name=" + JSON.stringify("" + a) + '][type="radio"]'), a = 0; a < d.length; a++) {
            var m = d[a];
            if (m !== r && m.form === r.form) {
              var x = td(m);
              if (!x) throw Error(n(90));
              We(m), st(m, x);
            }
          }
        }
        break;
      case "textarea":
        ye(r, d);
        break;
      case "select":
        a = d.value, a != null && yt(r, !!d.multiple, a, !1);
    }
  }, Ut = vm, Yt = Ea;
  var S5 = { usingClientEntryPoint: !1, Events: [nu, gs, td, Jt, $n, vm] }, gu = { findFiberByHostInstance: ya, bundleType: 0, version: "18.3.1", rendererPackageName: "react-dom" }, w5 = { bundleType: gu.bundleType, version: gu.version, rendererPackageName: gu.rendererPackageName, rendererConfig: gu.rendererConfig, overrideHookState: null, overrideHookStateDeletePath: null, overrideHookStateRenamePath: null, overrideProps: null, overridePropsDeletePath: null, overridePropsRenamePath: null, setErrorHandler: null, setSuspenseHandler: null, scheduleUpdate: null, currentDispatcherRef: b.ReactCurrentDispatcher, findHostInstanceByFiber: function(r) {
    return r = vo(r), r === null ? null : r.stateNode;
  }, findFiberByHostInstance: gu.findFiberByHostInstance, findHostInstancesForRefresh: null, scheduleRefresh: null, scheduleRoot: null, setRefreshHandler: null, getCurrentFiber: null, reconcilerVersion: "18.3.1-next-f1338f8080-20240426" };
  if (typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ < "u") {
    var Dd = __REACT_DEVTOOLS_GLOBAL_HOOK__;
    if (!Dd.isDisabled && Dd.supportsFiber) try {
      fa = Dd.inject(w5), Qr = Dd;
    } catch {
    }
  }
  return Ir.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED = S5, Ir.createPortal = function(r, a) {
    var d = 2 < arguments.length && arguments[2] !== void 0 ? arguments[2] : null;
    if (!km(a)) throw Error(n(200));
    return v5(r, a, null, d);
  }, Ir.createRoot = function(r, a) {
    if (!km(r)) throw Error(n(299));
    var d = !1, m = "", x = Lb;
    return a != null && (a.unstable_strictMode === !0 && (d = !0), a.identifierPrefix !== void 0 && (m = a.identifierPrefix), a.onRecoverableError !== void 0 && (x = a.onRecoverableError)), a = xm(r, 1, !1, null, null, d, !1, m, x), r[oi] = a.current, Zl(r.nodeType === 8 ? r.parentNode : r), new _m(a);
  }, Ir.findDOMNode = function(r) {
    if (r == null) return null;
    if (r.nodeType === 1) return r;
    var a = r._reactInternals;
    if (a === void 0)
      throw typeof r.render == "function" ? Error(n(188)) : (r = Object.keys(r).join(","), Error(n(268, r)));
    return r = vo(a), r = r === null ? null : r.stateNode, r;
  }, Ir.flushSync = function(r) {
    return Ea(r);
  }, Ir.hydrate = function(r, a, d) {
    if (!Fd(a)) throw Error(n(200));
    return Ld(null, r, a, !0, d);
  }, Ir.hydrateRoot = function(r, a, d) {
    if (!km(r)) throw Error(n(405));
    var m = d != null && d.hydratedSources || null, x = !1, P = "", M = Lb;
    if (d != null && (d.unstable_strictMode === !0 && (x = !0), d.identifierPrefix !== void 0 && (P = d.identifierPrefix), d.onRecoverableError !== void 0 && (M = d.onRecoverableError)), a = Nb(a, null, r, 1, d ?? null, x, !1, P, M), r[oi] = a.current, Zl(r), m) for (r = 0; r < m.length; r++) d = m[r], x = d._getVersion, x = x(d._source), a.mutableSourceEagerHydrationData == null ? a.mutableSourceEagerHydrationData = [d, x] : a.mutableSourceEagerHydrationData.push(
      d,
      x
    );
    return new Nd(a);
  }, Ir.render = function(r, a, d) {
    if (!Fd(a)) throw Error(n(200));
    return Ld(null, r, a, !1, d);
  }, Ir.unmountComponentAtNode = function(r) {
    if (!Fd(r)) throw Error(n(40));
    return r._reactRootContainer ? (Ea(function() {
      Ld(null, null, r, !1, function() {
        r._reactRootContainer = null, r[oi] = null;
      });
    }), !0) : !1;
  }, Ir.unstable_batchedUpdates = vm, Ir.unstable_renderSubtreeIntoContainer = function(r, a, d, m) {
    if (!Fd(d)) throw Error(n(200));
    if (r == null || r._reactInternals === void 0) throw Error(n(38));
    return Ld(r, a, d, !1, m);
  }, Ir.version = "18.3.1-next-f1338f8080-20240426", Ir;
}
function d$() {
  if (!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ > "u" || typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE != "function"))
    try {
      __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(d$);
    } catch (e) {
      console.error(e);
    }
}
d$(), c$.exports = O5();
var iy = c$.exports;
const zd = /* @__PURE__ */ Ft(iy);
var f$, Wb = iy;
f$ = Wb.createRoot, Wb.hydrateRoot;
var A5 = Object.defineProperty, I5 = (e, t, n) => t in e ? A5(e, t, { enumerable: !0, configurable: !0, writable: !0, value: n }) : e[t] = n, Bd = (e, t, n) => I5(e, typeof t != "symbol" ? t + "" : t, n);
const j5 = {
  stringify: (e) => e ? "true" : "false",
  parse: (e) => /^[ty1-9]/i.test(e)
}, R5 = {
  stringify: (e) => e.name,
  parse: (e, t, n) => {
    const o = (() => {
      if (typeof window < "u" && e in window)
        return window[e];
      if (typeof global < "u" && e in global)
        return global[e];
    })();
    return typeof o == "function" ? o.bind(n) : void 0;
  }
}, T5 = {
  stringify: (e) => JSON.stringify(e),
  parse: (e) => JSON.parse(e)
}, M5 = {
  stringify: (e) => `${e}`,
  parse: (e) => parseFloat(e)
}, N5 = {
  stringify: (e) => e,
  parse: (e) => e
}, Am = {
  string: N5,
  number: M5,
  boolean: j5,
  function: R5,
  json: T5
};
function F5(e) {
  return e.replace(
    /([a-z0-9])([A-Z])/g,
    (t, n, o) => `${n}-${o.toLowerCase()}`
  );
}
const Ud = Symbol.for("r2wc.render"), Vd = Symbol.for("r2wc.connected"), Oa = Symbol.for("r2wc.context"), fi = Symbol.for("r2wc.props");
function L5(e, t, n) {
  var o, i, s;
  t.props || (t.props = e.propTypes ? Object.keys(e.propTypes) : []), t.events || (t.events = []);
  const l = Array.isArray(t.props) ? t.props.slice() : Object.keys(t.props), u = Array.isArray(t.events) ? t.events.slice() : Object.keys(t.events), c = {}, f = {}, p = {}, g = {};
  for (const y of l) {
    c[y] = Array.isArray(t.props) ? "string" : t.props[y];
    const v = F5(y);
    p[y] = v, g[v] = y;
  }
  for (const y of u)
    f[y] = Array.isArray(t.events) ? {} : t.events[y];
  class h extends HTMLElement {
    constructor() {
      super(), Bd(this, s, !0), Bd(this, i), Bd(this, o, {}), Bd(this, "container"), t.shadow ? this.container = this.attachShadow({
        mode: t.shadow
      }) : this.container = this, this[fi].container = this.container;
      for (const v of l) {
        const C = p[v], $ = this.getAttribute(C), S = c[v], w = S ? Am[S] : null;
        w != null && w.parse && $ && (this[fi][v] = w.parse($, C, this));
      }
      for (const v of u)
        this[fi][v] = (C) => {
          const $ = v.replace(/^on/, "").toLowerCase();
          this.dispatchEvent(
            new CustomEvent($, { detail: C, ...f[v] })
          );
        };
    }
    static get observedAttributes() {
      return Object.keys(g);
    }
    connectedCallback() {
      this[Vd] = !0, this[Ud]();
    }
    disconnectedCallback() {
      this[Vd] = !1, this[Oa] && n.unmount(this[Oa]), delete this[Oa];
    }
    attributeChangedCallback(v, C, $) {
      const S = g[v], w = c[S], _ = w ? Am[w] : null;
      S in c && _ != null && _.parse && $ && (this[fi][S] = _.parse($, v, this), this[Ud]());
    }
    [(s = Vd, i = Oa, o = fi, Ud)]() {
      this[Vd] && (this[Oa] ? n.update(this[Oa], this[fi]) : this[Oa] = n.mount(
        this.container,
        e,
        this[fi]
      ));
    }
  }
  for (const y of l) {
    const v = p[y], C = c[y];
    Object.defineProperty(h.prototype, y, {
      enumerable: !0,
      configurable: !0,
      get() {
        return this[fi][y];
      },
      set($) {
        this[fi][y] = $;
        const S = C ? Am[C] : null;
        if (S != null && S.stringify) {
          const w = S.stringify($, v, this);
          this.getAttribute(v) !== w && this.setAttribute(v, w);
        } else
          this[Ud]();
      }
    });
  }
  return h;
}
function D5(e, t, n) {
  const o = f$(e), i = lo.createElement(t, n);
  return o.render(i), {
    root: o,
    ReactComponent: t
  };
}
function z5({ root: e, ReactComponent: t }, n) {
  const o = lo.createElement(t, n);
  e.render(o);
}
function B5({ root: e }) {
  e.unmount();
}
function ay(e, t = {}) {
  return L5(e, t, { mount: D5, update: z5, unmount: B5 });
}
var p$ = { exports: {} }, vu = {};
/**
 * @license React
 * react-jsx-runtime.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var qb;
function U5() {
  if (qb) return vu;
  qb = 1;
  var e = z, t = Symbol.for("react.element"), n = Symbol.for("react.fragment"), o = Object.prototype.hasOwnProperty, i = e.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.ReactCurrentOwner, s = { key: !0, ref: !0, __self: !0, __source: !0 };
  function l(u, c, f) {
    var p, g = {}, h = null, y = null;
    f !== void 0 && (h = "" + f), c.key !== void 0 && (h = "" + c.key), c.ref !== void 0 && (y = c.ref);
    for (p in c) o.call(c, p) && !s.hasOwnProperty(p) && (g[p] = c[p]);
    if (u && u.defaultProps) for (p in c = u.defaultProps, c) g[p] === void 0 && (g[p] = c[p]);
    return { $$typeof: t, type: u, key: h, ref: y, props: g, _owner: i.current };
  }
  return vu.Fragment = n, vu.jsx = l, vu.jsxs = l, vu;
}
p$.exports = U5();
var A = p$.exports;
function tn(e) {
  return typeof e != "object" || e === null || typeof e.lastModified == "number" && typeof File < "u" && e instanceof File || typeof e.getMonth == "function" && typeof Date < "u" && e instanceof Date ? !1 : !Array.isArray(e);
}
function V5(e) {
  return e.additionalItems === !0 && console.warn("additionalItems=true is currently not supported"), tn(e.additionalItems);
}
function Kb(e) {
  if (e === "")
    return;
  if (e === null)
    return null;
  if (/\.$/.test(e) || /\.0$/.test(e) || /\.\d*0$/.test(e))
    return e;
  const t = Number(e);
  return typeof t == "number" && !Number.isNaN(t) ? t : e;
}
const gl = "__additional_property", cg = "additionalProperties", gc = "allOf", Go = "anyOf", nl = "const", W5 = "default", Zf = "dependencies", q5 = "enum", Vr = "__errors", Yi = "$id", K5 = "if", Xi = "items", H5 = "_$junk_option_schema_id$_", pf = "$name", uo = "oneOf", wn = "properties", G5 = "required", Pf = "submitButtonOptions", Dn = "$ref", sy = "__rjsf_additionalProperties", h$ = "__rjsf_rootSchema", J5 = "ui:field", ly = "ui:widget", Bu = "ui:options", Y5 = "ui:globalOptions";
function Ot(e = {}, t = {}) {
  return Object.keys(e).filter((n) => n.indexOf("ui:") === 0).reduce((n, o) => {
    const i = e[o];
    return o === ly && tn(i) ? (console.error("Setting options via ui:widget object is no longer supported, use ui:options instead"), n) : o === Bu && tn(i) ? { ...n, ...i } : { ...n, [o.substring(3)]: i };
  }, { ...t });
}
function m$(e, t = {}, n) {
  if (!e.additionalProperties)
    return !1;
  const { expandable: o = !0 } = Ot(t);
  return o === !1 ? o : e.maxProperties !== void 0 && n ? Object.keys(n).length < e.maxProperties : !0;
}
var Q5 = typeof Tu == "object" && Tu && Tu.Object === Object && Tu, g$ = Q5, X5 = g$, Z5 = typeof self == "object" && self && self.Object === Object && self, eP = X5 || Z5 || Function("return this")(), ei = eP, tP = ei, nP = tP.Symbol, yl = nP, Hb = yl, y$ = Object.prototype, rP = y$.hasOwnProperty, oP = y$.toString, bu = Hb ? Hb.toStringTag : void 0;
function iP(e) {
  var t = rP.call(e, bu), n = e[bu];
  try {
    e[bu] = void 0;
    var o = !0;
  } catch {
  }
  var i = oP.call(e);
  return o && (t ? e[bu] = n : delete e[bu]), i;
}
var aP = iP, sP = Object.prototype, lP = sP.toString;
function uP(e) {
  return lP.call(e);
}
var cP = uP, Gb = yl, dP = aP, fP = cP, pP = "[object Null]", hP = "[object Undefined]", Jb = Gb ? Gb.toStringTag : void 0;
function mP(e) {
  return e == null ? e === void 0 ? hP : pP : Jb && Jb in Object(e) ? dP(e) : fP(e);
}
var ki = mP;
function gP(e, t) {
  return function(n) {
    return e(t(n));
  };
}
var v$ = gP, yP = v$, vP = yP(Object.getPrototypeOf, Object), ep = vP;
function bP(e) {
  return e != null && typeof e == "object";
}
var mo = bP, SP = ki, wP = ep, $P = mo, CP = "[object Object]", xP = Function.prototype, EP = Object.prototype, b$ = xP.toString, _P = EP.hasOwnProperty, kP = b$.call(Object);
function PP(e) {
  if (!$P(e) || SP(e) != CP)
    return !1;
  var t = wP(e);
  if (t === null)
    return !0;
  var n = _P.call(t, "constructor") && t.constructor;
  return typeof n == "function" && n instanceof n && b$.call(n) == kP;
}
var vl = PP;
const uy = /* @__PURE__ */ Ft(vl);
function dg(e) {
  const t = {
    // We store the list of errors for this node in a property named __errors
    // to avoid name collision with a possible sub schema field named
    // 'errors' (see `utils.toErrorSchema`).
    [Vr]: [],
    addError(n) {
      this[Vr].push(n);
    }
  };
  if (Array.isArray(e))
    return e.reduce((n, o, i) => ({ ...n, [i]: dg(o) }), t);
  if (uy(e)) {
    const n = e;
    return Object.keys(n).reduce((o, i) => ({ ...o, [i]: dg(n[i]) }), t);
  }
  return t;
}
function OP() {
  this.__data__ = [], this.size = 0;
}
var AP = OP;
function IP(e, t) {
  return e === t || e !== e && t !== t;
}
var bl = IP, jP = bl;
function RP(e, t) {
  for (var n = e.length; n--; )
    if (jP(e[n][0], t))
      return n;
  return -1;
}
var tp = RP, TP = tp, MP = Array.prototype, NP = MP.splice;
function FP(e) {
  var t = this.__data__, n = TP(t, e);
  if (n < 0)
    return !1;
  var o = t.length - 1;
  return n == o ? t.pop() : NP.call(t, n, 1), --this.size, !0;
}
var LP = FP, DP = tp;
function zP(e) {
  var t = this.__data__, n = DP(t, e);
  return n < 0 ? void 0 : t[n][1];
}
var BP = zP, UP = tp;
function VP(e) {
  return UP(this.__data__, e) > -1;
}
var WP = VP, qP = tp;
function KP(e, t) {
  var n = this.__data__, o = qP(n, e);
  return o < 0 ? (++this.size, n.push([e, t])) : n[o][1] = t, this;
}
var HP = KP, GP = AP, JP = LP, YP = BP, QP = WP, XP = HP;
function Sl(e) {
  var t = -1, n = e == null ? 0 : e.length;
  for (this.clear(); ++t < n; ) {
    var o = e[t];
    this.set(o[0], o[1]);
  }
}
Sl.prototype.clear = GP;
Sl.prototype.delete = JP;
Sl.prototype.get = YP;
Sl.prototype.has = QP;
Sl.prototype.set = XP;
var np = Sl, ZP = np;
function eO() {
  this.__data__ = new ZP(), this.size = 0;
}
var tO = eO;
function nO(e) {
  var t = this.__data__, n = t.delete(e);
  return this.size = t.size, n;
}
var rO = nO;
function oO(e) {
  return this.__data__.get(e);
}
var iO = oO;
function aO(e) {
  return this.__data__.has(e);
}
var sO = aO;
function lO(e) {
  var t = typeof e;
  return e != null && (t == "object" || t == "function");
}
var Fr = lO;
const co = /* @__PURE__ */ Ft(Fr);
var uO = ki, cO = Fr, dO = "[object AsyncFunction]", fO = "[object Function]", pO = "[object GeneratorFunction]", hO = "[object Proxy]";
function mO(e) {
  if (!cO(e))
    return !1;
  var t = uO(e);
  return t == fO || t == pO || t == dO || t == hO;
}
var rp = mO, gO = ei, yO = gO["__core-js_shared__"], vO = yO, Im = vO, Yb = function() {
  var e = /[^.]+$/.exec(Im && Im.keys && Im.keys.IE_PROTO || "");
  return e ? "Symbol(src)_1." + e : "";
}();
function bO(e) {
  return !!Yb && Yb in e;
}
var SO = bO, wO = Function.prototype, $O = wO.toString;
function CO(e) {
  if (e != null) {
    try {
      return $O.call(e);
    } catch {
    }
    try {
      return e + "";
    } catch {
    }
  }
  return "";
}
var S$ = CO, xO = rp, EO = SO, _O = Fr, kO = S$, PO = /[\\^$.*+?()[\]{}|]/g, OO = /^\[object .+?Constructor\]$/, AO = Function.prototype, IO = Object.prototype, jO = AO.toString, RO = IO.hasOwnProperty, TO = RegExp(
  "^" + jO.call(RO).replace(PO, "\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g, "$1.*?") + "$"
);
function MO(e) {
  if (!_O(e) || EO(e))
    return !1;
  var t = xO(e) ? TO : OO;
  return t.test(kO(e));
}
var NO = MO;
function FO(e, t) {
  return e == null ? void 0 : e[t];
}
var LO = FO, DO = NO, zO = LO;
function BO(e, t) {
  var n = zO(e, t);
  return DO(n) ? n : void 0;
}
var Ya = BO, UO = Ya, VO = ei, WO = UO(VO, "Map"), cy = WO, qO = Ya, KO = qO(Object, "create"), op = KO, Qb = op;
function HO() {
  this.__data__ = Qb ? Qb(null) : {}, this.size = 0;
}
var GO = HO;
function JO(e) {
  var t = this.has(e) && delete this.__data__[e];
  return this.size -= t ? 1 : 0, t;
}
var YO = JO, QO = op, XO = "__lodash_hash_undefined__", ZO = Object.prototype, eA = ZO.hasOwnProperty;
function tA(e) {
  var t = this.__data__;
  if (QO) {
    var n = t[e];
    return n === XO ? void 0 : n;
  }
  return eA.call(t, e) ? t[e] : void 0;
}
var nA = tA, rA = op, oA = Object.prototype, iA = oA.hasOwnProperty;
function aA(e) {
  var t = this.__data__;
  return rA ? t[e] !== void 0 : iA.call(t, e);
}
var sA = aA, lA = op, uA = "__lodash_hash_undefined__";
function cA(e, t) {
  var n = this.__data__;
  return this.size += this.has(e) ? 0 : 1, n[e] = lA && t === void 0 ? uA : t, this;
}
var dA = cA, fA = GO, pA = YO, hA = nA, mA = sA, gA = dA;
function wl(e) {
  var t = -1, n = e == null ? 0 : e.length;
  for (this.clear(); ++t < n; ) {
    var o = e[t];
    this.set(o[0], o[1]);
  }
}
wl.prototype.clear = fA;
wl.prototype.delete = pA;
wl.prototype.get = hA;
wl.prototype.has = mA;
wl.prototype.set = gA;
var yA = wl, Xb = yA, vA = np, bA = cy;
function SA() {
  this.size = 0, this.__data__ = {
    hash: new Xb(),
    map: new (bA || vA)(),
    string: new Xb()
  };
}
var wA = SA;
function $A(e) {
  var t = typeof e;
  return t == "string" || t == "number" || t == "symbol" || t == "boolean" ? e !== "__proto__" : e === null;
}
var CA = $A, xA = CA;
function EA(e, t) {
  var n = e.__data__;
  return xA(t) ? n[typeof t == "string" ? "string" : "hash"] : n.map;
}
var ip = EA, _A = ip;
function kA(e) {
  var t = _A(this, e).delete(e);
  return this.size -= t ? 1 : 0, t;
}
var PA = kA, OA = ip;
function AA(e) {
  return OA(this, e).get(e);
}
var IA = AA, jA = ip;
function RA(e) {
  return jA(this, e).has(e);
}
var TA = RA, MA = ip;
function NA(e, t) {
  var n = MA(this, e), o = n.size;
  return n.set(e, t), this.size += n.size == o ? 0 : 1, this;
}
var FA = NA, LA = wA, DA = PA, zA = IA, BA = TA, UA = FA;
function $l(e) {
  var t = -1, n = e == null ? 0 : e.length;
  for (this.clear(); ++t < n; ) {
    var o = e[t];
    this.set(o[0], o[1]);
  }
}
$l.prototype.clear = LA;
$l.prototype.delete = DA;
$l.prototype.get = zA;
$l.prototype.has = BA;
$l.prototype.set = UA;
var dy = $l, VA = np, WA = cy, qA = dy, KA = 200;
function HA(e, t) {
  var n = this.__data__;
  if (n instanceof VA) {
    var o = n.__data__;
    if (!WA || o.length < KA - 1)
      return o.push([e, t]), this.size = ++n.size, this;
    n = this.__data__ = new qA(o);
  }
  return n.set(e, t), this.size = n.size, this;
}
var GA = HA, JA = np, YA = tO, QA = rO, XA = iO, ZA = sO, eI = GA;
function Cl(e) {
  var t = this.__data__ = new JA(e);
  this.size = t.size;
}
Cl.prototype.clear = YA;
Cl.prototype.delete = QA;
Cl.prototype.get = XA;
Cl.prototype.has = ZA;
Cl.prototype.set = eI;
var ap = Cl, tI = "__lodash_hash_undefined__";
function nI(e) {
  return this.__data__.set(e, tI), this;
}
var rI = nI;
function oI(e) {
  return this.__data__.has(e);
}
var iI = oI, aI = dy, sI = rI, lI = iI;
function Of(e) {
  var t = -1, n = e == null ? 0 : e.length;
  for (this.__data__ = new aI(); ++t < n; )
    this.add(e[t]);
}
Of.prototype.add = Of.prototype.push = sI;
Of.prototype.has = lI;
var sp = Of;
function uI(e, t) {
  for (var n = -1, o = e == null ? 0 : e.length; ++n < o; )
    if (t(e[n], n, e))
      return !0;
  return !1;
}
var cI = uI;
function dI(e, t) {
  return e.has(t);
}
var lp = dI, fI = sp, pI = cI, hI = lp, mI = 1, gI = 2;
function yI(e, t, n, o, i, s) {
  var l = n & mI, u = e.length, c = t.length;
  if (u != c && !(l && c > u))
    return !1;
  var f = s.get(e), p = s.get(t);
  if (f && p)
    return f == t && p == e;
  var g = -1, h = !0, y = n & gI ? new fI() : void 0;
  for (s.set(e, t), s.set(t, e); ++g < u; ) {
    var v = e[g], C = t[g];
    if (o)
      var $ = l ? o(C, v, g, t, e, s) : o(v, C, g, e, t, s);
    if ($ !== void 0) {
      if ($)
        continue;
      h = !1;
      break;
    }
    if (y) {
      if (!pI(t, function(S, w) {
        if (!hI(y, w) && (v === S || i(v, S, n, o, s)))
          return y.push(w);
      })) {
        h = !1;
        break;
      }
    } else if (!(v === C || i(v, C, n, o, s))) {
      h = !1;
      break;
    }
  }
  return s.delete(e), s.delete(t), h;
}
var w$ = yI, vI = ei, bI = vI.Uint8Array, $$ = bI;
function SI(e) {
  var t = -1, n = Array(e.size);
  return e.forEach(function(o, i) {
    n[++t] = [i, o];
  }), n;
}
var wI = SI;
function $I(e) {
  var t = -1, n = Array(e.size);
  return e.forEach(function(o) {
    n[++t] = o;
  }), n;
}
var fy = $I, Zb = yl, eS = $$, CI = bl, xI = w$, EI = wI, _I = fy, kI = 1, PI = 2, OI = "[object Boolean]", AI = "[object Date]", II = "[object Error]", jI = "[object Map]", RI = "[object Number]", TI = "[object RegExp]", MI = "[object Set]", NI = "[object String]", FI = "[object Symbol]", LI = "[object ArrayBuffer]", DI = "[object DataView]", tS = Zb ? Zb.prototype : void 0, jm = tS ? tS.valueOf : void 0;
function zI(e, t, n, o, i, s, l) {
  switch (n) {
    case DI:
      if (e.byteLength != t.byteLength || e.byteOffset != t.byteOffset)
        return !1;
      e = e.buffer, t = t.buffer;
    case LI:
      return !(e.byteLength != t.byteLength || !s(new eS(e), new eS(t)));
    case OI:
    case AI:
    case RI:
      return CI(+e, +t);
    case II:
      return e.name == t.name && e.message == t.message;
    case TI:
    case NI:
      return e == t + "";
    case jI:
      var u = EI;
    case MI:
      var c = o & kI;
      if (u || (u = _I), e.size != t.size && !c)
        return !1;
      var f = l.get(e);
      if (f)
        return f == t;
      o |= PI, l.set(e, t);
      var p = xI(u(e), u(t), o, i, s, l);
      return l.delete(e), p;
    case FI:
      if (jm)
        return jm.call(e) == jm.call(t);
  }
  return !1;
}
var BI = zI;
function UI(e, t) {
  for (var n = -1, o = t.length, i = e.length; ++n < o; )
    e[i + n] = t[n];
  return e;
}
var py = UI, VI = Array.isArray, Kn = VI, WI = py, qI = Kn;
function KI(e, t, n) {
  var o = t(e);
  return qI(e) ? o : WI(o, n(e));
}
var C$ = KI;
function HI(e, t) {
  for (var n = -1, o = e == null ? 0 : e.length, i = 0, s = []; ++n < o; ) {
    var l = e[n];
    t(l, n, e) && (s[i++] = l);
  }
  return s;
}
var GI = HI;
function JI() {
  return [];
}
var x$ = JI, YI = GI, QI = x$, XI = Object.prototype, ZI = XI.propertyIsEnumerable, nS = Object.getOwnPropertySymbols, ej = nS ? function(e) {
  return e == null ? [] : (e = Object(e), YI(nS(e), function(t) {
    return ZI.call(e, t);
  }));
} : QI, hy = ej;
function tj(e, t) {
  for (var n = -1, o = Array(e); ++n < e; )
    o[n] = t(n);
  return o;
}
var E$ = tj, nj = ki, rj = mo, oj = "[object Arguments]";
function ij(e) {
  return rj(e) && nj(e) == oj;
}
var aj = ij, rS = aj, sj = mo, _$ = Object.prototype, lj = _$.hasOwnProperty, uj = _$.propertyIsEnumerable, cj = rS(/* @__PURE__ */ function() {
  return arguments;
}()) ? rS : function(e) {
  return sj(e) && lj.call(e, "callee") && !uj.call(e, "callee");
}, yc = cj, Af = { exports: {} };
function dj() {
  return !1;
}
var fj = dj;
Af.exports;
(function(e, t) {
  var n = ei, o = fj, i = t && !t.nodeType && t, s = i && !0 && e && !e.nodeType && e, l = s && s.exports === i, u = l ? n.Buffer : void 0, c = u ? u.isBuffer : void 0, f = c || o;
  e.exports = f;
})(Af, Af.exports);
var xl = Af.exports, pj = 9007199254740991, hj = /^(?:0|[1-9]\d*)$/;
function mj(e, t) {
  var n = typeof e;
  return t = t ?? pj, !!t && (n == "number" || n != "symbol" && hj.test(e)) && e > -1 && e % 1 == 0 && e < t;
}
var up = mj, gj = 9007199254740991;
function yj(e) {
  return typeof e == "number" && e > -1 && e % 1 == 0 && e <= gj;
}
var my = yj, vj = ki, bj = my, Sj = mo, wj = "[object Arguments]", $j = "[object Array]", Cj = "[object Boolean]", xj = "[object Date]", Ej = "[object Error]", _j = "[object Function]", kj = "[object Map]", Pj = "[object Number]", Oj = "[object Object]", Aj = "[object RegExp]", Ij = "[object Set]", jj = "[object String]", Rj = "[object WeakMap]", Tj = "[object ArrayBuffer]", Mj = "[object DataView]", Nj = "[object Float32Array]", Fj = "[object Float64Array]", Lj = "[object Int8Array]", Dj = "[object Int16Array]", zj = "[object Int32Array]", Bj = "[object Uint8Array]", Uj = "[object Uint8ClampedArray]", Vj = "[object Uint16Array]", Wj = "[object Uint32Array]", ln = {};
ln[Nj] = ln[Fj] = ln[Lj] = ln[Dj] = ln[zj] = ln[Bj] = ln[Uj] = ln[Vj] = ln[Wj] = !0;
ln[wj] = ln[$j] = ln[Tj] = ln[Cj] = ln[Mj] = ln[xj] = ln[Ej] = ln[_j] = ln[kj] = ln[Pj] = ln[Oj] = ln[Aj] = ln[Ij] = ln[jj] = ln[Rj] = !1;
function qj(e) {
  return Sj(e) && bj(e.length) && !!ln[vj(e)];
}
var Kj = qj;
function Hj(e) {
  return function(t) {
    return e(t);
  };
}
var Qa = Hj, If = { exports: {} };
If.exports;
(function(e, t) {
  var n = g$, o = t && !t.nodeType && t, i = o && !0 && e && !e.nodeType && e, s = i && i.exports === o, l = s && n.process, u = function() {
    try {
      var c = i && i.require && i.require("util").types;
      return c || l && l.binding && l.binding("util");
    } catch {
    }
  }();
  e.exports = u;
})(If, If.exports);
var gy = If.exports, Gj = Kj, Jj = Qa, oS = gy, iS = oS && oS.isTypedArray, Yj = iS ? Jj(iS) : Gj, vc = Yj, Qj = E$, Xj = yc, Zj = Kn, eR = xl, tR = up, nR = vc, rR = Object.prototype, oR = rR.hasOwnProperty;
function iR(e, t) {
  var n = Zj(e), o = !n && Xj(e), i = !n && !o && eR(e), s = !n && !o && !i && nR(e), l = n || o || i || s, u = l ? Qj(e.length, String) : [], c = u.length;
  for (var f in e)
    (t || oR.call(e, f)) && !(l && // Safari 9 has enumerable `arguments.length` in strict mode.
    (f == "length" || // Node.js 0.10 has enumerable non-index properties on buffers.
    i && (f == "offset" || f == "parent") || // PhantomJS 2 has enumerable non-index properties on typed arrays.
    s && (f == "buffer" || f == "byteLength" || f == "byteOffset") || // Skip index properties.
    tR(f, c))) && u.push(f);
  return u;
}
var k$ = iR, aR = Object.prototype;
function sR(e) {
  var t = e && e.constructor, n = typeof t == "function" && t.prototype || aR;
  return e === n;
}
var cp = sR, lR = v$, uR = lR(Object.keys, Object), cR = uR, dR = cp, fR = cR, pR = Object.prototype, hR = pR.hasOwnProperty;
function mR(e) {
  if (!dR(e))
    return fR(e);
  var t = [];
  for (var n in Object(e))
    hR.call(e, n) && n != "constructor" && t.push(n);
  return t;
}
var P$ = mR, gR = rp, yR = my;
function vR(e) {
  return e != null && yR(e.length) && !gR(e);
}
var Xa = vR, bR = k$, SR = P$, wR = Xa;
function $R(e) {
  return wR(e) ? bR(e) : SR(e);
}
var bc = $R, CR = C$, xR = hy, ER = bc;
function _R(e) {
  return CR(e, ER, xR);
}
var O$ = _R, aS = O$, kR = 1, PR = Object.prototype, OR = PR.hasOwnProperty;
function AR(e, t, n, o, i, s) {
  var l = n & kR, u = aS(e), c = u.length, f = aS(t), p = f.length;
  if (c != p && !l)
    return !1;
  for (var g = c; g--; ) {
    var h = u[g];
    if (!(l ? h in t : OR.call(t, h)))
      return !1;
  }
  var y = s.get(e), v = s.get(t);
  if (y && v)
    return y == t && v == e;
  var C = !0;
  s.set(e, t), s.set(t, e);
  for (var $ = l; ++g < c; ) {
    h = u[g];
    var S = e[h], w = t[h];
    if (o)
      var _ = l ? o(w, S, h, t, e, s) : o(S, w, h, e, t, s);
    if (!(_ === void 0 ? S === w || i(S, w, n, o, s) : _)) {
      C = !1;
      break;
    }
    $ || ($ = h == "constructor");
  }
  if (C && !$) {
    var b = e.constructor, k = t.constructor;
    b != k && "constructor" in e && "constructor" in t && !(typeof b == "function" && b instanceof b && typeof k == "function" && k instanceof k) && (C = !1);
  }
  return s.delete(e), s.delete(t), C;
}
var IR = AR, jR = Ya, RR = ei, TR = jR(RR, "DataView"), MR = TR, NR = Ya, FR = ei, LR = NR(FR, "Promise"), DR = LR, zR = Ya, BR = ei, UR = zR(BR, "Set"), A$ = UR, VR = Ya, WR = ei, qR = VR(WR, "WeakMap"), KR = qR, fg = MR, pg = cy, hg = DR, mg = A$, gg = KR, I$ = ki, El = S$, sS = "[object Map]", HR = "[object Object]", lS = "[object Promise]", uS = "[object Set]", cS = "[object WeakMap]", dS = "[object DataView]", GR = El(fg), JR = El(pg), YR = El(hg), QR = El(mg), XR = El(gg), ja = I$;
(fg && ja(new fg(new ArrayBuffer(1))) != dS || pg && ja(new pg()) != sS || hg && ja(hg.resolve()) != lS || mg && ja(new mg()) != uS || gg && ja(new gg()) != cS) && (ja = function(e) {
  var t = I$(e), n = t == HR ? e.constructor : void 0, o = n ? El(n) : "";
  if (o)
    switch (o) {
      case GR:
        return dS;
      case JR:
        return sS;
      case YR:
        return lS;
      case QR:
        return uS;
      case XR:
        return cS;
    }
  return t;
});
var Sc = ja, Rm = ap, ZR = w$, eT = BI, tT = IR, fS = Sc, pS = Kn, hS = xl, nT = vc, rT = 1, mS = "[object Arguments]", gS = "[object Array]", Wd = "[object Object]", oT = Object.prototype, yS = oT.hasOwnProperty;
function iT(e, t, n, o, i, s) {
  var l = pS(e), u = pS(t), c = l ? gS : fS(e), f = u ? gS : fS(t);
  c = c == mS ? Wd : c, f = f == mS ? Wd : f;
  var p = c == Wd, g = f == Wd, h = c == f;
  if (h && hS(e)) {
    if (!hS(t))
      return !1;
    l = !0, p = !1;
  }
  if (h && !p)
    return s || (s = new Rm()), l || nT(e) ? ZR(e, t, n, o, i, s) : eT(e, t, c, n, o, i, s);
  if (!(n & rT)) {
    var y = p && yS.call(e, "__wrapped__"), v = g && yS.call(t, "__wrapped__");
    if (y || v) {
      var C = y ? e.value() : e, $ = v ? t.value() : t;
      return s || (s = new Rm()), i(C, $, n, o, s);
    }
  }
  return h ? (s || (s = new Rm()), tT(e, t, n, o, i, s)) : !1;
}
var aT = iT, sT = aT, vS = mo;
function j$(e, t, n, o, i) {
  return e === t ? !0 : e == null || t == null || !vS(e) && !vS(t) ? e !== e && t !== t : sT(e, t, n, o, j$, i);
}
var dp = j$, lT = dp;
function uT(e, t, n) {
  n = typeof n == "function" ? n : void 0;
  var o = n ? n(e, t) : void 0;
  return o === void 0 ? lT(e, t, void 0, n) : !!o;
}
var cT = uT;
const dT = /* @__PURE__ */ Ft(cT);
function Sr(e, t) {
  return dT(e, t, (n, o) => {
    if (typeof n == "function" && typeof o == "function")
      return !0;
  });
}
var fT = ki, pT = mo, hT = "[object Symbol]";
function mT(e) {
  return typeof e == "symbol" || pT(e) && fT(e) == hT;
}
var _l = mT, gT = Kn, yT = _l, vT = /\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/, bT = /^\w*$/;
function ST(e, t) {
  if (gT(e))
    return !1;
  var n = typeof e;
  return n == "number" || n == "symbol" || n == "boolean" || e == null || yT(e) ? !0 : bT.test(e) || !vT.test(e) || t != null && e in Object(t);
}
var yy = ST, R$ = dy, wT = "Expected a function";
function vy(e, t) {
  if (typeof e != "function" || t != null && typeof t != "function")
    throw new TypeError(wT);
  var n = function() {
    var o = arguments, i = t ? t.apply(this, o) : o[0], s = n.cache;
    if (s.has(i))
      return s.get(i);
    var l = e.apply(this, o);
    return n.cache = s.set(i, l) || s, l;
  };
  return n.cache = new (vy.Cache || R$)(), n;
}
vy.Cache = R$;
var $T = vy, CT = $T, xT = 500;
function ET(e) {
  var t = CT(e, function(o) {
    return n.size === xT && n.clear(), o;
  }), n = t.cache;
  return t;
}
var _T = ET, kT = _T, PT = /[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g, OT = /\\(\\)?/g, AT = kT(function(e) {
  var t = [];
  return e.charCodeAt(0) === 46 && t.push(""), e.replace(PT, function(n, o, i, s) {
    t.push(i ? s.replace(OT, "$1") : o || n);
  }), t;
}), T$ = AT;
function IT(e, t) {
  for (var n = -1, o = e == null ? 0 : e.length, i = Array(o); ++n < o; )
    i[n] = t(e[n], n, e);
  return i;
}
var Pi = IT, bS = yl, jT = Pi, RT = Kn, TT = _l, SS = bS ? bS.prototype : void 0, wS = SS ? SS.toString : void 0;
function M$(e) {
  if (typeof e == "string")
    return e;
  if (RT(e))
    return jT(e, M$) + "";
  if (TT(e))
    return wS ? wS.call(e) : "";
  var t = e + "";
  return t == "0" && 1 / e == -1 / 0 ? "-0" : t;
}
var MT = M$, NT = MT;
function FT(e) {
  return e == null ? "" : NT(e);
}
var N$ = FT, LT = Kn, DT = yy, zT = T$, BT = N$;
function UT(e, t) {
  return LT(e) ? e : DT(e, t) ? [e] : zT(BT(e));
}
var kl = UT, VT = _l;
function WT(e) {
  if (typeof e == "string" || VT(e))
    return e;
  var t = e + "";
  return t == "0" && 1 / e == -1 / 0 ? "-0" : t;
}
var Za = WT, qT = kl, KT = Za;
function HT(e, t) {
  t = qT(t, e);
  for (var n = 0, o = t.length; e != null && n < o; )
    e = e[KT(t[n++])];
  return n && n == o ? e : void 0;
}
var wc = HT, GT = wc;
function JT(e, t, n) {
  var o = e == null ? void 0 : GT(e, t);
  return o === void 0 ? n : o;
}
var F$ = JT;
const Je = /* @__PURE__ */ Ft(F$);
var YT = P$, QT = Sc, XT = yc, ZT = Kn, e3 = Xa, t3 = xl, n3 = cp, r3 = vc, o3 = "[object Map]", i3 = "[object Set]", a3 = Object.prototype, s3 = a3.hasOwnProperty;
function l3(e) {
  if (e == null)
    return !0;
  if (e3(e) && (ZT(e) || typeof e == "string" || typeof e.splice == "function" || t3(e) || r3(e) || XT(e)))
    return !e.length;
  var t = QT(e);
  if (t == o3 || t == i3)
    return !e.size;
  if (n3(e))
    return !YT(e).length;
  for (var n in e)
    if (s3.call(e, n))
      return !1;
  return !0;
}
var u3 = l3;
const oa = /* @__PURE__ */ Ft(u3);
var fp = {}, c3 = /~/, d3 = /~[01]/g;
function f3(e) {
  switch (e) {
    case "~1":
      return "/";
    case "~0":
      return "~";
  }
  throw new Error("Invalid tilde escape: " + e);
}
function L$(e) {
  return c3.test(e) ? e.replace(d3, f3) : e;
}
function p3(e, t, n) {
  for (var o, i, s = 1, l = t.length; s < l; ) {
    if (t[s] === "constructor" || t[s] === "prototype" || t[s] === "__proto__") return e;
    if (o = L$(t[s++]), i = l > s, typeof e[o] > "u" && (Array.isArray(e) && o === "-" && (o = e.length), i && (t[s] !== "" && t[s] < 1 / 0 || t[s] === "-" ? e[o] = [] : e[o] = {})), !i) break;
    e = e[o];
  }
  var u = e[o];
  return n === void 0 ? delete e[o] : e[o] = n, u;
}
function by(e) {
  if (typeof e == "string") {
    if (e = e.split("/"), e[0] === "") return e;
    throw new Error("Invalid JSON pointer.");
  } else if (Array.isArray(e)) {
    for (const t of e)
      if (typeof t != "string" && typeof t != "number")
        throw new Error("Invalid JSON pointer. Must be of type string or number.");
    return e;
  }
  throw new Error("Invalid JSON pointer.");
}
function D$(e, t) {
  if (typeof e != "object") throw new Error("Invalid input object.");
  t = by(t);
  var n = t.length;
  if (n === 1) return e;
  for (var o = 1; o < n; ) {
    if (e = e[L$(t[o++])], n === o) return e;
    if (typeof e != "object" || e === null) return;
  }
}
function z$(e, t, n) {
  if (typeof e != "object") throw new Error("Invalid input object.");
  if (t = by(t), t.length === 0) throw new Error("Invalid JSON pointer for set.");
  return p3(e, t, n);
}
function h3(e) {
  var t = by(e);
  return {
    get: function(n) {
      return D$(n, t);
    },
    set: function(n, o) {
      return z$(n, t, o);
    }
  };
}
fp.get = D$;
fp.set = z$;
fp.compile = h3;
function m3(e, t) {
  for (var n = -1, o = e == null ? 0 : e.length; ++n < o && t(e[n], n, e) !== !1; )
    ;
  return e;
}
var Sy = m3, g3 = Ya, y3 = function() {
  try {
    var e = g3(Object, "defineProperty");
    return e({}, "", {}), e;
  } catch {
  }
}(), B$ = y3, $S = B$;
function v3(e, t, n) {
  t == "__proto__" && $S ? $S(e, t, {
    configurable: !0,
    enumerable: !0,
    value: n,
    writable: !0
  }) : e[t] = n;
}
var wy = v3, b3 = wy, S3 = bl, w3 = Object.prototype, $3 = w3.hasOwnProperty;
function C3(e, t, n) {
  var o = e[t];
  (!($3.call(e, t) && S3(o, n)) || n === void 0 && !(t in e)) && b3(e, t, n);
}
var $y = C3, x3 = $y, E3 = wy;
function _3(e, t, n, o) {
  var i = !n;
  n || (n = {});
  for (var s = -1, l = t.length; ++s < l; ) {
    var u = t[s], c = o ? o(n[u], e[u], u, n, e) : void 0;
    c === void 0 && (c = e[u]), i ? E3(n, u, c) : x3(n, u, c);
  }
  return n;
}
var Pl = _3, k3 = Pl, P3 = bc;
function O3(e, t) {
  return e && k3(t, P3(t), e);
}
var A3 = O3;
function I3(e) {
  var t = [];
  if (e != null)
    for (var n in Object(e))
      t.push(n);
  return t;
}
var j3 = I3, R3 = Fr, T3 = cp, M3 = j3, N3 = Object.prototype, F3 = N3.hasOwnProperty;
function L3(e) {
  if (!R3(e))
    return M3(e);
  var t = T3(e), n = [];
  for (var o in e)
    o == "constructor" && (t || !F3.call(e, o)) || n.push(o);
  return n;
}
var D3 = L3, z3 = k$, B3 = D3, U3 = Xa;
function V3(e) {
  return U3(e) ? z3(e, !0) : B3(e);
}
var Ol = V3, W3 = Pl, q3 = Ol;
function K3(e, t) {
  return e && W3(t, q3(t), e);
}
var H3 = K3, jf = { exports: {} };
jf.exports;
(function(e, t) {
  var n = ei, o = t && !t.nodeType && t, i = o && !0 && e && !e.nodeType && e, s = i && i.exports === o, l = s ? n.Buffer : void 0, u = l ? l.allocUnsafe : void 0;
  function c(f, p) {
    if (p)
      return f.slice();
    var g = f.length, h = u ? u(g) : new f.constructor(g);
    return f.copy(h), h;
  }
  e.exports = c;
})(jf, jf.exports);
var U$ = jf.exports;
function G3(e, t) {
  var n = -1, o = e.length;
  for (t || (t = Array(o)); ++n < o; )
    t[n] = e[n];
  return t;
}
var pp = G3, J3 = Pl, Y3 = hy;
function Q3(e, t) {
  return J3(e, Y3(e), t);
}
var X3 = Q3, Z3 = py, e4 = ep, t4 = hy, n4 = x$, r4 = Object.getOwnPropertySymbols, o4 = r4 ? function(e) {
  for (var t = []; e; )
    Z3(t, t4(e)), e = e4(e);
  return t;
} : n4, V$ = o4, i4 = Pl, a4 = V$;
function s4(e, t) {
  return i4(e, a4(e), t);
}
var l4 = s4, u4 = C$, c4 = V$, d4 = Ol;
function f4(e) {
  return u4(e, d4, c4);
}
var W$ = f4, p4 = Object.prototype, h4 = p4.hasOwnProperty;
function m4(e) {
  var t = e.length, n = new e.constructor(t);
  return t && typeof e[0] == "string" && h4.call(e, "index") && (n.index = e.index, n.input = e.input), n;
}
var g4 = m4, CS = $$;
function y4(e) {
  var t = new e.constructor(e.byteLength);
  return new CS(t).set(new CS(e)), t;
}
var Cy = y4, v4 = Cy;
function b4(e, t) {
  var n = t ? v4(e.buffer) : e.buffer;
  return new e.constructor(n, e.byteOffset, e.byteLength);
}
var S4 = b4, w4 = /\w*$/;
function $4(e) {
  var t = new e.constructor(e.source, w4.exec(e));
  return t.lastIndex = e.lastIndex, t;
}
var C4 = $4, xS = yl, ES = xS ? xS.prototype : void 0, _S = ES ? ES.valueOf : void 0;
function x4(e) {
  return _S ? Object(_S.call(e)) : {};
}
var E4 = x4, _4 = Cy;
function k4(e, t) {
  var n = t ? _4(e.buffer) : e.buffer;
  return new e.constructor(n, e.byteOffset, e.length);
}
var q$ = k4, P4 = Cy, O4 = S4, A4 = C4, I4 = E4, j4 = q$, R4 = "[object Boolean]", T4 = "[object Date]", M4 = "[object Map]", N4 = "[object Number]", F4 = "[object RegExp]", L4 = "[object Set]", D4 = "[object String]", z4 = "[object Symbol]", B4 = "[object ArrayBuffer]", U4 = "[object DataView]", V4 = "[object Float32Array]", W4 = "[object Float64Array]", q4 = "[object Int8Array]", K4 = "[object Int16Array]", H4 = "[object Int32Array]", G4 = "[object Uint8Array]", J4 = "[object Uint8ClampedArray]", Y4 = "[object Uint16Array]", Q4 = "[object Uint32Array]";
function X4(e, t, n) {
  var o = e.constructor;
  switch (t) {
    case B4:
      return P4(e);
    case R4:
    case T4:
      return new o(+e);
    case U4:
      return O4(e, n);
    case V4:
    case W4:
    case q4:
    case K4:
    case H4:
    case G4:
    case J4:
    case Y4:
    case Q4:
      return j4(e, n);
    case M4:
      return new o();
    case N4:
    case D4:
      return new o(e);
    case F4:
      return A4(e);
    case L4:
      return new o();
    case z4:
      return I4(e);
  }
}
var Z4 = X4, eM = Fr, kS = Object.create, tM = /* @__PURE__ */ function() {
  function e() {
  }
  return function(t) {
    if (!eM(t))
      return {};
    if (kS)
      return kS(t);
    e.prototype = t;
    var n = new e();
    return e.prototype = void 0, n;
  };
}(), K$ = tM, nM = K$, rM = ep, oM = cp;
function iM(e) {
  return typeof e.constructor == "function" && !oM(e) ? nM(rM(e)) : {};
}
var H$ = iM, aM = Sc, sM = mo, lM = "[object Map]";
function uM(e) {
  return sM(e) && aM(e) == lM;
}
var cM = uM, dM = cM, fM = Qa, PS = gy, OS = PS && PS.isMap, pM = OS ? fM(OS) : dM, hM = pM, mM = Sc, gM = mo, yM = "[object Set]";
function vM(e) {
  return gM(e) && mM(e) == yM;
}
var bM = vM, SM = bM, wM = Qa, AS = gy, IS = AS && AS.isSet, $M = IS ? wM(IS) : SM, CM = $M, xM = ap, EM = Sy, _M = $y, kM = A3, PM = H3, OM = U$, AM = pp, IM = X3, jM = l4, RM = O$, TM = W$, MM = Sc, NM = g4, FM = Z4, LM = H$, DM = Kn, zM = xl, BM = hM, UM = Fr, VM = CM, WM = bc, qM = Ol, KM = 1, HM = 2, GM = 4, G$ = "[object Arguments]", JM = "[object Array]", YM = "[object Boolean]", QM = "[object Date]", XM = "[object Error]", J$ = "[object Function]", ZM = "[object GeneratorFunction]", eN = "[object Map]", tN = "[object Number]", Y$ = "[object Object]", nN = "[object RegExp]", rN = "[object Set]", oN = "[object String]", iN = "[object Symbol]", aN = "[object WeakMap]", sN = "[object ArrayBuffer]", lN = "[object DataView]", uN = "[object Float32Array]", cN = "[object Float64Array]", dN = "[object Int8Array]", fN = "[object Int16Array]", pN = "[object Int32Array]", hN = "[object Uint8Array]", mN = "[object Uint8ClampedArray]", gN = "[object Uint16Array]", yN = "[object Uint32Array]", rn = {};
rn[G$] = rn[JM] = rn[sN] = rn[lN] = rn[YM] = rn[QM] = rn[uN] = rn[cN] = rn[dN] = rn[fN] = rn[pN] = rn[eN] = rn[tN] = rn[Y$] = rn[nN] = rn[rN] = rn[oN] = rn[iN] = rn[hN] = rn[mN] = rn[gN] = rn[yN] = !0;
rn[XM] = rn[J$] = rn[aN] = !1;
function hf(e, t, n, o, i, s) {
  var l, u = t & KM, c = t & HM, f = t & GM;
  if (n && (l = i ? n(e, o, i, s) : n(e)), l !== void 0)
    return l;
  if (!UM(e))
    return e;
  var p = DM(e);
  if (p) {
    if (l = NM(e), !u)
      return AM(e, l);
  } else {
    var g = MM(e), h = g == J$ || g == ZM;
    if (zM(e))
      return OM(e, u);
    if (g == Y$ || g == G$ || h && !i) {
      if (l = c || h ? {} : LM(e), !u)
        return c ? jM(e, PM(l, e)) : IM(e, kM(l, e));
    } else {
      if (!rn[g])
        return i ? e : {};
      l = FM(e, g, u);
    }
  }
  s || (s = new xM());
  var y = s.get(e);
  if (y)
    return y;
  s.set(e, l), VM(e) ? e.forEach(function($) {
    l.add(hf($, t, n, $, e, s));
  }) : BM(e) && e.forEach(function($, S) {
    l.set(S, hf($, t, n, S, e, s));
  });
  var v = f ? c ? TM : RM : c ? qM : WM, C = p ? void 0 : v(e);
  return EM(C || e, function($, S) {
    C && (S = $, $ = e[S]), _M(l, S, hf($, t, n, S, e, s));
  }), l;
}
var Q$ = hf;
function vN(e) {
  var t = e == null ? 0 : e.length;
  return t ? e[t - 1] : void 0;
}
var X$ = vN;
function bN(e, t, n) {
  var o = -1, i = e.length;
  t < 0 && (t = -t > i ? 0 : i + t), n = n > i ? i : n, n < 0 && (n += i), i = t > n ? 0 : n - t >>> 0, t >>>= 0;
  for (var s = Array(i); ++o < i; )
    s[o] = e[o + t];
  return s;
}
var SN = bN, wN = wc, $N = SN;
function CN(e, t) {
  return t.length < 2 ? e : wN(e, $N(t, 0, -1));
}
var xN = CN, EN = kl, _N = X$, kN = xN, PN = Za;
function ON(e, t) {
  return t = EN(t, e), e = kN(e, t), e == null || delete e[PN(_N(t))];
}
var Z$ = ON, AN = vl;
function IN(e) {
  return AN(e) ? void 0 : e;
}
var jN = IN, jS = yl, RN = yc, TN = Kn, RS = jS ? jS.isConcatSpreadable : void 0;
function MN(e) {
  return TN(e) || RN(e) || !!(RS && e && e[RS]);
}
var NN = MN, FN = py, LN = NN;
function eC(e, t, n, o, i) {
  var s = -1, l = e.length;
  for (n || (n = LN), i || (i = []); ++s < l; ) {
    var u = e[s];
    t > 0 && n(u) ? t > 1 ? eC(u, t - 1, n, o, i) : FN(i, u) : o || (i[i.length] = u);
  }
  return i;
}
var hp = eC, DN = hp;
function zN(e) {
  var t = e == null ? 0 : e.length;
  return t ? DN(e, 1) : [];
}
var xy = zN;
function BN(e, t, n) {
  switch (n.length) {
    case 0:
      return e.call(t);
    case 1:
      return e.call(t, n[0]);
    case 2:
      return e.call(t, n[0], n[1]);
    case 3:
      return e.call(t, n[0], n[1], n[2]);
  }
  return e.apply(t, n);
}
var tC = BN, UN = tC, TS = Math.max;
function VN(e, t, n) {
  return t = TS(t === void 0 ? e.length - 1 : t, 0), function() {
    for (var o = arguments, i = -1, s = TS(o.length - t, 0), l = Array(s); ++i < s; )
      l[i] = o[t + i];
    i = -1;
    for (var u = Array(t + 1); ++i < t; )
      u[i] = o[i];
    return u[t] = n(l), UN(e, this, u);
  };
}
var nC = VN;
function WN(e) {
  return function() {
    return e;
  };
}
var qN = WN;
function KN(e) {
  return e;
}
var $c = KN, HN = qN, MS = B$, GN = $c, JN = MS ? function(e, t) {
  return MS(e, "toString", {
    configurable: !0,
    enumerable: !1,
    value: HN(t),
    writable: !0
  });
} : GN, YN = JN, QN = 800, XN = 16, ZN = Date.now;
function e6(e) {
  var t = 0, n = 0;
  return function() {
    var o = ZN(), i = XN - (o - n);
    if (n = o, i > 0) {
      if (++t >= QN)
        return arguments[0];
    } else
      t = 0;
    return e.apply(void 0, arguments);
  };
}
var t6 = e6, n6 = YN, r6 = t6, o6 = r6(n6), rC = o6, i6 = xy, a6 = nC, s6 = rC;
function l6(e) {
  return s6(a6(e, void 0, i6), e + "");
}
var oC = l6, u6 = Pi, c6 = Q$, d6 = Z$, f6 = kl, p6 = Pl, h6 = jN, m6 = oC, g6 = W$, y6 = 1, v6 = 2, b6 = 4, S6 = m6(function(e, t) {
  var n = {};
  if (e == null)
    return n;
  var o = !1;
  t = u6(t, function(s) {
    return s = f6(s, e), o || (o = s.length > 1), s;
  }), p6(e, g6(e), n), o && (n = c6(n, y6 | v6 | b6, h6));
  for (var i = t.length; i--; )
    d6(n, t[i]);
  return n;
}), w6 = S6;
const Rf = /* @__PURE__ */ Ft(w6);
function Ey(e, t) {
  const n = t[e];
  return [Rf(t, [e]), n];
}
function iC(e, t = {}, n = []) {
  const o = e || "";
  let i;
  if (o.startsWith("#"))
    i = decodeURIComponent(o.substring(1));
  else
    throw new Error(`Could not find a definition for ${e}.`);
  const s = fp.get(t, i);
  if (s === void 0)
    throw new Error(`Could not find a definition for ${e}.`);
  const l = s[Dn];
  if (l) {
    if (n.includes(l)) {
      if (n.length === 1)
        throw new Error(`Definition for ${e} is a circular reference`);
      const [p, ...g] = n, h = [...g, o, p].join(" -> ");
      throw new Error(`Definition for ${p} contains a circular reference through ${h}`);
    }
    const [u, c] = Ey(Dn, s), f = iC(c, t, [...n, o]);
    return Object.keys(u).length > 0 ? { ...u, ...f } : f;
  }
  return s;
}
function aC(e, t = {}) {
  return iC(e, t, []);
}
var $6 = Object.prototype, C6 = $6.hasOwnProperty;
function x6(e, t) {
  return e != null && C6.call(e, t);
}
var E6 = x6, _6 = kl, k6 = yc, P6 = Kn, O6 = up, A6 = my, I6 = Za;
function j6(e, t, n) {
  t = _6(t, e);
  for (var o = -1, i = t.length, s = !1; ++o < i; ) {
    var l = I6(t[o]);
    if (!(s = e != null && n(e, l)))
      break;
    e = e[l];
  }
  return s || ++o != i ? s : (i = e == null ? 0 : e.length, !!i && A6(i) && O6(l, i) && (P6(e) || k6(e)));
}
var sC = j6, R6 = E6, T6 = sC;
function M6(e, t) {
  return e != null && T6(e, t, R6);
}
var N6 = M6;
const br = /* @__PURE__ */ Ft(N6);
var F6 = ki, L6 = mo, D6 = "[object Number]";
function z6(e) {
  return typeof e == "number" || L6(e) && F6(e) == D6;
}
var B6 = z6;
const lC = /* @__PURE__ */ Ft(B6);
var U6 = ki, V6 = Kn, W6 = mo, q6 = "[object String]";
function K6(e) {
  return typeof e == "string" || !V6(e) && W6(e) && U6(e) == q6;
}
var H6 = K6;
const _y = /* @__PURE__ */ Ft(H6);
function G6(e, t, n, o) {
  var i = -1, s = e == null ? 0 : e.length;
  for (o && s && (n = e[++i]); ++i < s; )
    n = t(n, e[i], i, e);
  return n;
}
var J6 = G6;
function Y6(e) {
  return function(t, n, o) {
    for (var i = -1, s = Object(t), l = o(t), u = l.length; u--; ) {
      var c = l[e ? u : ++i];
      if (n(s[c], c, s) === !1)
        break;
    }
    return t;
  };
}
var Q6 = Y6, X6 = Q6, Z6 = X6(), uC = Z6, eF = uC, tF = bc;
function nF(e, t) {
  return e && eF(e, t, tF);
}
var cC = nF, rF = Xa;
function oF(e, t) {
  return function(n, o) {
    if (n == null)
      return n;
    if (!rF(n))
      return e(n, o);
    for (var i = n.length, s = t ? i : -1, l = Object(n); (t ? s-- : ++s < i) && o(l[s], s, l) !== !1; )
      ;
    return n;
  };
}
var iF = oF, aF = cC, sF = iF, lF = sF(aF), ky = lF, uF = ap, cF = dp, dF = 1, fF = 2;
function pF(e, t, n, o) {
  var i = n.length, s = i, l = !o;
  if (e == null)
    return !s;
  for (e = Object(e); i--; ) {
    var u = n[i];
    if (l && u[2] ? u[1] !== e[u[0]] : !(u[0] in e))
      return !1;
  }
  for (; ++i < s; ) {
    u = n[i];
    var c = u[0], f = e[c], p = u[1];
    if (l && u[2]) {
      if (f === void 0 && !(c in e))
        return !1;
    } else {
      var g = new uF();
      if (o)
        var h = o(f, p, c, e, t, g);
      if (!(h === void 0 ? cF(p, f, dF | fF, o, g) : h))
        return !1;
    }
  }
  return !0;
}
var hF = pF, mF = Fr;
function gF(e) {
  return e === e && !mF(e);
}
var dC = gF, yF = dC, vF = bc;
function bF(e) {
  for (var t = vF(e), n = t.length; n--; ) {
    var o = t[n], i = e[o];
    t[n] = [o, i, yF(i)];
  }
  return t;
}
var SF = bF;
function wF(e, t) {
  return function(n) {
    return n == null ? !1 : n[e] === t && (t !== void 0 || e in Object(n));
  };
}
var fC = wF, $F = hF, CF = SF, xF = fC;
function EF(e) {
  var t = CF(e);
  return t.length == 1 && t[0][2] ? xF(t[0][0], t[0][1]) : function(n) {
    return n === e || $F(n, e, t);
  };
}
var _F = EF;
function kF(e, t) {
  return e != null && t in Object(e);
}
var PF = kF, OF = PF, AF = sC;
function IF(e, t) {
  return e != null && AF(e, t, OF);
}
var pC = IF, jF = dp, RF = F$, TF = pC, MF = yy, NF = dC, FF = fC, LF = Za, DF = 1, zF = 2;
function BF(e, t) {
  return MF(e) && NF(t) ? FF(LF(e), t) : function(n) {
    var o = RF(n, e);
    return o === void 0 && o === t ? TF(n, e) : jF(t, o, DF | zF);
  };
}
var UF = BF;
function VF(e) {
  return function(t) {
    return t == null ? void 0 : t[e];
  };
}
var WF = VF, qF = wc;
function KF(e) {
  return function(t) {
    return qF(t, e);
  };
}
var HF = KF, GF = WF, JF = HF, YF = yy, QF = Za;
function XF(e) {
  return YF(e) ? GF(QF(e)) : JF(e);
}
var ZF = XF, eL = _F, tL = UF, nL = $c, rL = Kn, oL = ZF;
function iL(e) {
  return typeof e == "function" ? e : e == null ? nL : typeof e == "object" ? rL(e) ? tL(e[0], e[1]) : eL(e) : oL(e);
}
var Py = iL;
function aL(e, t, n, o, i) {
  return i(e, function(s, l, u) {
    n = o ? (o = !1, s) : t(n, s, l, u);
  }), n;
}
var sL = aL, lL = J6, uL = ky, cL = Py, dL = sL, fL = Kn;
function pL(e, t, n) {
  var o = fL(e) ? lL : dL, i = arguments.length < 3;
  return o(e, cL(t), n, i, uL);
}
var hL = pL;
const mL = /* @__PURE__ */ Ft(hL);
var gL = $c;
function yL(e) {
  return typeof e == "function" ? e : gL;
}
var hC = yL, vL = /\s/;
function bL(e) {
  for (var t = e.length; t-- && vL.test(e.charAt(t)); )
    ;
  return t;
}
var SL = bL, wL = SL, $L = /^\s+/;
function CL(e) {
  return e && e.slice(0, wL(e) + 1).replace($L, "");
}
var xL = CL, EL = xL, NS = Fr, _L = _l, FS = NaN, kL = /^[-+]0x[0-9a-f]+$/i, PL = /^0b[01]+$/i, OL = /^0o[0-7]+$/i, AL = parseInt;
function IL(e) {
  if (typeof e == "number")
    return e;
  if (_L(e))
    return FS;
  if (NS(e)) {
    var t = typeof e.valueOf == "function" ? e.valueOf() : e;
    e = NS(t) ? t + "" : t;
  }
  if (typeof e != "string")
    return e === 0 ? e : +e;
  e = EL(e);
  var n = PL.test(e);
  return n || OL.test(e) ? AL(e.slice(2), n ? 2 : 8) : kL.test(e) ? FS : +e;
}
var jL = IL, RL = jL, TL = 1 / 0, ML = 17976931348623157e292;
function NL(e) {
  if (!e)
    return e === 0 ? e : 0;
  if (e = RL(e), e === TL || e === -1 / 0) {
    var t = e < 0 ? -1 : 1;
    return t * ML;
  }
  return e === e ? e : 0;
}
var FL = NL, LL = FL;
function DL(e) {
  var t = LL(e), n = t % 1;
  return t === t ? n ? t - n : t : 0;
}
var zL = DL, BL = E$, UL = hC, VL = zL, WL = 9007199254740991, Tm = 4294967295, qL = Math.min;
function KL(e, t) {
  if (e = VL(e), e < 1 || e > WL)
    return [];
  var n = Tm, o = qL(e, Tm);
  t = UL(t), e -= Tm;
  for (var i = BL(o, t); ++n < e; )
    t(n);
  return i;
}
var HL = KL;
const mC = /* @__PURE__ */ Ft(HL);
function gC(e, t, n) {
  var o;
  if (e && n) {
    const i = Je(e, n);
    if (i === void 0)
      return;
    for (let s = 0; s < t.length; s++) {
      const l = t[s], u = Je(l, [wn, n], {});
      if (!(u.type === "object" || u.type === "array") && (u.const === i || !((o = u.enum) === null || o === void 0) && o.includes(i)))
        return s;
    }
  }
}
function yC(e, t, n, o, i) {
  if (t === void 0)
    return 0;
  const s = gC(t, n, i);
  if (lC(s))
    return s;
  for (let l = 0; l < n.length; l++) {
    const u = n[l];
    if (i && br(u, [wn, i])) {
      const c = Je(t, i), f = Je(u, [wn, i], {});
      if (e.isValid(f, c, o))
        return l;
    } else if (u[wn]) {
      const c = {
        anyOf: Object.keys(u[wn]).map((p) => ({
          required: [p]
        }))
      };
      let f;
      if (u.anyOf) {
        const { ...p } = u;
        p.allOf ? p.allOf = p.allOf.slice() : p.allOf = [], p.allOf.push(c), f = p;
      } else
        f = Object.assign({}, u, c);
      if (delete f.required, e.isValid(f, t, o))
        return l;
    } else if (e.isValid(u, t, o))
      return l;
  }
  return 0;
}
function Oy(e, t, n, o, i) {
  return yC(e, t, n, o, i);
}
var GL = dp;
function JL(e, t) {
  return GL(e, t);
}
var Ay = JL;
const Ua = /* @__PURE__ */ Ft(Ay);
var YL = $y, QL = kl, XL = up, LS = Fr, ZL = Za;
function e9(e, t, n, o) {
  if (!LS(e))
    return e;
  t = QL(t, e);
  for (var i = -1, s = t.length, l = s - 1, u = e; u != null && ++i < s; ) {
    var c = ZL(t[i]), f = n;
    if (c === "__proto__" || c === "constructor" || c === "prototype")
      return e;
    if (i != l) {
      var p = u[c];
      f = o ? o(p, c, u) : void 0, f === void 0 && (f = LS(p) ? p : XL(t[i + 1]) ? [] : {});
    }
    YL(u, c, f), u = u[c];
  }
  return e;
}
var Iy = e9, t9 = Iy;
function n9(e, t, n) {
  return e == null ? e : t9(e, t, n);
}
var r9 = n9;
const Yn = /* @__PURE__ */ Ft(r9);
var o9 = Sy, i9 = K$, a9 = cC, s9 = Py, l9 = ep, u9 = Kn, c9 = xl, d9 = rp, f9 = Fr, p9 = vc;
function h9(e, t, n) {
  var o = u9(e), i = o || c9(e) || p9(e);
  if (t = s9(t), n == null) {
    var s = e && e.constructor;
    i ? n = o ? new s() : [] : f9(e) ? n = d9(s) ? i9(l9(e)) : {} : n = {};
  }
  return (i ? o9 : a9)(e, function(l, u, c) {
    return t(n, l, u, c);
  }), n;
}
var m9 = h9;
const g9 = /* @__PURE__ */ Ft(m9);
var y9 = wy, v9 = bl;
function b9(e, t, n) {
  (n !== void 0 && !v9(e[t], n) || n === void 0 && !(t in e)) && y9(e, t, n);
}
var vC = b9, S9 = Xa, w9 = mo;
function $9(e) {
  return w9(e) && S9(e);
}
var mp = $9;
function C9(e, t) {
  if (!(t === "constructor" && typeof e[t] == "function") && t != "__proto__")
    return e[t];
}
var bC = C9, x9 = Pl, E9 = Ol;
function _9(e) {
  return x9(e, E9(e));
}
var k9 = _9, DS = vC, P9 = U$, O9 = q$, A9 = pp, I9 = H$, zS = yc, BS = Kn, j9 = mp, R9 = xl, T9 = rp, M9 = Fr, N9 = vl, F9 = vc, US = bC, L9 = k9;
function D9(e, t, n, o, i, s, l) {
  var u = US(e, n), c = US(t, n), f = l.get(c);
  if (f) {
    DS(e, n, f);
    return;
  }
  var p = s ? s(u, c, n + "", e, t, l) : void 0, g = p === void 0;
  if (g) {
    var h = BS(c), y = !h && R9(c), v = !h && !y && F9(c);
    p = c, h || y || v ? BS(u) ? p = u : j9(u) ? p = A9(u) : y ? (g = !1, p = P9(c, !0)) : v ? (g = !1, p = O9(c, !0)) : p = [] : N9(c) || zS(c) ? (p = u, zS(u) ? p = L9(u) : (!M9(u) || T9(u)) && (p = I9(c))) : g = !1;
  }
  g && (l.set(c, p), i(p, c, o, s, l), l.delete(c)), DS(e, n, p);
}
var z9 = D9, B9 = ap, U9 = vC, V9 = uC, W9 = z9, q9 = Fr, K9 = Ol, H9 = bC;
function SC(e, t, n, o, i) {
  e !== t && V9(t, function(s, l) {
    if (i || (i = new B9()), q9(s))
      W9(e, t, l, n, SC, o, i);
    else {
      var u = o ? o(H9(e, l), s, l + "", e, t, i) : void 0;
      u === void 0 && (u = s), U9(e, l, u);
    }
  }, K9);
}
var jy = SC, G9 = $c, J9 = nC, Y9 = rC;
function Q9(e, t) {
  return Y9(J9(e, t, G9), e + "");
}
var la = Q9, X9 = bl, Z9 = Xa, eD = up, tD = Fr;
function nD(e, t, n) {
  if (!tD(n))
    return !1;
  var o = typeof t;
  return (o == "number" ? Z9(n) && eD(t, n.length) : o == "string" && t in n) ? X9(n[t], e) : !1;
}
var Ry = nD, rD = la, oD = Ry;
function iD(e) {
  return rD(function(t, n) {
    var o = -1, i = n.length, s = i > 1 ? n[i - 1] : void 0, l = i > 2 ? n[2] : void 0;
    for (s = e.length > 3 && typeof s == "function" ? (i--, s) : void 0, l && oD(n[0], n[1], l) && (s = i < 3 ? void 0 : s, i = 1), t = Object(t); ++o < i; ) {
      var u = n[o];
      u && e(t, u, o, s);
    }
    return t;
  });
}
var wC = iD, aD = jy, sD = wC, lD = sD(function(e, t, n) {
  aD(e, t, n);
}), uD = lD;
const cD = /* @__PURE__ */ Ft(uD);
var dD = hp, fD = 1 / 0;
function pD(e) {
  var t = e == null ? 0 : e.length;
  return t ? dD(e, fD) : [];
}
var Ty = pD;
const hD = /* @__PURE__ */ Ft(Ty);
function mD(e, t, n, o) {
  for (var i = e.length, s = n + (o ? 1 : -1); o ? s-- : ++s < i; )
    if (t(e[s], s, e))
      return s;
  return -1;
}
var gD = mD;
function yD(e) {
  return e !== e;
}
var vD = yD;
function bD(e, t, n) {
  for (var o = n - 1, i = e.length; ++o < i; )
    if (e[o] === t)
      return o;
  return -1;
}
var SD = bD, wD = gD, $D = vD, CD = SD;
function xD(e, t, n) {
  return t === t ? CD(e, t, n) : wD(e, $D, n);
}
var $C = xD, ED = $C;
function _D(e, t) {
  var n = e == null ? 0 : e.length;
  return !!n && ED(e, t, 0) > -1;
}
var My = _D;
function kD(e, t, n) {
  for (var o = -1, i = e == null ? 0 : e.length; ++o < i; )
    if (n(t, e[o]))
      return !0;
  return !1;
}
var Ny = kD;
function PD() {
}
var OD = PD, Mm = A$, AD = OD, ID = fy, jD = 1 / 0, RD = Mm && 1 / ID(new Mm([, -0]))[1] == jD ? function(e) {
  return new Mm(e);
} : AD, TD = RD, MD = sp, ND = My, FD = Ny, LD = lp, DD = TD, zD = fy, BD = 200;
function UD(e, t, n) {
  var o = -1, i = ND, s = e.length, l = !0, u = [], c = u;
  if (n)
    l = !1, i = FD;
  else if (s >= BD) {
    var f = t ? null : DD(e);
    if (f)
      return zD(f);
    l = !1, i = LD, c = new MD();
  } else
    c = t ? [] : u;
  e:
    for (; ++o < s; ) {
      var p = e[o], g = t ? t(p) : p;
      if (p = n || p !== 0 ? p : 0, l && g === g) {
        for (var h = c.length; h--; )
          if (c[h] === g)
            continue e;
        t && c.push(g), u.push(p);
      } else i(c, g, n) || (c !== u && c.push(g), u.push(p));
    }
  return u;
}
var Fy = UD, VD = Fy;
function WD(e) {
  return e && e.length ? VD(e) : [];
}
var gp = WD;
const qD = /* @__PURE__ */ Ft(gp);
var KD = Q$, HD = 1, GD = 4;
function JD(e) {
  return KD(e, HD | GD);
}
var CC = JD;
const xC = /* @__PURE__ */ Ft(CC);
var YD = ky, QD = Xa;
function XD(e, t) {
  var n = -1, o = QD(e) ? Array(e.length) : [];
  return YD(e, function(i, s, l) {
    o[++n] = t(i, s, l);
  }), o;
}
var ZD = XD;
function e8(e, t) {
  var n = e.length;
  for (e.sort(t); n--; )
    e[n] = e[n].value;
  return e;
}
var t8 = e8, VS = _l;
function n8(e, t) {
  if (e !== t) {
    var n = e !== void 0, o = e === null, i = e === e, s = VS(e), l = t !== void 0, u = t === null, c = t === t, f = VS(t);
    if (!u && !f && !s && e > t || s && l && c && !u && !f || o && l && c || !n && c || !i)
      return 1;
    if (!o && !s && !f && e < t || f && n && i && !o && !s || u && n && i || !l && i || !c)
      return -1;
  }
  return 0;
}
var r8 = n8, o8 = r8;
function i8(e, t, n) {
  for (var o = -1, i = e.criteria, s = t.criteria, l = i.length, u = n.length; ++o < l; ) {
    var c = o8(i[o], s[o]);
    if (c) {
      if (o >= u)
        return c;
      var f = n[o];
      return c * (f == "desc" ? -1 : 1);
    }
  }
  return e.index - t.index;
}
var a8 = i8, Nm = Pi, s8 = wc, l8 = Py, u8 = ZD, c8 = t8, d8 = Qa, f8 = a8, p8 = $c, h8 = Kn;
function m8(e, t, n) {
  t.length ? t = Nm(t, function(s) {
    return h8(s) ? function(l) {
      return s8(l, s.length === 1 ? s[0] : s);
    } : s;
  }) : t = [p8];
  var o = -1;
  t = Nm(t, d8(l8));
  var i = u8(e, function(s, l, u) {
    var c = Nm(t, function(f) {
      return f(s);
    });
    return { criteria: c, index: ++o, value: s };
  });
  return c8(i, function(s, l) {
    return f8(s, l, n);
  });
}
var g8 = m8, y8 = hp, v8 = g8, b8 = la, WS = Ry, S8 = b8(function(e, t) {
  if (e == null)
    return [];
  var n = t.length;
  return n > 1 && WS(e, t[0], t[1]) ? t = [] : n > 2 && WS(t[0], t[1], t[2]) && (t = [t[0]]), v8(e, y8(t, 1), []);
}), EC = S8, w8 = Fy;
function $8(e, t) {
  return t = typeof t == "function" ? t : void 0, e && e.length ? w8(e, void 0, t) : [];
}
var Ly = $8, C8 = la, x8 = bl, E8 = Ry, _8 = Ol, _C = Object.prototype, k8 = _C.hasOwnProperty, P8 = C8(function(e, t) {
  e = Object(e);
  var n = -1, o = t.length, i = o > 2 ? t[2] : void 0;
  for (i && E8(t[0], t[1], i) && (o = 1); ++n < o; )
    for (var s = t[n], l = _8(s), u = -1, c = l.length; ++u < c; ) {
      var f = l[u], p = e[f];
      (p === void 0 || x8(p, _C[f]) && !k8.call(e, f)) && (e[f] = s[f]);
    }
  return e;
}), O8 = P8, A8 = sp, I8 = My, j8 = Ny, R8 = Pi, T8 = Qa, qS = lp, M8 = Math.min;
function N8(e, t, n) {
  for (var o = n ? j8 : I8, i = e[0].length, s = e.length, l = s, u = Array(s), c = 1 / 0, f = []; l--; ) {
    var p = e[l];
    l && t && (p = R8(p, T8(t))), c = M8(p.length, c), u[l] = !n && (t || i >= 120 && p.length >= 120) ? new A8(l && p) : void 0;
  }
  p = e[0];
  var g = -1, h = u[0];
  e:
    for (; ++g < i && f.length < c; ) {
      var y = p[g], v = t ? t(y) : y;
      if (y = n || y !== 0 ? y : 0, !(h ? qS(h, v) : o(f, v, n))) {
        for (l = s; --l; ) {
          var C = u[l];
          if (!(C ? qS(C, v) : o(e[l], v, n)))
            continue e;
        }
        h && h.push(v), f.push(y);
      }
    }
  return f;
}
var kC = N8, F8 = mp;
function L8(e) {
  return F8(e) ? e : [];
}
var PC = L8, D8 = Pi, z8 = kC, B8 = la, U8 = PC, V8 = X$, W8 = B8(function(e) {
  var t = V8(e), n = D8(e, U8);
  return t = typeof t == "function" ? t : void 0, t && n.pop(), n.length && n[0] === e[0] ? z8(n, void 0, t) : [];
}), OC = W8, q8 = ki, K8 = mo, H8 = "[object Boolean]";
function G8(e) {
  return e === !0 || e === !1 || K8(e) && q8(e) == H8;
}
var J8 = G8, Io = Ay, Y8 = EC, Dy = gp, KS = Ly, Q8 = O8, X8 = OC, Tf = vl, Fm = J8, HS = (e) => Array.isArray(e) ? e : [e], fo = (e) => e === void 0, qd = (e) => Tf(e) || Array.isArray(e) ? Object.keys(e) : [], Hs = (e, t) => e.hasOwnProperty(t), rl = (e) => Y8(Dy(e)), GS = (e) => fo(e) || Array.isArray(e) && e.length === 0, Z8 = (e, t, n, o) => t && Hs(t, n) && e && Hs(e, n) && o(e[n], t[n]), Lm = (e, t) => fo(e) && t === 0 || fo(t) && e === 0 || Io(e, t), ez = (e, t) => fo(e) && t === !1 || fo(t) && e === !1 || Io(e, t), JS = (e) => fo(e) || Io(e, {}) || e === !0, Kd = (e) => fo(e) || Io(e, {}), YS = (e) => fo(e) || Tf(e) || e === !0 || e === !1;
function QS(e, t) {
  return GS(e) && GS(t) ? !0 : Io(rl(e), rl(t));
}
function tz(e, t) {
  return e = HS(e), t = HS(t), Io(rl(e), rl(t));
}
function mf(e, t, n, o) {
  var i = Dy(qd(e).concat(qd(t)));
  return Kd(e) && Kd(t) ? !0 : Kd(e) && qd(t).length || Kd(t) && qd(e).length ? !1 : i.every(function(s) {
    var l = e[s], u = t[s];
    return Array.isArray(l) && Array.isArray(u) ? Io(rl(e), rl(t)) : Array.isArray(l) && !Array.isArray(u) || Array.isArray(u) && !Array.isArray(l) ? !1 : Z8(e, t, s, o);
  });
}
function nz(e, t, n, o) {
  return Tf(e) && Tf(t) ? o(e, t) : Array.isArray(e) && Array.isArray(t) ? mf(e, t, n, o) : Io(e, t);
}
function Dm(e, t, n, o) {
  var i = KS(e, o), s = KS(t, o), l = X8(i, s, o);
  return l.length === Math.max(i.length, s.length);
}
var rz = {
  title: Io,
  uniqueItems: ez,
  minLength: Lm,
  minItems: Lm,
  minProperties: Lm,
  required: QS,
  enum: QS,
  type: tz,
  items: nz,
  anyOf: Dm,
  allOf: Dm,
  oneOf: Dm,
  properties: mf,
  patternProperties: mf,
  dependencies: mf
}, oz = [
  "properties",
  "patternProperties",
  "dependencies",
  "uniqueItems",
  "minLength",
  "minItems",
  "minProperties",
  "required"
], iz = ["additionalProperties", "additionalItems", "contains", "propertyNames", "not"];
function yg(e, t, n) {
  if (n = Q8(n, {
    ignore: []
  }), JS(e) && JS(t))
    return !0;
  if (!YS(e) || !YS(t))
    throw new Error("Either of the values are not a JSON schema.");
  if (e === t)
    return !0;
  if (Fm(e) && Fm(t))
    return e === t;
  if (e === void 0 && t === !1 || t === void 0 && e === !1 || fo(e) && !fo(t) || !fo(e) && fo(t))
    return !1;
  var o = Dy(Object.keys(e).concat(Object.keys(t)));
  if (n.ignore.length && (o = o.filter((s) => n.ignore.indexOf(s) === -1)), !o.length)
    return !0;
  function i(s, l) {
    return yg(s, l, n);
  }
  return o.every(function(s) {
    var l = e[s], u = t[s];
    if (iz.indexOf(s) !== -1)
      return yg(l, u, n);
    var c = rz[s];
    if (c || (c = Io), Io(l, u))
      return !0;
    if (oz.indexOf(s) === -1 && (!Hs(e, s) && Hs(t, s) || Hs(e, s) && !Hs(t, s)))
      return l === u;
    var f = c(l, u, s, i);
    if (!Fm(f))
      throw new Error("Comparer must return true or false");
    return f;
  });
}
var zy = yg;
function az(e) {
  return Object.prototype.toString.call(e) === "[object Array]";
}
var By = Array.isArray || az;
function sz(e) {
  return (typeof e == "number" || Object.prototype.toString.call(e) === "[object Number]") && e.valueOf() === e.valueOf();
}
var lz = sz, uz = lz;
function cz(e) {
  return uz(e) && e % 1 === 0;
}
var dz = cz, fz = By, pz = dz;
function hz(e) {
  var t;
  if (!fz(e) || (t = e.length, !t))
    return !1;
  for (var n = 0; n < t; n++)
    if (!pz(e[n]))
      return !1;
  return !0;
}
var AC = hz;
function mz(e) {
  return typeof e == "function";
}
var IC = mz, gz = By, XS = AC, yz = IC, Hd = Math.pow(2, 31) - 1;
function ZS(e, t) {
  var n = 1, o;
  if (e === 0)
    return t;
  if (t === 0)
    return e;
  for (; e % 2 === 0 && t % 2 === 0; )
    e = e / 2, t = t / 2, n = n * 2;
  for (; e % 2 === 0; )
    e = e / 2;
  for (; t; ) {
    for (; t % 2 === 0; )
      t = t / 2;
    e > t && (o = t, t = e, e = o), t = t - e;
  }
  return n * e;
}
function e2(e, t) {
  var n = 0, o;
  if (e === 0)
    return t;
  if (t === 0)
    return e;
  for (; !(e & 1) && !(t & 1); )
    e >>>= 1, t >>>= 1, n++;
  for (; !(e & 1); )
    e >>>= 1;
  for (; t; ) {
    for (; !(t & 1); )
      t >>>= 1;
    e > t && (o = t, t = e, e = o), t = t - e;
  }
  return e << n;
}
function vz() {
  var e = arguments.length, t, n, o, i, s, l, u;
  for (t = new Array(e), u = 0; u < e; u++)
    t[u] = arguments[u];
  if (XS(t)) {
    if (e === 2)
      return s = t[0], l = t[1], s < 0 && (s = -s), l < 0 && (l = -l), s <= Hd && l <= Hd ? e2(s, l) : ZS(s, l);
    o = t;
  } else if (gz(t[0]))
    if (e > 1) {
      if (o = t[0], n = t[1], !yz(n))
        throw new TypeError("gcd()::invalid input argument. Accessor must be a function. Value: `" + n + "`.");
    } else
      o = t[0];
  else throw new TypeError("gcd()::invalid input argument. Must provide an array of integers. Value: `" + t[0] + "`.");
  if (i = o.length, i < 2)
    return null;
  if (n) {
    for (s = new Array(i), u = 0; u < i; u++)
      s[u] = n(o[u], u);
    o = s;
  }
  if (e < 3 && !XS(o))
    throw new TypeError("gcd()::invalid input argument. Accessed array values must be integers. Value: `" + o + "`.");
  for (u = 0; u < i; u++)
    s = o[u], s < 0 && (o[u] = -s);
  for (s = o[0], u = 1; u < i; u++)
    l = o[u], l <= Hd && s <= Hd ? s = e2(s, l) : s = ZS(s, l);
  return s;
}
var bz = vz, t2 = bz, Sz = By, n2 = AC, wz = IC;
function $z() {
  var e = arguments.length, t, n, o, i, s, l, u;
  for (t = new Array(e), u = 0; u < e; u++)
    t[u] = arguments[u];
  if (n2(t)) {
    if (e === 2)
      return s = t[0], l = t[1], s < 0 && (s = -s), l < 0 && (l = -l), s === 0 || l === 0 ? 0 : s / t2(s, l) * l;
    o = t;
  } else if (Sz(t[0]))
    if (e > 1) {
      if (o = t[0], n = t[1], !wz(n))
        throw new TypeError("lcm()::invalid input argument. Accessor must be a function. Value: `" + n + "`.");
    } else
      o = t[0];
  else throw new TypeError("lcm()::invalid input argument. Must provide an array of integers. Value: `" + t[0] + "`.");
  if (i = o.length, i < 2)
    return null;
  if (n) {
    for (s = new Array(i), u = 0; u < i; u++)
      s[u] = n(o[u], u);
    o = s;
  }
  if (e < 3 && !n2(o))
    throw new TypeError("lcm()::invalid input argument. Accessed array values must be integers. Value: `" + o + "`.");
  for (u = 0; u < i; u++)
    s = o[u], s < 0 && (o[u] = -s);
  for (s = o[0], u = 1; u < i; u++) {
    if (l = o[u], s === 0 || l === 0)
      return 0;
    s = s / t2(s, l) * l;
  }
  return s;
}
var Cz = $z, xz = jy, r2 = Fr;
function jC(e, t, n, o, i, s) {
  return r2(e) && r2(t) && (s.set(t, e), xz(e, t, void 0, jC, s), s.delete(t)), e;
}
var Ez = jC, _z = jy, kz = wC, Pz = kz(function(e, t, n, o) {
  _z(e, t, n, o);
}), Oz = Pz, Az = tC, Iz = la, jz = Ez, Rz = Oz, Tz = Iz(function(e) {
  return e.push(void 0, jz), Az(Rz, void 0, e);
}), Mz = Tz, Nz = Pi, Fz = kC, Lz = la, Dz = PC, zz = Lz(function(e) {
  var t = Nz(e, Dz);
  return t.length && t[0] === e[0] ? Fz(t) : [];
}), Bz = zz;
function Uz(e, t, n, o) {
  for (var i = n - 1, s = e.length; ++i < s; )
    if (o(e[i], t))
      return i;
  return -1;
}
var Vz = Uz, Wz = Pi, qz = $C, Kz = Vz, Hz = Qa, Gz = pp, Jz = Array.prototype, o2 = Jz.splice;
function Yz(e, t, n, o) {
  var i = o ? Kz : qz, s = -1, l = t.length, u = e;
  for (e === t && (t = Gz(t)), n && (u = Wz(e, Hz(n))); ++s < l; )
    for (var c = 0, f = t[s], p = n ? n(f) : f; (c = i(u, p, c, o)) > -1; )
      u !== e && o2.call(u, c, 1), o2.call(e, c, 1);
  return e;
}
var Qz = Yz, Xz = Qz;
function Zz(e, t) {
  return e && e.length && t && t.length ? Xz(e, t) : e;
}
var eB = Zz, tB = Sy, nB = ky, rB = hC, oB = Kn;
function iB(e, t) {
  var n = oB(e) ? tB : nB;
  return n(e, rB(t));
}
var Uy = iB;
const aB = /* @__PURE__ */ Ft(Uy);
var sB = sp, lB = My, uB = Ny, cB = Pi, dB = Qa, fB = lp, pB = 200;
function hB(e, t, n, o) {
  var i = -1, s = lB, l = !0, u = e.length, c = [], f = t.length;
  if (!u)
    return c;
  n && (t = cB(t, dB(n))), o ? (s = uB, l = !1) : t.length >= pB && (s = fB, l = !1, t = new sB(t));
  e:
    for (; ++i < u; ) {
      var p = e[i], g = n == null ? p : n(p);
      if (p = o || p !== 0 ? p : 0, l && g === g) {
        for (var h = f; h--; )
          if (t[h] === g)
            continue e;
        c.push(p);
      } else s(t, g, o) || c.push(p);
    }
  return c;
}
var mB = hB, gB = mB, yB = la, vB = mp, bB = yB(function(e, t) {
  return vB(e) ? gB(e, t) : [];
}), SB = bB;
const wB = xy, $B = Ty, RC = vl, CB = gp, xB = Ly, EB = SB;
function _B(e) {
  for (const t in e)
    TC(e, t) && MC(e[t]) && delete e[t];
  return e;
}
const kB = (e) => CB($B(e.map(Vy))), PB = (e, t) => e.map((n) => n && n[t]), TC = (e, t) => Object.prototype.hasOwnProperty.call(e, t), Vy = (e) => RC(e) || Array.isArray(e) ? Object.keys(e) : [], OB = (e) => e !== void 0, AB = (e) => RC(e) || e === !0 || e === !1, MC = (e) => !Vy(e).length && e !== !1 && e !== !0, IB = (e, ...t) => EB.apply(null, [e].concat(wB(t)));
var NC = {
  allUniqueKeys: kB,
  deleteUndefinedProps: _B,
  getValues: PB,
  has: TC,
  isEmptySchema: MC,
  isSchema: AB,
  keys: Vy,
  notUndefined: OB,
  uniqWith: xB,
  withoutArr: IB
};
const jB = zy, RB = Uy, {
  allUniqueKeys: TB,
  deleteUndefinedProps: MB,
  getValues: NB,
  keys: Su,
  notUndefined: FB,
  uniqWith: LB,
  withoutArr: i2
} = NC;
function DB(e) {
  RB(e, function(t, n) {
    t === !1 && delete e[n];
  });
}
function a2(e, t) {
  return TB(e).reduce(function(o, i) {
    const s = NB(e, i), l = LB(s.filter(FB), jB);
    return o[i] = t(l, i), o;
  }, {});
}
var zB = {
  keywords: ["properties", "patternProperties", "additionalProperties"],
  resolver(e, t, n, o) {
    o.ignoreAdditionalProperties || (e.forEach(function(s) {
      const l = e.filter((p) => p !== s), u = Su(s.properties), f = Su(s.patternProperties).map((p) => new RegExp(p));
      l.forEach(function(p) {
        const g = Su(p.properties), h = g.filter((v) => f.some((C) => C.test(v)));
        i2(g, u, h).forEach(function(v) {
          p.properties[v] = n.properties([
            p.properties[v],
            s.additionalProperties
          ], v);
        });
      });
    }), e.forEach(function(s) {
      const l = e.filter((c) => c !== s), u = Su(s.patternProperties);
      s.additionalProperties === !1 && l.forEach(function(c) {
        const f = Su(c.patternProperties);
        i2(f, u).forEach((g) => delete c.patternProperties[g]);
      });
    }));
    const i = {
      additionalProperties: n.additionalProperties(e.map((s) => s.additionalProperties)),
      patternProperties: a2(e.map((s) => s.patternProperties), n.patternProperties),
      properties: a2(e.map((s) => s.properties), n.properties)
    };
    return i.additionalProperties === !1 && DB(i.properties), MB(i);
  }
};
const BB = zy, UB = Uy, {
  allUniqueKeys: VB,
  deleteUndefinedProps: WB,
  has: qB,
  isSchema: FC,
  notUndefined: LC,
  uniqWith: KB
} = NC;
function HB(e) {
  UB(e, function(t, n) {
    t === !1 && e.splice(n, 1);
  });
}
function GB(e, t) {
  return e.map(function(n) {
    if (n)
      if (Array.isArray(n.items)) {
        const o = n.items[t];
        if (FC(o))
          return o;
        if (qB(n, "additionalItems"))
          return n.additionalItems;
      } else
        return n.items;
  });
}
function JB(e) {
  return e.map(function(t) {
    if (t)
      return Array.isArray(t.items) ? t.additionalItems : t.items;
  });
}
function YB(e, t, n) {
  return VB(n).reduce(function(i, s) {
    const l = GB(e, s), u = KB(l.filter(LC), BB);
    return i[s] = t(u, s), i;
  }, []);
}
var QB = {
  keywords: ["items", "additionalItems"],
  resolver(e, t, n) {
    const o = e.map((u) => u.items), i = o.filter(LC), s = {};
    i.every(FC) ? s.items = n.items(o) : s.items = YB(e, n.items, o);
    let l;
    return i.every(Array.isArray) ? l = e.map((u) => u.additionalItems) : i.some(Array.isArray) && (l = JB(e)), l && (s.additionalItems = n.additionalItems(l)), s.additionalItems === !1 && Array.isArray(s.items) && HB(s.items), WB(s);
  }
};
const DC = CC, Mf = zy, XB = Cz, ZB = Mz, zC = xy, Wy = Ty, e7 = Bz, t7 = OC, vg = Ay, ol = vl, n7 = eB, BC = EC, qy = gp, Js = Ly, UC = zB, VC = QB, Gd = (e, t) => e.indexOf(t) !== -1, r7 = (e) => ol(e) || e === !0 || e === !1, o7 = (e) => e === !1, WC = (e) => e === !0, yp = (e, t, n) => n(e), qC = (e) => BC(qy(Wy(e))), Nf = (e) => e !== void 0, KC = (e) => qy(Wy(e.map(c7))), Al = (e) => e[0], i7 = (e) => qC(e), Cc = (e) => Math.max.apply(Math, e), xc = (e) => Math.min.apply(Math, e), a7 = (e) => e.some(WC), s7 = (e) => Js(zC(e), vg);
function l7(e) {
  return function(t, n) {
    return Mf({
      [e]: t
    }, { [e]: n });
  };
}
function HC(e) {
  let { allOf: t = [], ...n } = e;
  return n = ol(e) ? n : e, [n, ...t.map(HC)];
}
function GC(e, t) {
  return e.map((n) => n && n[t]);
}
function u7(e, t) {
  return e.map(function(n, o) {
    try {
      return t(n, o);
    } catch {
      return;
    }
  }).filter(Nf);
}
function c7(e) {
  return ol(e) || Array.isArray(e) ? Object.keys(e) : [];
}
function bg(e, t) {
  if (t = t || [], !e.length)
    return t;
  const n = e.slice(0).shift(), o = e.slice(1);
  return t.length ? bg(o, zC(t.map((i) => n.map((s) => [s].concat(i))))) : bg(o, n.map((i) => i));
}
function JC(e, t) {
  let n;
  try {
    n = e.map(function(o) {
      return JSON.stringify(o, null, 2);
    }).join(`
`);
  } catch {
    n = e.join(", ");
  }
  throw new Error('Could not resolve values for path:"' + t.join(".") + `". They are probably incompatible. Values: 
` + n);
}
function d7(e, t, n, o, i, s) {
  if (e.length) {
    const l = i.complexResolvers[t];
    if (!l || !l.resolver)
      throw new Error("No resolver found for " + t);
    const u = n.map((g) => e.reduce((h, y) => (g[y] !== void 0 && (h[y] = g[y]), h), {})), c = Js(u, Mf), f = l.keywords.reduce((g, h) => ({
      ...g,
      [h]: (y, v = []) => o(y, null, s.concat(h, v))
    }), {}), p = l.resolver(c, s.concat(t), f, i);
    return ol(p) || JC(c, s.concat(t)), p;
  }
}
function f7(e) {
  return { required: e };
}
const p7 = ["properties", "patternProperties", "definitions", "dependencies"], h7 = ["anyOf", "oneOf"], m7 = [
  "additionalProperties",
  "additionalItems",
  "contains",
  "propertyNames",
  "not",
  "items"
], Bt = {
  type(e) {
    if (e.some(Array.isArray)) {
      const t = e.map(function(o) {
        return Array.isArray(o) ? o : [o];
      }), n = e7.apply(null, t);
      if (n.length === 1)
        return n[0];
      if (n.length > 1)
        return qy(n);
    }
  },
  dependencies(e, t, n) {
    return KC(e).reduce(function(i, s) {
      const l = GC(e, s);
      let u = Js(l.filter(Nf), vg);
      const c = u.filter(Array.isArray);
      if (c.length) {
        if (c.length === u.length)
          i[s] = qC(u);
        else {
          const f = u.filter(r7), p = c.map(f7);
          i[s] = n(f.concat(p), s);
        }
        return i;
      }
      return u = Js(u, Mf), i[s] = n(u, s), i;
    }, {});
  },
  oneOf(e, t, n) {
    const o = bg(DC(e)), i = u7(o, n), s = Js(i, Mf);
    if (s.length)
      return s;
  },
  not(e) {
    return { anyOf: e };
  },
  pattern(e) {
    return e.map((t) => "(?=" + t + ")").join("");
  },
  multipleOf(e) {
    let t = e.slice(0), n = 1;
    for (; t.some((o) => !Number.isInteger(o)); )
      t = t.map((o) => o * 10), n = n * 10;
    return XB(t) / n;
  },
  enum(e) {
    const t = t7.apply(null, e.concat(vg));
    if (t.length)
      return BC(t);
  }
};
Bt.$id = Al;
Bt.$ref = Al;
Bt.$schema = Al;
Bt.additionalItems = yp;
Bt.additionalProperties = yp;
Bt.anyOf = Bt.oneOf;
Bt.contains = yp;
Bt.default = Al;
Bt.definitions = Bt.dependencies;
Bt.description = Al;
Bt.examples = s7;
Bt.exclusiveMaximum = xc;
Bt.exclusiveMinimum = Cc;
Bt.items = VC;
Bt.maximum = xc;
Bt.maxItems = xc;
Bt.maxLength = xc;
Bt.maxProperties = xc;
Bt.minimum = Cc;
Bt.minItems = Cc;
Bt.minLength = Cc;
Bt.minProperties = Cc;
Bt.properties = UC;
Bt.propertyNames = yp;
Bt.required = i7;
Bt.title = Al;
Bt.uniqueItems = a7;
const g7 = {
  properties: UC,
  items: VC
};
function Ky(e, t, n) {
  t = ZB(t, {
    ignoreAdditionalProperties: !1,
    resolvers: Bt,
    complexResolvers: g7,
    deep: !0
  });
  const o = Object.entries(t.complexResolvers);
  function i(u, c, f) {
    u = DC(u.filter(Nf)), f = f || [];
    const p = ol(c) ? c : {};
    if (!u.length)
      return;
    if (u.some(o7))
      return !1;
    if (u.every(WC))
      return !0;
    u = u.filter(ol);
    const g = KC(u);
    if (t.deep && Gd(g, "allOf"))
      return Ky({
        allOf: u
      }, t);
    const h = o.map(([y, v]) => g.filter((C) => v.keywords.includes(C)));
    return h.forEach((y) => n7(g, y)), g.forEach(function(y) {
      const v = GC(u, y), C = Js(v.filter(Nf), l7(y));
      if (C.length === 1 && Gd(h7, y))
        p[y] = C[0].map(($) => i([$], $));
      else if (C.length === 1 && !Gd(p7, y) && !Gd(m7, y))
        p[y] = C[0];
      else {
        const $ = t.resolvers[y] || t.resolvers.defaultResolver;
        if (!$) throw new Error("No resolver found for key " + y + ". You can provide a resolver for this keyword in the options, or provide a default resolver.");
        const S = (w, _ = []) => i(w, null, f.concat(y, _));
        p[y] = $(C, f.concat(y), S, t), p[y] === void 0 ? JC(C, f.concat(y)) : p[y] === void 0 && delete p[y];
      }
    }), o.reduce((y, [v, C], $) => ({
      ...y,
      ...d7(h[$], v, u, i, t, f)
    }), p);
  }
  const s = Wy(HC(e));
  return i(s);
}
Ky.options = {
  resolvers: Bt
};
var y7 = Ky;
const v7 = /* @__PURE__ */ Ft(y7);
function il(e) {
  let t;
  const n = Je(e, "discriminator.propertyName", void 0);
  return _y(n) ? t = n : n !== void 0 && console.warn(`Expecting discriminator to be a string, got "${typeof n}" instead`), t;
}
function ec(e) {
  return Array.isArray(e) ? "array" : typeof e == "string" ? "string" : e == null ? "null" : typeof e == "boolean" ? "boolean" : isNaN(e) ? typeof e == "object" ? "object" : "string" : "number";
}
var b7 = hp, S7 = la, w7 = Fy, $7 = mp, C7 = S7(function(e) {
  return w7(b7(e, 1, $7, !0));
}), x7 = C7;
const E7 = /* @__PURE__ */ Ft(x7);
function ia(e) {
  let { type: t } = e;
  return !t && e.const ? ec(e.const) : !t && e.enum ? "string" : !t && (e.properties || e.additionalProperties) ? "object" : (Array.isArray(t) && (t.length === 2 && t.includes("null") ? t = t.find((n) => n !== "null") : t = t[0]), t);
}
function $i(e, t) {
  const n = Object.assign({}, e);
  return Object.keys(t).reduce((o, i) => {
    const s = e ? e[i] : {}, l = t[i];
    return e && i in e && tn(l) ? o[i] = $i(s, l) : e && t && (ia(e) === "object" || ia(t) === "object") && i === G5 && Array.isArray(s) && Array.isArray(l) ? o[i] = E7(s, l) : o[i] = l, o;
  }, n);
}
function qr(e, t, n = {}, o, i) {
  return Jo(e, t, n, o, void 0, void 0, i)[0];
}
function _7(e, t, n, o, i, s, l) {
  const { if: u, then: c, else: f, ...p } = t, g = e.isValid(u, s || {}, n);
  let h = [p], y = [];
  if (o)
    c && typeof c != "boolean" && (y = y.concat(Jo(e, c, n, s, o, i, l))), f && typeof f != "boolean" && (y = y.concat(Jo(e, f, n, s, o, i, l)));
  else {
    const v = g ? c : f;
    v && typeof v != "boolean" && (y = y.concat(Jo(e, v, n, s, o, i, l)));
  }
  return y.length && (h = y.map((v) => $i(p, v))), h.flatMap((v) => Jo(e, v, n, s, o, i, l));
}
function YC(e) {
  return e.reduce(
    (n, o) => o.length > 1 ? o.flatMap((i) => mC(n.length, (s) => [...n[s]].concat(i))) : (n.forEach((i) => i.push(o[0])), n),
    [[]]
    // Start with an empty list
  );
}
function k7(e, t, n, o, i, s, l) {
  const u = QC(e, t, n, o, i, s);
  if (u.length > 1 || u[0] !== t)
    return u;
  if (Zf in t)
    return XC(e, t, n, o, i, s).flatMap((f) => Jo(e, f, n, s, o, i, l));
  if (gc in t && Array.isArray(t.allOf)) {
    const c = t.allOf.map((p) => Jo(e, p, n, s, o, i, l));
    return YC(c).map((p) => ({ ...t, allOf: p }));
  }
  return [t];
}
function QC(e, t, n, o, i, s, l) {
  const u = tc(t, n, i);
  return u !== t ? Jo(e, u, n, s, o, i, l) : [t];
}
function tc(e, t, n) {
  if (!tn(e))
    return e;
  let o = e;
  if (Dn in o) {
    const { $ref: i, ...s } = o;
    if (n.includes(i))
      return o;
    n.push(i), o = { ...aC(i, t), ...s };
  }
  if (wn in o) {
    const i = [], s = g9(o[wn], (l, u, c) => {
      const f = [...n];
      l[c] = tc(u, t, f), i.push(f);
    }, {});
    cD(n, qD(hD(i))), o = { ...o, [wn]: s };
  }
  return Xi in o && !Array.isArray(o.items) && typeof o.items != "boolean" && (o = {
    ...o,
    items: tc(o.items, t, n)
  }), Ua(e, o) ? e : o;
}
function P7(e, t, n, o, i) {
  const s = {
    ...t,
    properties: { ...t.properties }
  }, l = o && tn(o) ? o : {};
  return Object.keys(l).forEach((u) => {
    if (u in s.properties)
      return;
    let c = {};
    typeof s.additionalProperties != "boolean" ? Dn in s.additionalProperties ? c = qr(e, { $ref: Je(s.additionalProperties, [Dn]) }, n, l, i) : "type" in s.additionalProperties ? c = { ...s.additionalProperties } : Go in s.additionalProperties || uo in s.additionalProperties ? c = {
      type: "object",
      ...s.additionalProperties
    } : c = { type: ec(Je(l, [u])) } : c = { type: ec(Je(l, [u])) }, s.properties[u] = c, Yn(s.properties, [u, gl], !0);
  }), s;
}
function Jo(e, t, n, o, i = !1, s = [], l) {
  return tn(t) ? k7(e, t, n, i, s, o, l).flatMap((c) => {
    var f;
    let p = c;
    if (K5 in p)
      return _7(e, p, n, i, s, o, l);
    if (gc in p) {
      if (i) {
        const { allOf: h, ...y } = p;
        return [...h, y];
      }
      try {
        const h = [], y = [];
        (f = p.allOf) === null || f === void 0 || f.forEach((v) => {
          typeof v == "object" && v.contains ? h.push(v) : y.push(v);
        }), h.length && (p = { ...p, allOf: y }), p = l ? l(p) : v7(p, {
          deep: !1
        }), h.length && (p.allOf = h);
      } catch (h) {
        console.warn(`could not merge subschemas in allOf:
`, h);
        const { allOf: y, ...v } = p;
        return v;
      }
    }
    return cg in p && p.additionalProperties !== !1 ? P7(e, p, n, o, l) : p;
  }) : [{}];
}
function O7(e, t, n, o, i) {
  let s;
  const { oneOf: l, anyOf: u, ...c } = t;
  if (Array.isArray(l) ? s = l : Array.isArray(u) && (s = u), s) {
    const f = i === void 0 && o ? {} : i, p = il(t);
    s = s.map((h) => tc(h, n, []));
    const g = Oy(e, f, s, n, p);
    if (o)
      return s.map((h) => $i(c, h));
    t = $i(c, s[g]);
  }
  return [t];
}
function XC(e, t, n, o, i, s, l) {
  const { dependencies: u, ...c } = t;
  return O7(e, c, n, o, s).flatMap((p) => ZC(e, u, p, n, o, i, s, l));
}
function ZC(e, t, n, o, i, s, l, u) {
  let c = [n];
  for (const f in t) {
    if (!i && Je(l, [f]) === void 0 || n.properties && !(f in n.properties))
      continue;
    const [p, g] = Ey(f, t);
    return Array.isArray(g) ? c[0] = A7(n, g) : tn(g) && (c = I7(e, n, o, f, g, i, s, l, u)), c.flatMap((h) => ZC(e, p, h, o, i, s, l, u));
  }
  return c;
}
function A7(e, t) {
  if (!t)
    return e;
  const n = Array.isArray(e.required) ? Array.from(/* @__PURE__ */ new Set([...e.required, ...t])) : t;
  return { ...e, required: n };
}
function I7(e, t, n, o, i, s, l, u, c) {
  return Jo(e, i, n, u, s, l, c).flatMap((p) => {
    const { oneOf: g, ...h } = p;
    if (t = $i(t, h), g === void 0)
      return t;
    const y = g.map((C) => typeof C == "boolean" || !(Dn in C) ? [C] : QC(e, C, n, s, l, u));
    return YC(y).flatMap((C) => j7(e, t, n, o, C, s, l, u, c));
  });
}
function j7(e, t, n, o, i, s, l, u, c) {
  const f = i.filter((p) => {
    if (typeof p == "boolean" || !p || !p.properties)
      return !1;
    const { [o]: g } = p.properties;
    if (g) {
      const h = {
        type: "object",
        properties: {
          [o]: g
        }
      };
      return e.isValid(h, u, n) || s;
    }
    return !1;
  });
  return !s && f.length !== 1 ? (console.warn("ignoring oneOf in dependencies because there isn't exactly one subschema that is valid"), [t]) : f.flatMap((p) => {
    const g = p, [h] = Ey(o, g.properties), y = { ...g, properties: h };
    return Jo(e, y, n, u, s, l, c).map((C) => $i(t, C));
  });
}
const R7 = {
  type: "object",
  $id: H5,
  properties: {
    __not_really_there__: {
      type: "number"
    }
  }
};
function Sg(e, t, n, o, i) {
  let s = 0;
  return n && (co(n.properties) ? s += mL(n.properties, (l, u, c) => {
    const f = Je(o, c);
    if (typeof u == "boolean")
      return l;
    if (br(u, Dn)) {
      const p = qr(e, u, t, f, i);
      return l + Sg(e, t, p, f || {}, i);
    }
    if ((br(u, uo) || br(u, Go)) && f) {
      const p = br(u, uo) ? uo : Go, g = il(u);
      return l + nc(e, t, f, Je(u, p), -1, g, i);
    }
    if (u.type === "object")
      return co(f) && (l += 1), l + Sg(e, t, u, f, i);
    if (u.type === ec(f)) {
      let p = l + 1;
      return u.default ? p += f === u.default ? 1 : -1 : u.const && (p += f === u.const ? 1 : -1), p;
    }
    return l;
  }, 0) : _y(n.type) && n.type === ec(o) && (s += 1)), s;
}
function nc(e, t, n, o, i = -1, s, l) {
  const u = o.map((h) => tc(h, t, [])), c = gC(n, o, s);
  if (lC(c))
    return c;
  const f = u.reduce((h, y, v) => (Oy(e, n, [R7, y], t, s) === 1 && h.push(v), h), []);
  if (f.length === 1)
    return f[0];
  f.length || mC(u.length, (h) => f.push(h));
  const p = /* @__PURE__ */ new Set(), { bestIndex: g } = f.reduce((h, y) => {
    const { bestScore: v } = h, C = u[y], $ = Sg(e, t, C, n, l);
    return p.add($), $ > v ? { bestIndex: y, bestScore: $ } : h;
  }, { bestIndex: i, bestScore: 0 });
  return p.size === 1 && i >= 0 ? i : g;
}
function wg(e) {
  return Array.isArray(e.items) && e.items.length > 0 && e.items.every((t) => tn(t));
}
function rc(e, t, n = !1, o = !1) {
  if (Array.isArray(t)) {
    const i = Array.isArray(e) ? e : [], s = t.map((l, u) => i[u] ? rc(i[u], l, n, o) : l);
    return n && s.length < i.length && s.push(...i.slice(s.length)), s;
  }
  if (tn(t)) {
    const i = Object.assign({}, e);
    return Object.keys(t).reduce((s, l) => (s[l] = rc(e ? Je(e, l) : {}, Je(t, l), n, o), s), i);
  }
  return o && t === void 0 ? e : t;
}
function Zi(e, t, n = !1) {
  return Object.keys(t).reduce((o, i) => {
    const s = e ? e[i] : {}, l = t[i];
    if (e && i in e && tn(l))
      o[i] = Zi(s, l, n);
    else if (n && Array.isArray(s) && Array.isArray(l)) {
      let u = l;
      n === "preventDuplicates" && (u = l.reduce((c, f) => (s.includes(f) || c.push(f), c), [])), o[i] = s.concat(u);
    } else
      o[i] = l;
    return o;
  }, Object.assign({}, e));
}
function T7(e) {
  return Array.isArray(e.enum) && e.enum.length === 1 || nl in e;
}
function ex(e, t, n = {}, o) {
  const i = qr(e, t, n, void 0, o), s = i.oneOf || i.anyOf;
  return Array.isArray(i.enum) ? !0 : Array.isArray(s) ? s.every((l) => typeof l != "boolean" && T7(l)) : !1;
}
function Hy(e, t, n, o) {
  return !t.uniqueItems || !t.items || typeof t.items == "boolean" ? !1 : ex(e, t.items, n, o);
}
const M7 = ["string", "number", "integer", "boolean", "null"];
var al;
(function(e) {
  e[e.Ignore = 0] = "Ignore", e[e.Invert = 1] = "Invert", e[e.Fallback = 2] = "Fallback";
})(al || (al = {}));
function zm(e, t = al.Ignore, n = -1) {
  if (n >= 0) {
    if (Array.isArray(e.items) && n < e.items.length) {
      const o = e.items[n];
      if (typeof o != "boolean")
        return o;
    }
  } else if (e.items && !Array.isArray(e.items) && typeof e.items != "boolean")
    return e.items;
  return t !== al.Ignore && tn(e.additionalItems) ? e.additionalItems : {};
}
function s2(e, t, n, o, i, s = [], l = {}, u = !1) {
  const { emptyObjectFields: c = "populateAllDefaults" } = l;
  if (o || u)
    e[t] = n;
  else if (c !== "skipDefaults")
    if (tn(n)) {
      const f = i === void 0 ? s.includes(t) : i;
      c === "skipEmptyDefaults" ? oa(n) || (e[t] = n) : (!oa(n) || s.includes(t)) && (f || c !== "populateRequiredDefaults") && (e[t] = n);
    } else // Store computedDefault if it's a defined primitive (e.g., true) and satisfies certain conditions
    // Condition 1: computedDefault is not undefined
    // Condition 2: If emptyObjectFields is 'populateAllDefaults' or 'skipEmptyDefaults) or if the key is a required field
    n !== void 0 && (c === "populateAllDefaults" || c === "skipEmptyDefaults" || s.includes(t)) && (e[t] = n);
}
function ea(e, t, n = {}) {
  const { parentDefaults: o, rawFormData: i, rootSchema: s = {}, includeUndefinedValues: l = !1, _recurseList: u = [], experimental_defaultFormStateBehavior: c = void 0, experimental_customMergeAllOf: f = void 0, required: p } = n, g = tn(i) ? i : {}, h = tn(t) ? t : {};
  let y = o, v = null, C = c, $ = u;
  if (h[nl] && (c == null ? void 0 : c.constAsDefaults) !== "never")
    y = h.const;
  else if (tn(y) && tn(h.default))
    y = Zi(y, h.default);
  else if (W5 in h)
    y = h.default;
  else if (Dn in h) {
    const w = h[Dn];
    u.includes(w) || ($ = u.concat(w), v = aC(w, s));
  } else if (Zf in h) {
    const w = {
      ...l2(e, h, n, y),
      ...g
    };
    v = XC(e, h, s, !1, [], w, f)[0];
  } else if (wg(h))
    y = h.items.map((w, _) => ea(e, w, {
      rootSchema: s,
      includeUndefinedValues: l,
      _recurseList: u,
      experimental_defaultFormStateBehavior: c,
      experimental_customMergeAllOf: f,
      parentDefaults: Array.isArray(o) ? o[_] : void 0,
      rawFormData: g,
      required: p
    }));
  else if (uo in h) {
    const { oneOf: w, ..._ } = h;
    if (w.length === 0)
      return;
    const b = il(h), { type: k = "null" } = _;
    !Array.isArray(k) && M7.includes(k) && (C == null ? void 0 : C.constAsDefaults) === "skipOneOf" && (C = { ...C, constAsDefaults: "never" }), v = w[nc(e, s, oa(g) ? void 0 : g, w, 0, b, f)], v = $i(_, v);
  } else if (Go in h) {
    const { anyOf: w, ..._ } = h;
    if (w.length === 0)
      return;
    const b = il(h);
    v = w[nc(e, s, oa(g) ? void 0 : g, w, 0, b, f)], v = $i(_, v);
  }
  if (v)
    return ea(e, v, {
      rootSchema: s,
      includeUndefinedValues: l,
      _recurseList: $,
      experimental_defaultFormStateBehavior: C,
      experimental_customMergeAllOf: f,
      parentDefaults: y,
      rawFormData: g,
      required: p
    });
  y === void 0 && (y = h.default);
  const S = l2(e, h, n, y);
  return S ?? y;
}
function N7(e, t, { rawFormData: n, rootSchema: o = {}, includeUndefinedValues: i = !1, _recurseList: s = [], experimental_defaultFormStateBehavior: l = void 0, experimental_customMergeAllOf: u = void 0, required: c } = {}, f) {
  {
    const p = tn(n) ? n : {}, g = t, h = (l == null ? void 0 : l.allOf) === "populateDefaults" && gc in g ? qr(e, g, o, p, u) : g, y = h[nl], v = Object.keys(h.properties || {}).reduce((C, $) => {
      var S;
      const w = Je(h, [wn, $]), _ = tn(y) && y[$] !== void 0, b = (tn(w) && nl in w || _) && (l == null ? void 0 : l.constAsDefaults) !== "never", k = ea(e, w, {
        rootSchema: o,
        _recurseList: s,
        experimental_defaultFormStateBehavior: l,
        experimental_customMergeAllOf: u,
        includeUndefinedValues: i === !0,
        parentDefaults: Je(f, [$]),
        rawFormData: Je(p, [$]),
        required: (S = h.required) === null || S === void 0 ? void 0 : S.includes($)
      });
      return s2(C, $, k, i, c, h.required, l, b), C;
    }, {});
    if (h.additionalProperties) {
      const C = tn(h.additionalProperties) ? h.additionalProperties : {}, $ = /* @__PURE__ */ new Set();
      tn(f) && Object.keys(f).filter((w) => !h.properties || !h.properties[w]).forEach((w) => $.add(w));
      const S = [];
      Object.keys(p).filter((w) => !h.properties || !h.properties[w]).forEach((w) => {
        $.add(w), S.push(w);
      }), $.forEach((w) => {
        var _;
        const b = ea(e, C, {
          rootSchema: o,
          _recurseList: s,
          experimental_defaultFormStateBehavior: l,
          experimental_customMergeAllOf: u,
          includeUndefinedValues: i === !0,
          parentDefaults: Je(f, [w]),
          rawFormData: Je(p, [w]),
          required: (_ = h.required) === null || _ === void 0 ? void 0 : _.includes(w)
        });
        s2(v, w, b, i, c, S);
      });
    }
    return v;
  }
}
function F7(e, t, { rawFormData: n, rootSchema: o = {}, _recurseList: i = [], experimental_defaultFormStateBehavior: s = void 0, experimental_customMergeAllOf: l = void 0, required: u } = {}, c) {
  var f, p;
  const g = t, h = (f = s == null ? void 0 : s.arrayMinItems) !== null && f !== void 0 ? f : {}, { populate: y, mergeExtraDefaults: v } = h, C = y === "never", $ = y === "requiredOnly", S = y === "all" || !C && !$, w = (p = h == null ? void 0 : h.computeSkipPopulate) !== null && p !== void 0 ? p : () => !1, b = (s == null ? void 0 : s.emptyObjectFields) === "skipEmptyDefaults" ? void 0 : [];
  if (Array.isArray(c) && (c = c.map((V, W) => {
    const q = zm(g, al.Fallback, W);
    return ea(e, q, {
      rootSchema: o,
      _recurseList: i,
      experimental_defaultFormStateBehavior: s,
      experimental_customMergeAllOf: l,
      parentDefaults: V,
      required: u
    });
  })), Array.isArray(n)) {
    const V = zm(g);
    if (C)
      c = n;
    else {
      const W = n.map((Q, X) => ea(e, V, {
        rootSchema: o,
        _recurseList: i,
        experimental_defaultFormStateBehavior: s,
        experimental_customMergeAllOf: l,
        rawFormData: Q,
        parentDefaults: Je(c, [X]),
        required: u
      }));
      c = rc(c, W, ($ && u || S) && v);
    }
  }
  if ((tn(g) && nl in g && (s == null ? void 0 : s.constAsDefaults) !== "never") === !1) {
    if (C)
      return c ?? b;
    if ($ && !u)
      return c || void 0;
  }
  const E = Array.isArray(c) ? c.length : 0;
  if (!g.minItems || Hy(e, g, o, l) || w(e, g, o) || g.minItems <= E)
    return c || b;
  const j = c || [], L = zm(g, al.Invert), B = L.default, O = new Array(g.minItems - E).fill(ea(e, L, {
    parentDefaults: B,
    rootSchema: o,
    _recurseList: i,
    experimental_defaultFormStateBehavior: s,
    experimental_customMergeAllOf: l,
    required: u
  }));
  return j.concat(O);
}
function l2(e, t, n = {}, o) {
  switch (ia(t)) {
    case "object":
      return N7(e, t, n, o);
    case "array":
      return F7(e, t, n, o);
  }
}
function tx(e, t, n, o, i = !1, s, l) {
  if (!tn(t))
    throw new Error("Invalid schema: " + t);
  const u = qr(e, t, o, n, l), c = ea(e, u, {
    rootSchema: o,
    includeUndefinedValues: i,
    experimental_defaultFormStateBehavior: s,
    experimental_customMergeAllOf: l,
    rawFormData: n
  });
  if (n == null || typeof n == "number" && isNaN(n))
    return c;
  const { mergeDefaultsIntoFormData: f, arrayMinItems: p = {} } = s || {}, { mergeExtraDefaults: g } = p, h = f === "useDefaultIfFormDataUndefined";
  return tn(n) || Array.isArray(n) ? rc(c, n, g, h) : n;
}
function nx(e = {}) {
  return (
    // TODO: Remove the `&& uiSchema['ui:widget'] !== 'hidden'` once we support hidden widgets for arrays.
    // https://rjsf-team.github.io/react-jsonschema-form/docs/usage/widgets/#hidden-widgets
    "widget" in Ot(e) && Ot(e).widget !== "hidden"
  );
}
function rx(e, t, n = {}, o, i) {
  if (n[ly] === "files")
    return !0;
  if (t.items) {
    const s = qr(e, t.items, o, void 0, i);
    return s.type === "string" && s.format === "data-url";
  }
  return !1;
}
function L7(e, t, n = {}, o, i, s) {
  const l = Ot(n, i), { label: u = !0 } = l;
  let c = !!u;
  const f = ia(t);
  return f === "array" && (c = Hy(e, t, o, s) || rx(e, t, n, o, s) || nx(n)), f === "object" && (c = !1), f === "boolean" && !n[ly] && (c = !1), n[J5] && (c = !1), c;
}
function D7(e, t, n) {
  if (!n)
    return t;
  const { errors: o, errorSchema: i } = t;
  let s = e.toErrorList(n), l = n;
  return oa(i) || (l = Zi(i, n, !0), s = [...o].concat(s)), { errorSchema: l, errors: s };
}
const Is = Symbol("no Value");
function $g(e, t, n, o, i = {}, s) {
  let l;
  if (br(n, wn)) {
    const u = {};
    if (br(o, wn)) {
      const p = Je(o, wn, {});
      Object.keys(p).forEach((g) => {
        br(i, g) && (u[g] = void 0);
      });
    }
    const c = Object.keys(Je(n, wn, {})), f = {};
    c.forEach((p) => {
      const g = Je(i, p);
      let h = Je(o, [wn, p], {}), y = Je(n, [wn, p], {});
      br(h, Dn) && (h = qr(e, h, t, g, s)), br(y, Dn) && (y = qr(e, y, t, g, s));
      const v = Je(h, "type"), C = Je(y, "type");
      if (!v || v === C)
        if (br(u, p) && delete u[p], C === "object" || C === "array" && Array.isArray(g)) {
          const $ = $g(e, t, y, h, g, s);
          ($ !== void 0 || C === "array") && (f[p] = $);
        } else {
          const $ = Je(y, "default", Is), S = Je(h, "default", Is);
          $ !== Is && $ !== g && (S === g ? u[p] = $ : Je(y, "readOnly") === !0 && (u[p] = void 0));
          const w = Je(y, "const", Is), _ = Je(h, "const", Is);
          w !== Is && w !== g && (u[p] = _ === g ? w : void 0);
        }
    }), l = {
      ...typeof i == "string" || Array.isArray(i) ? void 0 : i,
      ...u,
      ...f
    };
  } else if (Je(o, "type") === "array" && Je(n, "type") === "array" && Array.isArray(i)) {
    let u = Je(o, "items"), c = Je(n, "items");
    if (typeof u == "object" && typeof c == "object" && !Array.isArray(u) && !Array.isArray(c)) {
      br(u, Dn) && (u = qr(e, u, t, i, s)), br(c, Dn) && (c = qr(e, c, t, i, s));
      const f = Je(u, "type"), p = Je(c, "type");
      if (!f || f === p) {
        const g = Je(n, "maxItems", -1);
        p === "object" ? l = i.reduce((h, y) => {
          const v = $g(e, t, c, u, y, s);
          return v !== void 0 && (g < 0 || h.length < g) && h.push(v), h;
        }, []) : l = g > 0 && i.length > g ? i.slice(0, g) : i;
      }
    } else typeof u == "boolean" && typeof c == "boolean" && u === c && (l = i);
  }
  return l;
}
function gf(e, t, n, o, i, s, l, u = [], c) {
  if (Dn in t || Zf in t || gc in t) {
    const g = qr(e, t, s, l, c);
    if (u.findIndex((y) => Ua(y, g)) === -1)
      return gf(e, g, n, o, i, s, l, u.concat(g), c);
  }
  if (Xi in t && !Je(t, [Xi, Dn]))
    return gf(e, Je(t, Xi), n, o, i, s, l, u, c);
  const p = { $id: i || n };
  if (ia(t) === "object" && wn in t)
    for (const g in t.properties) {
      const h = Je(t, [wn, g]), y = p[Yi] + o + g;
      p[g] = gf(
        e,
        tn(h) ? h : {},
        n,
        o,
        y,
        s,
        // It's possible that formData is not an object -- this can happen if an
        // array item has just been added, but not populated with data yet
        Je(l, [g]),
        u,
        c
      );
    }
  return p;
}
function z7(e, t, n, o, i, s = "root", l = "_", u) {
  return gf(e, t, s, l, n, o, i, void 0, u);
}
function Ra(e, t, n, o, i, s = [], l) {
  if (Dn in t || Zf in t || gc in t) {
    const c = qr(e, t, o, i, l);
    if (s.findIndex((p) => Ua(p, c)) === -1)
      return Ra(e, c, n, o, i, s.concat(c), l);
  }
  let u = {
    [pf]: n.replace(/^\./, "")
  };
  if (uo in t || Go in t) {
    const c = uo in t ? t.oneOf : t.anyOf, f = il(t), p = nc(e, o, i, c, 0, f, l), g = c[p];
    u = {
      ...u,
      ...Ra(e, g, n, o, i, s, l)
    };
  }
  if (cg in t && t[cg] !== !1 && Yn(u, sy, !0), Xi in t && Array.isArray(i)) {
    const { items: c, additionalItems: f } = t;
    Array.isArray(c) ? i.forEach((p, g) => {
      c[g] ? u[g] = Ra(e, c[g], `${n}.${g}`, o, p, s, l) : f ? u[g] = Ra(e, f, `${n}.${g}`, o, p, s, l) : console.warn(`Unable to generate path schema for "${n}.${g}". No schema defined for it`);
    }) : i.forEach((p, g) => {
      u[g] = Ra(e, c, `${n}.${g}`, o, p, s, l);
    });
  } else if (wn in t)
    for (const c in t.properties) {
      const f = Je(t, [wn, c]);
      u[c] = Ra(
        e,
        f,
        `${n}.${c}`,
        o,
        // It's possible that formData is not an object -- this can happen if an
        // array item has just been added, but not populated with data yet
        Je(i, [c]),
        s,
        l
      );
    }
  return u;
}
function B7(e, t, n = "", o, i, s) {
  return Ra(e, t, n, o, i, void 0, s);
}
class U7 {
  /** Constructs the `SchemaUtils` instance with the given `validator` and `rootSchema` stored as instance variables
   *
   * @param validator - An implementation of the `ValidatorType` interface that will be forwarded to all the APIs
   * @param rootSchema - The root schema that will be forwarded to all the APIs
   * @param experimental_defaultFormStateBehavior - Configuration flags to allow users to override default form state behavior
   * @param [experimental_customMergeAllOf] - Optional function that allows for custom merging of `allOf` schemas
   */
  constructor(t, n, o, i) {
    this.rootSchema = n, this.validator = t, this.experimental_defaultFormStateBehavior = o, this.experimental_customMergeAllOf = i;
  }
  /** Returns the `ValidatorType` in the `SchemaUtilsType`
   *
   * @returns - The `ValidatorType`
   */
  getValidator() {
    return this.validator;
  }
  /** Determines whether either the `validator` and `rootSchema` differ from the ones associated with this instance of
   * the `SchemaUtilsType`. If either `validator` or `rootSchema` are falsy, then return false to prevent the creation
   * of a new `SchemaUtilsType` with incomplete properties.
   *
   * @param validator - An implementation of the `ValidatorType` interface that will be compared against the current one
   * @param rootSchema - The root schema that will be compared against the current one
   * @param [experimental_defaultFormStateBehavior] Optional configuration object, if provided, allows users to override default form state behavior
   * @param [experimental_customMergeAllOf] - Optional function that allows for custom merging of `allOf` schemas
   * @returns - True if the `SchemaUtilsType` differs from the given `validator` or `rootSchema`
   */
  doesSchemaUtilsDiffer(t, n, o = {}, i) {
    return !t || !n ? !1 : this.validator !== t || !Sr(this.rootSchema, n) || !Sr(this.experimental_defaultFormStateBehavior, o) || this.experimental_customMergeAllOf !== i;
  }
  /** Returns the superset of `formData` that includes the given set updated to include any missing fields that have
   * computed to have defaults provided in the `schema`.
   *
   * @param schema - The schema for which the default state is desired
   * @param [formData] - The current formData, if any, onto which to provide any missing defaults
   * @param [includeUndefinedValues=false] - Optional flag, if true, cause undefined values to be added as defaults.
   *          If "excludeObjectChildren", pass `includeUndefinedValues` as false when computing defaults for any nested
   *          object properties.
   * @returns - The resulting `formData` with all the defaults provided
   */
  getDefaultFormState(t, n, o = !1) {
    return tx(this.validator, t, n, this.rootSchema, o, this.experimental_defaultFormStateBehavior, this.experimental_customMergeAllOf);
  }
  /** Determines whether the combination of `schema` and `uiSchema` properties indicates that the label for the `schema`
   * should be displayed in a UI.
   *
   * @param schema - The schema for which the display label flag is desired
   * @param [uiSchema] - The UI schema from which to derive potentially displayable information
   * @param [globalOptions={}] - The optional Global UI Schema from which to get any fallback `xxx` options
   * @returns - True if the label should be displayed or false if it should not
   */
  getDisplayLabel(t, n, o) {
    return L7(this.validator, t, n, this.rootSchema, o, this.experimental_customMergeAllOf);
  }
  /** Determines which of the given `options` provided most closely matches the `formData`.
   * Returns the index of the option that is valid and is the closest match, or 0 if there is no match.
   *
   * The closest match is determined using the number of matching properties, and more heavily favors options with
   * matching readOnly, default, or const values.
   *
   * @param formData - The form data associated with the schema
   * @param options - The list of options that can be selected from
   * @param [selectedOption] - The index of the currently selected option, defaulted to -1 if not specified
   * @param [discriminatorField] - The optional name of the field within the options object whose value is used to
   *          determine which option is selected
   * @returns - The index of the option that is the closest match to the `formData` or the `selectedOption` if no match
   */
  getClosestMatchingOption(t, n, o, i) {
    return nc(this.validator, this.rootSchema, t, n, o, i, this.experimental_customMergeAllOf);
  }
  /** Given the `formData` and list of `options`, attempts to find the index of the first option that matches the data.
   * Always returns the first option if there is nothing that matches.
   *
   * @param formData - The current formData, if any, used to figure out a match
   * @param options - The list of options to find a matching options from
   * @param [discriminatorField] - The optional name of the field within the options object whose value is used to
   *          determine which option is selected
   * @returns - The firstindex of the matched option or 0 if none is available
   */
  getFirstMatchingOption(t, n, o) {
    return Oy(this.validator, t, n, this.rootSchema, o);
  }
  /** Given the `formData` and list of `options`, attempts to find the index of the option that best matches the data.
   * Deprecated, use `getFirstMatchingOption()` instead.
   *
   * @param formData - The current formData, if any, onto which to provide any missing defaults
   * @param options - The list of options to find a matching options from
   * @param [discriminatorField] - The optional name of the field within the options object whose value is used to
   *          determine which option is selected
   * @returns - The index of the matched option or 0 if none is available
   * @deprecated
   */
  getMatchingOption(t, n, o) {
    return yC(this.validator, t, n, this.rootSchema, o);
  }
  /** Checks to see if the `schema` and `uiSchema` combination represents an array of files
   *
   * @param schema - The schema for which check for array of files flag is desired
   * @param [uiSchema] - The UI schema from which to check the widget
   * @returns - True if schema/uiSchema contains an array of files, otherwise false
   */
  isFilesArray(t, n) {
    return rx(this.validator, t, n, this.rootSchema, this.experimental_customMergeAllOf);
  }
  /** Checks to see if the `schema` combination represents a multi-select
   *
   * @param schema - The schema for which check for a multi-select flag is desired
   * @returns - True if schema contains a multi-select, otherwise false
   */
  isMultiSelect(t) {
    return Hy(this.validator, t, this.rootSchema, this.experimental_customMergeAllOf);
  }
  /** Checks to see if the `schema` combination represents a select
   *
   * @param schema - The schema for which check for a select flag is desired
   * @returns - True if schema contains a select, otherwise false
   */
  isSelect(t) {
    return ex(this.validator, t, this.rootSchema, this.experimental_customMergeAllOf);
  }
  /** Merges the errors in `additionalErrorSchema` into the existing `validationData` by combining the hierarchies in
   * the two `ErrorSchema`s and then appending the error list from the `additionalErrorSchema` obtained by calling
   * `getValidator().toErrorList()` onto the `errors` in the `validationData`. If no `additionalErrorSchema` is passed,
   * then `validationData` is returned.
   *
   * @param validationData - The current `ValidationData` into which to merge the additional errors
   * @param [additionalErrorSchema] - The additional set of errors
   * @returns - The `validationData` with the additional errors from `additionalErrorSchema` merged into it, if provided.
   * @deprecated - Use the `validationDataMerge()` function exported from `@rjsf/utils` instead. This function will be
   *        removed in the next major release.
   */
  mergeValidationData(t, n) {
    return D7(this.validator, t, n);
  }
  /** Retrieves an expanded schema that has had all of its conditions, additional properties, references and
   * dependencies resolved and merged into the `schema` given a `rawFormData` that is used to do the potentially
   * recursive resolution.
   *
   * @param schema - The schema for which retrieving a schema is desired
   * @param [rawFormData] - The current formData, if any, to assist retrieving a schema
   * @returns - The schema having its conditions, additional properties, references and dependencies resolved
   */
  retrieveSchema(t, n) {
    return qr(this.validator, t, this.rootSchema, n, this.experimental_customMergeAllOf);
  }
  /** Sanitize the `data` associated with the `oldSchema` so it is considered appropriate for the `newSchema`. If the
   * new schema does not contain any properties, then `undefined` is returned to clear all the form data. Due to the
   * nature of schemas, this sanitization happens recursively for nested objects of data. Also, any properties in the
   * old schemas that are non-existent in the new schema are set to `undefined`.
   *
   * @param [newSchema] - The new schema for which the data is being sanitized
   * @param [oldSchema] - The old schema from which the data originated
   * @param [data={}] - The form data associated with the schema, defaulting to an empty object when undefined
   * @returns - The new form data, with all the fields uniquely associated with the old schema set
   *      to `undefined`. Will return `undefined` if the new schema is not an object containing properties.
   */
  sanitizeDataForNewSchema(t, n, o) {
    return $g(this.validator, this.rootSchema, t, n, o, this.experimental_customMergeAllOf);
  }
  /** Generates an `IdSchema` object for the `schema`, recursively
   *
   * @param schema - The schema for which the display label flag is desired
   * @param [id] - The base id for the schema
   * @param [formData] - The current formData, if any, onto which to provide any missing defaults
   * @param [idPrefix='root'] - The prefix to use for the id
   * @param [idSeparator='_'] - The separator to use for the path segments in the id
   * @returns - The `IdSchema` object for the `schema`
   */
  toIdSchema(t, n, o, i = "root", s = "_") {
    return z7(this.validator, t, n, this.rootSchema, o, i, s, this.experimental_customMergeAllOf);
  }
  /** Generates an `PathSchema` object for the `schema`, recursively
   *
   * @param schema - The schema for which the display label flag is desired
   * @param [name] - The base name for the schema
   * @param [formData] - The current formData, if any, onto which to provide any missing defaults
   * @returns - The `PathSchema` object for the `schema`
   */
  toPathSchema(t, n, o) {
    return B7(this.validator, t, n, this.rootSchema, o, this.experimental_customMergeAllOf);
  }
}
function V7(e, t, n = {}, o) {
  return new U7(e, t, n, o);
}
function W7(e) {
  var t;
  if (e.indexOf("data:") === -1)
    throw new Error("File is invalid: URI must be a dataURI");
  const o = e.slice(5).split(";base64,");
  if (o.length !== 2)
    throw new Error("File is invalid: dataURI must be base64");
  const [i, s] = o, [l, ...u] = i.split(";"), c = l || "", f = decodeURI(
    // parse the parameters into key-value pairs, find a key, and extract a value
    // if no key is found, then the name is unknown
    ((t = u.map((p) => p.split("=")).find(([p]) => p === "name")) === null || t === void 0 ? void 0 : t[1]) || "unknown"
  );
  try {
    const p = atob(s), g = new Array(p.length);
    for (let y = 0; y < p.length; y++)
      g[y] = p.charCodeAt(y);
    return { blob: new window.Blob([new Uint8Array(g)], { type: c }), name: f };
  } catch (p) {
    throw new Error("File is invalid: " + p.message);
  }
}
function Ji(e, t) {
  let n = String(e);
  for (; n.length < t; )
    n = "0" + n;
  return n;
}
function ox(e, t) {
  if (e <= 0 && t <= 0)
    e = (/* @__PURE__ */ new Date()).getFullYear() + e, t = (/* @__PURE__ */ new Date()).getFullYear() + t;
  else if (e < 0 || t < 0)
    throw new Error(`Both start (${e}) and stop (${t}) must both be <= 0 or > 0, got one of each`);
  if (e > t)
    return ox(t, e).reverse();
  const n = [];
  for (let o = e; o <= t; o++)
    n.push({ value: o, label: Ji(o, 2) });
  return n;
}
function q7(e, t) {
  let n = e;
  if (Array.isArray(t)) {
    const o = n.split(/(%\d)/);
    t.forEach((i, s) => {
      const l = o.findIndex((u) => u === `%${s + 1}`);
      l >= 0 && (o[l] = i);
    }), n = o.join("");
  }
  return n;
}
function K7(e, t) {
  return q7(e, t);
}
function qn(e, t = [], n) {
  if (Array.isArray(e))
    return e.map((s) => qn(s, t)).filter((s) => s !== n);
  const o = e === "" || e === null ? -1 : Number(e), i = t[o];
  return i ? i.value : n;
}
function ix(e, t, n = []) {
  const o = qn(e, n);
  return Array.isArray(t) ? t.filter((i) => !Ua(i, o)) : Ua(o, t) ? void 0 : t;
}
function vp(e, t) {
  return Array.isArray(t) ? t.some((n) => Ua(n, e)) : Ua(t, e);
}
function Gy(e, t = [], n = !1) {
  const o = t.map((i, s) => vp(i.value, e) ? String(s) : void 0).filter((i) => typeof i < "u");
  return n ? o : o[0];
}
function H7(e) {
  return e == null;
}
var G7 = H7;
const ax = /* @__PURE__ */ Ft(G7);
function sx(e, t, n = []) {
  const o = qn(e, n);
  if (!ax(o)) {
    const i = n.findIndex((u) => o === u.value), s = n.map(({ value: u }) => u);
    return t.slice(0, i).concat(o, t.slice(i)).sort((u, c) => +(s.indexOf(u) > s.indexOf(c)));
  }
  return t;
}
var J7 = Iy;
function Y7(e, t, n, o) {
  return o = typeof o == "function" ? o : void 0, e == null ? e : J7(e, t, n, o);
}
var Q7 = Y7;
const X7 = /* @__PURE__ */ Ft(Q7);
class Z7 {
  /** Construct an `ErrorSchemaBuilder` with an optional initial set of errors in an `ErrorSchema`.
   *
   * @param [initialSchema] - The optional set of initial errors, that will be cloned into the class
   */
  constructor(t) {
    this.errorSchema = {}, this.resetAllErrors(t);
  }
  /** Returns the `ErrorSchema` that has been updated by the methods of the `ErrorSchemaBuilder`
   */
  get ErrorSchema() {
    return this.errorSchema;
  }
  /** Will get an existing `ErrorSchema` at the specified `pathOfError` or create and return one.
   *
   * @param [pathOfError] - The optional path into the `ErrorSchema` at which to add the error(s)
   * @returns - The error block for the given `pathOfError` or the root if not provided
   * @private
   */
  getOrCreateErrorBlock(t) {
    let o = Array.isArray(t) && t.length > 0 || typeof t == "string" ? Je(this.errorSchema, t) : this.errorSchema;
    return !o && t && (o = {}, X7(this.errorSchema, t, o, Object)), o;
  }
  /** Resets all errors in the `ErrorSchemaBuilder` back to the `initialSchema` if provided, otherwise an empty set.
   *
   * @param [initialSchema] - The optional set of initial errors, that will be cloned into the class
   * @returns - The `ErrorSchemaBuilder` object for chaining purposes
   */
  resetAllErrors(t) {
    return this.errorSchema = t ? xC(t) : {}, this;
  }
  /** Adds the `errorOrList` to the list of errors in the `ErrorSchema` at either the root level or the location within
   * the schema described by the `pathOfError`. For more information about how to specify the path see the
   * [eslint lodash plugin docs](https://github.com/wix/eslint-plugin-lodash/blob/master/docs/rules/path-style.md).
   *
   * @param errorOrList - The error or list of errors to add into the `ErrorSchema`
   * @param [pathOfError] - The optional path into the `ErrorSchema` at which to add the error(s)
   * @returns - The `ErrorSchemaBuilder` object for chaining purposes
   */
  addErrors(t, n) {
    const o = this.getOrCreateErrorBlock(n);
    let i = Je(o, Vr);
    return Array.isArray(i) || (i = [], o[Vr] = i), Array.isArray(t) ? Yn(o, Vr, [.../* @__PURE__ */ new Set([...i, ...t])]) : Yn(o, Vr, [.../* @__PURE__ */ new Set([...i, t])]), this;
  }
  /** Sets/replaces the `errorOrList` as the error(s) in the `ErrorSchema` at either the root level or the location
   * within the schema described by the `pathOfError`. For more information about how to specify the path see the
   * [eslint lodash plugin docs](https://github.com/wix/eslint-plugin-lodash/blob/master/docs/rules/path-style.md).
   *
   * @param errorOrList - The error or list of errors to set into the `ErrorSchema`
   * @param [pathOfError] - The optional path into the `ErrorSchema` at which to set the error(s)
   * @returns - The `ErrorSchemaBuilder` object for chaining purposes
   */
  setErrors(t, n) {
    const o = this.getOrCreateErrorBlock(n), i = Array.isArray(t) ? [.../* @__PURE__ */ new Set([...t])] : [t];
    return Yn(o, Vr, i), this;
  }
  /** Clears the error(s) in the `ErrorSchema` at either the root level or the location within the schema described by
   * the `pathOfError`. For more information about how to specify the path see the
   * [eslint lodash plugin docs](https://github.com/wix/eslint-plugin-lodash/blob/master/docs/rules/path-style.md).
   *
   * @param [pathOfError] - The optional path into the `ErrorSchema` at which to clear the error(s)
   * @returns - The `ErrorSchemaBuilder` object for chaining purposes
   */
  clearErrors(t) {
    const n = this.getOrCreateErrorBlock(t);
    return Yn(n, Vr, []), this;
  }
}
function eU(e, t, n = [1900, (/* @__PURE__ */ new Date()).getFullYear() + 2], o = "YMD") {
  const { day: i, month: s, year: l, hour: u, minute: c, second: f } = e, p = { type: "day", range: [1, 31], value: i }, g = { type: "month", range: [1, 12], value: s }, h = { type: "year", range: n, value: l }, y = [];
  switch (o) {
    case "MDY":
      y.push(g, p, h);
      break;
    case "DMY":
      y.push(p, g, h);
      break;
    case "YMD":
    default:
      y.push(h, g, p);
  }
  return t && y.push({ type: "hour", range: [0, 23], value: u }, { type: "minute", range: [0, 59], value: c }, { type: "second", range: [0, 59], value: f }), y;
}
function lx(e) {
  const t = {};
  return e.multipleOf && (t.step = e.multipleOf), (e.minimum || e.minimum === 0) && (t.min = e.minimum), (e.maximum || e.maximum === 0) && (t.max = e.maximum), t;
}
function ux(e, t, n = {}, o = !0) {
  const i = {
    type: t || "text",
    ...lx(e)
  };
  return n.inputType ? i.type = n.inputType : t || (e.type === "number" ? (i.type = "number", o && i.step === void 0 && (i.step = "any")) : e.type === "integer" && (i.type = "number", i.step === void 0 && (i.step = 1))), n.autocomplete && (i.autoComplete = n.autocomplete), i;
}
const u2 = {
  props: {
    disabled: !1
  },
  submitText: "Submit",
  norender: !1
};
function cx(e = {}) {
  const t = Ot(e);
  if (t && t[Pf]) {
    const n = t[Pf];
    return { ...u2, ...n };
  }
  return u2;
}
function _t(e, t, n = {}) {
  const { templates: o } = t;
  return e === "ButtonTemplates" ? o[e] : (
    // Evaluating uiOptions[name] results in TS2590: Expression produces a union type that is too complex to represent
    // To avoid that, we cast uiOptions to `any` before accessing the name field
    n[e] || o[e]
  );
}
var dx = { exports: {} }, Vt = {};
/**
 * @license React
 * react-is.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var c2;
function tU() {
  if (c2) return Vt;
  c2 = 1;
  var e = Symbol.for("react.element"), t = Symbol.for("react.portal"), n = Symbol.for("react.fragment"), o = Symbol.for("react.strict_mode"), i = Symbol.for("react.profiler"), s = Symbol.for("react.provider"), l = Symbol.for("react.context"), u = Symbol.for("react.server_context"), c = Symbol.for("react.forward_ref"), f = Symbol.for("react.suspense"), p = Symbol.for("react.suspense_list"), g = Symbol.for("react.memo"), h = Symbol.for("react.lazy"), y = Symbol.for("react.offscreen"), v;
  v = Symbol.for("react.module.reference");
  function C($) {
    if (typeof $ == "object" && $ !== null) {
      var S = $.$$typeof;
      switch (S) {
        case e:
          switch ($ = $.type, $) {
            case n:
            case i:
            case o:
            case f:
            case p:
              return $;
            default:
              switch ($ = $ && $.$$typeof, $) {
                case u:
                case l:
                case c:
                case h:
                case g:
                case s:
                  return $;
                default:
                  return S;
              }
          }
        case t:
          return S;
      }
    }
  }
  return Vt.ContextConsumer = l, Vt.ContextProvider = s, Vt.Element = e, Vt.ForwardRef = c, Vt.Fragment = n, Vt.Lazy = h, Vt.Memo = g, Vt.Portal = t, Vt.Profiler = i, Vt.StrictMode = o, Vt.Suspense = f, Vt.SuspenseList = p, Vt.isAsyncMode = function() {
    return !1;
  }, Vt.isConcurrentMode = function() {
    return !1;
  }, Vt.isContextConsumer = function($) {
    return C($) === l;
  }, Vt.isContextProvider = function($) {
    return C($) === s;
  }, Vt.isElement = function($) {
    return typeof $ == "object" && $ !== null && $.$$typeof === e;
  }, Vt.isForwardRef = function($) {
    return C($) === c;
  }, Vt.isFragment = function($) {
    return C($) === n;
  }, Vt.isLazy = function($) {
    return C($) === h;
  }, Vt.isMemo = function($) {
    return C($) === g;
  }, Vt.isPortal = function($) {
    return C($) === t;
  }, Vt.isProfiler = function($) {
    return C($) === i;
  }, Vt.isStrictMode = function($) {
    return C($) === o;
  }, Vt.isSuspense = function($) {
    return C($) === f;
  }, Vt.isSuspenseList = function($) {
    return C($) === p;
  }, Vt.isValidElementType = function($) {
    return typeof $ == "string" || typeof $ == "function" || $ === n || $ === i || $ === o || $ === f || $ === p || $ === y || typeof $ == "object" && $ !== null && ($.$$typeof === h || $.$$typeof === g || $.$$typeof === s || $.$$typeof === l || $.$$typeof === c || $.$$typeof === v || $.getModuleId !== void 0);
  }, Vt.typeOf = C, Vt;
}
dx.exports = tU();
var nU = dx.exports;
const d2 = /* @__PURE__ */ Ft(nU), Bm = {
  boolean: {
    checkbox: "CheckboxWidget",
    radio: "RadioWidget",
    select: "SelectWidget",
    hidden: "HiddenWidget"
  },
  string: {
    text: "TextWidget",
    password: "PasswordWidget",
    email: "EmailWidget",
    hostname: "TextWidget",
    ipv4: "TextWidget",
    ipv6: "TextWidget",
    uri: "URLWidget",
    "data-url": "FileWidget",
    radio: "RadioWidget",
    select: "SelectWidget",
    textarea: "TextareaWidget",
    hidden: "HiddenWidget",
    date: "DateWidget",
    datetime: "DateTimeWidget",
    "date-time": "DateTimeWidget",
    "alt-date": "AltDateWidget",
    "alt-datetime": "AltDateTimeWidget",
    time: "TimeWidget",
    color: "ColorWidget",
    file: "FileWidget"
  },
  number: {
    text: "TextWidget",
    select: "SelectWidget",
    updown: "UpDownWidget",
    range: "RangeWidget",
    radio: "RadioWidget",
    hidden: "HiddenWidget"
  },
  integer: {
    text: "TextWidget",
    select: "SelectWidget",
    updown: "UpDownWidget",
    range: "RangeWidget",
    radio: "RadioWidget",
    hidden: "HiddenWidget"
  },
  array: {
    select: "SelectWidget",
    checkboxes: "CheckboxesWidget",
    files: "FileWidget",
    hidden: "HiddenWidget"
  }
};
function rU(e) {
  let t = Je(e, "MergedWidget");
  if (!t) {
    const n = e.defaultProps && e.defaultProps.options || {};
    t = ({ options: o, ...i }) => A.jsx(e, { options: { ...n, ...o }, ...i }), Yn(e, "MergedWidget", t);
  }
  return t;
}
function gi(e, t, n = {}) {
  const o = ia(e);
  if (typeof t == "function" || t && d2.isForwardRef(z.createElement(t)) || d2.isMemo(t))
    return rU(t);
  if (typeof t != "string")
    throw new Error(`Unsupported widget definition: ${typeof t}`);
  if (t in n) {
    const i = n[t];
    return gi(e, i, n);
  }
  if (typeof o == "string") {
    if (!(o in Bm))
      throw new Error(`No widget for type '${o}'`);
    if (t in Bm[o]) {
      const i = n[Bm[o][t]];
      return gi(e, i, n);
    }
  }
  throw new Error(`No widget '${t}' for type '${o}'`);
}
function oU(e) {
  let t = 0;
  for (let n = 0; n < e.length; n += 1) {
    const o = e.charCodeAt(n);
    t = (t << 5) - t + o, t = t & t;
  }
  return t.toString(16);
}
function iU(e) {
  const t = /* @__PURE__ */ new Set();
  return JSON.stringify(e, (n, o) => (t.add(n), o)), oU(JSON.stringify(e, Array.from(t).sort()));
}
function aU(e, t, n = {}) {
  try {
    return gi(e, t, n), !0;
  } catch (o) {
    const i = o;
    if (i.message && (i.message.startsWith("No widget") || i.message.startsWith("Unsupported widget")))
      return !1;
    throw o;
  }
}
function Ec(e, t) {
  return `${_y(e) ? e : e[Yi]}__${t}`;
}
function es(e) {
  return Ec(e, "description");
}
function Jy(e) {
  return Ec(e, "error");
}
function oc(e) {
  return Ec(e, "examples");
}
function Yy(e) {
  return Ec(e, "help");
}
function Qy(e) {
  return Ec(e, "title");
}
function Gr(e, t = !1) {
  const n = t ? ` ${oc(e)}` : "";
  return `${Jy(e)} ${es(e)} ${Yy(e)}${n}`;
}
function bp(e, t) {
  return `${e}-${t}`;
}
function ts(e, t, n) {
  return t ? n : e;
}
function sU(e) {
  return e ? new Date(e).toJSON() : void 0;
}
function lU(e) {
  if (q5 in e && Array.isArray(e.enum) && e.enum.length === 1)
    return e.enum[0];
  if (nl in e)
    return e.const;
  throw new Error("schema cannot be inferred as a constant");
}
function Ff(e, t) {
  const n = e;
  if (e.enum) {
    let s;
    if (t) {
      const { enumNames: l } = Ot(t);
      s = l;
    }
    return !s && n.enumNames && (s = n.enumNames), e.enum.map((l, u) => ({ label: (s == null ? void 0 : s[u]) || String(l), value: l }));
  }
  let o, i;
  return e.anyOf ? (o = e.anyOf, i = t == null ? void 0 : t.anyOf) : e.oneOf && (o = e.oneOf, i = t == null ? void 0 : t.oneOf), o && o.map((s, l) => {
    const { title: u } = Ot(i == null ? void 0 : i[l]), c = s, f = lU(c), p = u || c.title || String(f);
    return {
      schema: c,
      label: p,
      value: f
    };
  });
}
function uU(e, t) {
  if (!Array.isArray(t))
    return e;
  const n = (p) => p.reduce((g, h) => (g[h] = !0, g), {}), o = (p) => p.length > 1 ? `properties '${p.join("', '")}'` : `property '${p[0]}'`, i = n(e), s = t.filter((p) => p === "*" || i[p]), l = n(s), u = e.filter((p) => !l[p]), c = s.indexOf("*");
  if (c === -1) {
    if (u.length)
      throw new Error(`uiSchema order list does not contain ${o(u)}`);
    return s;
  }
  if (c !== s.lastIndexOf("*"))
    throw new Error("uiSchema order list contains more than one wildcard item");
  const f = [...s];
  return f.splice(c, 1, ...u), f;
}
function Um(e, t = !0) {
  if (!e)
    return {
      year: -1,
      month: -1,
      day: -1,
      hour: t ? -1 : 0,
      minute: t ? -1 : 0,
      second: t ? -1 : 0
    };
  const n = new Date(e);
  if (Number.isNaN(n.getTime()))
    throw new Error("Unable to parse date " + e);
  return {
    year: n.getUTCFullYear(),
    month: n.getUTCMonth() + 1,
    day: n.getUTCDate(),
    hour: t ? n.getUTCHours() : 0,
    minute: t ? n.getUTCMinutes() : 0,
    second: t ? n.getUTCSeconds() : 0
  };
}
function Uu(e) {
  if (e.const || e.enum && e.enum.length === 1 && e.enum[0] === !0)
    return !0;
  if (e.anyOf && e.anyOf.length === 1)
    return Uu(e.anyOf[0]);
  if (e.oneOf && e.oneOf.length === 1)
    return Uu(e.oneOf[0]);
  if (e.allOf) {
    const t = (n) => Uu(n);
    return e.allOf.some(t);
  }
  return !1;
}
function cU(e, t, n) {
  const { props: o, state: i } = e;
  return !Sr(o, t) || !Sr(i, n);
}
function f2(e, t = !0) {
  const { year: n, month: o, day: i, hour: s = 0, minute: l = 0, second: u = 0 } = e, c = Date.UTC(n, o - 1, i, s, l, u), f = new Date(c).toJSON();
  return t ? f : f.slice(0, 10);
}
function ic(e, t = []) {
  if (!e)
    return [];
  let n = [];
  return Vr in e && (n = n.concat(e[Vr].map((o) => {
    const i = `.${t.join(".")}`;
    return {
      property: i,
      message: o,
      stack: `${i} ${o}`
    };
  }))), Object.keys(e).reduce((o, i) => {
    if (i !== Vr) {
      const s = e[i];
      uy(s) && (o = o.concat(ic(s, [...t, i])));
    }
    return o;
  }, n);
}
var dU = Pi, fU = pp, pU = Kn, hU = _l, mU = T$, gU = Za, yU = N$;
function vU(e) {
  return pU(e) ? dU(e, gU) : hU(e) ? [e] : fU(mU(yU(e)));
}
var bU = vU;
const fx = /* @__PURE__ */ Ft(bU);
function SU(e) {
  const t = new Z7();
  return e.length && e.forEach((n) => {
    const { property: o, message: i } = n, s = o === "." ? [] : fx(o);
    s.length > 0 && s[0] === "" && s.splice(0, 1), i && t.addErrors(i, s);
  }), t.ErrorSchema;
}
function px(e) {
  return Object.keys(e).reduce((t, n) => {
    if (n === "addError")
      return t;
    {
      const o = e[n];
      return uy(o) ? {
        ...t,
        [n]: px(o)
      } : { ...t, [n]: o };
    }
  }, {});
}
function wU(e) {
  if (!e)
    return "";
  const t = new Date(e), n = Ji(t.getFullYear(), 4), o = Ji(t.getMonth() + 1, 2), i = Ji(t.getDate(), 2), s = Ji(t.getHours(), 2), l = Ji(t.getMinutes(), 2), u = Ji(t.getSeconds(), 2), c = Ji(t.getMilliseconds(), 3);
  return `${n}-${o}-${i}T${s}:${l}:${u}.${c}`;
}
function yf(e, t) {
  if (!t)
    return e;
  const { errors: n, errorSchema: o } = e;
  let i = ic(t), s = t;
  return oa(o) || (s = Zi(o, t, !0), i = [...n].concat(i)), { errorSchema: s, errors: i };
}
function $U(e) {
  for (const t in e) {
    const n = e, o = n[t];
    t === Dn && typeof o == "string" && o.startsWith("#") ? n[t] = h$ + o : n[t] = Xy(o);
  }
  return e;
}
function CU(e) {
  for (let t = 0; t < e.length; t++)
    e[t] = Xy(e[t]);
  return e;
}
function Xy(e) {
  return Array.isArray(e) ? CU([...e]) : co(e) ? $U({ ...e }) : e;
}
var jt;
(function(e) {
  e.ArrayItemTitle = "Item", e.MissingItems = "Missing items definition", e.YesLabel = "Yes", e.NoLabel = "No", e.CloseLabel = "Close", e.ErrorsLabel = "Errors", e.NewStringDefault = "New Value", e.AddButton = "Add", e.AddItemButton = "Add Item", e.CopyButton = "Copy", e.MoveDownButton = "Move down", e.MoveUpButton = "Move up", e.RemoveButton = "Remove", e.NowLabel = "Now", e.ClearLabel = "Clear", e.AriaDateLabel = "Select a date", e.PreviewLabel = "Preview", e.DecrementAriaLabel = "Decrease value by 1", e.IncrementAriaLabel = "Increase value by 1", e.UnknownFieldType = "Unknown field type %1", e.OptionPrefix = "Option %1", e.TitleOptionPrefix = "%1 option %2", e.KeyLabel = "%1 Key", e.InvalidObjectField = 'Invalid "%1" object field configuration: _%2_.', e.UnsupportedField = "Unsupported field schema.", e.UnsupportedFieldWithId = "Unsupported field schema for field `%1`.", e.UnsupportedFieldWithReason = "Unsupported field schema: _%1_.", e.UnsupportedFieldWithIdAndReason = "Unsupported field schema for field `%1`: _%2_.", e.FilesInfo = "**%1** (%2, %3 bytes)";
})(jt || (jt = {}));
var xU = wc, EU = Iy, _U = kl;
function kU(e, t, n) {
  for (var o = -1, i = t.length, s = {}; ++o < i; ) {
    var l = t[o], u = xU(e, l);
    n(u, l) && EU(s, _U(l, e), u);
  }
  return s;
}
var PU = kU, OU = PU, AU = pC;
function IU(e, t) {
  return OU(e, t, function(n, o) {
    return AU(e, o);
  });
}
var jU = IU, RU = jU, TU = oC, MU = TU(function(e, t) {
  return e == null ? {} : RU(e, t);
}), NU = MU;
const p2 = /* @__PURE__ */ Ft(NU);
let FU = (e = 21) => crypto.getRandomValues(new Uint8Array(e)).reduce((t, n) => (n &= 63, n < 36 ? t += n.toString(36) : n < 62 ? t += (n - 26).toString(36).toUpperCase() : n > 62 ? t += "-" : t += "_", t), "");
function Cg() {
  return FU();
}
function h2(e) {
  return Array.isArray(e) ? e.map((t) => ({
    key: Cg(),
    item: t
  })) : [];
}
function wu(e) {
  return Array.isArray(e) ? e.map((t) => t.item) : [];
}
class LU extends z.Component {
  /** Constructs an `ArrayField` from the `props`, generating the initial keyed data from the `formData`
   *
   * @param props - The `FieldProps` for this template
   */
  constructor(t) {
    super(t), this._getNewFormDataRow = () => {
      const { schema: i, registry: s } = this.props, { schemaUtils: l } = s;
      let u = i.items;
      return wg(i) && V5(i) && (u = i.additionalItems), l.getDefaultFormState(u);
    }, this.onAddClick = (i) => {
      this._handleAddClick(i);
    }, this.onAddIndexClick = (i) => (s) => {
      this._handleAddClick(s, i);
    }, this.onCopyIndexClick = (i) => (s) => {
      s && s.preventDefault();
      const { onChange: l, errorSchema: u } = this.props, { keyedFormData: c } = this.state;
      let f;
      if (u) {
        f = {};
        for (const h in u) {
          const y = parseInt(h);
          y <= i ? Yn(f, [y], u[h]) : y > i && Yn(f, [y + 1], u[h]);
        }
      }
      const p = {
        key: Cg(),
        item: xC(c[i].item)
      }, g = [...c];
      i !== void 0 ? g.splice(i + 1, 0, p) : g.push(p), this.setState({
        keyedFormData: g,
        updatedKeyedFormData: !0
      }, () => l(wu(g), f));
    }, this.onDropIndexClick = (i) => (s) => {
      s && s.preventDefault();
      const { onChange: l, errorSchema: u } = this.props, { keyedFormData: c } = this.state;
      let f;
      if (u) {
        f = {};
        for (const g in u) {
          const h = parseInt(g);
          h < i ? Yn(f, [h], u[g]) : h > i && Yn(f, [h - 1], u[g]);
        }
      }
      const p = c.filter((g, h) => h !== i);
      this.setState({
        keyedFormData: p,
        updatedKeyedFormData: !0
      }, () => l(wu(p), f));
    }, this.onReorderClick = (i, s) => (l) => {
      l && (l.preventDefault(), l.currentTarget.blur());
      const { onChange: u, errorSchema: c } = this.props;
      let f;
      if (c) {
        f = {};
        for (const y in c) {
          const v = parseInt(y);
          v == i ? Yn(f, [s], c[i]) : v == s ? Yn(f, [i], c[s]) : Yn(f, [y], c[v]);
        }
      }
      const { keyedFormData: p } = this.state;
      function g() {
        const y = p.slice();
        return y.splice(i, 1), y.splice(s, 0, p[i]), y;
      }
      const h = g();
      this.setState({
        keyedFormData: h
      }, () => u(wu(h), f));
    }, this.onChangeForIndex = (i) => (s, l, u) => {
      const { formData: c, onChange: f, errorSchema: p } = this.props, h = (Array.isArray(c) ? c : []).map((y, v) => i === v ? typeof s > "u" ? null : s : y);
      f(h, p && p && {
        ...p,
        [i]: l
      }, u);
    }, this.onSelectChange = (i) => {
      const { onChange: s, idSchema: l } = this.props;
      s(i, void 0, l && l.$id);
    };
    const { formData: n = [] } = t, o = h2(n);
    this.state = {
      keyedFormData: o,
      updatedKeyedFormData: !1
    };
  }
  /** React lifecycle method that is called when the props are about to change allowing the state to be updated. It
   * regenerates the keyed form data and returns it
   *
   * @param nextProps - The next set of props data
   * @param prevState - The previous set of state data
   */
  static getDerivedStateFromProps(t, n) {
    if (n.updatedKeyedFormData)
      return {
        updatedKeyedFormData: !1
      };
    const o = Array.isArray(t.formData) ? t.formData : [], i = n.keyedFormData || [];
    return {
      keyedFormData: o.length === i.length ? i.map((l, u) => ({
        key: l.key,
        item: o[u]
      })) : h2(o)
    };
  }
  /** Returns the appropriate title for an item by getting first the title from the schema.items, then falling back to
   * the description from the schema.items, and finally the string "Item"
   */
  get itemTitle() {
    const { schema: t, registry: n } = this.props, { translateString: o } = n;
    return Je(t, [Xi, "title"], Je(t, [Xi, "description"], o(jt.ArrayItemTitle)));
  }
  /** Determines whether the item described in the schema is always required, which is determined by whether any item
   * may be null.
   *
   * @param itemSchema - The schema for the item
   * @return - True if the item schema type does not contain the "null" type
   */
  isItemRequired(t) {
    return Array.isArray(t.type) ? !t.type.includes("null") : t.type !== "null";
  }
  /** Determines whether more items can be added to the array. If the uiSchema indicates the array doesn't allow adding
   * then false is returned. Otherwise, if the schema indicates that there are a maximum number of items and the
   * `formData` matches that value, then false is returned, otherwise true is returned.
   *
   * @param formItems - The list of items in the form
   * @returns - True if the item is addable otherwise false
   */
  canAddItem(t) {
    const { schema: n, uiSchema: o, registry: i } = this.props;
    let { addable: s } = Ot(o, i.globalUiOptions);
    return s !== !1 && (n.maxItems !== void 0 ? s = t.length < n.maxItems : s = !0), s;
  }
  /** Callback handler for when the user clicks on the add or add at index buttons. Creates a new row of keyed form data
   * either at the end of the list (when index is not specified) or inserted at the `index` when it is, adding it into
   * the state, and then returning `onChange()` with the plain form data converted from the keyed data
   *
   * @param event - The event for the click
   * @param [index] - The optional index at which to add the new data
   */
  _handleAddClick(t, n) {
    t && t.preventDefault();
    const { onChange: o, errorSchema: i } = this.props, { keyedFormData: s } = this.state;
    let l;
    if (i) {
      l = {};
      for (const f in i) {
        const p = parseInt(f);
        n === void 0 || p < n ? Yn(l, [p], i[f]) : p >= n && Yn(l, [p + 1], i[f]);
      }
    }
    const u = {
      key: Cg(),
      item: this._getNewFormDataRow()
    }, c = [...s];
    n !== void 0 ? c.splice(n, 0, u) : c.push(u), this.setState({
      keyedFormData: c,
      updatedKeyedFormData: !0
    }, () => o(wu(c), l));
  }
  /** Renders the `ArrayField` depending on the specific needs of the schema and uischema elements
   */
  render() {
    const { schema: t, uiSchema: n, idSchema: o, registry: i } = this.props, { schemaUtils: s, translateString: l } = i;
    if (!(Xi in t)) {
      const u = Ot(n), c = _t("UnsupportedFieldTemplate", i, u);
      return A.jsx(c, { schema: t, idSchema: o, reason: l(jt.MissingItems), registry: i });
    }
    return s.isMultiSelect(t) ? this.renderMultiSelect() : nx(n) ? this.renderCustomWidget() : wg(t) ? this.renderFixedArray() : s.isFilesArray(t, n) ? this.renderFiles() : this.renderNormalArray();
  }
  /** Renders a normal array without any limitations of length
   */
  renderNormalArray() {
    const { schema: t, uiSchema: n = {}, errorSchema: o, idSchema: i, name: s, title: l, disabled: u = !1, readonly: c = !1, autofocus: f = !1, required: p = !1, registry: g, onBlur: h, onFocus: y, idPrefix: v, idSeparator: C = "_", rawErrors: $ } = this.props, { keyedFormData: S } = this.state, w = t.title || l || s, { schemaUtils: _, formContext: b } = g, k = Ot(n), E = co(t.items) ? t.items : {}, j = _.retrieveSchema(E), L = wu(this.state.keyedFormData), B = this.canAddItem(L), O = {
      canAdd: B,
      items: S.map((W, q) => {
        const { key: Q, item: X } = W, ce = X, J = _.retrieveSchema(E, ce), ie = o ? o[q] : void 0, re = i.$id + C + q, Y = _.toIdSchema(J, re, ce, v, C);
        return this.renderArrayFieldItem({
          key: Q,
          index: q,
          name: s && `${s}-${q}`,
          title: w ? `${w}-${q + 1}` : void 0,
          canAdd: B,
          canMoveUp: q > 0,
          canMoveDown: q < L.length - 1,
          itemSchema: J,
          itemIdSchema: Y,
          itemErrorSchema: ie,
          itemData: ce,
          itemUiSchema: n.items,
          autofocus: f && q === 0,
          onBlur: h,
          onFocus: y,
          rawErrors: $,
          totalItems: S.length
        });
      }),
      className: `field field-array field-array-of-${j.type}`,
      disabled: u,
      idSchema: i,
      uiSchema: n,
      onAddClick: this.onAddClick,
      readonly: c,
      required: p,
      schema: t,
      title: w,
      formContext: b,
      formData: L,
      rawErrors: $,
      registry: g
    }, V = _t("ArrayFieldTemplate", g, k);
    return A.jsx(V, { ...O });
  }
  /** Renders an array using the custom widget provided by the user in the `uiSchema`
   */
  renderCustomWidget() {
    var t;
    const { schema: n, idSchema: o, uiSchema: i, disabled: s = !1, readonly: l = !1, autofocus: u = !1, required: c = !1, hideError: f, placeholder: p, onBlur: g, onFocus: h, formData: y = [], registry: v, rawErrors: C, name: $ } = this.props, { widgets: S, formContext: w, globalUiOptions: _, schemaUtils: b } = v, { widget: k, title: E, ...j } = Ot(i, _), L = gi(n, k, S), B = (t = E ?? n.title) !== null && t !== void 0 ? t : $, O = b.getDisplayLabel(n, i, _);
    return A.jsx(L, { id: o.$id, name: $, multiple: !0, onChange: this.onSelectChange, onBlur: g, onFocus: h, options: j, schema: n, uiSchema: i, registry: v, value: y, disabled: s, readonly: l, hideError: f, required: c, label: B, hideLabel: !O, placeholder: p, formContext: w, autofocus: u, rawErrors: C });
  }
  /** Renders an array as a set of checkboxes
   */
  renderMultiSelect() {
    var t;
    const { schema: n, idSchema: o, uiSchema: i, formData: s = [], disabled: l = !1, readonly: u = !1, autofocus: c = !1, required: f = !1, placeholder: p, onBlur: g, onFocus: h, registry: y, rawErrors: v, name: C } = this.props, { widgets: $, schemaUtils: S, formContext: w, globalUiOptions: _ } = y, b = S.retrieveSchema(n.items, s), k = Ff(b, i), { widget: E = "select", title: j, ...L } = Ot(i, _), B = gi(n, E, $), O = (t = j ?? n.title) !== null && t !== void 0 ? t : C, V = S.getDisplayLabel(n, i, _);
    return A.jsx(B, { id: o.$id, name: C, multiple: !0, onChange: this.onSelectChange, onBlur: g, onFocus: h, options: { ...L, enumOptions: k }, schema: n, uiSchema: i, registry: y, value: s, disabled: l, readonly: u, required: f, label: O, hideLabel: !V, placeholder: p, formContext: w, autofocus: c, rawErrors: v });
  }
  /** Renders an array of files using the `FileWidget`
   */
  renderFiles() {
    var t;
    const { schema: n, uiSchema: o, idSchema: i, name: s, disabled: l = !1, readonly: u = !1, autofocus: c = !1, required: f = !1, onBlur: p, onFocus: g, registry: h, formData: y = [], rawErrors: v } = this.props, { widgets: C, formContext: $, globalUiOptions: S, schemaUtils: w } = h, { widget: _ = "files", title: b, ...k } = Ot(o, S), E = gi(n, _, C), j = (t = b ?? n.title) !== null && t !== void 0 ? t : s, L = w.getDisplayLabel(n, o, S);
    return A.jsx(E, { options: k, id: i.$id, name: s, multiple: !0, onChange: this.onSelectChange, onBlur: p, onFocus: g, schema: n, uiSchema: o, value: y, disabled: l, readonly: u, required: f, registry: h, formContext: $, autofocus: c, rawErrors: v, label: j, hideLabel: !L });
  }
  /** Renders an array that has a maximum limit of items
   */
  renderFixedArray() {
    const { schema: t, uiSchema: n = {}, formData: o = [], errorSchema: i, idPrefix: s, idSeparator: l = "_", idSchema: u, name: c, title: f, disabled: p = !1, readonly: g = !1, autofocus: h = !1, required: y = !1, registry: v, onBlur: C, onFocus: $, rawErrors: S } = this.props, { keyedFormData: w } = this.state;
    let { formData: _ = [] } = this.props;
    const b = t.title || f || c, k = Ot(n), { schemaUtils: E, formContext: j } = v, B = (co(t.items) ? t.items : []).map((Q, X) => E.retrieveSchema(Q, o[X])), O = co(t.additionalItems) ? E.retrieveSchema(t.additionalItems, o) : null;
    (!_ || _.length < B.length) && (_ = _ || [], _ = _.concat(new Array(B.length - _.length)));
    const V = this.canAddItem(_) && !!O, W = {
      canAdd: V,
      className: "field field-array field-array-fixed-items",
      disabled: p,
      idSchema: u,
      formData: o,
      items: w.map((Q, X) => {
        const { key: ce, item: J } = Q, ie = J, re = X >= B.length, Y = (re && co(t.additionalItems) ? E.retrieveSchema(t.additionalItems, ie) : B[X]) || {}, T = u.$id + l + X, H = E.toIdSchema(Y, T, ie, s, l), G = re ? n.additionalItems || {} : Array.isArray(n.items) ? n.items[X] : n.items || {}, N = i ? i[X] : void 0;
        return this.renderArrayFieldItem({
          key: ce,
          index: X,
          name: c && `${c}-${X}`,
          title: b ? `${b}-${X + 1}` : void 0,
          canAdd: V,
          canRemove: re,
          canMoveUp: X >= B.length + 1,
          canMoveDown: re && X < _.length - 1,
          itemSchema: Y,
          itemData: ie,
          itemUiSchema: G,
          itemIdSchema: H,
          itemErrorSchema: N,
          autofocus: h && X === 0,
          onBlur: C,
          onFocus: $,
          rawErrors: S,
          totalItems: w.length
        });
      }),
      onAddClick: this.onAddClick,
      readonly: g,
      required: y,
      registry: v,
      schema: t,
      uiSchema: n,
      title: b,
      formContext: j,
      errorSchema: i,
      rawErrors: S
    }, q = _t("ArrayFieldTemplate", v, k);
    return A.jsx(q, { ...W });
  }
  /** Renders the individual array item using a `SchemaField` along with the additional properties required to be send
   * back to the `ArrayFieldItemTemplate`.
   *
   * @param props - The props for the individual array item to be rendered
   */
  renderArrayFieldItem(t) {
    const { key: n, index: o, name: i, canAdd: s, canRemove: l = !0, canMoveUp: u, canMoveDown: c, itemSchema: f, itemData: p, itemUiSchema: g, itemIdSchema: h, itemErrorSchema: y, autofocus: v, onBlur: C, onFocus: $, rawErrors: S, totalItems: w, title: _ } = t, { disabled: b, hideError: k, idPrefix: E, idSeparator: j, readonly: L, uiSchema: B, registry: O, formContext: V } = this.props, { fields: { ArraySchemaField: W, SchemaField: q }, globalUiOptions: Q } = O, X = W || q, { orderable: ce = !0, removable: J = !0, copyable: ie = !1 } = Ot(B, Q), re = {
      moveUp: ce && u,
      moveDown: ce && c,
      copy: ie && s,
      remove: J && l,
      toolbar: !1
    };
    return re.toolbar = Object.keys(re).some((Y) => re[Y]), {
      children: A.jsx(X, { name: i, title: _, index: o, schema: f, uiSchema: g, formData: p, formContext: V, errorSchema: y, idPrefix: E, idSeparator: j, idSchema: h, required: this.isItemRequired(f), onChange: this.onChangeForIndex(o), onBlur: C, onFocus: $, registry: O, disabled: b, readonly: L, hideError: k, autofocus: v, rawErrors: S }),
      className: "array-item",
      disabled: b,
      canAdd: s,
      hasCopy: re.copy,
      hasToolbar: re.toolbar,
      hasMoveUp: re.moveUp,
      hasMoveDown: re.moveDown,
      hasRemove: re.remove,
      index: o,
      totalItems: w,
      key: n,
      onAddIndexClick: this.onAddIndexClick,
      onCopyIndexClick: this.onCopyIndexClick,
      onDropIndexClick: this.onDropIndexClick,
      onReorderClick: this.onReorderClick,
      readonly: L,
      registry: O,
      schema: f,
      uiSchema: g
    };
  }
}
function DU(e) {
  var t, n, o;
  const { schema: i, name: s, uiSchema: l, idSchema: u, formData: c, registry: f, required: p, disabled: g, readonly: h, hideError: y, autofocus: v, title: C, onChange: $, onFocus: S, onBlur: w, rawErrors: _ } = e, { title: b } = i, { widgets: k, formContext: E, translateString: j, globalUiOptions: L } = f, {
    widget: B = "checkbox",
    title: O,
    // Unlike the other fields, don't use `getDisplayLabel()` since it always returns false for the boolean type
    label: V = !0,
    ...W
  } = Ot(l, L), q = gi(i, B, k), Q = j(jt.YesLabel), X = j(jt.NoLabel);
  let ce;
  const J = (n = (t = O ?? b) !== null && t !== void 0 ? t : C) !== null && n !== void 0 ? n : s;
  if (Array.isArray(i.oneOf))
    ce = Ff({
      oneOf: i.oneOf.map((ie) => {
        if (co(ie))
          return {
            ...ie,
            title: ie.title || (ie.const === !0 ? Q : X)
          };
      }).filter((ie) => ie)
      // cast away the error that typescript can't grok is fixed
    }, l);
  else {
    const ie = i, re = (o = i.enum) !== null && o !== void 0 ? o : [!0, !1];
    !ie.enumNames && re.length === 2 && re.every((Y) => typeof Y == "boolean") ? ce = [
      {
        value: re[0],
        label: re[0] ? Q : X
      },
      {
        value: re[1],
        label: re[1] ? Q : X
      }
    ] : ce = Ff({
      enum: re,
      // NOTE: enumNames is deprecated, but still supported for now.
      enumNames: ie.enumNames
    }, l);
  }
  return A.jsx(q, { options: { ...W, enumOptions: ce }, schema: i, uiSchema: l, id: u.$id, name: s, onChange: $, onFocus: S, onBlur: w, label: J, hideLabel: !V, value: c, required: p, disabled: g, readonly: h, hideError: y, registry: f, formContext: E, autofocus: v, rawErrors: _ });
}
class m2 extends z.Component {
  /** Constructs an `AnyOfField` with the given `props` to initialize the initially selected option in state
   *
   * @param props - The `FieldProps` for this template
   */
  constructor(t) {
    super(t), this.onOptionChange = (l) => {
      const { selectedOption: u, retrievedOptions: c } = this.state, { formData: f, onChange: p, registry: g } = this.props, { schemaUtils: h } = g, y = l !== void 0 ? parseInt(l, 10) : -1;
      if (y === u)
        return;
      const v = y >= 0 ? c[y] : void 0, C = u >= 0 ? c[u] : void 0;
      let $ = h.sanitizeDataForNewSchema(v, C, f);
      v && ($ = h.getDefaultFormState(v, $, "excludeObjectChildren")), this.setState({ selectedOption: y }, () => {
        p($, void 0, this.getFieldId());
      });
    };
    const { formData: n, options: o, registry: { schemaUtils: i } } = this.props, s = o.map((l) => i.retrieveSchema(l, n));
    this.state = {
      retrievedOptions: s,
      selectedOption: this.getMatchingOption(0, n, s)
    };
  }
  /** React lifecycle method that is called when the props and/or state for this component is updated. It recomputes the
   * currently selected option based on the overall `formData`
   *
   * @param prevProps - The previous `FieldProps` for this template
   * @param prevState - The previous `AnyOfFieldState` for this template
   */
  componentDidUpdate(t, n) {
    const { formData: o, options: i, idSchema: s } = this.props, { selectedOption: l } = this.state;
    let u = this.state;
    if (!Sr(t.options, i)) {
      const { registry: { schemaUtils: c } } = this.props, f = i.map((p) => c.retrieveSchema(p, o));
      u = { selectedOption: l, retrievedOptions: f };
    }
    if (!Sr(o, t.formData) && s.$id === t.idSchema.$id) {
      const { retrievedOptions: c } = u, f = this.getMatchingOption(l, o, c);
      n && f !== l && (u = { selectedOption: f, retrievedOptions: c });
    }
    u !== this.state && this.setState(u);
  }
  /** Determines the best matching option for the given `formData` and `options`.
   *
   * @param formData - The new formData
   * @param options - The list of options to choose from
   * @return - The index of the `option` that best matches the `formData`
   */
  getMatchingOption(t, n, o) {
    const { schema: i, registry: { schemaUtils: s } } = this.props, l = il(i);
    return s.getClosestMatchingOption(n, o, t, l);
  }
  getFieldId() {
    const { idSchema: t, schema: n } = this.props;
    return `${t.$id}${n.oneOf ? "__oneof_select" : "__anyof_select"}`;
  }
  /** Renders the `AnyOfField` selector along with a `SchemaField` for the value of the `formData`
   */
  render() {
    const { name: t, disabled: n = !1, errorSchema: o = {}, formContext: i, onBlur: s, onFocus: l, registry: u, schema: c, uiSchema: f } = this.props, { widgets: p, fields: g, translateString: h, globalUiOptions: y, schemaUtils: v } = u, { SchemaField: C } = g, { selectedOption: $, retrievedOptions: S } = this.state, { widget: w = "select", placeholder: _, autofocus: b, autocomplete: k, title: E = c.title, ...j } = Ot(f, y), L = gi({ type: "number" }, w, p), B = Je(o, Vr, []), O = Rf(o, [Vr]), V = v.getDisplayLabel(c, f, y), W = $ >= 0 && S[$] || null;
    let q;
    if (W) {
      const { required: re } = c;
      q = re ? $i({ required: re }, W) : W;
    }
    let Q = [];
    uo in c && f && uo in f ? Array.isArray(f[uo]) ? Q = f[uo] : console.warn(`uiSchema.oneOf is not an array for "${E || t}"`) : Go in c && f && Go in f && (Array.isArray(f[Go]) ? Q = f[Go] : console.warn(`uiSchema.anyOf is not an array for "${E || t}"`));
    let X = f;
    $ >= 0 && Q.length > $ && (X = Q[$]);
    const ce = E ? jt.TitleOptionPrefix : jt.OptionPrefix, J = E ? [E] : [], ie = S.map((re, Y) => {
      const { title: T = re.title } = Ot(Q[Y]);
      return {
        label: T || h(ce, J.concat(String(Y + 1))),
        value: Y
      };
    });
    return A.jsxs("div", { className: "panel panel-default panel-body", children: [A.jsx("div", { className: "form-group", children: A.jsx(L, { id: this.getFieldId(), name: `${t}${c.oneOf ? "__oneof_select" : "__anyof_select"}`, schema: { type: "number", default: 0 }, onChange: this.onOptionChange, onBlur: s, onFocus: l, disabled: n || oa(ie), multiple: !1, rawErrors: B, errorSchema: O, value: $ >= 0 ? $ : void 0, options: { enumOptions: ie, ...j }, registry: u, formContext: i, placeholder: _, autocomplete: k, autofocus: b, label: E ?? t, hideLabel: !V }) }), q && A.jsx(C, { ...this.props, schema: q, uiSchema: X })] });
  }
}
const zU = /\.([0-9]*0)*$/, BU = /[0.]0*$/;
function UU(e) {
  const { registry: t, onChange: n, formData: o, value: i } = e, [s, l] = z.useState(i), { StringField: u } = t.fields;
  let c = o;
  const f = z.useCallback((p, g, h) => {
    l(p), `${p}`.charAt(0) === "." && (p = `0${p}`);
    const y = typeof p == "string" && p.match(zU) ? Kb(p.replace(BU, "")) : Kb(p);
    n(y, g, h);
  }, [n]);
  if (typeof s == "string" && typeof c == "number") {
    const p = new RegExp(`^(${String(c).replace(".", "\\.")})?\\.?0*$`);
    s.match(p) && (c = s);
  }
  return A.jsx(u, { ...e, formData: c, onChange: f });
}
function Na() {
  return Na = Object.assign ? Object.assign.bind() : function(e) {
    for (var t = 1; t < arguments.length; t++) {
      var n = arguments[t];
      for (var o in n) Object.prototype.hasOwnProperty.call(n, o) && (e[o] = n[o]);
    }
    return e;
  }, Na.apply(this, arguments);
}
const VU = ["children", "options"], it = { blockQuote: "0", breakLine: "1", breakThematic: "2", codeBlock: "3", codeFenced: "4", codeInline: "5", footnote: "6", footnoteReference: "7", gfmTask: "8", heading: "9", headingSetext: "10", htmlBlock: "11", htmlComment: "12", htmlSelfClosing: "13", image: "14", link: "15", linkAngleBraceStyleDetector: "16", linkBareUrlDetector: "17", linkMailtoDetector: "18", newlineCoalescer: "19", orderedList: "20", paragraph: "21", ref: "22", refImage: "23", refLink: "24", table: "25", tableSeparator: "26", text: "27", textBolded: "28", textEmphasized: "29", textEscaped: "30", textMarked: "31", textStrikethroughed: "32", unorderedList: "33" };
var g2;
(function(e) {
  e[e.MAX = 0] = "MAX", e[e.HIGH = 1] = "HIGH", e[e.MED = 2] = "MED", e[e.LOW = 3] = "LOW", e[e.MIN = 4] = "MIN";
})(g2 || (g2 = {}));
const y2 = ["allowFullScreen", "allowTransparency", "autoComplete", "autoFocus", "autoPlay", "cellPadding", "cellSpacing", "charSet", "classId", "colSpan", "contentEditable", "contextMenu", "crossOrigin", "encType", "formAction", "formEncType", "formMethod", "formNoValidate", "formTarget", "frameBorder", "hrefLang", "inputMode", "keyParams", "keyType", "marginHeight", "marginWidth", "maxLength", "mediaGroup", "minLength", "noValidate", "radioGroup", "readOnly", "rowSpan", "spellCheck", "srcDoc", "srcLang", "srcSet", "tabIndex", "useMap"].reduce((e, t) => (e[t.toLowerCase()] = t, e), { class: "className", for: "htmlFor" }), v2 = { amp: "&", apos: "'", gt: ">", lt: "<", nbsp: " ", quot: "“" }, WU = ["style", "script"], qU = /([-A-Z0-9_:]+)(?:\s*=\s*(?:(?:"((?:\\.|[^"])*)")|(?:'((?:\\.|[^'])*)')|(?:\{((?:\\.|{[^}]*?}|[^}])*)\})))?/gi, KU = /mailto:/i, HU = /\n{2,}$/, hx = /^(\s*>[\s\S]*?)(?=\n\n|$)/, GU = /^ *> ?/gm, JU = /^(?:\[!([^\]]*)\]\n)?([\s\S]*)/, YU = /^ {2,}\n/, QU = /^(?:( *[-*_])){3,} *(?:\n *)+\n/, mx = /^(?: {1,3})?(`{3,}|~{3,}) *(\S+)? *([^\n]*?)?\n([\s\S]*?)(?:\1\n?|$)/, gx = /^(?: {4}[^\n]+\n*)+(?:\n *)+\n?/, XU = /^(`+)((?:\\`|[^`])+)\1/, ZU = /^(?:\n *)*\n/, eV = /\r\n?/g, tV = /^\[\^([^\]]+)](:(.*)((\n+ {4,}.*)|(\n(?!\[\^).+))*)/, nV = /^\[\^([^\]]+)]/, rV = /\f/g, oV = /^---[ \t]*\n(.|\n)*\n---[ \t]*\n/, iV = /^\s*?\[(x|\s)\]/, yx = /^ *(#{1,6}) *([^\n]+?)(?: +#*)?(?:\n *)*(?:\n|$)/, vx = /^ *(#{1,6}) +([^\n]+?)(?: +#*)?(?:\n *)*(?:\n|$)/, bx = /^([^\n]+)\n *(=|-){3,} *(?:\n *)+\n/, xg = /^ *(?!<[a-z][^ >/]* ?\/>)<([a-z][^ >/]*) ?((?:[^>]*[^/])?)>\n?(\s*(?:<\1[^>]*?>[\s\S]*?<\/\1>|(?!<\1\b)[\s\S])*?)<\/\1>(?!<\/\1>)\n*/i, aV = /&([a-z0-9]+|#[0-9]{1,6}|#x[0-9a-fA-F]{1,6});/gi, Sx = /^<!--[\s\S]*?(?:-->)/, sV = /^(data|aria|x)-[a-z_][a-z\d_.-]*$/, Eg = /^ *<([a-z][a-z0-9:]*)(?:\s+((?:<.*?>|[^>])*))?\/?>(?!<\/\1>)(\s*\n)?/i, lV = /^\{.*\}$/, uV = /^(https?:\/\/[^\s<]+[^<.,:;"')\]\s])/, cV = /^<([^ >]+@[^ >]+)>/, dV = /^<([^ >]+:\/[^ >]+)>/, fV = /-([a-z])?/gi, wx = /^(\|.*)\n(?: *(\|? *[-:]+ *\|[-| :]*)\n((?:.*\|.*\n)*))?\n?/, pV = /^\[([^\]]*)\]:\s+<?([^\s>]+)>?\s*("([^"]*)")?/, hV = /^!\[([^\]]*)\] ?\[([^\]]*)\]/, mV = /^\[([^\]]*)\] ?\[([^\]]*)\]/, gV = /(\n|^[-*]\s|^#|^ {2,}|^-{2,}|^>\s)/, yV = /\t/g, vV = /(^ *\||\| *$)/g, bV = /^ *:-+: *$/, SV = /^ *:-+ *$/, wV = /^ *-+: *$/, Sp = "((?:\\[.*?\\][([].*?[)\\]]|<.*?>(?:.*?<.*?>)?|`.*?`|\\\\\\1|[\\s\\S])+?)", $V = new RegExp(`^([*_])\\1${Sp}\\1\\1(?!\\1)`), CV = new RegExp(`^([*_])${Sp}\\1(?!\\1)`), xV = new RegExp(`^(==)${Sp}\\1`), EV = new RegExp(`^(~~)${Sp}\\1`), _V = /^\\([^0-9A-Za-z\s])/, Vm = /\\([^0-9A-Za-z\s])/g, kV = /^([\s\S](?:(?!  |[0-9]\.)[^*_~\-\n<`\\\[!])*)/, PV = /^\n+/, OV = /^([ \t]*)/, AV = /\\([^\\])/g, IV = /(?:^|\n)( *)$/, Zy = "(?:\\d+\\.)", e0 = "(?:[*+-])";
function $x(e) {
  return "( *)(" + (e === 1 ? Zy : e0) + ") +";
}
const Cx = $x(1), xx = $x(2);
function Ex(e) {
  return new RegExp("^" + (e === 1 ? Cx : xx));
}
const jV = Ex(1), RV = Ex(2);
function _x(e) {
  return new RegExp("^" + (e === 1 ? Cx : xx) + "[^\\n]*(?:\\n(?!\\1" + (e === 1 ? Zy : e0) + " )[^\\n]*)*(\\n|$)", "gm");
}
const TV = _x(1), MV = _x(2);
function kx(e) {
  const t = e === 1 ? Zy : e0;
  return new RegExp("^( *)(" + t + ") [\\s\\S]+?(?:\\n{2,}(?! )(?!\\1" + t + " (?!" + t + " ))\\n*|\\s*\\n*$)");
}
const Px = kx(1), Ox = kx(2);
function b2(e, t) {
  const n = t === 1, o = n ? Px : Ox, i = n ? TV : MV, s = n ? jV : RV;
  return { match: sl(function(l, u) {
    const c = IV.exec(u.prevCapture);
    return c && (u.list || !u.inline && !u.simple) ? o.exec(l = c[1] + l) : null;
  }), order: 1, parse(l, u, c) {
    const f = n ? +l[2] : void 0, p = l[0].replace(HU, `
`).match(i);
    let g = !1;
    return { items: p.map(function(h, y) {
      const v = s.exec(h)[0].length, C = new RegExp("^ {1," + v + "}", "gm"), $ = h.replace(C, "").replace(s, ""), S = y === p.length - 1, w = $.indexOf(`

`) !== -1 || S && g;
      g = w;
      const _ = c.inline, b = c.list;
      let k;
      c.list = !0, w ? (c.inline = !1, k = ac($) + `

`) : (c.inline = !0, k = ac($));
      const E = u(k, c);
      return c.inline = _, c.list = b, E;
    }), ordered: n, start: f };
  }, render: (l, u, c) => e(l.ordered ? "ol" : "ul", { key: c.key, start: l.type === it.orderedList ? l.start : void 0 }, l.items.map(function(f, p) {
    return e("li", { key: p }, u(f, c));
  })) };
}
const NV = new RegExp(`^\\[((?:\\[[^\\]]*\\]|[^\\[\\]]|\\](?=[^\\[]*\\]))*)\\]\\(\\s*<?((?:\\([^)]*\\)|[^\\s\\\\]|\\\\.)*?)>?(?:\\s+['"]([\\s\\S]*?)['"])?\\s*\\)`), FV = /^!\[(.*?)\]\( *((?:\([^)]*\)|[^() ])*) *"?([^)"]*)?"?\)/, Ax = [hx, mx, gx, yx, bx, vx, wx, Px, Ox], LV = [...Ax, /^[^\n]+(?:  \n|\n{2,})/, xg, Sx, Eg];
function ac(e) {
  let t = e.length;
  for (; t > 0 && e[t - 1] <= " "; ) t--;
  return e.slice(0, t);
}
function $u(e) {
  return e.replace(/[ÀÁÂÃÄÅàáâãäåæÆ]/g, "a").replace(/[çÇ]/g, "c").replace(/[ðÐ]/g, "d").replace(/[ÈÉÊËéèêë]/g, "e").replace(/[ÏïÎîÍíÌì]/g, "i").replace(/[Ññ]/g, "n").replace(/[øØœŒÕõÔôÓóÒò]/g, "o").replace(/[ÜüÛûÚúÙù]/g, "u").replace(/[ŸÿÝý]/g, "y").replace(/[^a-z0-9- ]/gi, "").replace(/ /gi, "-").toLowerCase();
}
function DV(e) {
  return wV.test(e) ? "right" : bV.test(e) ? "center" : SV.test(e) ? "left" : null;
}
function S2(e, t, n, o) {
  const i = n.inTable;
  n.inTable = !0;
  let s = [[]], l = "";
  function u() {
    if (!l) return;
    const c = s[s.length - 1];
    c.push.apply(c, t(l, n)), l = "";
  }
  return e.trim().split(/(`[^`]*`|\\\||\|)/).filter(Boolean).forEach((c, f, p) => {
    c.trim() === "|" && (u(), o) ? f !== 0 && f !== p.length - 1 && s.push([]) : l += c;
  }), u(), n.inTable = i, s;
}
function zV(e, t, n) {
  n.inline = !0;
  const o = e[2] ? e[2].replace(vV, "").split("|").map(DV) : [], i = e[3] ? function(l, u, c) {
    return l.trim().split(`
`).map(function(f) {
      return S2(f, u, c, !0);
    });
  }(e[3], t, n) : [], s = S2(e[1], t, n, !!i.length);
  return n.inline = !1, i.length ? { align: o, cells: i, header: s, type: it.table } : { children: s, type: it.paragraph };
}
function w2(e, t) {
  return e.align[t] == null ? {} : { textAlign: e.align[t] };
}
function sl(e) {
  return e.inline = 1, e;
}
function Wi(e) {
  return sl(function(t, n) {
    return n.inline ? e.exec(t) : null;
  });
}
function qi(e) {
  return sl(function(t, n) {
    return n.inline || n.simple ? e.exec(t) : null;
  });
}
function pi(e) {
  return function(t, n) {
    return n.inline || n.simple ? null : e.exec(t);
  };
}
function Cu(e) {
  return sl(function(t) {
    return e.exec(t);
  });
}
function BV(e, t) {
  if (t.inline || t.simple) return null;
  let n = "";
  e.split(`
`).every((i) => (i += `
`, !Ax.some((s) => s.test(i)) && (n += i, !!i.trim())));
  const o = ac(n);
  return o == "" ? null : [n, , o];
}
function UV(e) {
  try {
    if (decodeURIComponent(e).replace(/[^A-Za-z0-9/:]/g, "").match(/^\s*(javascript|vbscript|data(?!:image)):/i)) return null;
  } catch {
    return null;
  }
  return e;
}
function $2(e) {
  return e.replace(AV, "$1");
}
function vf(e, t, n) {
  const o = n.inline || !1, i = n.simple || !1;
  n.inline = !0, n.simple = !0;
  const s = e(t, n);
  return n.inline = o, n.simple = i, s;
}
function VV(e, t, n) {
  const o = n.inline || !1, i = n.simple || !1;
  n.inline = !1, n.simple = !0;
  const s = e(t, n);
  return n.inline = o, n.simple = i, s;
}
function WV(e, t, n) {
  const o = n.inline || !1;
  n.inline = !1;
  const i = e(t, n);
  return n.inline = o, i;
}
const Wm = (e, t, n) => ({ children: vf(t, e[2], n) });
function qm() {
  return {};
}
function Km() {
  return null;
}
function qV(...e) {
  return e.filter(Boolean).join(" ");
}
function Hm(e, t, n) {
  let o = e;
  const i = t.split(".");
  for (; i.length && (o = o[i[0]], o !== void 0); ) i.shift();
  return o || n;
}
function KV(e = "", t = {}) {
  function n(h, y, ...v) {
    const C = Hm(t.overrides, `${h}.props`, {});
    return t.createElement(function($, S) {
      const w = Hm(S, $);
      return w ? typeof w == "function" || typeof w == "object" && "render" in w ? w : Hm(S, `${$}.component`, $) : $;
    }(h, t.overrides), Na({}, y, C, { className: qV(y == null ? void 0 : y.className, C.className) || void 0 }), ...v);
  }
  function o(h) {
    h = h.replace(oV, "");
    let y = !1;
    t.forceInline ? y = !0 : t.forceBlock || (y = gV.test(h) === !1);
    const v = f(c(y ? h : `${ac(h).replace(PV, "")}

`, { inline: y }));
    for (; typeof v[v.length - 1] == "string" && !v[v.length - 1].trim(); ) v.pop();
    if (t.wrapper === null) return v;
    const C = t.wrapper || (y ? "span" : "div");
    let $;
    if (v.length > 1 || t.forceWrapper) $ = v;
    else {
      if (v.length === 1) return $ = v[0], typeof $ == "string" ? n("span", { key: "outer" }, $) : $;
      $ = null;
    }
    return t.createElement(C, { key: "outer" }, $);
  }
  function i(h, y) {
    const v = y.match(qU);
    return v ? v.reduce(function(C, $) {
      const S = $.indexOf("=");
      if (S !== -1) {
        const w = function(E) {
          return E.indexOf("-") !== -1 && E.match(sV) === null && (E = E.replace(fV, function(j, L) {
            return L.toUpperCase();
          })), E;
        }($.slice(0, S)).trim(), _ = function(E) {
          const j = E[0];
          return (j === '"' || j === "'") && E.length >= 2 && E[E.length - 1] === j ? E.slice(1, -1) : E;
        }($.slice(S + 1).trim()), b = y2[w] || w;
        if (b === "ref") return C;
        const k = C[b] = function(E, j, L, B) {
          return j === "style" ? L.split(/;\s?/).reduce(function(O, V) {
            const W = V.slice(0, V.indexOf(":"));
            return O[W.trim().replace(/(-[a-z])/g, (q) => q[1].toUpperCase())] = V.slice(W.length + 1).trim(), O;
          }, {}) : j === "href" || j === "src" ? B(L, E, j) : (L.match(lV) && (L = L.slice(1, L.length - 1)), L === "true" || L !== "false" && L);
        }(h, w, _, t.sanitizer);
        typeof k == "string" && (xg.test(k) || Eg.test(k)) && (C[b] = o(k.trim()));
      } else $ !== "style" && (C[y2[$] || $] = !0);
      return C;
    }, {}) : null;
  }
  t.overrides = t.overrides || {}, t.sanitizer = t.sanitizer || UV, t.slugify = t.slugify || $u, t.namedCodesToUnicode = t.namedCodesToUnicode ? Na({}, v2, t.namedCodesToUnicode) : v2, t.createElement = t.createElement || z.createElement;
  const s = [], l = {}, u = { [it.blockQuote]: { match: pi(hx), order: 1, parse(h, y, v) {
    const [, C, $] = h[0].replace(GU, "").match(JU);
    return { alert: C, children: y($, v) };
  }, render(h, y, v) {
    const C = { key: v.key };
    return h.alert && (C.className = "markdown-alert-" + t.slugify(h.alert.toLowerCase(), $u), h.children.unshift({ attrs: {}, children: [{ type: it.text, text: h.alert }], noInnerParse: !0, type: it.htmlBlock, tag: "header" })), n("blockquote", C, y(h.children, v));
  } }, [it.breakLine]: { match: Cu(YU), order: 1, parse: qm, render: (h, y, v) => n("br", { key: v.key }) }, [it.breakThematic]: { match: pi(QU), order: 1, parse: qm, render: (h, y, v) => n("hr", { key: v.key }) }, [it.codeBlock]: { match: pi(gx), order: 0, parse: (h) => ({ lang: void 0, text: ac(h[0].replace(/^ {4}/gm, "")).replace(Vm, "$1") }), render: (h, y, v) => n("pre", { key: v.key }, n("code", Na({}, h.attrs, { className: h.lang ? `lang-${h.lang}` : "" }), h.text)) }, [it.codeFenced]: { match: pi(mx), order: 0, parse: (h) => ({ attrs: i("code", h[3] || ""), lang: h[2] || void 0, text: h[4].replace(Vm, "$1"), type: it.codeBlock }) }, [it.codeInline]: { match: qi(XU), order: 3, parse: (h) => ({ text: h[2].replace(Vm, "$1") }), render: (h, y, v) => n("code", { key: v.key }, h.text) }, [it.footnote]: { match: pi(tV), order: 0, parse: (h) => (s.push({ footnote: h[2], identifier: h[1] }), {}), render: Km }, [it.footnoteReference]: { match: Wi(nV), order: 1, parse: (h) => ({ target: `#${t.slugify(h[1], $u)}`, text: h[1] }), render: (h, y, v) => n("a", { key: v.key, href: t.sanitizer(h.target, "a", "href") }, n("sup", { key: v.key }, h.text)) }, [it.gfmTask]: { match: Wi(iV), order: 1, parse: (h) => ({ completed: h[1].toLowerCase() === "x" }), render: (h, y, v) => n("input", { checked: h.completed, key: v.key, readOnly: !0, type: "checkbox" }) }, [it.heading]: { match: pi(t.enforceAtxHeadings ? vx : yx), order: 1, parse: (h, y, v) => ({ children: vf(y, h[2], v), id: t.slugify(h[2], $u), level: h[1].length }), render: (h, y, v) => n(`h${h.level}`, { id: h.id, key: v.key }, y(h.children, v)) }, [it.headingSetext]: { match: pi(bx), order: 0, parse: (h, y, v) => ({ children: vf(y, h[1], v), level: h[2] === "=" ? 1 : 2, type: it.heading }) }, [it.htmlBlock]: { match: Cu(xg), order: 1, parse(h, y, v) {
    const [, C] = h[3].match(OV), $ = new RegExp(`^${C}`, "gm"), S = h[3].replace($, ""), w = (_ = S, LV.some((L) => L.test(_)) ? WV : vf);
    var _;
    const b = h[1].toLowerCase(), k = WU.indexOf(b) !== -1, E = (k ? b : h[1]).trim(), j = { attrs: i(E, h[2]), noInnerParse: k, tag: E };
    return v.inAnchor = v.inAnchor || b === "a", k ? j.text = h[3] : j.children = w(y, S, v), v.inAnchor = !1, j;
  }, render: (h, y, v) => n(h.tag, Na({ key: v.key }, h.attrs), h.text || (h.children ? y(h.children, v) : "")) }, [it.htmlSelfClosing]: { match: Cu(Eg), order: 1, parse(h) {
    const y = h[1].trim();
    return { attrs: i(y, h[2] || ""), tag: y };
  }, render: (h, y, v) => n(h.tag, Na({}, h.attrs, { key: v.key })) }, [it.htmlComment]: { match: Cu(Sx), order: 1, parse: () => ({}), render: Km }, [it.image]: { match: qi(FV), order: 1, parse: (h) => ({ alt: h[1], target: $2(h[2]), title: h[3] }), render: (h, y, v) => n("img", { key: v.key, alt: h.alt || void 0, title: h.title || void 0, src: t.sanitizer(h.target, "img", "src") }) }, [it.link]: { match: Wi(NV), order: 3, parse: (h, y, v) => ({ children: VV(y, h[1], v), target: $2(h[2]), title: h[3] }), render: (h, y, v) => n("a", { key: v.key, href: t.sanitizer(h.target, "a", "href"), title: h.title }, y(h.children, v)) }, [it.linkAngleBraceStyleDetector]: { match: Wi(dV), order: 0, parse: (h) => ({ children: [{ text: h[1], type: it.text }], target: h[1], type: it.link }) }, [it.linkBareUrlDetector]: { match: sl((h, y) => y.inAnchor || t.disableAutoLink ? null : Wi(uV)(h, y)), order: 0, parse: (h) => ({ children: [{ text: h[1], type: it.text }], target: h[1], title: void 0, type: it.link }) }, [it.linkMailtoDetector]: { match: Wi(cV), order: 0, parse(h) {
    let y = h[1], v = h[1];
    return KU.test(v) || (v = "mailto:" + v), { children: [{ text: y.replace("mailto:", ""), type: it.text }], target: v, type: it.link };
  } }, [it.orderedList]: b2(n, 1), [it.unorderedList]: b2(n, 2), [it.newlineCoalescer]: { match: pi(ZU), order: 3, parse: qm, render: () => `
` }, [it.paragraph]: { match: sl(BV), order: 3, parse: Wm, render: (h, y, v) => n("p", { key: v.key }, y(h.children, v)) }, [it.ref]: { match: Wi(pV), order: 0, parse: (h) => (l[h[1]] = { target: h[2], title: h[4] }, {}), render: Km }, [it.refImage]: { match: qi(hV), order: 0, parse: (h) => ({ alt: h[1] || void 0, ref: h[2] }), render: (h, y, v) => l[h.ref] ? n("img", { key: v.key, alt: h.alt, src: t.sanitizer(l[h.ref].target, "img", "src"), title: l[h.ref].title }) : null }, [it.refLink]: { match: Wi(mV), order: 0, parse: (h, y, v) => ({ children: y(h[1], v), fallbackChildren: h[0], ref: h[2] }), render: (h, y, v) => l[h.ref] ? n("a", { key: v.key, href: t.sanitizer(l[h.ref].target, "a", "href"), title: l[h.ref].title }, y(h.children, v)) : n("span", { key: v.key }, h.fallbackChildren) }, [it.table]: { match: pi(wx), order: 1, parse: zV, render(h, y, v) {
    const C = h;
    return n("table", { key: v.key }, n("thead", null, n("tr", null, C.header.map(function($, S) {
      return n("th", { key: S, style: w2(C, S) }, y($, v));
    }))), n("tbody", null, C.cells.map(function($, S) {
      return n("tr", { key: S }, $.map(function(w, _) {
        return n("td", { key: _, style: w2(C, _) }, y(w, v));
      }));
    })));
  } }, [it.text]: { match: Cu(kV), order: 4, parse: (h) => ({ text: h[0].replace(aV, (y, v) => t.namedCodesToUnicode[v] ? t.namedCodesToUnicode[v] : y) }), render: (h) => h.text }, [it.textBolded]: { match: qi($V), order: 2, parse: (h, y, v) => ({ children: y(h[2], v) }), render: (h, y, v) => n("strong", { key: v.key }, y(h.children, v)) }, [it.textEmphasized]: { match: qi(CV), order: 3, parse: (h, y, v) => ({ children: y(h[2], v) }), render: (h, y, v) => n("em", { key: v.key }, y(h.children, v)) }, [it.textEscaped]: { match: qi(_V), order: 1, parse: (h) => ({ text: h[1], type: it.text }) }, [it.textMarked]: { match: qi(xV), order: 3, parse: Wm, render: (h, y, v) => n("mark", { key: v.key }, y(h.children, v)) }, [it.textStrikethroughed]: { match: qi(EV), order: 3, parse: Wm, render: (h, y, v) => n("del", { key: v.key }, y(h.children, v)) } };
  t.disableParsingRawHTML === !0 && (delete u[it.htmlBlock], delete u[it.htmlSelfClosing]);
  const c = function(h) {
    let y = Object.keys(h);
    function v(C, $) {
      let S, w, _ = [], b = "", k = "";
      for ($.prevCapture = $.prevCapture || ""; C; ) {
        let E = 0;
        for (; E < y.length; ) {
          if (b = y[E], S = h[b], $.inline && !S.match.inline) {
            E++;
            continue;
          }
          const j = S.match(C, $);
          if (j) {
            k = j[0], $.prevCapture += k, C = C.substring(k.length), w = S.parse(j, v, $), w.type == null && (w.type = b), _.push(w);
            break;
          }
          E++;
        }
      }
      return $.prevCapture = "", _;
    }
    return y.sort(function(C, $) {
      let S = h[C].order, w = h[$].order;
      return S !== w ? S - w : C < $ ? -1 : 1;
    }), function(C, $) {
      return v(function(S) {
        return S.replace(eV, `
`).replace(rV, "").replace(yV, "    ");
      }(C), $);
    };
  }(u), f = (p = /* @__PURE__ */ function(h, y) {
    return function(v, C, $) {
      const S = h[v.type].render;
      return y ? y(() => S(v, C, $), v, C, $) : S(v, C, $);
    };
  }(u, t.renderRule), function h(y, v = {}) {
    if (Array.isArray(y)) {
      const C = v.key, $ = [];
      let S = !1;
      for (let w = 0; w < y.length; w++) {
        v.key = w;
        const _ = h(y[w], v), b = typeof _ == "string";
        b && S ? $[$.length - 1] += _ : _ !== null && $.push(_), S = b;
      }
      return v.key = C, $;
    }
    return p(y, h, v);
  });
  var p;
  const g = o(e);
  return s.length ? n("div", null, g, n("footer", { key: "footer" }, s.map(function(h) {
    return n("div", { id: t.slugify(h.identifier, $u), key: h.identifier }, h.identifier, f(c(h.footnote, { inline: !0 })));
  }))) : g;
}
const wp = (e) => {
  let { children: t = "", options: n } = e, o = function(i, s) {
    if (i == null) return {};
    var l, u, c = {}, f = Object.keys(i);
    for (u = 0; u < f.length; u++) s.indexOf(l = f[u]) >= 0 || (c[l] = i[l]);
    return c;
  }(e, VU);
  return z.cloneElement(KV(t, n), o);
};
var HV = Z$;
function GV(e, t) {
  return e == null ? !0 : HV(e, t);
}
var JV = GV;
const YV = /* @__PURE__ */ Ft(JV);
class QV extends z.Component {
  constructor() {
    super(...arguments), this.state = {
      wasPropertyKeyModified: !1,
      additionalProperties: {}
    }, this.onPropertyChange = (t, n = !1) => (o, i, s) => {
      const { formData: l, onChange: u, errorSchema: c } = this.props;
      o === void 0 && n && (o = "");
      const f = { ...l, [t]: o };
      u(f, c && c && {
        ...c,
        [t]: i
      }, s);
    }, this.onDropPropertyClick = (t) => (n) => {
      n.preventDefault();
      const { onChange: o, formData: i } = this.props, s = { ...i };
      YV(s, t), o(s);
    }, this.getAvailableKey = (t, n) => {
      const { uiSchema: o, registry: i } = this.props, { duplicateKeySuffixSeparator: s = "-" } = Ot(o, i.globalUiOptions);
      let l = 0, u = t;
      for (; br(n, u); )
        u = `${t}${s}${++l}`;
      return u;
    }, this.onKeyChange = (t) => (n, o) => {
      if (t === n)
        return;
      const { formData: i, onChange: s, errorSchema: l } = this.props;
      n = this.getAvailableKey(n, i);
      const u = {
        ...i
      }, c = { [t]: n }, f = Object.keys(u).map((g) => ({ [c[g] || g]: u[g] })), p = Object.assign({}, ...f);
      this.setState({ wasPropertyKeyModified: !0 }), s(p, l && l && {
        ...l,
        [n]: o
      });
    }, this.handleAddClick = (t) => () => {
      var n;
      if (!t.additionalProperties)
        return;
      const { formData: o, onChange: i, registry: s } = this.props, l = { ...o };
      let u, c, f;
      if (co(t.additionalProperties)) {
        u = t.additionalProperties.type, c = t.additionalProperties.const, f = t.additionalProperties.default;
        let h = t.additionalProperties;
        if (Dn in h) {
          const { schemaUtils: y } = s;
          h = y.retrieveSchema({ $ref: h[Dn] }, o), u = h.type, c = h.const, f = h.default;
        }
        !u && (Go in h || uo in h) && (u = "object");
      }
      const p = this.getAvailableKey("newKey", l), g = (n = c ?? f) !== null && n !== void 0 ? n : this.getDefaultValue(u);
      Yn(l, p, g), i(l);
    };
  }
  /** Returns a flag indicating whether the `name` field is required in the object schema
   *
   * @param name - The name of the field to check for required-ness
   * @returns - True if the field `name` is required, false otherwise
   */
  isRequired(t) {
    const { schema: n } = this.props;
    return Array.isArray(n.required) && n.required.indexOf(t) !== -1;
  }
  /** Returns a default value to be used for a new additional schema property of the given `type`
   *
   * @param type - The type of the new additional schema property
   */
  getDefaultValue(t) {
    const { registry: { translateString: n } } = this.props;
    switch (t) {
      case "array":
        return [];
      case "boolean":
        return !1;
      case "null":
        return null;
      case "number":
        return 0;
      case "object":
        return {};
      case "string":
      default:
        return n(jt.NewStringDefault);
    }
  }
  /** Renders the `ObjectField` from the given props
   */
  render() {
    var t, n, o, i;
    const { schema: s, uiSchema: l = {}, formData: u, errorSchema: c, idSchema: f, name: p, required: g = !1, disabled: h, readonly: y, hideError: v, idPrefix: C, idSeparator: $, onBlur: S, onFocus: w, registry: _, title: b } = this.props, { fields: k, formContext: E, schemaUtils: j, translateString: L, globalUiOptions: B } = _, { SchemaField: O } = k, V = j.retrieveSchema(s, u), W = Ot(l, B), { properties: q = {} } = V, Q = (o = (n = (t = W.title) !== null && t !== void 0 ? t : V.title) !== null && n !== void 0 ? n : b) !== null && o !== void 0 ? o : p, X = (i = W.description) !== null && i !== void 0 ? i : V.description;
    let ce;
    try {
      const re = Object.keys(q);
      ce = uU(re, W.order);
    } catch (re) {
      return A.jsxs("div", { children: [A.jsx("p", { className: "config-error", style: { color: "red" }, children: A.jsx(wp, { options: { disableParsingRawHTML: !0 }, children: L(jt.InvalidObjectField, [p || "root", re.message]) }) }), A.jsx("pre", { children: JSON.stringify(V) })] });
    }
    const J = _t("ObjectFieldTemplate", _, W), ie = {
      // getDisplayLabel() always returns false for object types, so just check the `uiOptions.label`
      title: W.label === !1 ? "" : Q,
      description: W.label === !1 ? void 0 : X,
      properties: ce.map((re) => {
        const Y = br(V, [wn, re, gl]), T = Y ? l.additionalProperties : l[re], H = Ot(T).widget === "hidden", G = Je(f, [re], {});
        return {
          content: A.jsx(O, { name: re, required: this.isRequired(re), schema: Je(V, [wn, re], {}), uiSchema: T, errorSchema: Je(c, re), idSchema: G, idPrefix: C, idSeparator: $, formData: Je(u, re), formContext: E, wasPropertyKeyModified: this.state.wasPropertyKeyModified, onKeyChange: this.onKeyChange(re), onChange: this.onPropertyChange(re, Y), onBlur: S, onFocus: w, registry: _, disabled: h, readonly: y, hideError: v, onDropPropertyClick: this.onDropPropertyClick }, re),
          name: re,
          readonly: y,
          disabled: h,
          required: g,
          hidden: H
        };
      }),
      readonly: y,
      disabled: h,
      required: g,
      idSchema: f,
      uiSchema: l,
      errorSchema: c,
      schema: V,
      formData: u,
      formContext: E,
      registry: _
    };
    return A.jsx(J, { ...ie, onAddClick: this.handleAddClick });
  }
}
const XV = {
  array: "ArrayField",
  boolean: "BooleanField",
  integer: "NumberField",
  number: "NumberField",
  object: "ObjectField",
  string: "StringField",
  null: "NullField"
};
function ZV(e, t, n, o) {
  const i = t.field, { fields: s, translateString: l } = o;
  if (typeof i == "function")
    return i;
  if (typeof i == "string" && i in s)
    return s[i];
  const u = ia(e), c = Array.isArray(u) ? u[0] : u || "", f = e.$id;
  let p = XV[c];
  return f && f in s && (p = f), !p && (e.anyOf || e.oneOf) ? () => null : p in s ? s[p] : () => {
    const g = _t("UnsupportedFieldTemplate", o, t);
    return A.jsx(g, { schema: e, idSchema: n, reason: l(jt.UnknownFieldType, [String(e.type)]), registry: o });
  };
}
function eW(e) {
  var t, n, o;
  const { schema: i, idSchema: s, uiSchema: l, formData: u, errorSchema: c, idPrefix: f, idSeparator: p, name: g, onChange: h, onKeyChange: y, onDropPropertyClick: v, required: C, registry: $, wasPropertyKeyModified: S = !1 } = e, { formContext: w, schemaUtils: _, globalUiOptions: b } = $, k = Ot(l, b), E = _t("FieldTemplate", $, k), j = _t("DescriptionFieldTemplate", $, k), L = _t("FieldHelpTemplate", $, k), B = _t("FieldErrorTemplate", $, k), O = _.retrieveSchema(i, u), V = s[Yi], W = Zi(_.toIdSchema(O, V, u, f, p), s), q = z.useCallback((pt, Se, gt) => h(pt, Se, gt || V), [V, h]), Q = ZV(O, k, W, $), X = !!((t = k.disabled) !== null && t !== void 0 ? t : e.disabled), ce = !!((n = k.readonly) !== null && n !== void 0 ? n : e.readonly || e.schema.readOnly || O.readOnly), J = k.hideError, ie = J === void 0 ? e.hideError : !!J, re = !!((o = k.autofocus) !== null && o !== void 0 ? o : e.autofocus);
  if (Object.keys(O).length === 0)
    return null;
  const Y = _.getDisplayLabel(O, l, b), { __errors: T, ...H } = c || {}, G = Rf(l, ["ui:classNames", "classNames", "ui:style"]);
  Bu in G && (G[Bu] = Rf(G[Bu], ["classNames", "style"]));
  const N = A.jsx(Q, { ...e, onChange: q, idSchema: W, schema: O, uiSchema: G, disabled: X, readonly: ce, hideError: ie, autofocus: re, errorSchema: H, formContext: w, rawErrors: T }), U = W[Yi];
  let Z;
  S ? Z = g : Z = gl in O ? g : k.title || e.schema.title || O.title || e.title || g;
  const me = k.description || e.schema.description || O.description || "", ue = k.enableMarkdownInDescription ? A.jsx(wp, { options: { disableParsingRawHTML: !0 }, children: me }) : me, Ce = k.help, $e = k.widget === "hidden", Te = ["form-group", "field", `field-${ia(O)}`];
  !ie && T && T.length > 0 && Te.push("field-error has-error has-danger"), l != null && l.classNames && Te.push(l.classNames), k.classNames && Te.push(k.classNames);
  const We = A.jsx(L, { help: Ce, idSchema: W, schema: O, uiSchema: l, hasErrors: !ie && T && T.length > 0, registry: $ }), et = ie || (O.anyOf || O.oneOf) && !_.isSelect(O) ? void 0 : A.jsx(B, { errors: T, errorSchema: c, idSchema: W, schema: O, uiSchema: l, registry: $ }), Xe = {
    description: A.jsx(j, { id: es(U), description: ue, schema: O, uiSchema: l, registry: $ }),
    rawDescription: me,
    help: We,
    rawHelp: typeof Ce == "string" ? Ce : void 0,
    errors: et,
    rawErrors: ie ? void 0 : T,
    id: U,
    label: Z,
    hidden: $e,
    onChange: h,
    onKeyChange: y,
    onDropPropertyClick: v,
    required: C,
    disabled: X,
    readonly: ce,
    hideError: ie,
    displayLabel: Y,
    classNames: Te.join(" ").trim(),
    style: k.style,
    formContext: w,
    formData: u,
    schema: O,
    uiSchema: l,
    registry: $
  }, ft = $.fields.AnyOfField, Lt = $.fields.OneOfField, st = (l == null ? void 0 : l["ui:field"]) && (l == null ? void 0 : l["ui:fieldReplacesAnyOrOneOf"]) === !0;
  return A.jsx(E, { ...Xe, children: A.jsxs(A.Fragment, { children: [N, O.anyOf && !st && !_.isSelect(O) && A.jsx(ft, { name: g, disabled: X, readonly: ce, hideError: ie, errorSchema: c, formData: u, formContext: w, idPrefix: f, idSchema: W, idSeparator: p, onBlur: e.onBlur, onChange: e.onChange, onFocus: e.onFocus, options: O.anyOf.map((pt) => _.retrieveSchema(co(pt) ? pt : {}, u)), registry: $, required: C, schema: O, uiSchema: l }), O.oneOf && !st && !_.isSelect(O) && A.jsx(Lt, { name: g, disabled: X, readonly: ce, hideError: ie, errorSchema: c, formData: u, formContext: w, idPrefix: f, idSchema: W, idSeparator: p, onBlur: e.onBlur, onChange: e.onChange, onFocus: e.onFocus, options: O.oneOf.map((pt) => _.retrieveSchema(co(pt) ? pt : {}, u)), registry: $, required: C, schema: O, uiSchema: l })] }) });
}
let tW = class extends z.Component {
  shouldComponentUpdate(t) {
    return !Sr(this.props, t);
  }
  render() {
    return A.jsx(eW, { ...this.props });
  }
};
function nW(e) {
  var t;
  const { schema: n, name: o, uiSchema: i, idSchema: s, formData: l, required: u, disabled: c = !1, readonly: f = !1, autofocus: p = !1, onChange: g, onBlur: h, onFocus: y, registry: v, rawErrors: C, hideError: $ } = e, { title: S, format: w } = n, { widgets: _, formContext: b, schemaUtils: k, globalUiOptions: E } = v, j = k.isSelect(n) ? Ff(n, i) : void 0;
  let L = j ? "select" : "text";
  w && aU(n, w, _) && (L = w);
  const { widget: B = L, placeholder: O = "", title: V, ...W } = Ot(i), q = k.getDisplayLabel(n, i, E), Q = (t = V ?? S) !== null && t !== void 0 ? t : o, X = gi(n, B, _);
  return A.jsx(X, { options: { ...W, enumOptions: j }, schema: n, uiSchema: i, id: s.$id, name: o, label: Q, hideLabel: !q, hideError: $, value: l, onChange: g, onBlur: h, onFocus: y, required: u, disabled: c, readonly: f, formContext: b, autofocus: p, registry: v, placeholder: O, rawErrors: C });
}
function rW(e) {
  const { formData: t, onChange: n } = e;
  return z.useEffect(() => {
    t === void 0 && n(null);
  }, [t, n]), null;
}
function oW() {
  return {
    AnyOfField: m2,
    ArrayField: LU,
    // ArrayField falls back to SchemaField if ArraySchemaField is not defined, which it isn't by default
    BooleanField: DU,
    NumberField: UU,
    ObjectField: QV,
    OneOfField: m2,
    SchemaField: tW,
    StringField: nW,
    NullField: rW
  };
}
function iW(e) {
  const { idSchema: t, description: n, registry: o, schema: i, uiSchema: s } = e, l = Ot(s, o.globalUiOptions), { label: u = !0 } = l;
  if (!n || !u)
    return null;
  const c = _t("DescriptionFieldTemplate", o, l);
  return A.jsx(c, { id: es(t), description: n, schema: i, uiSchema: s, registry: o });
}
function aW(e) {
  const { children: t, className: n, disabled: o, hasToolbar: i, hasMoveDown: s, hasMoveUp: l, hasRemove: u, hasCopy: c, index: f, onCopyIndexClick: p, onDropIndexClick: g, onReorderClick: h, readonly: y, registry: v, uiSchema: C } = e, { CopyButton: $, MoveDownButton: S, MoveUpButton: w, RemoveButton: _ } = v.templates.ButtonTemplates, b = {
    flex: 1,
    paddingLeft: 6,
    paddingRight: 6,
    fontWeight: "bold"
  };
  return A.jsxs("div", { className: n, children: [A.jsx("div", { className: i ? "col-xs-9" : "col-xs-12", children: t }), i && A.jsx("div", { className: "col-xs-3 array-item-toolbox", children: A.jsxs("div", { className: "btn-group", style: {
    display: "flex",
    justifyContent: "space-around"
  }, children: [(l || s) && A.jsx(w, { style: b, disabled: o || y || !l, onClick: h(f, f - 1), uiSchema: C, registry: v }), (l || s) && A.jsx(S, { style: b, disabled: o || y || !s, onClick: h(f, f + 1), uiSchema: C, registry: v }), c && A.jsx($, { style: b, disabled: o || y, onClick: p(f), uiSchema: C, registry: v }), u && A.jsx(_, { style: b, disabled: o || y, onClick: g(f), uiSchema: C, registry: v })] }) })] });
}
function sW(e) {
  const { canAdd: t, className: n, disabled: o, idSchema: i, uiSchema: s, items: l, onAddClick: u, readonly: c, registry: f, required: p, schema: g, title: h } = e, y = Ot(s), v = _t("ArrayFieldDescriptionTemplate", f, y), C = _t("ArrayFieldItemTemplate", f, y), $ = _t("ArrayFieldTitleTemplate", f, y), { ButtonTemplates: { AddButton: S } } = f.templates;
  return A.jsxs("fieldset", { className: n, id: i.$id, children: [A.jsx($, { idSchema: i, title: y.title || h, required: p, schema: g, uiSchema: s, registry: f }), A.jsx(v, { idSchema: i, description: y.description || g.description, schema: g, uiSchema: s, registry: f }), A.jsx("div", { className: "row array-item-list", children: l && l.map(({ key: w, ..._ }) => A.jsx(C, { ..._ }, w)) }), t && A.jsx(S, { className: "array-item-add", onClick: u, disabled: o || c, uiSchema: s, registry: f })] });
}
function lW(e) {
  const { idSchema: t, title: n, schema: o, uiSchema: i, required: s, registry: l } = e, u = Ot(i, l.globalUiOptions), { label: c = !0 } = u;
  if (!n || !c)
    return null;
  const f = _t("TitleFieldTemplate", l, u);
  return A.jsx(f, { id: Qy(t), title: n, required: s, schema: o, uiSchema: i, registry: l });
}
function uW(e) {
  const {
    id: t,
    name: n,
    // remove this from ...rest
    value: o,
    readonly: i,
    disabled: s,
    autofocus: l,
    onBlur: u,
    onFocus: c,
    onChange: f,
    onChangeOverride: p,
    options: g,
    schema: h,
    uiSchema: y,
    formContext: v,
    registry: C,
    rawErrors: $,
    type: S,
    hideLabel: w,
    // remove this from ...rest
    hideError: _,
    // remove this from ...rest
    ...b
  } = e;
  if (!t)
    throw console.log("No id for", e), new Error(`no id for props ${JSON.stringify(e)}`);
  const k = {
    ...b,
    ...ux(h, S, g)
  };
  let E;
  k.type === "number" || k.type === "integer" ? E = o || o === 0 ? o : "" : E = o ?? "";
  const j = z.useCallback(({ target: { value: O } }) => f(O === "" ? g.emptyValue : O), [f, g]), L = z.useCallback(({ target: O }) => u(t, O && O.value), [u, t]), B = z.useCallback(({ target: O }) => c(t, O && O.value), [c, t]);
  return A.jsxs(A.Fragment, { children: [A.jsx("input", { id: t, name: t, className: "form-control", readOnly: i, disabled: s, autoFocus: l, value: E, ...k, list: h.examples ? oc(t) : void 0, onChange: p || j, onBlur: L, onFocus: B, "aria-describedby": Gr(t, !!h.examples) }), Array.isArray(h.examples) && A.jsx("datalist", { id: oc(t), children: h.examples.concat(h.default && !h.examples.includes(h.default) ? [h.default] : []).map((O) => A.jsx("option", { value: O }, O)) }, `datalist_${t}`)] });
}
function cW({ uiSchema: e }) {
  const { submitText: t, norender: n, props: o = {} } = cx(e);
  return n ? null : A.jsx("div", { children: A.jsx("button", { type: "submit", ...o, className: `btn btn-info ${o.className || ""}`, children: t }) });
}
function _c(e) {
  const { iconType: t = "default", icon: n, className: o, uiSchema: i, registry: s, ...l } = e;
  return A.jsx("button", { type: "button", className: `btn btn-${t} ${o}`, ...l, children: A.jsx("i", { className: `glyphicon glyphicon-${n}` }) });
}
function dW(e) {
  const { registry: { translateString: t } } = e;
  return A.jsx(_c, { title: t(jt.CopyButton), className: "array-item-copy", ...e, icon: "copy" });
}
function fW(e) {
  const { registry: { translateString: t } } = e;
  return A.jsx(_c, { title: t(jt.MoveDownButton), className: "array-item-move-down", ...e, icon: "arrow-down" });
}
function pW(e) {
  const { registry: { translateString: t } } = e;
  return A.jsx(_c, { title: t(jt.MoveUpButton), className: "array-item-move-up", ...e, icon: "arrow-up" });
}
function hW(e) {
  const { registry: { translateString: t } } = e;
  return A.jsx(_c, { title: t(jt.RemoveButton), className: "array-item-remove", ...e, iconType: "danger", icon: "remove" });
}
function mW({ className: e, onClick: t, disabled: n, registry: o }) {
  const { translateString: i } = o;
  return A.jsx("div", { className: "row", children: A.jsx("p", { className: `col-xs-3 col-xs-offset-9 text-right ${e}`, children: A.jsx(_c, { iconType: "info", icon: "plus", className: "btn-add col-xs-12", title: i(jt.AddButton), onClick: t, disabled: n, registry: o }) }) });
}
function gW() {
  return {
    SubmitButton: cW,
    AddButton: mW,
    CopyButton: dW,
    MoveDownButton: fW,
    MoveUpButton: pW,
    RemoveButton: hW
  };
}
function yW(e) {
  const { id: t, description: n } = e;
  return n ? typeof n == "string" ? A.jsx("p", { id: t, className: "field-description", children: n }) : A.jsx("div", { id: t, className: "field-description", children: n }) : null;
}
function vW({ errors: e, registry: t }) {
  const { translateString: n } = t;
  return A.jsxs("div", { className: "panel panel-danger errors", children: [A.jsx("div", { className: "panel-heading", children: A.jsx("h3", { className: "panel-title", children: n(jt.ErrorsLabel) }) }), A.jsx("ul", { className: "list-group", children: e.map((o, i) => A.jsx("li", { className: "list-group-item text-danger", children: o.stack }, i)) })] });
}
const bW = "*";
function Ix(e) {
  const { label: t, required: n, id: o } = e;
  return t ? A.jsxs("label", { className: "control-label", htmlFor: o, children: [t, n && A.jsx("span", { className: "required", children: bW })] }) : null;
}
function SW(e) {
  const { id: t, label: n, children: o, errors: i, help: s, description: l, hidden: u, required: c, displayLabel: f, registry: p, uiSchema: g } = e, h = Ot(g), y = _t("WrapIfAdditionalTemplate", p, h);
  return u ? A.jsx("div", { className: "hidden", children: o }) : A.jsxs(y, { ...e, children: [f && A.jsx(Ix, { label: n, required: c, id: t }), f && l ? l : null, o, i, s] });
}
function wW(e) {
  const { errors: t = [], idSchema: n } = e;
  if (t.length === 0)
    return null;
  const o = Jy(n);
  return A.jsx("div", { children: A.jsx("ul", { id: o, className: "error-detail bs-callout bs-callout-info", children: t.filter((i) => !!i).map((i, s) => A.jsx("li", { className: "text-danger", children: i }, s)) }) });
}
function $W(e) {
  const { idSchema: t, help: n } = e;
  if (!n)
    return null;
  const o = Yy(t);
  return typeof n == "string" ? A.jsx("p", { id: o, className: "help-block", children: n }) : A.jsx("div", { id: o, className: "help-block", children: n });
}
function CW(e) {
  const { description: t, disabled: n, formData: o, idSchema: i, onAddClick: s, properties: l, readonly: u, registry: c, required: f, schema: p, title: g, uiSchema: h } = e, y = Ot(h), v = _t("TitleFieldTemplate", c, y), C = _t("DescriptionFieldTemplate", c, y), { ButtonTemplates: { AddButton: $ } } = c.templates;
  return A.jsxs("fieldset", { id: i.$id, children: [g && A.jsx(v, { id: Qy(i), title: g, required: f, schema: p, uiSchema: h, registry: c }), t && A.jsx(C, { id: es(i), description: t, schema: p, uiSchema: h, registry: c }), l.map((S) => S.content), m$(p, h, o) && A.jsx($, { className: "object-property-expand", onClick: s(p), disabled: n || u, uiSchema: h, registry: c })] });
}
const xW = "*";
function EW(e) {
  const { id: t, title: n, required: o } = e;
  return A.jsxs("legend", { id: t, children: [n, o && A.jsx("span", { className: "required", children: xW })] });
}
function _W(e) {
  const { schema: t, idSchema: n, reason: o, registry: i } = e, { translateString: s } = i;
  let l = jt.UnsupportedField;
  const u = [];
  return n && n.$id && (l = jt.UnsupportedFieldWithId, u.push(n.$id)), o && (l = l === jt.UnsupportedField ? jt.UnsupportedFieldWithReason : jt.UnsupportedFieldWithIdAndReason, u.push(o)), A.jsxs("div", { className: "unsupported-field", children: [A.jsx("p", { children: A.jsx(wp, { options: { disableParsingRawHTML: !0 }, children: s(l, u) }) }), t && A.jsx("pre", { children: JSON.stringify(t, null, 2) })] });
}
function kW(e) {
  const { id: t, classNames: n, style: o, disabled: i, label: s, onKeyChange: l, onDropPropertyClick: u, readonly: c, required: f, schema: p, children: g, uiSchema: h, registry: y } = e, { templates: v, translateString: C } = y, { RemoveButton: $ } = v.ButtonTemplates, S = C(jt.KeyLabel, [s]);
  return gl in p ? A.jsx("div", { className: n, style: o, children: A.jsxs("div", { className: "row", children: [A.jsx("div", { className: "col-xs-5 form-additional", children: A.jsxs("div", { className: "form-group", children: [A.jsx(Ix, { label: S, required: f, id: `${t}-key` }), A.jsx("input", { className: "form-control", type: "text", id: `${t}-key`, onBlur: ({ target: _ }) => l(_ && _.value), defaultValue: s })] }) }), A.jsx("div", { className: "form-additional form-group col-xs-5", children: g }), A.jsx("div", { className: "col-xs-2", children: A.jsx($, { className: "array-item-remove btn-block", style: { border: "0" }, disabled: i || c, onClick: u(s), uiSchema: h, registry: y }) })] }) }) : A.jsx("div", { className: n, style: o, children: g });
}
function PW() {
  return {
    ArrayFieldDescriptionTemplate: iW,
    ArrayFieldItemTemplate: aW,
    ArrayFieldTemplate: sW,
    ArrayFieldTitleTemplate: lW,
    ButtonTemplates: gW(),
    BaseInputTemplate: uW,
    DescriptionFieldTemplate: yW,
    ErrorListTemplate: vW,
    FieldTemplate: SW,
    FieldErrorTemplate: wW,
    FieldHelpTemplate: $W,
    ObjectFieldTemplate: CW,
    TitleFieldTemplate: EW,
    UnsupportedFieldTemplate: _W,
    WrapIfAdditionalTemplate: kW
  };
}
function OW(e) {
  return Object.values(e).every((t) => t !== -1);
}
function AW({ type: e, range: t, value: n, select: o, rootId: i, name: s, disabled: l, readonly: u, autofocus: c, registry: f, onBlur: p, onFocus: g }) {
  const h = i + "_" + e, { SelectWidget: y } = f.widgets;
  return A.jsx(y, { schema: { type: "integer" }, id: h, name: s, className: "form-control", options: { enumOptions: ox(t[0], t[1]) }, placeholder: e, value: n, disabled: l, readonly: u, autofocus: c, onChange: (v) => o(e, v), onBlur: p, onFocus: g, registry: f, label: "", "aria-describedby": Gr(i) });
}
function IW({ time: e = !1, disabled: t = !1, readonly: n = !1, autofocus: o = !1, options: i, id: s, name: l, registry: u, onBlur: c, onFocus: f, onChange: p, value: g }) {
  const { translateString: h } = u, [y, v] = z.useState(g), [C, $] = z.useReducer((b, k) => ({ ...b, ...k }), Um(g, e));
  z.useEffect(() => {
    const b = f2(C, e);
    OW(C) && b !== g ? p(b) : y !== g && (v(g), $(Um(g, e)));
  }, [e, g, p, C, y]);
  const S = z.useCallback((b, k) => {
    $({ [b]: k });
  }, []), w = z.useCallback((b) => {
    if (b.preventDefault(), t || n)
      return;
    const k = Um((/* @__PURE__ */ new Date()).toJSON(), e);
    p(f2(k, e));
  }, [t, n, e]), _ = z.useCallback((b) => {
    b.preventDefault(), !(t || n) && p(void 0);
  }, [t, n, p]);
  return A.jsxs("ul", { className: "list-inline", children: [eU(C, e, i.yearsRange, i.format).map((b, k) => A.jsx("li", { className: "list-inline-item", children: A.jsx(AW, { rootId: s, name: l, select: S, ...b, disabled: t, readonly: n, registry: u, onBlur: c, onFocus: f, autofocus: o && k === 0 }) }, k)), (i.hideNowButton !== "undefined" ? !i.hideNowButton : !0) && A.jsx("li", { className: "list-inline-item", children: A.jsx("a", { href: "#", className: "btn btn-info btn-now", onClick: w, children: h(jt.NowLabel) }) }), (i.hideClearButton !== "undefined" ? !i.hideClearButton : !0) && A.jsx("li", { className: "list-inline-item", children: A.jsx("a", { href: "#", className: "btn btn-warning btn-clear", onClick: _, children: h(jt.ClearLabel) }) })] });
}
function jW({ time: e = !0, ...t }) {
  const { AltDateWidget: n } = t.registry.widgets;
  return A.jsx(n, { time: e, ...t });
}
function RW({ schema: e, uiSchema: t, options: n, id: o, value: i, disabled: s, readonly: l, label: u, hideLabel: c, autofocus: f = !1, onBlur: p, onFocus: g, onChange: h, registry: y }) {
  var v;
  const C = _t("DescriptionFieldTemplate", y, n), $ = Uu(e), S = z.useCallback((k) => h(k.target.checked), [h]), w = z.useCallback((k) => p(o, k.target.checked), [p, o]), _ = z.useCallback((k) => g(o, k.target.checked), [g, o]), b = (v = n.description) !== null && v !== void 0 ? v : e.description;
  return A.jsxs("div", { className: `checkbox ${s || l ? "disabled" : ""}`, children: [!c && !!b && A.jsx(C, { id: es(o), description: b, schema: e, uiSchema: t, registry: y }), A.jsxs("label", { children: [A.jsx("input", { type: "checkbox", id: o, name: o, checked: typeof i > "u" ? !1 : i, required: $, disabled: s || l, autoFocus: f, onChange: S, onBlur: w, onFocus: _, "aria-describedby": Gr(o) }), ts(A.jsx("span", { children: u }), c)] })] });
}
function TW({ id: e, disabled: t, options: { inline: n = !1, enumOptions: o, enumDisabled: i, emptyValue: s }, value: l, autofocus: u = !1, readonly: c, onChange: f, onBlur: p, onFocus: g }) {
  const h = Array.isArray(l) ? l : [l], y = z.useCallback(({ target: C }) => p(e, qn(C && C.value, o, s)), [p, e]), v = z.useCallback(({ target: C }) => g(e, qn(C && C.value, o, s)), [g, e]);
  return A.jsx("div", { className: "checkboxes", id: e, children: Array.isArray(o) && o.map((C, $) => {
    const S = vp(C.value, h), w = Array.isArray(i) && i.indexOf(C.value) !== -1, _ = t || w || c ? "disabled" : "", b = (E) => {
      E.target.checked ? f(sx($, h, o)) : f(ix($, h, o));
    }, k = A.jsxs("span", { children: [A.jsx("input", { type: "checkbox", id: bp(e, $), name: e, checked: S, value: String($), disabled: t || w || c, autoFocus: u && $ === 0, onChange: b, onBlur: y, onFocus: v, "aria-describedby": Gr(e) }), A.jsx("span", { children: C.label })] });
    return n ? A.jsx("label", { className: `checkbox-inline ${_}`, children: k }, $) : A.jsx("div", { className: `checkbox ${_}`, children: A.jsx("label", { children: k }) }, $);
  }) });
}
function MW(e) {
  const { disabled: t, readonly: n, options: o, registry: i } = e, s = _t("BaseInputTemplate", i, o);
  return A.jsx(s, { type: "color", ...e, disabled: t || n });
}
function NW(e) {
  const { onChange: t, options: n, registry: o } = e, i = _t("BaseInputTemplate", o, n), s = z.useCallback((l) => t(l || void 0), [t]);
  return A.jsx(i, { type: "date", ...e, onChange: s });
}
function FW(e) {
  const { onChange: t, value: n, options: o, registry: i } = e, s = _t("BaseInputTemplate", i, o);
  return A.jsx(s, { type: "datetime-local", ...e, value: wU(n), onChange: (l) => t(sU(l)) });
}
function LW(e) {
  const { options: t, registry: n } = e, o = _t("BaseInputTemplate", n, t);
  return A.jsx(o, { type: "email", ...e });
}
function DW(e, t) {
  return e === null ? null : e.replace(";base64", `;name=${encodeURIComponent(t)};base64`);
}
function zW(e) {
  const { name: t, size: n, type: o } = e;
  return new Promise((i, s) => {
    const l = new window.FileReader();
    l.onerror = s, l.onload = (u) => {
      var c;
      typeof ((c = u.target) === null || c === void 0 ? void 0 : c.result) == "string" ? i({
        dataURL: DW(u.target.result, t),
        name: t,
        size: n,
        type: o
      }) : i({
        dataURL: null,
        name: t,
        size: n,
        type: o
      });
    }, l.readAsDataURL(e);
  });
}
function BW(e) {
  return Promise.all(Array.from(e).map(zW));
}
function UW({ fileInfo: e, registry: t }) {
  const { translateString: n } = t, { dataURL: o, type: i, name: s } = e;
  return o ? ["image/jpeg", "image/png"].includes(i) ? A.jsx("img", { src: o, style: { maxWidth: "100%" }, className: "file-preview" }) : A.jsxs(A.Fragment, { children: [" ", A.jsx("a", { download: `preview-${s}`, href: o, className: "file-download", children: n(jt.PreviewLabel) })] }) : null;
}
function VW({ filesInfo: e, registry: t, preview: n, onRemove: o, options: i }) {
  if (e.length === 0)
    return null;
  const { translateString: s } = t, { RemoveButton: l } = _t("ButtonTemplates", t, i);
  return A.jsx("ul", { className: "file-info", children: e.map((u, c) => {
    const { name: f, size: p, type: g } = u, h = () => o(c);
    return A.jsxs("li", { children: [A.jsx(wp, { children: s(jt.FilesInfo, [f, g, String(p)]) }), n && A.jsx(UW, { fileInfo: u, registry: t }), A.jsx(l, { onClick: h, registry: t })] }, c);
  }) });
}
function WW(e) {
  return e.reduce((t, n) => {
    if (!n)
      return t;
    try {
      const { blob: o, name: i } = W7(n);
      return [
        ...t,
        {
          dataURL: n,
          name: i,
          size: o.size,
          type: o.type
        }
      ];
    } catch {
      return t;
    }
  }, []);
}
function qW(e) {
  const { disabled: t, readonly: n, required: o, multiple: i, onChange: s, value: l, options: u, registry: c } = e, f = _t("BaseInputTemplate", c, u), p = z.useCallback((y) => {
    y.target.files && BW(y.target.files).then((v) => {
      const C = v.map(($) => $.dataURL);
      s(i ? l.concat(C) : C[0]);
    });
  }, [i, l, s]), g = z.useMemo(() => WW(Array.isArray(l) ? l : [l]), [l]), h = z.useCallback((y) => {
    if (i) {
      const v = l.filter((C, $) => $ !== y);
      s(v);
    } else
      s(void 0);
  }, [i, l, s]);
  return A.jsxs("div", { children: [A.jsx(f, { ...e, disabled: t || n, type: "file", required: l ? !1 : o, onChangeOverride: p, value: "", accept: u.accept ? String(u.accept) : void 0 }), A.jsx(VW, { filesInfo: g, onRemove: h, registry: c, preview: u.filePreview, options: u })] });
}
function KW({ id: e, value: t }) {
  return A.jsx("input", { type: "hidden", id: e, name: e, value: typeof t > "u" ? "" : t });
}
function HW(e) {
  const { options: t, registry: n } = e, o = _t("BaseInputTemplate", n, t);
  return A.jsx(o, { type: "password", ...e });
}
function GW({ options: e, value: t, required: n, disabled: o, readonly: i, autofocus: s = !1, onBlur: l, onFocus: u, onChange: c, id: f }) {
  const { enumOptions: p, enumDisabled: g, inline: h, emptyValue: y } = e, v = z.useCallback(({ target: $ }) => l(f, qn($ && $.value, p, y)), [l, f]), C = z.useCallback(({ target: $ }) => u(f, qn($ && $.value, p, y)), [u, f]);
  return A.jsx("div", { className: "field-radio-group", id: f, children: Array.isArray(p) && p.map(($, S) => {
    const w = vp($.value, t), _ = Array.isArray(g) && g.indexOf($.value) !== -1, b = o || _ || i ? "disabled" : "", k = () => c($.value), E = A.jsxs("span", { children: [A.jsx("input", { type: "radio", id: bp(f, S), checked: w, name: f, required: n, value: String(S), disabled: o || _ || i, autoFocus: s && S === 0, onChange: k, onBlur: v, onFocus: C, "aria-describedby": Gr(f) }), A.jsx("span", { children: $.label })] });
    return h ? A.jsx("label", { className: `radio-inline ${b}`, children: E }, S) : A.jsx("div", { className: `radio ${b}`, children: A.jsx("label", { children: E }) }, S);
  }) });
}
function JW(e) {
  const { value: t, registry: { templates: { BaseInputTemplate: n } } } = e;
  return A.jsxs("div", { className: "field-range-wrapper", children: [A.jsx(n, { type: "range", ...e }), A.jsx("span", { className: "range-view", children: t })] });
}
function Gm(e, t) {
  return t ? Array.from(e.target.options).slice().filter((n) => n.selected).map((n) => n.value) : e.target.value;
}
function YW({ schema: e, id: t, options: n, value: o, required: i, disabled: s, readonly: l, multiple: u = !1, autofocus: c = !1, onChange: f, onBlur: p, onFocus: g, placeholder: h }) {
  const { enumOptions: y, enumDisabled: v, emptyValue: C } = n, $ = u ? [] : "", S = z.useCallback((E) => {
    const j = Gm(E, u);
    return g(t, qn(j, y, C));
  }, [g, t, e, u, y, C]), w = z.useCallback((E) => {
    const j = Gm(E, u);
    return p(t, qn(j, y, C));
  }, [p, t, e, u, y, C]), _ = z.useCallback((E) => {
    const j = Gm(E, u);
    return f(qn(j, y, C));
  }, [f, e, u, y, C]), b = Gy(o, y, u), k = !u && e.default === void 0;
  return A.jsxs("select", { id: t, name: t, multiple: u, className: "form-control", value: typeof b > "u" ? $ : b, required: i, disabled: s || l, autoFocus: c, onBlur: w, onFocus: S, onChange: _, "aria-describedby": Gr(t), children: [k && A.jsx("option", { value: "", children: h }), Array.isArray(y) && y.map(({ value: E, label: j }, L) => {
    const B = v && v.indexOf(E) !== -1;
    return A.jsx("option", { value: String(L), disabled: B, children: j }, L);
  })] });
}
function jx({ id: e, options: t = {}, placeholder: n, value: o, required: i, disabled: s, readonly: l, autofocus: u = !1, onChange: c, onBlur: f, onFocus: p }) {
  const g = z.useCallback(({ target: { value: v } }) => c(v === "" ? t.emptyValue : v), [c, t.emptyValue]), h = z.useCallback(({ target: v }) => f(e, v && v.value), [f, e]), y = z.useCallback(({ target: v }) => p(e, v && v.value), [e, p]);
  return A.jsx("textarea", { id: e, name: e, className: "form-control", value: o || "", placeholder: n, required: i, disabled: s, readOnly: l, autoFocus: u, rows: t.rows, onBlur: h, onFocus: y, onChange: g, "aria-describedby": Gr(e) });
}
jx.defaultProps = {
  autofocus: !1,
  options: {}
};
function QW(e) {
  const { options: t, registry: n } = e, o = _t("BaseInputTemplate", n, t);
  return A.jsx(o, { ...e });
}
function XW(e) {
  const { onChange: t, options: n, registry: o } = e, i = _t("BaseInputTemplate", o, n), s = z.useCallback((l) => t(l ? `${l}:00` : void 0), [t]);
  return A.jsx(i, { type: "time", ...e, onChange: s });
}
function ZW(e) {
  const { options: t, registry: n } = e, o = _t("BaseInputTemplate", n, t);
  return A.jsx(o, { type: "url", ...e });
}
function eq(e) {
  const { options: t, registry: n } = e, o = _t("BaseInputTemplate", n, t);
  return A.jsx(o, { type: "number", ...e });
}
function tq() {
  return {
    AltDateWidget: IW,
    AltDateTimeWidget: jW,
    CheckboxWidget: RW,
    CheckboxesWidget: TW,
    ColorWidget: MW,
    DateWidget: NW,
    DateTimeWidget: FW,
    EmailWidget: LW,
    FileWidget: qW,
    HiddenWidget: KW,
    PasswordWidget: HW,
    RadioWidget: GW,
    RangeWidget: JW,
    SelectWidget: YW,
    TextWidget: QW,
    TextareaWidget: jx,
    TimeWidget: XW,
    UpDownWidget: eq,
    URLWidget: ZW
  };
}
function Rx() {
  return {
    fields: oW(),
    templates: PW(),
    widgets: tq(),
    rootSchema: {},
    formContext: {},
    translateString: K7
  };
}
class nq extends z.Component {
  /** Constructs the `Form` from the `props`. Will setup the initial state from the props. It will also call the
   * `onChange` handler if the initially provided `formData` is modified to add missing default values as part of the
   * state construction.
   *
   * @param props - The initial props for the `Form`
   */
  constructor(t) {
    if (super(t), this.getUsedFormData = (n, o) => {
      if (o.length === 0 && typeof n != "object")
        return n;
      const i = p2(n, o);
      return Array.isArray(n) ? Object.keys(i).map((s) => i[s]) : i;
    }, this.getFieldNames = (n, o) => {
      const i = (s, l = [], u = [[]]) => (Object.keys(s).forEach((c) => {
        if (typeof s[c] == "object") {
          const f = u.map((p) => [...p, c]);
          s[c][sy] && s[c][pf] !== "" ? l.push(s[c][pf]) : i(s[c], l, f);
        } else c === pf && s[c] !== "" && u.forEach((f) => {
          const p = Je(o, f);
          (typeof p != "object" || oa(p) || Array.isArray(p) && p.every((g) => typeof g != "object")) && l.push(f);
        });
      }), l);
      return i(n);
    }, this.omitExtraData = (n) => {
      const { schema: o, schemaUtils: i } = this.state, s = i.retrieveSchema(o, n), l = i.toPathSchema(s, "", n), u = this.getFieldNames(l, n);
      return this.getUsedFormData(n, u);
    }, this.onChange = (n, o, i) => {
      const { extraErrors: s, omitExtraData: l, liveOmit: u, noValidate: c, liveValidate: f, onChange: p } = this.props, { schemaUtils: g, schema: h, retrievedSchema: y } = this.state;
      (tn(n) || Array.isArray(n)) && (n = this.getStateFromProps(this.props, n, y).formData);
      const v = !c && f;
      let C = { formData: n, schema: h }, $ = n;
      if (l === !0 && u === !0 && ($ = this.omitExtraData(n), C = {
        formData: $
      }), v) {
        const S = this.validate($, h, g, y);
        let w = S.errors, _ = S.errorSchema;
        const b = w, k = _;
        if (s) {
          const E = yf(S, s);
          _ = E.errorSchema, w = E.errors;
        }
        if (o) {
          const E = this.filterErrorsBasedOnSchema(o, y, $);
          _ = Zi(_, E, "preventDuplicates");
        }
        C = {
          formData: $,
          errors: w,
          errorSchema: _,
          schemaValidationErrors: b,
          schemaValidationErrorSchema: k
        };
      } else if (!c && o) {
        const S = s ? Zi(o, s, "preventDuplicates") : o;
        C = {
          formData: $,
          errorSchema: S,
          errors: ic(S)
        };
      }
      this.setState(C, () => p && p({ ...this.state, ...C }, i));
    }, this.reset = () => {
      const { onChange: n } = this.props, s = {
        formData: this.getStateFromProps(this.props, void 0).formData,
        errorSchema: {},
        errors: [],
        schemaValidationErrors: [],
        schemaValidationErrorSchema: {}
      };
      this.setState(s, () => n && n({ ...this.state, ...s }));
    }, this.onBlur = (n, o) => {
      const { onBlur: i } = this.props;
      i && i(n, o);
    }, this.onFocus = (n, o) => {
      const { onFocus: i } = this.props;
      i && i(n, o);
    }, this.onSubmit = (n) => {
      if (n.preventDefault(), n.target !== n.currentTarget)
        return;
      n.persist();
      const { omitExtraData: o, extraErrors: i, noValidate: s, onSubmit: l } = this.props;
      let { formData: u } = this.state;
      if (o === !0 && (u = this.omitExtraData(u)), s || this.validateFormWithFormData(u)) {
        const c = i || {}, f = i ? ic(i) : [];
        this.setState({
          formData: u,
          errors: f,
          errorSchema: c,
          schemaValidationErrors: [],
          schemaValidationErrorSchema: {}
        }, () => {
          l && l({ ...this.state, formData: u, status: "submitted" }, n);
        });
      }
    }, this.submit = () => {
      if (this.formElement.current) {
        const n = new CustomEvent("submit", {
          cancelable: !0
        });
        n.preventDefault(), this.formElement.current.dispatchEvent(n), this.formElement.current.requestSubmit();
      }
    }, this.validateFormWithFormData = (n) => {
      const { extraErrors: o, extraErrorsBlockSubmit: i, focusOnFirstError: s, onError: l } = this.props, { errors: u } = this.state, c = this.validate(n);
      let f = c.errors, p = c.errorSchema;
      const g = f, h = p, y = f.length > 0 || o && i;
      if (y) {
        if (o) {
          const v = yf(c, o);
          p = v.errorSchema, f = v.errors;
        }
        s && (typeof s == "function" ? s(f[0]) : this.focusOnError(f[0])), this.setState({
          errors: f,
          errorSchema: p,
          schemaValidationErrors: g,
          schemaValidationErrorSchema: h
        }, () => {
          l ? l(f) : console.error("Form validation failed", f);
        });
      } else u.length > 0 && this.setState({
        errors: [],
        errorSchema: {},
        schemaValidationErrors: [],
        schemaValidationErrorSchema: {}
      });
      return !y;
    }, !t.validator)
      throw new Error("A validator is required for Form functionality to work");
    this.state = this.getStateFromProps(t, t.formData), this.props.onChange && !Sr(this.state.formData, this.props.formData) && this.props.onChange(this.state), this.formElement = z.createRef();
  }
  /**
   * `getSnapshotBeforeUpdate` is a React lifecycle method that is invoked right before the most recently rendered
   * output is committed to the DOM. It enables your component to capture current values (e.g., scroll position) before
   * they are potentially changed.
   *
   * In this case, it checks if the props have changed since the last render. If they have, it computes the next state
   * of the component using `getStateFromProps` method and returns it along with a `shouldUpdate` flag set to `true` IF
   * the `nextState` and `prevState` are different, otherwise `false`. This ensures that we have the most up-to-date
   * state ready to be applied in `componentDidUpdate`.
   *
   * If `formData` hasn't changed, it simply returns an object with `shouldUpdate` set to `false`, indicating that a
   * state update is not necessary.
   *
   * @param prevProps - The previous set of props before the update.
   * @param prevState - The previous state before the update.
   * @returns Either an object containing the next state and a flag indicating that an update should occur, or an object
   *        with a flag indicating that an update is not necessary.
   */
  getSnapshotBeforeUpdate(t, n) {
    if (!Sr(this.props, t)) {
      const o = !Sr(t.schema, this.props.schema), i = !Sr(t.formData, this.props.formData), s = this.getStateFromProps(
        this.props,
        this.props.formData,
        // If the `schema` has changed, we need to update the retrieved schema.
        // Or if the `formData` changes, for example in the case of a schema with dependencies that need to
        //  match one of the subSchemas, the retrieved schema must be updated.
        o || i ? void 0 : this.state.retrievedSchema,
        o
      ), l = !Sr(s, n);
      return { nextState: s, shouldUpdate: l };
    }
    return { shouldUpdate: !1 };
  }
  /**
   * `componentDidUpdate` is a React lifecycle method that is invoked immediately after updating occurs. This method is
   * not called for the initial render.
   *
   * Here, it checks if an update is necessary based on the `shouldUpdate` flag received from `getSnapshotBeforeUpdate`.
   * If an update is required, it applies the next state and, if needed, triggers the `onChange` handler to inform about
   * changes.
   *
   * This method effectively replaces the deprecated `UNSAFE_componentWillReceiveProps`, providing a safer alternative
   * to handle prop changes and state updates.
   *
   * @param _ - The previous set of props.
   * @param prevState - The previous state of the component before the update.
   * @param snapshot - The value returned from `getSnapshotBeforeUpdate`.
   */
  componentDidUpdate(t, n, o) {
    if (o.shouldUpdate) {
      const { nextState: i } = o;
      !Sr(i.formData, this.props.formData) && !Sr(i.formData, n.formData) && this.props.onChange && this.props.onChange(i), this.setState(i);
    }
  }
  /** Extracts the updated state from the given `props` and `inputFormData`. As part of this process, the
   * `inputFormData` is first processed to add any missing required defaults. After that, the data is run through the
   * validation process IF required by the `props`.
   *
   * @param props - The props passed to the `Form`
   * @param inputFormData - The new or current data for the `Form`
   * @param retrievedSchema - An expanded schema, if not provided, it will be retrieved from the `schema` and `formData`.
   * @param isSchemaChanged - A flag indicating whether the schema has changed.
   * @returns - The new state for the `Form`
   */
  getStateFromProps(t, n, o, i = !1) {
    var s;
    const l = this.state || {}, u = "schema" in t ? t.schema : this.props.schema, c = ("uiSchema" in t ? t.uiSchema : this.props.uiSchema) || {}, f = typeof n < "u", p = "liveValidate" in t ? t.liveValidate : this.props.liveValidate, g = f && !t.noValidate && p, h = u, y = "experimental_defaultFormStateBehavior" in t ? t.experimental_defaultFormStateBehavior : this.props.experimental_defaultFormStateBehavior, v = "experimental_customMergeAllOf" in t ? t.experimental_customMergeAllOf : this.props.experimental_customMergeAllOf;
    let C = l.schemaUtils;
    (!C || C.doesSchemaUtilsDiffer(t.validator, h, y, v)) && (C = V7(t.validator, h, y, v));
    const $ = C.getDefaultFormState(u, n), S = o ?? C.retrieveSchema(u, $), w = () => t.noValidate || i ? { errors: [], errorSchema: {} } : t.liveValidate ? {
      errors: l.errors || [],
      errorSchema: l.errorSchema || {}
    } : {
      errors: l.schemaValidationErrors || [],
      errorSchema: l.schemaValidationErrorSchema || {}
    };
    let _, b, k = l.schemaValidationErrors, E = l.schemaValidationErrorSchema;
    if (g) {
      const B = this.validate($, u, C, S);
      _ = B.errors, o === void 0 ? b = B.errorSchema : b = Zi((s = this.state) === null || s === void 0 ? void 0 : s.errorSchema, B.errorSchema, "preventDuplicates"), k = _, E = b;
    } else {
      const B = w();
      _ = B.errors, b = B.errorSchema;
    }
    if (t.extraErrors) {
      const B = yf({ errorSchema: b, errors: _ }, t.extraErrors);
      b = B.errorSchema, _ = B.errors;
    }
    const j = C.toIdSchema(S, c["ui:rootFieldId"], $, t.idPrefix, t.idSeparator);
    return {
      schemaUtils: C,
      schema: u,
      uiSchema: c,
      idSchema: j,
      formData: $,
      edit: f,
      errors: _,
      errorSchema: b,
      schemaValidationErrors: k,
      schemaValidationErrorSchema: E,
      retrievedSchema: S
    };
  }
  /** React lifecycle method that is used to determine whether component should be updated.
   *
   * @param nextProps - The next version of the props
   * @param nextState - The next version of the state
   * @returns - True if the component should be updated, false otherwise
   */
  shouldComponentUpdate(t, n) {
    return cU(this, t, n);
  }
  /** Validates the `formData` against the `schema` using the `altSchemaUtils` (if provided otherwise it uses the
   * `schemaUtils` in the state), returning the results.
   *
   * @param formData - The new form data to validate
   * @param schema - The schema used to validate against
   * @param altSchemaUtils - The alternate schemaUtils to use for validation
   */
  validate(t, n = this.props.schema, o, i) {
    const s = o || this.state.schemaUtils, { customValidate: l, transformErrors: u, uiSchema: c } = this.props, f = i ?? s.retrieveSchema(n, t);
    return s.getValidator().validateFormData(t, f, l, u, c);
  }
  /** Renders any errors contained in the `state` in using the `ErrorList`, if not disabled by `showErrorList`. */
  renderErrors(t) {
    const { errors: n, errorSchema: o, schema: i, uiSchema: s } = this.state, { formContext: l } = this.props, u = Ot(s), c = _t("ErrorListTemplate", t, u);
    return n && n.length ? A.jsx(c, { errors: n, errorSchema: o || {}, schema: i, uiSchema: s, formContext: l, registry: t }) : null;
  }
  // Filtering errors based on your retrieved schema to only show errors for properties in the selected branch.
  filterErrorsBasedOnSchema(t, n, o) {
    const { retrievedSchema: i, schemaUtils: s } = this.state, l = n ?? i, u = s.toPathSchema(l, "", o), c = this.getFieldNames(u, o), f = p2(t, c);
    (n == null ? void 0 : n.type) !== "object" && (n == null ? void 0 : n.type) !== "array" && (f.__errors = t.__errors);
    const p = (g) => (aB(g, (h, y) => {
      ax(h) ? delete g[y] : typeof h == "object" && !Array.isArray(h.__errors) && p(h);
    }), g);
    return p(f);
  }
  /** Returns the registry for the form */
  getRegistry() {
    var t;
    const { translateString: n, uiSchema: o = {} } = this.props, { schemaUtils: i } = this.state, { fields: s, templates: l, widgets: u, formContext: c, translateString: f } = Rx();
    return {
      fields: { ...s, ...this.props.fields },
      templates: {
        ...l,
        ...this.props.templates,
        ButtonTemplates: {
          ...l.ButtonTemplates,
          ...(t = this.props.templates) === null || t === void 0 ? void 0 : t.ButtonTemplates
        }
      },
      widgets: { ...u, ...this.props.widgets },
      rootSchema: this.props.schema,
      formContext: this.props.formContext || c,
      schemaUtils: i,
      translateString: n || f,
      globalUiOptions: o[Y5]
    };
  }
  /** Attempts to focus on the field associated with the `error`. Uses the `property` field to compute path of the error
   * field, then, using the `idPrefix` and `idSeparator` converts that path into an id. Then the input element with that
   * id is attempted to be found using the `formElement` ref. If it is located, then it is focused.
   *
   * @param error - The error on which to focus
   */
  focusOnError(t) {
    const { idPrefix: n = "root", idSeparator: o = "_" } = this.props, { property: i } = t, s = fx(i);
    s[0] === "" ? s[0] = n : s.unshift(n);
    const l = s.join(o);
    let u = this.formElement.current.elements[l];
    u || (u = this.formElement.current.querySelector(`input[id^="${l}"`)), u && u.length && (u = u[0]), u && u.focus();
  }
  /** Programmatically validate the form.  If `omitExtraData` is true, the `formData` will first be filtered to remove
   * any extra data not in a form field. If `onError` is provided, then it will be called with the list of errors the
   * same way as would happen on form submission.
   *
   * @returns - True if the form is valid, false otherwise.
   */
  validateForm() {
    const { omitExtraData: t } = this.props;
    let { formData: n } = this.state;
    return t === !0 && (n = this.omitExtraData(n)), this.validateFormWithFormData(n);
  }
  /** Renders the `Form` fields inside the <form> | `tagName` or `_internalFormWrapper`, rendering any errors if
   * needed along with the submit button or any children of the form.
   */
  render() {
    const { children: t, id: n, idPrefix: o, idSeparator: i, className: s = "", tagName: l, name: u, method: c, target: f, action: p, autoComplete: g, enctype: h, acceptcharset: y, acceptCharset: v, noHtml5Validate: C = !1, disabled: $, readonly: S, formContext: w, showErrorList: _ = "top", _internalFormWrapper: b } = this.props, { schema: k, uiSchema: E, formData: j, errorSchema: L, idSchema: B } = this.state, O = this.getRegistry(), { SchemaField: V } = O.fields, { SubmitButton: W } = O.templates.ButtonTemplates, q = b ? l : void 0, Q = b || l || "form";
    let { [Pf]: X = {} } = Ot(E);
    $ && (X = { ...X, props: { ...X.props, disabled: !0 } });
    const ce = { [Bu]: { [Pf]: X } };
    return A.jsxs(Q, { className: s || "rjsf", id: n, name: u, method: c, target: f, action: p, autoComplete: g, encType: h, acceptCharset: v || y, noValidate: C, onSubmit: this.onSubmit, as: q, ref: this.formElement, children: [_ === "top" && this.renderErrors(O), A.jsx(V, { name: "", schema: k, uiSchema: E, errorSchema: L, idSchema: B, idPrefix: o, idSeparator: i, formContext: w, formData: j, onChange: this.onChange, onBlur: this.onBlur, onFocus: this.onFocus, registry: O, disabled: $, readonly: S }), t || A.jsx(W, { uiSchema: ce, registry: O }), _ === "bottom" && this.renderErrors(O)] });
  }
}
function rq(e) {
  return z.forwardRef(({ fields: t, widgets: n, templates: o, ...i }, s) => {
    var l;
    return t = { ...e == null ? void 0 : e.fields, ...t }, n = { ...e == null ? void 0 : e.widgets, ...n }, o = {
      ...e == null ? void 0 : e.templates,
      ...o,
      ButtonTemplates: {
        ...(l = e == null ? void 0 : e.templates) === null || l === void 0 ? void 0 : l.ButtonTemplates,
        ...o == null ? void 0 : o.ButtonTemplates
      }
    }, A.jsx(nq, { ...e, ...i, fields: t, widgets: n, templates: o, ref: s });
  });
}
var Tx = { exports: {} }, Ht = {};
/**
 * @license React
 * react-is.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var C2;
function oq() {
  if (C2) return Ht;
  C2 = 1;
  var e = Symbol.for("react.transitional.element"), t = Symbol.for("react.portal"), n = Symbol.for("react.fragment"), o = Symbol.for("react.strict_mode"), i = Symbol.for("react.profiler"), s = Symbol.for("react.consumer"), l = Symbol.for("react.context"), u = Symbol.for("react.forward_ref"), c = Symbol.for("react.suspense"), f = Symbol.for("react.suspense_list"), p = Symbol.for("react.memo"), g = Symbol.for("react.lazy"), h = Symbol.for("react.offscreen"), y = Symbol.for("react.client.reference");
  function v(C) {
    if (typeof C == "object" && C !== null) {
      var $ = C.$$typeof;
      switch ($) {
        case e:
          switch (C = C.type, C) {
            case n:
            case i:
            case o:
            case c:
            case f:
              return C;
            default:
              switch (C = C && C.$$typeof, C) {
                case l:
                case u:
                case g:
                case p:
                  return C;
                case s:
                  return C;
                default:
                  return $;
              }
          }
        case t:
          return $;
      }
    }
  }
  return Ht.ContextConsumer = s, Ht.ContextProvider = l, Ht.Element = e, Ht.ForwardRef = u, Ht.Fragment = n, Ht.Lazy = g, Ht.Memo = p, Ht.Portal = t, Ht.Profiler = i, Ht.StrictMode = o, Ht.Suspense = c, Ht.SuspenseList = f, Ht.isContextConsumer = function(C) {
    return v(C) === s;
  }, Ht.isContextProvider = function(C) {
    return v(C) === l;
  }, Ht.isElement = function(C) {
    return typeof C == "object" && C !== null && C.$$typeof === e;
  }, Ht.isForwardRef = function(C) {
    return v(C) === u;
  }, Ht.isFragment = function(C) {
    return v(C) === n;
  }, Ht.isLazy = function(C) {
    return v(C) === g;
  }, Ht.isMemo = function(C) {
    return v(C) === p;
  }, Ht.isPortal = function(C) {
    return v(C) === t;
  }, Ht.isProfiler = function(C) {
    return v(C) === i;
  }, Ht.isStrictMode = function(C) {
    return v(C) === o;
  }, Ht.isSuspense = function(C) {
    return v(C) === c;
  }, Ht.isSuspenseList = function(C) {
    return v(C) === f;
  }, Ht.isValidElementType = function(C) {
    return typeof C == "string" || typeof C == "function" || C === n || C === i || C === o || C === c || C === f || C === h || typeof C == "object" && C !== null && (C.$$typeof === g || C.$$typeof === p || C.$$typeof === l || C.$$typeof === s || C.$$typeof === u || C.$$typeof === y || C.getModuleId !== void 0);
  }, Ht.typeOf = v, Ht;
}
Tx.exports = oq();
var Mx = Tx.exports;
function Ko(e) {
  if (typeof e != "object" || e === null)
    return !1;
  const t = Object.getPrototypeOf(e);
  return (t === null || t === Object.prototype || Object.getPrototypeOf(t) === null) && !(Symbol.toStringTag in e) && !(Symbol.iterator in e);
}
function Nx(e) {
  if (/* @__PURE__ */ z.isValidElement(e) || Mx.isValidElementType(e) || !Ko(e))
    return e;
  const t = {};
  return Object.keys(e).forEach((n) => {
    t[n] = Nx(e[n]);
  }), t;
}
function lr(e, t, n = {
  clone: !0
}) {
  const o = n.clone ? {
    ...e
  } : e;
  return Ko(e) && Ko(t) && Object.keys(t).forEach((i) => {
    /* @__PURE__ */ z.isValidElement(t[i]) || Mx.isValidElementType(t[i]) ? o[i] = t[i] : Ko(t[i]) && // Avoid prototype pollution
    Object.prototype.hasOwnProperty.call(e, i) && Ko(e[i]) ? o[i] = lr(e[i], t[i], n) : n.clone ? o[i] = Ko(t[i]) ? Nx(t[i]) : t[i] : o[i] = t[i];
  }), o;
}
function Ci(e, ...t) {
  const n = new URL(`https://mui.com/production-error/?code=${e}`);
  return t.forEach((o) => n.searchParams.append("args[]", o)), `Minified MUI error #${e}; visit ${n} for the full message.`;
}
function Ne(e) {
  if (typeof e != "string")
    throw new Error(Ci(7));
  return e.charAt(0).toUpperCase() + e.slice(1);
}
function _g(...e) {
  return e.reduce((t, n) => n == null ? t : function(...i) {
    t.apply(this, i), n.apply(this, i);
  }, () => {
  });
}
function Fx(e, t = 166) {
  let n;
  function o(...i) {
    const s = () => {
      e.apply(this, i);
    };
    clearTimeout(n), n = setTimeout(s, t);
  }
  return o.clear = () => {
    clearTimeout(n);
  }, o;
}
function Vu(e, t) {
  var n, o, i;
  return /* @__PURE__ */ z.isValidElement(e) && t.indexOf(
    // For server components `muiName` is avaialble in element.type._payload.value.muiName
    // relevant info - https://github.com/facebook/react/blob/2807d781a08db8e9873687fccc25c0f12b4fb3d4/packages/react/src/ReactLazy.js#L45
    // eslint-disable-next-line no-underscore-dangle
    e.type.muiName ?? ((i = (o = (n = e.type) == null ? void 0 : n._payload) == null ? void 0 : o.value) == null ? void 0 : i.muiName)
  ) !== -1;
}
function $r(e) {
  return e && e.ownerDocument || document;
}
function xi(e) {
  return $r(e).defaultView || window;
}
function x2(e, t) {
  typeof e == "function" ? e(t) : e && (e.current = t);
}
const Ei = typeof window < "u" ? z.useLayoutEffect : z.useEffect;
let E2 = 0;
function iq(e) {
  const [t, n] = z.useState(e), o = e || t;
  return z.useEffect(() => {
    t == null && (E2 += 1, n(`mui-${E2}`));
  }, [t]), o;
}
const aq = {
  ...Zu
}, _2 = aq.useId;
function t0(e) {
  if (_2 !== void 0) {
    const t = _2();
    return e ?? t;
  }
  return iq(e);
}
function sc({
  controlled: e,
  default: t,
  name: n,
  state: o = "value"
}) {
  const {
    current: i
  } = z.useRef(e !== void 0), [s, l] = z.useState(t), u = i ? e : s, c = z.useCallback((f) => {
    i || l(f);
  }, []);
  return [u, c];
}
function yi(e) {
  const t = z.useRef(e);
  return Ei(() => {
    t.current = e;
  }), z.useRef((...n) => (
    // @ts-expect-error hide `this`
    (0, t.current)(...n)
  )).current;
}
function _n(...e) {
  const t = z.useRef(void 0), n = z.useCallback((o) => {
    const i = e.map((s) => {
      if (s == null)
        return null;
      if (typeof s == "function") {
        const l = s, u = l(o);
        return typeof u == "function" ? u : () => {
          l(null);
        };
      }
      return s.current = o, () => {
        s.current = null;
      };
    });
    return () => {
      i.forEach((s) => s == null ? void 0 : s());
    };
  }, e);
  return z.useMemo(() => e.every((o) => o == null) ? null : (o) => {
    t.current && (t.current(), t.current = void 0), o != null && (t.current = n(o));
  }, e);
}
const k2 = {};
function Lx(e, t) {
  const n = z.useRef(k2);
  return n.current === k2 && (n.current = e(t)), n;
}
const sq = [];
function lq(e) {
  z.useEffect(e, sq);
}
class n0 {
  constructor() {
    yu(this, "currentId", null);
    yu(this, "clear", () => {
      this.currentId !== null && (clearTimeout(this.currentId), this.currentId = null);
    });
    yu(this, "disposeEffect", () => this.clear);
  }
  static create() {
    return new n0();
  }
  /**
   * Executes `fn` after `delay`, clearing any previously scheduled call.
   */
  start(t, n) {
    this.clear(), this.currentId = setTimeout(() => {
      this.currentId = null, n();
    }, t);
  }
}
function r0() {
  const e = Lx(n0.create).current;
  return lq(e.disposeEffect), e;
}
function Lf(e) {
  try {
    return e.matches(":focus-visible");
  } catch {
  }
  return !1;
}
function Dx(e = window) {
  const t = e.document.documentElement.clientWidth;
  return e.innerWidth - t;
}
const uq = {
  border: 0,
  clip: "rect(0 0 0 0)",
  height: "1px",
  margin: "-1px",
  overflow: "hidden",
  padding: 0,
  position: "absolute",
  whiteSpace: "nowrap",
  width: "1px"
};
function lc(e, t) {
  const n = {
    ...t
  };
  for (const o in e)
    if (Object.prototype.hasOwnProperty.call(e, o)) {
      const i = o;
      if (i === "components" || i === "slots")
        n[i] = {
          ...e[i],
          ...n[i]
        };
      else if (i === "componentsProps" || i === "slotProps") {
        const s = e[i], l = t[i];
        if (!l)
          n[i] = s || {};
        else if (!s)
          n[i] = l;
        else {
          n[i] = {
            ...l
          };
          for (const u in s)
            if (Object.prototype.hasOwnProperty.call(s, u)) {
              const c = u;
              n[i][c] = lc(s[c], l[c]);
            }
        }
      } else n[i] === void 0 && (n[i] = e[i]);
    }
  return n;
}
function mt(e, t, n = void 0) {
  const o = {};
  for (const i in e) {
    const s = e[i];
    let l = "", u = !0;
    for (let c = 0; c < s.length; c += 1) {
      const f = s[c];
      f && (l += (u === !0 ? "" : " ") + t(f), u = !1, n && n[f] && (l += " " + n[f]));
    }
    o[i] = l;
  }
  return o;
}
const P2 = (e) => e, cq = () => {
  let e = P2;
  return {
    configure(t) {
      e = t;
    },
    generate(t) {
      return e(t);
    },
    reset() {
      e = P2;
    }
  };
}, zx = cq(), dq = {
  active: "active",
  checked: "checked",
  completed: "completed",
  disabled: "disabled",
  error: "error",
  expanded: "expanded",
  focused: "focused",
  focusVisible: "focusVisible",
  open: "open",
  readOnly: "readOnly",
  required: "required",
  selected: "selected"
};
function ht(e, t, n = "Mui") {
  const o = dq[t];
  return o ? `${n}-${o}` : `${zx.generate(e)}-${t}`;
}
function dt(e, t, n = "Mui") {
  const o = {};
  return t.forEach((i) => {
    o[i] = ht(e, i, n);
  }), o;
}
function Vs(e, t = Number.MIN_SAFE_INTEGER, n = Number.MAX_SAFE_INTEGER) {
  return Math.max(t, Math.min(e, n));
}
function fq(e) {
  return typeof e == "string";
}
function Bx(e, t, n) {
  return e === void 0 || fq(e) ? t : {
    ...t,
    ownerState: {
      ...t.ownerState,
      ...n
    }
  };
}
function Ux(e) {
  var t, n, o = "";
  if (typeof e == "string" || typeof e == "number") o += e;
  else if (typeof e == "object") if (Array.isArray(e)) {
    var i = e.length;
    for (t = 0; t < i; t++) e[t] && (n = Ux(e[t])) && (o && (o += " "), o += n);
  } else for (n in e) e[n] && (o && (o += " "), o += n);
  return o;
}
function Ve() {
  for (var e, t, n = 0, o = "", i = arguments.length; n < i; n++) (e = arguments[n]) && (t = Ux(e)) && (o && (o += " "), o += t);
  return o;
}
function Wu(e, t = []) {
  if (e === void 0)
    return {};
  const n = {};
  return Object.keys(e).filter((o) => o.match(/^on[A-Z]/) && typeof e[o] == "function" && !t.includes(o)).forEach((o) => {
    n[o] = e[o];
  }), n;
}
function O2(e) {
  if (e === void 0)
    return {};
  const t = {};
  return Object.keys(e).filter((n) => !(n.match(/^on[A-Z]/) && typeof e[n] == "function")).forEach((n) => {
    t[n] = e[n];
  }), t;
}
function Vx(e) {
  const {
    getSlotProps: t,
    additionalProps: n,
    externalSlotProps: o,
    externalForwardedProps: i,
    className: s
  } = e;
  if (!t) {
    const y = Ve(n == null ? void 0 : n.className, s, i == null ? void 0 : i.className, o == null ? void 0 : o.className), v = {
      ...n == null ? void 0 : n.style,
      ...i == null ? void 0 : i.style,
      ...o == null ? void 0 : o.style
    }, C = {
      ...n,
      ...i,
      ...o
    };
    return y.length > 0 && (C.className = y), Object.keys(v).length > 0 && (C.style = v), {
      props: C,
      internalRef: void 0
    };
  }
  const l = Wu({
    ...i,
    ...o
  }), u = O2(o), c = O2(i), f = t(l), p = Ve(f == null ? void 0 : f.className, n == null ? void 0 : n.className, s, i == null ? void 0 : i.className, o == null ? void 0 : o.className), g = {
    ...f == null ? void 0 : f.style,
    ...n == null ? void 0 : n.style,
    ...i == null ? void 0 : i.style,
    ...o == null ? void 0 : o.style
  }, h = {
    ...f,
    ...n,
    ...c,
    ...u
  };
  return p.length > 0 && (h.className = p), Object.keys(g).length > 0 && (h.style = g), {
    props: h,
    internalRef: f.ref
  };
}
function Wx(e, t, n) {
  return typeof e == "function" ? e(t, n) : e;
}
function Vo(e) {
  var g;
  const {
    elementType: t,
    externalSlotProps: n,
    ownerState: o,
    skipResolvingSlotProps: i = !1,
    ...s
  } = e, l = i ? {} : Wx(n, o), {
    props: u,
    internalRef: c
  } = Vx({
    ...s,
    externalSlotProps: l
  }), f = _n(c, l == null ? void 0 : l.ref, (g = e.additionalProps) == null ? void 0 : g.ref);
  return Bx(t, {
    ...u,
    ref: f
  }, o);
}
function kc(e) {
  var t;
  return parseInt(z.version, 10) >= 19 ? ((t = e == null ? void 0 : e.props) == null ? void 0 : t.ref) || null : (e == null ? void 0 : e.ref) || null;
}
function qu(e, t) {
  return t ? lr(e, t, {
    clone: !1
    // No need to clone deep, it's way faster.
  }) : e;
}
function pq(e, t) {
  if (!e.containerQueries)
    return t;
  const n = Object.keys(t).filter((o) => o.startsWith("@container")).sort((o, i) => {
    var l, u;
    const s = /min-width:\s*([0-9.]+)/;
    return +(((l = o.match(s)) == null ? void 0 : l[1]) || 0) - +(((u = i.match(s)) == null ? void 0 : u[1]) || 0);
  });
  return n.length ? n.reduce((o, i) => {
    const s = t[i];
    return delete o[i], o[i] = s, o;
  }, {
    ...t
  }) : t;
}
function hq(e, t) {
  return t === "@" || t.startsWith("@") && (e.some((n) => t.startsWith(`@${n}`)) || !!t.match(/^@\d/));
}
function mq(e, t) {
  const n = t.match(/^@([^/]+)?\/?(.+)?$/);
  if (!n)
    return null;
  const [, o, i] = n, s = Number.isNaN(+o) ? o || 0 : +o;
  return e.containerQueries(i).up(s);
}
function gq(e) {
  const t = (s, l) => s.replace("@media", l ? `@container ${l}` : "@container");
  function n(s, l) {
    s.up = (...u) => t(e.breakpoints.up(...u), l), s.down = (...u) => t(e.breakpoints.down(...u), l), s.between = (...u) => t(e.breakpoints.between(...u), l), s.only = (...u) => t(e.breakpoints.only(...u), l), s.not = (...u) => {
      const c = t(e.breakpoints.not(...u), l);
      return c.includes("not all and") ? c.replace("not all and ", "").replace("min-width:", "width<").replace("max-width:", "width>").replace("and", "or") : c;
    };
  }
  const o = {}, i = (s) => (n(o, s), o);
  return n(i), {
    ...e,
    containerQueries: i
  };
}
const $p = {
  xs: 0,
  // phone
  sm: 600,
  // tablet
  md: 900,
  // small laptop
  lg: 1200,
  // desktop
  xl: 1536
  // large screen
}, A2 = {
  // Sorted ASC by size. That's important.
  // It can't be configured as it's used statically for propTypes.
  keys: ["xs", "sm", "md", "lg", "xl"],
  up: (e) => `@media (min-width:${$p[e]}px)`
}, yq = {
  containerQueries: (e) => ({
    up: (t) => {
      let n = typeof t == "number" ? t : $p[t] || t;
      return typeof n == "number" && (n = `${n}px`), e ? `@container ${e} (min-width:${n})` : `@container (min-width:${n})`;
    }
  })
};
function ho(e, t, n) {
  const o = e.theme || {};
  if (Array.isArray(t)) {
    const s = o.breakpoints || A2;
    return t.reduce((l, u, c) => (l[s.up(s.keys[c])] = n(t[c]), l), {});
  }
  if (typeof t == "object") {
    const s = o.breakpoints || A2;
    return Object.keys(t).reduce((l, u) => {
      if (hq(s.keys, u)) {
        const c = mq(o.containerQueries ? o : yq, u);
        c && (l[c] = n(t[u], u));
      } else if (Object.keys(s.values || $p).includes(u)) {
        const c = s.up(u);
        l[c] = n(t[u], u);
      } else {
        const c = u;
        l[c] = t[c];
      }
      return l;
    }, {});
  }
  return n(t);
}
function vq(e = {}) {
  var n;
  return ((n = e.keys) == null ? void 0 : n.reduce((o, i) => {
    const s = e.up(i);
    return o[s] = {}, o;
  }, {})) || {};
}
function bq(e, t) {
  return e.reduce((n, o) => {
    const i = n[o];
    return (!i || Object.keys(i).length === 0) && delete n[o], n;
  }, t);
}
function Sq(e, t) {
  if (typeof e != "object")
    return {};
  const n = {}, o = Object.keys(t);
  return Array.isArray(e) ? o.forEach((i, s) => {
    s < e.length && (n[i] = !0);
  }) : o.forEach((i) => {
    e[i] != null && (n[i] = !0);
  }), n;
}
function Cp({
  values: e,
  breakpoints: t,
  base: n
}) {
  const o = n || Sq(e, t), i = Object.keys(o);
  if (i.length === 0)
    return e;
  let s;
  return i.reduce((l, u, c) => (Array.isArray(e) ? (l[u] = e[c] != null ? e[c] : e[s], s = c) : typeof e == "object" ? (l[u] = e[u] != null ? e[u] : e[s], s = u) : l[u] = e, l), {});
}
function xp(e, t, n = !0) {
  if (!t || typeof t != "string")
    return null;
  if (e && e.vars && n) {
    const o = `vars.${t}`.split(".").reduce((i, s) => i && i[s] ? i[s] : null, e);
    if (o != null)
      return o;
  }
  return t.split(".").reduce((o, i) => o && o[i] != null ? o[i] : null, e);
}
function Df(e, t, n, o = n) {
  let i;
  return typeof e == "function" ? i = e(n) : Array.isArray(e) ? i = e[n] || o : i = xp(e, n) || o, t && (i = t(i, o, e)), i;
}
function Pn(e) {
  const {
    prop: t,
    cssProperty: n = e.prop,
    themeKey: o,
    transform: i
  } = e, s = (l) => {
    if (l[t] == null)
      return null;
    const u = l[t], c = l.theme, f = xp(c, o) || {};
    return ho(l, u, (g) => {
      let h = Df(f, i, g);
      return g === h && typeof g == "string" && (h = Df(f, i, `${t}${g === "default" ? "" : Ne(g)}`, g)), n === !1 ? h : {
        [n]: h
      };
    });
  };
  return s.propTypes = {}, s.filterProps = [t], s;
}
function wq(e) {
  const t = {};
  return (n) => (t[n] === void 0 && (t[n] = e(n)), t[n]);
}
const $q = {
  m: "margin",
  p: "padding"
}, Cq = {
  t: "Top",
  r: "Right",
  b: "Bottom",
  l: "Left",
  x: ["Left", "Right"],
  y: ["Top", "Bottom"]
}, I2 = {
  marginX: "mx",
  marginY: "my",
  paddingX: "px",
  paddingY: "py"
}, xq = wq((e) => {
  if (e.length > 2)
    if (I2[e])
      e = I2[e];
    else
      return [e];
  const [t, n] = e.split(""), o = $q[t], i = Cq[n] || "";
  return Array.isArray(i) ? i.map((s) => o + s) : [o + i];
}), o0 = ["m", "mt", "mr", "mb", "ml", "mx", "my", "margin", "marginTop", "marginRight", "marginBottom", "marginLeft", "marginX", "marginY", "marginInline", "marginInlineStart", "marginInlineEnd", "marginBlock", "marginBlockStart", "marginBlockEnd"], i0 = ["p", "pt", "pr", "pb", "pl", "px", "py", "padding", "paddingTop", "paddingRight", "paddingBottom", "paddingLeft", "paddingX", "paddingY", "paddingInline", "paddingInlineStart", "paddingInlineEnd", "paddingBlock", "paddingBlockStart", "paddingBlockEnd"];
[...o0, ...i0];
function Pc(e, t, n, o) {
  const i = xp(e, t, !0) ?? n;
  return typeof i == "number" || typeof i == "string" ? (s) => typeof s == "string" ? s : typeof i == "string" ? `calc(${s} * ${i})` : i * s : Array.isArray(i) ? (s) => {
    if (typeof s == "string")
      return s;
    const l = Math.abs(s), u = i[l];
    return s >= 0 ? u : typeof u == "number" ? -u : `-${u}`;
  } : typeof i == "function" ? i : () => {
  };
}
function a0(e) {
  return Pc(e, "spacing", 8);
}
function Oc(e, t) {
  return typeof t == "string" || t == null ? t : e(t);
}
function Eq(e, t) {
  return (n) => e.reduce((o, i) => (o[i] = Oc(t, n), o), {});
}
function _q(e, t, n, o) {
  if (!t.includes(n))
    return null;
  const i = xq(n), s = Eq(i, o), l = e[n];
  return ho(e, l, s);
}
function qx(e, t) {
  const n = a0(e.theme);
  return Object.keys(e).map((o) => _q(e, t, o, n)).reduce(qu, {});
}
function bn(e) {
  return qx(e, o0);
}
bn.propTypes = {};
bn.filterProps = o0;
function Sn(e) {
  return qx(e, i0);
}
Sn.propTypes = {};
Sn.filterProps = i0;
function Ep(...e) {
  const t = e.reduce((o, i) => (i.filterProps.forEach((s) => {
    o[s] = i;
  }), o), {}), n = (o) => Object.keys(o).reduce((i, s) => t[s] ? qu(i, t[s](o)) : i, {});
  return n.propTypes = {}, n.filterProps = e.reduce((o, i) => o.concat(i.filterProps), []), n;
}
function so(e) {
  return typeof e != "number" ? e : `${e}px solid`;
}
function go(e, t) {
  return Pn({
    prop: e,
    themeKey: "borders",
    transform: t
  });
}
const kq = go("border", so), Pq = go("borderTop", so), Oq = go("borderRight", so), Aq = go("borderBottom", so), Iq = go("borderLeft", so), jq = go("borderColor"), Rq = go("borderTopColor"), Tq = go("borderRightColor"), Mq = go("borderBottomColor"), Nq = go("borderLeftColor"), Fq = go("outline", so), Lq = go("outlineColor"), _p = (e) => {
  if (e.borderRadius !== void 0 && e.borderRadius !== null) {
    const t = Pc(e.theme, "shape.borderRadius", 4), n = (o) => ({
      borderRadius: Oc(t, o)
    });
    return ho(e, e.borderRadius, n);
  }
  return null;
};
_p.propTypes = {};
_p.filterProps = ["borderRadius"];
Ep(kq, Pq, Oq, Aq, Iq, jq, Rq, Tq, Mq, Nq, _p, Fq, Lq);
const kp = (e) => {
  if (e.gap !== void 0 && e.gap !== null) {
    const t = Pc(e.theme, "spacing", 8), n = (o) => ({
      gap: Oc(t, o)
    });
    return ho(e, e.gap, n);
  }
  return null;
};
kp.propTypes = {};
kp.filterProps = ["gap"];
const Pp = (e) => {
  if (e.columnGap !== void 0 && e.columnGap !== null) {
    const t = Pc(e.theme, "spacing", 8), n = (o) => ({
      columnGap: Oc(t, o)
    });
    return ho(e, e.columnGap, n);
  }
  return null;
};
Pp.propTypes = {};
Pp.filterProps = ["columnGap"];
const Op = (e) => {
  if (e.rowGap !== void 0 && e.rowGap !== null) {
    const t = Pc(e.theme, "spacing", 8), n = (o) => ({
      rowGap: Oc(t, o)
    });
    return ho(e, e.rowGap, n);
  }
  return null;
};
Op.propTypes = {};
Op.filterProps = ["rowGap"];
const Dq = Pn({
  prop: "gridColumn"
}), zq = Pn({
  prop: "gridRow"
}), Bq = Pn({
  prop: "gridAutoFlow"
}), Uq = Pn({
  prop: "gridAutoColumns"
}), Vq = Pn({
  prop: "gridAutoRows"
}), Wq = Pn({
  prop: "gridTemplateColumns"
}), qq = Pn({
  prop: "gridTemplateRows"
}), Kq = Pn({
  prop: "gridTemplateAreas"
}), Hq = Pn({
  prop: "gridArea"
});
Ep(kp, Pp, Op, Dq, zq, Bq, Uq, Vq, Wq, qq, Kq, Hq);
function Ys(e, t) {
  return t === "grey" ? t : e;
}
const Gq = Pn({
  prop: "color",
  themeKey: "palette",
  transform: Ys
}), Jq = Pn({
  prop: "bgcolor",
  cssProperty: "backgroundColor",
  themeKey: "palette",
  transform: Ys
}), Yq = Pn({
  prop: "backgroundColor",
  themeKey: "palette",
  transform: Ys
});
Ep(Gq, Jq, Yq);
function Wr(e) {
  return e <= 1 && e !== 0 ? `${e * 100}%` : e;
}
const Qq = Pn({
  prop: "width",
  transform: Wr
}), s0 = (e) => {
  if (e.maxWidth !== void 0 && e.maxWidth !== null) {
    const t = (n) => {
      var i, s, l, u, c;
      const o = ((l = (s = (i = e.theme) == null ? void 0 : i.breakpoints) == null ? void 0 : s.values) == null ? void 0 : l[n]) || $p[n];
      return o ? ((c = (u = e.theme) == null ? void 0 : u.breakpoints) == null ? void 0 : c.unit) !== "px" ? {
        maxWidth: `${o}${e.theme.breakpoints.unit}`
      } : {
        maxWidth: o
      } : {
        maxWidth: Wr(n)
      };
    };
    return ho(e, e.maxWidth, t);
  }
  return null;
};
s0.filterProps = ["maxWidth"];
const Xq = Pn({
  prop: "minWidth",
  transform: Wr
}), Zq = Pn({
  prop: "height",
  transform: Wr
}), eK = Pn({
  prop: "maxHeight",
  transform: Wr
}), tK = Pn({
  prop: "minHeight",
  transform: Wr
});
Pn({
  prop: "size",
  cssProperty: "width",
  transform: Wr
});
Pn({
  prop: "size",
  cssProperty: "height",
  transform: Wr
});
const nK = Pn({
  prop: "boxSizing"
});
Ep(Qq, s0, Xq, Zq, eK, tK, nK);
const Ac = {
  // borders
  border: {
    themeKey: "borders",
    transform: so
  },
  borderTop: {
    themeKey: "borders",
    transform: so
  },
  borderRight: {
    themeKey: "borders",
    transform: so
  },
  borderBottom: {
    themeKey: "borders",
    transform: so
  },
  borderLeft: {
    themeKey: "borders",
    transform: so
  },
  borderColor: {
    themeKey: "palette"
  },
  borderTopColor: {
    themeKey: "palette"
  },
  borderRightColor: {
    themeKey: "palette"
  },
  borderBottomColor: {
    themeKey: "palette"
  },
  borderLeftColor: {
    themeKey: "palette"
  },
  outline: {
    themeKey: "borders",
    transform: so
  },
  outlineColor: {
    themeKey: "palette"
  },
  borderRadius: {
    themeKey: "shape.borderRadius",
    style: _p
  },
  // palette
  color: {
    themeKey: "palette",
    transform: Ys
  },
  bgcolor: {
    themeKey: "palette",
    cssProperty: "backgroundColor",
    transform: Ys
  },
  backgroundColor: {
    themeKey: "palette",
    transform: Ys
  },
  // spacing
  p: {
    style: Sn
  },
  pt: {
    style: Sn
  },
  pr: {
    style: Sn
  },
  pb: {
    style: Sn
  },
  pl: {
    style: Sn
  },
  px: {
    style: Sn
  },
  py: {
    style: Sn
  },
  padding: {
    style: Sn
  },
  paddingTop: {
    style: Sn
  },
  paddingRight: {
    style: Sn
  },
  paddingBottom: {
    style: Sn
  },
  paddingLeft: {
    style: Sn
  },
  paddingX: {
    style: Sn
  },
  paddingY: {
    style: Sn
  },
  paddingInline: {
    style: Sn
  },
  paddingInlineStart: {
    style: Sn
  },
  paddingInlineEnd: {
    style: Sn
  },
  paddingBlock: {
    style: Sn
  },
  paddingBlockStart: {
    style: Sn
  },
  paddingBlockEnd: {
    style: Sn
  },
  m: {
    style: bn
  },
  mt: {
    style: bn
  },
  mr: {
    style: bn
  },
  mb: {
    style: bn
  },
  ml: {
    style: bn
  },
  mx: {
    style: bn
  },
  my: {
    style: bn
  },
  margin: {
    style: bn
  },
  marginTop: {
    style: bn
  },
  marginRight: {
    style: bn
  },
  marginBottom: {
    style: bn
  },
  marginLeft: {
    style: bn
  },
  marginX: {
    style: bn
  },
  marginY: {
    style: bn
  },
  marginInline: {
    style: bn
  },
  marginInlineStart: {
    style: bn
  },
  marginInlineEnd: {
    style: bn
  },
  marginBlock: {
    style: bn
  },
  marginBlockStart: {
    style: bn
  },
  marginBlockEnd: {
    style: bn
  },
  // display
  displayPrint: {
    cssProperty: !1,
    transform: (e) => ({
      "@media print": {
        display: e
      }
    })
  },
  display: {},
  overflow: {},
  textOverflow: {},
  visibility: {},
  whiteSpace: {},
  // flexbox
  flexBasis: {},
  flexDirection: {},
  flexWrap: {},
  justifyContent: {},
  alignItems: {},
  alignContent: {},
  order: {},
  flex: {},
  flexGrow: {},
  flexShrink: {},
  alignSelf: {},
  justifyItems: {},
  justifySelf: {},
  // grid
  gap: {
    style: kp
  },
  rowGap: {
    style: Op
  },
  columnGap: {
    style: Pp
  },
  gridColumn: {},
  gridRow: {},
  gridAutoFlow: {},
  gridAutoColumns: {},
  gridAutoRows: {},
  gridTemplateColumns: {},
  gridTemplateRows: {},
  gridTemplateAreas: {},
  gridArea: {},
  // positions
  position: {},
  zIndex: {
    themeKey: "zIndex"
  },
  top: {},
  right: {},
  bottom: {},
  left: {},
  // shadows
  boxShadow: {
    themeKey: "shadows"
  },
  // sizing
  width: {
    transform: Wr
  },
  maxWidth: {
    style: s0
  },
  minWidth: {
    transform: Wr
  },
  height: {
    transform: Wr
  },
  maxHeight: {
    transform: Wr
  },
  minHeight: {
    transform: Wr
  },
  boxSizing: {},
  // typography
  font: {
    themeKey: "font"
  },
  fontFamily: {
    themeKey: "typography"
  },
  fontSize: {
    themeKey: "typography"
  },
  fontStyle: {
    themeKey: "typography"
  },
  fontWeight: {
    themeKey: "typography"
  },
  letterSpacing: {},
  textTransform: {},
  lineHeight: {},
  textAlign: {},
  typography: {
    cssProperty: !1,
    themeKey: "typography"
  }
};
function rK(...e) {
  const t = e.reduce((o, i) => o.concat(Object.keys(i)), []), n = new Set(t);
  return e.every((o) => n.size === Object.keys(o).length);
}
function oK(e, t) {
  return typeof e == "function" ? e(t) : e;
}
function iK() {
  function e(n, o, i, s) {
    const l = {
      [n]: o,
      theme: i
    }, u = s[n];
    if (!u)
      return {
        [n]: o
      };
    const {
      cssProperty: c = n,
      themeKey: f,
      transform: p,
      style: g
    } = u;
    if (o == null)
      return null;
    if (f === "typography" && o === "inherit")
      return {
        [n]: o
      };
    const h = xp(i, f) || {};
    return g ? g(l) : ho(l, o, (v) => {
      let C = Df(h, p, v);
      return v === C && typeof v == "string" && (C = Df(h, p, `${n}${v === "default" ? "" : Ne(v)}`, v)), c === !1 ? C : {
        [c]: C
      };
    });
  }
  function t(n) {
    const {
      sx: o,
      theme: i = {}
    } = n || {};
    if (!o)
      return null;
    const s = i.unstable_sxConfig ?? Ac;
    function l(u) {
      let c = u;
      if (typeof u == "function")
        c = u(i);
      else if (typeof u != "object")
        return u;
      if (!c)
        return null;
      const f = vq(i.breakpoints), p = Object.keys(f);
      let g = f;
      return Object.keys(c).forEach((h) => {
        const y = oK(c[h], i);
        if (y != null)
          if (typeof y == "object")
            if (s[h])
              g = qu(g, e(h, y, i, s));
            else {
              const v = ho({
                theme: i
              }, y, (C) => ({
                [h]: C
              }));
              rK(v, y) ? g[h] = t({
                sx: y,
                theme: i
              }) : g = qu(g, v);
            }
          else
            g = qu(g, e(h, y, i, s));
      }), pq(i, bq(p, g));
    }
    return Array.isArray(o) ? o.map(l) : l(o);
  }
  return t;
}
const Va = iK();
Va.filterProps = ["sx"];
const aK = (e) => {
  var o;
  const t = {
    systemProps: {},
    otherProps: {}
  }, n = ((o = e == null ? void 0 : e.theme) == null ? void 0 : o.unstable_sxConfig) ?? Ac;
  return Object.keys(e).forEach((i) => {
    n[i] ? t.systemProps[i] = e[i] : t.otherProps[i] = e[i];
  }), t;
};
function Ap(e) {
  const {
    sx: t,
    ...n
  } = e, {
    systemProps: o,
    otherProps: i
  } = aK(n);
  let s;
  return Array.isArray(t) ? s = [o, ...t] : typeof t == "function" ? s = (...l) => {
    const u = t(...l);
    return Ko(u) ? {
      ...o,
      ...u
    } : o;
  } : s = {
    ...o,
    ...t
  }, {
    ...i,
    sx: s
  };
}
function zf() {
  return zf = Object.assign ? Object.assign.bind() : function(e) {
    for (var t = 1; t < arguments.length; t++) {
      var n = arguments[t];
      for (var o in n) ({}).hasOwnProperty.call(n, o) && (e[o] = n[o]);
    }
    return e;
  }, zf.apply(null, arguments);
}
function sK(e) {
  if (e.sheet)
    return e.sheet;
  for (var t = 0; t < document.styleSheets.length; t++)
    if (document.styleSheets[t].ownerNode === e)
      return document.styleSheets[t];
}
function lK(e) {
  var t = document.createElement("style");
  return t.setAttribute("data-emotion", e.key), e.nonce !== void 0 && t.setAttribute("nonce", e.nonce), t.appendChild(document.createTextNode("")), t.setAttribute("data-s", ""), t;
}
var uK = /* @__PURE__ */ function() {
  function e(n) {
    var o = this;
    this._insertTag = function(i) {
      var s;
      o.tags.length === 0 ? o.insertionPoint ? s = o.insertionPoint.nextSibling : o.prepend ? s = o.container.firstChild : s = o.before : s = o.tags[o.tags.length - 1].nextSibling, o.container.insertBefore(i, s), o.tags.push(i);
    }, this.isSpeedy = n.speedy === void 0 ? !0 : n.speedy, this.tags = [], this.ctr = 0, this.nonce = n.nonce, this.key = n.key, this.container = n.container, this.prepend = n.prepend, this.insertionPoint = n.insertionPoint, this.before = null;
  }
  var t = e.prototype;
  return t.hydrate = function(o) {
    o.forEach(this._insertTag);
  }, t.insert = function(o) {
    this.ctr % (this.isSpeedy ? 65e3 : 1) === 0 && this._insertTag(lK(this));
    var i = this.tags[this.tags.length - 1];
    if (this.isSpeedy) {
      var s = sK(i);
      try {
        s.insertRule(o, s.cssRules.length);
      } catch {
      }
    } else
      i.appendChild(document.createTextNode(o));
    this.ctr++;
  }, t.flush = function() {
    this.tags.forEach(function(o) {
      var i;
      return (i = o.parentNode) == null ? void 0 : i.removeChild(o);
    }), this.tags = [], this.ctr = 0;
  }, e;
}(), ar = "-ms-", Bf = "-moz-", Dt = "-webkit-", Kx = "comm", l0 = "rule", u0 = "decl", cK = "@import", Hx = "@keyframes", dK = "@layer", fK = Math.abs, Ip = String.fromCharCode, pK = Object.assign;
function hK(e, t) {
  return Qn(e, 0) ^ 45 ? (((t << 2 ^ Qn(e, 0)) << 2 ^ Qn(e, 1)) << 2 ^ Qn(e, 2)) << 2 ^ Qn(e, 3) : 0;
}
function Gx(e) {
  return e.trim();
}
function mK(e, t) {
  return (e = t.exec(e)) ? e[0] : e;
}
function zt(e, t, n) {
  return e.replace(t, n);
}
function kg(e, t) {
  return e.indexOf(t);
}
function Qn(e, t) {
  return e.charCodeAt(t) | 0;
}
function uc(e, t, n) {
  return e.slice(t, n);
}
function Wo(e) {
  return e.length;
}
function c0(e) {
  return e.length;
}
function Jd(e, t) {
  return t.push(e), e;
}
function gK(e, t) {
  return e.map(t).join("");
}
var jp = 1, ll = 1, Jx = 0, Nr = 0, In = 0, Il = "";
function Rp(e, t, n, o, i, s, l) {
  return { value: e, root: t, parent: n, type: o, props: i, children: s, line: jp, column: ll, length: l, return: "" };
}
function xu(e, t) {
  return pK(Rp("", null, null, "", null, null, 0), e, { length: -e.length }, t);
}
function yK() {
  return In;
}
function vK() {
  return In = Nr > 0 ? Qn(Il, --Nr) : 0, ll--, In === 10 && (ll = 1, jp--), In;
}
function Hr() {
  return In = Nr < Jx ? Qn(Il, Nr++) : 0, ll++, In === 10 && (ll = 1, jp++), In;
}
function Xo() {
  return Qn(Il, Nr);
}
function bf() {
  return Nr;
}
function Ic(e, t) {
  return uc(Il, e, t);
}
function cc(e) {
  switch (e) {
    case 0:
    case 9:
    case 10:
    case 13:
    case 32:
      return 5;
    case 33:
    case 43:
    case 44:
    case 47:
    case 62:
    case 64:
    case 126:
    case 59:
    case 123:
    case 125:
      return 4;
    case 58:
      return 3;
    case 34:
    case 39:
    case 40:
    case 91:
      return 2;
    case 41:
    case 93:
      return 1;
  }
  return 0;
}
function Yx(e) {
  return jp = ll = 1, Jx = Wo(Il = e), Nr = 0, [];
}
function Qx(e) {
  return Il = "", e;
}
function Sf(e) {
  return Gx(Ic(Nr - 1, Pg(e === 91 ? e + 2 : e === 40 ? e + 1 : e)));
}
function bK(e) {
  for (; (In = Xo()) && In < 33; )
    Hr();
  return cc(e) > 2 || cc(In) > 3 ? "" : " ";
}
function SK(e, t) {
  for (; --t && Hr() && !(In < 48 || In > 102 || In > 57 && In < 65 || In > 70 && In < 97); )
    ;
  return Ic(e, bf() + (t < 6 && Xo() == 32 && Hr() == 32));
}
function Pg(e) {
  for (; Hr(); )
    switch (In) {
      case e:
        return Nr;
      case 34:
      case 39:
        e !== 34 && e !== 39 && Pg(In);
        break;
      case 40:
        e === 41 && Pg(e);
        break;
      case 92:
        Hr();
        break;
    }
  return Nr;
}
function wK(e, t) {
  for (; Hr() && e + In !== 57; )
    if (e + In === 84 && Xo() === 47)
      break;
  return "/*" + Ic(t, Nr - 1) + "*" + Ip(e === 47 ? e : Hr());
}
function $K(e) {
  for (; !cc(Xo()); )
    Hr();
  return Ic(e, Nr);
}
function CK(e) {
  return Qx(wf("", null, null, null, [""], e = Yx(e), 0, [0], e));
}
function wf(e, t, n, o, i, s, l, u, c) {
  for (var f = 0, p = 0, g = l, h = 0, y = 0, v = 0, C = 1, $ = 1, S = 1, w = 0, _ = "", b = i, k = s, E = o, j = _; $; )
    switch (v = w, w = Hr()) {
      case 40:
        if (v != 108 && Qn(j, g - 1) == 58) {
          kg(j += zt(Sf(w), "&", "&\f"), "&\f") != -1 && (S = -1);
          break;
        }
      case 34:
      case 39:
      case 91:
        j += Sf(w);
        break;
      case 9:
      case 10:
      case 13:
      case 32:
        j += bK(v);
        break;
      case 92:
        j += SK(bf() - 1, 7);
        continue;
      case 47:
        switch (Xo()) {
          case 42:
          case 47:
            Jd(xK(wK(Hr(), bf()), t, n), c);
            break;
          default:
            j += "/";
        }
        break;
      case 123 * C:
        u[f++] = Wo(j) * S;
      case 125 * C:
      case 59:
      case 0:
        switch (w) {
          case 0:
          case 125:
            $ = 0;
          case 59 + p:
            S == -1 && (j = zt(j, /\f/g, "")), y > 0 && Wo(j) - g && Jd(y > 32 ? R2(j + ";", o, n, g - 1) : R2(zt(j, " ", "") + ";", o, n, g - 2), c);
            break;
          case 59:
            j += ";";
          default:
            if (Jd(E = j2(j, t, n, f, p, i, u, _, b = [], k = [], g), s), w === 123)
              if (p === 0)
                wf(j, t, E, E, b, s, g, u, k);
              else
                switch (h === 99 && Qn(j, 3) === 110 ? 100 : h) {
                  case 100:
                  case 108:
                  case 109:
                  case 115:
                    wf(e, E, E, o && Jd(j2(e, E, E, 0, 0, i, u, _, i, b = [], g), k), i, k, g, u, o ? b : k);
                    break;
                  default:
                    wf(j, E, E, E, [""], k, 0, u, k);
                }
        }
        f = p = y = 0, C = S = 1, _ = j = "", g = l;
        break;
      case 58:
        g = 1 + Wo(j), y = v;
      default:
        if (C < 1) {
          if (w == 123)
            --C;
          else if (w == 125 && C++ == 0 && vK() == 125)
            continue;
        }
        switch (j += Ip(w), w * C) {
          case 38:
            S = p > 0 ? 1 : (j += "\f", -1);
            break;
          case 44:
            u[f++] = (Wo(j) - 1) * S, S = 1;
            break;
          case 64:
            Xo() === 45 && (j += Sf(Hr())), h = Xo(), p = g = Wo(_ = j += $K(bf())), w++;
            break;
          case 45:
            v === 45 && Wo(j) == 2 && (C = 0);
        }
    }
  return s;
}
function j2(e, t, n, o, i, s, l, u, c, f, p) {
  for (var g = i - 1, h = i === 0 ? s : [""], y = c0(h), v = 0, C = 0, $ = 0; v < o; ++v)
    for (var S = 0, w = uc(e, g + 1, g = fK(C = l[v])), _ = e; S < y; ++S)
      (_ = Gx(C > 0 ? h[S] + " " + w : zt(w, /&\f/g, h[S]))) && (c[$++] = _);
  return Rp(e, t, n, i === 0 ? l0 : u, c, f, p);
}
function xK(e, t, n) {
  return Rp(e, t, n, Kx, Ip(yK()), uc(e, 2, -2), 0);
}
function R2(e, t, n, o) {
  return Rp(e, t, n, u0, uc(e, 0, o), uc(e, o + 1, -1), o);
}
function Qs(e, t) {
  for (var n = "", o = c0(e), i = 0; i < o; i++)
    n += t(e[i], i, e, t) || "";
  return n;
}
function EK(e, t, n, o) {
  switch (e.type) {
    case dK:
      if (e.children.length) break;
    case cK:
    case u0:
      return e.return = e.return || e.value;
    case Kx:
      return "";
    case Hx:
      return e.return = e.value + "{" + Qs(e.children, o) + "}";
    case l0:
      e.value = e.props.join(",");
  }
  return Wo(n = Qs(e.children, o)) ? e.return = e.value + "{" + n + "}" : "";
}
function _K(e) {
  var t = c0(e);
  return function(n, o, i, s) {
    for (var l = "", u = 0; u < t; u++)
      l += e[u](n, o, i, s) || "";
    return l;
  };
}
function kK(e) {
  return function(t) {
    t.root || (t = t.return) && e(t);
  };
}
function Xx(e) {
  var t = /* @__PURE__ */ Object.create(null);
  return function(n) {
    return t[n] === void 0 && (t[n] = e(n)), t[n];
  };
}
var PK = function(t, n, o) {
  for (var i = 0, s = 0; i = s, s = Xo(), i === 38 && s === 12 && (n[o] = 1), !cc(s); )
    Hr();
  return Ic(t, Nr);
}, OK = function(t, n) {
  var o = -1, i = 44;
  do
    switch (cc(i)) {
      case 0:
        i === 38 && Xo() === 12 && (n[o] = 1), t[o] += PK(Nr - 1, n, o);
        break;
      case 2:
        t[o] += Sf(i);
        break;
      case 4:
        if (i === 44) {
          t[++o] = Xo() === 58 ? "&\f" : "", n[o] = t[o].length;
          break;
        }
      default:
        t[o] += Ip(i);
    }
  while (i = Hr());
  return t;
}, AK = function(t, n) {
  return Qx(OK(Yx(t), n));
}, T2 = /* @__PURE__ */ new WeakMap(), IK = function(t) {
  if (!(t.type !== "rule" || !t.parent || // positive .length indicates that this rule contains pseudo
  // negative .length indicates that this rule has been already prefixed
  t.length < 1)) {
    for (var n = t.value, o = t.parent, i = t.column === o.column && t.line === o.line; o.type !== "rule"; )
      if (o = o.parent, !o) return;
    if (!(t.props.length === 1 && n.charCodeAt(0) !== 58 && !T2.get(o)) && !i) {
      T2.set(t, !0);
      for (var s = [], l = AK(n, s), u = o.props, c = 0, f = 0; c < l.length; c++)
        for (var p = 0; p < u.length; p++, f++)
          t.props[f] = s[c] ? l[c].replace(/&\f/g, u[p]) : u[p] + " " + l[c];
    }
  }
}, jK = function(t) {
  if (t.type === "decl") {
    var n = t.value;
    // charcode for l
    n.charCodeAt(0) === 108 && // charcode for b
    n.charCodeAt(2) === 98 && (t.return = "", t.value = "");
  }
};
function Zx(e, t) {
  switch (hK(e, t)) {
    case 5103:
      return Dt + "print-" + e + e;
    case 5737:
    case 4201:
    case 3177:
    case 3433:
    case 1641:
    case 4457:
    case 2921:
    case 5572:
    case 6356:
    case 5844:
    case 3191:
    case 6645:
    case 3005:
    case 6391:
    case 5879:
    case 5623:
    case 6135:
    case 4599:
    case 4855:
    case 4215:
    case 6389:
    case 5109:
    case 5365:
    case 5621:
    case 3829:
      return Dt + e + e;
    case 5349:
    case 4246:
    case 4810:
    case 6968:
    case 2756:
      return Dt + e + Bf + e + ar + e + e;
    case 6828:
    case 4268:
      return Dt + e + ar + e + e;
    case 6165:
      return Dt + e + ar + "flex-" + e + e;
    case 5187:
      return Dt + e + zt(e, /(\w+).+(:[^]+)/, Dt + "box-$1$2" + ar + "flex-$1$2") + e;
    case 5443:
      return Dt + e + ar + "flex-item-" + zt(e, /flex-|-self/, "") + e;
    case 4675:
      return Dt + e + ar + "flex-line-pack" + zt(e, /align-content|flex-|-self/, "") + e;
    case 5548:
      return Dt + e + ar + zt(e, "shrink", "negative") + e;
    case 5292:
      return Dt + e + ar + zt(e, "basis", "preferred-size") + e;
    case 6060:
      return Dt + "box-" + zt(e, "-grow", "") + Dt + e + ar + zt(e, "grow", "positive") + e;
    case 4554:
      return Dt + zt(e, /([^-])(transform)/g, "$1" + Dt + "$2") + e;
    case 6187:
      return zt(zt(zt(e, /(zoom-|grab)/, Dt + "$1"), /(image-set)/, Dt + "$1"), e, "") + e;
    case 5495:
    case 3959:
      return zt(e, /(image-set\([^]*)/, Dt + "$1$`$1");
    case 4968:
      return zt(zt(e, /(.+:)(flex-)?(.*)/, Dt + "box-pack:$3" + ar + "flex-pack:$3"), /s.+-b[^;]+/, "justify") + Dt + e + e;
    case 4095:
    case 3583:
    case 4068:
    case 2532:
      return zt(e, /(.+)-inline(.+)/, Dt + "$1$2") + e;
    case 8116:
    case 7059:
    case 5753:
    case 5535:
    case 5445:
    case 5701:
    case 4933:
    case 4677:
    case 5533:
    case 5789:
    case 5021:
    case 4765:
      if (Wo(e) - 1 - t > 6) switch (Qn(e, t + 1)) {
        case 109:
          if (Qn(e, t + 4) !== 45) break;
        case 102:
          return zt(e, /(.+:)(.+)-([^]+)/, "$1" + Dt + "$2-$3$1" + Bf + (Qn(e, t + 3) == 108 ? "$3" : "$2-$3")) + e;
        case 115:
          return ~kg(e, "stretch") ? Zx(zt(e, "stretch", "fill-available"), t) + e : e;
      }
      break;
    case 4949:
      if (Qn(e, t + 1) !== 115) break;
    case 6444:
      switch (Qn(e, Wo(e) - 3 - (~kg(e, "!important") && 10))) {
        case 107:
          return zt(e, ":", ":" + Dt) + e;
        case 101:
          return zt(e, /(.+:)([^;!]+)(;|!.+)?/, "$1" + Dt + (Qn(e, 14) === 45 ? "inline-" : "") + "box$3$1" + Dt + "$2$3$1" + ar + "$2box$3") + e;
      }
      break;
    case 5936:
      switch (Qn(e, t + 11)) {
        case 114:
          return Dt + e + ar + zt(e, /[svh]\w+-[tblr]{2}/, "tb") + e;
        case 108:
          return Dt + e + ar + zt(e, /[svh]\w+-[tblr]{2}/, "tb-rl") + e;
        case 45:
          return Dt + e + ar + zt(e, /[svh]\w+-[tblr]{2}/, "lr") + e;
      }
      return Dt + e + ar + e + e;
  }
  return e;
}
var RK = function(t, n, o, i) {
  if (t.length > -1 && !t.return) switch (t.type) {
    case u0:
      t.return = Zx(t.value, t.length);
      break;
    case Hx:
      return Qs([xu(t, {
        value: zt(t.value, "@", "@" + Dt)
      })], i);
    case l0:
      if (t.length) return gK(t.props, function(s) {
        switch (mK(s, /(::plac\w+|:read-\w+)/)) {
          case ":read-only":
          case ":read-write":
            return Qs([xu(t, {
              props: [zt(s, /:(read-\w+)/, ":" + Bf + "$1")]
            })], i);
          case "::placeholder":
            return Qs([xu(t, {
              props: [zt(s, /:(plac\w+)/, ":" + Dt + "input-$1")]
            }), xu(t, {
              props: [zt(s, /:(plac\w+)/, ":" + Bf + "$1")]
            }), xu(t, {
              props: [zt(s, /:(plac\w+)/, ar + "input-$1")]
            })], i);
        }
        return "";
      });
  }
}, TK = [RK], MK = function(t) {
  var n = t.key;
  if (n === "css") {
    var o = document.querySelectorAll("style[data-emotion]:not([data-s])");
    Array.prototype.forEach.call(o, function(C) {
      var $ = C.getAttribute("data-emotion");
      $.indexOf(" ") !== -1 && (document.head.appendChild(C), C.setAttribute("data-s", ""));
    });
  }
  var i = t.stylisPlugins || TK, s = {}, l, u = [];
  l = t.container || document.head, Array.prototype.forEach.call(
    // this means we will ignore elements which don't have a space in them which
    // means that the style elements we're looking at are only Emotion 11 server-rendered style elements
    document.querySelectorAll('style[data-emotion^="' + n + ' "]'),
    function(C) {
      for (var $ = C.getAttribute("data-emotion").split(" "), S = 1; S < $.length; S++)
        s[$[S]] = !0;
      u.push(C);
    }
  );
  var c, f = [IK, jK];
  {
    var p, g = [EK, kK(function(C) {
      p.insert(C);
    })], h = _K(f.concat(i, g)), y = function($) {
      return Qs(CK($), h);
    };
    c = function($, S, w, _) {
      p = w, y($ ? $ + "{" + S.styles + "}" : S.styles), _ && (v.inserted[S.name] = !0);
    };
  }
  var v = {
    key: n,
    sheet: new uK({
      key: n,
      container: l,
      nonce: t.nonce,
      speedy: t.speedy,
      prepend: t.prepend,
      insertionPoint: t.insertionPoint
    }),
    nonce: t.nonce,
    inserted: s,
    registered: {},
    insert: c
  };
  return v.sheet.hydrate(u), v;
}, eE = { exports: {} }, Wt = {};
/** @license React v16.13.1
 * react-is.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var M2;
function NK() {
  if (M2) return Wt;
  M2 = 1;
  var e = typeof Symbol == "function" && Symbol.for, t = e ? Symbol.for("react.element") : 60103, n = e ? Symbol.for("react.portal") : 60106, o = e ? Symbol.for("react.fragment") : 60107, i = e ? Symbol.for("react.strict_mode") : 60108, s = e ? Symbol.for("react.profiler") : 60114, l = e ? Symbol.for("react.provider") : 60109, u = e ? Symbol.for("react.context") : 60110, c = e ? Symbol.for("react.async_mode") : 60111, f = e ? Symbol.for("react.concurrent_mode") : 60111, p = e ? Symbol.for("react.forward_ref") : 60112, g = e ? Symbol.for("react.suspense") : 60113, h = e ? Symbol.for("react.suspense_list") : 60120, y = e ? Symbol.for("react.memo") : 60115, v = e ? Symbol.for("react.lazy") : 60116, C = e ? Symbol.for("react.block") : 60121, $ = e ? Symbol.for("react.fundamental") : 60117, S = e ? Symbol.for("react.responder") : 60118, w = e ? Symbol.for("react.scope") : 60119;
  function _(k) {
    if (typeof k == "object" && k !== null) {
      var E = k.$$typeof;
      switch (E) {
        case t:
          switch (k = k.type, k) {
            case c:
            case f:
            case o:
            case s:
            case i:
            case g:
              return k;
            default:
              switch (k = k && k.$$typeof, k) {
                case u:
                case p:
                case v:
                case y:
                case l:
                  return k;
                default:
                  return E;
              }
          }
        case n:
          return E;
      }
    }
  }
  function b(k) {
    return _(k) === f;
  }
  return Wt.AsyncMode = c, Wt.ConcurrentMode = f, Wt.ContextConsumer = u, Wt.ContextProvider = l, Wt.Element = t, Wt.ForwardRef = p, Wt.Fragment = o, Wt.Lazy = v, Wt.Memo = y, Wt.Portal = n, Wt.Profiler = s, Wt.StrictMode = i, Wt.Suspense = g, Wt.isAsyncMode = function(k) {
    return b(k) || _(k) === c;
  }, Wt.isConcurrentMode = b, Wt.isContextConsumer = function(k) {
    return _(k) === u;
  }, Wt.isContextProvider = function(k) {
    return _(k) === l;
  }, Wt.isElement = function(k) {
    return typeof k == "object" && k !== null && k.$$typeof === t;
  }, Wt.isForwardRef = function(k) {
    return _(k) === p;
  }, Wt.isFragment = function(k) {
    return _(k) === o;
  }, Wt.isLazy = function(k) {
    return _(k) === v;
  }, Wt.isMemo = function(k) {
    return _(k) === y;
  }, Wt.isPortal = function(k) {
    return _(k) === n;
  }, Wt.isProfiler = function(k) {
    return _(k) === s;
  }, Wt.isStrictMode = function(k) {
    return _(k) === i;
  }, Wt.isSuspense = function(k) {
    return _(k) === g;
  }, Wt.isValidElementType = function(k) {
    return typeof k == "string" || typeof k == "function" || k === o || k === f || k === s || k === i || k === g || k === h || typeof k == "object" && k !== null && (k.$$typeof === v || k.$$typeof === y || k.$$typeof === l || k.$$typeof === u || k.$$typeof === p || k.$$typeof === $ || k.$$typeof === S || k.$$typeof === w || k.$$typeof === C);
  }, Wt.typeOf = _, Wt;
}
eE.exports = NK();
var FK = eE.exports, tE = FK, LK = {
  $$typeof: !0,
  render: !0,
  defaultProps: !0,
  displayName: !0,
  propTypes: !0
}, DK = {
  $$typeof: !0,
  compare: !0,
  defaultProps: !0,
  displayName: !0,
  propTypes: !0,
  type: !0
}, nE = {};
nE[tE.ForwardRef] = LK;
nE[tE.Memo] = DK;
var zK = !0;
function rE(e, t, n) {
  var o = "";
  return n.split(" ").forEach(function(i) {
    e[i] !== void 0 ? t.push(e[i] + ";") : i && (o += i + " ");
  }), o;
}
var d0 = function(t, n, o) {
  var i = t.key + "-" + n.name;
  // we only need to add the styles to the registered cache if the
  // class name could be used further down
  // the tree but if it's a string tag, we know it won't
  // so we don't have to add it to registered cache.
  // this improves memory usage since we can avoid storing the whole style string
  (o === !1 || // we need to always store it if we're in compat mode and
  // in node since emotion-server relies on whether a style is in
  // the registered cache to know whether a style is global or not
  // also, note that this check will be dead code eliminated in the browser
  zK === !1) && t.registered[i] === void 0 && (t.registered[i] = n.styles);
}, f0 = function(t, n, o) {
  d0(t, n, o);
  var i = t.key + "-" + n.name;
  if (t.inserted[n.name] === void 0) {
    var s = n;
    do
      t.insert(n === s ? "." + i : "", s, t.sheet, !0), s = s.next;
    while (s !== void 0);
  }
};
function BK(e) {
  for (var t = 0, n, o = 0, i = e.length; i >= 4; ++o, i -= 4)
    n = e.charCodeAt(o) & 255 | (e.charCodeAt(++o) & 255) << 8 | (e.charCodeAt(++o) & 255) << 16 | (e.charCodeAt(++o) & 255) << 24, n = /* Math.imul(k, m): */
    (n & 65535) * 1540483477 + ((n >>> 16) * 59797 << 16), n ^= /* k >>> r: */
    n >>> 24, t = /* Math.imul(k, m): */
    (n & 65535) * 1540483477 + ((n >>> 16) * 59797 << 16) ^ /* Math.imul(h, m): */
    (t & 65535) * 1540483477 + ((t >>> 16) * 59797 << 16);
  switch (i) {
    case 3:
      t ^= (e.charCodeAt(o + 2) & 255) << 16;
    case 2:
      t ^= (e.charCodeAt(o + 1) & 255) << 8;
    case 1:
      t ^= e.charCodeAt(o) & 255, t = /* Math.imul(h, m): */
      (t & 65535) * 1540483477 + ((t >>> 16) * 59797 << 16);
  }
  return t ^= t >>> 13, t = /* Math.imul(h, m): */
  (t & 65535) * 1540483477 + ((t >>> 16) * 59797 << 16), ((t ^ t >>> 15) >>> 0).toString(36);
}
var UK = {
  animationIterationCount: 1,
  aspectRatio: 1,
  borderImageOutset: 1,
  borderImageSlice: 1,
  borderImageWidth: 1,
  boxFlex: 1,
  boxFlexGroup: 1,
  boxOrdinalGroup: 1,
  columnCount: 1,
  columns: 1,
  flex: 1,
  flexGrow: 1,
  flexPositive: 1,
  flexShrink: 1,
  flexNegative: 1,
  flexOrder: 1,
  gridRow: 1,
  gridRowEnd: 1,
  gridRowSpan: 1,
  gridRowStart: 1,
  gridColumn: 1,
  gridColumnEnd: 1,
  gridColumnSpan: 1,
  gridColumnStart: 1,
  msGridRow: 1,
  msGridRowSpan: 1,
  msGridColumn: 1,
  msGridColumnSpan: 1,
  fontWeight: 1,
  lineHeight: 1,
  opacity: 1,
  order: 1,
  orphans: 1,
  scale: 1,
  tabSize: 1,
  widows: 1,
  zIndex: 1,
  zoom: 1,
  WebkitLineClamp: 1,
  // SVG-related properties
  fillOpacity: 1,
  floodOpacity: 1,
  stopOpacity: 1,
  strokeDasharray: 1,
  strokeDashoffset: 1,
  strokeMiterlimit: 1,
  strokeOpacity: 1,
  strokeWidth: 1
}, VK = /[A-Z]|^ms/g, WK = /_EMO_([^_]+?)_([^]*?)_EMO_/g, oE = function(t) {
  return t.charCodeAt(1) === 45;
}, N2 = function(t) {
  return t != null && typeof t != "boolean";
}, Jm = /* @__PURE__ */ Xx(function(e) {
  return oE(e) ? e : e.replace(VK, "-$&").toLowerCase();
}), F2 = function(t, n) {
  switch (t) {
    case "animation":
    case "animationName":
      if (typeof n == "string")
        return n.replace(WK, function(o, i, s) {
          return qo = {
            name: i,
            styles: s,
            next: qo
          }, i;
        });
  }
  return UK[t] !== 1 && !oE(t) && typeof n == "number" && n !== 0 ? n + "px" : n;
};
function dc(e, t, n) {
  if (n == null)
    return "";
  var o = n;
  if (o.__emotion_styles !== void 0)
    return o;
  switch (typeof n) {
    case "boolean":
      return "";
    case "object": {
      var i = n;
      if (i.anim === 1)
        return qo = {
          name: i.name,
          styles: i.styles,
          next: qo
        }, i.name;
      var s = n;
      if (s.styles !== void 0) {
        var l = s.next;
        if (l !== void 0)
          for (; l !== void 0; )
            qo = {
              name: l.name,
              styles: l.styles,
              next: qo
            }, l = l.next;
        var u = s.styles + ";";
        return u;
      }
      return qK(e, t, n);
    }
    case "function": {
      if (e !== void 0) {
        var c = qo, f = n(e);
        return qo = c, dc(e, t, f);
      }
      break;
    }
  }
  var p = n;
  if (t == null)
    return p;
  var g = t[p];
  return g !== void 0 ? g : p;
}
function qK(e, t, n) {
  var o = "";
  if (Array.isArray(n))
    for (var i = 0; i < n.length; i++)
      o += dc(e, t, n[i]) + ";";
  else
    for (var s in n) {
      var l = n[s];
      if (typeof l != "object") {
        var u = l;
        t != null && t[u] !== void 0 ? o += s + "{" + t[u] + "}" : N2(u) && (o += Jm(s) + ":" + F2(s, u) + ";");
      } else if (Array.isArray(l) && typeof l[0] == "string" && (t == null || t[l[0]] === void 0))
        for (var c = 0; c < l.length; c++)
          N2(l[c]) && (o += Jm(s) + ":" + F2(s, l[c]) + ";");
      else {
        var f = dc(e, t, l);
        switch (s) {
          case "animation":
          case "animationName": {
            o += Jm(s) + ":" + f + ";";
            break;
          }
          default:
            o += s + "{" + f + "}";
        }
      }
    }
  return o;
}
var L2 = /label:\s*([^\s;{]+)\s*(;|$)/g, qo;
function jc(e, t, n) {
  if (e.length === 1 && typeof e[0] == "object" && e[0] !== null && e[0].styles !== void 0)
    return e[0];
  var o = !0, i = "";
  qo = void 0;
  var s = e[0];
  if (s == null || s.raw === void 0)
    o = !1, i += dc(n, t, s);
  else {
    var l = s;
    i += l[0];
  }
  for (var u = 1; u < e.length; u++)
    if (i += dc(n, t, e[u]), o) {
      var c = s;
      i += c[u];
    }
  L2.lastIndex = 0;
  for (var f = "", p; (p = L2.exec(i)) !== null; )
    f += "-" + p[1];
  var g = BK(i) + f;
  return {
    name: g,
    styles: i,
    next: qo
  };
}
var KK = function(t) {
  return t();
}, iE = Zu.useInsertionEffect ? Zu.useInsertionEffect : !1, HK = iE || KK, D2 = iE || z.useLayoutEffect, aE = /* @__PURE__ */ z.createContext(
  // we're doing this to avoid preconstruct's dead code elimination in this one case
  // because this module is primarily intended for the browser and node
  // but it's also required in react native and similar environments sometimes
  // and we could have a special build just for that
  // but this is much easier and the native packages
  // might use a different theme context in the future anyway
  typeof HTMLElement < "u" ? /* @__PURE__ */ MK({
    key: "css"
  }) : null
);
aE.Provider;
var p0 = function(t) {
  return /* @__PURE__ */ z.forwardRef(function(n, o) {
    var i = z.useContext(aE);
    return t(n, i, o);
  });
}, Tp = /* @__PURE__ */ z.createContext({}), h0 = {}.hasOwnProperty, Og = "__EMOTION_TYPE_PLEASE_DO_NOT_USE__", GK = function(t, n) {
  var o = {};
  for (var i in n)
    h0.call(n, i) && (o[i] = n[i]);
  return o[Og] = t, o;
}, JK = function(t) {
  var n = t.cache, o = t.serialized, i = t.isStringTag;
  return d0(n, o, i), HK(function() {
    return f0(n, o, i);
  }), null;
}, YK = /* @__PURE__ */ p0(function(e, t, n) {
  var o = e.css;
  typeof o == "string" && t.registered[o] !== void 0 && (o = t.registered[o]);
  var i = e[Og], s = [o], l = "";
  typeof e.className == "string" ? l = rE(t.registered, s, e.className) : e.className != null && (l = e.className + " ");
  var u = jc(s, void 0, z.useContext(Tp));
  l += t.key + "-" + u.name;
  var c = {};
  for (var f in e)
    h0.call(e, f) && f !== "css" && f !== Og && (c[f] = e[f]);
  return c.className = l, n && (c.ref = n), /* @__PURE__ */ z.createElement(z.Fragment, null, /* @__PURE__ */ z.createElement(JK, {
    cache: t,
    serialized: u,
    isStringTag: typeof i == "string"
  }), /* @__PURE__ */ z.createElement(i, c));
}), QK = YK, XK = function(t, n) {
  var o = arguments;
  if (n == null || !h0.call(n, "css"))
    return z.createElement.apply(void 0, o);
  var i = o.length, s = new Array(i);
  s[0] = QK, s[1] = GK(t, n);
  for (var l = 2; l < i; l++)
    s[l] = o[l];
  return z.createElement.apply(null, s);
};
(function(e) {
  var t;
  t || (t = e.JSX || (e.JSX = {}));
})(XK);
var ZK = /* @__PURE__ */ p0(function(e, t) {
  var n = e.styles, o = jc([n], void 0, z.useContext(Tp)), i = z.useRef();
  return D2(function() {
    var s = t.key + "-global", l = new t.sheet.constructor({
      key: s,
      nonce: t.sheet.nonce,
      container: t.sheet.container,
      speedy: t.sheet.isSpeedy
    }), u = !1, c = document.querySelector('style[data-emotion="' + s + " " + o.name + '"]');
    return t.sheet.tags.length && (l.before = t.sheet.tags[0]), c !== null && (u = !0, c.setAttribute("data-emotion", s), l.hydrate([c])), i.current = [l, u], function() {
      l.flush();
    };
  }, [t]), D2(function() {
    var s = i.current, l = s[0], u = s[1];
    if (u) {
      s[1] = !1;
      return;
    }
    if (o.next !== void 0 && f0(t, o.next, !0), l.tags.length) {
      var c = l.tags[l.tags.length - 1].nextElementSibling;
      l.before = c, l.flush();
    }
    t.insert("", o, l, !1);
  }, [t, o.name]), null;
});
function jl() {
  for (var e = arguments.length, t = new Array(e), n = 0; n < e; n++)
    t[n] = arguments[n];
  return jc(t);
}
function ua() {
  var e = jl.apply(void 0, arguments), t = "animation-" + e.name;
  return {
    name: t,
    styles: "@keyframes " + t + "{" + e.styles + "}",
    anim: 1,
    toString: function() {
      return "_EMO_" + this.name + "_" + this.styles + "_EMO_";
    }
  };
}
var eH = function(t) {
  return t();
}, tH = Zu.useInsertionEffect ? Zu.useInsertionEffect : !1, nH = tH || eH, rH = /^((children|dangerouslySetInnerHTML|key|ref|autoFocus|defaultValue|defaultChecked|innerHTML|suppressContentEditableWarning|suppressHydrationWarning|valueLink|abbr|accept|acceptCharset|accessKey|action|allow|allowUserMedia|allowPaymentRequest|allowFullScreen|allowTransparency|alt|async|autoComplete|autoPlay|capture|cellPadding|cellSpacing|challenge|charSet|checked|cite|classID|className|cols|colSpan|content|contentEditable|contextMenu|controls|controlsList|coords|crossOrigin|data|dateTime|decoding|default|defer|dir|disabled|disablePictureInPicture|disableRemotePlayback|download|draggable|encType|enterKeyHint|fetchpriority|fetchPriority|form|formAction|formEncType|formMethod|formNoValidate|formTarget|frameBorder|headers|height|hidden|high|href|hrefLang|htmlFor|httpEquiv|id|inputMode|integrity|is|keyParams|keyType|kind|label|lang|list|loading|loop|low|marginHeight|marginWidth|max|maxLength|media|mediaGroup|method|min|minLength|multiple|muted|name|nonce|noValidate|open|optimum|pattern|placeholder|playsInline|poster|preload|profile|radioGroup|readOnly|referrerPolicy|rel|required|reversed|role|rows|rowSpan|sandbox|scope|scoped|scrolling|seamless|selected|shape|size|sizes|slot|span|spellCheck|src|srcDoc|srcLang|srcSet|start|step|style|summary|tabIndex|target|title|translate|type|useMap|value|width|wmode|wrap|about|datatype|inlist|prefix|property|resource|typeof|vocab|autoCapitalize|autoCorrect|autoSave|color|incremental|fallback|inert|itemProp|itemScope|itemType|itemID|itemRef|on|option|results|security|unselectable|accentHeight|accumulate|additive|alignmentBaseline|allowReorder|alphabetic|amplitude|arabicForm|ascent|attributeName|attributeType|autoReverse|azimuth|baseFrequency|baselineShift|baseProfile|bbox|begin|bias|by|calcMode|capHeight|clip|clipPathUnits|clipPath|clipRule|colorInterpolation|colorInterpolationFilters|colorProfile|colorRendering|contentScriptType|contentStyleType|cursor|cx|cy|d|decelerate|descent|diffuseConstant|direction|display|divisor|dominantBaseline|dur|dx|dy|edgeMode|elevation|enableBackground|end|exponent|externalResourcesRequired|fill|fillOpacity|fillRule|filter|filterRes|filterUnits|floodColor|floodOpacity|focusable|fontFamily|fontSize|fontSizeAdjust|fontStretch|fontStyle|fontVariant|fontWeight|format|from|fr|fx|fy|g1|g2|glyphName|glyphOrientationHorizontal|glyphOrientationVertical|glyphRef|gradientTransform|gradientUnits|hanging|horizAdvX|horizOriginX|ideographic|imageRendering|in|in2|intercept|k|k1|k2|k3|k4|kernelMatrix|kernelUnitLength|kerning|keyPoints|keySplines|keyTimes|lengthAdjust|letterSpacing|lightingColor|limitingConeAngle|local|markerEnd|markerMid|markerStart|markerHeight|markerUnits|markerWidth|mask|maskContentUnits|maskUnits|mathematical|mode|numOctaves|offset|opacity|operator|order|orient|orientation|origin|overflow|overlinePosition|overlineThickness|panose1|paintOrder|pathLength|patternContentUnits|patternTransform|patternUnits|pointerEvents|points|pointsAtX|pointsAtY|pointsAtZ|preserveAlpha|preserveAspectRatio|primitiveUnits|r|radius|refX|refY|renderingIntent|repeatCount|repeatDur|requiredExtensions|requiredFeatures|restart|result|rotate|rx|ry|scale|seed|shapeRendering|slope|spacing|specularConstant|specularExponent|speed|spreadMethod|startOffset|stdDeviation|stemh|stemv|stitchTiles|stopColor|stopOpacity|strikethroughPosition|strikethroughThickness|string|stroke|strokeDasharray|strokeDashoffset|strokeLinecap|strokeLinejoin|strokeMiterlimit|strokeOpacity|strokeWidth|surfaceScale|systemLanguage|tableValues|targetX|targetY|textAnchor|textDecoration|textRendering|textLength|to|transform|u1|u2|underlinePosition|underlineThickness|unicode|unicodeBidi|unicodeRange|unitsPerEm|vAlphabetic|vHanging|vIdeographic|vMathematical|values|vectorEffect|version|vertAdvY|vertOriginX|vertOriginY|viewBox|viewTarget|visibility|widths|wordSpacing|writingMode|x|xHeight|x1|x2|xChannelSelector|xlinkActuate|xlinkArcrole|xlinkHref|xlinkRole|xlinkShow|xlinkTitle|xlinkType|xmlBase|xmlns|xmlnsXlink|xmlLang|xmlSpace|y|y1|y2|yChannelSelector|z|zoomAndPan|for|class|autofocus)|(([Dd][Aa][Tt][Aa]|[Aa][Rr][Ii][Aa]|x)-.*))$/, oH = /* @__PURE__ */ Xx(
  function(e) {
    return rH.test(e) || e.charCodeAt(0) === 111 && e.charCodeAt(1) === 110 && e.charCodeAt(2) < 91;
  }
  /* Z+1 */
), iH = oH, aH = function(t) {
  return t !== "theme";
}, z2 = function(t) {
  return typeof t == "string" && // 96 is one less than the char code
  // for "a" so this is checking that
  // it's a lowercase character
  t.charCodeAt(0) > 96 ? iH : aH;
}, B2 = function(t, n, o) {
  var i;
  if (n) {
    var s = n.shouldForwardProp;
    i = t.__emotion_forwardProp && s ? function(l) {
      return t.__emotion_forwardProp(l) && s(l);
    } : s;
  }
  return typeof i != "function" && o && (i = t.__emotion_forwardProp), i;
}, sH = function(t) {
  var n = t.cache, o = t.serialized, i = t.isStringTag;
  return d0(n, o, i), nH(function() {
    return f0(n, o, i);
  }), null;
}, lH = function e(t, n) {
  var o = t.__emotion_real === t, i = o && t.__emotion_base || t, s, l;
  n !== void 0 && (s = n.label, l = n.target);
  var u = B2(t, n, o), c = u || z2(i), f = !c("as");
  return function() {
    var p = arguments, g = o && t.__emotion_styles !== void 0 ? t.__emotion_styles.slice(0) : [];
    if (s !== void 0 && g.push("label:" + s + ";"), p[0] == null || p[0].raw === void 0)
      g.push.apply(g, p);
    else {
      var h = p[0];
      g.push(h[0]);
      for (var y = p.length, v = 1; v < y; v++)
        g.push(p[v], h[v]);
    }
    var C = p0(function($, S, w) {
      var _ = f && $.as || i, b = "", k = [], E = $;
      if ($.theme == null) {
        E = {};
        for (var j in $)
          E[j] = $[j];
        E.theme = z.useContext(Tp);
      }
      typeof $.className == "string" ? b = rE(S.registered, k, $.className) : $.className != null && (b = $.className + " ");
      var L = jc(g.concat(k), S.registered, E);
      b += S.key + "-" + L.name, l !== void 0 && (b += " " + l);
      var B = f && u === void 0 ? z2(_) : c, O = {};
      for (var V in $)
        f && V === "as" || B(V) && (O[V] = $[V]);
      return O.className = b, w && (O.ref = w), /* @__PURE__ */ z.createElement(z.Fragment, null, /* @__PURE__ */ z.createElement(sH, {
        cache: S,
        serialized: L,
        isStringTag: typeof _ == "string"
      }), /* @__PURE__ */ z.createElement(_, O));
    });
    return C.displayName = s !== void 0 ? s : "Styled(" + (typeof i == "string" ? i : i.displayName || i.name || "Component") + ")", C.defaultProps = t.defaultProps, C.__emotion_real = C, C.__emotion_base = i, C.__emotion_styles = g, C.__emotion_forwardProp = u, Object.defineProperty(C, "toString", {
      value: function() {
        return "." + l;
      }
    }), C.withComponent = function($, S) {
      var w = e($, zf({}, n, S, {
        shouldForwardProp: B2(C, S, !0)
      }));
      return w.apply(void 0, g);
    }, C;
  };
}, uH = [
  "a",
  "abbr",
  "address",
  "area",
  "article",
  "aside",
  "audio",
  "b",
  "base",
  "bdi",
  "bdo",
  "big",
  "blockquote",
  "body",
  "br",
  "button",
  "canvas",
  "caption",
  "cite",
  "code",
  "col",
  "colgroup",
  "data",
  "datalist",
  "dd",
  "del",
  "details",
  "dfn",
  "dialog",
  "div",
  "dl",
  "dt",
  "em",
  "embed",
  "fieldset",
  "figcaption",
  "figure",
  "footer",
  "form",
  "h1",
  "h2",
  "h3",
  "h4",
  "h5",
  "h6",
  "head",
  "header",
  "hgroup",
  "hr",
  "html",
  "i",
  "iframe",
  "img",
  "input",
  "ins",
  "kbd",
  "keygen",
  "label",
  "legend",
  "li",
  "link",
  "main",
  "map",
  "mark",
  "marquee",
  "menu",
  "menuitem",
  "meta",
  "meter",
  "nav",
  "noscript",
  "object",
  "ol",
  "optgroup",
  "option",
  "output",
  "p",
  "param",
  "picture",
  "pre",
  "progress",
  "q",
  "rp",
  "rt",
  "ruby",
  "s",
  "samp",
  "script",
  "section",
  "select",
  "small",
  "source",
  "span",
  "strong",
  "style",
  "sub",
  "summary",
  "sup",
  "table",
  "tbody",
  "td",
  "textarea",
  "tfoot",
  "th",
  "thead",
  "time",
  "title",
  "tr",
  "track",
  "u",
  "ul",
  "var",
  "video",
  "wbr",
  // SVG
  "circle",
  "clipPath",
  "defs",
  "ellipse",
  "foreignObject",
  "g",
  "image",
  "line",
  "linearGradient",
  "mask",
  "path",
  "pattern",
  "polygon",
  "polyline",
  "radialGradient",
  "rect",
  "stop",
  "svg",
  "text",
  "tspan"
], Ag = lH.bind(null);
uH.forEach(function(e) {
  Ag[e] = Ag(e);
});
function cH(e) {
  return e == null || Object.keys(e).length === 0;
}
function dH(e) {
  const {
    styles: t,
    defaultTheme: n = {}
  } = e, o = typeof t == "function" ? (i) => t(cH(i) ? n : i) : t;
  return /* @__PURE__ */ A.jsx(ZK, {
    styles: o
  });
}
function sE(e, t) {
  return Ag(e, t);
}
function fH(e, t) {
  Array.isArray(e.__emotion_styles) && (e.__emotion_styles = t(e.__emotion_styles));
}
const U2 = [];
function V2(e) {
  return U2[0] = e, jc(U2);
}
const pH = (e) => {
  const t = Object.keys(e).map((n) => ({
    key: n,
    val: e[n]
  })) || [];
  return t.sort((n, o) => n.val - o.val), t.reduce((n, o) => ({
    ...n,
    [o.key]: o.val
  }), {});
};
function hH(e) {
  const {
    // The breakpoint **start** at this value.
    // For instance with the first breakpoint xs: [xs, sm).
    values: t = {
      xs: 0,
      // phone
      sm: 600,
      // tablet
      md: 900,
      // small laptop
      lg: 1200,
      // desktop
      xl: 1536
      // large screen
    },
    unit: n = "px",
    step: o = 5,
    ...i
  } = e, s = pH(t), l = Object.keys(s);
  function u(h) {
    return `@media (min-width:${typeof t[h] == "number" ? t[h] : h}${n})`;
  }
  function c(h) {
    return `@media (max-width:${(typeof t[h] == "number" ? t[h] : h) - o / 100}${n})`;
  }
  function f(h, y) {
    const v = l.indexOf(y);
    return `@media (min-width:${typeof t[h] == "number" ? t[h] : h}${n}) and (max-width:${(v !== -1 && typeof t[l[v]] == "number" ? t[l[v]] : y) - o / 100}${n})`;
  }
  function p(h) {
    return l.indexOf(h) + 1 < l.length ? f(h, l[l.indexOf(h) + 1]) : u(h);
  }
  function g(h) {
    const y = l.indexOf(h);
    return y === 0 ? u(l[1]) : y === l.length - 1 ? c(l[y]) : f(h, l[l.indexOf(h) + 1]).replace("@media", "@media not all and");
  }
  return {
    keys: l,
    values: s,
    up: u,
    down: c,
    between: f,
    only: p,
    not: g,
    unit: n,
    ...i
  };
}
const mH = {
  borderRadius: 4
};
function lE(e = 8, t = a0({
  spacing: e
})) {
  if (e.mui)
    return e;
  const n = (...o) => (o.length === 0 ? [1] : o).map((s) => {
    const l = t(s);
    return typeof l == "number" ? `${l}px` : l;
  }).join(" ");
  return n.mui = !0, n;
}
function gH(e, t) {
  var o;
  const n = this;
  if (n.vars) {
    if (!((o = n.colorSchemes) != null && o[e]) || typeof n.getColorSchemeSelector != "function")
      return {};
    let i = n.getColorSchemeSelector(e);
    return i === "&" ? t : ((i.includes("data-") || i.includes(".")) && (i = `*:where(${i.replace(/\s*&$/, "")}) &`), {
      [i]: t
    });
  }
  return n.palette.mode === e ? t : {};
}
function Mp(e = {}, ...t) {
  const {
    breakpoints: n = {},
    palette: o = {},
    spacing: i,
    shape: s = {},
    ...l
  } = e, u = hH(n), c = lE(i);
  let f = lr({
    breakpoints: u,
    direction: "ltr",
    components: {},
    // Inject component definitions.
    palette: {
      mode: "light",
      ...o
    },
    spacing: c,
    shape: {
      ...mH,
      ...s
    }
  }, l);
  return f = gq(f), f.applyStyles = gH, f = t.reduce((p, g) => lr(p, g), f), f.unstable_sxConfig = {
    ...Ac,
    ...l == null ? void 0 : l.unstable_sxConfig
  }, f.unstable_sx = function(g) {
    return Va({
      sx: g,
      theme: this
    });
  }, f;
}
function yH(e) {
  return Object.keys(e).length === 0;
}
function vH(e = null) {
  const t = z.useContext(Tp);
  return !t || yH(t) ? e : t;
}
const bH = Mp();
function Rc(e = bH) {
  return vH(e);
}
function SH({
  styles: e,
  themeId: t,
  defaultTheme: n = {}
}) {
  const o = Rc(n), i = typeof e == "function" ? e(t && o[t] || o) : e;
  return /* @__PURE__ */ A.jsx(dH, {
    styles: i
  });
}
function wH(e = {}) {
  const {
    themeId: t,
    defaultTheme: n,
    defaultClassName: o = "MuiBox-root",
    generateClassName: i
  } = e, s = sE("div", {
    shouldForwardProp: (u) => u !== "theme" && u !== "sx" && u !== "as"
  })(Va);
  return /* @__PURE__ */ z.forwardRef(function(c, f) {
    const p = Rc(n), {
      className: g,
      component: h = "div",
      ...y
    } = Ap(c);
    return /* @__PURE__ */ A.jsx(s, {
      as: h,
      ref: f,
      className: Ve(g, i ? i(o) : o),
      theme: t && p[t] || p,
      ...y
    });
  });
}
function uE(e) {
  const {
    variants: t,
    ...n
  } = e, o = {
    variants: t,
    style: V2(n),
    isProcessed: !0
  };
  return o.style === n || t && t.forEach((i) => {
    typeof i.style != "function" && (i.style = V2(i.style));
  }), o;
}
const $H = Mp();
function Ym(e) {
  return e !== "ownerState" && e !== "theme" && e !== "sx" && e !== "as";
}
function CH(e) {
  return e ? (t, n) => n[e] : null;
}
function xH(e, t, n) {
  e.theme = _H(e.theme) ? n : e.theme[t] || e.theme;
}
function $f(e, t) {
  const n = typeof t == "function" ? t(e) : t;
  if (Array.isArray(n))
    return n.flatMap((o) => $f(e, o));
  if (Array.isArray(n == null ? void 0 : n.variants)) {
    let o;
    if (n.isProcessed)
      o = n.style;
    else {
      const {
        variants: i,
        ...s
      } = n;
      o = s;
    }
    return cE(e, n.variants, [o]);
  }
  return n != null && n.isProcessed ? n.style : n;
}
function cE(e, t, n = []) {
  var i;
  let o;
  e: for (let s = 0; s < t.length; s += 1) {
    const l = t[s];
    if (typeof l.props == "function") {
      if (o ?? (o = {
        ...e,
        ...e.ownerState,
        ownerState: e.ownerState
      }), !l.props(o))
        continue;
    } else
      for (const u in l.props)
        if (e[u] !== l.props[u] && ((i = e.ownerState) == null ? void 0 : i[u]) !== l.props[u])
          continue e;
    typeof l.style == "function" ? (o ?? (o = {
      ...e,
      ...e.ownerState,
      ownerState: e.ownerState
    }), n.push(l.style(o))) : n.push(l.style);
  }
  return n;
}
function dE(e = {}) {
  const {
    themeId: t,
    defaultTheme: n = $H,
    rootShouldForwardProp: o = Ym,
    slotShouldForwardProp: i = Ym
  } = e;
  function s(u) {
    xH(u, t, n);
  }
  return (u, c = {}) => {
    fH(u, (k) => k.filter((E) => E !== Va));
    const {
      name: f,
      slot: p,
      skipVariantsResolver: g,
      skipSx: h,
      // TODO v6: remove `lowercaseFirstLetter()` in the next major release
      // For more details: https://github.com/mui/material-ui/pull/37908
      overridesResolver: y = CH(PH(p)),
      ...v
    } = c, C = g !== void 0 ? g : (
      // TODO v6: remove `Root` in the next major release
      // For more details: https://github.com/mui/material-ui/pull/37908
      p && p !== "Root" && p !== "root" || !1
    ), $ = h || !1;
    let S = Ym;
    p === "Root" || p === "root" ? S = o : p ? S = i : kH(u) && (S = void 0);
    const w = sE(u, {
      shouldForwardProp: S,
      label: EH(),
      ...v
    }), _ = (k) => {
      if (typeof k == "function" && k.__emotion_real !== k)
        return function(j) {
          return $f(j, k);
        };
      if (Ko(k)) {
        const E = uE(k);
        return E.variants ? function(L) {
          return $f(L, E);
        } : E.style;
      }
      return k;
    }, b = (...k) => {
      const E = [], j = k.map(_), L = [];
      if (E.push(s), f && y && L.push(function(W) {
        var ce, J;
        const Q = (J = (ce = W.theme.components) == null ? void 0 : ce[f]) == null ? void 0 : J.styleOverrides;
        if (!Q)
          return null;
        const X = {};
        for (const ie in Q)
          X[ie] = $f(W, Q[ie]);
        return y(W, X);
      }), f && !C && L.push(function(W) {
        var X, ce;
        const q = W.theme, Q = (ce = (X = q == null ? void 0 : q.components) == null ? void 0 : X[f]) == null ? void 0 : ce.variants;
        return Q ? cE(W, Q) : null;
      }), $ || L.push(Va), Array.isArray(j[0])) {
        const V = j.shift(), W = new Array(E.length).fill(""), q = new Array(L.length).fill("");
        let Q;
        Q = [...W, ...V, ...q], Q.raw = [...W, ...V.raw, ...q], E.unshift(Q);
      }
      const B = [...E, ...j, ...L], O = w(...B);
      return u.muiName && (O.muiName = u.muiName), O;
    };
    return w.withConfig && (b.withConfig = w.withConfig), b;
  };
}
function EH(e, t) {
  return void 0;
}
function _H(e) {
  for (const t in e)
    return !1;
  return !0;
}
function kH(e) {
  return typeof e == "string" && // 96 is one less than the char code
  // for "a" so this is checking that
  // it's a lowercase character
  e.charCodeAt(0) > 96;
}
function PH(e) {
  return e && e.charAt(0).toLowerCase() + e.slice(1);
}
const OH = dE();
function AH(e) {
  const {
    theme: t,
    name: n,
    props: o
  } = e;
  return !t || !t.components || !t.components[n] || !t.components[n].defaultProps ? o : lc(t.components[n].defaultProps, o);
}
function IH({
  props: e,
  name: t,
  defaultTheme: n,
  themeId: o
}) {
  let i = Rc(n);
  return o && (i = i[o] || i), AH({
    theme: i,
    name: t,
    props: e
  });
}
function m0(e, t = 0, n = 1) {
  return Vs(e, t, n);
}
function jH(e) {
  e = e.slice(1);
  const t = new RegExp(`.{1,${e.length >= 6 ? 2 : 1}}`, "g");
  let n = e.match(t);
  return n && n[0].length === 1 && (n = n.map((o) => o + o)), n ? `rgb${n.length === 4 ? "a" : ""}(${n.map((o, i) => i < 3 ? parseInt(o, 16) : Math.round(parseInt(o, 16) / 255 * 1e3) / 1e3).join(", ")})` : "";
}
function aa(e) {
  if (e.type)
    return e;
  if (e.charAt(0) === "#")
    return aa(jH(e));
  const t = e.indexOf("("), n = e.substring(0, t);
  if (!["rgb", "rgba", "hsl", "hsla", "color"].includes(n))
    throw new Error(Ci(9, e));
  let o = e.substring(t + 1, e.length - 1), i;
  if (n === "color") {
    if (o = o.split(" "), i = o.shift(), o.length === 4 && o[3].charAt(0) === "/" && (o[3] = o[3].slice(1)), !["srgb", "display-p3", "a98-rgb", "prophoto-rgb", "rec-2020"].includes(i))
      throw new Error(Ci(10, i));
  } else
    o = o.split(",");
  return o = o.map((s) => parseFloat(s)), {
    type: n,
    values: o,
    colorSpace: i
  };
}
const RH = (e) => {
  const t = aa(e);
  return t.values.slice(0, 3).map((n, o) => t.type.includes("hsl") && o !== 0 ? `${n}%` : n).join(" ");
}, Mu = (e, t) => {
  try {
    return RH(e);
  } catch {
    return e;
  }
};
function Np(e) {
  const {
    type: t,
    colorSpace: n
  } = e;
  let {
    values: o
  } = e;
  return t.includes("rgb") ? o = o.map((i, s) => s < 3 ? parseInt(i, 10) : i) : t.includes("hsl") && (o[1] = `${o[1]}%`, o[2] = `${o[2]}%`), t.includes("color") ? o = `${n} ${o.join(" ")}` : o = `${o.join(", ")}`, `${t}(${o})`;
}
function fE(e) {
  e = aa(e);
  const {
    values: t
  } = e, n = t[0], o = t[1] / 100, i = t[2] / 100, s = o * Math.min(i, 1 - i), l = (f, p = (f + n / 30) % 12) => i - s * Math.max(Math.min(p - 3, 9 - p, 1), -1);
  let u = "rgb";
  const c = [Math.round(l(0) * 255), Math.round(l(8) * 255), Math.round(l(4) * 255)];
  return e.type === "hsla" && (u += "a", c.push(t[3])), Np({
    type: u,
    values: c
  });
}
function Ig(e) {
  e = aa(e);
  let t = e.type === "hsl" || e.type === "hsla" ? aa(fE(e)).values : e.values;
  return t = t.map((n) => (e.type !== "color" && (n /= 255), n <= 0.03928 ? n / 12.92 : ((n + 0.055) / 1.055) ** 2.4)), Number((0.2126 * t[0] + 0.7152 * t[1] + 0.0722 * t[2]).toFixed(3));
}
function TH(e, t) {
  const n = Ig(e), o = Ig(t);
  return (Math.max(n, o) + 0.05) / (Math.min(n, o) + 0.05);
}
function En(e, t) {
  return e = aa(e), t = m0(t), (e.type === "rgb" || e.type === "hsl") && (e.type += "a"), e.type === "color" ? e.values[3] = `/${t}` : e.values[3] = t, Np(e);
}
function Yd(e, t, n) {
  try {
    return En(e, t);
  } catch {
    return e;
  }
}
function ul(e, t) {
  if (e = aa(e), t = m0(t), e.type.includes("hsl"))
    e.values[2] *= 1 - t;
  else if (e.type.includes("rgb") || e.type.includes("color"))
    for (let n = 0; n < 3; n += 1)
      e.values[n] *= 1 - t;
  return Np(e);
}
function Xt(e, t, n) {
  try {
    return ul(e, t);
  } catch {
    return e;
  }
}
function cl(e, t) {
  if (e = aa(e), t = m0(t), e.type.includes("hsl"))
    e.values[2] += (100 - e.values[2]) * t;
  else if (e.type.includes("rgb"))
    for (let n = 0; n < 3; n += 1)
      e.values[n] += (255 - e.values[n]) * t;
  else if (e.type.includes("color"))
    for (let n = 0; n < 3; n += 1)
      e.values[n] += (1 - e.values[n]) * t;
  return Np(e);
}
function Zt(e, t, n) {
  try {
    return cl(e, t);
  } catch {
    return e;
  }
}
function MH(e, t = 0.15) {
  return Ig(e) > 0.5 ? ul(e, t) : cl(e, t);
}
function Qd(e, t, n) {
  try {
    return MH(e, t);
  } catch {
    return e;
  }
}
const NH = /* @__PURE__ */ z.createContext(), g0 = () => z.useContext(NH) ?? !1, FH = /* @__PURE__ */ z.createContext(void 0);
function LH(e) {
  const {
    theme: t,
    name: n,
    props: o
  } = e;
  if (!t || !t.components || !t.components[n])
    return o;
  const i = t.components[n];
  return i.defaultProps ? lc(i.defaultProps, o) : !i.styleOverrides && !i.variants ? lc(i, o) : o;
}
function DH({
  props: e,
  name: t
}) {
  const n = z.useContext(FH);
  return LH({
    props: e,
    name: t,
    theme: {
      components: n
    }
  });
}
const W2 = {
  theme: void 0
};
function zH(e) {
  let t, n;
  return function(i) {
    let s = t;
    return (s === void 0 || i.theme !== n) && (W2.theme = i.theme, s = uE(e(W2)), t = s, n = i.theme), s;
  };
}
function BH(e = "") {
  function t(...o) {
    if (!o.length)
      return "";
    const i = o[0];
    return typeof i == "string" && !i.match(/(#|\(|\)|(-?(\d*\.)?\d+)(px|em|%|ex|ch|rem|vw|vh|vmin|vmax|cm|mm|in|pt|pc))|^(-?(\d*\.)?\d+)$|(\d+ \d+ \d+)/) ? `, var(--${e ? `${e}-` : ""}${i}${t(...o.slice(1))})` : `, ${i}`;
  }
  return (o, ...i) => `var(--${e ? `${e}-` : ""}${o}${t(...i)})`;
}
const q2 = (e, t, n, o = []) => {
  let i = e;
  t.forEach((s, l) => {
    l === t.length - 1 ? Array.isArray(i) ? i[Number(s)] = n : i && typeof i == "object" && (i[s] = n) : i && typeof i == "object" && (i[s] || (i[s] = o.includes(s) ? [] : {}), i = i[s]);
  });
}, UH = (e, t, n) => {
  function o(i, s = [], l = []) {
    Object.entries(i).forEach(([u, c]) => {
      (!n || !n([...s, u])) && c != null && (typeof c == "object" && Object.keys(c).length > 0 ? o(c, [...s, u], Array.isArray(c) ? [...l, u] : l) : t([...s, u], c, l));
    });
  }
  o(e);
}, VH = (e, t) => typeof t == "number" ? ["lineHeight", "fontWeight", "opacity", "zIndex"].some((o) => e.includes(o)) || e[e.length - 1].toLowerCase().includes("opacity") ? t : `${t}px` : t;
function Qm(e, t) {
  const {
    prefix: n,
    shouldSkipGeneratingVar: o
  } = t || {}, i = {}, s = {}, l = {};
  return UH(
    e,
    (u, c, f) => {
      if ((typeof c == "string" || typeof c == "number") && (!o || !o(u, c))) {
        const p = `--${n ? `${n}-` : ""}${u.join("-")}`, g = VH(u, c);
        Object.assign(i, {
          [p]: g
        }), q2(s, u, `var(${p})`, f), q2(l, u, `var(${p}, ${g})`, f);
      }
    },
    (u) => u[0] === "vars"
    // skip 'vars/*' paths
  ), {
    css: i,
    vars: s,
    varsWithDefaults: l
  };
}
function WH(e, t = {}) {
  const {
    getSelector: n = $,
    disableCssColorScheme: o,
    colorSchemeSelector: i
  } = t, {
    colorSchemes: s = {},
    components: l,
    defaultColorScheme: u = "light",
    ...c
  } = e, {
    vars: f,
    css: p,
    varsWithDefaults: g
  } = Qm(c, t);
  let h = g;
  const y = {}, {
    [u]: v,
    ...C
  } = s;
  if (Object.entries(C || {}).forEach(([_, b]) => {
    const {
      vars: k,
      css: E,
      varsWithDefaults: j
    } = Qm(b, t);
    h = lr(h, j), y[_] = {
      css: E,
      vars: k
    };
  }), v) {
    const {
      css: _,
      vars: b,
      varsWithDefaults: k
    } = Qm(v, t);
    h = lr(h, k), y[u] = {
      css: _,
      vars: b
    };
  }
  function $(_, b) {
    var E, j;
    let k = i;
    if (i === "class" && (k = ".%s"), i === "data" && (k = "[data-%s]"), i != null && i.startsWith("data-") && !i.includes("%s") && (k = `[${i}="%s"]`), _) {
      if (k === "media")
        return e.defaultColorScheme === _ ? ":root" : {
          [`@media (prefers-color-scheme: ${((j = (E = s[_]) == null ? void 0 : E.palette) == null ? void 0 : j.mode) || _})`]: {
            ":root": b
          }
        };
      if (k)
        return e.defaultColorScheme === _ ? `:root, ${k.replace("%s", String(_))}` : k.replace("%s", String(_));
    }
    return ":root";
  }
  return {
    vars: h,
    generateThemeVars: () => {
      let _ = {
        ...f
      };
      return Object.entries(y).forEach(([, {
        vars: b
      }]) => {
        _ = lr(_, b);
      }), _;
    },
    generateStyleSheets: () => {
      var L, B;
      const _ = [], b = e.defaultColorScheme || "light";
      function k(O, V) {
        Object.keys(V).length && _.push(typeof O == "string" ? {
          [O]: {
            ...V
          }
        } : O);
      }
      k(n(void 0, {
        ...p
      }), p);
      const {
        [b]: E,
        ...j
      } = y;
      if (E) {
        const {
          css: O
        } = E, V = (B = (L = s[b]) == null ? void 0 : L.palette) == null ? void 0 : B.mode, W = !o && V ? {
          colorScheme: V,
          ...O
        } : {
          ...O
        };
        k(n(b, {
          ...W
        }), W);
      }
      return Object.entries(j).forEach(([O, {
        css: V
      }]) => {
        var Q, X;
        const W = (X = (Q = s[O]) == null ? void 0 : Q.palette) == null ? void 0 : X.mode, q = !o && W ? {
          colorScheme: W,
          ...V
        } : {
          ...V
        };
        k(n(O, {
          ...q
        }), q);
      }), _;
    }
  };
}
function qH(e) {
  return function(n) {
    return e === "media" ? `@media (prefers-color-scheme: ${n})` : e ? e.startsWith("data-") && !e.includes("%s") ? `[${e}="${n}"] &` : e === "class" ? `.${n} &` : e === "data" ? `[data-${n}] &` : `${e.replace("%s", n)} &` : "&";
  };
}
const KH = (e, t) => e.filter((n) => t.includes(n)), Rl = (e, t, n) => {
  const o = e.keys[0];
  Array.isArray(t) ? t.forEach((i, s) => {
    n((l, u) => {
      s <= e.keys.length - 1 && (s === 0 ? Object.assign(l, u) : l[e.up(e.keys[s])] = u);
    }, i);
  }) : t && typeof t == "object" ? (Object.keys(t).length > e.keys.length ? e.keys : KH(e.keys, Object.keys(t))).forEach((s) => {
    if (e.keys.includes(s)) {
      const l = t[s];
      l !== void 0 && n((u, c) => {
        o === s ? Object.assign(u, c) : u[e.up(s)] = c;
      }, l);
    }
  }) : (typeof t == "number" || typeof t == "string") && n((i, s) => {
    Object.assign(i, s);
  }, t);
};
function Uf(e) {
  return `--Grid-${e}Spacing`;
}
function Fp(e) {
  return `--Grid-parent-${e}Spacing`;
}
const K2 = "--Grid-columns", Xs = "--Grid-parent-columns", HH = ({
  theme: e,
  ownerState: t
}) => {
  const n = {};
  return Rl(e.breakpoints, t.size, (o, i) => {
    let s = {};
    i === "grow" && (s = {
      flexBasis: 0,
      flexGrow: 1,
      maxWidth: "100%"
    }), i === "auto" && (s = {
      flexBasis: "auto",
      flexGrow: 0,
      flexShrink: 0,
      maxWidth: "none",
      width: "auto"
    }), typeof i == "number" && (s = {
      flexGrow: 0,
      flexBasis: "auto",
      width: `calc(100% * ${i} / var(${Xs}) - (var(${Xs}) - ${i}) * (var(${Fp("column")}) / var(${Xs})))`
    }), o(n, s);
  }), n;
}, GH = ({
  theme: e,
  ownerState: t
}) => {
  const n = {};
  return Rl(e.breakpoints, t.offset, (o, i) => {
    let s = {};
    i === "auto" && (s = {
      marginLeft: "auto"
    }), typeof i == "number" && (s = {
      marginLeft: i === 0 ? "0px" : `calc(100% * ${i} / var(${Xs}) + var(${Fp("column")}) * ${i} / var(${Xs}))`
    }), o(n, s);
  }), n;
}, JH = ({
  theme: e,
  ownerState: t
}) => {
  if (!t.container)
    return {};
  const n = {
    [K2]: 12
  };
  return Rl(e.breakpoints, t.columns, (o, i) => {
    const s = i ?? 12;
    o(n, {
      [K2]: s,
      "> *": {
        [Xs]: s
      }
    });
  }), n;
}, YH = ({
  theme: e,
  ownerState: t
}) => {
  if (!t.container)
    return {};
  const n = {};
  return Rl(e.breakpoints, t.rowSpacing, (o, i) => {
    var l;
    const s = typeof i == "string" ? i : (l = e.spacing) == null ? void 0 : l.call(e, i);
    o(n, {
      [Uf("row")]: s,
      "> *": {
        [Fp("row")]: s
      }
    });
  }), n;
}, QH = ({
  theme: e,
  ownerState: t
}) => {
  if (!t.container)
    return {};
  const n = {};
  return Rl(e.breakpoints, t.columnSpacing, (o, i) => {
    var l;
    const s = typeof i == "string" ? i : (l = e.spacing) == null ? void 0 : l.call(e, i);
    o(n, {
      [Uf("column")]: s,
      "> *": {
        [Fp("column")]: s
      }
    });
  }), n;
}, XH = ({
  theme: e,
  ownerState: t
}) => {
  if (!t.container)
    return {};
  const n = {};
  return Rl(e.breakpoints, t.direction, (o, i) => {
    o(n, {
      flexDirection: i
    });
  }), n;
}, ZH = ({
  ownerState: e
}) => ({
  minWidth: 0,
  boxSizing: "border-box",
  ...e.container && {
    display: "flex",
    flexWrap: "wrap",
    ...e.wrap && e.wrap !== "wrap" && {
      flexWrap: e.wrap
    },
    gap: `var(${Uf("row")}) var(${Uf("column")})`
  }
}), eG = (e) => {
  const t = [];
  return Object.entries(e).forEach(([n, o]) => {
    o !== !1 && o !== void 0 && t.push(`grid-${n}-${String(o)}`);
  }), t;
}, tG = (e, t = "xs") => {
  function n(o) {
    return o === void 0 ? !1 : typeof o == "string" && !Number.isNaN(Number(o)) || typeof o == "number" && o > 0;
  }
  if (n(e))
    return [`spacing-${t}-${String(e)}`];
  if (typeof e == "object" && !Array.isArray(e)) {
    const o = [];
    return Object.entries(e).forEach(([i, s]) => {
      n(s) && o.push(`spacing-${i}-${String(s)}`);
    }), o;
  }
  return [];
}, nG = (e) => e === void 0 ? [] : typeof e == "object" ? Object.entries(e).map(([t, n]) => `direction-${t}-${n}`) : [`direction-xs-${String(e)}`];
function rG(e, t) {
  e.item !== void 0 && delete e.item, e.zeroMinWidth !== void 0 && delete e.zeroMinWidth, t.keys.forEach((n) => {
    e[n] !== void 0 && delete e[n];
  });
}
const oG = Mp(), iG = OH("div", {
  name: "MuiGrid",
  slot: "Root",
  overridesResolver: (e, t) => t.root
});
function aG(e) {
  return IH({
    props: e,
    name: "MuiGrid",
    defaultTheme: oG
  });
}
function sG(e = {}) {
  const {
    // This will allow adding custom styled fn (for example for custom sx style function)
    createStyledComponent: t = iG,
    useThemeProps: n = aG,
    useTheme: o = Rc,
    componentName: i = "MuiGrid"
  } = e, s = (f, p) => {
    const {
      container: g,
      direction: h,
      spacing: y,
      wrap: v,
      size: C
    } = f, $ = {
      root: ["root", g && "container", v !== "wrap" && `wrap-xs-${String(v)}`, ...nG(h), ...eG(C), ...g ? tG(y, p.breakpoints.keys[0]) : []]
    };
    return mt($, (S) => ht(i, S), {});
  };
  function l(f, p, g = () => !0) {
    const h = {};
    return f === null || (Array.isArray(f) ? f.forEach((y, v) => {
      y !== null && g(y) && p.keys[v] && (h[p.keys[v]] = y);
    }) : typeof f == "object" ? Object.keys(f).forEach((y) => {
      const v = f[y];
      v != null && g(v) && (h[y] = v);
    }) : h[p.keys[0]] = f), h;
  }
  const u = t(JH, QH, YH, HH, XH, ZH, GH), c = /* @__PURE__ */ z.forwardRef(function(p, g) {
    const h = o(), y = n(p), v = Ap(y);
    rG(v, h.breakpoints);
    const {
      className: C,
      children: $,
      columns: S = 12,
      container: w = !1,
      component: _ = "div",
      direction: b = "row",
      wrap: k = "wrap",
      size: E = {},
      offset: j = {},
      spacing: L = 0,
      rowSpacing: B = L,
      columnSpacing: O = L,
      unstable_level: V = 0,
      ...W
    } = v, q = l(E, h.breakpoints, (T) => T !== !1), Q = l(j, h.breakpoints), X = p.columns ?? (V ? void 0 : S), ce = p.spacing ?? (V ? void 0 : L), J = p.rowSpacing ?? p.spacing ?? (V ? void 0 : B), ie = p.columnSpacing ?? p.spacing ?? (V ? void 0 : O), re = {
      ...v,
      level: V,
      columns: X,
      container: w,
      direction: b,
      wrap: k,
      spacing: ce,
      rowSpacing: J,
      columnSpacing: ie,
      size: q,
      offset: Q
    }, Y = s(re, h);
    return /* @__PURE__ */ A.jsx(u, {
      ref: g,
      as: _,
      ownerState: re,
      className: Ve(Y.root, C),
      ...W,
      children: z.Children.map($, (T) => {
        var H;
        return /* @__PURE__ */ z.isValidElement(T) && Vu(T, ["Grid"]) && w && T.props.container ? /* @__PURE__ */ z.cloneElement(T, {
          unstable_level: ((H = T.props) == null ? void 0 : H.unstable_level) ?? V + 1
        }) : T;
      })
    });
  });
  return c.muiName = "Grid", c;
}
const fc = {
  black: "#000",
  white: "#fff"
}, lG = {
  50: "#fafafa",
  100: "#f5f5f5",
  200: "#eeeeee",
  300: "#e0e0e0",
  400: "#bdbdbd",
  500: "#9e9e9e",
  600: "#757575",
  700: "#616161",
  800: "#424242",
  900: "#212121",
  A100: "#f5f5f5",
  A200: "#eeeeee",
  A400: "#bdbdbd",
  A700: "#616161"
}, js = {
  50: "#f3e5f5",
  100: "#e1bee7",
  200: "#ce93d8",
  300: "#ba68c8",
  400: "#ab47bc",
  500: "#9c27b0",
  600: "#8e24aa",
  700: "#7b1fa2",
  800: "#6a1b9a",
  900: "#4a148c",
  A100: "#ea80fc",
  A200: "#e040fb",
  A400: "#d500f9",
  A700: "#aa00ff"
}, Rs = {
  50: "#ffebee",
  100: "#ffcdd2",
  200: "#ef9a9a",
  300: "#e57373",
  400: "#ef5350",
  500: "#f44336",
  600: "#e53935",
  700: "#d32f2f",
  800: "#c62828",
  900: "#b71c1c",
  A100: "#ff8a80",
  A200: "#ff5252",
  A400: "#ff1744",
  A700: "#d50000"
}, Eu = {
  50: "#fff3e0",
  100: "#ffe0b2",
  200: "#ffcc80",
  300: "#ffb74d",
  400: "#ffa726",
  500: "#ff9800",
  600: "#fb8c00",
  700: "#f57c00",
  800: "#ef6c00",
  900: "#e65100",
  A100: "#ffd180",
  A200: "#ffab40",
  A400: "#ff9100",
  A700: "#ff6d00"
}, Ts = {
  50: "#e3f2fd",
  100: "#bbdefb",
  200: "#90caf9",
  300: "#64b5f6",
  400: "#42a5f5",
  500: "#2196f3",
  600: "#1e88e5",
  700: "#1976d2",
  800: "#1565c0",
  900: "#0d47a1",
  A100: "#82b1ff",
  A200: "#448aff",
  A400: "#2979ff",
  A700: "#2962ff"
}, Ms = {
  50: "#e1f5fe",
  100: "#b3e5fc",
  200: "#81d4fa",
  300: "#4fc3f7",
  400: "#29b6f6",
  500: "#03a9f4",
  600: "#039be5",
  700: "#0288d1",
  800: "#0277bd",
  900: "#01579b",
  A100: "#80d8ff",
  A200: "#40c4ff",
  A400: "#00b0ff",
  A700: "#0091ea"
}, Ns = {
  50: "#e8f5e9",
  100: "#c8e6c9",
  200: "#a5d6a7",
  300: "#81c784",
  400: "#66bb6a",
  500: "#4caf50",
  600: "#43a047",
  700: "#388e3c",
  800: "#2e7d32",
  900: "#1b5e20",
  A100: "#b9f6ca",
  A200: "#69f0ae",
  A400: "#00e676",
  A700: "#00c853"
};
function pE() {
  return {
    // The colors used to style the text.
    text: {
      // The most important text.
      primary: "rgba(0, 0, 0, 0.87)",
      // Secondary text.
      secondary: "rgba(0, 0, 0, 0.6)",
      // Disabled text have even lower visual prominence.
      disabled: "rgba(0, 0, 0, 0.38)"
    },
    // The color used to divide different elements.
    divider: "rgba(0, 0, 0, 0.12)",
    // The background colors used to style the surfaces.
    // Consistency between these values is important.
    background: {
      paper: fc.white,
      default: fc.white
    },
    // The colors used to style the action elements.
    action: {
      // The color of an active action like an icon button.
      active: "rgba(0, 0, 0, 0.54)",
      // The color of an hovered action.
      hover: "rgba(0, 0, 0, 0.04)",
      hoverOpacity: 0.04,
      // The color of a selected action.
      selected: "rgba(0, 0, 0, 0.08)",
      selectedOpacity: 0.08,
      // The color of a disabled action.
      disabled: "rgba(0, 0, 0, 0.26)",
      // The background color of a disabled action.
      disabledBackground: "rgba(0, 0, 0, 0.12)",
      disabledOpacity: 0.38,
      focus: "rgba(0, 0, 0, 0.12)",
      focusOpacity: 0.12,
      activatedOpacity: 0.12
    }
  };
}
const uG = pE();
function hE() {
  return {
    text: {
      primary: fc.white,
      secondary: "rgba(255, 255, 255, 0.7)",
      disabled: "rgba(255, 255, 255, 0.5)",
      icon: "rgba(255, 255, 255, 0.5)"
    },
    divider: "rgba(255, 255, 255, 0.12)",
    background: {
      paper: "#121212",
      default: "#121212"
    },
    action: {
      active: fc.white,
      hover: "rgba(255, 255, 255, 0.08)",
      hoverOpacity: 0.08,
      selected: "rgba(255, 255, 255, 0.16)",
      selectedOpacity: 0.16,
      disabled: "rgba(255, 255, 255, 0.3)",
      disabledBackground: "rgba(255, 255, 255, 0.12)",
      disabledOpacity: 0.38,
      focus: "rgba(255, 255, 255, 0.12)",
      focusOpacity: 0.12,
      activatedOpacity: 0.24
    }
  };
}
const H2 = hE();
function G2(e, t, n, o) {
  const i = o.light || o, s = o.dark || o * 1.5;
  e[t] || (e.hasOwnProperty(n) ? e[t] = e[n] : t === "light" ? e.light = cl(e.main, i) : t === "dark" && (e.dark = ul(e.main, s)));
}
function cG(e = "light") {
  return e === "dark" ? {
    main: Ts[200],
    light: Ts[50],
    dark: Ts[400]
  } : {
    main: Ts[700],
    light: Ts[400],
    dark: Ts[800]
  };
}
function dG(e = "light") {
  return e === "dark" ? {
    main: js[200],
    light: js[50],
    dark: js[400]
  } : {
    main: js[500],
    light: js[300],
    dark: js[700]
  };
}
function fG(e = "light") {
  return e === "dark" ? {
    main: Rs[500],
    light: Rs[300],
    dark: Rs[700]
  } : {
    main: Rs[700],
    light: Rs[400],
    dark: Rs[800]
  };
}
function pG(e = "light") {
  return e === "dark" ? {
    main: Ms[400],
    light: Ms[300],
    dark: Ms[700]
  } : {
    main: Ms[700],
    light: Ms[500],
    dark: Ms[900]
  };
}
function hG(e = "light") {
  return e === "dark" ? {
    main: Ns[400],
    light: Ns[300],
    dark: Ns[700]
  } : {
    main: Ns[800],
    light: Ns[500],
    dark: Ns[900]
  };
}
function mG(e = "light") {
  return e === "dark" ? {
    main: Eu[400],
    light: Eu[300],
    dark: Eu[700]
  } : {
    main: "#ed6c02",
    // closest to orange[800] that pass 3:1.
    light: Eu[500],
    dark: Eu[900]
  };
}
function y0(e) {
  const {
    mode: t = "light",
    contrastThreshold: n = 3,
    tonalOffset: o = 0.2,
    ...i
  } = e, s = e.primary || cG(t), l = e.secondary || dG(t), u = e.error || fG(t), c = e.info || pG(t), f = e.success || hG(t), p = e.warning || mG(t);
  function g(C) {
    return TH(C, H2.text.primary) >= n ? H2.text.primary : uG.text.primary;
  }
  const h = ({
    color: C,
    name: $,
    mainShade: S = 500,
    lightShade: w = 300,
    darkShade: _ = 700
  }) => {
    if (C = {
      ...C
    }, !C.main && C[S] && (C.main = C[S]), !C.hasOwnProperty("main"))
      throw new Error(Ci(11, $ ? ` (${$})` : "", S));
    if (typeof C.main != "string")
      throw new Error(Ci(12, $ ? ` (${$})` : "", JSON.stringify(C.main)));
    return G2(C, "light", w, o), G2(C, "dark", _, o), C.contrastText || (C.contrastText = g(C.main)), C;
  };
  let y;
  return t === "light" ? y = pE() : t === "dark" && (y = hE()), lr({
    // A collection of common colors.
    common: {
      ...fc
    },
    // prevent mutable object.
    // The palette mode, can be light or dark.
    mode: t,
    // The colors used to represent primary interface elements for a user.
    primary: h({
      color: s,
      name: "primary"
    }),
    // The colors used to represent secondary interface elements for a user.
    secondary: h({
      color: l,
      name: "secondary",
      mainShade: "A400",
      lightShade: "A200",
      darkShade: "A700"
    }),
    // The colors used to represent interface elements that the user should be made aware of.
    error: h({
      color: u,
      name: "error"
    }),
    // The colors used to represent potentially dangerous actions or important messages.
    warning: h({
      color: p,
      name: "warning"
    }),
    // The colors used to present information to the user that is neutral and not necessarily important.
    info: h({
      color: c,
      name: "info"
    }),
    // The colors used to indicate the successful completion of an action that user triggered.
    success: h({
      color: f,
      name: "success"
    }),
    // The grey colors.
    grey: lG,
    // Used by `getContrastText()` to maximize the contrast between
    // the background and the text.
    contrastThreshold: n,
    // Takes a background color and returns the text color that maximizes the contrast.
    getContrastText: g,
    // Generate a rich color object.
    augmentColor: h,
    // Used by the functions below to shift a color's luminance by approximately
    // two indexes within its tonal palette.
    // E.g., shift from Red 500 to Red 300 or Red 700.
    tonalOffset: o,
    // The light and dark mode object.
    ...y
  }, i);
}
function gG(e) {
  const t = {};
  return Object.entries(e).forEach((o) => {
    const [i, s] = o;
    typeof s == "object" && (t[i] = `${s.fontStyle ? `${s.fontStyle} ` : ""}${s.fontVariant ? `${s.fontVariant} ` : ""}${s.fontWeight ? `${s.fontWeight} ` : ""}${s.fontStretch ? `${s.fontStretch} ` : ""}${s.fontSize || ""}${s.lineHeight ? `/${s.lineHeight} ` : ""}${s.fontFamily || ""}`);
  }), t;
}
function yG(e, t) {
  return {
    toolbar: {
      minHeight: 56,
      [e.up("xs")]: {
        "@media (orientation: landscape)": {
          minHeight: 48
        }
      },
      [e.up("sm")]: {
        minHeight: 64
      }
    },
    ...t
  };
}
function vG(e) {
  return Math.round(e * 1e5) / 1e5;
}
const J2 = {
  textTransform: "uppercase"
}, Y2 = '"Roboto", "Helvetica", "Arial", sans-serif';
function bG(e, t) {
  const {
    fontFamily: n = Y2,
    // The default font size of the Material Specification.
    fontSize: o = 14,
    // px
    fontWeightLight: i = 300,
    fontWeightRegular: s = 400,
    fontWeightMedium: l = 500,
    fontWeightBold: u = 700,
    // Tell MUI what's the font-size on the html element.
    // 16px is the default font-size used by browsers.
    htmlFontSize: c = 16,
    // Apply the CSS properties to all the variants.
    allVariants: f,
    pxToRem: p,
    ...g
  } = typeof t == "function" ? t(e) : t, h = o / 14, y = p || (($) => `${$ / c * h}rem`), v = ($, S, w, _, b) => ({
    fontFamily: n,
    fontWeight: $,
    fontSize: y(S),
    // Unitless following https://meyerweb.com/eric/thoughts/2006/02/08/unitless-line-heights/
    lineHeight: w,
    // The letter spacing was designed for the Roboto font-family. Using the same letter-spacing
    // across font-families can cause issues with the kerning.
    ...n === Y2 ? {
      letterSpacing: `${vG(_ / S)}em`
    } : {},
    ...b,
    ...f
  }), C = {
    h1: v(i, 96, 1.167, -1.5),
    h2: v(i, 60, 1.2, -0.5),
    h3: v(s, 48, 1.167, 0),
    h4: v(s, 34, 1.235, 0.25),
    h5: v(s, 24, 1.334, 0),
    h6: v(l, 20, 1.6, 0.15),
    subtitle1: v(s, 16, 1.75, 0.15),
    subtitle2: v(l, 14, 1.57, 0.1),
    body1: v(s, 16, 1.5, 0.15),
    body2: v(s, 14, 1.43, 0.15),
    button: v(l, 14, 1.75, 0.4, J2),
    caption: v(s, 12, 1.66, 0.4),
    overline: v(s, 12, 2.66, 1, J2),
    // TODO v6: Remove handling of 'inherit' variant from the theme as it is already handled in Material UI's Typography component. Also, remember to remove the associated types.
    inherit: {
      fontFamily: "inherit",
      fontWeight: "inherit",
      fontSize: "inherit",
      lineHeight: "inherit",
      letterSpacing: "inherit"
    }
  };
  return lr({
    htmlFontSize: c,
    pxToRem: y,
    fontFamily: n,
    fontSize: o,
    fontWeightLight: i,
    fontWeightRegular: s,
    fontWeightMedium: l,
    fontWeightBold: u,
    ...C
  }, g, {
    clone: !1
    // No need to clone deep
  });
}
const SG = 0.2, wG = 0.14, $G = 0.12;
function hn(...e) {
  return [`${e[0]}px ${e[1]}px ${e[2]}px ${e[3]}px rgba(0,0,0,${SG})`, `${e[4]}px ${e[5]}px ${e[6]}px ${e[7]}px rgba(0,0,0,${wG})`, `${e[8]}px ${e[9]}px ${e[10]}px ${e[11]}px rgba(0,0,0,${$G})`].join(",");
}
const CG = ["none", hn(0, 2, 1, -1, 0, 1, 1, 0, 0, 1, 3, 0), hn(0, 3, 1, -2, 0, 2, 2, 0, 0, 1, 5, 0), hn(0, 3, 3, -2, 0, 3, 4, 0, 0, 1, 8, 0), hn(0, 2, 4, -1, 0, 4, 5, 0, 0, 1, 10, 0), hn(0, 3, 5, -1, 0, 5, 8, 0, 0, 1, 14, 0), hn(0, 3, 5, -1, 0, 6, 10, 0, 0, 1, 18, 0), hn(0, 4, 5, -2, 0, 7, 10, 1, 0, 2, 16, 1), hn(0, 5, 5, -3, 0, 8, 10, 1, 0, 3, 14, 2), hn(0, 5, 6, -3, 0, 9, 12, 1, 0, 3, 16, 2), hn(0, 6, 6, -3, 0, 10, 14, 1, 0, 4, 18, 3), hn(0, 6, 7, -4, 0, 11, 15, 1, 0, 4, 20, 3), hn(0, 7, 8, -4, 0, 12, 17, 2, 0, 5, 22, 4), hn(0, 7, 8, -4, 0, 13, 19, 2, 0, 5, 24, 4), hn(0, 7, 9, -4, 0, 14, 21, 2, 0, 5, 26, 4), hn(0, 8, 9, -5, 0, 15, 22, 2, 0, 6, 28, 5), hn(0, 8, 10, -5, 0, 16, 24, 2, 0, 6, 30, 5), hn(0, 8, 11, -5, 0, 17, 26, 2, 0, 6, 32, 5), hn(0, 9, 11, -5, 0, 18, 28, 2, 0, 7, 34, 6), hn(0, 9, 12, -6, 0, 19, 29, 2, 0, 7, 36, 6), hn(0, 10, 13, -6, 0, 20, 31, 3, 0, 8, 38, 7), hn(0, 10, 13, -6, 0, 21, 33, 3, 0, 8, 40, 7), hn(0, 10, 14, -6, 0, 22, 35, 3, 0, 8, 42, 7), hn(0, 11, 14, -7, 0, 23, 36, 3, 0, 9, 44, 8), hn(0, 11, 15, -7, 0, 24, 38, 3, 0, 9, 46, 8)], xG = {
  // This is the most common easing curve.
  easeInOut: "cubic-bezier(0.4, 0, 0.2, 1)",
  // Objects enter the screen at full velocity from off-screen and
  // slowly decelerate to a resting point.
  easeOut: "cubic-bezier(0.0, 0, 0.2, 1)",
  // Objects leave the screen at full velocity. They do not decelerate when off-screen.
  easeIn: "cubic-bezier(0.4, 0, 1, 1)",
  // The sharp curve is used by objects that may return to the screen at any time.
  sharp: "cubic-bezier(0.4, 0, 0.6, 1)"
}, mE = {
  shortest: 150,
  shorter: 200,
  short: 250,
  // most basic recommended timing
  standard: 300,
  // this is to be used in complex animations
  complex: 375,
  // recommended when something is entering screen
  enteringScreen: 225,
  // recommended when something is leaving screen
  leavingScreen: 195
};
function Q2(e) {
  return `${Math.round(e)}ms`;
}
function EG(e) {
  if (!e)
    return 0;
  const t = e / 36;
  return Math.min(Math.round((4 + 15 * t ** 0.25 + t / 5) * 10), 3e3);
}
function _G(e) {
  const t = {
    ...xG,
    ...e.easing
  }, n = {
    ...mE,
    ...e.duration
  };
  return {
    getAutoHeightDuration: EG,
    create: (i = ["all"], s = {}) => {
      const {
        duration: l = n.standard,
        easing: u = t.easeInOut,
        delay: c = 0,
        ...f
      } = s;
      return (Array.isArray(i) ? i : [i]).map((p) => `${p} ${typeof l == "string" ? l : Q2(l)} ${u} ${typeof c == "string" ? c : Q2(c)}`).join(",");
    },
    ...e,
    easing: t,
    duration: n
  };
}
const kG = {
  mobileStepper: 1e3,
  fab: 1050,
  speedDial: 1050,
  appBar: 1100,
  drawer: 1200,
  modal: 1300,
  snackbar: 1400,
  tooltip: 1500
};
function PG(e) {
  return Ko(e) || typeof e > "u" || typeof e == "string" || typeof e == "boolean" || typeof e == "number" || Array.isArray(e);
}
function gE(e = {}) {
  const t = {
    ...e
  };
  function n(o) {
    const i = Object.entries(o);
    for (let s = 0; s < i.length; s++) {
      const [l, u] = i[s];
      !PG(u) || l.startsWith("unstable_") ? delete o[l] : Ko(u) && (o[l] = {
        ...u
      }, n(o[l]));
    }
  }
  return n(t), `import { unstable_createBreakpoints as createBreakpoints, createTransitions } from '@mui/material/styles';

const theme = ${JSON.stringify(t, null, 2)};

theme.breakpoints = createBreakpoints(theme.breakpoints || {});
theme.transitions = createTransitions(theme.transitions || {});

export default theme;`;
}
function jg(e = {}, ...t) {
  const {
    breakpoints: n,
    mixins: o = {},
    spacing: i,
    palette: s = {},
    transitions: l = {},
    typography: u = {},
    shape: c,
    ...f
  } = e;
  if (e.vars)
    throw new Error(Ci(20));
  const p = y0(s), g = Mp(e);
  let h = lr(g, {
    mixins: yG(g.breakpoints, o),
    palette: p,
    // Don't use [...shadows] until you've verified its transpiled code is not invoking the iterator protocol.
    shadows: CG.slice(),
    typography: bG(p, u),
    transitions: _G(l),
    zIndex: {
      ...kG
    }
  });
  return h = lr(h, f), h = t.reduce((y, v) => lr(y, v), h), h.unstable_sxConfig = {
    ...Ac,
    ...f == null ? void 0 : f.unstable_sxConfig
  }, h.unstable_sx = function(v) {
    return Va({
      sx: v,
      theme: this
    });
  }, h.toRuntimeSource = gE, h;
}
function Rg(e) {
  let t;
  return e < 1 ? t = 5.11916 * e ** 2 : t = 4.5 * Math.log(e + 1) + 2, Math.round(t * 10) / 1e3;
}
const OG = [...Array(25)].map((e, t) => {
  if (t === 0)
    return "none";
  const n = Rg(t);
  return `linear-gradient(rgba(255 255 255 / ${n}), rgba(255 255 255 / ${n}))`;
});
function yE(e) {
  return {
    inputPlaceholder: e === "dark" ? 0.5 : 0.42,
    inputUnderline: e === "dark" ? 0.7 : 0.42,
    switchTrackDisabled: e === "dark" ? 0.2 : 0.12,
    switchTrack: e === "dark" ? 0.3 : 0.38
  };
}
function vE(e) {
  return e === "dark" ? OG : [];
}
function AG(e) {
  const {
    palette: t = {
      mode: "light"
    },
    // need to cast to avoid module augmentation test
    opacity: n,
    overlays: o,
    ...i
  } = e, s = y0(t);
  return {
    palette: s,
    opacity: {
      ...yE(s.mode),
      ...n
    },
    overlays: o || vE(s.mode),
    ...i
  };
}
function IG(e) {
  var t;
  return !!e[0].match(/(cssVarPrefix|colorSchemeSelector|rootSelector|typography|mixins|breakpoints|direction|transitions)/) || !!e[0].match(/sxConfig$/) || // ends with sxConfig
  e[0] === "palette" && !!((t = e[1]) != null && t.match(/(mode|contrastThreshold|tonalOffset)/));
}
const jG = (e) => [...[...Array(25)].map((t, n) => `--${e ? `${e}-` : ""}overlays-${n}`), `--${e ? `${e}-` : ""}palette-AppBar-darkBg`, `--${e ? `${e}-` : ""}palette-AppBar-darkColor`], RG = (e) => (t, n) => {
  const o = e.rootSelector || ":root", i = e.colorSchemeSelector;
  let s = i;
  if (i === "class" && (s = ".%s"), i === "data" && (s = "[data-%s]"), i != null && i.startsWith("data-") && !i.includes("%s") && (s = `[${i}="%s"]`), e.defaultColorScheme === t) {
    if (t === "dark") {
      const l = {};
      return jG(e.cssVarPrefix).forEach((u) => {
        l[u] = n[u], delete n[u];
      }), s === "media" ? {
        [o]: n,
        "@media (prefers-color-scheme: dark)": {
          [o]: l
        }
      } : s ? {
        [s.replace("%s", t)]: l,
        [`${o}, ${s.replace("%s", t)}`]: n
      } : {
        [o]: {
          ...n,
          ...l
        }
      };
    }
    if (s && s !== "media")
      return `${o}, ${s.replace("%s", String(t))}`;
  } else if (t) {
    if (s === "media")
      return {
        [`@media (prefers-color-scheme: ${String(t)})`]: {
          [o]: n
        }
      };
    if (s)
      return s.replace("%s", String(t));
  }
  return o;
};
function TG(e, t) {
  t.forEach((n) => {
    e[n] || (e[n] = {});
  });
}
function ge(e, t, n) {
  !e[t] && n && (e[t] = n);
}
function Nu(e) {
  return typeof e != "string" || !e.startsWith("hsl") ? e : fE(e);
}
function hi(e, t) {
  `${t}Channel` in e || (e[`${t}Channel`] = Mu(Nu(e[t])));
}
function MG(e) {
  return typeof e == "number" ? `${e}px` : typeof e == "string" || typeof e == "function" || Array.isArray(e) ? e : "8px";
}
const zo = (e) => {
  try {
    return e();
  } catch {
  }
}, NG = (e = "mui") => BH(e);
function Xm(e, t, n, o) {
  if (!t)
    return;
  t = t === !0 ? {} : t;
  const i = o === "dark" ? "dark" : "light";
  if (!n) {
    e[o] = AG({
      ...t,
      palette: {
        mode: i,
        ...t == null ? void 0 : t.palette
      }
    });
    return;
  }
  const {
    palette: s,
    ...l
  } = jg({
    ...n,
    palette: {
      mode: i,
      ...t == null ? void 0 : t.palette
    }
  });
  return e[o] = {
    ...t,
    palette: s,
    opacity: {
      ...yE(i),
      ...t == null ? void 0 : t.opacity
    },
    overlays: (t == null ? void 0 : t.overlays) || vE(i)
  }, l;
}
function FG(e = {}, ...t) {
  const {
    colorSchemes: n = {
      light: !0
    },
    defaultColorScheme: o,
    disableCssColorScheme: i = !1,
    cssVarPrefix: s = "mui",
    shouldSkipGeneratingVar: l = IG,
    colorSchemeSelector: u = n.light && n.dark ? "media" : void 0,
    rootSelector: c = ":root",
    ...f
  } = e, p = Object.keys(n)[0], g = o || (n.light && p !== "light" ? "light" : p), h = NG(s), {
    [g]: y,
    light: v,
    dark: C,
    ...$
  } = n, S = {
    ...$
  };
  let w = y;
  if ((g === "dark" && !("dark" in n) || g === "light" && !("light" in n)) && (w = !0), !w)
    throw new Error(Ci(21, g));
  const _ = Xm(S, w, f, g);
  v && !S.light && Xm(S, v, void 0, "light"), C && !S.dark && Xm(S, C, void 0, "dark");
  let b = {
    defaultColorScheme: g,
    ..._,
    cssVarPrefix: s,
    colorSchemeSelector: u,
    rootSelector: c,
    getCssVar: h,
    colorSchemes: S,
    font: {
      ...gG(_.typography),
      ..._.font
    },
    spacing: MG(f.spacing)
  };
  Object.keys(b.colorSchemes).forEach((B) => {
    const O = b.colorSchemes[B].palette, V = (W) => {
      const q = W.split("-"), Q = q[1], X = q[2];
      return h(W, O[Q][X]);
    };
    if (O.mode === "light" && (ge(O.common, "background", "#fff"), ge(O.common, "onBackground", "#000")), O.mode === "dark" && (ge(O.common, "background", "#000"), ge(O.common, "onBackground", "#fff")), TG(O, ["Alert", "AppBar", "Avatar", "Button", "Chip", "FilledInput", "LinearProgress", "Skeleton", "Slider", "SnackbarContent", "SpeedDialAction", "StepConnector", "StepContent", "Switch", "TableCell", "Tooltip"]), O.mode === "light") {
      ge(O.Alert, "errorColor", Xt(O.error.light, 0.6)), ge(O.Alert, "infoColor", Xt(O.info.light, 0.6)), ge(O.Alert, "successColor", Xt(O.success.light, 0.6)), ge(O.Alert, "warningColor", Xt(O.warning.light, 0.6)), ge(O.Alert, "errorFilledBg", V("palette-error-main")), ge(O.Alert, "infoFilledBg", V("palette-info-main")), ge(O.Alert, "successFilledBg", V("palette-success-main")), ge(O.Alert, "warningFilledBg", V("palette-warning-main")), ge(O.Alert, "errorFilledColor", zo(() => O.getContrastText(O.error.main))), ge(O.Alert, "infoFilledColor", zo(() => O.getContrastText(O.info.main))), ge(O.Alert, "successFilledColor", zo(() => O.getContrastText(O.success.main))), ge(O.Alert, "warningFilledColor", zo(() => O.getContrastText(O.warning.main))), ge(O.Alert, "errorStandardBg", Zt(O.error.light, 0.9)), ge(O.Alert, "infoStandardBg", Zt(O.info.light, 0.9)), ge(O.Alert, "successStandardBg", Zt(O.success.light, 0.9)), ge(O.Alert, "warningStandardBg", Zt(O.warning.light, 0.9)), ge(O.Alert, "errorIconColor", V("palette-error-main")), ge(O.Alert, "infoIconColor", V("palette-info-main")), ge(O.Alert, "successIconColor", V("palette-success-main")), ge(O.Alert, "warningIconColor", V("palette-warning-main")), ge(O.AppBar, "defaultBg", V("palette-grey-100")), ge(O.Avatar, "defaultBg", V("palette-grey-400")), ge(O.Button, "inheritContainedBg", V("palette-grey-300")), ge(O.Button, "inheritContainedHoverBg", V("palette-grey-A100")), ge(O.Chip, "defaultBorder", V("palette-grey-400")), ge(O.Chip, "defaultAvatarColor", V("palette-grey-700")), ge(O.Chip, "defaultIconColor", V("palette-grey-700")), ge(O.FilledInput, "bg", "rgba(0, 0, 0, 0.06)"), ge(O.FilledInput, "hoverBg", "rgba(0, 0, 0, 0.09)"), ge(O.FilledInput, "disabledBg", "rgba(0, 0, 0, 0.12)"), ge(O.LinearProgress, "primaryBg", Zt(O.primary.main, 0.62)), ge(O.LinearProgress, "secondaryBg", Zt(O.secondary.main, 0.62)), ge(O.LinearProgress, "errorBg", Zt(O.error.main, 0.62)), ge(O.LinearProgress, "infoBg", Zt(O.info.main, 0.62)), ge(O.LinearProgress, "successBg", Zt(O.success.main, 0.62)), ge(O.LinearProgress, "warningBg", Zt(O.warning.main, 0.62)), ge(O.Skeleton, "bg", `rgba(${V("palette-text-primaryChannel")} / 0.11)`), ge(O.Slider, "primaryTrack", Zt(O.primary.main, 0.62)), ge(O.Slider, "secondaryTrack", Zt(O.secondary.main, 0.62)), ge(O.Slider, "errorTrack", Zt(O.error.main, 0.62)), ge(O.Slider, "infoTrack", Zt(O.info.main, 0.62)), ge(O.Slider, "successTrack", Zt(O.success.main, 0.62)), ge(O.Slider, "warningTrack", Zt(O.warning.main, 0.62));
      const W = Qd(O.background.default, 0.8);
      ge(O.SnackbarContent, "bg", W), ge(O.SnackbarContent, "color", zo(() => O.getContrastText(W))), ge(O.SpeedDialAction, "fabHoverBg", Qd(O.background.paper, 0.15)), ge(O.StepConnector, "border", V("palette-grey-400")), ge(O.StepContent, "border", V("palette-grey-400")), ge(O.Switch, "defaultColor", V("palette-common-white")), ge(O.Switch, "defaultDisabledColor", V("palette-grey-100")), ge(O.Switch, "primaryDisabledColor", Zt(O.primary.main, 0.62)), ge(O.Switch, "secondaryDisabledColor", Zt(O.secondary.main, 0.62)), ge(O.Switch, "errorDisabledColor", Zt(O.error.main, 0.62)), ge(O.Switch, "infoDisabledColor", Zt(O.info.main, 0.62)), ge(O.Switch, "successDisabledColor", Zt(O.success.main, 0.62)), ge(O.Switch, "warningDisabledColor", Zt(O.warning.main, 0.62)), ge(O.TableCell, "border", Zt(Yd(O.divider, 1), 0.88)), ge(O.Tooltip, "bg", Yd(O.grey[700], 0.92));
    }
    if (O.mode === "dark") {
      ge(O.Alert, "errorColor", Zt(O.error.light, 0.6)), ge(O.Alert, "infoColor", Zt(O.info.light, 0.6)), ge(O.Alert, "successColor", Zt(O.success.light, 0.6)), ge(O.Alert, "warningColor", Zt(O.warning.light, 0.6)), ge(O.Alert, "errorFilledBg", V("palette-error-dark")), ge(O.Alert, "infoFilledBg", V("palette-info-dark")), ge(O.Alert, "successFilledBg", V("palette-success-dark")), ge(O.Alert, "warningFilledBg", V("palette-warning-dark")), ge(O.Alert, "errorFilledColor", zo(() => O.getContrastText(O.error.dark))), ge(O.Alert, "infoFilledColor", zo(() => O.getContrastText(O.info.dark))), ge(O.Alert, "successFilledColor", zo(() => O.getContrastText(O.success.dark))), ge(O.Alert, "warningFilledColor", zo(() => O.getContrastText(O.warning.dark))), ge(O.Alert, "errorStandardBg", Xt(O.error.light, 0.9)), ge(O.Alert, "infoStandardBg", Xt(O.info.light, 0.9)), ge(O.Alert, "successStandardBg", Xt(O.success.light, 0.9)), ge(O.Alert, "warningStandardBg", Xt(O.warning.light, 0.9)), ge(O.Alert, "errorIconColor", V("palette-error-main")), ge(O.Alert, "infoIconColor", V("palette-info-main")), ge(O.Alert, "successIconColor", V("palette-success-main")), ge(O.Alert, "warningIconColor", V("palette-warning-main")), ge(O.AppBar, "defaultBg", V("palette-grey-900")), ge(O.AppBar, "darkBg", V("palette-background-paper")), ge(O.AppBar, "darkColor", V("palette-text-primary")), ge(O.Avatar, "defaultBg", V("palette-grey-600")), ge(O.Button, "inheritContainedBg", V("palette-grey-800")), ge(O.Button, "inheritContainedHoverBg", V("palette-grey-700")), ge(O.Chip, "defaultBorder", V("palette-grey-700")), ge(O.Chip, "defaultAvatarColor", V("palette-grey-300")), ge(O.Chip, "defaultIconColor", V("palette-grey-300")), ge(O.FilledInput, "bg", "rgba(255, 255, 255, 0.09)"), ge(O.FilledInput, "hoverBg", "rgba(255, 255, 255, 0.13)"), ge(O.FilledInput, "disabledBg", "rgba(255, 255, 255, 0.12)"), ge(O.LinearProgress, "primaryBg", Xt(O.primary.main, 0.5)), ge(O.LinearProgress, "secondaryBg", Xt(O.secondary.main, 0.5)), ge(O.LinearProgress, "errorBg", Xt(O.error.main, 0.5)), ge(O.LinearProgress, "infoBg", Xt(O.info.main, 0.5)), ge(O.LinearProgress, "successBg", Xt(O.success.main, 0.5)), ge(O.LinearProgress, "warningBg", Xt(O.warning.main, 0.5)), ge(O.Skeleton, "bg", `rgba(${V("palette-text-primaryChannel")} / 0.13)`), ge(O.Slider, "primaryTrack", Xt(O.primary.main, 0.5)), ge(O.Slider, "secondaryTrack", Xt(O.secondary.main, 0.5)), ge(O.Slider, "errorTrack", Xt(O.error.main, 0.5)), ge(O.Slider, "infoTrack", Xt(O.info.main, 0.5)), ge(O.Slider, "successTrack", Xt(O.success.main, 0.5)), ge(O.Slider, "warningTrack", Xt(O.warning.main, 0.5));
      const W = Qd(O.background.default, 0.98);
      ge(O.SnackbarContent, "bg", W), ge(O.SnackbarContent, "color", zo(() => O.getContrastText(W))), ge(O.SpeedDialAction, "fabHoverBg", Qd(O.background.paper, 0.15)), ge(O.StepConnector, "border", V("palette-grey-600")), ge(O.StepContent, "border", V("palette-grey-600")), ge(O.Switch, "defaultColor", V("palette-grey-300")), ge(O.Switch, "defaultDisabledColor", V("palette-grey-600")), ge(O.Switch, "primaryDisabledColor", Xt(O.primary.main, 0.55)), ge(O.Switch, "secondaryDisabledColor", Xt(O.secondary.main, 0.55)), ge(O.Switch, "errorDisabledColor", Xt(O.error.main, 0.55)), ge(O.Switch, "infoDisabledColor", Xt(O.info.main, 0.55)), ge(O.Switch, "successDisabledColor", Xt(O.success.main, 0.55)), ge(O.Switch, "warningDisabledColor", Xt(O.warning.main, 0.55)), ge(O.TableCell, "border", Xt(Yd(O.divider, 1), 0.68)), ge(O.Tooltip, "bg", Yd(O.grey[700], 0.92));
    }
    hi(O.background, "default"), hi(O.background, "paper"), hi(O.common, "background"), hi(O.common, "onBackground"), hi(O, "divider"), Object.keys(O).forEach((W) => {
      const q = O[W];
      W !== "tonalOffset" && q && typeof q == "object" && (q.main && ge(O[W], "mainChannel", Mu(Nu(q.main))), q.light && ge(O[W], "lightChannel", Mu(Nu(q.light))), q.dark && ge(O[W], "darkChannel", Mu(Nu(q.dark))), q.contrastText && ge(O[W], "contrastTextChannel", Mu(Nu(q.contrastText))), W === "text" && (hi(O[W], "primary"), hi(O[W], "secondary")), W === "action" && (q.active && hi(O[W], "active"), q.selected && hi(O[W], "selected")));
    });
  }), b = t.reduce((B, O) => lr(B, O), b);
  const k = {
    prefix: s,
    disableCssColorScheme: i,
    shouldSkipGeneratingVar: l,
    getSelector: RG(b)
  }, {
    vars: E,
    generateThemeVars: j,
    generateStyleSheets: L
  } = WH(b, k);
  return b.vars = E, Object.entries(b.colorSchemes[b.defaultColorScheme]).forEach(([B, O]) => {
    b[B] = O;
  }), b.generateThemeVars = j, b.generateStyleSheets = L, b.generateSpacing = function() {
    return lE(f.spacing, a0(this));
  }, b.getColorSchemeSelector = qH(u), b.spacing = b.generateSpacing(), b.shouldSkipGeneratingVar = l, b.unstable_sxConfig = {
    ...Ac,
    ...f == null ? void 0 : f.unstable_sxConfig
  }, b.unstable_sx = function(O) {
    return Va({
      sx: O,
      theme: this
    });
  }, b.toRuntimeSource = gE, b;
}
function X2(e, t, n) {
  e.colorSchemes && n && (e.colorSchemes[t] = {
    ...n !== !0 && n,
    palette: y0({
      ...n === !0 ? {} : n.palette,
      mode: t
    })
    // cast type to skip module augmentation test
  });
}
function bE(e = {}, ...t) {
  const {
    palette: n,
    cssVariables: o = !1,
    colorSchemes: i = n ? void 0 : {
      light: !0
    },
    defaultColorScheme: s = n == null ? void 0 : n.mode,
    ...l
  } = e, u = s || "light", c = i == null ? void 0 : i[u], f = {
    ...i,
    ...n ? {
      [u]: {
        ...typeof c != "boolean" && c,
        palette: n
      }
    } : void 0
  };
  if (o === !1) {
    if (!("colorSchemes" in e))
      return jg(e, ...t);
    let p = n;
    "palette" in e || f[u] && (f[u] !== !0 ? p = f[u].palette : u === "dark" && (p = {
      mode: "dark"
    }));
    const g = jg({
      ...e,
      palette: p
    }, ...t);
    return g.defaultColorScheme = u, g.colorSchemes = f, g.palette.mode === "light" && (g.colorSchemes.light = {
      ...f.light !== !0 && f.light,
      palette: g.palette
    }, X2(g, "dark", f.dark)), g.palette.mode === "dark" && (g.colorSchemes.dark = {
      ...f.dark !== !0 && f.dark,
      palette: g.palette
    }, X2(g, "light", f.light)), g;
  }
  return !n && !("light" in f) && u === "light" && (f.light = !0), FG({
    ...l,
    colorSchemes: f,
    defaultColorScheme: u,
    ...typeof o != "boolean" && o
  }, ...t);
}
const v0 = bE(), Lp = "$$material";
function Tl() {
  const e = Rc(v0);
  return e[Lp] || e;
}
function LG(e) {
  return /* @__PURE__ */ A.jsx(SH, {
    ...e,
    defaultTheme: v0,
    themeId: Lp
  });
}
function Dp(e) {
  return e !== "ownerState" && e !== "theme" && e !== "sx" && e !== "as";
}
const ur = (e) => Dp(e) && e !== "classes", Oe = dE({
  themeId: Lp,
  defaultTheme: v0,
  rootShouldForwardProp: ur
});
function DG(e) {
  return function(n) {
    return (
      // Pigment CSS `globalCss` support callback with theme inside an object but `GlobalStyles` support theme as a callback value.
      /* @__PURE__ */ A.jsx(LG, {
        styles: typeof e == "function" ? (o) => e({
          theme: o,
          ...n
        }) : e
      })
    );
  };
}
function zG() {
  return Ap;
}
const ct = zH;
function bt(e) {
  return DH(e);
}
function BG(e) {
  return ht("MuiSvgIcon", e);
}
dt("MuiSvgIcon", ["root", "colorPrimary", "colorSecondary", "colorAction", "colorError", "colorDisabled", "fontSizeInherit", "fontSizeSmall", "fontSizeMedium", "fontSizeLarge"]);
const UG = (e) => {
  const {
    color: t,
    fontSize: n,
    classes: o
  } = e, i = {
    root: ["root", t !== "inherit" && `color${Ne(t)}`, `fontSize${Ne(n)}`]
  };
  return mt(i, BG, o);
}, VG = Oe("svg", {
  name: "MuiSvgIcon",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, n.color !== "inherit" && t[`color${Ne(n.color)}`], t[`fontSize${Ne(n.fontSize)}`]];
  }
})(ct(({
  theme: e
}) => {
  var t, n, o, i, s, l, u, c, f, p, g, h, y, v;
  return {
    userSelect: "none",
    width: "1em",
    height: "1em",
    display: "inline-block",
    flexShrink: 0,
    transition: (i = (t = e.transitions) == null ? void 0 : t.create) == null ? void 0 : i.call(t, "fill", {
      duration: (o = (n = (e.vars ?? e).transitions) == null ? void 0 : n.duration) == null ? void 0 : o.shorter
    }),
    variants: [
      {
        props: (C) => !C.hasSvgAsChild,
        style: {
          // the <svg> will define the property that has `currentColor`
          // for example heroicons uses fill="none" and stroke="currentColor"
          fill: "currentColor"
        }
      },
      {
        props: {
          fontSize: "inherit"
        },
        style: {
          fontSize: "inherit"
        }
      },
      {
        props: {
          fontSize: "small"
        },
        style: {
          fontSize: ((l = (s = e.typography) == null ? void 0 : s.pxToRem) == null ? void 0 : l.call(s, 20)) || "1.25rem"
        }
      },
      {
        props: {
          fontSize: "medium"
        },
        style: {
          fontSize: ((c = (u = e.typography) == null ? void 0 : u.pxToRem) == null ? void 0 : c.call(u, 24)) || "1.5rem"
        }
      },
      {
        props: {
          fontSize: "large"
        },
        style: {
          fontSize: ((p = (f = e.typography) == null ? void 0 : f.pxToRem) == null ? void 0 : p.call(f, 35)) || "2.1875rem"
        }
      },
      // TODO v5 deprecate color prop, v6 remove for sx
      ...Object.entries((e.vars ?? e).palette).filter(([, C]) => C && C.main).map(([C]) => {
        var $, S;
        return {
          props: {
            color: C
          },
          style: {
            color: (S = ($ = (e.vars ?? e).palette) == null ? void 0 : $[C]) == null ? void 0 : S.main
          }
        };
      }),
      {
        props: {
          color: "action"
        },
        style: {
          color: (h = (g = (e.vars ?? e).palette) == null ? void 0 : g.action) == null ? void 0 : h.active
        }
      },
      {
        props: {
          color: "disabled"
        },
        style: {
          color: (v = (y = (e.vars ?? e).palette) == null ? void 0 : y.action) == null ? void 0 : v.disabled
        }
      },
      {
        props: {
          color: "inherit"
        },
        style: {
          color: void 0
        }
      }
    ]
  };
})), Tg = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiSvgIcon"
  }), {
    children: i,
    className: s,
    color: l = "inherit",
    component: u = "svg",
    fontSize: c = "medium",
    htmlColor: f,
    inheritViewBox: p = !1,
    titleAccess: g,
    viewBox: h = "0 0 24 24",
    ...y
  } = o, v = /* @__PURE__ */ z.isValidElement(i) && i.type === "svg", C = {
    ...o,
    color: l,
    component: u,
    fontSize: c,
    instanceFontSize: t.fontSize,
    inheritViewBox: p,
    viewBox: h,
    hasSvgAsChild: v
  }, $ = {};
  p || ($.viewBox = h);
  const S = UG(C);
  return /* @__PURE__ */ A.jsxs(VG, {
    as: u,
    className: Ve(S.root, s),
    focusable: "false",
    color: f,
    "aria-hidden": g ? void 0 : !0,
    role: g ? "img" : void 0,
    ref: n,
    ...$,
    ...y,
    ...v && i.props,
    ownerState: C,
    children: [v ? i.props.children : i, g ? /* @__PURE__ */ A.jsx("title", {
      children: g
    }) : null]
  });
});
Tg.muiName = "SvgIcon";
function Cr(e, t) {
  function n(o, i) {
    return /* @__PURE__ */ A.jsx(Tg, {
      "data-testid": `${t}Icon`,
      ref: i,
      ...o,
      children: e
    });
  }
  return n.muiName = Tg.muiName, /* @__PURE__ */ z.memo(/* @__PURE__ */ z.forwardRef(n));
}
const WG = Cr(/* @__PURE__ */ A.jsx("path", {
  d: "M19 13h-6v6h-2v-6H5v-2h6V5h2v6h6z"
}), "Add");
function qG(e) {
  return typeof e.main == "string";
}
function KG(e, t = []) {
  if (!qG(e))
    return !1;
  for (const n of t)
    if (!e.hasOwnProperty(n) || typeof e[n] != "string")
      return !1;
  return !0;
}
function kn(e = []) {
  return ([, t]) => t && KG(t, e);
}
class Vf {
  constructor() {
    yu(this, "mountEffect", () => {
      this.shouldMount && !this.didMount && this.ref.current !== null && (this.didMount = !0, this.mounted.resolve());
    });
    this.ref = {
      current: null
    }, this.mounted = null, this.didMount = !1, this.shouldMount = !1, this.setShouldMount = null;
  }
  /** React ref to the ripple instance */
  /** If the ripple component should be mounted */
  /** Promise that resolves when the ripple component is mounted */
  /** If the ripple component has been mounted */
  /** React state hook setter */
  static create() {
    return new Vf();
  }
  static use() {
    const t = Lx(Vf.create).current, [n, o] = z.useState(!1);
    return t.shouldMount = n, t.setShouldMount = o, z.useEffect(t.mountEffect, [n]), t;
  }
  mount() {
    return this.mounted || (this.mounted = GG(), this.shouldMount = !0, this.setShouldMount(this.shouldMount)), this.mounted;
  }
  /* Ripple API */
  start(...t) {
    this.mount().then(() => {
      var n;
      return (n = this.ref.current) == null ? void 0 : n.start(...t);
    });
  }
  stop(...t) {
    this.mount().then(() => {
      var n;
      return (n = this.ref.current) == null ? void 0 : n.stop(...t);
    });
  }
  pulsate(...t) {
    this.mount().then(() => {
      var n;
      return (n = this.ref.current) == null ? void 0 : n.pulsate(...t);
    });
  }
}
function HG() {
  return Vf.use();
}
function GG() {
  let e, t;
  const n = new Promise((o, i) => {
    e = o, t = i;
  });
  return n.resolve = e, n.reject = t, n;
}
function SE(e, t) {
  if (e == null) return {};
  var n = {};
  for (var o in e) if ({}.hasOwnProperty.call(e, o)) {
    if (t.includes(o)) continue;
    n[o] = e[o];
  }
  return n;
}
function Mg(e, t) {
  return Mg = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function(n, o) {
    return n.__proto__ = o, n;
  }, Mg(e, t);
}
function wE(e, t) {
  e.prototype = Object.create(t.prototype), e.prototype.constructor = e, Mg(e, t);
}
const Z2 = {
  disabled: !1
}, Wf = lo.createContext(null);
var JG = function(t) {
  return t.scrollTop;
}, Fu = "unmounted", Ta = "exited", Ma = "entering", Ws = "entered", Ng = "exiting", jo = /* @__PURE__ */ function(e) {
  wE(t, e);
  function t(o, i) {
    var s;
    s = e.call(this, o, i) || this;
    var l = i, u = l && !l.isMounting ? o.enter : o.appear, c;
    return s.appearStatus = null, o.in ? u ? (c = Ta, s.appearStatus = Ma) : c = Ws : o.unmountOnExit || o.mountOnEnter ? c = Fu : c = Ta, s.state = {
      status: c
    }, s.nextCallback = null, s;
  }
  t.getDerivedStateFromProps = function(i, s) {
    var l = i.in;
    return l && s.status === Fu ? {
      status: Ta
    } : null;
  };
  var n = t.prototype;
  return n.componentDidMount = function() {
    this.updateStatus(!0, this.appearStatus);
  }, n.componentDidUpdate = function(i) {
    var s = null;
    if (i !== this.props) {
      var l = this.state.status;
      this.props.in ? l !== Ma && l !== Ws && (s = Ma) : (l === Ma || l === Ws) && (s = Ng);
    }
    this.updateStatus(!1, s);
  }, n.componentWillUnmount = function() {
    this.cancelNextCallback();
  }, n.getTimeouts = function() {
    var i = this.props.timeout, s, l, u;
    return s = l = u = i, i != null && typeof i != "number" && (s = i.exit, l = i.enter, u = i.appear !== void 0 ? i.appear : l), {
      exit: s,
      enter: l,
      appear: u
    };
  }, n.updateStatus = function(i, s) {
    if (i === void 0 && (i = !1), s !== null)
      if (this.cancelNextCallback(), s === Ma) {
        if (this.props.unmountOnExit || this.props.mountOnEnter) {
          var l = this.props.nodeRef ? this.props.nodeRef.current : zd.findDOMNode(this);
          l && JG(l);
        }
        this.performEnter(i);
      } else
        this.performExit();
    else this.props.unmountOnExit && this.state.status === Ta && this.setState({
      status: Fu
    });
  }, n.performEnter = function(i) {
    var s = this, l = this.props.enter, u = this.context ? this.context.isMounting : i, c = this.props.nodeRef ? [u] : [zd.findDOMNode(this), u], f = c[0], p = c[1], g = this.getTimeouts(), h = u ? g.appear : g.enter;
    if (!i && !l || Z2.disabled) {
      this.safeSetState({
        status: Ws
      }, function() {
        s.props.onEntered(f);
      });
      return;
    }
    this.props.onEnter(f, p), this.safeSetState({
      status: Ma
    }, function() {
      s.props.onEntering(f, p), s.onTransitionEnd(h, function() {
        s.safeSetState({
          status: Ws
        }, function() {
          s.props.onEntered(f, p);
        });
      });
    });
  }, n.performExit = function() {
    var i = this, s = this.props.exit, l = this.getTimeouts(), u = this.props.nodeRef ? void 0 : zd.findDOMNode(this);
    if (!s || Z2.disabled) {
      this.safeSetState({
        status: Ta
      }, function() {
        i.props.onExited(u);
      });
      return;
    }
    this.props.onExit(u), this.safeSetState({
      status: Ng
    }, function() {
      i.props.onExiting(u), i.onTransitionEnd(l.exit, function() {
        i.safeSetState({
          status: Ta
        }, function() {
          i.props.onExited(u);
        });
      });
    });
  }, n.cancelNextCallback = function() {
    this.nextCallback !== null && (this.nextCallback.cancel(), this.nextCallback = null);
  }, n.safeSetState = function(i, s) {
    s = this.setNextCallback(s), this.setState(i, s);
  }, n.setNextCallback = function(i) {
    var s = this, l = !0;
    return this.nextCallback = function(u) {
      l && (l = !1, s.nextCallback = null, i(u));
    }, this.nextCallback.cancel = function() {
      l = !1;
    }, this.nextCallback;
  }, n.onTransitionEnd = function(i, s) {
    this.setNextCallback(s);
    var l = this.props.nodeRef ? this.props.nodeRef.current : zd.findDOMNode(this), u = i == null && !this.props.addEndListener;
    if (!l || u) {
      setTimeout(this.nextCallback, 0);
      return;
    }
    if (this.props.addEndListener) {
      var c = this.props.nodeRef ? [this.nextCallback] : [l, this.nextCallback], f = c[0], p = c[1];
      this.props.addEndListener(f, p);
    }
    i != null && setTimeout(this.nextCallback, i);
  }, n.render = function() {
    var i = this.state.status;
    if (i === Fu)
      return null;
    var s = this.props, l = s.children;
    s.in, s.mountOnEnter, s.unmountOnExit, s.appear, s.enter, s.exit, s.timeout, s.addEndListener, s.onEnter, s.onEntering, s.onEntered, s.onExit, s.onExiting, s.onExited, s.nodeRef;
    var u = SE(s, ["children", "in", "mountOnEnter", "unmountOnExit", "appear", "enter", "exit", "timeout", "addEndListener", "onEnter", "onEntering", "onEntered", "onExit", "onExiting", "onExited", "nodeRef"]);
    return (
      // allows for nested Transitions
      /* @__PURE__ */ lo.createElement(Wf.Provider, {
        value: null
      }, typeof l == "function" ? l(i, u) : lo.cloneElement(lo.Children.only(l), u))
    );
  }, t;
}(lo.Component);
jo.contextType = Wf;
jo.propTypes = {};
function Fs() {
}
jo.defaultProps = {
  in: !1,
  mountOnEnter: !1,
  unmountOnExit: !1,
  appear: !1,
  enter: !0,
  exit: !0,
  onEnter: Fs,
  onEntering: Fs,
  onEntered: Fs,
  onExit: Fs,
  onExiting: Fs,
  onExited: Fs
};
jo.UNMOUNTED = Fu;
jo.EXITED = Ta;
jo.ENTERING = Ma;
jo.ENTERED = Ws;
jo.EXITING = Ng;
function YG(e) {
  if (e === void 0) throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  return e;
}
function b0(e, t) {
  var n = function(s) {
    return t && z.isValidElement(s) ? t(s) : s;
  }, o = /* @__PURE__ */ Object.create(null);
  return e && z.Children.map(e, function(i) {
    return i;
  }).forEach(function(i) {
    o[i.key] = n(i);
  }), o;
}
function QG(e, t) {
  e = e || {}, t = t || {};
  function n(p) {
    return p in t ? t[p] : e[p];
  }
  var o = /* @__PURE__ */ Object.create(null), i = [];
  for (var s in e)
    s in t ? i.length && (o[s] = i, i = []) : i.push(s);
  var l, u = {};
  for (var c in t) {
    if (o[c])
      for (l = 0; l < o[c].length; l++) {
        var f = o[c][l];
        u[o[c][l]] = n(f);
      }
    u[c] = n(c);
  }
  for (l = 0; l < i.length; l++)
    u[i[l]] = n(i[l]);
  return u;
}
function za(e, t, n) {
  return n[t] != null ? n[t] : e.props[t];
}
function XG(e, t) {
  return b0(e.children, function(n) {
    return z.cloneElement(n, {
      onExited: t.bind(null, n),
      in: !0,
      appear: za(n, "appear", e),
      enter: za(n, "enter", e),
      exit: za(n, "exit", e)
    });
  });
}
function ZG(e, t, n) {
  var o = b0(e.children), i = QG(t, o);
  return Object.keys(i).forEach(function(s) {
    var l = i[s];
    if (z.isValidElement(l)) {
      var u = s in t, c = s in o, f = t[s], p = z.isValidElement(f) && !f.props.in;
      c && (!u || p) ? i[s] = z.cloneElement(l, {
        onExited: n.bind(null, l),
        in: !0,
        exit: za(l, "exit", e),
        enter: za(l, "enter", e)
      }) : !c && u && !p ? i[s] = z.cloneElement(l, {
        in: !1
      }) : c && u && z.isValidElement(f) && (i[s] = z.cloneElement(l, {
        onExited: n.bind(null, l),
        in: f.props.in,
        exit: za(l, "exit", e),
        enter: za(l, "enter", e)
      }));
    }
  }), i;
}
var eJ = Object.values || function(e) {
  return Object.keys(e).map(function(t) {
    return e[t];
  });
}, tJ = {
  component: "div",
  childFactory: function(t) {
    return t;
  }
}, S0 = /* @__PURE__ */ function(e) {
  wE(t, e);
  function t(o, i) {
    var s;
    s = e.call(this, o, i) || this;
    var l = s.handleExited.bind(YG(s));
    return s.state = {
      contextValue: {
        isMounting: !0
      },
      handleExited: l,
      firstRender: !0
    }, s;
  }
  var n = t.prototype;
  return n.componentDidMount = function() {
    this.mounted = !0, this.setState({
      contextValue: {
        isMounting: !1
      }
    });
  }, n.componentWillUnmount = function() {
    this.mounted = !1;
  }, t.getDerivedStateFromProps = function(i, s) {
    var l = s.children, u = s.handleExited, c = s.firstRender;
    return {
      children: c ? XG(i, u) : ZG(i, l, u),
      firstRender: !1
    };
  }, n.handleExited = function(i, s) {
    var l = b0(this.props.children);
    i.key in l || (i.props.onExited && i.props.onExited(s), this.mounted && this.setState(function(u) {
      var c = zf({}, u.children);
      return delete c[i.key], {
        children: c
      };
    }));
  }, n.render = function() {
    var i = this.props, s = i.component, l = i.childFactory, u = SE(i, ["component", "childFactory"]), c = this.state.contextValue, f = eJ(this.state.children).map(l);
    return delete u.appear, delete u.enter, delete u.exit, s === null ? /* @__PURE__ */ lo.createElement(Wf.Provider, {
      value: c
    }, f) : /* @__PURE__ */ lo.createElement(Wf.Provider, {
      value: c
    }, /* @__PURE__ */ lo.createElement(s, u, f));
  }, t;
}(lo.Component);
S0.propTypes = {};
S0.defaultProps = tJ;
function nJ(e) {
  const {
    className: t,
    classes: n,
    pulsate: o = !1,
    rippleX: i,
    rippleY: s,
    rippleSize: l,
    in: u,
    onExited: c,
    timeout: f
  } = e, [p, g] = z.useState(!1), h = Ve(t, n.ripple, n.rippleVisible, o && n.ripplePulsate), y = {
    width: l,
    height: l,
    top: -(l / 2) + s,
    left: -(l / 2) + i
  }, v = Ve(n.child, p && n.childLeaving, o && n.childPulsate);
  return !u && !p && g(!0), z.useEffect(() => {
    if (!u && c != null) {
      const C = setTimeout(c, f);
      return () => {
        clearTimeout(C);
      };
    }
  }, [c, u, f]), /* @__PURE__ */ A.jsx("span", {
    className: h,
    style: y,
    children: /* @__PURE__ */ A.jsx("span", {
      className: v
    })
  });
}
const ao = dt("MuiTouchRipple", ["root", "ripple", "rippleVisible", "ripplePulsate", "child", "childLeaving", "childPulsate"]), Fg = 550, rJ = 80, oJ = ua`
  0% {
    transform: scale(0);
    opacity: 0.1;
  }

  100% {
    transform: scale(1);
    opacity: 0.3;
  }
`, iJ = ua`
  0% {
    opacity: 1;
  }

  100% {
    opacity: 0;
  }
`, aJ = ua`
  0% {
    transform: scale(1);
  }

  50% {
    transform: scale(0.92);
  }

  100% {
    transform: scale(1);
  }
`, sJ = Oe("span", {
  name: "MuiTouchRipple",
  slot: "Root"
})({
  overflow: "hidden",
  pointerEvents: "none",
  position: "absolute",
  zIndex: 0,
  top: 0,
  right: 0,
  bottom: 0,
  left: 0,
  borderRadius: "inherit"
}), lJ = Oe(nJ, {
  name: "MuiTouchRipple",
  slot: "Ripple"
})`
  opacity: 0;
  position: absolute;

  &.${ao.rippleVisible} {
    opacity: 0.3;
    transform: scale(1);
    animation-name: ${oJ};
    animation-duration: ${Fg}ms;
    animation-timing-function: ${({
  theme: e
}) => e.transitions.easing.easeInOut};
  }

  &.${ao.ripplePulsate} {
    animation-duration: ${({
  theme: e
}) => e.transitions.duration.shorter}ms;
  }

  & .${ao.child} {
    opacity: 1;
    display: block;
    width: 100%;
    height: 100%;
    border-radius: 50%;
    background-color: currentColor;
  }

  & .${ao.childLeaving} {
    opacity: 0;
    animation-name: ${iJ};
    animation-duration: ${Fg}ms;
    animation-timing-function: ${({
  theme: e
}) => e.transitions.easing.easeInOut};
  }

  & .${ao.childPulsate} {
    position: absolute;
    /* @noflip */
    left: 0px;
    top: 0;
    animation-name: ${aJ};
    animation-duration: 2500ms;
    animation-timing-function: ${({
  theme: e
}) => e.transitions.easing.easeInOut};
    animation-iteration-count: infinite;
    animation-delay: 200ms;
  }
`, uJ = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiTouchRipple"
  }), {
    center: i = !1,
    classes: s = {},
    className: l,
    ...u
  } = o, [c, f] = z.useState([]), p = z.useRef(0), g = z.useRef(null);
  z.useEffect(() => {
    g.current && (g.current(), g.current = null);
  }, [c]);
  const h = z.useRef(!1), y = r0(), v = z.useRef(null), C = z.useRef(null), $ = z.useCallback((b) => {
    const {
      pulsate: k,
      rippleX: E,
      rippleY: j,
      rippleSize: L,
      cb: B
    } = b;
    f((O) => [...O, /* @__PURE__ */ A.jsx(lJ, {
      classes: {
        ripple: Ve(s.ripple, ao.ripple),
        rippleVisible: Ve(s.rippleVisible, ao.rippleVisible),
        ripplePulsate: Ve(s.ripplePulsate, ao.ripplePulsate),
        child: Ve(s.child, ao.child),
        childLeaving: Ve(s.childLeaving, ao.childLeaving),
        childPulsate: Ve(s.childPulsate, ao.childPulsate)
      },
      timeout: Fg,
      pulsate: k,
      rippleX: E,
      rippleY: j,
      rippleSize: L
    }, p.current)]), p.current += 1, g.current = B;
  }, [s]), S = z.useCallback((b = {}, k = {}, E = () => {
  }) => {
    const {
      pulsate: j = !1,
      center: L = i || k.pulsate,
      fakeElement: B = !1
      // For test purposes
    } = k;
    if ((b == null ? void 0 : b.type) === "mousedown" && h.current) {
      h.current = !1;
      return;
    }
    (b == null ? void 0 : b.type) === "touchstart" && (h.current = !0);
    const O = B ? null : C.current, V = O ? O.getBoundingClientRect() : {
      width: 0,
      height: 0,
      left: 0,
      top: 0
    };
    let W, q, Q;
    if (L || b === void 0 || b.clientX === 0 && b.clientY === 0 || !b.clientX && !b.touches)
      W = Math.round(V.width / 2), q = Math.round(V.height / 2);
    else {
      const {
        clientX: X,
        clientY: ce
      } = b.touches && b.touches.length > 0 ? b.touches[0] : b;
      W = Math.round(X - V.left), q = Math.round(ce - V.top);
    }
    if (L)
      Q = Math.sqrt((2 * V.width ** 2 + V.height ** 2) / 3), Q % 2 === 0 && (Q += 1);
    else {
      const X = Math.max(Math.abs((O ? O.clientWidth : 0) - W), W) * 2 + 2, ce = Math.max(Math.abs((O ? O.clientHeight : 0) - q), q) * 2 + 2;
      Q = Math.sqrt(X ** 2 + ce ** 2);
    }
    b != null && b.touches ? v.current === null && (v.current = () => {
      $({
        pulsate: j,
        rippleX: W,
        rippleY: q,
        rippleSize: Q,
        cb: E
      });
    }, y.start(rJ, () => {
      v.current && (v.current(), v.current = null);
    })) : $({
      pulsate: j,
      rippleX: W,
      rippleY: q,
      rippleSize: Q,
      cb: E
    });
  }, [i, $, y]), w = z.useCallback(() => {
    S({}, {
      pulsate: !0
    });
  }, [S]), _ = z.useCallback((b, k) => {
    if (y.clear(), (b == null ? void 0 : b.type) === "touchend" && v.current) {
      v.current(), v.current = null, y.start(0, () => {
        _(b, k);
      });
      return;
    }
    v.current = null, f((E) => E.length > 0 ? E.slice(1) : E), g.current = k;
  }, [y]);
  return z.useImperativeHandle(n, () => ({
    pulsate: w,
    start: S,
    stop: _
  }), [w, S, _]), /* @__PURE__ */ A.jsx(sJ, {
    className: Ve(ao.root, s.root, l),
    ref: C,
    ...u,
    children: /* @__PURE__ */ A.jsx(S0, {
      component: null,
      exit: !0,
      children: c
    })
  });
});
function cJ(e) {
  return ht("MuiButtonBase", e);
}
const dJ = dt("MuiButtonBase", ["root", "disabled", "focusVisible"]), fJ = (e) => {
  const {
    disabled: t,
    focusVisible: n,
    focusVisibleClassName: o,
    classes: i
  } = e, l = mt({
    root: ["root", t && "disabled", n && "focusVisible"]
  }, cJ, i);
  return n && o && (l.root += ` ${o}`), l;
}, pJ = Oe("button", {
  name: "MuiButtonBase",
  slot: "Root",
  overridesResolver: (e, t) => t.root
})({
  display: "inline-flex",
  alignItems: "center",
  justifyContent: "center",
  position: "relative",
  boxSizing: "border-box",
  WebkitTapHighlightColor: "transparent",
  backgroundColor: "transparent",
  // Reset default value
  // We disable the focus ring for mouse, touch and keyboard users.
  outline: 0,
  border: 0,
  margin: 0,
  // Remove the margin in Safari
  borderRadius: 0,
  padding: 0,
  // Remove the padding in Firefox
  cursor: "pointer",
  userSelect: "none",
  verticalAlign: "middle",
  MozAppearance: "none",
  // Reset
  WebkitAppearance: "none",
  // Reset
  textDecoration: "none",
  // So we take precedent over the style of a native <a /> element.
  color: "inherit",
  "&::-moz-focus-inner": {
    borderStyle: "none"
    // Remove Firefox dotted outline.
  },
  [`&.${dJ.disabled}`]: {
    pointerEvents: "none",
    // Disable link interactions
    cursor: "default"
  },
  "@media print": {
    colorAdjust: "exact"
  }
}), Tc = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiButtonBase"
  }), {
    action: i,
    centerRipple: s = !1,
    children: l,
    className: u,
    component: c = "button",
    disabled: f = !1,
    disableRipple: p = !1,
    disableTouchRipple: g = !1,
    focusRipple: h = !1,
    focusVisibleClassName: y,
    LinkComponent: v = "a",
    onBlur: C,
    onClick: $,
    onContextMenu: S,
    onDragLeave: w,
    onFocus: _,
    onFocusVisible: b,
    onKeyDown: k,
    onKeyUp: E,
    onMouseDown: j,
    onMouseLeave: L,
    onMouseUp: B,
    onTouchEnd: O,
    onTouchMove: V,
    onTouchStart: W,
    tabIndex: q = 0,
    TouchRippleProps: Q,
    touchRippleRef: X,
    type: ce,
    ...J
  } = o, ie = z.useRef(null), re = HG(), Y = _n(re.ref, X), [T, H] = z.useState(!1);
  f && T && H(!1), z.useImperativeHandle(i, () => ({
    focusVisible: () => {
      H(!0), ie.current.focus();
    }
  }), []);
  const G = re.shouldMount && !p && !f;
  z.useEffect(() => {
    T && h && !p && re.pulsate();
  }, [p, h, T, re]);
  const N = mi(re, "start", j, g), U = mi(re, "stop", S, g), Z = mi(re, "stop", w, g), me = mi(re, "stop", B, g), ue = mi(re, "stop", (Le) => {
    T && Le.preventDefault(), L && L(Le);
  }, g), Ce = mi(re, "start", W, g), $e = mi(re, "stop", O, g), Te = mi(re, "stop", V, g), We = mi(re, "stop", (Le) => {
    Lf(Le.target) || H(!1), C && C(Le);
  }, !1), et = yi((Le) => {
    ie.current || (ie.current = Le.currentTarget), Lf(Le.target) && (H(!0), b && b(Le)), _ && _(Le);
  }), Xe = () => {
    const Le = ie.current;
    return c && c !== "button" && !(Le.tagName === "A" && Le.href);
  }, ft = yi((Le) => {
    h && !Le.repeat && T && Le.key === " " && re.stop(Le, () => {
      re.start(Le);
    }), Le.target === Le.currentTarget && Xe() && Le.key === " " && Le.preventDefault(), k && k(Le), Le.target === Le.currentTarget && Xe() && Le.key === "Enter" && !f && (Le.preventDefault(), $ && $(Le));
  }), Lt = yi((Le) => {
    h && Le.key === " " && T && !Le.defaultPrevented && re.stop(Le, () => {
      re.pulsate(Le);
    }), E && E(Le), $ && Le.target === Le.currentTarget && Xe() && Le.key === " " && !Le.defaultPrevented && $(Le);
  });
  let st = c;
  st === "button" && (J.href || J.to) && (st = v);
  const pt = {};
  st === "button" ? (pt.type = ce === void 0 ? "button" : ce, pt.disabled = f) : (!J.href && !J.to && (pt.role = "button"), f && (pt["aria-disabled"] = f));
  const Se = _n(n, ie), gt = {
    ...o,
    centerRipple: s,
    component: c,
    disabled: f,
    disableRipple: p,
    disableTouchRipple: g,
    focusRipple: h,
    tabIndex: q,
    focusVisible: T
  }, yt = fJ(gt);
  return /* @__PURE__ */ A.jsxs(pJ, {
    as: st,
    className: Ve(yt.root, u),
    ownerState: gt,
    onBlur: We,
    onClick: $,
    onContextMenu: U,
    onFocus: et,
    onKeyDown: ft,
    onKeyUp: Lt,
    onMouseDown: N,
    onMouseLeave: ue,
    onMouseUp: me,
    onDragLeave: Z,
    onTouchEnd: $e,
    onTouchMove: Te,
    onTouchStart: Ce,
    ref: Se,
    tabIndex: f ? -1 : q,
    type: ce,
    ...pt,
    ...J,
    children: [l, G ? /* @__PURE__ */ A.jsx(uJ, {
      ref: Y,
      center: s,
      ...Q
    }) : null]
  });
});
function mi(e, t, n, o = !1) {
  return yi((i) => (n && n(i), o || e[t](i), !0));
}
function hJ(e) {
  return ht("MuiIconButton", e);
}
const mJ = dt("MuiIconButton", ["root", "disabled", "colorInherit", "colorPrimary", "colorSecondary", "colorError", "colorInfo", "colorSuccess", "colorWarning", "edgeStart", "edgeEnd", "sizeSmall", "sizeMedium", "sizeLarge"]), gJ = (e) => {
  const {
    classes: t,
    disabled: n,
    color: o,
    edge: i,
    size: s
  } = e, l = {
    root: ["root", n && "disabled", o !== "default" && `color${Ne(o)}`, i && `edge${Ne(i)}`, `size${Ne(s)}`]
  };
  return mt(l, hJ, t);
}, yJ = Oe(Tc, {
  name: "MuiIconButton",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, n.color !== "default" && t[`color${Ne(n.color)}`], n.edge && t[`edge${Ne(n.edge)}`], t[`size${Ne(n.size)}`]];
  }
})(ct(({
  theme: e
}) => ({
  textAlign: "center",
  flex: "0 0 auto",
  fontSize: e.typography.pxToRem(24),
  padding: 8,
  borderRadius: "50%",
  color: (e.vars || e).palette.action.active,
  transition: e.transitions.create("background-color", {
    duration: e.transitions.duration.shortest
  }),
  variants: [{
    props: (t) => !t.disableRipple,
    style: {
      "--IconButton-hoverBg": e.vars ? `rgba(${e.vars.palette.action.activeChannel} / ${e.vars.palette.action.hoverOpacity})` : En(e.palette.action.active, e.palette.action.hoverOpacity),
      "&:hover": {
        backgroundColor: "var(--IconButton-hoverBg)",
        // Reset on touch devices, it doesn't add specificity
        "@media (hover: none)": {
          backgroundColor: "transparent"
        }
      }
    }
  }, {
    props: {
      edge: "start"
    },
    style: {
      marginLeft: -12
    }
  }, {
    props: {
      edge: "start",
      size: "small"
    },
    style: {
      marginLeft: -3
    }
  }, {
    props: {
      edge: "end"
    },
    style: {
      marginRight: -12
    }
  }, {
    props: {
      edge: "end",
      size: "small"
    },
    style: {
      marginRight: -3
    }
  }]
})), ct(({
  theme: e
}) => ({
  variants: [{
    props: {
      color: "inherit"
    },
    style: {
      color: "inherit"
    }
  }, ...Object.entries(e.palette).filter(kn()).map(([t]) => ({
    props: {
      color: t
    },
    style: {
      color: (e.vars || e).palette[t].main
    }
  })), ...Object.entries(e.palette).filter(kn()).map(([t]) => ({
    props: {
      color: t
    },
    style: {
      "--IconButton-hoverBg": e.vars ? `rgba(${(e.vars || e).palette[t].mainChannel} / ${e.vars.palette.action.hoverOpacity})` : En((e.vars || e).palette[t].main, e.palette.action.hoverOpacity)
    }
  })), {
    props: {
      size: "small"
    },
    style: {
      padding: 5,
      fontSize: e.typography.pxToRem(18)
    }
  }, {
    props: {
      size: "large"
    },
    style: {
      padding: 12,
      fontSize: e.typography.pxToRem(28)
    }
  }],
  [`&.${mJ.disabled}`]: {
    backgroundColor: "transparent",
    color: (e.vars || e).palette.action.disabled
  }
}))), $E = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiIconButton"
  }), {
    edge: i = !1,
    children: s,
    className: l,
    color: u = "default",
    disabled: c = !1,
    disableFocusRipple: f = !1,
    size: p = "medium",
    ...g
  } = o, h = {
    ...o,
    edge: i,
    color: u,
    disabled: c,
    disableFocusRipple: f,
    size: p
  }, y = gJ(h);
  return /* @__PURE__ */ A.jsx(yJ, {
    className: Ve(y.root, l),
    centerRipple: !0,
    focusRipple: !f,
    disabled: c,
    ref: n,
    ...g,
    ownerState: h,
    children: s
  });
});
function vJ({ uiSchema: e, registry: t, ...n }) {
  const { translateString: o } = t;
  return A.jsx($E, { title: o(jt.AddItemButton), ...n, color: "primary", children: A.jsx(WG, {}) });
}
const bJ = dt("MuiBox", ["root"]), SJ = bE(), wr = wH({
  themeId: Lp,
  defaultTheme: SJ,
  defaultClassName: bJ.root,
  generateClassName: zx.generate
}), ew = /* @__PURE__ */ z.createContext();
function wJ(e) {
  return ht("MuiGrid", e);
}
const $J = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10], CJ = ["column-reverse", "column", "row-reverse", "row"], xJ = ["nowrap", "wrap-reverse", "wrap"], _u = ["auto", !0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12], pc = dt("MuiGrid", [
  "root",
  "container",
  "item",
  "zeroMinWidth",
  // spacings
  ...$J.map((e) => `spacing-xs-${e}`),
  // direction values
  ...CJ.map((e) => `direction-xs-${e}`),
  // wrap values
  ...xJ.map((e) => `wrap-xs-${e}`),
  // grid sizes for all breakpoints
  ..._u.map((e) => `grid-xs-${e}`),
  ..._u.map((e) => `grid-sm-${e}`),
  ..._u.map((e) => `grid-md-${e}`),
  ..._u.map((e) => `grid-lg-${e}`),
  ..._u.map((e) => `grid-xl-${e}`)
]);
function EJ({
  theme: e,
  ownerState: t
}) {
  let n;
  return e.breakpoints.keys.reduce((o, i) => {
    let s = {};
    if (t[i] && (n = t[i]), !n)
      return o;
    if (n === !0)
      s = {
        flexBasis: 0,
        flexGrow: 1,
        maxWidth: "100%"
      };
    else if (n === "auto")
      s = {
        flexBasis: "auto",
        flexGrow: 0,
        flexShrink: 0,
        maxWidth: "none",
        width: "auto"
      };
    else {
      const l = Cp({
        values: t.columns,
        breakpoints: e.breakpoints.values
      }), u = typeof l == "object" ? l[i] : l;
      if (u == null)
        return o;
      const c = `${Math.round(n / u * 1e8) / 1e6}%`;
      let f = {};
      if (t.container && t.item && t.columnSpacing !== 0) {
        const p = e.spacing(t.columnSpacing);
        if (p !== "0px") {
          const g = `calc(${c} + ${p})`;
          f = {
            flexBasis: g,
            maxWidth: g
          };
        }
      }
      s = {
        flexBasis: c,
        flexGrow: 0,
        maxWidth: c,
        ...f
      };
    }
    return e.breakpoints.values[i] === 0 ? Object.assign(o, s) : o[e.breakpoints.up(i)] = s, o;
  }, {});
}
function _J({
  theme: e,
  ownerState: t
}) {
  const n = Cp({
    values: t.direction,
    breakpoints: e.breakpoints.values
  });
  return ho({
    theme: e
  }, n, (o) => {
    const i = {
      flexDirection: o
    };
    return o.startsWith("column") && (i[`& > .${pc.item}`] = {
      maxWidth: "none"
    }), i;
  });
}
function CE({
  breakpoints: e,
  values: t
}) {
  let n = "";
  Object.keys(t).forEach((i) => {
    n === "" && t[i] !== 0 && (n = i);
  });
  const o = Object.keys(e).sort((i, s) => e[i] - e[s]);
  return o.slice(0, o.indexOf(n));
}
function kJ({
  theme: e,
  ownerState: t
}) {
  const {
    container: n,
    rowSpacing: o
  } = t;
  let i = {};
  if (n && o !== 0) {
    const s = Cp({
      values: o,
      breakpoints: e.breakpoints.values
    });
    let l;
    typeof s == "object" && (l = CE({
      breakpoints: e.breakpoints.values,
      values: s
    })), i = ho({
      theme: e
    }, s, (u, c) => {
      const f = e.spacing(u);
      return f !== "0px" ? {
        marginTop: `calc(-1 * ${f})`,
        [`& > .${pc.item}`]: {
          paddingTop: f
        }
      } : l != null && l.includes(c) ? {} : {
        marginTop: 0,
        [`& > .${pc.item}`]: {
          paddingTop: 0
        }
      };
    });
  }
  return i;
}
function PJ({
  theme: e,
  ownerState: t
}) {
  const {
    container: n,
    columnSpacing: o
  } = t;
  let i = {};
  if (n && o !== 0) {
    const s = Cp({
      values: o,
      breakpoints: e.breakpoints.values
    });
    let l;
    typeof s == "object" && (l = CE({
      breakpoints: e.breakpoints.values,
      values: s
    })), i = ho({
      theme: e
    }, s, (u, c) => {
      const f = e.spacing(u);
      if (f !== "0px") {
        const p = `calc(-1 * ${f})`;
        return {
          width: `calc(100% + ${f})`,
          marginLeft: p,
          [`& > .${pc.item}`]: {
            paddingLeft: f
          }
        };
      }
      return l != null && l.includes(c) ? {} : {
        width: "100%",
        marginLeft: 0,
        [`& > .${pc.item}`]: {
          paddingLeft: 0
        }
      };
    });
  }
  return i;
}
function OJ(e, t, n = {}) {
  if (!e || e <= 0)
    return [];
  if (typeof e == "string" && !Number.isNaN(Number(e)) || typeof e == "number")
    return [n[`spacing-xs-${String(e)}`]];
  const o = [];
  return t.forEach((i) => {
    const s = e[i];
    Number(s) > 0 && o.push(n[`spacing-${i}-${String(s)}`]);
  }), o;
}
const AJ = Oe("div", {
  name: "MuiGrid",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e, {
      container: o,
      direction: i,
      item: s,
      spacing: l,
      wrap: u,
      zeroMinWidth: c,
      breakpoints: f
    } = n;
    let p = [];
    o && (p = OJ(l, f, t));
    const g = [];
    return f.forEach((h) => {
      const y = n[h];
      y && g.push(t[`grid-${h}-${String(y)}`]);
    }), [t.root, o && t.container, s && t.item, c && t.zeroMinWidth, ...p, i !== "row" && t[`direction-xs-${String(i)}`], u !== "wrap" && t[`wrap-xs-${String(u)}`], ...g];
  }
})(
  // FIXME(romgrk): Can't use memoTheme here
  ({
    ownerState: e
  }) => ({
    boxSizing: "border-box",
    ...e.container && {
      display: "flex",
      flexWrap: "wrap",
      width: "100%"
    },
    ...e.item && {
      margin: 0
      // For instance, it's useful when used with a `figure` element.
    },
    ...e.zeroMinWidth && {
      minWidth: 0
    },
    ...e.wrap !== "wrap" && {
      flexWrap: e.wrap
    }
  }),
  _J,
  kJ,
  PJ,
  EJ
);
function IJ(e, t) {
  if (!e || e <= 0)
    return [];
  if (typeof e == "string" && !Number.isNaN(Number(e)) || typeof e == "number")
    return [`spacing-xs-${String(e)}`];
  const n = [];
  return t.forEach((o) => {
    const i = e[o];
    if (Number(i) > 0) {
      const s = `spacing-${o}-${String(i)}`;
      n.push(s);
    }
  }), n;
}
const jJ = (e) => {
  const {
    classes: t,
    container: n,
    direction: o,
    item: i,
    spacing: s,
    wrap: l,
    zeroMinWidth: u,
    breakpoints: c
  } = e;
  let f = [];
  n && (f = IJ(s, c));
  const p = [];
  c.forEach((h) => {
    const y = e[h];
    y && p.push(`grid-${h}-${String(y)}`);
  });
  const g = {
    root: ["root", n && "container", i && "item", u && "zeroMinWidth", ...f, o !== "row" && `direction-xs-${String(o)}`, l !== "wrap" && `wrap-xs-${String(l)}`, ...p]
  };
  return mt(g, wJ, t);
}, Kr = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiGrid"
  }), {
    breakpoints: i
  } = Tl(), s = Ap(o), {
    className: l,
    columns: u,
    columnSpacing: c,
    component: f = "div",
    container: p = !1,
    direction: g = "row",
    item: h = !1,
    rowSpacing: y,
    spacing: v = 0,
    wrap: C = "wrap",
    zeroMinWidth: $ = !1,
    ...S
  } = s, w = y || v, _ = c || v, b = z.useContext(ew), k = p ? u || 12 : b, E = {}, j = {
    ...S
  };
  i.keys.forEach((O) => {
    S[O] != null && (E[O] = S[O], delete j[O]);
  });
  const L = {
    ...s,
    columns: k,
    container: p,
    direction: g,
    item: h,
    rowSpacing: w,
    columnSpacing: _,
    wrap: C,
    zeroMinWidth: $,
    spacing: v,
    ...E,
    breakpoints: i.keys
  }, B = jJ(L);
  return /* @__PURE__ */ A.jsx(ew.Provider, {
    value: k,
    children: /* @__PURE__ */ A.jsx(AJ, {
      ownerState: L,
      className: Ve(B.root, l),
      as: f,
      ref: n,
      ...j
    })
  });
});
function RJ(e) {
  return ht("MuiPaper", e);
}
dt("MuiPaper", ["root", "rounded", "outlined", "elevation", "elevation0", "elevation1", "elevation2", "elevation3", "elevation4", "elevation5", "elevation6", "elevation7", "elevation8", "elevation9", "elevation10", "elevation11", "elevation12", "elevation13", "elevation14", "elevation15", "elevation16", "elevation17", "elevation18", "elevation19", "elevation20", "elevation21", "elevation22", "elevation23", "elevation24"]);
const TJ = (e) => {
  const {
    square: t,
    elevation: n,
    variant: o,
    classes: i
  } = e, s = {
    root: ["root", o, !t && "rounded", o === "elevation" && `elevation${n}`]
  };
  return mt(s, RJ, i);
}, MJ = Oe("div", {
  name: "MuiPaper",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, t[n.variant], !n.square && t.rounded, n.variant === "elevation" && t[`elevation${n.elevation}`]];
  }
})(ct(({
  theme: e
}) => ({
  backgroundColor: (e.vars || e).palette.background.paper,
  color: (e.vars || e).palette.text.primary,
  transition: e.transitions.create("box-shadow"),
  variants: [{
    props: ({
      ownerState: t
    }) => !t.square,
    style: {
      borderRadius: e.shape.borderRadius
    }
  }, {
    props: {
      variant: "outlined"
    },
    style: {
      border: `1px solid ${(e.vars || e).palette.divider}`
    }
  }, {
    props: {
      variant: "elevation"
    },
    style: {
      boxShadow: "var(--Paper-shadow)",
      backgroundImage: "var(--Paper-overlay)"
    }
  }]
}))), Ml = /* @__PURE__ */ z.forwardRef(function(t, n) {
  var y;
  const o = bt({
    props: t,
    name: "MuiPaper"
  }), i = Tl(), {
    className: s,
    component: l = "div",
    elevation: u = 1,
    square: c = !1,
    variant: f = "elevation",
    ...p
  } = o, g = {
    ...o,
    component: l,
    elevation: u,
    square: c,
    variant: f
  }, h = TJ(g);
  return /* @__PURE__ */ A.jsx(MJ, {
    as: l,
    ownerState: g,
    className: Ve(h.root, s),
    ref: n,
    ...p,
    style: {
      ...f === "elevation" && {
        "--Paper-shadow": (i.vars || i).shadows[u],
        ...i.vars && {
          "--Paper-overlay": (y = i.vars.overlays) == null ? void 0 : y[u]
        },
        ...!i.vars && i.palette.mode === "dark" && {
          "--Paper-overlay": `linear-gradient(${En("#fff", Rg(u))}, ${En("#fff", Rg(u))})`
        }
      },
      ...p.style
    }
  });
});
function NJ(e) {
  const { children: t, disabled: n, hasToolbar: o, hasCopy: i, hasMoveDown: s, hasMoveUp: l, hasRemove: u, index: c, onCopyIndexClick: f, onDropIndexClick: p, onReorderClick: g, readonly: h, uiSchema: y, registry: v } = e, { CopyButton: C, MoveDownButton: $, MoveUpButton: S, RemoveButton: w } = v.templates.ButtonTemplates, _ = {
    flex: 1,
    paddingLeft: 6,
    paddingRight: 6,
    fontWeight: "bold",
    minWidth: 0
  };
  return A.jsxs(Kr, { container: !0, alignItems: "center", children: [A.jsx(Kr, { item: !0, xs: !0, style: { overflow: "auto" }, children: A.jsx(wr, { mb: 2, children: A.jsx(Ml, { elevation: 2, children: A.jsx(wr, { p: 2, children: t }) }) }) }), o && A.jsxs(Kr, { item: !0, children: [(l || s) && A.jsx(S, { style: _, disabled: n || h || !l, onClick: g(c, c - 1), uiSchema: y, registry: v }), (l || s) && A.jsx($, { style: _, disabled: n || h || !s, onClick: g(c, c + 1), uiSchema: y, registry: v }), i && A.jsx(C, { style: _, disabled: n || h, onClick: f(c), uiSchema: y, registry: v }), u && A.jsx(w, { style: _, disabled: n || h, onClick: p(c), uiSchema: y, registry: v })] })] });
}
function FJ(e) {
  const { canAdd: t, disabled: n, idSchema: o, uiSchema: i, items: s, onAddClick: l, readonly: u, registry: c, required: f, schema: p, title: g } = e, h = Ot(i), y = _t("ArrayFieldDescriptionTemplate", c, h), v = _t("ArrayFieldItemTemplate", c, h), C = _t("ArrayFieldTitleTemplate", c, h), { ButtonTemplates: { AddButton: $ } } = c.templates;
  return A.jsx(Ml, { elevation: 2, children: A.jsxs(wr, { p: 2, children: [A.jsx(C, { idSchema: o, title: h.title || g, schema: p, uiSchema: i, required: f, registry: c }), A.jsx(y, { idSchema: o, description: h.description || p.description, schema: p, uiSchema: i, registry: c }), s && s.map(({ key: S, ...w }) => A.jsx(v, { ...w }, S)), t && A.jsx(Kr, { container: !0, justifyContent: "flex-end", children: A.jsx(Kr, { item: !0, children: A.jsx(wr, { mt: 2, children: A.jsx($, { className: "array-item-add", onClick: l, disabled: n || u, uiSchema: i, registry: c }) }) }) })] }) });
}
function Xd(e) {
  return parseInt(e, 10) || 0;
}
const LJ = {
  shadow: {
    // Visibility needed to hide the extra text area on iPads
    visibility: "hidden",
    // Remove from the content flow
    position: "absolute",
    // Ignore the scrollbar width
    overflow: "hidden",
    height: 0,
    top: 0,
    left: 0,
    // Create a new layer, increase the isolation of the computed values
    transform: "translateZ(0)"
  }
};
function DJ(e) {
  return e == null || Object.keys(e).length === 0 || e.outerHeightStyle === 0 && !e.overflowing;
}
const zJ = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const {
    onChange: o,
    maxRows: i,
    minRows: s = 1,
    style: l,
    value: u,
    ...c
  } = t, {
    current: f
  } = z.useRef(u != null), p = z.useRef(null), g = _n(n, p), h = z.useRef(null), y = z.useRef(null), v = z.useCallback(() => {
    const S = p.current, _ = xi(S).getComputedStyle(S);
    if (_.width === "0px")
      return {
        outerHeightStyle: 0,
        overflowing: !1
      };
    const b = y.current;
    b.style.width = _.width, b.value = S.value || t.placeholder || "x", b.value.slice(-1) === `
` && (b.value += " ");
    const k = _.boxSizing, E = Xd(_.paddingBottom) + Xd(_.paddingTop), j = Xd(_.borderBottomWidth) + Xd(_.borderTopWidth), L = b.scrollHeight;
    b.value = "x";
    const B = b.scrollHeight;
    let O = L;
    s && (O = Math.max(Number(s) * B, O)), i && (O = Math.min(Number(i) * B, O)), O = Math.max(O, B);
    const V = O + (k === "border-box" ? E + j : 0), W = Math.abs(O - L) <= 1;
    return {
      outerHeightStyle: V,
      overflowing: W
    };
  }, [i, s, t.placeholder]), C = z.useCallback(() => {
    const S = v();
    if (DJ(S))
      return;
    const w = S.outerHeightStyle, _ = p.current;
    h.current !== w && (h.current = w, _.style.height = `${w}px`), _.style.overflow = S.overflowing ? "hidden" : "";
  }, [v]);
  Ei(() => {
    const S = () => {
      C();
    };
    let w;
    const _ = Fx(S), b = p.current, k = xi(b);
    k.addEventListener("resize", _);
    let E;
    return typeof ResizeObserver < "u" && (E = new ResizeObserver(S), E.observe(b)), () => {
      _.clear(), cancelAnimationFrame(w), k.removeEventListener("resize", _), E && E.disconnect();
    };
  }, [v, C]), Ei(() => {
    C();
  });
  const $ = (S) => {
    f || C(), o && o(S);
  };
  return /* @__PURE__ */ A.jsxs(z.Fragment, {
    children: [/* @__PURE__ */ A.jsx("textarea", {
      value: u,
      onChange: $,
      ref: g,
      rows: s,
      style: l,
      ...c
    }), /* @__PURE__ */ A.jsx("textarea", {
      "aria-hidden": !0,
      className: t.className,
      readOnly: !0,
      ref: y,
      tabIndex: -1,
      style: {
        ...LJ.shadow,
        ...l,
        paddingTop: 0,
        paddingBottom: 0
      }
    })]
  });
});
function vi(e) {
  return typeof e == "string";
}
function ca({
  props: e,
  states: t,
  muiFormControl: n
}) {
  return t.reduce((o, i) => (o[i] = e[i], n && typeof e[i] > "u" && (o[i] = n[i]), o), {});
}
const w0 = /* @__PURE__ */ z.createContext(void 0);
function ti() {
  return z.useContext(w0);
}
function tw(e) {
  return e != null && !(Array.isArray(e) && e.length === 0);
}
function qf(e, t = !1) {
  return e && (tw(e.value) && e.value !== "" || t && tw(e.defaultValue) && e.defaultValue !== "");
}
function BJ(e) {
  return e.startAdornment;
}
function UJ(e) {
  return ht("MuiInputBase", e);
}
const dl = dt("MuiInputBase", ["root", "formControl", "focused", "disabled", "adornedStart", "adornedEnd", "error", "sizeSmall", "multiline", "colorSecondary", "fullWidth", "hiddenLabel", "readOnly", "input", "inputSizeSmall", "inputMultiline", "inputTypeSearch", "inputAdornedStart", "inputAdornedEnd", "inputHiddenLabel"]);
var nw;
const zp = (e, t) => {
  const {
    ownerState: n
  } = e;
  return [t.root, n.formControl && t.formControl, n.startAdornment && t.adornedStart, n.endAdornment && t.adornedEnd, n.error && t.error, n.size === "small" && t.sizeSmall, n.multiline && t.multiline, n.color && t[`color${Ne(n.color)}`], n.fullWidth && t.fullWidth, n.hiddenLabel && t.hiddenLabel];
}, Bp = (e, t) => {
  const {
    ownerState: n
  } = e;
  return [t.input, n.size === "small" && t.inputSizeSmall, n.multiline && t.inputMultiline, n.type === "search" && t.inputTypeSearch, n.startAdornment && t.inputAdornedStart, n.endAdornment && t.inputAdornedEnd, n.hiddenLabel && t.inputHiddenLabel];
}, VJ = (e) => {
  const {
    classes: t,
    color: n,
    disabled: o,
    error: i,
    endAdornment: s,
    focused: l,
    formControl: u,
    fullWidth: c,
    hiddenLabel: f,
    multiline: p,
    readOnly: g,
    size: h,
    startAdornment: y,
    type: v
  } = e, C = {
    root: ["root", `color${Ne(n)}`, o && "disabled", i && "error", c && "fullWidth", l && "focused", u && "formControl", h && h !== "medium" && `size${Ne(h)}`, p && "multiline", y && "adornedStart", s && "adornedEnd", f && "hiddenLabel", g && "readOnly"],
    input: ["input", o && "disabled", v === "search" && "inputTypeSearch", p && "inputMultiline", h === "small" && "inputSizeSmall", f && "inputHiddenLabel", y && "inputAdornedStart", s && "inputAdornedEnd", g && "readOnly"]
  };
  return mt(C, UJ, t);
}, Up = Oe("div", {
  name: "MuiInputBase",
  slot: "Root",
  overridesResolver: zp
})(ct(({
  theme: e
}) => ({
  ...e.typography.body1,
  color: (e.vars || e).palette.text.primary,
  lineHeight: "1.4375em",
  // 23px
  boxSizing: "border-box",
  // Prevent padding issue with fullWidth.
  position: "relative",
  cursor: "text",
  display: "inline-flex",
  alignItems: "center",
  [`&.${dl.disabled}`]: {
    color: (e.vars || e).palette.text.disabled,
    cursor: "default"
  },
  variants: [{
    props: ({
      ownerState: t
    }) => t.multiline,
    style: {
      padding: "4px 0 5px"
    }
  }, {
    props: ({
      ownerState: t,
      size: n
    }) => t.multiline && n === "small",
    style: {
      paddingTop: 1
    }
  }, {
    props: ({
      ownerState: t
    }) => t.fullWidth,
    style: {
      width: "100%"
    }
  }]
}))), Vp = Oe("input", {
  name: "MuiInputBase",
  slot: "Input",
  overridesResolver: Bp
})(ct(({
  theme: e
}) => {
  const t = e.palette.mode === "light", n = {
    color: "currentColor",
    ...e.vars ? {
      opacity: e.vars.opacity.inputPlaceholder
    } : {
      opacity: t ? 0.42 : 0.5
    },
    transition: e.transitions.create("opacity", {
      duration: e.transitions.duration.shorter
    })
  }, o = {
    opacity: "0 !important"
  }, i = e.vars ? {
    opacity: e.vars.opacity.inputPlaceholder
  } : {
    opacity: t ? 0.42 : 0.5
  };
  return {
    font: "inherit",
    letterSpacing: "inherit",
    color: "currentColor",
    padding: "4px 0 5px",
    border: 0,
    boxSizing: "content-box",
    background: "none",
    height: "1.4375em",
    // Reset 23pxthe native input line-height
    margin: 0,
    // Reset for Safari
    WebkitTapHighlightColor: "transparent",
    display: "block",
    // Make the flex item shrink with Firefox
    minWidth: 0,
    width: "100%",
    "&::-webkit-input-placeholder": n,
    "&::-moz-placeholder": n,
    // Firefox 19+
    "&::-ms-input-placeholder": n,
    // Edge
    "&:focus": {
      outline: 0
    },
    // Reset Firefox invalid required input style
    "&:invalid": {
      boxShadow: "none"
    },
    "&::-webkit-search-decoration": {
      // Remove the padding when type=search.
      WebkitAppearance: "none"
    },
    // Show and hide the placeholder logic
    [`label[data-shrink=false] + .${dl.formControl} &`]: {
      "&::-webkit-input-placeholder": o,
      "&::-moz-placeholder": o,
      // Firefox 19+
      "&::-ms-input-placeholder": o,
      // Edge
      "&:focus::-webkit-input-placeholder": i,
      "&:focus::-moz-placeholder": i,
      // Firefox 19+
      "&:focus::-ms-input-placeholder": i
      // Edge
    },
    [`&.${dl.disabled}`]: {
      opacity: 1,
      // Reset iOS opacity
      WebkitTextFillColor: (e.vars || e).palette.text.disabled
      // Fix opacity Safari bug
    },
    variants: [{
      props: ({
        ownerState: s
      }) => !s.disableInjectingGlobalStyles,
      style: {
        animationName: "mui-auto-fill-cancel",
        animationDuration: "10ms",
        "&:-webkit-autofill": {
          animationDuration: "5000s",
          animationName: "mui-auto-fill"
        }
      }
    }, {
      props: {
        size: "small"
      },
      style: {
        paddingTop: 1
      }
    }, {
      props: ({
        ownerState: s
      }) => s.multiline,
      style: {
        height: "auto",
        resize: "none",
        padding: 0,
        paddingTop: 0
      }
    }, {
      props: {
        type: "search"
      },
      style: {
        MozAppearance: "textfield"
        // Improve type search style.
      }
    }]
  };
})), rw = DG({
  "@keyframes mui-auto-fill": {
    from: {
      display: "block"
    }
  },
  "@keyframes mui-auto-fill-cancel": {
    from: {
      display: "block"
    }
  }
}), $0 = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiInputBase"
  }), {
    "aria-describedby": i,
    autoComplete: s,
    autoFocus: l,
    className: u,
    color: c,
    components: f = {},
    componentsProps: p = {},
    defaultValue: g,
    disabled: h,
    disableInjectingGlobalStyles: y,
    endAdornment: v,
    error: C,
    fullWidth: $ = !1,
    id: S,
    inputComponent: w = "input",
    inputProps: _ = {},
    inputRef: b,
    margin: k,
    maxRows: E,
    minRows: j,
    multiline: L = !1,
    name: B,
    onBlur: O,
    onChange: V,
    onClick: W,
    onFocus: q,
    onKeyDown: Q,
    onKeyUp: X,
    placeholder: ce,
    readOnly: J,
    renderSuffix: ie,
    rows: re,
    size: Y,
    slotProps: T = {},
    slots: H = {},
    startAdornment: G,
    type: N = "text",
    value: U,
    ...Z
  } = o, me = _.value != null ? _.value : U, {
    current: ue
  } = z.useRef(me != null), Ce = z.useRef(), $e = z.useCallback((be) => {
  }, []), Te = _n(Ce, b, _.ref, $e), [We, et] = z.useState(!1), Xe = ti(), ft = ca({
    props: o,
    muiFormControl: Xe,
    states: ["color", "disabled", "error", "hiddenLabel", "size", "required", "filled"]
  });
  ft.focused = Xe ? Xe.focused : We, z.useEffect(() => {
    !Xe && h && We && (et(!1), O && O());
  }, [Xe, h, We, O]);
  const Lt = Xe && Xe.onFilled, st = Xe && Xe.onEmpty, pt = z.useCallback((be) => {
    qf(be) ? Lt && Lt() : st && st();
  }, [Lt, st]);
  Ei(() => {
    ue && pt({
      value: me
    });
  }, [me, pt, ue]);
  const Se = (be) => {
    q && q(be), _.onFocus && _.onFocus(be), Xe && Xe.onFocus ? Xe.onFocus(be) : et(!0);
  }, gt = (be) => {
    O && O(be), _.onBlur && _.onBlur(be), Xe && Xe.onBlur ? Xe.onBlur(be) : et(!1);
  }, yt = (be, ...Re) => {
    if (!ue) {
      const Ge = be.target || Ce.current;
      if (Ge == null)
        throw new Error(Ci(1));
      pt({
        value: Ge.value
      });
    }
    _.onChange && _.onChange(be, ...Re), V && V(be, ...Re);
  };
  z.useEffect(() => {
    pt(Ce.current);
  }, []);
  const Le = (be) => {
    Ce.current && be.currentTarget === be.target && Ce.current.focus(), W && W(be);
  };
  let cn = w, ye = _;
  L && cn === "input" && (re ? ye = {
    type: void 0,
    minRows: re,
    maxRows: re,
    ...ye
  } : ye = {
    type: void 0,
    maxRows: E,
    minRows: j,
    ...ye
  }, cn = zJ);
  const Pe = (be) => {
    pt(be.animationName === "mui-auto-fill-cancel" ? Ce.current : {
      value: "x"
    });
  };
  z.useEffect(() => {
    Xe && Xe.setAdornedStart(!!G);
  }, [Xe, G]);
  const Me = {
    ...o,
    color: ft.color || "primary",
    disabled: ft.disabled,
    endAdornment: v,
    error: ft.error,
    focused: ft.focused,
    formControl: Xe,
    fullWidth: $,
    hiddenLabel: ft.hiddenLabel,
    multiline: L,
    size: ft.size,
    startAdornment: G,
    type: N
  }, Ye = VJ(Me), ot = H.root || f.Root || Up, se = T.root || p.root || {}, ae = H.input || f.Input || Vp;
  return ye = {
    ...ye,
    ...T.input ?? p.input
  }, /* @__PURE__ */ A.jsxs(z.Fragment, {
    children: [!y && typeof rw == "function" && // For Emotion/Styled-components, InputGlobalStyles will be a function
    // For Pigment CSS, this has no effect because the InputGlobalStyles will be null.
    (nw || (nw = /* @__PURE__ */ A.jsx(rw, {}))), /* @__PURE__ */ A.jsxs(ot, {
      ...se,
      ref: n,
      onClick: Le,
      ...Z,
      ...!vi(ot) && {
        ownerState: {
          ...Me,
          ...se.ownerState
        }
      },
      className: Ve(Ye.root, se.className, u, J && "MuiInputBase-readOnly"),
      children: [G, /* @__PURE__ */ A.jsx(w0.Provider, {
        value: null,
        children: /* @__PURE__ */ A.jsx(ae, {
          "aria-invalid": ft.error,
          "aria-describedby": i,
          autoComplete: s,
          autoFocus: l,
          defaultValue: g,
          disabled: ft.disabled,
          id: S,
          onAnimationStart: Pe,
          name: B,
          placeholder: ce,
          readOnly: J,
          required: ft.required,
          rows: re,
          value: me,
          onKeyDown: Q,
          onKeyUp: X,
          type: N,
          ...ye,
          ...!vi(ae) && {
            as: cn,
            ownerState: {
              ...Me,
              ...ye.ownerState
            }
          },
          ref: Te,
          className: Ve(Ye.input, ye.className, J && "MuiInputBase-readOnly"),
          onBlur: gt,
          onChange: yt,
          onFocus: Se
        })
      }), v, ie ? ie({
        ...ft,
        startAdornment: G
      }) : null]
    })]
  });
});
function WJ(e) {
  return ht("MuiInput", e);
}
const ku = {
  ...dl,
  ...dt("MuiInput", ["root", "underline", "input"])
}, qJ = (e) => {
  const {
    classes: t,
    disableUnderline: n
  } = e, i = mt({
    root: ["root", !n && "underline"],
    input: ["input"]
  }, WJ, t);
  return {
    ...t,
    // forward classes to the InputBase
    ...i
  };
}, KJ = Oe(Up, {
  shouldForwardProp: (e) => ur(e) || e === "classes",
  name: "MuiInput",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [...zp(e, t), !n.disableUnderline && t.underline];
  }
})(ct(({
  theme: e
}) => {
  let n = e.palette.mode === "light" ? "rgba(0, 0, 0, 0.42)" : "rgba(255, 255, 255, 0.7)";
  return e.vars && (n = `rgba(${e.vars.palette.common.onBackgroundChannel} / ${e.vars.opacity.inputUnderline})`), {
    position: "relative",
    variants: [{
      props: ({
        ownerState: o
      }) => o.formControl,
      style: {
        "label + &": {
          marginTop: 16
        }
      }
    }, {
      props: ({
        ownerState: o
      }) => !o.disableUnderline,
      style: {
        "&::after": {
          left: 0,
          bottom: 0,
          content: '""',
          position: "absolute",
          right: 0,
          transform: "scaleX(0)",
          transition: e.transitions.create("transform", {
            duration: e.transitions.duration.shorter,
            easing: e.transitions.easing.easeOut
          }),
          pointerEvents: "none"
          // Transparent to the hover style.
        },
        [`&.${ku.focused}:after`]: {
          // translateX(0) is a workaround for Safari transform scale bug
          // See https://github.com/mui/material-ui/issues/31766
          transform: "scaleX(1) translateX(0)"
        },
        [`&.${ku.error}`]: {
          "&::before, &::after": {
            borderBottomColor: (e.vars || e).palette.error.main
          }
        },
        "&::before": {
          borderBottom: `1px solid ${n}`,
          left: 0,
          bottom: 0,
          content: '"\\00a0"',
          position: "absolute",
          right: 0,
          transition: e.transitions.create("border-bottom-color", {
            duration: e.transitions.duration.shorter
          }),
          pointerEvents: "none"
          // Transparent to the hover style.
        },
        [`&:hover:not(.${ku.disabled}, .${ku.error}):before`]: {
          borderBottom: `2px solid ${(e.vars || e).palette.text.primary}`,
          // Reset on touch devices, it doesn't add specificity
          "@media (hover: none)": {
            borderBottom: `1px solid ${n}`
          }
        },
        [`&.${ku.disabled}:before`]: {
          borderBottomStyle: "dotted"
        }
      }
    }, ...Object.entries(e.palette).filter(kn()).map(([o]) => ({
      props: {
        color: o,
        disableUnderline: !1
      },
      style: {
        "&::after": {
          borderBottom: `2px solid ${(e.vars || e).palette[o].main}`
        }
      }
    }))]
  };
})), HJ = Oe(Vp, {
  name: "MuiInput",
  slot: "Input",
  overridesResolver: Bp
})({}), C0 = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiInput"
  }), {
    disableUnderline: i = !1,
    components: s = {},
    componentsProps: l,
    fullWidth: u = !1,
    inputComponent: c = "input",
    multiline: f = !1,
    slotProps: p,
    slots: g = {},
    type: h = "text",
    ...y
  } = o, v = qJ(o), $ = {
    root: {
      ownerState: {
        disableUnderline: i
      }
    }
  }, S = p ?? l ? lr(p ?? l, $) : $, w = g.root ?? s.Root ?? KJ, _ = g.input ?? s.Input ?? HJ;
  return /* @__PURE__ */ A.jsx($0, {
    slots: {
      root: w,
      input: _
    },
    slotProps: S,
    fullWidth: u,
    inputComponent: c,
    multiline: f,
    ref: n,
    type: h,
    ...y,
    classes: v
  });
});
C0.muiName = "Input";
function GJ(e) {
  return ht("MuiFilledInput", e);
}
const Aa = {
  ...dl,
  ...dt("MuiFilledInput", ["root", "underline", "input", "adornedStart", "adornedEnd", "sizeSmall", "multiline", "hiddenLabel"])
}, JJ = (e) => {
  const {
    classes: t,
    disableUnderline: n,
    startAdornment: o,
    endAdornment: i,
    size: s,
    hiddenLabel: l,
    multiline: u
  } = e, c = {
    root: ["root", !n && "underline", o && "adornedStart", i && "adornedEnd", s === "small" && `size${Ne(s)}`, l && "hiddenLabel", u && "multiline"],
    input: ["input"]
  }, f = mt(c, GJ, t);
  return {
    ...t,
    // forward classes to the InputBase
    ...f
  };
}, YJ = Oe(Up, {
  shouldForwardProp: (e) => ur(e) || e === "classes",
  name: "MuiFilledInput",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [...zp(e, t), !n.disableUnderline && t.underline];
  }
})(ct(({
  theme: e
}) => {
  const t = e.palette.mode === "light", n = t ? "rgba(0, 0, 0, 0.42)" : "rgba(255, 255, 255, 0.7)", o = t ? "rgba(0, 0, 0, 0.06)" : "rgba(255, 255, 255, 0.09)", i = t ? "rgba(0, 0, 0, 0.09)" : "rgba(255, 255, 255, 0.13)", s = t ? "rgba(0, 0, 0, 0.12)" : "rgba(255, 255, 255, 0.12)";
  return {
    position: "relative",
    backgroundColor: e.vars ? e.vars.palette.FilledInput.bg : o,
    borderTopLeftRadius: (e.vars || e).shape.borderRadius,
    borderTopRightRadius: (e.vars || e).shape.borderRadius,
    transition: e.transitions.create("background-color", {
      duration: e.transitions.duration.shorter,
      easing: e.transitions.easing.easeOut
    }),
    "&:hover": {
      backgroundColor: e.vars ? e.vars.palette.FilledInput.hoverBg : i,
      // Reset on touch devices, it doesn't add specificity
      "@media (hover: none)": {
        backgroundColor: e.vars ? e.vars.palette.FilledInput.bg : o
      }
    },
    [`&.${Aa.focused}`]: {
      backgroundColor: e.vars ? e.vars.palette.FilledInput.bg : o
    },
    [`&.${Aa.disabled}`]: {
      backgroundColor: e.vars ? e.vars.palette.FilledInput.disabledBg : s
    },
    variants: [{
      props: ({
        ownerState: l
      }) => !l.disableUnderline,
      style: {
        "&::after": {
          left: 0,
          bottom: 0,
          content: '""',
          position: "absolute",
          right: 0,
          transform: "scaleX(0)",
          transition: e.transitions.create("transform", {
            duration: e.transitions.duration.shorter,
            easing: e.transitions.easing.easeOut
          }),
          pointerEvents: "none"
          // Transparent to the hover style.
        },
        [`&.${Aa.focused}:after`]: {
          // translateX(0) is a workaround for Safari transform scale bug
          // See https://github.com/mui/material-ui/issues/31766
          transform: "scaleX(1) translateX(0)"
        },
        [`&.${Aa.error}`]: {
          "&::before, &::after": {
            borderBottomColor: (e.vars || e).palette.error.main
          }
        },
        "&::before": {
          borderBottom: `1px solid ${e.vars ? `rgba(${e.vars.palette.common.onBackgroundChannel} / ${e.vars.opacity.inputUnderline})` : n}`,
          left: 0,
          bottom: 0,
          content: '"\\00a0"',
          position: "absolute",
          right: 0,
          transition: e.transitions.create("border-bottom-color", {
            duration: e.transitions.duration.shorter
          }),
          pointerEvents: "none"
          // Transparent to the hover style.
        },
        [`&:hover:not(.${Aa.disabled}, .${Aa.error}):before`]: {
          borderBottom: `1px solid ${(e.vars || e).palette.text.primary}`
        },
        [`&.${Aa.disabled}:before`]: {
          borderBottomStyle: "dotted"
        }
      }
    }, ...Object.entries(e.palette).filter(kn()).map(([l]) => {
      var u;
      return {
        props: {
          disableUnderline: !1,
          color: l
        },
        style: {
          "&::after": {
            borderBottom: `2px solid ${(u = (e.vars || e).palette[l]) == null ? void 0 : u.main}`
          }
        }
      };
    }), {
      props: ({
        ownerState: l
      }) => l.startAdornment,
      style: {
        paddingLeft: 12
      }
    }, {
      props: ({
        ownerState: l
      }) => l.endAdornment,
      style: {
        paddingRight: 12
      }
    }, {
      props: ({
        ownerState: l
      }) => l.multiline,
      style: {
        padding: "25px 12px 8px"
      }
    }, {
      props: ({
        ownerState: l,
        size: u
      }) => l.multiline && u === "small",
      style: {
        paddingTop: 21,
        paddingBottom: 4
      }
    }, {
      props: ({
        ownerState: l
      }) => l.multiline && l.hiddenLabel,
      style: {
        paddingTop: 16,
        paddingBottom: 17
      }
    }, {
      props: ({
        ownerState: l
      }) => l.multiline && l.hiddenLabel && l.size === "small",
      style: {
        paddingTop: 8,
        paddingBottom: 9
      }
    }]
  };
})), QJ = Oe(Vp, {
  name: "MuiFilledInput",
  slot: "Input",
  overridesResolver: Bp
})(ct(({
  theme: e
}) => ({
  paddingTop: 25,
  paddingRight: 12,
  paddingBottom: 8,
  paddingLeft: 12,
  ...!e.vars && {
    "&:-webkit-autofill": {
      WebkitBoxShadow: e.palette.mode === "light" ? null : "0 0 0 100px #266798 inset",
      WebkitTextFillColor: e.palette.mode === "light" ? null : "#fff",
      caretColor: e.palette.mode === "light" ? null : "#fff",
      borderTopLeftRadius: "inherit",
      borderTopRightRadius: "inherit"
    }
  },
  ...e.vars && {
    "&:-webkit-autofill": {
      borderTopLeftRadius: "inherit",
      borderTopRightRadius: "inherit"
    },
    [e.getColorSchemeSelector("dark")]: {
      "&:-webkit-autofill": {
        WebkitBoxShadow: "0 0 0 100px #266798 inset",
        WebkitTextFillColor: "#fff",
        caretColor: "#fff"
      }
    }
  },
  variants: [{
    props: {
      size: "small"
    },
    style: {
      paddingTop: 21,
      paddingBottom: 4
    }
  }, {
    props: ({
      ownerState: t
    }) => t.hiddenLabel,
    style: {
      paddingTop: 16,
      paddingBottom: 17
    }
  }, {
    props: ({
      ownerState: t
    }) => t.startAdornment,
    style: {
      paddingLeft: 0
    }
  }, {
    props: ({
      ownerState: t
    }) => t.endAdornment,
    style: {
      paddingRight: 0
    }
  }, {
    props: ({
      ownerState: t
    }) => t.hiddenLabel && t.size === "small",
    style: {
      paddingTop: 8,
      paddingBottom: 9
    }
  }, {
    props: ({
      ownerState: t
    }) => t.multiline,
    style: {
      paddingTop: 0,
      paddingBottom: 0,
      paddingLeft: 0,
      paddingRight: 0
    }
  }]
}))), x0 = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiFilledInput"
  }), {
    disableUnderline: i = !1,
    components: s = {},
    componentsProps: l,
    fullWidth: u = !1,
    hiddenLabel: c,
    // declare here to prevent spreading to DOM
    inputComponent: f = "input",
    multiline: p = !1,
    slotProps: g,
    slots: h = {},
    type: y = "text",
    ...v
  } = o, C = {
    ...o,
    disableUnderline: i,
    fullWidth: u,
    inputComponent: f,
    multiline: p,
    type: y
  }, $ = JJ(o), S = {
    root: {
      ownerState: C
    },
    input: {
      ownerState: C
    }
  }, w = g ?? l ? lr(S, g ?? l) : S, _ = h.root ?? s.Root ?? YJ, b = h.input ?? s.Input ?? QJ;
  return /* @__PURE__ */ A.jsx($0, {
    slots: {
      root: _,
      input: b
    },
    slotProps: w,
    fullWidth: u,
    inputComponent: f,
    multiline: p,
    ref: n,
    type: y,
    ...v,
    classes: $
  });
});
x0.muiName = "Input";
var ow;
const XJ = Oe("fieldset", {
  shouldForwardProp: ur
})({
  textAlign: "left",
  position: "absolute",
  bottom: 0,
  right: 0,
  top: -5,
  left: 0,
  margin: 0,
  padding: "0 8px",
  pointerEvents: "none",
  borderRadius: "inherit",
  borderStyle: "solid",
  borderWidth: 1,
  overflow: "hidden",
  minWidth: "0%"
}), ZJ = Oe("legend", {
  shouldForwardProp: ur
})(ct(({
  theme: e
}) => ({
  float: "unset",
  // Fix conflict with bootstrap
  width: "auto",
  // Fix conflict with bootstrap
  overflow: "hidden",
  // Fix Horizontal scroll when label too long
  variants: [{
    props: ({
      ownerState: t
    }) => !t.withLabel,
    style: {
      padding: 0,
      lineHeight: "11px",
      // sync with `height` in `legend` styles
      transition: e.transitions.create("width", {
        duration: 150,
        easing: e.transitions.easing.easeOut
      })
    }
  }, {
    props: ({
      ownerState: t
    }) => t.withLabel,
    style: {
      display: "block",
      // Fix conflict with normalize.css and sanitize.css
      padding: 0,
      height: 11,
      // sync with `lineHeight` in `legend` styles
      fontSize: "0.75em",
      visibility: "hidden",
      maxWidth: 0.01,
      transition: e.transitions.create("max-width", {
        duration: 50,
        easing: e.transitions.easing.easeOut
      }),
      whiteSpace: "nowrap",
      "& > span": {
        paddingLeft: 5,
        paddingRight: 5,
        display: "inline-block",
        opacity: 0,
        visibility: "visible"
      }
    }
  }, {
    props: ({
      ownerState: t
    }) => t.withLabel && t.notched,
    style: {
      maxWidth: "100%",
      transition: e.transitions.create("max-width", {
        duration: 100,
        easing: e.transitions.easing.easeOut,
        delay: 50
      })
    }
  }]
})));
function eY(e) {
  const {
    children: t,
    classes: n,
    className: o,
    label: i,
    notched: s,
    ...l
  } = e, u = i != null && i !== "", c = {
    ...e,
    notched: s,
    withLabel: u
  };
  return /* @__PURE__ */ A.jsx(XJ, {
    "aria-hidden": !0,
    className: o,
    ownerState: c,
    ...l,
    children: /* @__PURE__ */ A.jsx(ZJ, {
      ownerState: c,
      children: u ? /* @__PURE__ */ A.jsx("span", {
        children: i
      }) : (
        // notranslate needed while Google Translate will not fix zero-width space issue
        ow || (ow = /* @__PURE__ */ A.jsx("span", {
          className: "notranslate",
          "aria-hidden": !0,
          children: "​"
        }))
      )
    })
  });
}
function tY(e) {
  return ht("MuiOutlinedInput", e);
}
const Bo = {
  ...dl,
  ...dt("MuiOutlinedInput", ["root", "notchedOutline", "input"])
}, nY = (e) => {
  const {
    classes: t
  } = e, o = mt({
    root: ["root"],
    notchedOutline: ["notchedOutline"],
    input: ["input"]
  }, tY, t);
  return {
    ...t,
    // forward classes to the InputBase
    ...o
  };
}, rY = Oe(Up, {
  shouldForwardProp: (e) => ur(e) || e === "classes",
  name: "MuiOutlinedInput",
  slot: "Root",
  overridesResolver: zp
})(ct(({
  theme: e
}) => {
  const t = e.palette.mode === "light" ? "rgba(0, 0, 0, 0.23)" : "rgba(255, 255, 255, 0.23)";
  return {
    position: "relative",
    borderRadius: (e.vars || e).shape.borderRadius,
    [`&:hover .${Bo.notchedOutline}`]: {
      borderColor: (e.vars || e).palette.text.primary
    },
    // Reset on touch devices, it doesn't add specificity
    "@media (hover: none)": {
      [`&:hover .${Bo.notchedOutline}`]: {
        borderColor: e.vars ? `rgba(${e.vars.palette.common.onBackgroundChannel} / 0.23)` : t
      }
    },
    [`&.${Bo.focused} .${Bo.notchedOutline}`]: {
      borderWidth: 2
    },
    variants: [...Object.entries(e.palette).filter(kn()).map(([n]) => ({
      props: {
        color: n
      },
      style: {
        [`&.${Bo.focused} .${Bo.notchedOutline}`]: {
          borderColor: (e.vars || e).palette[n].main
        }
      }
    })), {
      props: {},
      // to overide the above style
      style: {
        [`&.${Bo.error} .${Bo.notchedOutline}`]: {
          borderColor: (e.vars || e).palette.error.main
        },
        [`&.${Bo.disabled} .${Bo.notchedOutline}`]: {
          borderColor: (e.vars || e).palette.action.disabled
        }
      }
    }, {
      props: ({
        ownerState: n
      }) => n.startAdornment,
      style: {
        paddingLeft: 14
      }
    }, {
      props: ({
        ownerState: n
      }) => n.endAdornment,
      style: {
        paddingRight: 14
      }
    }, {
      props: ({
        ownerState: n
      }) => n.multiline,
      style: {
        padding: "16.5px 14px"
      }
    }, {
      props: ({
        ownerState: n,
        size: o
      }) => n.multiline && o === "small",
      style: {
        padding: "8.5px 14px"
      }
    }]
  };
})), oY = Oe(eY, {
  name: "MuiOutlinedInput",
  slot: "NotchedOutline",
  overridesResolver: (e, t) => t.notchedOutline
})(ct(({
  theme: e
}) => {
  const t = e.palette.mode === "light" ? "rgba(0, 0, 0, 0.23)" : "rgba(255, 255, 255, 0.23)";
  return {
    borderColor: e.vars ? `rgba(${e.vars.palette.common.onBackgroundChannel} / 0.23)` : t
  };
})), iY = Oe(Vp, {
  name: "MuiOutlinedInput",
  slot: "Input",
  overridesResolver: Bp
})(ct(({
  theme: e
}) => ({
  padding: "16.5px 14px",
  ...!e.vars && {
    "&:-webkit-autofill": {
      WebkitBoxShadow: e.palette.mode === "light" ? null : "0 0 0 100px #266798 inset",
      WebkitTextFillColor: e.palette.mode === "light" ? null : "#fff",
      caretColor: e.palette.mode === "light" ? null : "#fff",
      borderRadius: "inherit"
    }
  },
  ...e.vars && {
    "&:-webkit-autofill": {
      borderRadius: "inherit"
    },
    [e.getColorSchemeSelector("dark")]: {
      "&:-webkit-autofill": {
        WebkitBoxShadow: "0 0 0 100px #266798 inset",
        WebkitTextFillColor: "#fff",
        caretColor: "#fff"
      }
    }
  },
  variants: [{
    props: {
      size: "small"
    },
    style: {
      padding: "8.5px 14px"
    }
  }, {
    props: ({
      ownerState: t
    }) => t.multiline,
    style: {
      padding: 0
    }
  }, {
    props: ({
      ownerState: t
    }) => t.startAdornment,
    style: {
      paddingLeft: 0
    }
  }, {
    props: ({
      ownerState: t
    }) => t.endAdornment,
    style: {
      paddingRight: 0
    }
  }]
}))), E0 = /* @__PURE__ */ z.forwardRef(function(t, n) {
  var o;
  const i = bt({
    props: t,
    name: "MuiOutlinedInput"
  }), {
    components: s = {},
    fullWidth: l = !1,
    inputComponent: u = "input",
    label: c,
    multiline: f = !1,
    notched: p,
    slots: g = {},
    type: h = "text",
    ...y
  } = i, v = nY(i), C = ti(), $ = ca({
    props: i,
    muiFormControl: C,
    states: ["color", "disabled", "error", "focused", "hiddenLabel", "size", "required"]
  }), S = {
    ...i,
    color: $.color || "primary",
    disabled: $.disabled,
    error: $.error,
    focused: $.focused,
    formControl: C,
    fullWidth: l,
    hiddenLabel: $.hiddenLabel,
    multiline: f,
    size: $.size,
    type: h
  }, w = g.root ?? s.Root ?? rY, _ = g.input ?? s.Input ?? iY;
  return /* @__PURE__ */ A.jsx($0, {
    slots: {
      root: w,
      input: _
    },
    renderSuffix: (b) => /* @__PURE__ */ A.jsx(oY, {
      ownerState: S,
      className: v.notchedOutline,
      label: c != null && c !== "" && $.required ? o || (o = /* @__PURE__ */ A.jsxs(z.Fragment, {
        children: [c, " ", "*"]
      })) : c,
      notched: typeof p < "u" ? p : !!(b.startAdornment || b.filled || b.focused)
    }),
    fullWidth: l,
    inputComponent: u,
    multiline: f,
    ref: n,
    type: h,
    ...y,
    classes: {
      ...v,
      notchedOutline: null
    }
  });
});
E0.muiName = "Input";
function aY(e) {
  return ht("MuiFormLabel", e);
}
const Ku = dt("MuiFormLabel", ["root", "colorSecondary", "focused", "disabled", "error", "filled", "required", "asterisk"]), sY = (e) => {
  const {
    classes: t,
    color: n,
    focused: o,
    disabled: i,
    error: s,
    filled: l,
    required: u
  } = e, c = {
    root: ["root", `color${Ne(n)}`, i && "disabled", s && "error", l && "filled", o && "focused", u && "required"],
    asterisk: ["asterisk", s && "error"]
  };
  return mt(c, aY, t);
}, lY = Oe("label", {
  name: "MuiFormLabel",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, n.color === "secondary" && t.colorSecondary, n.filled && t.filled];
  }
})(ct(({
  theme: e
}) => ({
  color: (e.vars || e).palette.text.secondary,
  ...e.typography.body1,
  lineHeight: "1.4375em",
  padding: 0,
  position: "relative",
  variants: [...Object.entries(e.palette).filter(kn()).map(([t]) => ({
    props: {
      color: t
    },
    style: {
      [`&.${Ku.focused}`]: {
        color: (e.vars || e).palette[t].main
      }
    }
  })), {
    props: {},
    style: {
      [`&.${Ku.disabled}`]: {
        color: (e.vars || e).palette.text.disabled
      },
      [`&.${Ku.error}`]: {
        color: (e.vars || e).palette.error.main
      }
    }
  }]
}))), uY = Oe("span", {
  name: "MuiFormLabel",
  slot: "Asterisk",
  overridesResolver: (e, t) => t.asterisk
})(ct(({
  theme: e
}) => ({
  [`&.${Ku.error}`]: {
    color: (e.vars || e).palette.error.main
  }
}))), Wp = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiFormLabel"
  }), {
    children: i,
    className: s,
    color: l,
    component: u = "label",
    disabled: c,
    error: f,
    filled: p,
    focused: g,
    required: h,
    ...y
  } = o, v = ti(), C = ca({
    props: o,
    muiFormControl: v,
    states: ["color", "required", "focused", "disabled", "error", "filled"]
  }), $ = {
    ...o,
    color: C.color || "primary",
    component: u,
    disabled: C.disabled,
    error: C.error,
    filled: C.filled,
    focused: C.focused,
    required: C.required
  }, S = sY($);
  return /* @__PURE__ */ A.jsxs(lY, {
    as: u,
    ownerState: $,
    className: Ve(S.root, s),
    ref: n,
    ...y,
    children: [i, C.required && /* @__PURE__ */ A.jsxs(uY, {
      ownerState: $,
      "aria-hidden": !0,
      className: S.asterisk,
      children: [" ", "*"]
    })]
  });
});
function cY(e) {
  return ht("MuiInputLabel", e);
}
dt("MuiInputLabel", ["root", "focused", "disabled", "error", "required", "asterisk", "formControl", "sizeSmall", "shrink", "animated", "standard", "filled", "outlined"]);
const dY = (e) => {
  const {
    classes: t,
    formControl: n,
    size: o,
    shrink: i,
    disableAnimation: s,
    variant: l,
    required: u
  } = e, c = {
    root: ["root", n && "formControl", !s && "animated", i && "shrink", o && o !== "normal" && `size${Ne(o)}`, l],
    asterisk: [u && "asterisk"]
  }, f = mt(c, cY, t);
  return {
    ...t,
    // forward the focused, disabled, etc. classes to the FormLabel
    ...f
  };
}, fY = Oe(Wp, {
  shouldForwardProp: (e) => ur(e) || e === "classes",
  name: "MuiInputLabel",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [{
      [`& .${Ku.asterisk}`]: t.asterisk
    }, t.root, n.formControl && t.formControl, n.size === "small" && t.sizeSmall, n.shrink && t.shrink, !n.disableAnimation && t.animated, n.focused && t.focused, t[n.variant]];
  }
})(ct(({
  theme: e
}) => ({
  display: "block",
  transformOrigin: "top left",
  whiteSpace: "nowrap",
  overflow: "hidden",
  textOverflow: "ellipsis",
  maxWidth: "100%",
  variants: [{
    props: ({
      ownerState: t
    }) => t.formControl,
    style: {
      position: "absolute",
      left: 0,
      top: 0,
      // slight alteration to spec spacing to match visual spec result
      transform: "translate(0, 20px) scale(1)"
    }
  }, {
    props: {
      size: "small"
    },
    style: {
      // Compensation for the `Input.inputSizeSmall` style.
      transform: "translate(0, 17px) scale(1)"
    }
  }, {
    props: ({
      ownerState: t
    }) => t.shrink,
    style: {
      transform: "translate(0, -1.5px) scale(0.75)",
      transformOrigin: "top left",
      maxWidth: "133%"
    }
  }, {
    props: ({
      ownerState: t
    }) => !t.disableAnimation,
    style: {
      transition: e.transitions.create(["color", "transform", "max-width"], {
        duration: e.transitions.duration.shorter,
        easing: e.transitions.easing.easeOut
      })
    }
  }, {
    props: {
      variant: "filled"
    },
    style: {
      // Chrome's autofill feature gives the input field a yellow background.
      // Since the input field is behind the label in the HTML tree,
      // the input field is drawn last and hides the label with an opaque background color.
      // zIndex: 1 will raise the label above opaque background-colors of input.
      zIndex: 1,
      pointerEvents: "none",
      transform: "translate(12px, 16px) scale(1)",
      maxWidth: "calc(100% - 24px)"
    }
  }, {
    props: {
      variant: "filled",
      size: "small"
    },
    style: {
      transform: "translate(12px, 13px) scale(1)"
    }
  }, {
    props: ({
      variant: t,
      ownerState: n
    }) => t === "filled" && n.shrink,
    style: {
      userSelect: "none",
      pointerEvents: "auto",
      transform: "translate(12px, 7px) scale(0.75)",
      maxWidth: "calc(133% - 24px)"
    }
  }, {
    props: ({
      variant: t,
      ownerState: n,
      size: o
    }) => t === "filled" && n.shrink && o === "small",
    style: {
      transform: "translate(12px, 4px) scale(0.75)"
    }
  }, {
    props: {
      variant: "outlined"
    },
    style: {
      // see comment above on filled.zIndex
      zIndex: 1,
      pointerEvents: "none",
      transform: "translate(14px, 16px) scale(1)",
      maxWidth: "calc(100% - 24px)"
    }
  }, {
    props: {
      variant: "outlined",
      size: "small"
    },
    style: {
      transform: "translate(14px, 9px) scale(1)"
    }
  }, {
    props: ({
      variant: t,
      ownerState: n
    }) => t === "outlined" && n.shrink,
    style: {
      userSelect: "none",
      pointerEvents: "auto",
      // Theoretically, we should have (8+5)*2/0.75 = 34px
      // but it feels a better when it bleeds a bit on the left, so 32px.
      maxWidth: "calc(133% - 32px)",
      transform: "translate(14px, -9px) scale(0.75)"
    }
  }]
}))), pY = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    name: "MuiInputLabel",
    props: t
  }), {
    disableAnimation: i = !1,
    margin: s,
    shrink: l,
    variant: u,
    className: c,
    ...f
  } = o, p = ti();
  let g = l;
  typeof g > "u" && p && (g = p.filled || p.focused || p.adornedStart);
  const h = ca({
    props: o,
    muiFormControl: p,
    states: ["size", "variant", "required", "focused"]
  }), y = {
    ...o,
    disableAnimation: i,
    formControl: p,
    shrink: g,
    size: h.size,
    variant: h.variant,
    required: h.required,
    focused: h.focused
  }, v = dY(y);
  return /* @__PURE__ */ A.jsx(fY, {
    "data-shrink": g,
    ref: n,
    className: Ve(v.root, c),
    ...f,
    ownerState: y,
    classes: v
  });
});
function hY(e) {
  return ht("MuiFormControl", e);
}
dt("MuiFormControl", ["root", "marginNone", "marginNormal", "marginDense", "fullWidth", "disabled"]);
const mY = (e) => {
  const {
    classes: t,
    margin: n,
    fullWidth: o
  } = e, i = {
    root: ["root", n !== "none" && `margin${Ne(n)}`, o && "fullWidth"]
  };
  return mt(i, hY, t);
}, gY = Oe("div", {
  name: "MuiFormControl",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, t[`margin${Ne(n.margin)}`], n.fullWidth && t.fullWidth];
  }
})({
  display: "inline-flex",
  flexDirection: "column",
  position: "relative",
  // Reset fieldset default style.
  minWidth: 0,
  padding: 0,
  margin: 0,
  border: 0,
  verticalAlign: "top",
  // Fix alignment issue on Safari.
  variants: [{
    props: {
      margin: "normal"
    },
    style: {
      marginTop: 16,
      marginBottom: 8
    }
  }, {
    props: {
      margin: "dense"
    },
    style: {
      marginTop: 8,
      marginBottom: 4
    }
  }, {
    props: {
      fullWidth: !0
    },
    style: {
      width: "100%"
    }
  }]
}), xE = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiFormControl"
  }), {
    children: i,
    className: s,
    color: l = "primary",
    component: u = "div",
    disabled: c = !1,
    error: f = !1,
    focused: p,
    fullWidth: g = !1,
    hiddenLabel: h = !1,
    margin: y = "none",
    required: v = !1,
    size: C = "medium",
    variant: $ = "outlined",
    ...S
  } = o, w = {
    ...o,
    color: l,
    component: u,
    disabled: c,
    error: f,
    fullWidth: g,
    hiddenLabel: h,
    margin: y,
    required: v,
    size: C,
    variant: $
  }, _ = mY(w), [b, k] = z.useState(() => {
    let q = !1;
    return i && z.Children.forEach(i, (Q) => {
      if (!Vu(Q, ["Input", "Select"]))
        return;
      const X = Vu(Q, ["Select"]) ? Q.props.input : Q;
      X && BJ(X.props) && (q = !0);
    }), q;
  }), [E, j] = z.useState(() => {
    let q = !1;
    return i && z.Children.forEach(i, (Q) => {
      Vu(Q, ["Input", "Select"]) && (qf(Q.props, !0) || qf(Q.props.inputProps, !0)) && (q = !0);
    }), q;
  }), [L, B] = z.useState(!1);
  c && L && B(!1);
  const O = p !== void 0 && !c ? p : L;
  let V;
  z.useRef(!1);
  const W = z.useMemo(() => ({
    adornedStart: b,
    setAdornedStart: k,
    color: l,
    disabled: c,
    error: f,
    filled: E,
    focused: O,
    fullWidth: g,
    hiddenLabel: h,
    size: C,
    onBlur: () => {
      B(!1);
    },
    onEmpty: () => {
      j(!1);
    },
    onFilled: () => {
      j(!0);
    },
    onFocus: () => {
      B(!0);
    },
    registerEffect: V,
    required: v,
    variant: $
  }), [b, l, c, f, E, O, g, h, V, v, C, $]);
  return /* @__PURE__ */ A.jsx(w0.Provider, {
    value: W,
    children: /* @__PURE__ */ A.jsx(gY, {
      as: u,
      ownerState: w,
      className: Ve(_.root, s),
      ref: n,
      ...S,
      children: i
    })
  });
});
function yY(e) {
  return ht("MuiFormHelperText", e);
}
const iw = dt("MuiFormHelperText", ["root", "error", "disabled", "sizeSmall", "sizeMedium", "contained", "focused", "filled", "required"]);
var aw;
const vY = (e) => {
  const {
    classes: t,
    contained: n,
    size: o,
    disabled: i,
    error: s,
    filled: l,
    focused: u,
    required: c
  } = e, f = {
    root: ["root", i && "disabled", s && "error", o && `size${Ne(o)}`, n && "contained", u && "focused", l && "filled", c && "required"]
  };
  return mt(f, yY, t);
}, bY = Oe("p", {
  name: "MuiFormHelperText",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, n.size && t[`size${Ne(n.size)}`], n.contained && t.contained, n.filled && t.filled];
  }
})(ct(({
  theme: e
}) => ({
  color: (e.vars || e).palette.text.secondary,
  ...e.typography.caption,
  textAlign: "left",
  marginTop: 3,
  marginRight: 0,
  marginBottom: 0,
  marginLeft: 0,
  [`&.${iw.disabled}`]: {
    color: (e.vars || e).palette.text.disabled
  },
  [`&.${iw.error}`]: {
    color: (e.vars || e).palette.error.main
  },
  variants: [{
    props: {
      size: "small"
    },
    style: {
      marginTop: 4
    }
  }, {
    props: ({
      ownerState: t
    }) => t.contained,
    style: {
      marginLeft: 14,
      marginRight: 14
    }
  }]
}))), _0 = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiFormHelperText"
  }), {
    children: i,
    className: s,
    component: l = "p",
    disabled: u,
    error: c,
    filled: f,
    focused: p,
    margin: g,
    required: h,
    variant: y,
    ...v
  } = o, C = ti(), $ = ca({
    props: o,
    muiFormControl: C,
    states: ["variant", "size", "disabled", "error", "filled", "focused", "required"]
  }), S = {
    ...o,
    component: l,
    contained: $.variant === "filled" || $.variant === "outlined",
    variant: $.variant,
    size: $.size,
    disabled: $.disabled,
    error: $.error,
    filled: $.filled,
    focused: $.focused,
    required: $.required
  };
  delete S.ownerState;
  const w = vY(S);
  return /* @__PURE__ */ A.jsx(bY, {
    as: l,
    className: Ve(w.root, s),
    ref: n,
    ...v,
    ownerState: S,
    children: i === " " ? (
      // notranslate needed while Google Translate will not fix zero-width space issue
      aw || (aw = /* @__PURE__ */ A.jsx("span", {
        className: "notranslate",
        "aria-hidden": !0,
        children: "​"
      }))
    ) : i
  });
}), bi = /* @__PURE__ */ z.createContext({});
function SY(e) {
  return ht("MuiList", e);
}
dt("MuiList", ["root", "padding", "dense", "subheader"]);
const wY = (e) => {
  const {
    classes: t,
    disablePadding: n,
    dense: o,
    subheader: i
  } = e;
  return mt({
    root: ["root", !n && "padding", o && "dense", i && "subheader"]
  }, SY, t);
}, $Y = Oe("ul", {
  name: "MuiList",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, !n.disablePadding && t.padding, n.dense && t.dense, n.subheader && t.subheader];
  }
})({
  listStyle: "none",
  margin: 0,
  padding: 0,
  position: "relative",
  variants: [{
    props: ({
      ownerState: e
    }) => !e.disablePadding,
    style: {
      paddingTop: 8,
      paddingBottom: 8
    }
  }, {
    props: ({
      ownerState: e
    }) => e.subheader,
    style: {
      paddingTop: 0
    }
  }]
}), k0 = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiList"
  }), {
    children: i,
    className: s,
    component: l = "ul",
    dense: u = !1,
    disablePadding: c = !1,
    subheader: f,
    ...p
  } = o, g = z.useMemo(() => ({
    dense: u
  }), [u]), h = {
    ...o,
    component: l,
    dense: u,
    disablePadding: c
  }, y = wY(h);
  return /* @__PURE__ */ A.jsx(bi.Provider, {
    value: g,
    children: /* @__PURE__ */ A.jsxs($Y, {
      as: l,
      className: Ve(y.root, s),
      ref: n,
      ownerState: h,
      ...p,
      children: [f, i]
    })
  });
});
function Zm(e, t, n) {
  return e === t ? e.firstChild : t && t.nextElementSibling ? t.nextElementSibling : n ? null : e.firstChild;
}
function sw(e, t, n) {
  return e === t ? n ? e.firstChild : e.lastChild : t && t.previousElementSibling ? t.previousElementSibling : n ? null : e.lastChild;
}
function EE(e, t) {
  if (t === void 0)
    return !0;
  let n = e.innerText;
  return n === void 0 && (n = e.textContent), n = n.trim().toLowerCase(), n.length === 0 ? !1 : t.repeating ? n[0] === t.keys[0] : n.startsWith(t.keys.join(""));
}
function Pu(e, t, n, o, i, s) {
  let l = !1, u = i(e, t, t ? n : !1);
  for (; u; ) {
    if (u === e.firstChild) {
      if (l)
        return !1;
      l = !0;
    }
    const c = o ? !1 : u.disabled || u.getAttribute("aria-disabled") === "true";
    if (!u.hasAttribute("tabindex") || !EE(u, s) || c)
      u = i(e, u, n);
    else
      return u.focus(), !0;
  }
  return !1;
}
const CY = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const {
    // private
    // eslint-disable-next-line react/prop-types
    actions: o,
    autoFocus: i = !1,
    autoFocusItem: s = !1,
    children: l,
    className: u,
    disabledItemsFocusable: c = !1,
    disableListWrap: f = !1,
    onKeyDown: p,
    variant: g = "selectedMenu",
    ...h
  } = t, y = z.useRef(null), v = z.useRef({
    keys: [],
    repeating: !0,
    previousKeyMatched: !0,
    lastTime: null
  });
  Ei(() => {
    i && y.current.focus();
  }, [i]), z.useImperativeHandle(o, () => ({
    adjustStyleForScrollbar: (_, {
      direction: b
    }) => {
      const k = !y.current.style.width;
      if (_.clientHeight < y.current.clientHeight && k) {
        const E = `${Dx(xi(_))}px`;
        y.current.style[b === "rtl" ? "paddingLeft" : "paddingRight"] = E, y.current.style.width = `calc(100% + ${E})`;
      }
      return y.current;
    }
  }), []);
  const C = (_) => {
    const b = y.current, k = _.key;
    if (_.ctrlKey || _.metaKey || _.altKey) {
      p && p(_);
      return;
    }
    const j = $r(b).activeElement;
    if (k === "ArrowDown")
      _.preventDefault(), Pu(b, j, f, c, Zm);
    else if (k === "ArrowUp")
      _.preventDefault(), Pu(b, j, f, c, sw);
    else if (k === "Home")
      _.preventDefault(), Pu(b, null, f, c, Zm);
    else if (k === "End")
      _.preventDefault(), Pu(b, null, f, c, sw);
    else if (k.length === 1) {
      const L = v.current, B = k.toLowerCase(), O = performance.now();
      L.keys.length > 0 && (O - L.lastTime > 500 ? (L.keys = [], L.repeating = !0, L.previousKeyMatched = !0) : L.repeating && B !== L.keys[0] && (L.repeating = !1)), L.lastTime = O, L.keys.push(B);
      const V = j && !L.repeating && EE(j, L);
      L.previousKeyMatched && (V || Pu(b, j, !1, c, Zm, L)) ? _.preventDefault() : L.previousKeyMatched = !1;
    }
    p && p(_);
  }, $ = _n(y, n);
  let S = -1;
  z.Children.forEach(l, (_, b) => {
    if (!/* @__PURE__ */ z.isValidElement(_)) {
      S === b && (S += 1, S >= l.length && (S = -1));
      return;
    }
    _.props.disabled || (g === "selectedMenu" && _.props.selected || S === -1) && (S = b), S === b && (_.props.disabled || _.props.muiSkipListHighlight || _.type.muiSkipListHighlight) && (S += 1, S >= l.length && (S = -1));
  });
  const w = z.Children.map(l, (_, b) => {
    if (b === S) {
      const k = {};
      return s && (k.autoFocus = !0), _.props.tabIndex === void 0 && g === "selectedMenu" && (k.tabIndex = 0), /* @__PURE__ */ z.cloneElement(_, k);
    }
    return _;
  });
  return /* @__PURE__ */ A.jsx(k0, {
    role: "menu",
    ref: $,
    className: u,
    onKeyDown: C,
    tabIndex: i ? 0 : -1,
    ...h,
    children: w
  });
}), _E = (e) => e.scrollTop;
function fl(e, t) {
  const {
    timeout: n,
    easing: o,
    style: i = {}
  } = e;
  return {
    duration: i.transitionDuration ?? (typeof n == "number" ? n : n[t.mode] || 0),
    easing: i.transitionTimingFunction ?? (typeof o == "object" ? o[t.mode] : o),
    delay: i.transitionDelay
  };
}
function Lg(e) {
  return `scale(${e}, ${e ** 2})`;
}
const xY = {
  entering: {
    opacity: 1,
    transform: Lg(1)
  },
  entered: {
    opacity: 1,
    transform: "none"
  }
}, eg = typeof navigator < "u" && /^((?!chrome|android).)*(safari|mobile)/i.test(navigator.userAgent) && /(os |version\/)15(.|_)4/i.test(navigator.userAgent), Dg = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const {
    addEndListener: o,
    appear: i = !0,
    children: s,
    easing: l,
    in: u,
    onEnter: c,
    onEntered: f,
    onEntering: p,
    onExit: g,
    onExited: h,
    onExiting: y,
    style: v,
    timeout: C = "auto",
    // eslint-disable-next-line react/prop-types
    TransitionComponent: $ = jo,
    ...S
  } = t, w = r0(), _ = z.useRef(), b = Tl(), k = z.useRef(null), E = _n(k, kc(s), n), j = (X) => (ce) => {
    if (X) {
      const J = k.current;
      ce === void 0 ? X(J) : X(J, ce);
    }
  }, L = j(p), B = j((X, ce) => {
    _E(X);
    const {
      duration: J,
      delay: ie,
      easing: re
    } = fl({
      style: v,
      timeout: C,
      easing: l
    }, {
      mode: "enter"
    });
    let Y;
    C === "auto" ? (Y = b.transitions.getAutoHeightDuration(X.clientHeight), _.current = Y) : Y = J, X.style.transition = [b.transitions.create("opacity", {
      duration: Y,
      delay: ie
    }), b.transitions.create("transform", {
      duration: eg ? Y : Y * 0.666,
      delay: ie,
      easing: re
    })].join(","), c && c(X, ce);
  }), O = j(f), V = j(y), W = j((X) => {
    const {
      duration: ce,
      delay: J,
      easing: ie
    } = fl({
      style: v,
      timeout: C,
      easing: l
    }, {
      mode: "exit"
    });
    let re;
    C === "auto" ? (re = b.transitions.getAutoHeightDuration(X.clientHeight), _.current = re) : re = ce, X.style.transition = [b.transitions.create("opacity", {
      duration: re,
      delay: J
    }), b.transitions.create("transform", {
      duration: eg ? re : re * 0.666,
      delay: eg ? J : J || re * 0.333,
      easing: ie
    })].join(","), X.style.opacity = 0, X.style.transform = Lg(0.75), g && g(X);
  }), q = j(h), Q = (X) => {
    C === "auto" && w.start(_.current || 0, X), o && o(k.current, X);
  };
  return /* @__PURE__ */ A.jsx($, {
    appear: i,
    in: u,
    nodeRef: k,
    onEnter: B,
    onEntered: O,
    onEntering: L,
    onExit: W,
    onExited: q,
    onExiting: V,
    addEndListener: Q,
    timeout: C === "auto" ? null : C,
    ...S,
    children: (X, {
      ownerState: ce,
      ...J
    }) => /* @__PURE__ */ z.cloneElement(s, {
      style: {
        opacity: 0,
        transform: Lg(0.75),
        visibility: X === "exited" && !u ? "hidden" : void 0,
        ...xY[X],
        ...v,
        ...s.props.style
      },
      ref: E,
      ...J
    })
  });
});
Dg && (Dg.muiSupportAuto = !0);
function EY(e) {
  const t = $r(e);
  return t.body === e ? xi(e).innerWidth > t.documentElement.clientWidth : e.scrollHeight > e.clientHeight;
}
function Hu(e, t) {
  t ? e.setAttribute("aria-hidden", "true") : e.removeAttribute("aria-hidden");
}
function lw(e) {
  return parseInt(xi(e).getComputedStyle(e).paddingRight, 10) || 0;
}
function _Y(e) {
  const n = ["TEMPLATE", "SCRIPT", "STYLE", "LINK", "MAP", "META", "NOSCRIPT", "PICTURE", "COL", "COLGROUP", "PARAM", "SLOT", "SOURCE", "TRACK"].includes(e.tagName), o = e.tagName === "INPUT" && e.getAttribute("type") === "hidden";
  return n || o;
}
function uw(e, t, n, o, i) {
  const s = [t, n, ...o];
  [].forEach.call(e.children, (l) => {
    const u = !s.includes(l), c = !_Y(l);
    u && c && Hu(l, i);
  });
}
function tg(e, t) {
  let n = -1;
  return e.some((o, i) => t(o) ? (n = i, !0) : !1), n;
}
function kY(e, t) {
  const n = [], o = e.container;
  if (!t.disableScrollLock) {
    if (EY(o)) {
      const l = Dx(xi(o));
      n.push({
        value: o.style.paddingRight,
        property: "padding-right",
        el: o
      }), o.style.paddingRight = `${lw(o) + l}px`;
      const u = $r(o).querySelectorAll(".mui-fixed");
      [].forEach.call(u, (c) => {
        n.push({
          value: c.style.paddingRight,
          property: "padding-right",
          el: c
        }), c.style.paddingRight = `${lw(c) + l}px`;
      });
    }
    let s;
    if (o.parentNode instanceof DocumentFragment)
      s = $r(o).body;
    else {
      const l = o.parentElement, u = xi(o);
      s = (l == null ? void 0 : l.nodeName) === "HTML" && u.getComputedStyle(l).overflowY === "scroll" ? l : o;
    }
    n.push({
      value: s.style.overflow,
      property: "overflow",
      el: s
    }, {
      value: s.style.overflowX,
      property: "overflow-x",
      el: s
    }, {
      value: s.style.overflowY,
      property: "overflow-y",
      el: s
    }), s.style.overflow = "hidden";
  }
  return () => {
    n.forEach(({
      value: s,
      el: l,
      property: u
    }) => {
      s ? l.style.setProperty(u, s) : l.style.removeProperty(u);
    });
  };
}
function PY(e) {
  const t = [];
  return [].forEach.call(e.children, (n) => {
    n.getAttribute("aria-hidden") === "true" && t.push(n);
  }), t;
}
class OY {
  constructor() {
    this.modals = [], this.containers = [];
  }
  add(t, n) {
    let o = this.modals.indexOf(t);
    if (o !== -1)
      return o;
    o = this.modals.length, this.modals.push(t), t.modalRef && Hu(t.modalRef, !1);
    const i = PY(n);
    uw(n, t.mount, t.modalRef, i, !0);
    const s = tg(this.containers, (l) => l.container === n);
    return s !== -1 ? (this.containers[s].modals.push(t), o) : (this.containers.push({
      modals: [t],
      container: n,
      restore: null,
      hiddenSiblings: i
    }), o);
  }
  mount(t, n) {
    const o = tg(this.containers, (s) => s.modals.includes(t)), i = this.containers[o];
    i.restore || (i.restore = kY(i, n));
  }
  remove(t, n = !0) {
    const o = this.modals.indexOf(t);
    if (o === -1)
      return o;
    const i = tg(this.containers, (l) => l.modals.includes(t)), s = this.containers[i];
    if (s.modals.splice(s.modals.indexOf(t), 1), this.modals.splice(o, 1), s.modals.length === 0)
      s.restore && s.restore(), t.modalRef && Hu(t.modalRef, n), uw(s.container, t.mount, t.modalRef, s.hiddenSiblings, !1), this.containers.splice(i, 1);
    else {
      const l = s.modals[s.modals.length - 1];
      l.modalRef && Hu(l.modalRef, !1);
    }
    return o;
  }
  isTopModal(t) {
    return this.modals.length > 0 && this.modals[this.modals.length - 1] === t;
  }
}
const AY = ["input", "select", "textarea", "a[href]", "button", "[tabindex]", "audio[controls]", "video[controls]", '[contenteditable]:not([contenteditable="false"])'].join(",");
function IY(e) {
  const t = parseInt(e.getAttribute("tabindex") || "", 10);
  return Number.isNaN(t) ? e.contentEditable === "true" || (e.nodeName === "AUDIO" || e.nodeName === "VIDEO" || e.nodeName === "DETAILS") && e.getAttribute("tabindex") === null ? 0 : e.tabIndex : t;
}
function jY(e) {
  if (e.tagName !== "INPUT" || e.type !== "radio" || !e.name)
    return !1;
  const t = (o) => e.ownerDocument.querySelector(`input[type="radio"]${o}`);
  let n = t(`[name="${e.name}"]:checked`);
  return n || (n = t(`[name="${e.name}"]`)), n !== e;
}
function RY(e) {
  return !(e.disabled || e.tagName === "INPUT" && e.type === "hidden" || jY(e));
}
function TY(e) {
  const t = [], n = [];
  return Array.from(e.querySelectorAll(AY)).forEach((o, i) => {
    const s = IY(o);
    s === -1 || !RY(o) || (s === 0 ? t.push(o) : n.push({
      documentOrder: i,
      tabIndex: s,
      node: o
    }));
  }), n.sort((o, i) => o.tabIndex === i.tabIndex ? o.documentOrder - i.documentOrder : o.tabIndex - i.tabIndex).map((o) => o.node).concat(t);
}
function MY() {
  return !0;
}
function NY(e) {
  const {
    children: t,
    disableAutoFocus: n = !1,
    disableEnforceFocus: o = !1,
    disableRestoreFocus: i = !1,
    getTabbable: s = TY,
    isEnabled: l = MY,
    open: u
  } = e, c = z.useRef(!1), f = z.useRef(null), p = z.useRef(null), g = z.useRef(null), h = z.useRef(null), y = z.useRef(!1), v = z.useRef(null), C = _n(kc(t), v), $ = z.useRef(null);
  z.useEffect(() => {
    !u || !v.current || (y.current = !n);
  }, [n, u]), z.useEffect(() => {
    if (!u || !v.current)
      return;
    const _ = $r(v.current);
    return v.current.contains(_.activeElement) || (v.current.hasAttribute("tabIndex") || v.current.setAttribute("tabIndex", "-1"), y.current && v.current.focus()), () => {
      i || (g.current && g.current.focus && (c.current = !0, g.current.focus()), g.current = null);
    };
  }, [u]), z.useEffect(() => {
    if (!u || !v.current)
      return;
    const _ = $r(v.current), b = (j) => {
      $.current = j, !(o || !l() || j.key !== "Tab") && _.activeElement === v.current && j.shiftKey && (c.current = !0, p.current && p.current.focus());
    }, k = () => {
      var B, O;
      const j = v.current;
      if (j === null)
        return;
      if (!_.hasFocus() || !l() || c.current) {
        c.current = !1;
        return;
      }
      if (j.contains(_.activeElement) || o && _.activeElement !== f.current && _.activeElement !== p.current)
        return;
      if (_.activeElement !== h.current)
        h.current = null;
      else if (h.current !== null)
        return;
      if (!y.current)
        return;
      let L = [];
      if ((_.activeElement === f.current || _.activeElement === p.current) && (L = s(v.current)), L.length > 0) {
        const V = !!((B = $.current) != null && B.shiftKey && ((O = $.current) == null ? void 0 : O.key) === "Tab"), W = L[0], q = L[L.length - 1];
        typeof W != "string" && typeof q != "string" && (V ? q.focus() : W.focus());
      } else
        j.focus();
    };
    _.addEventListener("focusin", k), _.addEventListener("keydown", b, !0);
    const E = setInterval(() => {
      _.activeElement && _.activeElement.tagName === "BODY" && k();
    }, 50);
    return () => {
      clearInterval(E), _.removeEventListener("focusin", k), _.removeEventListener("keydown", b, !0);
    };
  }, [n, o, i, l, u, s]);
  const S = (_) => {
    g.current === null && (g.current = _.relatedTarget), y.current = !0, h.current = _.target;
    const b = t.props.onFocus;
    b && b(_);
  }, w = (_) => {
    g.current === null && (g.current = _.relatedTarget), y.current = !0;
  };
  return /* @__PURE__ */ A.jsxs(z.Fragment, {
    children: [/* @__PURE__ */ A.jsx("div", {
      tabIndex: u ? 0 : -1,
      onFocus: w,
      ref: f,
      "data-testid": "sentinelStart"
    }), /* @__PURE__ */ z.cloneElement(t, {
      ref: C,
      onFocus: S
    }), /* @__PURE__ */ A.jsx("div", {
      tabIndex: u ? 0 : -1,
      onFocus: w,
      ref: p,
      "data-testid": "sentinelEnd"
    })]
  });
}
function FY(e) {
  return typeof e == "function" ? e() : e;
}
const LY = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const {
    children: o,
    container: i,
    disablePortal: s = !1
  } = t, [l, u] = z.useState(null), c = _n(/* @__PURE__ */ z.isValidElement(o) ? kc(o) : null, n);
  if (Ei(() => {
    s || u(FY(i) || document.body);
  }, [i, s]), Ei(() => {
    if (l && !s)
      return x2(n, l), () => {
        x2(n, null);
      };
  }, [n, l, s]), s) {
    if (/* @__PURE__ */ z.isValidElement(o)) {
      const f = {
        ref: c
      };
      return /* @__PURE__ */ z.cloneElement(o, f);
    }
    return o;
  }
  return l && /* @__PURE__ */ iy.createPortal(o, l);
});
function Tr(e, t) {
  const {
    className: n,
    elementType: o,
    ownerState: i,
    externalForwardedProps: s,
    internalForwardedProps: l,
    ...u
  } = t, {
    component: c,
    slots: f = {
      [e]: void 0
    },
    slotProps: p = {
      [e]: void 0
    },
    ...g
  } = s, h = f[e] || o, y = Wx(p[e], i), {
    props: {
      component: v,
      ...C
    },
    internalRef: $
  } = Vx({
    className: n,
    ...u,
    externalForwardedProps: e === "root" ? g : void 0,
    externalSlotProps: y
  }), S = _n($, y == null ? void 0 : y.ref, t.ref), w = e === "root" ? v || c : v, _ = Bx(h, {
    ...e === "root" && !c && !f[e] && l,
    ...e !== "root" && !f[e] && l,
    ...C,
    ...w && {
      as: w
    },
    ref: S
  }, i);
  return [h, _];
}
const DY = {
  entering: {
    opacity: 1
  },
  entered: {
    opacity: 1
  }
}, zY = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = Tl(), i = {
    enter: o.transitions.duration.enteringScreen,
    exit: o.transitions.duration.leavingScreen
  }, {
    addEndListener: s,
    appear: l = !0,
    children: u,
    easing: c,
    in: f,
    onEnter: p,
    onEntered: g,
    onEntering: h,
    onExit: y,
    onExited: v,
    onExiting: C,
    style: $,
    timeout: S = i,
    // eslint-disable-next-line react/prop-types
    TransitionComponent: w = jo,
    ..._
  } = t, b = z.useRef(null), k = _n(b, kc(u), n), E = (Q) => (X) => {
    if (Q) {
      const ce = b.current;
      X === void 0 ? Q(ce) : Q(ce, X);
    }
  }, j = E(h), L = E((Q, X) => {
    _E(Q);
    const ce = fl({
      style: $,
      timeout: S,
      easing: c
    }, {
      mode: "enter"
    });
    Q.style.webkitTransition = o.transitions.create("opacity", ce), Q.style.transition = o.transitions.create("opacity", ce), p && p(Q, X);
  }), B = E(g), O = E(C), V = E((Q) => {
    const X = fl({
      style: $,
      timeout: S,
      easing: c
    }, {
      mode: "exit"
    });
    Q.style.webkitTransition = o.transitions.create("opacity", X), Q.style.transition = o.transitions.create("opacity", X), y && y(Q);
  }), W = E(v), q = (Q) => {
    s && s(b.current, Q);
  };
  return /* @__PURE__ */ A.jsx(w, {
    appear: l,
    in: f,
    nodeRef: b,
    onEnter: L,
    onEntered: B,
    onEntering: j,
    onExit: V,
    onExited: W,
    onExiting: O,
    addEndListener: q,
    timeout: S,
    ..._,
    children: (Q, {
      ownerState: X,
      ...ce
    }) => /* @__PURE__ */ z.cloneElement(u, {
      style: {
        opacity: 0,
        visibility: Q === "exited" && !f ? "hidden" : void 0,
        ...DY[Q],
        ...$,
        ...u.props.style
      },
      ref: k,
      ...ce
    })
  });
});
function BY(e) {
  return ht("MuiBackdrop", e);
}
dt("MuiBackdrop", ["root", "invisible"]);
const UY = (e) => {
  const {
    classes: t,
    invisible: n
  } = e;
  return mt({
    root: ["root", n && "invisible"]
  }, BY, t);
}, VY = Oe("div", {
  name: "MuiBackdrop",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, n.invisible && t.invisible];
  }
})({
  position: "fixed",
  display: "flex",
  alignItems: "center",
  justifyContent: "center",
  right: 0,
  bottom: 0,
  top: 0,
  left: 0,
  backgroundColor: "rgba(0, 0, 0, 0.5)",
  WebkitTapHighlightColor: "transparent",
  variants: [{
    props: {
      invisible: !0
    },
    style: {
      backgroundColor: "transparent"
    }
  }]
}), WY = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiBackdrop"
  }), {
    children: i,
    className: s,
    component: l = "div",
    invisible: u = !1,
    open: c,
    components: f = {},
    componentsProps: p = {},
    slotProps: g = {},
    slots: h = {},
    TransitionComponent: y,
    transitionDuration: v,
    ...C
  } = o, $ = {
    ...o,
    component: l,
    invisible: u
  }, S = UY($), w = {
    transition: y,
    root: f.Root,
    ...h
  }, _ = {
    ...p,
    ...g
  }, b = {
    slots: w,
    slotProps: _
  }, [k, E] = Tr("root", {
    elementType: VY,
    externalForwardedProps: b,
    className: Ve(S.root, s),
    ownerState: $
  }), [j, L] = Tr("transition", {
    elementType: zY,
    externalForwardedProps: b,
    ownerState: $
  });
  return /* @__PURE__ */ A.jsx(j, {
    in: c,
    timeout: v,
    ...C,
    ...L,
    children: /* @__PURE__ */ A.jsx(k, {
      "aria-hidden": !0,
      ...E,
      classes: S,
      ref: n,
      children: i
    })
  });
});
function qY(e) {
  return typeof e == "function" ? e() : e;
}
function KY(e) {
  return e ? e.props.hasOwnProperty("in") : !1;
}
const cw = () => {
}, Zd = new OY();
function HY(e) {
  const {
    container: t,
    disableEscapeKeyDown: n = !1,
    disableScrollLock: o = !1,
    closeAfterTransition: i = !1,
    onTransitionEnter: s,
    onTransitionExited: l,
    children: u,
    onClose: c,
    open: f,
    rootRef: p
  } = e, g = z.useRef({}), h = z.useRef(null), y = z.useRef(null), v = _n(y, p), [C, $] = z.useState(!f), S = KY(u);
  let w = !0;
  (e["aria-hidden"] === "false" || e["aria-hidden"] === !1) && (w = !1);
  const _ = () => $r(h.current), b = () => (g.current.modalRef = y.current, g.current.mount = h.current, g.current), k = () => {
    Zd.mount(b(), {
      disableScrollLock: o
    }), y.current && (y.current.scrollTop = 0);
  }, E = yi(() => {
    const X = qY(t) || _().body;
    Zd.add(b(), X), y.current && k();
  }), j = () => Zd.isTopModal(b()), L = yi((X) => {
    h.current = X, X && (f && j() ? k() : y.current && Hu(y.current, w));
  }), B = z.useCallback(() => {
    Zd.remove(b(), w);
  }, [w]);
  z.useEffect(() => () => {
    B();
  }, [B]), z.useEffect(() => {
    f ? E() : (!S || !i) && B();
  }, [f, B, S, i, E]);
  const O = (X) => (ce) => {
    var J;
    (J = X.onKeyDown) == null || J.call(X, ce), !(ce.key !== "Escape" || ce.which === 229 || // Wait until IME is settled.
    !j()) && (n || (ce.stopPropagation(), c && c(ce, "escapeKeyDown")));
  }, V = (X) => (ce) => {
    var J;
    (J = X.onClick) == null || J.call(X, ce), ce.target === ce.currentTarget && c && c(ce, "backdropClick");
  };
  return {
    getRootProps: (X = {}) => {
      const ce = Wu(e);
      delete ce.onTransitionEnter, delete ce.onTransitionExited;
      const J = {
        ...ce,
        ...X
      };
      return {
        /*
         * Marking an element with the role presentation indicates to assistive technology
         * that this element should be ignored; it exists to support the web application and
         * is not meant for humans to interact with directly.
         * https://github.com/evcohen/eslint-plugin-jsx-a11y/blob/master/docs/rules/no-static-element-interactions.md
         */
        role: "presentation",
        ...J,
        onKeyDown: O(J),
        ref: v
      };
    },
    getBackdropProps: (X = {}) => {
      const ce = X;
      return {
        "aria-hidden": !0,
        ...ce,
        onClick: V(ce),
        open: f
      };
    },
    getTransitionProps: () => {
      const X = () => {
        $(!1), s && s();
      }, ce = () => {
        $(!0), l && l(), i && B();
      };
      return {
        onEnter: _g(X, (u == null ? void 0 : u.props.onEnter) ?? cw),
        onExited: _g(ce, (u == null ? void 0 : u.props.onExited) ?? cw)
      };
    },
    rootRef: v,
    portalRef: L,
    isTopModal: j,
    exited: C,
    hasTransition: S
  };
}
function GY(e) {
  return ht("MuiModal", e);
}
dt("MuiModal", ["root", "hidden", "backdrop"]);
const JY = (e) => {
  const {
    open: t,
    exited: n,
    classes: o
  } = e;
  return mt({
    root: ["root", !t && n && "hidden"],
    backdrop: ["backdrop"]
  }, GY, o);
}, YY = Oe("div", {
  name: "MuiModal",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, !n.open && n.exited && t.hidden];
  }
})(ct(({
  theme: e
}) => ({
  position: "fixed",
  zIndex: (e.vars || e).zIndex.modal,
  right: 0,
  bottom: 0,
  top: 0,
  left: 0,
  variants: [{
    props: ({
      ownerState: t
    }) => !t.open && t.exited,
    style: {
      visibility: "hidden"
    }
  }]
}))), QY = Oe(WY, {
  name: "MuiModal",
  slot: "Backdrop",
  overridesResolver: (e, t) => t.backdrop
})({
  zIndex: -1
}), XY = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    name: "MuiModal",
    props: t
  }), {
    BackdropComponent: i = QY,
    BackdropProps: s,
    classes: l,
    className: u,
    closeAfterTransition: c = !1,
    children: f,
    container: p,
    component: g,
    components: h = {},
    componentsProps: y = {},
    disableAutoFocus: v = !1,
    disableEnforceFocus: C = !1,
    disableEscapeKeyDown: $ = !1,
    disablePortal: S = !1,
    disableRestoreFocus: w = !1,
    disableScrollLock: _ = !1,
    hideBackdrop: b = !1,
    keepMounted: k = !1,
    onBackdropClick: E,
    onClose: j,
    onTransitionEnter: L,
    onTransitionExited: B,
    open: O,
    slotProps: V = {},
    slots: W = {},
    // eslint-disable-next-line react/prop-types
    theme: q,
    ...Q
  } = o, X = {
    ...o,
    closeAfterTransition: c,
    disableAutoFocus: v,
    disableEnforceFocus: C,
    disableEscapeKeyDown: $,
    disablePortal: S,
    disableRestoreFocus: w,
    disableScrollLock: _,
    hideBackdrop: b,
    keepMounted: k
  }, {
    getRootProps: ce,
    getBackdropProps: J,
    getTransitionProps: ie,
    portalRef: re,
    isTopModal: Y,
    exited: T,
    hasTransition: H
  } = HY({
    ...X,
    rootRef: n
  }), G = {
    ...X,
    exited: T
  }, N = JY(G), U = {};
  if (f.props.tabIndex === void 0 && (U.tabIndex = "-1"), H) {
    const {
      onEnter: We,
      onExited: et
    } = ie();
    U.onEnter = We, U.onExited = et;
  }
  const Z = {
    ...Q,
    slots: {
      root: h.Root,
      backdrop: h.Backdrop,
      ...W
    },
    slotProps: {
      ...y,
      ...V
    }
  }, [me, ue] = Tr("root", {
    elementType: YY,
    externalForwardedProps: Z,
    getSlotProps: ce,
    additionalProps: {
      ref: n,
      as: g
    },
    ownerState: G,
    className: Ve(u, N == null ? void 0 : N.root, !G.open && G.exited && (N == null ? void 0 : N.hidden))
  }), [Ce, $e] = Tr("backdrop", {
    elementType: i,
    externalForwardedProps: Z,
    additionalProps: s,
    getSlotProps: (We) => J({
      ...We,
      onClick: (et) => {
        E && E(et), We != null && We.onClick && We.onClick(et);
      }
    }),
    className: Ve(s == null ? void 0 : s.className, N == null ? void 0 : N.backdrop),
    ownerState: G
  }), Te = _n(s == null ? void 0 : s.ref, $e.ref);
  return !k && !O && (!H || T) ? null : /* @__PURE__ */ A.jsx(LY, {
    ref: re,
    container: p,
    disablePortal: S,
    children: /* @__PURE__ */ A.jsxs(me, {
      ...ue,
      children: [!b && i ? /* @__PURE__ */ A.jsx(Ce, {
        ...$e,
        ref: Te
      }) : null, /* @__PURE__ */ A.jsx(NY, {
        disableEnforceFocus: C,
        disableAutoFocus: v,
        disableRestoreFocus: w,
        isEnabled: Y,
        open: O,
        children: /* @__PURE__ */ z.cloneElement(f, U)
      })]
    })
  });
});
function ZY(e) {
  return ht("MuiPopover", e);
}
dt("MuiPopover", ["root", "paper"]);
function dw(e, t) {
  let n = 0;
  return typeof t == "number" ? n = t : t === "center" ? n = e.height / 2 : t === "bottom" && (n = e.height), n;
}
function fw(e, t) {
  let n = 0;
  return typeof t == "number" ? n = t : t === "center" ? n = e.width / 2 : t === "right" && (n = e.width), n;
}
function pw(e) {
  return [e.horizontal, e.vertical].map((t) => typeof t == "number" ? `${t}px` : t).join(" ");
}
function ng(e) {
  return typeof e == "function" ? e() : e;
}
const eQ = (e) => {
  const {
    classes: t
  } = e;
  return mt({
    root: ["root"],
    paper: ["paper"]
  }, ZY, t);
}, tQ = Oe(XY, {
  name: "MuiPopover",
  slot: "Root",
  overridesResolver: (e, t) => t.root
})({}), kE = Oe(Ml, {
  name: "MuiPopover",
  slot: "Paper",
  overridesResolver: (e, t) => t.paper
})({
  position: "absolute",
  overflowY: "auto",
  overflowX: "hidden",
  // So we see the popover when it's empty.
  // It's most likely on issue on userland.
  minWidth: 16,
  minHeight: 16,
  maxWidth: "calc(100% - 32px)",
  maxHeight: "calc(100% - 32px)",
  // We disable the focus ring for mouse, touch and keyboard users.
  outline: 0
}), nQ = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiPopover"
  }), {
    action: i,
    anchorEl: s,
    anchorOrigin: l = {
      vertical: "top",
      horizontal: "left"
    },
    anchorPosition: u,
    anchorReference: c = "anchorEl",
    children: f,
    className: p,
    container: g,
    elevation: h = 8,
    marginThreshold: y = 16,
    open: v,
    PaperProps: C = {},
    slots: $ = {},
    slotProps: S = {},
    transformOrigin: w = {
      vertical: "top",
      horizontal: "left"
    },
    TransitionComponent: _ = Dg,
    transitionDuration: b = "auto",
    TransitionProps: {
      onEntering: k,
      ...E
    } = {},
    disableScrollLock: j = !1,
    ...L
  } = o, B = (S == null ? void 0 : S.paper) ?? C, O = z.useRef(), V = {
    ...o,
    anchorOrigin: l,
    anchorReference: c,
    elevation: h,
    marginThreshold: y,
    externalPaperSlotProps: B,
    transformOrigin: w,
    TransitionComponent: _,
    transitionDuration: b,
    TransitionProps: E
  }, W = eQ(V), q = z.useCallback(() => {
    if (c === "anchorPosition")
      return u;
    const $e = ng(s), We = ($e && $e.nodeType === 1 ? $e : $r(O.current).body).getBoundingClientRect();
    return {
      top: We.top + dw(We, l.vertical),
      left: We.left + fw(We, l.horizontal)
    };
  }, [s, l.horizontal, l.vertical, u, c]), Q = z.useCallback(($e) => ({
    vertical: dw($e, w.vertical),
    horizontal: fw($e, w.horizontal)
  }), [w.horizontal, w.vertical]), X = z.useCallback(($e) => {
    const Te = {
      width: $e.offsetWidth,
      height: $e.offsetHeight
    }, We = Q(Te);
    if (c === "none")
      return {
        top: null,
        left: null,
        transformOrigin: pw(We)
      };
    const et = q();
    let Xe = et.top - We.vertical, ft = et.left - We.horizontal;
    const Lt = Xe + Te.height, st = ft + Te.width, pt = xi(ng(s)), Se = pt.innerHeight - y, gt = pt.innerWidth - y;
    if (y !== null && Xe < y) {
      const yt = Xe - y;
      Xe -= yt, We.vertical += yt;
    } else if (y !== null && Lt > Se) {
      const yt = Lt - Se;
      Xe -= yt, We.vertical += yt;
    }
    if (y !== null && ft < y) {
      const yt = ft - y;
      ft -= yt, We.horizontal += yt;
    } else if (st > gt) {
      const yt = st - gt;
      ft -= yt, We.horizontal += yt;
    }
    return {
      top: `${Math.round(Xe)}px`,
      left: `${Math.round(ft)}px`,
      transformOrigin: pw(We)
    };
  }, [s, c, q, Q, y]), [ce, J] = z.useState(v), ie = z.useCallback(() => {
    const $e = O.current;
    if (!$e)
      return;
    const Te = X($e);
    Te.top !== null && $e.style.setProperty("top", Te.top), Te.left !== null && ($e.style.left = Te.left), $e.style.transformOrigin = Te.transformOrigin, J(!0);
  }, [X]);
  z.useEffect(() => (j && window.addEventListener("scroll", ie), () => window.removeEventListener("scroll", ie)), [s, j, ie]);
  const re = ($e, Te) => {
    k && k($e, Te), ie();
  }, Y = () => {
    J(!1);
  };
  z.useEffect(() => {
    v && ie();
  }), z.useImperativeHandle(i, () => v ? {
    updatePosition: () => {
      ie();
    }
  } : null, [v, ie]), z.useEffect(() => {
    if (!v)
      return;
    const $e = Fx(() => {
      ie();
    }), Te = xi(s);
    return Te.addEventListener("resize", $e), () => {
      $e.clear(), Te.removeEventListener("resize", $e);
    };
  }, [s, v, ie]);
  let T = b;
  b === "auto" && !_.muiSupportAuto && (T = void 0);
  const H = g || (s ? $r(ng(s)).body : void 0), G = {
    slots: $,
    slotProps: {
      ...S,
      paper: B
    }
  }, [N, U] = Tr("paper", {
    elementType: kE,
    externalForwardedProps: G,
    additionalProps: {
      elevation: h,
      className: Ve(W.paper, B == null ? void 0 : B.className),
      style: ce ? B.style : {
        ...B.style,
        opacity: 0
      }
    },
    ownerState: V
  }), [Z, {
    slotProps: me,
    ...ue
  }] = Tr("root", {
    elementType: tQ,
    externalForwardedProps: G,
    additionalProps: {
      slotProps: {
        backdrop: {
          invisible: !0
        }
      },
      container: H,
      open: v
    },
    ownerState: V,
    className: Ve(W.root, p)
  }), Ce = _n(O, U.ref);
  return /* @__PURE__ */ A.jsx(Z, {
    ...ue,
    ...!vi(Z) && {
      slotProps: me,
      disableScrollLock: j
    },
    ...L,
    ref: n,
    children: /* @__PURE__ */ A.jsx(_, {
      appear: !0,
      in: v,
      onEntering: re,
      onExited: Y,
      timeout: T,
      ...E,
      children: /* @__PURE__ */ A.jsx(N, {
        ...U,
        ref: Ce,
        children: f
      })
    })
  });
});
function rQ(e) {
  return ht("MuiMenu", e);
}
dt("MuiMenu", ["root", "paper", "list"]);
const oQ = {
  vertical: "top",
  horizontal: "right"
}, iQ = {
  vertical: "top",
  horizontal: "left"
}, aQ = (e) => {
  const {
    classes: t
  } = e;
  return mt({
    root: ["root"],
    paper: ["paper"],
    list: ["list"]
  }, rQ, t);
}, sQ = Oe(nQ, {
  shouldForwardProp: (e) => ur(e) || e === "classes",
  name: "MuiMenu",
  slot: "Root",
  overridesResolver: (e, t) => t.root
})({}), lQ = Oe(kE, {
  name: "MuiMenu",
  slot: "Paper",
  overridesResolver: (e, t) => t.paper
})({
  // specZ: The maximum height of a simple menu should be one or more rows less than the view
  // height. This ensures a tappable area outside of the simple menu with which to dismiss
  // the menu.
  maxHeight: "calc(100% - 96px)",
  // Add iOS momentum scrolling for iOS < 13.0
  WebkitOverflowScrolling: "touch"
}), uQ = Oe(CY, {
  name: "MuiMenu",
  slot: "List",
  overridesResolver: (e, t) => t.list
})({
  // We disable the focus ring for mouse, touch and keyboard users.
  outline: 0
}), cQ = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiMenu"
  }), {
    autoFocus: i = !0,
    children: s,
    className: l,
    disableAutoFocusItem: u = !1,
    MenuListProps: c = {},
    onClose: f,
    open: p,
    PaperProps: g = {},
    PopoverClasses: h,
    transitionDuration: y = "auto",
    TransitionProps: {
      onEntering: v,
      ...C
    } = {},
    variant: $ = "selectedMenu",
    slots: S = {},
    slotProps: w = {},
    ..._
  } = o, b = g0(), k = {
    ...o,
    autoFocus: i,
    disableAutoFocusItem: u,
    MenuListProps: c,
    onEntering: v,
    PaperProps: g,
    transitionDuration: y,
    TransitionProps: C,
    variant: $
  }, E = aQ(k), j = i && !u && p, L = z.useRef(null), B = (ce, J) => {
    L.current && L.current.adjustStyleForScrollbar(ce, {
      direction: b ? "rtl" : "ltr"
    }), v && v(ce, J);
  }, O = (ce) => {
    ce.key === "Tab" && (ce.preventDefault(), f && f(ce, "tabKeyDown"));
  };
  let V = -1;
  z.Children.map(s, (ce, J) => {
    /* @__PURE__ */ z.isValidElement(ce) && (ce.props.disabled || ($ === "selectedMenu" && ce.props.selected || V === -1) && (V = J));
  });
  const W = S.paper ?? lQ, q = w.paper ?? g, Q = Vo({
    elementType: S.root,
    externalSlotProps: w.root,
    ownerState: k,
    className: [E.root, l]
  }), X = Vo({
    elementType: W,
    externalSlotProps: q,
    ownerState: k,
    className: E.paper
  });
  return /* @__PURE__ */ A.jsx(sQ, {
    onClose: f,
    anchorOrigin: {
      vertical: "bottom",
      horizontal: b ? "right" : "left"
    },
    transformOrigin: b ? oQ : iQ,
    slots: {
      paper: W,
      root: S.root
    },
    slotProps: {
      root: Q,
      paper: X
    },
    open: p,
    ref: n,
    transitionDuration: y,
    TransitionProps: {
      onEntering: B,
      ...C
    },
    ownerState: k,
    ..._,
    classes: h,
    children: /* @__PURE__ */ A.jsx(uQ, {
      onKeyDown: O,
      actions: L,
      autoFocus: i && (V === -1 || u),
      autoFocusItem: j,
      variant: $,
      ...c,
      className: Ve(E.list, c.className),
      children: s
    })
  });
});
function dQ(e) {
  return ht("MuiNativeSelect", e);
}
const P0 = dt("MuiNativeSelect", ["root", "select", "multiple", "filled", "outlined", "standard", "disabled", "icon", "iconOpen", "iconFilled", "iconOutlined", "iconStandard", "nativeInput", "error"]), fQ = (e) => {
  const {
    classes: t,
    variant: n,
    disabled: o,
    multiple: i,
    open: s,
    error: l
  } = e, u = {
    select: ["select", n, o && "disabled", i && "multiple", l && "error"],
    icon: ["icon", `icon${Ne(n)}`, s && "iconOpen", o && "disabled"]
  };
  return mt(u, dQ, t);
}, PE = Oe("select")(({
  theme: e
}) => ({
  // Reset
  MozAppearance: "none",
  // Reset
  WebkitAppearance: "none",
  // When interacting quickly, the text can end up selected.
  // Native select can't be selected either.
  userSelect: "none",
  // Reset
  borderRadius: 0,
  cursor: "pointer",
  "&:focus": {
    // Reset Chrome style
    borderRadius: 0
  },
  [`&.${P0.disabled}`]: {
    cursor: "default"
  },
  "&[multiple]": {
    height: "auto"
  },
  "&:not([multiple]) option, &:not([multiple]) optgroup": {
    backgroundColor: (e.vars || e).palette.background.paper
  },
  variants: [{
    props: ({
      ownerState: t
    }) => t.variant !== "filled" && t.variant !== "outlined",
    style: {
      // Bump specificity to allow extending custom inputs
      "&&&": {
        paddingRight: 24,
        minWidth: 16
        // So it doesn't collapse.
      }
    }
  }, {
    props: {
      variant: "filled"
    },
    style: {
      "&&&": {
        paddingRight: 32
      }
    }
  }, {
    props: {
      variant: "outlined"
    },
    style: {
      borderRadius: (e.vars || e).shape.borderRadius,
      "&:focus": {
        borderRadius: (e.vars || e).shape.borderRadius
        // Reset the reset for Chrome style
      },
      "&&&": {
        paddingRight: 32
      }
    }
  }]
})), pQ = Oe(PE, {
  name: "MuiNativeSelect",
  slot: "Select",
  shouldForwardProp: ur,
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.select, t[n.variant], n.error && t.error, {
      [`&.${P0.multiple}`]: t.multiple
    }];
  }
})({}), OE = Oe("svg")(({
  theme: e
}) => ({
  // We use a position absolute over a flexbox in order to forward the pointer events
  // to the input and to support wrapping tags..
  position: "absolute",
  right: 0,
  // Center vertically, height is 1em
  top: "calc(50% - .5em)",
  // Don't block pointer events on the select under the icon.
  pointerEvents: "none",
  color: (e.vars || e).palette.action.active,
  [`&.${P0.disabled}`]: {
    color: (e.vars || e).palette.action.disabled
  },
  variants: [{
    props: ({
      ownerState: t
    }) => t.open,
    style: {
      transform: "rotate(180deg)"
    }
  }, {
    props: {
      variant: "filled"
    },
    style: {
      right: 7
    }
  }, {
    props: {
      variant: "outlined"
    },
    style: {
      right: 7
    }
  }]
})), hQ = Oe(OE, {
  name: "MuiNativeSelect",
  slot: "Icon",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.icon, n.variant && t[`icon${Ne(n.variant)}`], n.open && t.iconOpen];
  }
})({}), mQ = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const {
    className: o,
    disabled: i,
    error: s,
    IconComponent: l,
    inputRef: u,
    variant: c = "standard",
    ...f
  } = t, p = {
    ...t,
    disabled: i,
    variant: c,
    error: s
  }, g = fQ(p);
  return /* @__PURE__ */ A.jsxs(z.Fragment, {
    children: [/* @__PURE__ */ A.jsx(pQ, {
      ownerState: p,
      className: Ve(g.select, o),
      disabled: i,
      ref: u || n,
      ...f
    }), t.multiple ? null : /* @__PURE__ */ A.jsx(hQ, {
      as: l,
      ownerState: p,
      className: g.icon
    })]
  });
});
function gQ(e) {
  return ht("MuiSelect", e);
}
const Ou = dt("MuiSelect", ["root", "select", "multiple", "filled", "outlined", "standard", "disabled", "focused", "icon", "iconOpen", "iconFilled", "iconOutlined", "iconStandard", "nativeInput", "error"]);
var hw;
const yQ = Oe(PE, {
  name: "MuiSelect",
  slot: "Select",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [
      // Win specificity over the input base
      {
        [`&.${Ou.select}`]: t.select
      },
      {
        [`&.${Ou.select}`]: t[n.variant]
      },
      {
        [`&.${Ou.error}`]: t.error
      },
      {
        [`&.${Ou.multiple}`]: t.multiple
      }
    ];
  }
})({
  // Win specificity over the input base
  [`&.${Ou.select}`]: {
    height: "auto",
    // Resets for multiple select with chips
    minHeight: "1.4375em",
    // Required for select\text-field height consistency
    textOverflow: "ellipsis",
    whiteSpace: "nowrap",
    overflow: "hidden"
  }
}), vQ = Oe(OE, {
  name: "MuiSelect",
  slot: "Icon",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.icon, n.variant && t[`icon${Ne(n.variant)}`], n.open && t.iconOpen];
  }
})({}), bQ = Oe("input", {
  shouldForwardProp: (e) => Dp(e) && e !== "classes",
  name: "MuiSelect",
  slot: "NativeInput",
  overridesResolver: (e, t) => t.nativeInput
})({
  bottom: 0,
  left: 0,
  position: "absolute",
  opacity: 0,
  pointerEvents: "none",
  width: "100%",
  boxSizing: "border-box"
});
function mw(e, t) {
  return typeof t == "object" && t !== null ? e === t : String(e) === String(t);
}
function SQ(e) {
  return e == null || typeof e == "string" && !e.trim();
}
const wQ = (e) => {
  const {
    classes: t,
    variant: n,
    disabled: o,
    multiple: i,
    open: s,
    error: l
  } = e, u = {
    select: ["select", n, o && "disabled", i && "multiple", l && "error"],
    icon: ["icon", `icon${Ne(n)}`, s && "iconOpen", o && "disabled"],
    nativeInput: ["nativeInput"]
  };
  return mt(u, gQ, t);
}, $Q = /* @__PURE__ */ z.forwardRef(function(t, n) {
  var Rt;
  const {
    "aria-describedby": o,
    "aria-label": i,
    autoFocus: s,
    autoWidth: l,
    children: u,
    className: c,
    defaultOpen: f,
    defaultValue: p,
    disabled: g,
    displayEmpty: h,
    error: y = !1,
    IconComponent: v,
    inputRef: C,
    labelId: $,
    MenuProps: S = {},
    multiple: w,
    name: _,
    onBlur: b,
    onChange: k,
    onClose: E,
    onFocus: j,
    onOpen: L,
    open: B,
    readOnly: O,
    renderValue: V,
    required: W,
    SelectDisplayProps: q = {},
    tabIndex: Q,
    // catching `type` from Input which makes no sense for SelectInput
    type: X,
    value: ce,
    variant: J = "standard",
    ...ie
  } = t, [re, Y] = sc({
    controlled: ce,
    default: p,
    name: "Select"
  }), [T, H] = sc({
    controlled: B,
    default: f,
    name: "Select"
  }), G = z.useRef(null), N = z.useRef(null), [U, Z] = z.useState(null), {
    current: me
  } = z.useRef(B != null), [ue, Ce] = z.useState(), $e = _n(n, C), Te = z.useCallback((Fe) => {
    N.current = Fe, Fe && Z(Fe);
  }, []), We = U == null ? void 0 : U.parentNode;
  z.useImperativeHandle($e, () => ({
    focus: () => {
      N.current.focus();
    },
    node: G.current,
    value: re
  }), [re]), z.useEffect(() => {
    f && T && U && !me && (Ce(l ? null : We.clientWidth), N.current.focus());
  }, [U, l]), z.useEffect(() => {
    s && N.current.focus();
  }, [s]), z.useEffect(() => {
    if (!$)
      return;
    const Fe = $r(N.current).getElementById($);
    if (Fe) {
      const tt = () => {
        getSelection().isCollapsed && N.current.focus();
      };
      return Fe.addEventListener("click", tt), () => {
        Fe.removeEventListener("click", tt);
      };
    }
  }, [$]);
  const et = (Fe, tt) => {
    Fe ? L && L(tt) : E && E(tt), me || (Ce(l ? null : We.clientWidth), H(Fe));
  }, Xe = (Fe) => {
    Fe.button === 0 && (Fe.preventDefault(), N.current.focus(), et(!0, Fe));
  }, ft = (Fe) => {
    et(!1, Fe);
  }, Lt = z.Children.toArray(u), st = (Fe) => {
    const tt = Lt.find((wt) => wt.props.value === Fe.target.value);
    tt !== void 0 && (Y(tt.props.value), k && k(Fe, tt));
  }, pt = (Fe) => (tt) => {
    let wt;
    if (tt.currentTarget.hasAttribute("tabindex")) {
      if (w) {
        wt = Array.isArray(re) ? re.slice() : [];
        const Gt = re.indexOf(Fe.props.value);
        Gt === -1 ? wt.push(Fe.props.value) : wt.splice(Gt, 1);
      } else
        wt = Fe.props.value;
      if (Fe.props.onClick && Fe.props.onClick(tt), re !== wt && (Y(wt), k)) {
        const Gt = tt.nativeEvent || tt, dn = new Gt.constructor(Gt.type, Gt);
        Object.defineProperty(dn, "target", {
          writable: !0,
          value: {
            value: wt,
            name: _
          }
        }), k(dn, Fe);
      }
      w || et(!1, tt);
    }
  }, Se = (Fe) => {
    O || [
      " ",
      "ArrowUp",
      "ArrowDown",
      // The native select doesn't respond to enter on macOS, but it's recommended by
      // https://www.w3.org/WAI/ARIA/apg/patterns/combobox/examples/combobox-select-only/
      "Enter"
    ].includes(Fe.key) && (Fe.preventDefault(), et(!0, Fe));
  }, gt = U !== null && T, yt = (Fe) => {
    !gt && b && (Object.defineProperty(Fe, "target", {
      writable: !0,
      value: {
        value: re,
        name: _
      }
    }), b(Fe));
  };
  delete ie["aria-invalid"];
  let Le, cn;
  const ye = [];
  let Pe = !1;
  (qf({
    value: re
  }) || h) && (V ? Le = V(re) : Pe = !0);
  const Me = Lt.map((Fe) => {
    if (!/* @__PURE__ */ z.isValidElement(Fe))
      return null;
    let tt;
    if (w) {
      if (!Array.isArray(re))
        throw new Error(Ci(2));
      tt = re.some((wt) => mw(wt, Fe.props.value)), tt && Pe && ye.push(Fe.props.children);
    } else
      tt = mw(re, Fe.props.value), tt && Pe && (cn = Fe.props.children);
    return /* @__PURE__ */ z.cloneElement(Fe, {
      "aria-selected": tt ? "true" : "false",
      onClick: pt(Fe),
      onKeyUp: (wt) => {
        wt.key === " " && wt.preventDefault(), Fe.props.onKeyUp && Fe.props.onKeyUp(wt);
      },
      role: "option",
      selected: tt,
      value: void 0,
      // The value is most likely not a valid HTML attribute.
      "data-value": Fe.props.value
      // Instead, we provide it as a data attribute.
    });
  });
  Pe && (w ? ye.length === 0 ? Le = null : Le = ye.reduce((Fe, tt, wt) => (Fe.push(tt), wt < ye.length - 1 && Fe.push(", "), Fe), []) : Le = cn);
  let Ye = ue;
  !l && me && U && (Ye = We.clientWidth);
  let ot;
  typeof Q < "u" ? ot = Q : ot = g ? null : 0;
  const se = q.id || (_ ? `mui-component-select-${_}` : void 0), ae = {
    ...t,
    variant: J,
    value: re,
    open: gt,
    error: y
  }, be = wQ(ae), Re = {
    ...S.PaperProps,
    ...(Rt = S.slotProps) == null ? void 0 : Rt.paper
  }, Ge = t0();
  return /* @__PURE__ */ A.jsxs(z.Fragment, {
    children: [/* @__PURE__ */ A.jsx(yQ, {
      as: "div",
      ref: Te,
      tabIndex: ot,
      role: "combobox",
      "aria-controls": Ge,
      "aria-disabled": g ? "true" : void 0,
      "aria-expanded": gt ? "true" : "false",
      "aria-haspopup": "listbox",
      "aria-label": i,
      "aria-labelledby": [$, se].filter(Boolean).join(" ") || void 0,
      "aria-describedby": o,
      "aria-required": W ? "true" : void 0,
      "aria-invalid": y ? "true" : void 0,
      onKeyDown: Se,
      onMouseDown: g || O ? null : Xe,
      onBlur: yt,
      onFocus: j,
      ...q,
      ownerState: ae,
      className: Ve(q.className, be.select, c),
      id: se,
      children: SQ(Le) ? (
        // notranslate needed while Google Translate will not fix zero-width space issue
        hw || (hw = /* @__PURE__ */ A.jsx("span", {
          className: "notranslate",
          "aria-hidden": !0,
          children: "​"
        }))
      ) : Le
    }), /* @__PURE__ */ A.jsx(bQ, {
      "aria-invalid": y,
      value: Array.isArray(re) ? re.join(",") : re,
      name: _,
      ref: G,
      "aria-hidden": !0,
      onChange: st,
      tabIndex: -1,
      disabled: g,
      className: be.nativeInput,
      autoFocus: s,
      required: W,
      ...ie,
      ownerState: ae
    }), /* @__PURE__ */ A.jsx(vQ, {
      as: v,
      className: be.icon,
      ownerState: ae
    }), /* @__PURE__ */ A.jsx(cQ, {
      id: `menu-${_ || ""}`,
      anchorEl: We,
      open: gt,
      onClose: ft,
      anchorOrigin: {
        vertical: "bottom",
        horizontal: "center"
      },
      transformOrigin: {
        vertical: "top",
        horizontal: "center"
      },
      ...S,
      MenuListProps: {
        "aria-labelledby": $,
        role: "listbox",
        "aria-multiselectable": w ? "true" : void 0,
        disableListWrap: !0,
        id: Ge,
        ...S.MenuListProps
      },
      slotProps: {
        ...S.slotProps,
        paper: {
          ...Re,
          style: {
            minWidth: Ye,
            ...Re != null ? Re.style : null
          }
        }
      },
      children: Me
    })]
  });
}), CQ = Cr(/* @__PURE__ */ A.jsx("path", {
  d: "M7 10l5 5 5-5z"
}), "ArrowDropDown"), xQ = (e) => {
  const {
    classes: t
  } = e;
  return t;
}, O0 = {
  name: "MuiSelect",
  overridesResolver: (e, t) => t.root,
  shouldForwardProp: (e) => ur(e) && e !== "variant",
  slot: "Root"
}, EQ = Oe(C0, O0)(""), _Q = Oe(E0, O0)(""), kQ = Oe(x0, O0)(""), AE = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    name: "MuiSelect",
    props: t
  }), {
    autoWidth: i = !1,
    children: s,
    classes: l = {},
    className: u,
    defaultOpen: c = !1,
    displayEmpty: f = !1,
    IconComponent: p = CQ,
    id: g,
    input: h,
    inputProps: y,
    label: v,
    labelId: C,
    MenuProps: $,
    multiple: S = !1,
    native: w = !1,
    onClose: _,
    onOpen: b,
    open: k,
    renderValue: E,
    SelectDisplayProps: j,
    variant: L = "outlined",
    ...B
  } = o, O = w ? mQ : $Q, V = ti(), W = ca({
    props: o,
    muiFormControl: V,
    states: ["variant", "error"]
  }), q = W.variant || L, Q = {
    ...o,
    variant: q,
    classes: l
  }, X = xQ(Q), {
    root: ce,
    ...J
  } = X, ie = h || {
    standard: /* @__PURE__ */ A.jsx(EQ, {
      ownerState: Q
    }),
    outlined: /* @__PURE__ */ A.jsx(_Q, {
      label: v,
      ownerState: Q
    }),
    filled: /* @__PURE__ */ A.jsx(kQ, {
      ownerState: Q
    })
  }[q], re = _n(n, kc(ie));
  return /* @__PURE__ */ A.jsx(z.Fragment, {
    children: /* @__PURE__ */ z.cloneElement(ie, {
      // Most of the logic is implemented in `SelectInput`.
      // The `Select` component is a simple API wrapper to expose something better to play with.
      inputComponent: O,
      inputProps: {
        children: s,
        error: W.error,
        IconComponent: p,
        variant: q,
        type: void 0,
        // We render a select. We can ignore the type provided by the `Input`.
        multiple: S,
        ...w ? {
          id: g
        } : {
          autoWidth: i,
          defaultOpen: c,
          displayEmpty: f,
          labelId: C,
          MenuProps: $,
          onClose: _,
          onOpen: b,
          open: k,
          renderValue: E,
          SelectDisplayProps: {
            id: g,
            ...j
          }
        },
        ...y,
        classes: y ? lr(J, y.classes) : J,
        ...h ? h.props.inputProps : {}
      },
      ...(S && w || f) && q === "outlined" ? {
        notched: !0
      } : {},
      ref: re,
      className: Ve(ie.props.className, u, X.root),
      // If a custom input is provided via 'input' prop, do not allow 'variant' to be propagated to it's root element. See https://github.com/mui/material-ui/issues/33894.
      ...!h && {
        variant: q
      },
      ...B
    })
  });
});
AE.muiName = "Select";
function PQ(e) {
  return ht("MuiTextField", e);
}
dt("MuiTextField", ["root"]);
const OQ = {
  standard: C0,
  filled: x0,
  outlined: E0
}, AQ = (e) => {
  const {
    classes: t
  } = e;
  return mt({
    root: ["root"]
  }, PQ, t);
}, IQ = Oe(xE, {
  name: "MuiTextField",
  slot: "Root",
  overridesResolver: (e, t) => t.root
})({}), A0 = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiTextField"
  }), {
    autoComplete: i,
    autoFocus: s = !1,
    children: l,
    className: u,
    color: c = "primary",
    defaultValue: f,
    disabled: p = !1,
    error: g = !1,
    FormHelperTextProps: h,
    fullWidth: y = !1,
    helperText: v,
    id: C,
    InputLabelProps: $,
    inputProps: S,
    InputProps: w,
    inputRef: _,
    label: b,
    maxRows: k,
    minRows: E,
    multiline: j = !1,
    name: L,
    onBlur: B,
    onChange: O,
    onFocus: V,
    placeholder: W,
    required: q = !1,
    rows: Q,
    select: X = !1,
    SelectProps: ce,
    slots: J = {},
    slotProps: ie = {},
    type: re,
    value: Y,
    variant: T = "outlined",
    ...H
  } = o, G = {
    ...o,
    autoFocus: s,
    color: c,
    disabled: p,
    error: g,
    fullWidth: y,
    multiline: j,
    required: q,
    select: X,
    variant: T
  }, N = AQ(G), U = t0(C), Z = v && U ? `${U}-helper-text` : void 0, me = b && U ? `${U}-label` : void 0, ue = OQ[T], Ce = {
    slots: J,
    slotProps: {
      input: w,
      inputLabel: $,
      htmlInput: S,
      formHelperText: h,
      select: ce,
      ...ie
    }
  }, $e = {}, Te = Ce.slotProps.inputLabel;
  T === "outlined" && (Te && typeof Te.shrink < "u" && ($e.notched = Te.shrink), $e.label = b), X && ((!ce || !ce.native) && ($e.id = void 0), $e["aria-describedby"] = void 0);
  const [We, et] = Tr("input", {
    elementType: ue,
    externalForwardedProps: Ce,
    additionalProps: $e,
    ownerState: G
  }), [Xe, ft] = Tr("inputLabel", {
    elementType: pY,
    externalForwardedProps: Ce,
    ownerState: G
  }), [Lt, st] = Tr("htmlInput", {
    elementType: "input",
    externalForwardedProps: Ce,
    ownerState: G
  }), [pt, Se] = Tr("formHelperText", {
    elementType: _0,
    externalForwardedProps: Ce,
    ownerState: G
  }), [gt, yt] = Tr("select", {
    elementType: AE,
    externalForwardedProps: Ce,
    ownerState: G
  }), Le = /* @__PURE__ */ A.jsx(We, {
    "aria-describedby": Z,
    autoComplete: i,
    autoFocus: s,
    defaultValue: f,
    fullWidth: y,
    multiline: j,
    name: L,
    rows: Q,
    maxRows: k,
    minRows: E,
    type: re,
    value: Y,
    id: U,
    inputRef: _,
    onBlur: B,
    onChange: O,
    onFocus: V,
    placeholder: W,
    inputProps: st,
    slots: {
      input: J.htmlInput ? Lt : void 0
    },
    ...et
  });
  return /* @__PURE__ */ A.jsxs(IQ, {
    className: Ve(N.root, u),
    disabled: p,
    error: g,
    fullWidth: y,
    ref: n,
    required: q,
    color: c,
    variant: T,
    ownerState: G,
    ...H,
    children: [b != null && b !== "" && /* @__PURE__ */ A.jsx(Xe, {
      htmlFor: U,
      id: me,
      ...ft,
      children: b
    }), X ? /* @__PURE__ */ A.jsx(gt, {
      "aria-describedby": Z,
      id: U,
      labelId: me,
      value: Y,
      input: Le,
      ...yt,
      children: l
    }) : Le, v && /* @__PURE__ */ A.jsx(pt, {
      id: Z,
      ...Se,
      children: v
    })]
  });
}), jQ = ["date", "datetime-local", "file", "time"];
function RQ(e) {
  const {
    id: t,
    name: n,
    // remove this from textFieldProps
    placeholder: o,
    required: i,
    readonly: s,
    disabled: l,
    type: u,
    label: c,
    hideLabel: f,
    hideError: p,
    value: g,
    onChange: h,
    onChangeOverride: y,
    onBlur: v,
    onFocus: C,
    autofocus: $,
    options: S,
    schema: w,
    uiSchema: _,
    rawErrors: b = [],
    errorSchema: k,
    formContext: E,
    registry: j,
    InputLabelProps: L,
    ...B
  } = e, O = ux(w, u, S), { step: V, min: W, max: q, ...Q } = O, X = {
    inputProps: {
      step: V,
      min: W,
      max: q,
      ...w.examples ? { list: oc(t) } : void 0
    },
    ...Q
  }, ce = ({ target: { value: Y } }) => h(Y === "" ? S.emptyValue : Y), J = ({ target: Y }) => v(t, Y && Y.value), ie = ({ target: Y }) => C(t, Y && Y.value), re = jQ.includes(u) ? {
    ...L,
    shrink: !0
  } : L;
  return A.jsxs(A.Fragment, { children: [A.jsx(A0, { id: t, name: t, placeholder: o, label: ts(c || void 0, f, void 0), autoFocus: $, required: i, disabled: l || s, ...X, value: g || g === 0 ? g : "", error: b.length > 0, onChange: y || ce, onBlur: J, onFocus: ie, InputLabelProps: re, ...B, "aria-describedby": Gr(t, !!w.examples) }), Array.isArray(w.examples) && A.jsx("datalist", { id: oc(t), children: w.examples.concat(w.default && !w.examples.includes(w.default) ? [w.default] : []).map((Y) => A.jsx("option", { value: Y }, Y)) })] });
}
function TQ(e) {
  return ht("MuiTypography", e);
}
const gw = dt("MuiTypography", ["root", "h1", "h2", "h3", "h4", "h5", "h6", "subtitle1", "subtitle2", "body1", "body2", "inherit", "button", "caption", "overline", "alignLeft", "alignRight", "alignCenter", "alignJustify", "noWrap", "gutterBottom", "paragraph"]), MQ = {
  primary: !0,
  secondary: !0,
  error: !0,
  info: !0,
  success: !0,
  warning: !0,
  textPrimary: !0,
  textSecondary: !0,
  textDisabled: !0
}, NQ = zG(), FQ = (e) => {
  const {
    align: t,
    gutterBottom: n,
    noWrap: o,
    paragraph: i,
    variant: s,
    classes: l
  } = e, u = {
    root: ["root", s, e.align !== "inherit" && `align${Ne(t)}`, n && "gutterBottom", o && "noWrap", i && "paragraph"]
  };
  return mt(u, TQ, l);
}, LQ = Oe("span", {
  name: "MuiTypography",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, n.variant && t[n.variant], n.align !== "inherit" && t[`align${Ne(n.align)}`], n.noWrap && t.noWrap, n.gutterBottom && t.gutterBottom, n.paragraph && t.paragraph];
  }
})(ct(({
  theme: e
}) => {
  var t;
  return {
    margin: 0,
    variants: [{
      props: {
        variant: "inherit"
      },
      style: {
        // Some elements, like <button> on Chrome have default font that doesn't inherit, reset this.
        font: "inherit",
        lineHeight: "inherit",
        letterSpacing: "inherit"
      }
    }, ...Object.entries(e.typography).filter(([n, o]) => n !== "inherit" && o && typeof o == "object").map(([n, o]) => ({
      props: {
        variant: n
      },
      style: o
    })), ...Object.entries(e.palette).filter(kn()).map(([n]) => ({
      props: {
        color: n
      },
      style: {
        color: (e.vars || e).palette[n].main
      }
    })), ...Object.entries(((t = e.palette) == null ? void 0 : t.text) || {}).filter(([, n]) => typeof n == "string").map(([n]) => ({
      props: {
        color: `text${Ne(n)}`
      },
      style: {
        color: (e.vars || e).palette.text[n]
      }
    })), {
      props: ({
        ownerState: n
      }) => n.align !== "inherit",
      style: {
        textAlign: "var(--Typography-textAlign)"
      }
    }, {
      props: ({
        ownerState: n
      }) => n.noWrap,
      style: {
        overflow: "hidden",
        textOverflow: "ellipsis",
        whiteSpace: "nowrap"
      }
    }, {
      props: ({
        ownerState: n
      }) => n.gutterBottom,
      style: {
        marginBottom: "0.35em"
      }
    }, {
      props: ({
        ownerState: n
      }) => n.paragraph,
      style: {
        marginBottom: 16
      }
    }]
  };
})), yw = {
  h1: "h1",
  h2: "h2",
  h3: "h3",
  h4: "h4",
  h5: "h5",
  h6: "h6",
  subtitle1: "h6",
  subtitle2: "h6",
  body1: "p",
  body2: "p",
  inherit: "p"
}, sr = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const {
    color: o,
    ...i
  } = bt({
    props: t,
    name: "MuiTypography"
  }), s = !MQ[o], l = NQ({
    ...i,
    ...s && {
      color: o
    }
  }), {
    align: u = "inherit",
    className: c,
    component: f,
    gutterBottom: p = !1,
    noWrap: g = !1,
    paragraph: h = !1,
    variant: y = "body1",
    variantMapping: v = yw,
    ...C
  } = l, $ = {
    ...l,
    align: u,
    color: o,
    className: c,
    component: f,
    gutterBottom: p,
    noWrap: g,
    paragraph: h,
    variant: y,
    variantMapping: v
  }, S = f || (h ? "p" : v[y] || yw[y]) || "span", w = FQ($);
  return /* @__PURE__ */ A.jsx(LQ, {
    as: S,
    ref: n,
    className: Ve(w.root, c),
    ...C,
    ownerState: $,
    style: {
      ...u !== "inherit" && {
        "--Typography-textAlign": u
      },
      ...C.style
    }
  });
});
function DQ(e) {
  const { id: t, description: n } = e;
  return n ? A.jsx(sr, { id: t, variant: "subtitle2", style: { marginTop: "5px" }, children: n }) : null;
}
const zQ = Cr(/* @__PURE__ */ A.jsx("path", {
  d: "M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2m1 15h-2v-2h2zm0-4h-2V7h2z"
}), "Error");
function BQ(e) {
  return ht("MuiListItem", e);
}
dt("MuiListItem", ["root", "container", "dense", "alignItemsFlexStart", "divider", "gutters", "padding", "secondaryAction"]);
const UQ = dt("MuiListItemButton", ["root", "focusVisible", "dense", "alignItemsFlexStart", "disabled", "divider", "gutters", "selected"]);
function VQ(e) {
  return ht("MuiListItemSecondaryAction", e);
}
dt("MuiListItemSecondaryAction", ["root", "disableGutters"]);
const WQ = (e) => {
  const {
    disableGutters: t,
    classes: n
  } = e;
  return mt({
    root: ["root", t && "disableGutters"]
  }, VQ, n);
}, qQ = Oe("div", {
  name: "MuiListItemSecondaryAction",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, n.disableGutters && t.disableGutters];
  }
})({
  position: "absolute",
  right: 16,
  top: "50%",
  transform: "translateY(-50%)",
  variants: [{
    props: ({
      ownerState: e
    }) => e.disableGutters,
    style: {
      right: 0
    }
  }]
}), IE = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiListItemSecondaryAction"
  }), {
    className: i,
    ...s
  } = o, l = z.useContext(bi), u = {
    ...o,
    disableGutters: l.disableGutters
  }, c = WQ(u);
  return /* @__PURE__ */ A.jsx(qQ, {
    className: Ve(c.root, i),
    ownerState: u,
    ref: n,
    ...s
  });
});
IE.muiName = "ListItemSecondaryAction";
const KQ = (e, t) => {
  const {
    ownerState: n
  } = e;
  return [t.root, n.dense && t.dense, n.alignItems === "flex-start" && t.alignItemsFlexStart, n.divider && t.divider, !n.disableGutters && t.gutters, !n.disablePadding && t.padding, n.hasSecondaryAction && t.secondaryAction];
}, HQ = (e) => {
  const {
    alignItems: t,
    classes: n,
    dense: o,
    disableGutters: i,
    disablePadding: s,
    divider: l,
    hasSecondaryAction: u
  } = e;
  return mt({
    root: ["root", o && "dense", !i && "gutters", !s && "padding", l && "divider", t === "flex-start" && "alignItemsFlexStart", u && "secondaryAction"],
    container: ["container"]
  }, BQ, n);
}, GQ = Oe("div", {
  name: "MuiListItem",
  slot: "Root",
  overridesResolver: KQ
})(ct(({
  theme: e
}) => ({
  display: "flex",
  justifyContent: "flex-start",
  alignItems: "center",
  position: "relative",
  textDecoration: "none",
  width: "100%",
  boxSizing: "border-box",
  textAlign: "left",
  variants: [{
    props: ({
      ownerState: t
    }) => !t.disablePadding,
    style: {
      paddingTop: 8,
      paddingBottom: 8
    }
  }, {
    props: ({
      ownerState: t
    }) => !t.disablePadding && t.dense,
    style: {
      paddingTop: 4,
      paddingBottom: 4
    }
  }, {
    props: ({
      ownerState: t
    }) => !t.disablePadding && !t.disableGutters,
    style: {
      paddingLeft: 16,
      paddingRight: 16
    }
  }, {
    props: ({
      ownerState: t
    }) => !t.disablePadding && !!t.secondaryAction,
    style: {
      // Add some space to avoid collision as `ListItemSecondaryAction`
      // is absolutely positioned.
      paddingRight: 48
    }
  }, {
    props: ({
      ownerState: t
    }) => !!t.secondaryAction,
    style: {
      [`& > .${UQ.root}`]: {
        paddingRight: 48
      }
    }
  }, {
    props: {
      alignItems: "flex-start"
    },
    style: {
      alignItems: "flex-start"
    }
  }, {
    props: ({
      ownerState: t
    }) => t.divider,
    style: {
      borderBottom: `1px solid ${(e.vars || e).palette.divider}`,
      backgroundClip: "padding-box"
    }
  }, {
    props: ({
      ownerState: t
    }) => t.button,
    style: {
      transition: e.transitions.create("background-color", {
        duration: e.transitions.duration.shortest
      }),
      "&:hover": {
        textDecoration: "none",
        backgroundColor: (e.vars || e).palette.action.hover,
        // Reset on touch devices, it doesn't add specificity
        "@media (hover: none)": {
          backgroundColor: "transparent"
        }
      }
    }
  }, {
    props: ({
      ownerState: t
    }) => t.hasSecondaryAction,
    style: {
      // Add some space to avoid collision as `ListItemSecondaryAction`
      // is absolutely positioned.
      paddingRight: 48
    }
  }]
}))), JQ = Oe("li", {
  name: "MuiListItem",
  slot: "Container",
  overridesResolver: (e, t) => t.container
})({
  position: "relative"
}), jE = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiListItem"
  }), {
    alignItems: i = "center",
    children: s,
    className: l,
    component: u,
    components: c = {},
    componentsProps: f = {},
    ContainerComponent: p = "li",
    ContainerProps: {
      className: g,
      ...h
    } = {},
    dense: y = !1,
    disableGutters: v = !1,
    disablePadding: C = !1,
    divider: $ = !1,
    secondaryAction: S,
    slotProps: w = {},
    slots: _ = {},
    ...b
  } = o, k = z.useContext(bi), E = z.useMemo(() => ({
    dense: y || k.dense || !1,
    alignItems: i,
    disableGutters: v
  }), [i, k.dense, y, v]), j = z.useRef(null), L = z.Children.toArray(s), B = L.length && Vu(L[L.length - 1], ["ListItemSecondaryAction"]), O = {
    ...o,
    alignItems: i,
    dense: E.dense,
    disableGutters: v,
    disablePadding: C,
    divider: $,
    hasSecondaryAction: B
  }, V = HQ(O), W = _n(j, n), q = _.root || c.Root || GQ, Q = w.root || f.root || {}, X = {
    className: Ve(V.root, Q.className, l),
    ...b
  };
  let ce = u || "li";
  return B ? (ce = !X.component && !u ? "div" : ce, p === "li" && (ce === "li" ? ce = "div" : X.component === "li" && (X.component = "div")), /* @__PURE__ */ A.jsx(bi.Provider, {
    value: E,
    children: /* @__PURE__ */ A.jsxs(JQ, {
      as: p,
      className: Ve(V.container, g),
      ref: W,
      ownerState: O,
      ...h,
      children: [/* @__PURE__ */ A.jsx(q, {
        ...Q,
        ...!vi(q) && {
          as: ce,
          ownerState: {
            ...O,
            ...Q.ownerState
          }
        },
        ...X,
        children: L
      }), L.pop()]
    })
  })) : /* @__PURE__ */ A.jsx(bi.Provider, {
    value: E,
    children: /* @__PURE__ */ A.jsxs(q, {
      ...Q,
      as: ce,
      ref: W,
      ...!vi(q) && {
        ownerState: {
          ...O,
          ...Q.ownerState
        }
      },
      ...X,
      children: [L, S && /* @__PURE__ */ A.jsx(IE, {
        children: S
      })]
    })
  });
});
function YQ(e) {
  return ht("MuiListItemIcon", e);
}
const vw = dt("MuiListItemIcon", ["root", "alignItemsFlexStart"]), QQ = (e) => {
  const {
    alignItems: t,
    classes: n
  } = e;
  return mt({
    root: ["root", t === "flex-start" && "alignItemsFlexStart"]
  }, YQ, n);
}, XQ = Oe("div", {
  name: "MuiListItemIcon",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, n.alignItems === "flex-start" && t.alignItemsFlexStart];
  }
})(ct(({
  theme: e
}) => ({
  minWidth: 56,
  color: (e.vars || e).palette.action.active,
  flexShrink: 0,
  display: "inline-flex",
  variants: [{
    props: {
      alignItems: "flex-start"
    },
    style: {
      marginTop: 8
    }
  }]
}))), ZQ = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiListItemIcon"
  }), {
    className: i,
    ...s
  } = o, l = z.useContext(bi), u = {
    ...o,
    alignItems: l.alignItems
  }, c = QQ(u);
  return /* @__PURE__ */ A.jsx(XQ, {
    className: Ve(c.root, i),
    ownerState: u,
    ref: n,
    ...s
  });
});
function eX(e) {
  return ht("MuiListItemText", e);
}
const Gs = dt("MuiListItemText", ["root", "multiline", "dense", "inset", "primary", "secondary"]), tX = (e) => {
  const {
    classes: t,
    inset: n,
    primary: o,
    secondary: i,
    dense: s
  } = e;
  return mt({
    root: ["root", n && "inset", s && "dense", o && i && "multiline"],
    primary: ["primary"],
    secondary: ["secondary"]
  }, eX, t);
}, nX = Oe("div", {
  name: "MuiListItemText",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [{
      [`& .${Gs.primary}`]: t.primary
    }, {
      [`& .${Gs.secondary}`]: t.secondary
    }, t.root, n.inset && t.inset, n.primary && n.secondary && t.multiline, n.dense && t.dense];
  }
})({
  flex: "1 1 auto",
  minWidth: 0,
  marginTop: 4,
  marginBottom: 4,
  [`.${gw.root}:where(& .${Gs.primary})`]: {
    display: "block"
  },
  [`.${gw.root}:where(& .${Gs.secondary})`]: {
    display: "block"
  },
  variants: [{
    props: ({
      ownerState: e
    }) => e.primary && e.secondary,
    style: {
      marginTop: 6,
      marginBottom: 6
    }
  }, {
    props: ({
      ownerState: e
    }) => e.inset,
    style: {
      paddingLeft: 56
    }
  }]
}), rX = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiListItemText"
  }), {
    children: i,
    className: s,
    disableTypography: l = !1,
    inset: u = !1,
    primary: c,
    primaryTypographyProps: f,
    secondary: p,
    secondaryTypographyProps: g,
    slots: h = {},
    slotProps: y = {},
    ...v
  } = o, {
    dense: C
  } = z.useContext(bi);
  let $ = c ?? i, S = p;
  const w = {
    ...o,
    disableTypography: l,
    inset: u,
    primary: !!$,
    secondary: !!S,
    dense: C
  }, _ = tX(w), b = {
    slots: h,
    slotProps: {
      primary: f,
      secondary: g,
      ...y
    }
  }, [k, E] = Tr("primary", {
    className: _.primary,
    elementType: sr,
    externalForwardedProps: b,
    ownerState: w
  }), [j, L] = Tr("secondary", {
    className: _.secondary,
    elementType: sr,
    externalForwardedProps: b,
    ownerState: w
  });
  return $ != null && $.type !== sr && !l && ($ = /* @__PURE__ */ A.jsx(k, {
    variant: C ? "body2" : "body1",
    component: E != null && E.variant ? void 0 : "span",
    ...E,
    children: $
  })), S != null && S.type !== sr && !l && (S = /* @__PURE__ */ A.jsx(j, {
    variant: "body2",
    color: "textSecondary",
    ...L,
    children: S
  })), /* @__PURE__ */ A.jsxs(nX, {
    className: Ve(_.root, s),
    ownerState: w,
    ref: n,
    ...v,
    children: [$, S]
  });
});
function oX({ errors: e, registry: t }) {
  const { translateString: n } = t;
  return A.jsx(Ml, { elevation: 2, children: A.jsxs(wr, { mb: 2, p: 2, children: [A.jsx(sr, { variant: "h6", children: n(jt.ErrorsLabel) }), A.jsx(k0, { dense: !0, children: e.map((o, i) => A.jsxs(jE, { children: [A.jsx(ZQ, { children: A.jsx(zQ, { color: "error" }) }), A.jsx(rX, { primary: o.stack })] }, i)) })] }) });
}
const iX = Cr(/* @__PURE__ */ A.jsx("path", {
  d: "m20 12-1.41-1.41L13 16.17V4h-2v12.17l-5.58-5.59L4 12l8 8z"
}), "ArrowDownward"), aX = Cr(/* @__PURE__ */ A.jsx("path", {
  d: "m4 12 1.41 1.41L11 7.83V20h2V7.83l5.58 5.59L20 12l-8-8z"
}), "ArrowUpward"), sX = Cr(/* @__PURE__ */ A.jsx("path", {
  d: "M16 1H4c-1.1 0-2 .9-2 2v14h2V3h12zm3 4H8c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h11c1.1 0 2-.9 2-2V7c0-1.1-.9-2-2-2m0 16H8V7h11z"
}), "ContentCopy"), lX = Cr(/* @__PURE__ */ A.jsx("path", {
  d: "M19 13H5v-2h14z"
}), "Remove");
function qp(e) {
  const { icon: t, color: n, uiSchema: o, registry: i, ...s } = e;
  return A.jsx($E, { ...s, size: "small", color: n, children: t });
}
function uX(e) {
  const { registry: { translateString: t } } = e;
  return A.jsx(qp, { title: t(jt.CopyButton), ...e, icon: A.jsx(sX, { fontSize: "small" }) });
}
function cX(e) {
  const { registry: { translateString: t } } = e;
  return A.jsx(qp, { title: t(jt.MoveDownButton), ...e, icon: A.jsx(iX, { fontSize: "small" }) });
}
function dX(e) {
  const { registry: { translateString: t } } = e;
  return A.jsx(qp, { title: t(jt.MoveUpButton), ...e, icon: A.jsx(aX, { fontSize: "small" }) });
}
function fX(e) {
  const { iconType: t, ...n } = e, { registry: { translateString: o } } = n;
  return A.jsx(qp, { title: o(jt.RemoveButton), ...n, color: "error", icon: A.jsx(lX, { fontSize: t === "default" ? void 0 : "small" }) });
}
function pX(e) {
  const { errors: t = [], idSchema: n } = e;
  if (t.length === 0)
    return null;
  const o = Jy(n);
  return A.jsx(k0, { id: o, dense: !0, disablePadding: !0, children: t.map((i, s) => A.jsx(jE, { disableGutters: !0, children: A.jsx(_0, { component: "div", id: `${o}-${s}`, children: i }) }, s)) });
}
function hX(e) {
  const { idSchema: t, help: n } = e;
  if (!n)
    return null;
  const o = Yy(t);
  return A.jsx(_0, { component: "div", id: o, children: n });
}
function mX(e) {
  const { id: t, children: n, classNames: o, style: i, disabled: s, displayLabel: l, hidden: u, label: c, onDropPropertyClick: f, onKeyChange: p, readonly: g, required: h, rawErrors: y = [], errors: v, help: C, description: $, rawDescription: S, schema: w, uiSchema: _, registry: b } = e, k = Ot(_), E = _t("WrapIfAdditionalTemplate", b, k);
  return u ? A.jsx("div", { style: { display: "none" }, children: n }) : A.jsx(E, { classNames: o, style: i, disabled: s, id: t, label: c, onDropPropertyClick: f, onKeyChange: p, readonly: g, required: h, schema: w, uiSchema: _, registry: b, children: A.jsxs(xE, { fullWidth: !0, error: !!y.length, required: h, children: [n, l && S ? A.jsx(sr, { variant: "caption", color: "textSecondary", children: $ }) : null, v, C] }) });
}
function gX(e) {
  const { description: t, title: n, properties: o, required: i, disabled: s, readonly: l, uiSchema: u, idSchema: c, schema: f, formData: p, onAddClick: g, registry: h } = e, y = Ot(u), v = _t("TitleFieldTemplate", h, y), C = _t("DescriptionFieldTemplate", h, y), { ButtonTemplates: { AddButton: $ } } = h.templates;
  return A.jsxs(A.Fragment, { children: [n && A.jsx(v, { id: Qy(c), title: n, required: i, schema: f, uiSchema: u, registry: h }), t && A.jsx(C, { id: es(c), description: t, schema: f, uiSchema: u, registry: h }), A.jsxs(Kr, { container: !0, spacing: 2, style: { marginTop: "10px" }, children: [o.map((S, w) => (
    // Remove the <Grid> if the inner element is hidden as the <Grid>
    // itself would otherwise still take up space.
    S.hidden ? S.content : A.jsx(Kr, { item: !0, xs: 12, style: { marginBottom: "10px" }, children: S.content }, w)
  )), m$(f, u, p) && A.jsx(Kr, { container: !0, justifyContent: "flex-end", children: A.jsx(Kr, { item: !0, children: A.jsx($, { className: "object-property-expand", onClick: g(f), disabled: s || l, uiSchema: u, registry: h }) }) })] })] });
}
function yX(e) {
  return ht("MuiButton", e);
}
const Ls = dt("MuiButton", ["root", "text", "textInherit", "textPrimary", "textSecondary", "textSuccess", "textError", "textInfo", "textWarning", "outlined", "outlinedInherit", "outlinedPrimary", "outlinedSecondary", "outlinedSuccess", "outlinedError", "outlinedInfo", "outlinedWarning", "contained", "containedInherit", "containedPrimary", "containedSecondary", "containedSuccess", "containedError", "containedInfo", "containedWarning", "disableElevation", "focusVisible", "disabled", "colorInherit", "colorPrimary", "colorSecondary", "colorSuccess", "colorError", "colorInfo", "colorWarning", "textSizeSmall", "textSizeMedium", "textSizeLarge", "outlinedSizeSmall", "outlinedSizeMedium", "outlinedSizeLarge", "containedSizeSmall", "containedSizeMedium", "containedSizeLarge", "sizeMedium", "sizeSmall", "sizeLarge", "fullWidth", "startIcon", "endIcon", "icon", "iconSizeSmall", "iconSizeMedium", "iconSizeLarge"]), vX = /* @__PURE__ */ z.createContext({}), bX = /* @__PURE__ */ z.createContext(void 0), SX = (e) => {
  const {
    color: t,
    disableElevation: n,
    fullWidth: o,
    size: i,
    variant: s,
    classes: l
  } = e, u = {
    root: ["root", s, `${s}${Ne(t)}`, `size${Ne(i)}`, `${s}Size${Ne(i)}`, `color${Ne(t)}`, n && "disableElevation", o && "fullWidth"],
    label: ["label"],
    startIcon: ["icon", "startIcon", `iconSize${Ne(i)}`],
    endIcon: ["icon", "endIcon", `iconSize${Ne(i)}`]
  }, c = mt(u, yX, l);
  return {
    ...l,
    // forward the focused, disabled, etc. classes to the ButtonBase
    ...c
  };
}, RE = [{
  props: {
    size: "small"
  },
  style: {
    "& > *:nth-of-type(1)": {
      fontSize: 18
    }
  }
}, {
  props: {
    size: "medium"
  },
  style: {
    "& > *:nth-of-type(1)": {
      fontSize: 20
    }
  }
}, {
  props: {
    size: "large"
  },
  style: {
    "& > *:nth-of-type(1)": {
      fontSize: 22
    }
  }
}], wX = Oe(Tc, {
  shouldForwardProp: (e) => ur(e) || e === "classes",
  name: "MuiButton",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, t[n.variant], t[`${n.variant}${Ne(n.color)}`], t[`size${Ne(n.size)}`], t[`${n.variant}Size${Ne(n.size)}`], n.color === "inherit" && t.colorInherit, n.disableElevation && t.disableElevation, n.fullWidth && t.fullWidth];
  }
})(ct(({
  theme: e
}) => {
  const t = e.palette.mode === "light" ? e.palette.grey[300] : e.palette.grey[800], n = e.palette.mode === "light" ? e.palette.grey.A100 : e.palette.grey[700];
  return {
    ...e.typography.button,
    minWidth: 64,
    padding: "6px 16px",
    border: 0,
    borderRadius: (e.vars || e).shape.borderRadius,
    transition: e.transitions.create(["background-color", "box-shadow", "border-color", "color"], {
      duration: e.transitions.duration.short
    }),
    "&:hover": {
      textDecoration: "none"
    },
    [`&.${Ls.disabled}`]: {
      color: (e.vars || e).palette.action.disabled
    },
    variants: [{
      props: {
        variant: "contained"
      },
      style: {
        color: "var(--variant-containedColor)",
        backgroundColor: "var(--variant-containedBg)",
        boxShadow: (e.vars || e).shadows[2],
        "&:hover": {
          boxShadow: (e.vars || e).shadows[4],
          // Reset on touch devices, it doesn't add specificity
          "@media (hover: none)": {
            boxShadow: (e.vars || e).shadows[2]
          }
        },
        "&:active": {
          boxShadow: (e.vars || e).shadows[8]
        },
        [`&.${Ls.focusVisible}`]: {
          boxShadow: (e.vars || e).shadows[6]
        },
        [`&.${Ls.disabled}`]: {
          color: (e.vars || e).palette.action.disabled,
          boxShadow: (e.vars || e).shadows[0],
          backgroundColor: (e.vars || e).palette.action.disabledBackground
        }
      }
    }, {
      props: {
        variant: "outlined"
      },
      style: {
        padding: "5px 15px",
        border: "1px solid currentColor",
        borderColor: "var(--variant-outlinedBorder, currentColor)",
        backgroundColor: "var(--variant-outlinedBg)",
        color: "var(--variant-outlinedColor)",
        [`&.${Ls.disabled}`]: {
          border: `1px solid ${(e.vars || e).palette.action.disabledBackground}`
        }
      }
    }, {
      props: {
        variant: "text"
      },
      style: {
        padding: "6px 8px",
        color: "var(--variant-textColor)",
        backgroundColor: "var(--variant-textBg)"
      }
    }, ...Object.entries(e.palette).filter(kn()).map(([o]) => ({
      props: {
        color: o
      },
      style: {
        "--variant-textColor": (e.vars || e).palette[o].main,
        "--variant-outlinedColor": (e.vars || e).palette[o].main,
        "--variant-outlinedBorder": e.vars ? `rgba(${e.vars.palette[o].mainChannel} / 0.5)` : En(e.palette[o].main, 0.5),
        "--variant-containedColor": (e.vars || e).palette[o].contrastText,
        "--variant-containedBg": (e.vars || e).palette[o].main,
        "@media (hover: hover)": {
          "&:hover": {
            "--variant-containedBg": (e.vars || e).palette[o].dark,
            "--variant-textBg": e.vars ? `rgba(${e.vars.palette[o].mainChannel} / ${e.vars.palette.action.hoverOpacity})` : En(e.palette[o].main, e.palette.action.hoverOpacity),
            "--variant-outlinedBorder": (e.vars || e).palette[o].main,
            "--variant-outlinedBg": e.vars ? `rgba(${e.vars.palette[o].mainChannel} / ${e.vars.palette.action.hoverOpacity})` : En(e.palette[o].main, e.palette.action.hoverOpacity)
          }
        }
      }
    })), {
      props: {
        color: "inherit"
      },
      style: {
        color: "inherit",
        borderColor: "currentColor",
        "--variant-containedBg": e.vars ? e.vars.palette.Button.inheritContainedBg : t,
        "@media (hover: hover)": {
          "&:hover": {
            "--variant-containedBg": e.vars ? e.vars.palette.Button.inheritContainedHoverBg : n,
            "--variant-textBg": e.vars ? `rgba(${e.vars.palette.text.primaryChannel} / ${e.vars.palette.action.hoverOpacity})` : En(e.palette.text.primary, e.palette.action.hoverOpacity),
            "--variant-outlinedBg": e.vars ? `rgba(${e.vars.palette.text.primaryChannel} / ${e.vars.palette.action.hoverOpacity})` : En(e.palette.text.primary, e.palette.action.hoverOpacity)
          }
        }
      }
    }, {
      props: {
        size: "small",
        variant: "text"
      },
      style: {
        padding: "4px 5px",
        fontSize: e.typography.pxToRem(13)
      }
    }, {
      props: {
        size: "large",
        variant: "text"
      },
      style: {
        padding: "8px 11px",
        fontSize: e.typography.pxToRem(15)
      }
    }, {
      props: {
        size: "small",
        variant: "outlined"
      },
      style: {
        padding: "3px 9px",
        fontSize: e.typography.pxToRem(13)
      }
    }, {
      props: {
        size: "large",
        variant: "outlined"
      },
      style: {
        padding: "7px 21px",
        fontSize: e.typography.pxToRem(15)
      }
    }, {
      props: {
        size: "small",
        variant: "contained"
      },
      style: {
        padding: "4px 10px",
        fontSize: e.typography.pxToRem(13)
      }
    }, {
      props: {
        size: "large",
        variant: "contained"
      },
      style: {
        padding: "8px 22px",
        fontSize: e.typography.pxToRem(15)
      }
    }, {
      props: {
        disableElevation: !0
      },
      style: {
        boxShadow: "none",
        "&:hover": {
          boxShadow: "none"
        },
        [`&.${Ls.focusVisible}`]: {
          boxShadow: "none"
        },
        "&:active": {
          boxShadow: "none"
        },
        [`&.${Ls.disabled}`]: {
          boxShadow: "none"
        }
      }
    }, {
      props: {
        fullWidth: !0
      },
      style: {
        width: "100%"
      }
    }]
  };
})), $X = Oe("span", {
  name: "MuiButton",
  slot: "StartIcon",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.startIcon, t[`iconSize${Ne(n.size)}`]];
  }
})({
  display: "inherit",
  marginRight: 8,
  marginLeft: -4,
  variants: [{
    props: {
      size: "small"
    },
    style: {
      marginLeft: -2
    }
  }, ...RE]
}), CX = Oe("span", {
  name: "MuiButton",
  slot: "EndIcon",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.endIcon, t[`iconSize${Ne(n.size)}`]];
  }
})({
  display: "inherit",
  marginRight: -4,
  marginLeft: 8,
  variants: [{
    props: {
      size: "small"
    },
    style: {
      marginRight: -2
    }
  }, ...RE]
}), hc = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = z.useContext(vX), i = z.useContext(bX), s = lc(o, t), l = bt({
    props: s,
    name: "MuiButton"
  }), {
    children: u,
    color: c = "primary",
    component: f = "button",
    className: p,
    disabled: g = !1,
    disableElevation: h = !1,
    disableFocusRipple: y = !1,
    endIcon: v,
    focusVisibleClassName: C,
    fullWidth: $ = !1,
    size: S = "medium",
    startIcon: w,
    type: _,
    variant: b = "text",
    ...k
  } = l, E = {
    ...l,
    color: c,
    component: f,
    disabled: g,
    disableElevation: h,
    disableFocusRipple: y,
    fullWidth: $,
    size: S,
    type: _,
    variant: b
  }, j = SX(E), L = w && /* @__PURE__ */ A.jsx($X, {
    className: j.startIcon,
    ownerState: E,
    children: w
  }), B = v && /* @__PURE__ */ A.jsx(CX, {
    className: j.endIcon,
    ownerState: E,
    children: v
  }), O = i || "";
  return /* @__PURE__ */ A.jsxs(wX, {
    ownerState: E,
    className: Ve(o.className, j.root, p, O),
    component: f,
    disabled: g,
    focusRipple: !y,
    focusVisibleClassName: Ve(j.focusVisible, C),
    ref: n,
    type: _,
    ...k,
    classes: j,
    children: [L, u, B]
  });
});
function xX({ uiSchema: e }) {
  const { submitText: t, norender: n, props: o = {} } = cx(e);
  return n ? null : A.jsx(wr, { marginTop: 3, children: A.jsx(hc, { type: "submit", variant: "contained", color: "primary", ...o, children: t }) });
}
function EX(e) {
  return ht("MuiDivider", e);
}
const bw = dt("MuiDivider", ["root", "absolute", "fullWidth", "inset", "middle", "flexItem", "light", "vertical", "withChildren", "withChildrenVertical", "textAlignRight", "textAlignLeft", "wrapper", "wrapperVertical"]), _X = (e) => {
  const {
    absolute: t,
    children: n,
    classes: o,
    flexItem: i,
    light: s,
    orientation: l,
    textAlign: u,
    variant: c
  } = e;
  return mt({
    root: ["root", t && "absolute", c, s && "light", l === "vertical" && "vertical", i && "flexItem", n && "withChildren", n && l === "vertical" && "withChildrenVertical", u === "right" && l !== "vertical" && "textAlignRight", u === "left" && l !== "vertical" && "textAlignLeft"],
    wrapper: ["wrapper", l === "vertical" && "wrapperVertical"]
  }, EX, o);
}, kX = Oe("div", {
  name: "MuiDivider",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, n.absolute && t.absolute, t[n.variant], n.light && t.light, n.orientation === "vertical" && t.vertical, n.flexItem && t.flexItem, n.children && t.withChildren, n.children && n.orientation === "vertical" && t.withChildrenVertical, n.textAlign === "right" && n.orientation !== "vertical" && t.textAlignRight, n.textAlign === "left" && n.orientation !== "vertical" && t.textAlignLeft];
  }
})(ct(({
  theme: e
}) => ({
  margin: 0,
  // Reset browser default style.
  flexShrink: 0,
  borderWidth: 0,
  borderStyle: "solid",
  borderColor: (e.vars || e).palette.divider,
  borderBottomWidth: "thin",
  variants: [{
    props: {
      absolute: !0
    },
    style: {
      position: "absolute",
      bottom: 0,
      left: 0,
      width: "100%"
    }
  }, {
    props: {
      light: !0
    },
    style: {
      borderColor: e.vars ? `rgba(${e.vars.palette.dividerChannel} / 0.08)` : En(e.palette.divider, 0.08)
    }
  }, {
    props: {
      variant: "inset"
    },
    style: {
      marginLeft: 72
    }
  }, {
    props: {
      variant: "middle",
      orientation: "horizontal"
    },
    style: {
      marginLeft: e.spacing(2),
      marginRight: e.spacing(2)
    }
  }, {
    props: {
      variant: "middle",
      orientation: "vertical"
    },
    style: {
      marginTop: e.spacing(1),
      marginBottom: e.spacing(1)
    }
  }, {
    props: {
      orientation: "vertical"
    },
    style: {
      height: "100%",
      borderBottomWidth: 0,
      borderRightWidth: "thin"
    }
  }, {
    props: {
      flexItem: !0
    },
    style: {
      alignSelf: "stretch",
      height: "auto"
    }
  }, {
    props: ({
      ownerState: t
    }) => !!t.children,
    style: {
      display: "flex",
      textAlign: "center",
      border: 0,
      borderTopStyle: "solid",
      borderLeftStyle: "solid",
      "&::before, &::after": {
        content: '""',
        alignSelf: "center"
      }
    }
  }, {
    props: ({
      ownerState: t
    }) => t.children && t.orientation !== "vertical",
    style: {
      "&::before, &::after": {
        width: "100%",
        borderTop: `thin solid ${(e.vars || e).palette.divider}`,
        borderTopStyle: "inherit"
      }
    }
  }, {
    props: ({
      ownerState: t
    }) => t.orientation === "vertical" && t.children,
    style: {
      flexDirection: "column",
      "&::before, &::after": {
        height: "100%",
        borderLeft: `thin solid ${(e.vars || e).palette.divider}`,
        borderLeftStyle: "inherit"
      }
    }
  }, {
    props: ({
      ownerState: t
    }) => t.textAlign === "right" && t.orientation !== "vertical",
    style: {
      "&::before": {
        width: "90%"
      },
      "&::after": {
        width: "10%"
      }
    }
  }, {
    props: ({
      ownerState: t
    }) => t.textAlign === "left" && t.orientation !== "vertical",
    style: {
      "&::before": {
        width: "10%"
      },
      "&::after": {
        width: "90%"
      }
    }
  }]
}))), PX = Oe("span", {
  name: "MuiDivider",
  slot: "Wrapper",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.wrapper, n.orientation === "vertical" && t.wrapperVertical];
  }
})(ct(({
  theme: e
}) => ({
  display: "inline-block",
  paddingLeft: `calc(${e.spacing(1)} * 1.2)`,
  paddingRight: `calc(${e.spacing(1)} * 1.2)`,
  whiteSpace: "nowrap",
  variants: [{
    props: {
      orientation: "vertical"
    },
    style: {
      paddingTop: `calc(${e.spacing(1)} * 1.2)`,
      paddingBottom: `calc(${e.spacing(1)} * 1.2)`
    }
  }]
}))), zg = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiDivider"
  }), {
    absolute: i = !1,
    children: s,
    className: l,
    orientation: u = "horizontal",
    component: c = s || u === "vertical" ? "div" : "hr",
    flexItem: f = !1,
    light: p = !1,
    role: g = c !== "hr" ? "separator" : void 0,
    textAlign: h = "center",
    variant: y = "fullWidth",
    ...v
  } = o, C = {
    ...o,
    absolute: i,
    component: c,
    flexItem: f,
    light: p,
    orientation: u,
    role: g,
    textAlign: h,
    variant: y
  }, $ = _X(C);
  return /* @__PURE__ */ A.jsx(kX, {
    as: c,
    className: Ve($.root, l),
    role: g,
    ref: n,
    ownerState: C,
    "aria-orientation": g === "separator" && (c !== "hr" || u === "vertical") ? u : void 0,
    ...v,
    children: s ? /* @__PURE__ */ A.jsx(PX, {
      className: $.wrapper,
      ownerState: C,
      children: s
    }) : null
  });
});
zg && (zg.muiSkipListHighlight = !0);
function OX({ id: e, title: t }) {
  return A.jsxs(wr, { id: e, mb: 1, mt: 1, children: [A.jsx(sr, { variant: "h5", children: t }), A.jsx(zg, {})] });
}
function AX(e) {
  const { children: t, classNames: n, style: o, disabled: i, id: s, label: l, onDropPropertyClick: u, onKeyChange: c, readonly: f, required: p, schema: g, uiSchema: h, registry: y } = e, { templates: v, translateString: C } = y, { RemoveButton: $ } = v.ButtonTemplates, S = C(jt.KeyLabel, [l]), w = gl in g, _ = {
    flex: 1,
    paddingLeft: 6,
    paddingRight: 6,
    fontWeight: "bold"
  };
  if (!w)
    return A.jsx("div", { className: n, style: o, children: t });
  const b = ({ target: k }) => c(k && k.value);
  return A.jsxs(Kr, { container: !0, alignItems: "center", spacing: 2, className: n, style: o, children: [A.jsx(Kr, { item: !0, xs: !0, children: A.jsx(A0, { fullWidth: !0, required: p, label: S, defaultValue: l, disabled: i || f, id: `${s}-key`, name: `${s}-key`, onBlur: f ? void 0 : b, type: "text" }) }), A.jsx(Kr, { item: !0, xs: !0, children: t }), A.jsx(Kr, { item: !0, children: A.jsx($, { iconType: "default", style: _, disabled: i || f, onClick: u(l), uiSchema: h, registry: y }) })] }, `${s}-key`);
}
function IX() {
  return {
    ArrayFieldItemTemplate: NJ,
    ArrayFieldTemplate: FJ,
    BaseInputTemplate: RQ,
    ButtonTemplates: {
      AddButton: vJ,
      CopyButton: uX,
      MoveDownButton: cX,
      MoveUpButton: dX,
      RemoveButton: fX,
      SubmitButton: xX
    },
    DescriptionFieldTemplate: DQ,
    ErrorListTemplate: oX,
    FieldErrorTemplate: pX,
    FieldHelpTemplate: hX,
    FieldTemplate: mX,
    ObjectFieldTemplate: gX,
    TitleFieldTemplate: OX,
    WrapIfAdditionalTemplate: AX
  };
}
function jX(e) {
  return ht("PrivateSwitchBase", e);
}
dt("PrivateSwitchBase", ["root", "checked", "disabled", "input", "edgeStart", "edgeEnd"]);
const RX = (e) => {
  const {
    classes: t,
    checked: n,
    disabled: o,
    edge: i
  } = e, s = {
    root: ["root", n && "checked", o && "disabled", i && `edge${Ne(i)}`],
    input: ["input"]
  };
  return mt(s, jX, t);
}, TX = Oe(Tc)({
  padding: 9,
  borderRadius: "50%",
  variants: [{
    props: {
      edge: "start",
      size: "small"
    },
    style: {
      marginLeft: -3
    }
  }, {
    props: ({
      edge: e,
      ownerState: t
    }) => e === "start" && t.size !== "small",
    style: {
      marginLeft: -12
    }
  }, {
    props: {
      edge: "end",
      size: "small"
    },
    style: {
      marginRight: -3
    }
  }, {
    props: ({
      edge: e,
      ownerState: t
    }) => e === "end" && t.size !== "small",
    style: {
      marginRight: -12
    }
  }]
}), MX = Oe("input", {
  shouldForwardProp: ur
})({
  cursor: "inherit",
  position: "absolute",
  opacity: 0,
  width: "100%",
  height: "100%",
  top: 0,
  left: 0,
  margin: 0,
  padding: 0,
  zIndex: 1
}), TE = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const {
    autoFocus: o,
    checked: i,
    checkedIcon: s,
    className: l,
    defaultChecked: u,
    disabled: c,
    disableFocusRipple: f = !1,
    edge: p = !1,
    icon: g,
    id: h,
    inputProps: y,
    inputRef: v,
    name: C,
    onBlur: $,
    onChange: S,
    onFocus: w,
    readOnly: _,
    required: b = !1,
    tabIndex: k,
    type: E,
    value: j,
    ...L
  } = t, [B, O] = sc({
    controlled: i,
    default: !!u,
    name: "SwitchBase",
    state: "checked"
  }), V = ti(), W = (re) => {
    w && w(re), V && V.onFocus && V.onFocus(re);
  }, q = (re) => {
    $ && $(re), V && V.onBlur && V.onBlur(re);
  }, Q = (re) => {
    if (re.nativeEvent.defaultPrevented)
      return;
    const Y = re.target.checked;
    O(Y), S && S(re, Y);
  };
  let X = c;
  V && typeof X > "u" && (X = V.disabled);
  const ce = E === "checkbox" || E === "radio", J = {
    ...t,
    checked: B,
    disabled: X,
    disableFocusRipple: f,
    edge: p
  }, ie = RX(J);
  return /* @__PURE__ */ A.jsxs(TX, {
    component: "span",
    className: Ve(ie.root, l),
    centerRipple: !0,
    focusRipple: !f,
    disabled: X,
    tabIndex: null,
    role: void 0,
    onFocus: W,
    onBlur: q,
    ownerState: J,
    ref: n,
    ...L,
    children: [/* @__PURE__ */ A.jsx(MX, {
      autoFocus: o,
      checked: i,
      defaultChecked: u,
      className: ie.input,
      disabled: X,
      id: ce ? h : void 0,
      name: C,
      onChange: Q,
      readOnly: _,
      ref: v,
      required: b,
      ownerState: J,
      tabIndex: k,
      type: E,
      ...E === "checkbox" && j === void 0 ? {} : {
        value: j
      },
      ...y
    }), B ? s : g]
  });
}), NX = Cr(/* @__PURE__ */ A.jsx("path", {
  d: "M19 5v14H5V5h14m0-2H5c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2z"
}), "CheckBoxOutlineBlank"), FX = Cr(/* @__PURE__ */ A.jsx("path", {
  d: "M19 3H5c-1.11 0-2 .9-2 2v14c0 1.1.89 2 2 2h14c1.11 0 2-.9 2-2V5c0-1.1-.89-2-2-2zm-9 14l-5-5 1.41-1.41L10 14.17l7.59-7.59L19 8l-9 9z"
}), "CheckBox"), LX = Cr(/* @__PURE__ */ A.jsx("path", {
  d: "M19 3H5c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2zm-2 10H7v-2h10v2z"
}), "IndeterminateCheckBox");
function DX(e) {
  return ht("MuiCheckbox", e);
}
const rg = dt("MuiCheckbox", ["root", "checked", "disabled", "indeterminate", "colorPrimary", "colorSecondary", "sizeSmall", "sizeMedium"]), zX = (e) => {
  const {
    classes: t,
    indeterminate: n,
    color: o,
    size: i
  } = e, s = {
    root: ["root", n && "indeterminate", `color${Ne(o)}`, `size${Ne(i)}`]
  }, l = mt(s, DX, t);
  return {
    ...t,
    // forward the disabled and checked classes to the SwitchBase
    ...l
  };
}, BX = Oe(TE, {
  shouldForwardProp: (e) => ur(e) || e === "classes",
  name: "MuiCheckbox",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, n.indeterminate && t.indeterminate, t[`size${Ne(n.size)}`], n.color !== "default" && t[`color${Ne(n.color)}`]];
  }
})(ct(({
  theme: e
}) => ({
  color: (e.vars || e).palette.text.secondary,
  variants: [{
    props: {
      color: "default",
      disableRipple: !1
    },
    style: {
      "&:hover": {
        backgroundColor: e.vars ? `rgba(${e.vars.palette.action.activeChannel} / ${e.vars.palette.action.hoverOpacity})` : En(e.palette.action.active, e.palette.action.hoverOpacity)
      }
    }
  }, ...Object.entries(e.palette).filter(kn()).map(([t]) => ({
    props: {
      color: t,
      disableRipple: !1
    },
    style: {
      "&:hover": {
        backgroundColor: e.vars ? `rgba(${e.vars.palette[t].mainChannel} / ${e.vars.palette.action.hoverOpacity})` : En(e.palette[t].main, e.palette.action.hoverOpacity)
      }
    }
  })), ...Object.entries(e.palette).filter(kn()).map(([t]) => ({
    props: {
      color: t
    },
    style: {
      [`&.${rg.checked}, &.${rg.indeterminate}`]: {
        color: (e.vars || e).palette[t].main
      },
      [`&.${rg.disabled}`]: {
        color: (e.vars || e).palette.action.disabled
      }
    }
  })), {
    // Should be last to override other colors
    props: {
      disableRipple: !1
    },
    style: {
      // Reset on touch devices, it doesn't add specificity
      "&:hover": {
        "@media (hover: none)": {
          backgroundColor: "transparent"
        }
      }
    }
  }]
}))), UX = /* @__PURE__ */ A.jsx(FX, {}), VX = /* @__PURE__ */ A.jsx(NX, {}), WX = /* @__PURE__ */ A.jsx(LX, {}), ME = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiCheckbox"
  }), {
    checkedIcon: i = UX,
    color: s = "primary",
    icon: l = VX,
    indeterminate: u = !1,
    indeterminateIcon: c = WX,
    inputProps: f,
    size: p = "medium",
    disableRipple: g = !1,
    className: h,
    ...y
  } = o, v = u ? c : l, C = u ? c : i, $ = {
    ...o,
    disableRipple: g,
    color: s,
    indeterminate: u,
    size: p
  }, S = zX($);
  return /* @__PURE__ */ A.jsx(BX, {
    type: "checkbox",
    inputProps: {
      "data-indeterminate": u,
      ...f
    },
    icon: /* @__PURE__ */ z.cloneElement(v, {
      fontSize: v.props.fontSize ?? p
    }),
    checkedIcon: /* @__PURE__ */ z.cloneElement(C, {
      fontSize: C.props.fontSize ?? p
    }),
    ownerState: $,
    ref: n,
    className: Ve(S.root, h),
    disableRipple: g,
    ...y,
    classes: S
  });
});
function qX(e) {
  return ht("MuiFormControlLabel", e);
}
const Lu = dt("MuiFormControlLabel", ["root", "labelPlacementStart", "labelPlacementTop", "labelPlacementBottom", "disabled", "label", "error", "required", "asterisk"]), KX = (e) => {
  const {
    classes: t,
    disabled: n,
    labelPlacement: o,
    error: i,
    required: s
  } = e, l = {
    root: ["root", n && "disabled", `labelPlacement${Ne(o)}`, i && "error", s && "required"],
    label: ["label", n && "disabled"],
    asterisk: ["asterisk", i && "error"]
  };
  return mt(l, qX, t);
}, HX = Oe("label", {
  name: "MuiFormControlLabel",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [{
      [`& .${Lu.label}`]: t.label
    }, t.root, t[`labelPlacement${Ne(n.labelPlacement)}`]];
  }
})(ct(({
  theme: e
}) => ({
  display: "inline-flex",
  alignItems: "center",
  cursor: "pointer",
  // For correct alignment with the text.
  verticalAlign: "middle",
  WebkitTapHighlightColor: "transparent",
  marginLeft: -11,
  marginRight: 16,
  // used for row presentation of radio/checkbox
  [`&.${Lu.disabled}`]: {
    cursor: "default"
  },
  [`& .${Lu.label}`]: {
    [`&.${Lu.disabled}`]: {
      color: (e.vars || e).palette.text.disabled
    }
  },
  variants: [{
    props: {
      labelPlacement: "start"
    },
    style: {
      flexDirection: "row-reverse",
      marginRight: -11
    }
  }, {
    props: {
      labelPlacement: "top"
    },
    style: {
      flexDirection: "column-reverse"
    }
  }, {
    props: {
      labelPlacement: "bottom"
    },
    style: {
      flexDirection: "column"
    }
  }, {
    props: ({
      labelPlacement: t
    }) => t === "start" || t === "top" || t === "bottom",
    style: {
      marginLeft: 16
      // used for row presentation of radio/checkbox
    }
  }]
}))), GX = Oe("span", {
  name: "MuiFormControlLabel",
  slot: "Asterisk",
  overridesResolver: (e, t) => t.asterisk
})(ct(({
  theme: e
}) => ({
  [`&.${Lu.error}`]: {
    color: (e.vars || e).palette.error.main
  }
}))), I0 = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiFormControlLabel"
  }), {
    checked: i,
    className: s,
    componentsProps: l = {},
    control: u,
    disabled: c,
    disableTypography: f,
    inputRef: p,
    label: g,
    labelPlacement: h = "end",
    name: y,
    onChange: v,
    required: C,
    slots: $ = {},
    slotProps: S = {},
    value: w,
    ..._
  } = o, b = ti(), k = c ?? u.props.disabled ?? (b == null ? void 0 : b.disabled), E = C ?? u.props.required, j = {
    disabled: k,
    required: E
  };
  ["checked", "name", "onChange", "value", "inputRef"].forEach((X) => {
    typeof u.props[X] > "u" && typeof o[X] < "u" && (j[X] = o[X]);
  });
  const L = ca({
    props: o,
    muiFormControl: b,
    states: ["error"]
  }), B = {
    ...o,
    disabled: k,
    labelPlacement: h,
    required: E,
    error: L.error
  }, O = KX(B), V = {
    slots: $,
    slotProps: {
      ...l,
      ...S
    }
  }, [W, q] = Tr("typography", {
    elementType: sr,
    externalForwardedProps: V,
    ownerState: B
  });
  let Q = g;
  return Q != null && Q.type !== sr && !f && (Q = /* @__PURE__ */ A.jsx(W, {
    component: "span",
    ...q,
    className: Ve(O.label, q == null ? void 0 : q.className),
    children: Q
  })), /* @__PURE__ */ A.jsxs(HX, {
    className: Ve(O.root, s),
    ownerState: B,
    ref: n,
    ..._,
    children: [/* @__PURE__ */ z.cloneElement(u, j), E ? /* @__PURE__ */ A.jsxs("div", {
      children: [Q, /* @__PURE__ */ A.jsxs(GX, {
        ownerState: B,
        "aria-hidden": !0,
        className: O.asterisk,
        children: [" ", "*"]
      })]
    }) : Q]
  });
});
function JX(e) {
  var t;
  const { schema: n, id: o, value: i, disabled: s, readonly: l, label: u = "", hideLabel: c, autofocus: f, onChange: p, onBlur: g, onFocus: h, registry: y, options: v, uiSchema: C } = e, $ = _t("DescriptionFieldTemplate", y, v), S = Uu(n), w = (E, j) => p(j), _ = ({ target: E }) => g(o, E && E.value), b = ({ target: E }) => h(o, E && E.value), k = (t = v.description) !== null && t !== void 0 ? t : n.description;
  return A.jsxs(A.Fragment, { children: [!c && !!k && A.jsx($, { id: es(o), description: k, schema: n, uiSchema: C, registry: y }), A.jsx(I0, { control: A.jsx(ME, { id: o, name: o, checked: typeof i > "u" ? !1 : !!i, required: S, disabled: s || l, autoFocus: f, onChange: w, onBlur: _, onFocus: b, "aria-describedby": Gr(o) }), label: ts(u, c, !1) })] });
}
function YX(e) {
  return ht("MuiFormGroup", e);
}
dt("MuiFormGroup", ["root", "row", "error"]);
const QX = (e) => {
  const {
    classes: t,
    row: n,
    error: o
  } = e;
  return mt({
    root: ["root", n && "row", o && "error"]
  }, YX, t);
}, XX = Oe("div", {
  name: "MuiFormGroup",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, n.row && t.row];
  }
})({
  display: "flex",
  flexDirection: "column",
  flexWrap: "wrap",
  variants: [{
    props: {
      row: !0
    },
    style: {
      flexDirection: "row"
    }
  }]
}), NE = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiFormGroup"
  }), {
    className: i,
    row: s = !1,
    ...l
  } = o, u = ti(), c = ca({
    props: o,
    muiFormControl: u,
    states: ["error"]
  }), f = {
    ...o,
    row: s,
    error: c.error
  }, p = QX(f);
  return /* @__PURE__ */ A.jsx(XX, {
    className: Ve(p.root, i),
    ownerState: f,
    ref: n,
    ...l
  });
});
function ZX({ label: e, hideLabel: t, id: n, disabled: o, options: i, value: s, autofocus: l, readonly: u, required: c, onChange: f, onBlur: p, onFocus: g }) {
  const { enumOptions: h, enumDisabled: y, inline: v, emptyValue: C } = i, $ = Array.isArray(s) ? s : [s], S = (b) => ({ target: { checked: k } }) => {
    f(k ? sx(b, $, h) : ix(b, $, h));
  }, w = ({ target: b }) => p(n, qn(b && b.value, h, C)), _ = ({ target: b }) => g(n, qn(b && b.value, h, C));
  return A.jsxs(A.Fragment, { children: [ts(A.jsx(Wp, { required: c, htmlFor: n, children: e || void 0 }), t), A.jsx(NE, { id: n, row: !!v, children: Array.isArray(h) && h.map((b, k) => {
    const E = vp(b.value, $), j = Array.isArray(y) && y.indexOf(b.value) !== -1, L = A.jsx(ME, { id: bp(n, k), name: n, checked: E, disabled: o || j || u, autoFocus: l && k === 0, onChange: S(k), onBlur: w, onFocus: _, "aria-describedby": Gr(n) });
    return A.jsx(I0, { control: L, label: b.label }, k);
  }) })] });
}
const eZ = Cr(/* @__PURE__ */ A.jsx("path", {
  d: "M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z"
}), "RadioButtonUnchecked"), tZ = Cr(/* @__PURE__ */ A.jsx("path", {
  d: "M8.465 8.465C9.37 7.56 10.62 7 12 7C14.76 7 17 9.24 17 12C17 13.38 16.44 14.63 15.535 15.535C14.63 16.44 13.38 17 12 17C9.24 17 7 14.76 7 12C7 10.62 7.56 9.37 8.465 8.465Z"
}), "RadioButtonChecked"), nZ = Oe("span", {
  shouldForwardProp: ur
})({
  position: "relative",
  display: "flex"
}), rZ = Oe(eZ)({
  // Scale applied to prevent dot misalignment in Safari
  transform: "scale(1)"
}), oZ = Oe(tZ)(ct(({
  theme: e
}) => ({
  left: 0,
  position: "absolute",
  transform: "scale(0)",
  transition: e.transitions.create("transform", {
    easing: e.transitions.easing.easeIn,
    duration: e.transitions.duration.shortest
  }),
  variants: [{
    props: {
      checked: !0
    },
    style: {
      transform: "scale(1)",
      transition: e.transitions.create("transform", {
        easing: e.transitions.easing.easeOut,
        duration: e.transitions.duration.shortest
      })
    }
  }]
})));
function FE(e) {
  const {
    checked: t = !1,
    classes: n = {},
    fontSize: o
  } = e, i = {
    ...e,
    checked: t
  };
  return /* @__PURE__ */ A.jsxs(nZ, {
    className: n.root,
    ownerState: i,
    children: [/* @__PURE__ */ A.jsx(rZ, {
      fontSize: o,
      className: n.background,
      ownerState: i
    }), /* @__PURE__ */ A.jsx(oZ, {
      fontSize: o,
      className: n.dot,
      ownerState: i
    })]
  });
}
const LE = /* @__PURE__ */ z.createContext(void 0);
function iZ() {
  return z.useContext(LE);
}
function aZ(e) {
  return ht("MuiRadio", e);
}
const Sw = dt("MuiRadio", ["root", "checked", "disabled", "colorPrimary", "colorSecondary", "sizeSmall"]), sZ = (e) => {
  const {
    classes: t,
    color: n,
    size: o
  } = e, i = {
    root: ["root", `color${Ne(n)}`, o !== "medium" && `size${Ne(o)}`]
  };
  return {
    ...t,
    ...mt(i, aZ, t)
  };
}, lZ = Oe(TE, {
  shouldForwardProp: (e) => ur(e) || e === "classes",
  name: "MuiRadio",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, n.size !== "medium" && t[`size${Ne(n.size)}`], t[`color${Ne(n.color)}`]];
  }
})(ct(({
  theme: e
}) => ({
  color: (e.vars || e).palette.text.secondary,
  [`&.${Sw.disabled}`]: {
    color: (e.vars || e).palette.action.disabled
  },
  variants: [{
    props: {
      color: "default",
      disabled: !1,
      disableRipple: !1
    },
    style: {
      "&:hover": {
        backgroundColor: e.vars ? `rgba(${e.vars.palette.action.activeChannel} / ${e.vars.palette.action.hoverOpacity})` : En(e.palette.action.active, e.palette.action.hoverOpacity)
      }
    }
  }, ...Object.entries(e.palette).filter(kn()).map(([t]) => ({
    props: {
      color: t,
      disabled: !1,
      disableRipple: !1
    },
    style: {
      "&:hover": {
        backgroundColor: e.vars ? `rgba(${e.vars.palette[t].mainChannel} / ${e.vars.palette.action.hoverOpacity})` : En(e.palette[t].main, e.palette.action.hoverOpacity)
      }
    }
  })), ...Object.entries(e.palette).filter(kn()).map(([t]) => ({
    props: {
      color: t,
      disabled: !1
    },
    style: {
      [`&.${Sw.checked}`]: {
        color: (e.vars || e).palette[t].main
      }
    }
  })), {
    // Should be last to override other colors
    props: {
      disableRipple: !1
    },
    style: {
      // Reset on touch devices, it doesn't add specificity
      "&:hover": {
        "@media (hover: none)": {
          backgroundColor: "transparent"
        }
      }
    }
  }]
})));
function uZ(e, t) {
  return typeof t == "object" && t !== null ? e === t : String(e) === String(t);
}
const ww = /* @__PURE__ */ A.jsx(FE, {
  checked: !0
}), $w = /* @__PURE__ */ A.jsx(FE, {}), cZ = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiRadio"
  }), {
    checked: i,
    checkedIcon: s = ww,
    color: l = "primary",
    icon: u = $w,
    name: c,
    onChange: f,
    size: p = "medium",
    className: g,
    disabled: h,
    disableRipple: y = !1,
    ...v
  } = o, C = ti();
  let $ = h;
  C && typeof $ > "u" && ($ = C.disabled), $ ?? ($ = !1);
  const S = {
    ...o,
    disabled: $,
    disableRipple: y,
    color: l,
    size: p
  }, w = sZ(S), _ = iZ();
  let b = i;
  const k = _g(f, _ && _.onChange);
  let E = c;
  return _ && (typeof b > "u" && (b = uZ(_.value, o.value)), typeof E > "u" && (E = _.name)), /* @__PURE__ */ A.jsx(lZ, {
    type: "radio",
    icon: /* @__PURE__ */ z.cloneElement(u, {
      fontSize: $w.props.fontSize ?? p
    }),
    checkedIcon: /* @__PURE__ */ z.cloneElement(s, {
      fontSize: ww.props.fontSize ?? p
    }),
    disabled: $,
    ownerState: S,
    classes: w,
    name: E,
    checked: b,
    onChange: k,
    ref: n,
    className: Ve(w.root, g),
    ...v
  });
});
function dZ(e) {
  return ht("MuiRadioGroup", e);
}
dt("MuiRadioGroup", ["root", "row", "error"]);
const fZ = (e) => {
  const {
    classes: t,
    row: n,
    error: o
  } = e;
  return mt({
    root: ["root", n && "row", o && "error"]
  }, dZ, t);
}, pZ = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const {
    // private
    // eslint-disable-next-line react/prop-types
    actions: o,
    children: i,
    className: s,
    defaultValue: l,
    name: u,
    onChange: c,
    value: f,
    ...p
  } = t, g = z.useRef(null), h = fZ(t), [y, v] = sc({
    controlled: f,
    default: l,
    name: "RadioGroup"
  });
  z.useImperativeHandle(o, () => ({
    focus: () => {
      let w = g.current.querySelector("input:not(:disabled):checked");
      w || (w = g.current.querySelector("input:not(:disabled)")), w && w.focus();
    }
  }), []);
  const C = _n(n, g), $ = t0(u), S = z.useMemo(() => ({
    name: $,
    onChange(w) {
      v(w.target.value), c && c(w, w.target.value);
    },
    value: y
  }), [$, c, v, y]);
  return /* @__PURE__ */ A.jsx(LE.Provider, {
    value: S,
    children: /* @__PURE__ */ A.jsx(NE, {
      role: "radiogroup",
      ref: C,
      className: Ve(h.root, s),
      ...p,
      children: i
    })
  });
});
function hZ({ id: e, options: t, value: n, required: o, disabled: i, readonly: s, label: l, hideLabel: u, onChange: c, onBlur: f, onFocus: p }) {
  var g;
  const { enumOptions: h, enumDisabled: y, emptyValue: v } = t, C = (b, k) => c(qn(k, h, v)), $ = ({ target: b }) => f(e, qn(b && b.value, h, v)), S = ({ target: b }) => p(e, qn(b && b.value, h, v)), w = t ? t.inline : !1, _ = (g = Gy(n, h)) !== null && g !== void 0 ? g : null;
  return A.jsxs(A.Fragment, { children: [ts(A.jsx(Wp, { required: o, htmlFor: e, children: l || void 0 }), u), A.jsx(pZ, { id: e, name: e, value: _, row: w, onChange: C, onBlur: $, onFocus: S, "aria-describedby": Gr(e), children: Array.isArray(h) && h.map((b, k) => {
    const E = Array.isArray(y) && y.indexOf(b.value) !== -1;
    return A.jsx(I0, { control: A.jsx(cZ, { name: e, id: bp(e, k), color: "primary" }), label: b.label, value: String(k), disabled: i || E || s }, k);
  }) })] });
}
function mZ(e, t, n = (o, i) => o === i) {
  return e.length === t.length && e.every((o, i) => n(o, t[i]));
}
const gZ = 2;
function Ds(e, t, n, o, i) {
  return n === 1 ? Math.min(e + t, i) : Math.max(e - t, o);
}
function DE(e, t) {
  return e - t;
}
function Cw(e, t) {
  const {
    index: n
  } = e.reduce((o, i, s) => {
    const l = Math.abs(t - i);
    return o === null || l < o.distance || l === o.distance ? {
      distance: l,
      index: s
    } : o;
  }, null) ?? {};
  return n;
}
function ef(e, t) {
  if (t.current !== void 0 && e.changedTouches) {
    const n = e;
    for (let o = 0; o < n.changedTouches.length; o += 1) {
      const i = n.changedTouches[o];
      if (i.identifier === t.current)
        return {
          x: i.clientX,
          y: i.clientY
        };
    }
    return !1;
  }
  return {
    x: e.clientX,
    y: e.clientY
  };
}
function Kf(e, t, n) {
  return (e - t) * 100 / (n - t);
}
function yZ(e, t, n) {
  return (n - t) * e + t;
}
function vZ(e) {
  if (Math.abs(e) < 1) {
    const n = e.toExponential().split("e-"), o = n[0].split(".")[1];
    return (o ? o.length : 0) + parseInt(n[1], 10);
  }
  const t = e.toString().split(".")[1];
  return t ? t.length : 0;
}
function bZ(e, t, n) {
  const o = Math.round((e - n) / t) * t + n;
  return Number(o.toFixed(vZ(t)));
}
function xw({
  values: e,
  newValue: t,
  index: n
}) {
  const o = e.slice();
  return o[n] = t, o.sort(DE);
}
function tf({
  sliderRef: e,
  activeIndex: t,
  setActive: n
}) {
  var i, s, l;
  const o = $r(e.current);
  (!((i = e.current) != null && i.contains(o.activeElement)) || Number((s = o == null ? void 0 : o.activeElement) == null ? void 0 : s.getAttribute("data-index")) !== t) && ((l = e.current) == null || l.querySelector(`[type="range"][data-index="${t}"]`).focus()), n && n(t);
}
function nf(e, t) {
  return typeof e == "number" && typeof t == "number" ? e === t : typeof e == "object" && typeof t == "object" ? mZ(e, t) : !1;
}
const SZ = {
  horizontal: {
    offset: (e) => ({
      left: `${e}%`
    }),
    leap: (e) => ({
      width: `${e}%`
    })
  },
  "horizontal-reverse": {
    offset: (e) => ({
      right: `${e}%`
    }),
    leap: (e) => ({
      width: `${e}%`
    })
  },
  vertical: {
    offset: (e) => ({
      bottom: `${e}%`
    }),
    leap: (e) => ({
      height: `${e}%`
    })
  }
}, wZ = (e) => e;
let rf;
function Ew() {
  return rf === void 0 && (typeof CSS < "u" && typeof CSS.supports == "function" ? rf = CSS.supports("touch-action", "none") : rf = !0), rf;
}
function $Z(e) {
  const {
    "aria-labelledby": t,
    defaultValue: n,
    disabled: o = !1,
    disableSwap: i = !1,
    isRtl: s = !1,
    marks: l = !1,
    max: u = 100,
    min: c = 0,
    name: f,
    onChange: p,
    onChangeCommitted: g,
    orientation: h = "horizontal",
    rootRef: y,
    scale: v = wZ,
    step: C = 1,
    shiftStep: $ = 10,
    tabIndex: S,
    value: w
  } = e, _ = z.useRef(void 0), [b, k] = z.useState(-1), [E, j] = z.useState(-1), [L, B] = z.useState(!1), O = z.useRef(0), [V, W] = sc({
    controlled: w,
    default: n ?? c,
    name: "Slider"
  }), q = p && ((ye, Pe, Me) => {
    const Ye = ye.nativeEvent || ye, ot = new Ye.constructor(Ye.type, Ye);
    Object.defineProperty(ot, "target", {
      writable: !0,
      value: {
        value: Pe,
        name: f
      }
    }), p(ot, Pe, Me);
  }), Q = Array.isArray(V);
  let X = Q ? V.slice().sort(DE) : [V];
  X = X.map((ye) => ye == null ? c : Vs(ye, c, u));
  const ce = l === !0 && C !== null ? [...Array(Math.floor((u - c) / C) + 1)].map((ye, Pe) => ({
    value: c + C * Pe
  })) : l || [], J = ce.map((ye) => ye.value), [ie, re] = z.useState(-1), Y = z.useRef(null), T = _n(y, Y), H = (ye) => (Pe) => {
    var Ye;
    const Me = Number(Pe.currentTarget.getAttribute("data-index"));
    Lf(Pe.target) && re(Me), j(Me), (Ye = ye == null ? void 0 : ye.onFocus) == null || Ye.call(ye, Pe);
  }, G = (ye) => (Pe) => {
    var Me;
    Lf(Pe.target) || re(-1), j(-1), (Me = ye == null ? void 0 : ye.onBlur) == null || Me.call(ye, Pe);
  }, N = (ye, Pe) => {
    const Me = Number(ye.currentTarget.getAttribute("data-index")), Ye = X[Me], ot = J.indexOf(Ye);
    let se = Pe;
    if (ce && C == null) {
      const ae = J[J.length - 1];
      se > ae ? se = ae : se < J[0] ? se = J[0] : se = se < Ye ? J[ot - 1] : J[ot + 1];
    }
    if (se = Vs(se, c, u), Q) {
      i && (se = Vs(se, X[Me - 1] || -1 / 0, X[Me + 1] || 1 / 0));
      const ae = se;
      se = xw({
        values: X,
        newValue: se,
        index: Me
      });
      let be = Me;
      i || (be = se.indexOf(ae)), tf({
        sliderRef: Y,
        activeIndex: be
      });
    }
    W(se), re(Me), q && !nf(se, V) && q(ye, se, Me), g && g(ye, se);
  }, U = (ye) => (Pe) => {
    var Me;
    if (["ArrowUp", "ArrowDown", "ArrowLeft", "ArrowRight", "PageUp", "PageDown", "Home", "End"].includes(Pe.key)) {
      Pe.preventDefault();
      const Ye = Number(Pe.currentTarget.getAttribute("data-index")), ot = X[Ye];
      let se = null;
      if (C != null) {
        const ae = Pe.shiftKey ? $ : C;
        switch (Pe.key) {
          case "ArrowUp":
            se = Ds(ot, ae, 1, c, u);
            break;
          case "ArrowRight":
            se = Ds(ot, ae, s ? -1 : 1, c, u);
            break;
          case "ArrowDown":
            se = Ds(ot, ae, -1, c, u);
            break;
          case "ArrowLeft":
            se = Ds(ot, ae, s ? 1 : -1, c, u);
            break;
          case "PageUp":
            se = Ds(ot, $, 1, c, u);
            break;
          case "PageDown":
            se = Ds(ot, $, -1, c, u);
            break;
          case "Home":
            se = c;
            break;
          case "End":
            se = u;
            break;
        }
      } else if (ce) {
        const ae = J[J.length - 1], be = J.indexOf(ot), Re = [s ? "ArrowRight" : "ArrowLeft", "ArrowDown", "PageDown", "Home"], Ge = [s ? "ArrowLeft" : "ArrowRight", "ArrowUp", "PageUp", "End"];
        Re.includes(Pe.key) ? be === 0 ? se = J[0] : se = J[be - 1] : Ge.includes(Pe.key) && (be === J.length - 1 ? se = ae : se = J[be + 1]);
      }
      se != null && N(Pe, se);
    }
    (Me = ye == null ? void 0 : ye.onKeyDown) == null || Me.call(ye, Pe);
  };
  Ei(() => {
    var ye;
    o && Y.current.contains(document.activeElement) && ((ye = document.activeElement) == null || ye.blur());
  }, [o]), o && b !== -1 && k(-1), o && ie !== -1 && re(-1);
  const Z = (ye) => (Pe) => {
    var Me;
    (Me = ye.onChange) == null || Me.call(ye, Pe), N(Pe, Pe.target.valueAsNumber);
  }, me = z.useRef(void 0);
  let ue = h;
  s && h === "horizontal" && (ue += "-reverse");
  const Ce = ({
    finger: ye,
    move: Pe = !1
  }) => {
    const {
      current: Me
    } = Y, {
      width: Ye,
      height: ot,
      bottom: se,
      left: ae
    } = Me.getBoundingClientRect();
    let be;
    ue.startsWith("vertical") ? be = (se - ye.y) / ot : be = (ye.x - ae) / Ye, ue.includes("-reverse") && (be = 1 - be);
    let Re;
    if (Re = yZ(be, c, u), C)
      Re = bZ(Re, C, c);
    else {
      const Rt = Cw(J, Re);
      Re = J[Rt];
    }
    Re = Vs(Re, c, u);
    let Ge = 0;
    if (Q) {
      Pe ? Ge = me.current : Ge = Cw(X, Re), i && (Re = Vs(Re, X[Ge - 1] || -1 / 0, X[Ge + 1] || 1 / 0));
      const Rt = Re;
      Re = xw({
        values: X,
        newValue: Re,
        index: Ge
      }), i && Pe || (Ge = Re.indexOf(Rt), me.current = Ge);
    }
    return {
      newValue: Re,
      activeIndex: Ge
    };
  }, $e = yi((ye) => {
    const Pe = ef(ye, _);
    if (!Pe)
      return;
    if (O.current += 1, ye.type === "mousemove" && ye.buttons === 0) {
      Te(ye);
      return;
    }
    const {
      newValue: Me,
      activeIndex: Ye
    } = Ce({
      finger: Pe,
      move: !0
    });
    tf({
      sliderRef: Y,
      activeIndex: Ye,
      setActive: k
    }), W(Me), !L && O.current > gZ && B(!0), q && !nf(Me, V) && q(ye, Me, Ye);
  }), Te = yi((ye) => {
    const Pe = ef(ye, _);
    if (B(!1), !Pe)
      return;
    const {
      newValue: Me
    } = Ce({
      finger: Pe,
      move: !0
    });
    k(-1), ye.type === "touchend" && j(-1), g && g(ye, Me), _.current = void 0, et();
  }), We = yi((ye) => {
    if (o)
      return;
    Ew() || ye.preventDefault();
    const Pe = ye.changedTouches[0];
    Pe != null && (_.current = Pe.identifier);
    const Me = ef(ye, _);
    if (Me !== !1) {
      const {
        newValue: ot,
        activeIndex: se
      } = Ce({
        finger: Me
      });
      tf({
        sliderRef: Y,
        activeIndex: se,
        setActive: k
      }), W(ot), q && !nf(ot, V) && q(ye, ot, se);
    }
    O.current = 0;
    const Ye = $r(Y.current);
    Ye.addEventListener("touchmove", $e, {
      passive: !0
    }), Ye.addEventListener("touchend", Te, {
      passive: !0
    });
  }), et = z.useCallback(() => {
    const ye = $r(Y.current);
    ye.removeEventListener("mousemove", $e), ye.removeEventListener("mouseup", Te), ye.removeEventListener("touchmove", $e), ye.removeEventListener("touchend", Te);
  }, [Te, $e]);
  z.useEffect(() => {
    const {
      current: ye
    } = Y;
    return ye.addEventListener("touchstart", We, {
      passive: Ew()
    }), () => {
      ye.removeEventListener("touchstart", We), et();
    };
  }, [et, We]), z.useEffect(() => {
    o && et();
  }, [o, et]);
  const Xe = (ye) => (Pe) => {
    var ot;
    if ((ot = ye.onMouseDown) == null || ot.call(ye, Pe), o || Pe.defaultPrevented || Pe.button !== 0)
      return;
    Pe.preventDefault();
    const Me = ef(Pe, _);
    if (Me !== !1) {
      const {
        newValue: se,
        activeIndex: ae
      } = Ce({
        finger: Me
      });
      tf({
        sliderRef: Y,
        activeIndex: ae,
        setActive: k
      }), W(se), q && !nf(se, V) && q(Pe, se, ae);
    }
    O.current = 0;
    const Ye = $r(Y.current);
    Ye.addEventListener("mousemove", $e, {
      passive: !0
    }), Ye.addEventListener("mouseup", Te);
  }, ft = Kf(Q ? X[0] : c, c, u), Lt = Kf(X[X.length - 1], c, u) - ft, st = (ye = {}) => {
    const Pe = Wu(ye), Me = {
      onMouseDown: Xe(Pe || {})
    }, Ye = {
      ...Pe,
      ...Me
    };
    return {
      ...ye,
      ref: T,
      ...Ye
    };
  }, pt = (ye) => (Pe) => {
    var Ye;
    (Ye = ye.onMouseOver) == null || Ye.call(ye, Pe);
    const Me = Number(Pe.currentTarget.getAttribute("data-index"));
    j(Me);
  }, Se = (ye) => (Pe) => {
    var Me;
    (Me = ye.onMouseLeave) == null || Me.call(ye, Pe), j(-1);
  }, gt = (ye = {}) => {
    const Pe = Wu(ye), Me = {
      onMouseOver: pt(Pe || {}),
      onMouseLeave: Se(Pe || {})
    };
    return {
      ...ye,
      ...Pe,
      ...Me
    };
  }, yt = (ye) => ({
    // So the non active thumb doesn't show its label on hover.
    pointerEvents: b !== -1 && b !== ye ? "none" : void 0
  });
  let Le;
  return h === "vertical" && (Le = s ? "vertical-rl" : "vertical-lr"), {
    active: b,
    axis: ue,
    axisProps: SZ,
    dragging: L,
    focusedThumbIndex: ie,
    getHiddenInputProps: (ye = {}) => {
      const Pe = Wu(ye), Me = {
        onChange: Z(Pe || {}),
        onFocus: H(Pe || {}),
        onBlur: G(Pe || {}),
        onKeyDown: U(Pe || {})
      }, Ye = {
        ...Pe,
        ...Me
      };
      return {
        tabIndex: S,
        "aria-labelledby": t,
        "aria-orientation": h,
        "aria-valuemax": v(u),
        "aria-valuemin": v(c),
        name: f,
        type: "range",
        min: e.min,
        max: e.max,
        step: e.step === null && e.marks ? "any" : e.step ?? void 0,
        disabled: o,
        ...ye,
        ...Ye,
        style: {
          ...uq,
          direction: s ? "rtl" : "ltr",
          // So that VoiceOver's focus indicator matches the thumb's dimensions
          width: "100%",
          height: "100%",
          writingMode: Le
        }
      };
    },
    getRootProps: st,
    getThumbProps: gt,
    marks: ce,
    open: E,
    range: Q,
    rootRef: T,
    trackLeap: Lt,
    trackOffset: ft,
    values: X,
    getThumbStyle: yt
  };
}
const CZ = (e) => !e || !vi(e);
function xZ(e) {
  return ht("MuiSlider", e);
}
const po = dt("MuiSlider", ["root", "active", "colorPrimary", "colorSecondary", "colorError", "colorInfo", "colorSuccess", "colorWarning", "disabled", "dragging", "focusVisible", "mark", "markActive", "marked", "markLabel", "markLabelActive", "rail", "sizeSmall", "thumb", "thumbColorPrimary", "thumbColorSecondary", "thumbColorError", "thumbColorSuccess", "thumbColorInfo", "thumbColorWarning", "track", "trackInverted", "trackFalse", "thumbSizeSmall", "valueLabel", "valueLabelOpen", "valueLabelCircle", "valueLabelLabel", "vertical"]), EZ = (e) => {
  const {
    open: t
  } = e;
  return {
    offset: Ve(t && po.valueLabelOpen),
    circle: po.valueLabelCircle,
    label: po.valueLabelLabel
  };
};
function _Z(e) {
  const {
    children: t,
    className: n,
    value: o
  } = e, i = EZ(e);
  return t ? /* @__PURE__ */ z.cloneElement(t, {
    className: Ve(t.props.className)
  }, /* @__PURE__ */ A.jsxs(z.Fragment, {
    children: [t.props.children, /* @__PURE__ */ A.jsx("span", {
      className: Ve(i.offset, n),
      "aria-hidden": !0,
      children: /* @__PURE__ */ A.jsx("span", {
        className: i.circle,
        children: /* @__PURE__ */ A.jsx("span", {
          className: i.label,
          children: o
        })
      })
    })]
  })) : null;
}
function _w(e) {
  return e;
}
const kZ = Oe("span", {
  name: "MuiSlider",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, t[`color${Ne(n.color)}`], n.size !== "medium" && t[`size${Ne(n.size)}`], n.marked && t.marked, n.orientation === "vertical" && t.vertical, n.track === "inverted" && t.trackInverted, n.track === !1 && t.trackFalse];
  }
})(ct(({
  theme: e
}) => ({
  borderRadius: 12,
  boxSizing: "content-box",
  display: "inline-block",
  position: "relative",
  cursor: "pointer",
  touchAction: "none",
  WebkitTapHighlightColor: "transparent",
  "@media print": {
    colorAdjust: "exact"
  },
  [`&.${po.disabled}`]: {
    pointerEvents: "none",
    cursor: "default",
    color: (e.vars || e).palette.grey[400]
  },
  [`&.${po.dragging}`]: {
    [`& .${po.thumb}, & .${po.track}`]: {
      transition: "none"
    }
  },
  variants: [...Object.entries(e.palette).filter(kn()).map(([t]) => ({
    props: {
      color: t
    },
    style: {
      color: (e.vars || e).palette[t].main
    }
  })), {
    props: {
      orientation: "horizontal"
    },
    style: {
      height: 4,
      width: "100%",
      padding: "13px 0",
      // The primary input mechanism of the device includes a pointing device of limited accuracy.
      "@media (pointer: coarse)": {
        // Reach 42px touch target, about ~8mm on screen.
        padding: "20px 0"
      }
    }
  }, {
    props: {
      orientation: "horizontal",
      size: "small"
    },
    style: {
      height: 2
    }
  }, {
    props: {
      orientation: "horizontal",
      marked: !0
    },
    style: {
      marginBottom: 20
    }
  }, {
    props: {
      orientation: "vertical"
    },
    style: {
      height: "100%",
      width: 4,
      padding: "0 13px",
      // The primary input mechanism of the device includes a pointing device of limited accuracy.
      "@media (pointer: coarse)": {
        // Reach 42px touch target, about ~8mm on screen.
        padding: "0 20px"
      }
    }
  }, {
    props: {
      orientation: "vertical",
      size: "small"
    },
    style: {
      width: 2
    }
  }, {
    props: {
      orientation: "vertical",
      marked: !0
    },
    style: {
      marginRight: 44
    }
  }]
}))), PZ = Oe("span", {
  name: "MuiSlider",
  slot: "Rail",
  overridesResolver: (e, t) => t.rail
})({
  display: "block",
  position: "absolute",
  borderRadius: "inherit",
  backgroundColor: "currentColor",
  opacity: 0.38,
  variants: [{
    props: {
      orientation: "horizontal"
    },
    style: {
      width: "100%",
      height: "inherit",
      top: "50%",
      transform: "translateY(-50%)"
    }
  }, {
    props: {
      orientation: "vertical"
    },
    style: {
      height: "100%",
      width: "inherit",
      left: "50%",
      transform: "translateX(-50%)"
    }
  }, {
    props: {
      track: "inverted"
    },
    style: {
      opacity: 1
    }
  }]
}), OZ = Oe("span", {
  name: "MuiSlider",
  slot: "Track",
  overridesResolver: (e, t) => t.track
})(ct(({
  theme: e
}) => ({
  display: "block",
  position: "absolute",
  borderRadius: "inherit",
  border: "1px solid currentColor",
  backgroundColor: "currentColor",
  transition: e.transitions.create(["left", "width", "bottom", "height"], {
    duration: e.transitions.duration.shortest
  }),
  variants: [{
    props: {
      size: "small"
    },
    style: {
      border: "none"
    }
  }, {
    props: {
      orientation: "horizontal"
    },
    style: {
      height: "inherit",
      top: "50%",
      transform: "translateY(-50%)"
    }
  }, {
    props: {
      orientation: "vertical"
    },
    style: {
      width: "inherit",
      left: "50%",
      transform: "translateX(-50%)"
    }
  }, {
    props: {
      track: !1
    },
    style: {
      display: "none"
    }
  }, ...Object.entries(e.palette).filter(kn()).map(([t]) => ({
    props: {
      color: t,
      track: "inverted"
    },
    style: {
      ...e.vars ? {
        backgroundColor: e.vars.palette.Slider[`${t}Track`],
        borderColor: e.vars.palette.Slider[`${t}Track`]
      } : {
        backgroundColor: cl(e.palette[t].main, 0.62),
        borderColor: cl(e.palette[t].main, 0.62),
        ...e.applyStyles("dark", {
          backgroundColor: ul(e.palette[t].main, 0.5)
        }),
        ...e.applyStyles("dark", {
          borderColor: ul(e.palette[t].main, 0.5)
        })
      }
    }
  }))]
}))), AZ = Oe("span", {
  name: "MuiSlider",
  slot: "Thumb",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.thumb, t[`thumbColor${Ne(n.color)}`], n.size !== "medium" && t[`thumbSize${Ne(n.size)}`]];
  }
})(ct(({
  theme: e
}) => ({
  position: "absolute",
  width: 20,
  height: 20,
  boxSizing: "border-box",
  borderRadius: "50%",
  outline: 0,
  backgroundColor: "currentColor",
  display: "flex",
  alignItems: "center",
  justifyContent: "center",
  transition: e.transitions.create(["box-shadow", "left", "bottom"], {
    duration: e.transitions.duration.shortest
  }),
  "&::before": {
    position: "absolute",
    content: '""',
    borderRadius: "inherit",
    width: "100%",
    height: "100%",
    boxShadow: (e.vars || e).shadows[2]
  },
  "&::after": {
    position: "absolute",
    content: '""',
    borderRadius: "50%",
    // 42px is the hit target
    width: 42,
    height: 42,
    top: "50%",
    left: "50%",
    transform: "translate(-50%, -50%)"
  },
  [`&.${po.disabled}`]: {
    "&:hover": {
      boxShadow: "none"
    }
  },
  variants: [{
    props: {
      size: "small"
    },
    style: {
      width: 12,
      height: 12,
      "&::before": {
        boxShadow: "none"
      }
    }
  }, {
    props: {
      orientation: "horizontal"
    },
    style: {
      top: "50%",
      transform: "translate(-50%, -50%)"
    }
  }, {
    props: {
      orientation: "vertical"
    },
    style: {
      left: "50%",
      transform: "translate(-50%, 50%)"
    }
  }, ...Object.entries(e.palette).filter(kn()).map(([t]) => ({
    props: {
      color: t
    },
    style: {
      [`&:hover, &.${po.focusVisible}`]: {
        ...e.vars ? {
          boxShadow: `0px 0px 0px 8px rgba(${e.vars.palette[t].mainChannel} / 0.16)`
        } : {
          boxShadow: `0px 0px 0px 8px ${En(e.palette[t].main, 0.16)}`
        },
        "@media (hover: none)": {
          boxShadow: "none"
        }
      },
      [`&.${po.active}`]: {
        ...e.vars ? {
          boxShadow: `0px 0px 0px 14px rgba(${e.vars.palette[t].mainChannel} / 0.16)`
        } : {
          boxShadow: `0px 0px 0px 14px ${En(e.palette[t].main, 0.16)}`
        }
      }
    }
  }))]
}))), IZ = Oe(_Z, {
  name: "MuiSlider",
  slot: "ValueLabel",
  overridesResolver: (e, t) => t.valueLabel
})(ct(({
  theme: e
}) => ({
  zIndex: 1,
  whiteSpace: "nowrap",
  ...e.typography.body2,
  fontWeight: 500,
  transition: e.transitions.create(["transform"], {
    duration: e.transitions.duration.shortest
  }),
  position: "absolute",
  backgroundColor: (e.vars || e).palette.grey[600],
  borderRadius: 2,
  color: (e.vars || e).palette.common.white,
  display: "flex",
  alignItems: "center",
  justifyContent: "center",
  padding: "0.25rem 0.75rem",
  variants: [{
    props: {
      orientation: "horizontal"
    },
    style: {
      transform: "translateY(-100%) scale(0)",
      top: "-10px",
      transformOrigin: "bottom center",
      "&::before": {
        position: "absolute",
        content: '""',
        width: 8,
        height: 8,
        transform: "translate(-50%, 50%) rotate(45deg)",
        backgroundColor: "inherit",
        bottom: 0,
        left: "50%"
      },
      [`&.${po.valueLabelOpen}`]: {
        transform: "translateY(-100%) scale(1)"
      }
    }
  }, {
    props: {
      orientation: "vertical"
    },
    style: {
      transform: "translateY(-50%) scale(0)",
      right: "30px",
      top: "50%",
      transformOrigin: "right center",
      "&::before": {
        position: "absolute",
        content: '""',
        width: 8,
        height: 8,
        transform: "translate(-50%, -50%) rotate(45deg)",
        backgroundColor: "inherit",
        right: -8,
        top: "50%"
      },
      [`&.${po.valueLabelOpen}`]: {
        transform: "translateY(-50%) scale(1)"
      }
    }
  }, {
    props: {
      size: "small"
    },
    style: {
      fontSize: e.typography.pxToRem(12),
      padding: "0.25rem 0.5rem"
    }
  }, {
    props: {
      orientation: "vertical",
      size: "small"
    },
    style: {
      right: "20px"
    }
  }]
}))), jZ = Oe("span", {
  name: "MuiSlider",
  slot: "Mark",
  shouldForwardProp: (e) => Dp(e) && e !== "markActive",
  overridesResolver: (e, t) => {
    const {
      markActive: n
    } = e;
    return [t.mark, n && t.markActive];
  }
})(ct(({
  theme: e
}) => ({
  position: "absolute",
  width: 2,
  height: 2,
  borderRadius: 1,
  backgroundColor: "currentColor",
  variants: [{
    props: {
      orientation: "horizontal"
    },
    style: {
      top: "50%",
      transform: "translate(-1px, -50%)"
    }
  }, {
    props: {
      orientation: "vertical"
    },
    style: {
      left: "50%",
      transform: "translate(-50%, 1px)"
    }
  }, {
    props: {
      markActive: !0
    },
    style: {
      backgroundColor: (e.vars || e).palette.background.paper,
      opacity: 0.8
    }
  }]
}))), RZ = Oe("span", {
  name: "MuiSlider",
  slot: "MarkLabel",
  shouldForwardProp: (e) => Dp(e) && e !== "markLabelActive",
  overridesResolver: (e, t) => t.markLabel
})(ct(({
  theme: e
}) => ({
  ...e.typography.body2,
  color: (e.vars || e).palette.text.secondary,
  position: "absolute",
  whiteSpace: "nowrap",
  variants: [{
    props: {
      orientation: "horizontal"
    },
    style: {
      top: 30,
      transform: "translateX(-50%)",
      "@media (pointer: coarse)": {
        top: 40
      }
    }
  }, {
    props: {
      orientation: "vertical"
    },
    style: {
      left: 36,
      transform: "translateY(50%)",
      "@media (pointer: coarse)": {
        left: 44
      }
    }
  }, {
    props: {
      markLabelActive: !0
    },
    style: {
      color: (e.vars || e).palette.text.primary
    }
  }]
}))), TZ = (e) => {
  const {
    disabled: t,
    dragging: n,
    marked: o,
    orientation: i,
    track: s,
    classes: l,
    color: u,
    size: c
  } = e, f = {
    root: ["root", t && "disabled", n && "dragging", o && "marked", i === "vertical" && "vertical", s === "inverted" && "trackInverted", s === !1 && "trackFalse", u && `color${Ne(u)}`, c && `size${Ne(c)}`],
    rail: ["rail"],
    track: ["track"],
    mark: ["mark"],
    markActive: ["markActive"],
    markLabel: ["markLabel"],
    markLabelActive: ["markLabelActive"],
    valueLabel: ["valueLabel"],
    thumb: ["thumb", t && "disabled", c && `thumbSize${Ne(c)}`, u && `thumbColor${Ne(u)}`],
    active: ["active"],
    disabled: ["disabled"],
    focusVisible: ["focusVisible"]
  };
  return mt(f, xZ, l);
}, MZ = ({
  children: e
}) => e, NZ = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiSlider"
  }), i = g0(), {
    "aria-label": s,
    "aria-valuetext": l,
    "aria-labelledby": u,
    // eslint-disable-next-line react/prop-types
    component: c = "span",
    components: f = {},
    componentsProps: p = {},
    color: g = "primary",
    classes: h,
    className: y,
    disableSwap: v = !1,
    disabled: C = !1,
    getAriaLabel: $,
    getAriaValueText: S,
    marks: w = !1,
    max: _ = 100,
    min: b = 0,
    name: k,
    onChange: E,
    onChangeCommitted: j,
    orientation: L = "horizontal",
    shiftStep: B = 10,
    size: O = "medium",
    step: V = 1,
    scale: W = _w,
    slotProps: q,
    slots: Q,
    tabIndex: X,
    track: ce = "normal",
    value: J,
    valueLabelDisplay: ie = "off",
    valueLabelFormat: re = _w,
    ...Y
  } = o, T = {
    ...o,
    isRtl: i,
    max: _,
    min: b,
    classes: h,
    disabled: C,
    disableSwap: v,
    orientation: L,
    marks: w,
    color: g,
    size: O,
    step: V,
    shiftStep: B,
    scale: W,
    track: ce,
    valueLabelDisplay: ie,
    valueLabelFormat: re
  }, {
    axisProps: H,
    getRootProps: G,
    getHiddenInputProps: N,
    getThumbProps: U,
    open: Z,
    active: me,
    axis: ue,
    focusedThumbIndex: Ce,
    range: $e,
    dragging: Te,
    marks: We,
    values: et,
    trackOffset: Xe,
    trackLeap: ft,
    getThumbStyle: Lt
  } = $Z({
    ...T,
    rootRef: n
  });
  T.marked = We.length > 0 && We.some(($t) => $t.label), T.dragging = Te, T.focusedThumbIndex = Ce;
  const st = TZ(T), pt = (Q == null ? void 0 : Q.root) ?? f.Root ?? kZ, Se = (Q == null ? void 0 : Q.rail) ?? f.Rail ?? PZ, gt = (Q == null ? void 0 : Q.track) ?? f.Track ?? OZ, yt = (Q == null ? void 0 : Q.thumb) ?? f.Thumb ?? AZ, Le = (Q == null ? void 0 : Q.valueLabel) ?? f.ValueLabel ?? IZ, cn = (Q == null ? void 0 : Q.mark) ?? f.Mark ?? jZ, ye = (Q == null ? void 0 : Q.markLabel) ?? f.MarkLabel ?? RZ, Pe = (Q == null ? void 0 : Q.input) ?? f.Input ?? "input", Me = (q == null ? void 0 : q.root) ?? p.root, Ye = (q == null ? void 0 : q.rail) ?? p.rail, ot = (q == null ? void 0 : q.track) ?? p.track, se = (q == null ? void 0 : q.thumb) ?? p.thumb, ae = (q == null ? void 0 : q.valueLabel) ?? p.valueLabel, be = (q == null ? void 0 : q.mark) ?? p.mark, Re = (q == null ? void 0 : q.markLabel) ?? p.markLabel, Ge = (q == null ? void 0 : q.input) ?? p.input, Rt = Vo({
    elementType: pt,
    getSlotProps: G,
    externalSlotProps: Me,
    externalForwardedProps: Y,
    additionalProps: {
      ...CZ(pt) && {
        as: c
      }
    },
    ownerState: {
      ...T,
      ...Me == null ? void 0 : Me.ownerState
    },
    className: [st.root, y]
  }), Fe = Vo({
    elementType: Se,
    externalSlotProps: Ye,
    ownerState: T,
    className: st.rail
  }), tt = Vo({
    elementType: gt,
    externalSlotProps: ot,
    additionalProps: {
      style: {
        ...H[ue].offset(Xe),
        ...H[ue].leap(ft)
      }
    },
    ownerState: {
      ...T,
      ...ot == null ? void 0 : ot.ownerState
    },
    className: st.track
  }), wt = Vo({
    elementType: yt,
    getSlotProps: U,
    externalSlotProps: se,
    ownerState: {
      ...T,
      ...se == null ? void 0 : se.ownerState
    },
    className: st.thumb
  }), Gt = Vo({
    elementType: Le,
    externalSlotProps: ae,
    ownerState: {
      ...T,
      ...ae == null ? void 0 : ae.ownerState
    },
    className: st.valueLabel
  }), dn = Vo({
    elementType: cn,
    externalSlotProps: be,
    ownerState: T,
    className: st.mark
  }), jn = Vo({
    elementType: ye,
    externalSlotProps: Re,
    ownerState: T,
    className: st.markLabel
  }), qt = Vo({
    elementType: Pe,
    getSlotProps: N,
    externalSlotProps: Ge,
    ownerState: T
  });
  return /* @__PURE__ */ A.jsxs(pt, {
    ...Rt,
    children: [/* @__PURE__ */ A.jsx(Se, {
      ...Fe
    }), /* @__PURE__ */ A.jsx(gt, {
      ...tt
    }), We.filter(($t) => $t.value >= b && $t.value <= _).map(($t, At) => {
      const Jt = Kf($t.value, b, _), $n = H[ue].offset(Jt);
      let Ut;
      return ce === !1 ? Ut = et.includes($t.value) : Ut = ce === "normal" && ($e ? $t.value >= et[0] && $t.value <= et[et.length - 1] : $t.value <= et[0]) || ce === "inverted" && ($e ? $t.value <= et[0] || $t.value >= et[et.length - 1] : $t.value >= et[0]), /* @__PURE__ */ A.jsxs(z.Fragment, {
        children: [/* @__PURE__ */ A.jsx(cn, {
          "data-index": At,
          ...dn,
          ...!vi(cn) && {
            markActive: Ut
          },
          style: {
            ...$n,
            ...dn.style
          },
          className: Ve(dn.className, Ut && st.markActive)
        }), $t.label != null ? /* @__PURE__ */ A.jsx(ye, {
          "aria-hidden": !0,
          "data-index": At,
          ...jn,
          ...!vi(ye) && {
            markLabelActive: Ut
          },
          style: {
            ...$n,
            ...jn.style
          },
          className: Ve(st.markLabel, jn.className, Ut && st.markLabelActive),
          children: $t.label
        }) : null]
      }, At);
    }), et.map(($t, At) => {
      const Jt = Kf($t, b, _), $n = H[ue].offset(Jt), Ut = ie === "off" ? MZ : Le;
      return (
        /* TODO v6: Change component structure. It will help in avoiding the complicated React.cloneElement API added in SliderValueLabel component. Should be: Thumb -> Input, ValueLabel. Follow Joy UI's Slider structure. */
        /* @__PURE__ */ A.jsx(Ut, {
          ...!vi(Ut) && {
            valueLabelFormat: re,
            valueLabelDisplay: ie,
            value: typeof re == "function" ? re(W($t), At) : re,
            index: At,
            open: Z === At || me === At || ie === "on",
            disabled: C
          },
          ...Gt,
          children: /* @__PURE__ */ A.jsx(yt, {
            "data-index": At,
            ...wt,
            className: Ve(st.thumb, wt.className, me === At && st.active, Ce === At && st.focusVisible),
            style: {
              ...$n,
              ...Lt(At),
              ...wt.style
            },
            children: /* @__PURE__ */ A.jsx(Pe, {
              "data-index": At,
              "aria-label": $ ? $(At) : s,
              "aria-valuenow": W($t),
              "aria-labelledby": u,
              "aria-valuetext": S ? S(W($t), At) : l,
              value: et[At],
              ...qt
            })
          })
        }, At)
      );
    })]
  });
});
function FZ(e) {
  const { value: t, readonly: n, disabled: o, onBlur: i, onFocus: s, options: l, schema: u, onChange: c, required: f, label: p, hideLabel: g, id: h } = e, y = { value: t, label: p, id: h, name: h, ...lx(u) }, v = (S, w) => {
    c(w ?? l.emptyValue);
  }, C = ({ target: S }) => i(h, S && S.value), $ = ({ target: S }) => s(h, S && S.value);
  return A.jsxs(A.Fragment, { children: [ts(A.jsx(Wp, { required: f, htmlFor: h, children: p || void 0 }), g), A.jsx(NZ, { disabled: o || n, onChange: v, onBlur: C, onFocus: $, valueLabelDisplay: "auto", ...y, "aria-describedby": Gr(h) })] });
}
function LZ(e) {
  return ht("MuiMenuItem", e);
}
const Au = dt("MuiMenuItem", ["root", "focusVisible", "dense", "disabled", "divider", "gutters", "selected"]), DZ = (e, t) => {
  const {
    ownerState: n
  } = e;
  return [t.root, n.dense && t.dense, n.divider && t.divider, !n.disableGutters && t.gutters];
}, zZ = (e) => {
  const {
    disabled: t,
    dense: n,
    divider: o,
    disableGutters: i,
    selected: s,
    classes: l
  } = e, c = mt({
    root: ["root", n && "dense", t && "disabled", !i && "gutters", o && "divider", s && "selected"]
  }, LZ, l);
  return {
    ...l,
    ...c
  };
}, BZ = Oe(Tc, {
  shouldForwardProp: (e) => ur(e) || e === "classes",
  name: "MuiMenuItem",
  slot: "Root",
  overridesResolver: DZ
})(ct(({
  theme: e
}) => ({
  ...e.typography.body1,
  display: "flex",
  justifyContent: "flex-start",
  alignItems: "center",
  position: "relative",
  textDecoration: "none",
  minHeight: 48,
  paddingTop: 6,
  paddingBottom: 6,
  boxSizing: "border-box",
  whiteSpace: "nowrap",
  "&:hover": {
    textDecoration: "none",
    backgroundColor: (e.vars || e).palette.action.hover,
    // Reset on touch devices, it doesn't add specificity
    "@media (hover: none)": {
      backgroundColor: "transparent"
    }
  },
  [`&.${Au.selected}`]: {
    backgroundColor: e.vars ? `rgba(${e.vars.palette.primary.mainChannel} / ${e.vars.palette.action.selectedOpacity})` : En(e.palette.primary.main, e.palette.action.selectedOpacity),
    [`&.${Au.focusVisible}`]: {
      backgroundColor: e.vars ? `rgba(${e.vars.palette.primary.mainChannel} / calc(${e.vars.palette.action.selectedOpacity} + ${e.vars.palette.action.focusOpacity}))` : En(e.palette.primary.main, e.palette.action.selectedOpacity + e.palette.action.focusOpacity)
    }
  },
  [`&.${Au.selected}:hover`]: {
    backgroundColor: e.vars ? `rgba(${e.vars.palette.primary.mainChannel} / calc(${e.vars.palette.action.selectedOpacity} + ${e.vars.palette.action.hoverOpacity}))` : En(e.palette.primary.main, e.palette.action.selectedOpacity + e.palette.action.hoverOpacity),
    // Reset on touch devices, it doesn't add specificity
    "@media (hover: none)": {
      backgroundColor: e.vars ? `rgba(${e.vars.palette.primary.mainChannel} / ${e.vars.palette.action.selectedOpacity})` : En(e.palette.primary.main, e.palette.action.selectedOpacity)
    }
  },
  [`&.${Au.focusVisible}`]: {
    backgroundColor: (e.vars || e).palette.action.focus
  },
  [`&.${Au.disabled}`]: {
    opacity: (e.vars || e).palette.action.disabledOpacity
  },
  [`& + .${bw.root}`]: {
    marginTop: e.spacing(1),
    marginBottom: e.spacing(1)
  },
  [`& + .${bw.inset}`]: {
    marginLeft: 52
  },
  [`& .${Gs.root}`]: {
    marginTop: 0,
    marginBottom: 0
  },
  [`& .${Gs.inset}`]: {
    paddingLeft: 36
  },
  [`& .${vw.root}`]: {
    minWidth: 36
  },
  variants: [{
    props: ({
      ownerState: t
    }) => !t.disableGutters,
    style: {
      paddingLeft: 16,
      paddingRight: 16
    }
  }, {
    props: ({
      ownerState: t
    }) => t.divider,
    style: {
      borderBottom: `1px solid ${(e.vars || e).palette.divider}`,
      backgroundClip: "padding-box"
    }
  }, {
    props: ({
      ownerState: t
    }) => !t.dense,
    style: {
      [e.breakpoints.up("sm")]: {
        minHeight: "auto"
      }
    }
  }, {
    props: ({
      ownerState: t
    }) => t.dense,
    style: {
      minHeight: 32,
      // https://m2.material.io/components/menus#specs > Dense
      paddingTop: 4,
      paddingBottom: 4,
      ...e.typography.body2,
      [`& .${vw.root} svg`]: {
        fontSize: "1.25rem"
      }
    }
  }]
}))), kw = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiMenuItem"
  }), {
    autoFocus: i = !1,
    component: s = "li",
    dense: l = !1,
    divider: u = !1,
    disableGutters: c = !1,
    focusVisibleClassName: f,
    role: p = "menuitem",
    tabIndex: g,
    className: h,
    ...y
  } = o, v = z.useContext(bi), C = z.useMemo(() => ({
    dense: l || v.dense || !1,
    disableGutters: c
  }), [v.dense, l, c]), $ = z.useRef(null);
  Ei(() => {
    i && $.current && $.current.focus();
  }, [i]);
  const S = {
    ...o,
    dense: C.dense,
    divider: u,
    disableGutters: c
  }, w = zZ(o), _ = _n($, n);
  let b;
  return o.disabled || (b = g !== void 0 ? g : -1), /* @__PURE__ */ A.jsx(bi.Provider, {
    value: C,
    children: /* @__PURE__ */ A.jsx(BZ, {
      ref: _,
      role: p,
      tabIndex: b,
      component: s,
      focusVisibleClassName: Ve(w.focusVisible, f),
      className: Ve(w.root, h),
      ...y,
      ownerState: S,
      classes: w
    })
  });
});
function UZ({
  schema: e,
  id: t,
  name: n,
  // remove this from textFieldProps
  options: o,
  label: i,
  hideLabel: s,
  required: l,
  disabled: u,
  placeholder: c,
  readonly: f,
  value: p,
  multiple: g,
  autofocus: h,
  onChange: y,
  onBlur: v,
  onFocus: C,
  errorSchema: $,
  rawErrors: S = [],
  registry: w,
  uiSchema: _,
  hideError: b,
  formContext: k,
  ...E
}) {
  const { enumOptions: j, enumDisabled: L, emptyValue: B } = o;
  g = typeof g > "u" ? !1 : !!g;
  const O = g ? [] : "", V = typeof p > "u" || g && p.length < 1 || !g && p === O, W = ({ target: { value: T } }) => y(qn(T, j, B)), q = ({ target: T }) => v(t, qn(T && T.value, j, B)), Q = ({ target: T }) => C(t, qn(T && T.value, j, B)), X = Gy(p, j, g), { InputLabelProps: ce, SelectProps: J, autocomplete: ie, ...re } = E, Y = !g && e.default === void 0;
  return A.jsxs(A0, { id: t, name: t, label: ts(i || void 0, s, void 0), value: !V && typeof X < "u" ? X : O, required: l, disabled: u || f, autoFocus: h, autoComplete: ie, placeholder: c, error: S.length > 0, onChange: W, onBlur: q, onFocus: Q, ...re, select: !0, InputLabelProps: {
    ...ce,
    shrink: !V
  }, SelectProps: {
    ...J,
    multiple: g
  }, "aria-describedby": Gr(t), children: [Y && A.jsx(kw, { value: "", children: c }), Array.isArray(j) && j.map(({ value: T, label: H }, G) => {
    const N = Array.isArray(L) && L.indexOf(T) !== -1;
    return A.jsx(kw, { value: String(G), disabled: N, children: H }, G);
  })] });
}
function VZ(e) {
  const { options: t, registry: n } = e, o = _t("BaseInputTemplate", n, t);
  let i = 5;
  return (typeof t.rows == "string" || typeof t.rows == "number") && (i = t.rows), A.jsx(o, { ...e, multiline: !0, rows: i });
}
function WZ() {
  return {
    CheckboxWidget: JX,
    CheckboxesWidget: ZX,
    RadioWidget: hZ,
    RangeWidget: FZ,
    SelectWidget: UZ,
    TextareaWidget: VZ
  };
}
function qZ() {
  return {
    templates: IX(),
    widgets: WZ()
  };
}
function KZ() {
  return rq(qZ());
}
const HZ = KZ();
var Bg = { exports: {} }, zE = {}, Ao = {}, pl = {}, Mc = {}, St = {}, mc = {};
(function(e) {
  Object.defineProperty(e, "__esModule", { value: !0 }), e.regexpCode = e.getEsmExportName = e.getProperty = e.safeStringify = e.stringify = e.strConcat = e.addCodeArg = e.str = e._ = e.nil = e._Code = e.Name = e.IDENTIFIER = e._CodeOrName = void 0;
  class t {
  }
  e._CodeOrName = t, e.IDENTIFIER = /^[a-z$_][a-z$_0-9]*$/i;
  class n extends t {
    constructor(w) {
      if (super(), !e.IDENTIFIER.test(w))
        throw new Error("CodeGen: name must be a valid identifier");
      this.str = w;
    }
    toString() {
      return this.str;
    }
    emptyStr() {
      return !1;
    }
    get names() {
      return { [this.str]: 1 };
    }
  }
  e.Name = n;
  class o extends t {
    constructor(w) {
      super(), this._items = typeof w == "string" ? [w] : w;
    }
    toString() {
      return this.str;
    }
    emptyStr() {
      if (this._items.length > 1)
        return !1;
      const w = this._items[0];
      return w === "" || w === '""';
    }
    get str() {
      var w;
      return (w = this._str) !== null && w !== void 0 ? w : this._str = this._items.reduce((_, b) => `${_}${b}`, "");
    }
    get names() {
      var w;
      return (w = this._names) !== null && w !== void 0 ? w : this._names = this._items.reduce((_, b) => (b instanceof n && (_[b.str] = (_[b.str] || 0) + 1), _), {});
    }
  }
  e._Code = o, e.nil = new o("");
  function i(S, ...w) {
    const _ = [S[0]];
    let b = 0;
    for (; b < w.length; )
      u(_, w[b]), _.push(S[++b]);
    return new o(_);
  }
  e._ = i;
  const s = new o("+");
  function l(S, ...w) {
    const _ = [y(S[0])];
    let b = 0;
    for (; b < w.length; )
      _.push(s), u(_, w[b]), _.push(s, y(S[++b]));
    return c(_), new o(_);
  }
  e.str = l;
  function u(S, w) {
    w instanceof o ? S.push(...w._items) : w instanceof n ? S.push(w) : S.push(g(w));
  }
  e.addCodeArg = u;
  function c(S) {
    let w = 1;
    for (; w < S.length - 1; ) {
      if (S[w] === s) {
        const _ = f(S[w - 1], S[w + 1]);
        if (_ !== void 0) {
          S.splice(w - 1, 3, _);
          continue;
        }
        S[w++] = "+";
      }
      w++;
    }
  }
  function f(S, w) {
    if (w === '""')
      return S;
    if (S === '""')
      return w;
    if (typeof S == "string")
      return w instanceof n || S[S.length - 1] !== '"' ? void 0 : typeof w != "string" ? `${S.slice(0, -1)}${w}"` : w[0] === '"' ? S.slice(0, -1) + w.slice(1) : void 0;
    if (typeof w == "string" && w[0] === '"' && !(S instanceof n))
      return `"${S}${w.slice(1)}`;
  }
  function p(S, w) {
    return w.emptyStr() ? S : S.emptyStr() ? w : l`${S}${w}`;
  }
  e.strConcat = p;
  function g(S) {
    return typeof S == "number" || typeof S == "boolean" || S === null ? S : y(Array.isArray(S) ? S.join(",") : S);
  }
  function h(S) {
    return new o(y(S));
  }
  e.stringify = h;
  function y(S) {
    return JSON.stringify(S).replace(/\u2028/g, "\\u2028").replace(/\u2029/g, "\\u2029");
  }
  e.safeStringify = y;
  function v(S) {
    return typeof S == "string" && e.IDENTIFIER.test(S) ? new o(`.${S}`) : i`[${S}]`;
  }
  e.getProperty = v;
  function C(S) {
    if (typeof S == "string" && e.IDENTIFIER.test(S))
      return new o(`${S}`);
    throw new Error(`CodeGen: invalid export name: ${S}, use explicit $id name mapping`);
  }
  e.getEsmExportName = C;
  function $(S) {
    return new o(S.toString());
  }
  e.regexpCode = $;
})(mc);
var Ug = {};
(function(e) {
  Object.defineProperty(e, "__esModule", { value: !0 }), e.ValueScope = e.ValueScopeName = e.Scope = e.varKinds = e.UsedValueState = void 0;
  const t = mc;
  class n extends Error {
    constructor(f) {
      super(`CodeGen: "code" for ${f} not defined`), this.value = f.value;
    }
  }
  var o;
  (function(c) {
    c[c.Started = 0] = "Started", c[c.Completed = 1] = "Completed";
  })(o || (e.UsedValueState = o = {})), e.varKinds = {
    const: new t.Name("const"),
    let: new t.Name("let"),
    var: new t.Name("var")
  };
  class i {
    constructor({ prefixes: f, parent: p } = {}) {
      this._names = {}, this._prefixes = f, this._parent = p;
    }
    toName(f) {
      return f instanceof t.Name ? f : this.name(f);
    }
    name(f) {
      return new t.Name(this._newName(f));
    }
    _newName(f) {
      const p = this._names[f] || this._nameGroup(f);
      return `${f}${p.index++}`;
    }
    _nameGroup(f) {
      var p, g;
      if (!((g = (p = this._parent) === null || p === void 0 ? void 0 : p._prefixes) === null || g === void 0) && g.has(f) || this._prefixes && !this._prefixes.has(f))
        throw new Error(`CodeGen: prefix "${f}" is not allowed in this scope`);
      return this._names[f] = { prefix: f, index: 0 };
    }
  }
  e.Scope = i;
  class s extends t.Name {
    constructor(f, p) {
      super(p), this.prefix = f;
    }
    setValue(f, { property: p, itemIndex: g }) {
      this.value = f, this.scopePath = (0, t._)`.${new t.Name(p)}[${g}]`;
    }
  }
  e.ValueScopeName = s;
  const l = (0, t._)`\n`;
  class u extends i {
    constructor(f) {
      super(f), this._values = {}, this._scope = f.scope, this.opts = { ...f, _n: f.lines ? l : t.nil };
    }
    get() {
      return this._scope;
    }
    name(f) {
      return new s(f, this._newName(f));
    }
    value(f, p) {
      var g;
      if (p.ref === void 0)
        throw new Error("CodeGen: ref must be passed in value");
      const h = this.toName(f), { prefix: y } = h, v = (g = p.key) !== null && g !== void 0 ? g : p.ref;
      let C = this._values[y];
      if (C) {
        const w = C.get(v);
        if (w)
          return w;
      } else
        C = this._values[y] = /* @__PURE__ */ new Map();
      C.set(v, h);
      const $ = this._scope[y] || (this._scope[y] = []), S = $.length;
      return $[S] = p.ref, h.setValue(p, { property: y, itemIndex: S }), h;
    }
    getValue(f, p) {
      const g = this._values[f];
      if (g)
        return g.get(p);
    }
    scopeRefs(f, p = this._values) {
      return this._reduceValues(p, (g) => {
        if (g.scopePath === void 0)
          throw new Error(`CodeGen: name "${g}" has no value`);
        return (0, t._)`${f}${g.scopePath}`;
      });
    }
    scopeCode(f = this._values, p, g) {
      return this._reduceValues(f, (h) => {
        if (h.value === void 0)
          throw new Error(`CodeGen: name "${h}" has no value`);
        return h.value.code;
      }, p, g);
    }
    _reduceValues(f, p, g = {}, h) {
      let y = t.nil;
      for (const v in f) {
        const C = f[v];
        if (!C)
          continue;
        const $ = g[v] = g[v] || /* @__PURE__ */ new Map();
        C.forEach((S) => {
          if ($.has(S))
            return;
          $.set(S, o.Started);
          let w = p(S);
          if (w) {
            const _ = this.opts.es5 ? e.varKinds.var : e.varKinds.const;
            y = (0, t._)`${y}${_} ${S} = ${w};${this.opts._n}`;
          } else if (w = h == null ? void 0 : h(S))
            y = (0, t._)`${y}${w}${this.opts._n}`;
          else
            throw new n(S);
          $.set(S, o.Completed);
        });
      }
      return y;
    }
  }
  e.ValueScope = u;
})(Ug);
(function(e) {
  Object.defineProperty(e, "__esModule", { value: !0 }), e.or = e.and = e.not = e.CodeGen = e.operators = e.varKinds = e.ValueScopeName = e.ValueScope = e.Scope = e.Name = e.regexpCode = e.stringify = e.getProperty = e.nil = e.strConcat = e.str = e._ = void 0;
  const t = mc, n = Ug;
  var o = mc;
  Object.defineProperty(e, "_", { enumerable: !0, get: function() {
    return o._;
  } }), Object.defineProperty(e, "str", { enumerable: !0, get: function() {
    return o.str;
  } }), Object.defineProperty(e, "strConcat", { enumerable: !0, get: function() {
    return o.strConcat;
  } }), Object.defineProperty(e, "nil", { enumerable: !0, get: function() {
    return o.nil;
  } }), Object.defineProperty(e, "getProperty", { enumerable: !0, get: function() {
    return o.getProperty;
  } }), Object.defineProperty(e, "stringify", { enumerable: !0, get: function() {
    return o.stringify;
  } }), Object.defineProperty(e, "regexpCode", { enumerable: !0, get: function() {
    return o.regexpCode;
  } }), Object.defineProperty(e, "Name", { enumerable: !0, get: function() {
    return o.Name;
  } });
  var i = Ug;
  Object.defineProperty(e, "Scope", { enumerable: !0, get: function() {
    return i.Scope;
  } }), Object.defineProperty(e, "ValueScope", { enumerable: !0, get: function() {
    return i.ValueScope;
  } }), Object.defineProperty(e, "ValueScopeName", { enumerable: !0, get: function() {
    return i.ValueScopeName;
  } }), Object.defineProperty(e, "varKinds", { enumerable: !0, get: function() {
    return i.varKinds;
  } }), e.operators = {
    GT: new t._Code(">"),
    GTE: new t._Code(">="),
    LT: new t._Code("<"),
    LTE: new t._Code("<="),
    EQ: new t._Code("==="),
    NEQ: new t._Code("!=="),
    NOT: new t._Code("!"),
    OR: new t._Code("||"),
    AND: new t._Code("&&"),
    ADD: new t._Code("+")
  };
  class s {
    optimizeNodes() {
      return this;
    }
    optimizeNames(N, U) {
      return this;
    }
  }
  class l extends s {
    constructor(N, U, Z) {
      super(), this.varKind = N, this.name = U, this.rhs = Z;
    }
    render({ es5: N, _n: U }) {
      const Z = N ? n.varKinds.var : this.varKind, me = this.rhs === void 0 ? "" : ` = ${this.rhs}`;
      return `${Z} ${this.name}${me};` + U;
    }
    optimizeNames(N, U) {
      if (N[this.name.str])
        return this.rhs && (this.rhs = Q(this.rhs, N, U)), this;
    }
    get names() {
      return this.rhs instanceof t._CodeOrName ? this.rhs.names : {};
    }
  }
  class u extends s {
    constructor(N, U, Z) {
      super(), this.lhs = N, this.rhs = U, this.sideEffects = Z;
    }
    render({ _n: N }) {
      return `${this.lhs} = ${this.rhs};` + N;
    }
    optimizeNames(N, U) {
      if (!(this.lhs instanceof t.Name && !N[this.lhs.str] && !this.sideEffects))
        return this.rhs = Q(this.rhs, N, U), this;
    }
    get names() {
      const N = this.lhs instanceof t.Name ? {} : { ...this.lhs.names };
      return q(N, this.rhs);
    }
  }
  class c extends u {
    constructor(N, U, Z, me) {
      super(N, Z, me), this.op = U;
    }
    render({ _n: N }) {
      return `${this.lhs} ${this.op}= ${this.rhs};` + N;
    }
  }
  class f extends s {
    constructor(N) {
      super(), this.label = N, this.names = {};
    }
    render({ _n: N }) {
      return `${this.label}:` + N;
    }
  }
  class p extends s {
    constructor(N) {
      super(), this.label = N, this.names = {};
    }
    render({ _n: N }) {
      return `break${this.label ? ` ${this.label}` : ""};` + N;
    }
  }
  class g extends s {
    constructor(N) {
      super(), this.error = N;
    }
    render({ _n: N }) {
      return `throw ${this.error};` + N;
    }
    get names() {
      return this.error.names;
    }
  }
  class h extends s {
    constructor(N) {
      super(), this.code = N;
    }
    render({ _n: N }) {
      return `${this.code};` + N;
    }
    optimizeNodes() {
      return `${this.code}` ? this : void 0;
    }
    optimizeNames(N, U) {
      return this.code = Q(this.code, N, U), this;
    }
    get names() {
      return this.code instanceof t._CodeOrName ? this.code.names : {};
    }
  }
  class y extends s {
    constructor(N = []) {
      super(), this.nodes = N;
    }
    render(N) {
      return this.nodes.reduce((U, Z) => U + Z.render(N), "");
    }
    optimizeNodes() {
      const { nodes: N } = this;
      let U = N.length;
      for (; U--; ) {
        const Z = N[U].optimizeNodes();
        Array.isArray(Z) ? N.splice(U, 1, ...Z) : Z ? N[U] = Z : N.splice(U, 1);
      }
      return N.length > 0 ? this : void 0;
    }
    optimizeNames(N, U) {
      const { nodes: Z } = this;
      let me = Z.length;
      for (; me--; ) {
        const ue = Z[me];
        ue.optimizeNames(N, U) || (X(N, ue.names), Z.splice(me, 1));
      }
      return Z.length > 0 ? this : void 0;
    }
    get names() {
      return this.nodes.reduce((N, U) => W(N, U.names), {});
    }
  }
  class v extends y {
    render(N) {
      return "{" + N._n + super.render(N) + "}" + N._n;
    }
  }
  class C extends y {
  }
  class $ extends v {
  }
  $.kind = "else";
  class S extends v {
    constructor(N, U) {
      super(U), this.condition = N;
    }
    render(N) {
      let U = `if(${this.condition})` + super.render(N);
      return this.else && (U += "else " + this.else.render(N)), U;
    }
    optimizeNodes() {
      super.optimizeNodes();
      const N = this.condition;
      if (N === !0)
        return this.nodes;
      let U = this.else;
      if (U) {
        const Z = U.optimizeNodes();
        U = this.else = Array.isArray(Z) ? new $(Z) : Z;
      }
      if (U)
        return N === !1 ? U instanceof S ? U : U.nodes : this.nodes.length ? this : new S(ce(N), U instanceof S ? [U] : U.nodes);
      if (!(N === !1 || !this.nodes.length))
        return this;
    }
    optimizeNames(N, U) {
      var Z;
      if (this.else = (Z = this.else) === null || Z === void 0 ? void 0 : Z.optimizeNames(N, U), !!(super.optimizeNames(N, U) || this.else))
        return this.condition = Q(this.condition, N, U), this;
    }
    get names() {
      const N = super.names;
      return q(N, this.condition), this.else && W(N, this.else.names), N;
    }
  }
  S.kind = "if";
  class w extends v {
  }
  w.kind = "for";
  class _ extends w {
    constructor(N) {
      super(), this.iteration = N;
    }
    render(N) {
      return `for(${this.iteration})` + super.render(N);
    }
    optimizeNames(N, U) {
      if (super.optimizeNames(N, U))
        return this.iteration = Q(this.iteration, N, U), this;
    }
    get names() {
      return W(super.names, this.iteration.names);
    }
  }
  class b extends w {
    constructor(N, U, Z, me) {
      super(), this.varKind = N, this.name = U, this.from = Z, this.to = me;
    }
    render(N) {
      const U = N.es5 ? n.varKinds.var : this.varKind, { name: Z, from: me, to: ue } = this;
      return `for(${U} ${Z}=${me}; ${Z}<${ue}; ${Z}++)` + super.render(N);
    }
    get names() {
      const N = q(super.names, this.from);
      return q(N, this.to);
    }
  }
  class k extends w {
    constructor(N, U, Z, me) {
      super(), this.loop = N, this.varKind = U, this.name = Z, this.iterable = me;
    }
    render(N) {
      return `for(${this.varKind} ${this.name} ${this.loop} ${this.iterable})` + super.render(N);
    }
    optimizeNames(N, U) {
      if (super.optimizeNames(N, U))
        return this.iterable = Q(this.iterable, N, U), this;
    }
    get names() {
      return W(super.names, this.iterable.names);
    }
  }
  class E extends v {
    constructor(N, U, Z) {
      super(), this.name = N, this.args = U, this.async = Z;
    }
    render(N) {
      return `${this.async ? "async " : ""}function ${this.name}(${this.args})` + super.render(N);
    }
  }
  E.kind = "func";
  class j extends y {
    render(N) {
      return "return " + super.render(N);
    }
  }
  j.kind = "return";
  class L extends v {
    render(N) {
      let U = "try" + super.render(N);
      return this.catch && (U += this.catch.render(N)), this.finally && (U += this.finally.render(N)), U;
    }
    optimizeNodes() {
      var N, U;
      return super.optimizeNodes(), (N = this.catch) === null || N === void 0 || N.optimizeNodes(), (U = this.finally) === null || U === void 0 || U.optimizeNodes(), this;
    }
    optimizeNames(N, U) {
      var Z, me;
      return super.optimizeNames(N, U), (Z = this.catch) === null || Z === void 0 || Z.optimizeNames(N, U), (me = this.finally) === null || me === void 0 || me.optimizeNames(N, U), this;
    }
    get names() {
      const N = super.names;
      return this.catch && W(N, this.catch.names), this.finally && W(N, this.finally.names), N;
    }
  }
  class B extends v {
    constructor(N) {
      super(), this.error = N;
    }
    render(N) {
      return `catch(${this.error})` + super.render(N);
    }
  }
  B.kind = "catch";
  class O extends v {
    render(N) {
      return "finally" + super.render(N);
    }
  }
  O.kind = "finally";
  class V {
    constructor(N, U = {}) {
      this._values = {}, this._blockStarts = [], this._constants = {}, this.opts = { ...U, _n: U.lines ? `
` : "" }, this._extScope = N, this._scope = new n.Scope({ parent: N }), this._nodes = [new C()];
    }
    toString() {
      return this._root.render(this.opts);
    }
    // returns unique name in the internal scope
    name(N) {
      return this._scope.name(N);
    }
    // reserves unique name in the external scope
    scopeName(N) {
      return this._extScope.name(N);
    }
    // reserves unique name in the external scope and assigns value to it
    scopeValue(N, U) {
      const Z = this._extScope.value(N, U);
      return (this._values[Z.prefix] || (this._values[Z.prefix] = /* @__PURE__ */ new Set())).add(Z), Z;
    }
    getScopeValue(N, U) {
      return this._extScope.getValue(N, U);
    }
    // return code that assigns values in the external scope to the names that are used internally
    // (same names that were returned by gen.scopeName or gen.scopeValue)
    scopeRefs(N) {
      return this._extScope.scopeRefs(N, this._values);
    }
    scopeCode() {
      return this._extScope.scopeCode(this._values);
    }
    _def(N, U, Z, me) {
      const ue = this._scope.toName(U);
      return Z !== void 0 && me && (this._constants[ue.str] = Z), this._leafNode(new l(N, ue, Z)), ue;
    }
    // `const` declaration (`var` in es5 mode)
    const(N, U, Z) {
      return this._def(n.varKinds.const, N, U, Z);
    }
    // `let` declaration with optional assignment (`var` in es5 mode)
    let(N, U, Z) {
      return this._def(n.varKinds.let, N, U, Z);
    }
    // `var` declaration with optional assignment
    var(N, U, Z) {
      return this._def(n.varKinds.var, N, U, Z);
    }
    // assignment code
    assign(N, U, Z) {
      return this._leafNode(new u(N, U, Z));
    }
    // `+=` code
    add(N, U) {
      return this._leafNode(new c(N, e.operators.ADD, U));
    }
    // appends passed SafeExpr to code or executes Block
    code(N) {
      return typeof N == "function" ? N() : N !== t.nil && this._leafNode(new h(N)), this;
    }
    // returns code for object literal for the passed argument list of key-value pairs
    object(...N) {
      const U = ["{"];
      for (const [Z, me] of N)
        U.length > 1 && U.push(","), U.push(Z), (Z !== me || this.opts.es5) && (U.push(":"), (0, t.addCodeArg)(U, me));
      return U.push("}"), new t._Code(U);
    }
    // `if` clause (or statement if `thenBody` and, optionally, `elseBody` are passed)
    if(N, U, Z) {
      if (this._blockNode(new S(N)), U && Z)
        this.code(U).else().code(Z).endIf();
      else if (U)
        this.code(U).endIf();
      else if (Z)
        throw new Error('CodeGen: "else" body without "then" body');
      return this;
    }
    // `else if` clause - invalid without `if` or after `else` clauses
    elseIf(N) {
      return this._elseNode(new S(N));
    }
    // `else` clause - only valid after `if` or `else if` clauses
    else() {
      return this._elseNode(new $());
    }
    // end `if` statement (needed if gen.if was used only with condition)
    endIf() {
      return this._endBlockNode(S, $);
    }
    _for(N, U) {
      return this._blockNode(N), U && this.code(U).endFor(), this;
    }
    // a generic `for` clause (or statement if `forBody` is passed)
    for(N, U) {
      return this._for(new _(N), U);
    }
    // `for` statement for a range of values
    forRange(N, U, Z, me, ue = this.opts.es5 ? n.varKinds.var : n.varKinds.let) {
      const Ce = this._scope.toName(N);
      return this._for(new b(ue, Ce, U, Z), () => me(Ce));
    }
    // `for-of` statement (in es5 mode replace with a normal for loop)
    forOf(N, U, Z, me = n.varKinds.const) {
      const ue = this._scope.toName(N);
      if (this.opts.es5) {
        const Ce = U instanceof t.Name ? U : this.var("_arr", U);
        return this.forRange("_i", 0, (0, t._)`${Ce}.length`, ($e) => {
          this.var(ue, (0, t._)`${Ce}[${$e}]`), Z(ue);
        });
      }
      return this._for(new k("of", me, ue, U), () => Z(ue));
    }
    // `for-in` statement.
    // With option `ownProperties` replaced with a `for-of` loop for object keys
    forIn(N, U, Z, me = this.opts.es5 ? n.varKinds.var : n.varKinds.const) {
      if (this.opts.ownProperties)
        return this.forOf(N, (0, t._)`Object.keys(${U})`, Z);
      const ue = this._scope.toName(N);
      return this._for(new k("in", me, ue, U), () => Z(ue));
    }
    // end `for` loop
    endFor() {
      return this._endBlockNode(w);
    }
    // `label` statement
    label(N) {
      return this._leafNode(new f(N));
    }
    // `break` statement
    break(N) {
      return this._leafNode(new p(N));
    }
    // `return` statement
    return(N) {
      const U = new j();
      if (this._blockNode(U), this.code(N), U.nodes.length !== 1)
        throw new Error('CodeGen: "return" should have one node');
      return this._endBlockNode(j);
    }
    // `try` statement
    try(N, U, Z) {
      if (!U && !Z)
        throw new Error('CodeGen: "try" without "catch" and "finally"');
      const me = new L();
      if (this._blockNode(me), this.code(N), U) {
        const ue = this.name("e");
        this._currNode = me.catch = new B(ue), U(ue);
      }
      return Z && (this._currNode = me.finally = new O(), this.code(Z)), this._endBlockNode(B, O);
    }
    // `throw` statement
    throw(N) {
      return this._leafNode(new g(N));
    }
    // start self-balancing block
    block(N, U) {
      return this._blockStarts.push(this._nodes.length), N && this.code(N).endBlock(U), this;
    }
    // end the current self-balancing block
    endBlock(N) {
      const U = this._blockStarts.pop();
      if (U === void 0)
        throw new Error("CodeGen: not in self-balancing block");
      const Z = this._nodes.length - U;
      if (Z < 0 || N !== void 0 && Z !== N)
        throw new Error(`CodeGen: wrong number of nodes: ${Z} vs ${N} expected`);
      return this._nodes.length = U, this;
    }
    // `function` heading (or definition if funcBody is passed)
    func(N, U = t.nil, Z, me) {
      return this._blockNode(new E(N, U, Z)), me && this.code(me).endFunc(), this;
    }
    // end function definition
    endFunc() {
      return this._endBlockNode(E);
    }
    optimize(N = 1) {
      for (; N-- > 0; )
        this._root.optimizeNodes(), this._root.optimizeNames(this._root.names, this._constants);
    }
    _leafNode(N) {
      return this._currNode.nodes.push(N), this;
    }
    _blockNode(N) {
      this._currNode.nodes.push(N), this._nodes.push(N);
    }
    _endBlockNode(N, U) {
      const Z = this._currNode;
      if (Z instanceof N || U && Z instanceof U)
        return this._nodes.pop(), this;
      throw new Error(`CodeGen: not in block "${U ? `${N.kind}/${U.kind}` : N.kind}"`);
    }
    _elseNode(N) {
      const U = this._currNode;
      if (!(U instanceof S))
        throw new Error('CodeGen: "else" without "if"');
      return this._currNode = U.else = N, this;
    }
    get _root() {
      return this._nodes[0];
    }
    get _currNode() {
      const N = this._nodes;
      return N[N.length - 1];
    }
    set _currNode(N) {
      const U = this._nodes;
      U[U.length - 1] = N;
    }
  }
  e.CodeGen = V;
  function W(G, N) {
    for (const U in N)
      G[U] = (G[U] || 0) + (N[U] || 0);
    return G;
  }
  function q(G, N) {
    return N instanceof t._CodeOrName ? W(G, N.names) : G;
  }
  function Q(G, N, U) {
    if (G instanceof t.Name)
      return Z(G);
    if (!me(G))
      return G;
    return new t._Code(G._items.reduce((ue, Ce) => (Ce instanceof t.Name && (Ce = Z(Ce)), Ce instanceof t._Code ? ue.push(...Ce._items) : ue.push(Ce), ue), []));
    function Z(ue) {
      const Ce = U[ue.str];
      return Ce === void 0 || N[ue.str] !== 1 ? ue : (delete N[ue.str], Ce);
    }
    function me(ue) {
      return ue instanceof t._Code && ue._items.some((Ce) => Ce instanceof t.Name && N[Ce.str] === 1 && U[Ce.str] !== void 0);
    }
  }
  function X(G, N) {
    for (const U in N)
      G[U] = (G[U] || 0) - (N[U] || 0);
  }
  function ce(G) {
    return typeof G == "boolean" || typeof G == "number" || G === null ? !G : (0, t._)`!${H(G)}`;
  }
  e.not = ce;
  const J = T(e.operators.AND);
  function ie(...G) {
    return G.reduce(J);
  }
  e.and = ie;
  const re = T(e.operators.OR);
  function Y(...G) {
    return G.reduce(re);
  }
  e.or = Y;
  function T(G) {
    return (N, U) => N === t.nil ? U : U === t.nil ? N : (0, t._)`${H(N)} ${G} ${H(U)}`;
  }
  function H(G) {
    return G instanceof t.Name ? G : (0, t._)`(${G})`;
  }
})(St);
var ze = {};
Object.defineProperty(ze, "__esModule", { value: !0 });
ze.checkStrictMode = ze.getErrorPath = ze.Type = ze.useFunc = ze.setEvaluated = ze.evaluatedPropsToName = ze.mergeEvaluated = ze.eachItem = ze.unescapeJsonPointer = ze.escapeJsonPointer = ze.escapeFragment = ze.unescapeFragment = ze.schemaRefOrVal = ze.schemaHasRulesButRef = ze.schemaHasRules = ze.checkUnknownRules = ze.alwaysValidSchema = ze.toHash = void 0;
const en = St, GZ = mc;
function JZ(e) {
  const t = {};
  for (const n of e)
    t[n] = !0;
  return t;
}
ze.toHash = JZ;
function YZ(e, t) {
  return typeof t == "boolean" ? t : Object.keys(t).length === 0 ? !0 : (BE(e, t), !UE(t, e.self.RULES.all));
}
ze.alwaysValidSchema = YZ;
function BE(e, t = e.schema) {
  const { opts: n, self: o } = e;
  if (!n.strictSchema || typeof t == "boolean")
    return;
  const i = o.RULES.keywords;
  for (const s in t)
    i[s] || qE(e, `unknown keyword: "${s}"`);
}
ze.checkUnknownRules = BE;
function UE(e, t) {
  if (typeof e == "boolean")
    return !e;
  for (const n in e)
    if (t[n])
      return !0;
  return !1;
}
ze.schemaHasRules = UE;
function QZ(e, t) {
  if (typeof e == "boolean")
    return !e;
  for (const n in e)
    if (n !== "$ref" && t.all[n])
      return !0;
  return !1;
}
ze.schemaHasRulesButRef = QZ;
function XZ({ topSchemaRef: e, schemaPath: t }, n, o, i) {
  if (!i) {
    if (typeof n == "number" || typeof n == "boolean")
      return n;
    if (typeof n == "string")
      return (0, en._)`${n}`;
  }
  return (0, en._)`${e}${t}${(0, en.getProperty)(o)}`;
}
ze.schemaRefOrVal = XZ;
function ZZ(e) {
  return VE(decodeURIComponent(e));
}
ze.unescapeFragment = ZZ;
function eee(e) {
  return encodeURIComponent(j0(e));
}
ze.escapeFragment = eee;
function j0(e) {
  return typeof e == "number" ? `${e}` : e.replace(/~/g, "~0").replace(/\//g, "~1");
}
ze.escapeJsonPointer = j0;
function VE(e) {
  return e.replace(/~1/g, "/").replace(/~0/g, "~");
}
ze.unescapeJsonPointer = VE;
function tee(e, t) {
  if (Array.isArray(e))
    for (const n of e)
      t(n);
  else
    t(e);
}
ze.eachItem = tee;
function Pw({ mergeNames: e, mergeToName: t, mergeValues: n, resultToName: o }) {
  return (i, s, l, u) => {
    const c = l === void 0 ? s : l instanceof en.Name ? (s instanceof en.Name ? e(i, s, l) : t(i, s, l), l) : s instanceof en.Name ? (t(i, l, s), s) : n(s, l);
    return u === en.Name && !(c instanceof en.Name) ? o(i, c) : c;
  };
}
ze.mergeEvaluated = {
  props: Pw({
    mergeNames: (e, t, n) => e.if((0, en._)`${n} !== true && ${t} !== undefined`, () => {
      e.if((0, en._)`${t} === true`, () => e.assign(n, !0), () => e.assign(n, (0, en._)`${n} || {}`).code((0, en._)`Object.assign(${n}, ${t})`));
    }),
    mergeToName: (e, t, n) => e.if((0, en._)`${n} !== true`, () => {
      t === !0 ? e.assign(n, !0) : (e.assign(n, (0, en._)`${n} || {}`), R0(e, n, t));
    }),
    mergeValues: (e, t) => e === !0 ? !0 : { ...e, ...t },
    resultToName: WE
  }),
  items: Pw({
    mergeNames: (e, t, n) => e.if((0, en._)`${n} !== true && ${t} !== undefined`, () => e.assign(n, (0, en._)`${t} === true ? true : ${n} > ${t} ? ${n} : ${t}`)),
    mergeToName: (e, t, n) => e.if((0, en._)`${n} !== true`, () => e.assign(n, t === !0 ? !0 : (0, en._)`${n} > ${t} ? ${n} : ${t}`)),
    mergeValues: (e, t) => e === !0 ? !0 : Math.max(e, t),
    resultToName: (e, t) => e.var("items", t)
  })
};
function WE(e, t) {
  if (t === !0)
    return e.var("props", !0);
  const n = e.var("props", (0, en._)`{}`);
  return t !== void 0 && R0(e, n, t), n;
}
ze.evaluatedPropsToName = WE;
function R0(e, t, n) {
  Object.keys(n).forEach((o) => e.assign((0, en._)`${t}${(0, en.getProperty)(o)}`, !0));
}
ze.setEvaluated = R0;
const Ow = {};
function nee(e, t) {
  return e.scopeValue("func", {
    ref: t,
    code: Ow[t.code] || (Ow[t.code] = new GZ._Code(t.code))
  });
}
ze.useFunc = nee;
var Vg;
(function(e) {
  e[e.Num = 0] = "Num", e[e.Str = 1] = "Str";
})(Vg || (ze.Type = Vg = {}));
function ree(e, t, n) {
  if (e instanceof en.Name) {
    const o = t === Vg.Num;
    return n ? o ? (0, en._)`"[" + ${e} + "]"` : (0, en._)`"['" + ${e} + "']"` : o ? (0, en._)`"/" + ${e}` : (0, en._)`"/" + ${e}.replace(/~/g, "~0").replace(/\\//g, "~1")`;
  }
  return n ? (0, en.getProperty)(e).toString() : "/" + j0(e);
}
ze.getErrorPath = ree;
function qE(e, t, n = e.opts.strictSchema) {
  if (n) {
    if (t = `strict mode: ${t}`, n === !0)
      throw new Error(t);
    e.self.logger.warn(t);
  }
}
ze.checkStrictMode = qE;
var ni = {};
Object.defineProperty(ni, "__esModule", { value: !0 });
const ir = St, oee = {
  // validation function arguments
  data: new ir.Name("data"),
  // data passed to validation function
  // args passed from referencing schema
  valCxt: new ir.Name("valCxt"),
  // validation/data context - should not be used directly, it is destructured to the names below
  instancePath: new ir.Name("instancePath"),
  parentData: new ir.Name("parentData"),
  parentDataProperty: new ir.Name("parentDataProperty"),
  rootData: new ir.Name("rootData"),
  // root data - same as the data passed to the first/top validation function
  dynamicAnchors: new ir.Name("dynamicAnchors"),
  // used to support recursiveRef and dynamicRef
  // function scoped variables
  vErrors: new ir.Name("vErrors"),
  // null or array of validation errors
  errors: new ir.Name("errors"),
  // counter of validation errors
  this: new ir.Name("this"),
  // "globals"
  self: new ir.Name("self"),
  scope: new ir.Name("scope"),
  // JTD serialize/parse name for JSON string and position
  json: new ir.Name("json"),
  jsonPos: new ir.Name("jsonPos"),
  jsonLen: new ir.Name("jsonLen"),
  jsonPart: new ir.Name("jsonPart")
};
ni.default = oee;
(function(e) {
  Object.defineProperty(e, "__esModule", { value: !0 }), e.extendErrors = e.resetErrorsCount = e.reportExtraError = e.reportError = e.keyword$DataError = e.keywordError = void 0;
  const t = St, n = ze, o = ni;
  e.keywordError = {
    message: ({ keyword: $ }) => (0, t.str)`must pass "${$}" keyword validation`
  }, e.keyword$DataError = {
    message: ({ keyword: $, schemaType: S }) => S ? (0, t.str)`"${$}" keyword must be ${S} ($data)` : (0, t.str)`"${$}" keyword is invalid ($data)`
  };
  function i($, S = e.keywordError, w, _) {
    const { it: b } = $, { gen: k, compositeRule: E, allErrors: j } = b, L = g($, S, w);
    _ ?? (E || j) ? c(k, L) : f(b, (0, t._)`[${L}]`);
  }
  e.reportError = i;
  function s($, S = e.keywordError, w) {
    const { it: _ } = $, { gen: b, compositeRule: k, allErrors: E } = _, j = g($, S, w);
    c(b, j), k || E || f(_, o.default.vErrors);
  }
  e.reportExtraError = s;
  function l($, S) {
    $.assign(o.default.errors, S), $.if((0, t._)`${o.default.vErrors} !== null`, () => $.if(S, () => $.assign((0, t._)`${o.default.vErrors}.length`, S), () => $.assign(o.default.vErrors, null)));
  }
  e.resetErrorsCount = l;
  function u({ gen: $, keyword: S, schemaValue: w, data: _, errsCount: b, it: k }) {
    if (b === void 0)
      throw new Error("ajv implementation error");
    const E = $.name("err");
    $.forRange("i", b, o.default.errors, (j) => {
      $.const(E, (0, t._)`${o.default.vErrors}[${j}]`), $.if((0, t._)`${E}.instancePath === undefined`, () => $.assign((0, t._)`${E}.instancePath`, (0, t.strConcat)(o.default.instancePath, k.errorPath))), $.assign((0, t._)`${E}.schemaPath`, (0, t.str)`${k.errSchemaPath}/${S}`), k.opts.verbose && ($.assign((0, t._)`${E}.schema`, w), $.assign((0, t._)`${E}.data`, _));
    });
  }
  e.extendErrors = u;
  function c($, S) {
    const w = $.const("err", S);
    $.if((0, t._)`${o.default.vErrors} === null`, () => $.assign(o.default.vErrors, (0, t._)`[${w}]`), (0, t._)`${o.default.vErrors}.push(${w})`), $.code((0, t._)`${o.default.errors}++`);
  }
  function f($, S) {
    const { gen: w, validateName: _, schemaEnv: b } = $;
    b.$async ? w.throw((0, t._)`new ${$.ValidationError}(${S})`) : (w.assign((0, t._)`${_}.errors`, S), w.return(!1));
  }
  const p = {
    keyword: new t.Name("keyword"),
    schemaPath: new t.Name("schemaPath"),
    // also used in JTD errors
    params: new t.Name("params"),
    propertyName: new t.Name("propertyName"),
    message: new t.Name("message"),
    schema: new t.Name("schema"),
    parentSchema: new t.Name("parentSchema")
  };
  function g($, S, w) {
    const { createErrors: _ } = $.it;
    return _ === !1 ? (0, t._)`{}` : h($, S, w);
  }
  function h($, S, w = {}) {
    const { gen: _, it: b } = $, k = [
      y(b, w),
      v($, w)
    ];
    return C($, S, k), _.object(...k);
  }
  function y({ errorPath: $ }, { instancePath: S }) {
    const w = S ? (0, t.str)`${$}${(0, n.getErrorPath)(S, n.Type.Str)}` : $;
    return [o.default.instancePath, (0, t.strConcat)(o.default.instancePath, w)];
  }
  function v({ keyword: $, it: { errSchemaPath: S } }, { schemaPath: w, parentSchema: _ }) {
    let b = _ ? S : (0, t.str)`${S}/${$}`;
    return w && (b = (0, t.str)`${b}${(0, n.getErrorPath)(w, n.Type.Str)}`), [p.schemaPath, b];
  }
  function C($, { params: S, message: w }, _) {
    const { keyword: b, data: k, schemaValue: E, it: j } = $, { opts: L, propertyName: B, topSchemaRef: O, schemaPath: V } = j;
    _.push([p.keyword, b], [p.params, typeof S == "function" ? S($) : S || (0, t._)`{}`]), L.messages && _.push([p.message, typeof w == "function" ? w($) : w]), L.verbose && _.push([p.schema, E], [p.parentSchema, (0, t._)`${O}${V}`], [o.default.data, k]), B && _.push([p.propertyName, B]);
  }
})(Mc);
Object.defineProperty(pl, "__esModule", { value: !0 });
pl.boolOrEmptySchema = pl.topBoolOrEmptySchema = void 0;
const iee = Mc, aee = St, see = ni, lee = {
  message: "boolean schema is false"
};
function uee(e) {
  const { gen: t, schema: n, validateName: o } = e;
  n === !1 ? KE(e, !1) : typeof n == "object" && n.$async === !0 ? t.return(see.default.data) : (t.assign((0, aee._)`${o}.errors`, null), t.return(!0));
}
pl.topBoolOrEmptySchema = uee;
function cee(e, t) {
  const { gen: n, schema: o } = e;
  o === !1 ? (n.var(t, !1), KE(e)) : n.var(t, !0);
}
pl.boolOrEmptySchema = cee;
function KE(e, t) {
  const { gen: n, data: o } = e, i = {
    gen: n,
    keyword: "false schema",
    data: o,
    schema: !1,
    schemaCode: !1,
    schemaValue: !1,
    params: {},
    it: e
  };
  (0, iee.reportError)(i, lee, void 0, t);
}
var Ln = {}, Wa = {};
Object.defineProperty(Wa, "__esModule", { value: !0 });
Wa.getRules = Wa.isJSONType = void 0;
const dee = ["string", "number", "integer", "boolean", "null", "object", "array"], fee = new Set(dee);
function pee(e) {
  return typeof e == "string" && fee.has(e);
}
Wa.isJSONType = pee;
function hee() {
  const e = {
    number: { type: "number", rules: [] },
    string: { type: "string", rules: [] },
    array: { type: "array", rules: [] },
    object: { type: "object", rules: [] }
  };
  return {
    types: { ...e, integer: !0, boolean: !0, null: !0 },
    rules: [{ rules: [] }, e.number, e.string, e.array, e.object],
    post: { rules: [] },
    all: {},
    keywords: {}
  };
}
Wa.getRules = hee;
var Si = {};
Object.defineProperty(Si, "__esModule", { value: !0 });
Si.shouldUseRule = Si.shouldUseGroup = Si.schemaHasRulesForType = void 0;
function mee({ schema: e, self: t }, n) {
  const o = t.RULES.types[n];
  return o && o !== !0 && HE(e, o);
}
Si.schemaHasRulesForType = mee;
function HE(e, t) {
  return t.rules.some((n) => GE(e, n));
}
Si.shouldUseGroup = HE;
function GE(e, t) {
  var n;
  return e[t.keyword] !== void 0 || ((n = t.definition.implements) === null || n === void 0 ? void 0 : n.some((o) => e[o] !== void 0));
}
Si.shouldUseRule = GE;
Object.defineProperty(Ln, "__esModule", { value: !0 });
Ln.reportTypeError = Ln.checkDataTypes = Ln.checkDataType = Ln.coerceAndCheckDataType = Ln.getJSONTypes = Ln.getSchemaTypes = Ln.DataType = void 0;
const gee = Wa, yee = Si, vee = Mc, vt = St, JE = ze;
var Zs;
(function(e) {
  e[e.Correct = 0] = "Correct", e[e.Wrong = 1] = "Wrong";
})(Zs || (Ln.DataType = Zs = {}));
function bee(e) {
  const t = YE(e.type);
  if (t.includes("null")) {
    if (e.nullable === !1)
      throw new Error("type: null contradicts nullable: false");
  } else {
    if (!t.length && e.nullable !== void 0)
      throw new Error('"nullable" cannot be used without "type"');
    e.nullable === !0 && t.push("null");
  }
  return t;
}
Ln.getSchemaTypes = bee;
function YE(e) {
  const t = Array.isArray(e) ? e : e ? [e] : [];
  if (t.every(gee.isJSONType))
    return t;
  throw new Error("type must be JSONType or JSONType[]: " + t.join(","));
}
Ln.getJSONTypes = YE;
function See(e, t) {
  const { gen: n, data: o, opts: i } = e, s = wee(t, i.coerceTypes), l = t.length > 0 && !(s.length === 0 && t.length === 1 && (0, yee.schemaHasRulesForType)(e, t[0]));
  if (l) {
    const u = T0(t, o, i.strictNumbers, Zs.Wrong);
    n.if(u, () => {
      s.length ? $ee(e, t, s) : M0(e);
    });
  }
  return l;
}
Ln.coerceAndCheckDataType = See;
const QE = /* @__PURE__ */ new Set(["string", "number", "integer", "boolean", "null"]);
function wee(e, t) {
  return t ? e.filter((n) => QE.has(n) || t === "array" && n === "array") : [];
}
function $ee(e, t, n) {
  const { gen: o, data: i, opts: s } = e, l = o.let("dataType", (0, vt._)`typeof ${i}`), u = o.let("coerced", (0, vt._)`undefined`);
  s.coerceTypes === "array" && o.if((0, vt._)`${l} == 'object' && Array.isArray(${i}) && ${i}.length == 1`, () => o.assign(i, (0, vt._)`${i}[0]`).assign(l, (0, vt._)`typeof ${i}`).if(T0(t, i, s.strictNumbers), () => o.assign(u, i))), o.if((0, vt._)`${u} !== undefined`);
  for (const f of n)
    (QE.has(f) || f === "array" && s.coerceTypes === "array") && c(f);
  o.else(), M0(e), o.endIf(), o.if((0, vt._)`${u} !== undefined`, () => {
    o.assign(i, u), Cee(e, u);
  });
  function c(f) {
    switch (f) {
      case "string":
        o.elseIf((0, vt._)`${l} == "number" || ${l} == "boolean"`).assign(u, (0, vt._)`"" + ${i}`).elseIf((0, vt._)`${i} === null`).assign(u, (0, vt._)`""`);
        return;
      case "number":
        o.elseIf((0, vt._)`${l} == "boolean" || ${i} === null
              || (${l} == "string" && ${i} && ${i} == +${i})`).assign(u, (0, vt._)`+${i}`);
        return;
      case "integer":
        o.elseIf((0, vt._)`${l} === "boolean" || ${i} === null
              || (${l} === "string" && ${i} && ${i} == +${i} && !(${i} % 1))`).assign(u, (0, vt._)`+${i}`);
        return;
      case "boolean":
        o.elseIf((0, vt._)`${i} === "false" || ${i} === 0 || ${i} === null`).assign(u, !1).elseIf((0, vt._)`${i} === "true" || ${i} === 1`).assign(u, !0);
        return;
      case "null":
        o.elseIf((0, vt._)`${i} === "" || ${i} === 0 || ${i} === false`), o.assign(u, null);
        return;
      case "array":
        o.elseIf((0, vt._)`${l} === "string" || ${l} === "number"
              || ${l} === "boolean" || ${i} === null`).assign(u, (0, vt._)`[${i}]`);
    }
  }
}
function Cee({ gen: e, parentData: t, parentDataProperty: n }, o) {
  e.if((0, vt._)`${t} !== undefined`, () => e.assign((0, vt._)`${t}[${n}]`, o));
}
function Wg(e, t, n, o = Zs.Correct) {
  const i = o === Zs.Correct ? vt.operators.EQ : vt.operators.NEQ;
  let s;
  switch (e) {
    case "null":
      return (0, vt._)`${t} ${i} null`;
    case "array":
      s = (0, vt._)`Array.isArray(${t})`;
      break;
    case "object":
      s = (0, vt._)`${t} && typeof ${t} == "object" && !Array.isArray(${t})`;
      break;
    case "integer":
      s = l((0, vt._)`!(${t} % 1) && !isNaN(${t})`);
      break;
    case "number":
      s = l();
      break;
    default:
      return (0, vt._)`typeof ${t} ${i} ${e}`;
  }
  return o === Zs.Correct ? s : (0, vt.not)(s);
  function l(u = vt.nil) {
    return (0, vt.and)((0, vt._)`typeof ${t} == "number"`, u, n ? (0, vt._)`isFinite(${t})` : vt.nil);
  }
}
Ln.checkDataType = Wg;
function T0(e, t, n, o) {
  if (e.length === 1)
    return Wg(e[0], t, n, o);
  let i;
  const s = (0, JE.toHash)(e);
  if (s.array && s.object) {
    const l = (0, vt._)`typeof ${t} != "object"`;
    i = s.null ? l : (0, vt._)`!${t} || ${l}`, delete s.null, delete s.array, delete s.object;
  } else
    i = vt.nil;
  s.number && delete s.integer;
  for (const l in s)
    i = (0, vt.and)(i, Wg(l, t, n, o));
  return i;
}
Ln.checkDataTypes = T0;
const xee = {
  message: ({ schema: e }) => `must be ${e}`,
  params: ({ schema: e, schemaValue: t }) => typeof e == "string" ? (0, vt._)`{type: ${e}}` : (0, vt._)`{type: ${t}}`
};
function M0(e) {
  const t = Eee(e);
  (0, vee.reportError)(t, xee);
}
Ln.reportTypeError = M0;
function Eee(e) {
  const { gen: t, data: n, schema: o } = e, i = (0, JE.schemaRefOrVal)(e, o, "type");
  return {
    gen: t,
    keyword: "type",
    data: n,
    schema: o.type,
    schemaCode: i,
    schemaValue: i,
    parentSchema: o,
    params: {},
    it: e
  };
}
var Kp = {};
Object.defineProperty(Kp, "__esModule", { value: !0 });
Kp.assignDefaults = void 0;
const zs = St, _ee = ze;
function kee(e, t) {
  const { properties: n, items: o } = e.schema;
  if (t === "object" && n)
    for (const i in n)
      Aw(e, i, n[i].default);
  else t === "array" && Array.isArray(o) && o.forEach((i, s) => Aw(e, s, i.default));
}
Kp.assignDefaults = kee;
function Aw(e, t, n) {
  const { gen: o, compositeRule: i, data: s, opts: l } = e;
  if (n === void 0)
    return;
  const u = (0, zs._)`${s}${(0, zs.getProperty)(t)}`;
  if (i) {
    (0, _ee.checkStrictMode)(e, `default is ignored for: ${u}`);
    return;
  }
  let c = (0, zs._)`${u} === undefined`;
  l.useDefaults === "empty" && (c = (0, zs._)`${c} || ${u} === null || ${u} === ""`), o.if(c, (0, zs._)`${u} = ${(0, zs.stringify)(n)}`);
}
var Yo = {}, Et = {};
Object.defineProperty(Et, "__esModule", { value: !0 });
Et.validateUnion = Et.validateArray = Et.usePattern = Et.callValidateCode = Et.schemaProperties = Et.allSchemaProperties = Et.noPropertyInData = Et.propertyInData = Et.isOwnProperty = Et.hasPropFunc = Et.reportMissingProp = Et.checkMissingProp = Et.checkReportMissingProp = void 0;
const un = St, N0 = ze, Ki = ni, Pee = ze;
function Oee(e, t) {
  const { gen: n, data: o, it: i } = e;
  n.if(L0(n, o, t, i.opts.ownProperties), () => {
    e.setParams({ missingProperty: (0, un._)`${t}` }, !0), e.error();
  });
}
Et.checkReportMissingProp = Oee;
function Aee({ gen: e, data: t, it: { opts: n } }, o, i) {
  return (0, un.or)(...o.map((s) => (0, un.and)(L0(e, t, s, n.ownProperties), (0, un._)`${i} = ${s}`)));
}
Et.checkMissingProp = Aee;
function Iee(e, t) {
  e.setParams({ missingProperty: t }, !0), e.error();
}
Et.reportMissingProp = Iee;
function XE(e) {
  return e.scopeValue("func", {
    // eslint-disable-next-line @typescript-eslint/unbound-method
    ref: Object.prototype.hasOwnProperty,
    code: (0, un._)`Object.prototype.hasOwnProperty`
  });
}
Et.hasPropFunc = XE;
function F0(e, t, n) {
  return (0, un._)`${XE(e)}.call(${t}, ${n})`;
}
Et.isOwnProperty = F0;
function jee(e, t, n, o) {
  const i = (0, un._)`${t}${(0, un.getProperty)(n)} !== undefined`;
  return o ? (0, un._)`${i} && ${F0(e, t, n)}` : i;
}
Et.propertyInData = jee;
function L0(e, t, n, o) {
  const i = (0, un._)`${t}${(0, un.getProperty)(n)} === undefined`;
  return o ? (0, un.or)(i, (0, un.not)(F0(e, t, n))) : i;
}
Et.noPropertyInData = L0;
function ZE(e) {
  return e ? Object.keys(e).filter((t) => t !== "__proto__") : [];
}
Et.allSchemaProperties = ZE;
function Ree(e, t) {
  return ZE(t).filter((n) => !(0, N0.alwaysValidSchema)(e, t[n]));
}
Et.schemaProperties = Ree;
function Tee({ schemaCode: e, data: t, it: { gen: n, topSchemaRef: o, schemaPath: i, errorPath: s }, it: l }, u, c, f) {
  const p = f ? (0, un._)`${e}, ${t}, ${o}${i}` : t, g = [
    [Ki.default.instancePath, (0, un.strConcat)(Ki.default.instancePath, s)],
    [Ki.default.parentData, l.parentData],
    [Ki.default.parentDataProperty, l.parentDataProperty],
    [Ki.default.rootData, Ki.default.rootData]
  ];
  l.opts.dynamicRef && g.push([Ki.default.dynamicAnchors, Ki.default.dynamicAnchors]);
  const h = (0, un._)`${p}, ${n.object(...g)}`;
  return c !== un.nil ? (0, un._)`${u}.call(${c}, ${h})` : (0, un._)`${u}(${h})`;
}
Et.callValidateCode = Tee;
const Mee = (0, un._)`new RegExp`;
function Nee({ gen: e, it: { opts: t } }, n) {
  const o = t.unicodeRegExp ? "u" : "", { regExp: i } = t.code, s = i(n, o);
  return e.scopeValue("pattern", {
    key: s.toString(),
    ref: s,
    code: (0, un._)`${i.code === "new RegExp" ? Mee : (0, Pee.useFunc)(e, i)}(${n}, ${o})`
  });
}
Et.usePattern = Nee;
function Fee(e) {
  const { gen: t, data: n, keyword: o, it: i } = e, s = t.name("valid");
  if (i.allErrors) {
    const u = t.let("valid", !0);
    return l(() => t.assign(u, !1)), u;
  }
  return t.var(s, !0), l(() => t.break()), s;
  function l(u) {
    const c = t.const("len", (0, un._)`${n}.length`);
    t.forRange("i", 0, c, (f) => {
      e.subschema({
        keyword: o,
        dataProp: f,
        dataPropType: N0.Type.Num
      }, s), t.if((0, un.not)(s), u);
    });
  }
}
Et.validateArray = Fee;
function Lee(e) {
  const { gen: t, schema: n, keyword: o, it: i } = e;
  if (!Array.isArray(n))
    throw new Error("ajv implementation error");
  if (n.some((c) => (0, N0.alwaysValidSchema)(i, c)) && !i.opts.unevaluated)
    return;
  const l = t.let("valid", !1), u = t.name("_valid");
  t.block(() => n.forEach((c, f) => {
    const p = e.subschema({
      keyword: o,
      schemaProp: f,
      compositeRule: !0
    }, u);
    t.assign(l, (0, un._)`${l} || ${u}`), e.mergeValidEvaluated(p, u) || t.if((0, un.not)(l));
  })), e.result(l, () => e.reset(), () => e.error(!0));
}
Et.validateUnion = Lee;
Object.defineProperty(Yo, "__esModule", { value: !0 });
Yo.validateKeywordUsage = Yo.validSchemaType = Yo.funcKeywordCode = Yo.macroKeywordCode = void 0;
const vr = St, Fa = ni, Dee = Et, zee = Mc;
function Bee(e, t) {
  const { gen: n, keyword: o, schema: i, parentSchema: s, it: l } = e, u = t.macro.call(l.self, i, s, l), c = e_(n, o, u);
  l.opts.validateSchema !== !1 && l.self.validateSchema(u, !0);
  const f = n.name("valid");
  e.subschema({
    schema: u,
    schemaPath: vr.nil,
    errSchemaPath: `${l.errSchemaPath}/${o}`,
    topSchemaRef: c,
    compositeRule: !0
  }, f), e.pass(f, () => e.error(!0));
}
Yo.macroKeywordCode = Bee;
function Uee(e, t) {
  var n;
  const { gen: o, keyword: i, schema: s, parentSchema: l, $data: u, it: c } = e;
  Wee(c, t);
  const f = !u && t.compile ? t.compile.call(c.self, s, l, c) : t.validate, p = e_(o, i, f), g = o.let("valid");
  e.block$data(g, h), e.ok((n = t.valid) !== null && n !== void 0 ? n : g);
  function h() {
    if (t.errors === !1)
      C(), t.modifying && Iw(e), $(() => e.error());
    else {
      const S = t.async ? y() : v();
      t.modifying && Iw(e), $(() => Vee(e, S));
    }
  }
  function y() {
    const S = o.let("ruleErrs", null);
    return o.try(() => C((0, vr._)`await `), (w) => o.assign(g, !1).if((0, vr._)`${w} instanceof ${c.ValidationError}`, () => o.assign(S, (0, vr._)`${w}.errors`), () => o.throw(w))), S;
  }
  function v() {
    const S = (0, vr._)`${p}.errors`;
    return o.assign(S, null), C(vr.nil), S;
  }
  function C(S = t.async ? (0, vr._)`await ` : vr.nil) {
    const w = c.opts.passContext ? Fa.default.this : Fa.default.self, _ = !("compile" in t && !u || t.schema === !1);
    o.assign(g, (0, vr._)`${S}${(0, Dee.callValidateCode)(e, p, w, _)}`, t.modifying);
  }
  function $(S) {
    var w;
    o.if((0, vr.not)((w = t.valid) !== null && w !== void 0 ? w : g), S);
  }
}
Yo.funcKeywordCode = Uee;
function Iw(e) {
  const { gen: t, data: n, it: o } = e;
  t.if(o.parentData, () => t.assign(n, (0, vr._)`${o.parentData}[${o.parentDataProperty}]`));
}
function Vee(e, t) {
  const { gen: n } = e;
  n.if((0, vr._)`Array.isArray(${t})`, () => {
    n.assign(Fa.default.vErrors, (0, vr._)`${Fa.default.vErrors} === null ? ${t} : ${Fa.default.vErrors}.concat(${t})`).assign(Fa.default.errors, (0, vr._)`${Fa.default.vErrors}.length`), (0, zee.extendErrors)(e);
  }, () => e.error());
}
function Wee({ schemaEnv: e }, t) {
  if (t.async && !e.$async)
    throw new Error("async keyword in sync schema");
}
function e_(e, t, n) {
  if (n === void 0)
    throw new Error(`keyword "${t}" failed to compile`);
  return e.scopeValue("keyword", typeof n == "function" ? { ref: n } : { ref: n, code: (0, vr.stringify)(n) });
}
function qee(e, t, n = !1) {
  return !t.length || t.some((o) => o === "array" ? Array.isArray(e) : o === "object" ? e && typeof e == "object" && !Array.isArray(e) : typeof e == o || n && typeof e > "u");
}
Yo.validSchemaType = qee;
function Kee({ schema: e, opts: t, self: n, errSchemaPath: o }, i, s) {
  if (Array.isArray(i.keyword) ? !i.keyword.includes(s) : i.keyword !== s)
    throw new Error("ajv implementation error");
  const l = i.dependencies;
  if (l != null && l.some((u) => !Object.prototype.hasOwnProperty.call(e, u)))
    throw new Error(`parent schema must have dependencies of ${s}: ${l.join(",")}`);
  if (i.validateSchema && !i.validateSchema(e[s])) {
    const c = `keyword "${s}" value is invalid at path "${o}": ` + n.errorsText(i.validateSchema.errors);
    if (t.validateSchema === "log")
      n.logger.error(c);
    else
      throw new Error(c);
  }
}
Yo.validateKeywordUsage = Kee;
var ta = {};
Object.defineProperty(ta, "__esModule", { value: !0 });
ta.extendSubschemaMode = ta.extendSubschemaData = ta.getSubschema = void 0;
const Ho = St, t_ = ze;
function Hee(e, { keyword: t, schemaProp: n, schema: o, schemaPath: i, errSchemaPath: s, topSchemaRef: l }) {
  if (t !== void 0 && o !== void 0)
    throw new Error('both "keyword" and "schema" passed, only one allowed');
  if (t !== void 0) {
    const u = e.schema[t];
    return n === void 0 ? {
      schema: u,
      schemaPath: (0, Ho._)`${e.schemaPath}${(0, Ho.getProperty)(t)}`,
      errSchemaPath: `${e.errSchemaPath}/${t}`
    } : {
      schema: u[n],
      schemaPath: (0, Ho._)`${e.schemaPath}${(0, Ho.getProperty)(t)}${(0, Ho.getProperty)(n)}`,
      errSchemaPath: `${e.errSchemaPath}/${t}/${(0, t_.escapeFragment)(n)}`
    };
  }
  if (o !== void 0) {
    if (i === void 0 || s === void 0 || l === void 0)
      throw new Error('"schemaPath", "errSchemaPath" and "topSchemaRef" are required with "schema"');
    return {
      schema: o,
      schemaPath: i,
      topSchemaRef: l,
      errSchemaPath: s
    };
  }
  throw new Error('either "keyword" or "schema" must be passed');
}
ta.getSubschema = Hee;
function Gee(e, t, { dataProp: n, dataPropType: o, data: i, dataTypes: s, propertyName: l }) {
  if (i !== void 0 && n !== void 0)
    throw new Error('both "data" and "dataProp" passed, only one allowed');
  const { gen: u } = t;
  if (n !== void 0) {
    const { errorPath: f, dataPathArr: p, opts: g } = t, h = u.let("data", (0, Ho._)`${t.data}${(0, Ho.getProperty)(n)}`, !0);
    c(h), e.errorPath = (0, Ho.str)`${f}${(0, t_.getErrorPath)(n, o, g.jsPropertySyntax)}`, e.parentDataProperty = (0, Ho._)`${n}`, e.dataPathArr = [...p, e.parentDataProperty];
  }
  if (i !== void 0) {
    const f = i instanceof Ho.Name ? i : u.let("data", i, !0);
    c(f), l !== void 0 && (e.propertyName = l);
  }
  s && (e.dataTypes = s);
  function c(f) {
    e.data = f, e.dataLevel = t.dataLevel + 1, e.dataTypes = [], t.definedProperties = /* @__PURE__ */ new Set(), e.parentData = t.data, e.dataNames = [...t.dataNames, f];
  }
}
ta.extendSubschemaData = Gee;
function Jee(e, { jtdDiscriminator: t, jtdMetadata: n, compositeRule: o, createErrors: i, allErrors: s }) {
  o !== void 0 && (e.compositeRule = o), i !== void 0 && (e.createErrors = i), s !== void 0 && (e.allErrors = s), e.jtdDiscriminator = t, e.jtdMetadata = n;
}
ta.extendSubschemaMode = Jee;
var Xn = {}, n_ = function e(t, n) {
  if (t === n) return !0;
  if (t && n && typeof t == "object" && typeof n == "object") {
    if (t.constructor !== n.constructor) return !1;
    var o, i, s;
    if (Array.isArray(t)) {
      if (o = t.length, o != n.length) return !1;
      for (i = o; i-- !== 0; )
        if (!e(t[i], n[i])) return !1;
      return !0;
    }
    if (t.constructor === RegExp) return t.source === n.source && t.flags === n.flags;
    if (t.valueOf !== Object.prototype.valueOf) return t.valueOf() === n.valueOf();
    if (t.toString !== Object.prototype.toString) return t.toString() === n.toString();
    if (s = Object.keys(t), o = s.length, o !== Object.keys(n).length) return !1;
    for (i = o; i-- !== 0; )
      if (!Object.prototype.hasOwnProperty.call(n, s[i])) return !1;
    for (i = o; i-- !== 0; ) {
      var l = s[i];
      if (!e(t[l], n[l])) return !1;
    }
    return !0;
  }
  return t !== t && n !== n;
}, r_ = { exports: {} }, Qi = r_.exports = function(e, t, n) {
  typeof t == "function" && (n = t, t = {}), n = t.cb || n;
  var o = typeof n == "function" ? n : n.pre || function() {
  }, i = n.post || function() {
  };
  Cf(t, o, i, e, "", e);
};
Qi.keywords = {
  additionalItems: !0,
  items: !0,
  contains: !0,
  additionalProperties: !0,
  propertyNames: !0,
  not: !0,
  if: !0,
  then: !0,
  else: !0
};
Qi.arrayKeywords = {
  items: !0,
  allOf: !0,
  anyOf: !0,
  oneOf: !0
};
Qi.propsKeywords = {
  $defs: !0,
  definitions: !0,
  properties: !0,
  patternProperties: !0,
  dependencies: !0
};
Qi.skipKeywords = {
  default: !0,
  enum: !0,
  const: !0,
  required: !0,
  maximum: !0,
  minimum: !0,
  exclusiveMaximum: !0,
  exclusiveMinimum: !0,
  multipleOf: !0,
  maxLength: !0,
  minLength: !0,
  pattern: !0,
  format: !0,
  maxItems: !0,
  minItems: !0,
  uniqueItems: !0,
  maxProperties: !0,
  minProperties: !0
};
function Cf(e, t, n, o, i, s, l, u, c, f) {
  if (o && typeof o == "object" && !Array.isArray(o)) {
    t(o, i, s, l, u, c, f);
    for (var p in o) {
      var g = o[p];
      if (Array.isArray(g)) {
        if (p in Qi.arrayKeywords)
          for (var h = 0; h < g.length; h++)
            Cf(e, t, n, g[h], i + "/" + p + "/" + h, s, i, p, o, h);
      } else if (p in Qi.propsKeywords) {
        if (g && typeof g == "object")
          for (var y in g)
            Cf(e, t, n, g[y], i + "/" + p + "/" + Yee(y), s, i, p, o, y);
      } else (p in Qi.keywords || e.allKeys && !(p in Qi.skipKeywords)) && Cf(e, t, n, g, i + "/" + p, s, i, p, o);
    }
    n(o, i, s, l, u, c, f);
  }
}
function Yee(e) {
  return e.replace(/~/g, "~0").replace(/\//g, "~1");
}
var Qee = r_.exports;
Object.defineProperty(Xn, "__esModule", { value: !0 });
Xn.getSchemaRefs = Xn.resolveUrl = Xn.normalizeId = Xn._getFullPath = Xn.getFullPath = Xn.inlineRef = void 0;
const Xee = ze, Zee = n_, ete = Qee, tte = /* @__PURE__ */ new Set([
  "type",
  "format",
  "pattern",
  "maxLength",
  "minLength",
  "maxProperties",
  "minProperties",
  "maxItems",
  "minItems",
  "maximum",
  "minimum",
  "uniqueItems",
  "multipleOf",
  "required",
  "enum",
  "const"
]);
function nte(e, t = !0) {
  return typeof e == "boolean" ? !0 : t === !0 ? !qg(e) : t ? o_(e) <= t : !1;
}
Xn.inlineRef = nte;
const rte = /* @__PURE__ */ new Set([
  "$ref",
  "$recursiveRef",
  "$recursiveAnchor",
  "$dynamicRef",
  "$dynamicAnchor"
]);
function qg(e) {
  for (const t in e) {
    if (rte.has(t))
      return !0;
    const n = e[t];
    if (Array.isArray(n) && n.some(qg) || typeof n == "object" && qg(n))
      return !0;
  }
  return !1;
}
function o_(e) {
  let t = 0;
  for (const n in e) {
    if (n === "$ref")
      return 1 / 0;
    if (t++, !tte.has(n) && (typeof e[n] == "object" && (0, Xee.eachItem)(e[n], (o) => t += o_(o)), t === 1 / 0))
      return 1 / 0;
  }
  return t;
}
function i_(e, t = "", n) {
  n !== !1 && (t = el(t));
  const o = e.parse(t);
  return a_(e, o);
}
Xn.getFullPath = i_;
function a_(e, t) {
  return e.serialize(t).split("#")[0] + "#";
}
Xn._getFullPath = a_;
const ote = /#\/?$/;
function el(e) {
  return e ? e.replace(ote, "") : "";
}
Xn.normalizeId = el;
function ite(e, t, n) {
  return n = el(n), e.resolve(t, n);
}
Xn.resolveUrl = ite;
const ate = /^[a-z_][-a-z0-9._]*$/i;
function ste(e, t) {
  if (typeof e == "boolean")
    return {};
  const { schemaId: n, uriResolver: o } = this.opts, i = el(e[n] || t), s = { "": i }, l = i_(o, i, !1), u = {}, c = /* @__PURE__ */ new Set();
  return ete(e, { allKeys: !0 }, (g, h, y, v) => {
    if (v === void 0)
      return;
    const C = l + h;
    let $ = s[v];
    typeof g[n] == "string" && ($ = S.call(this, g[n])), w.call(this, g.$anchor), w.call(this, g.$dynamicAnchor), s[h] = $;
    function S(_) {
      const b = this.opts.uriResolver.resolve;
      if (_ = el($ ? b($, _) : _), c.has(_))
        throw p(_);
      c.add(_);
      let k = this.refs[_];
      return typeof k == "string" && (k = this.refs[k]), typeof k == "object" ? f(g, k.schema, _) : _ !== el(C) && (_[0] === "#" ? (f(g, u[_], _), u[_] = g) : this.refs[_] = C), _;
    }
    function w(_) {
      if (typeof _ == "string") {
        if (!ate.test(_))
          throw new Error(`invalid anchor "${_}"`);
        S.call(this, `#${_}`);
      }
    }
  }), u;
  function f(g, h, y) {
    if (h !== void 0 && !Zee(g, h))
      throw p(y);
  }
  function p(g) {
    return new Error(`reference "${g}" resolves to more than one schema`);
  }
}
Xn.getSchemaRefs = ste;
Object.defineProperty(Ao, "__esModule", { value: !0 });
Ao.getData = Ao.KeywordCxt = Ao.validateFunctionCode = void 0;
const s_ = pl, jw = Ln, D0 = Si, Hf = Ln, lte = Kp, Gu = Yo, og = ta, Qe = St, lt = ni, ute = Xn, wi = ze, Iu = Mc;
function cte(e) {
  if (c_(e) && (d_(e), u_(e))) {
    pte(e);
    return;
  }
  l_(e, () => (0, s_.topBoolOrEmptySchema)(e));
}
Ao.validateFunctionCode = cte;
function l_({ gen: e, validateName: t, schema: n, schemaEnv: o, opts: i }, s) {
  i.code.es5 ? e.func(t, (0, Qe._)`${lt.default.data}, ${lt.default.valCxt}`, o.$async, () => {
    e.code((0, Qe._)`"use strict"; ${Rw(n, i)}`), fte(e, i), e.code(s);
  }) : e.func(t, (0, Qe._)`${lt.default.data}, ${dte(i)}`, o.$async, () => e.code(Rw(n, i)).code(s));
}
function dte(e) {
  return (0, Qe._)`{${lt.default.instancePath}="", ${lt.default.parentData}, ${lt.default.parentDataProperty}, ${lt.default.rootData}=${lt.default.data}${e.dynamicRef ? (0, Qe._)`, ${lt.default.dynamicAnchors}={}` : Qe.nil}}={}`;
}
function fte(e, t) {
  e.if(lt.default.valCxt, () => {
    e.var(lt.default.instancePath, (0, Qe._)`${lt.default.valCxt}.${lt.default.instancePath}`), e.var(lt.default.parentData, (0, Qe._)`${lt.default.valCxt}.${lt.default.parentData}`), e.var(lt.default.parentDataProperty, (0, Qe._)`${lt.default.valCxt}.${lt.default.parentDataProperty}`), e.var(lt.default.rootData, (0, Qe._)`${lt.default.valCxt}.${lt.default.rootData}`), t.dynamicRef && e.var(lt.default.dynamicAnchors, (0, Qe._)`${lt.default.valCxt}.${lt.default.dynamicAnchors}`);
  }, () => {
    e.var(lt.default.instancePath, (0, Qe._)`""`), e.var(lt.default.parentData, (0, Qe._)`undefined`), e.var(lt.default.parentDataProperty, (0, Qe._)`undefined`), e.var(lt.default.rootData, lt.default.data), t.dynamicRef && e.var(lt.default.dynamicAnchors, (0, Qe._)`{}`);
  });
}
function pte(e) {
  const { schema: t, opts: n, gen: o } = e;
  l_(e, () => {
    n.$comment && t.$comment && p_(e), vte(e), o.let(lt.default.vErrors, null), o.let(lt.default.errors, 0), n.unevaluated && hte(e), f_(e), wte(e);
  });
}
function hte(e) {
  const { gen: t, validateName: n } = e;
  e.evaluated = t.const("evaluated", (0, Qe._)`${n}.evaluated`), t.if((0, Qe._)`${e.evaluated}.dynamicProps`, () => t.assign((0, Qe._)`${e.evaluated}.props`, (0, Qe._)`undefined`)), t.if((0, Qe._)`${e.evaluated}.dynamicItems`, () => t.assign((0, Qe._)`${e.evaluated}.items`, (0, Qe._)`undefined`));
}
function Rw(e, t) {
  const n = typeof e == "object" && e[t.schemaId];
  return n && (t.code.source || t.code.process) ? (0, Qe._)`/*# sourceURL=${n} */` : Qe.nil;
}
function mte(e, t) {
  if (c_(e) && (d_(e), u_(e))) {
    gte(e, t);
    return;
  }
  (0, s_.boolOrEmptySchema)(e, t);
}
function u_({ schema: e, self: t }) {
  if (typeof e == "boolean")
    return !e;
  for (const n in e)
    if (t.RULES.all[n])
      return !0;
  return !1;
}
function c_(e) {
  return typeof e.schema != "boolean";
}
function gte(e, t) {
  const { schema: n, gen: o, opts: i } = e;
  i.$comment && n.$comment && p_(e), bte(e), Ste(e);
  const s = o.const("_errs", lt.default.errors);
  f_(e, s), o.var(t, (0, Qe._)`${s} === ${lt.default.errors}`);
}
function d_(e) {
  (0, wi.checkUnknownRules)(e), yte(e);
}
function f_(e, t) {
  if (e.opts.jtd)
    return Tw(e, [], !1, t);
  const n = (0, jw.getSchemaTypes)(e.schema), o = (0, jw.coerceAndCheckDataType)(e, n);
  Tw(e, n, !o, t);
}
function yte(e) {
  const { schema: t, errSchemaPath: n, opts: o, self: i } = e;
  t.$ref && o.ignoreKeywordsWithRef && (0, wi.schemaHasRulesButRef)(t, i.RULES) && i.logger.warn(`$ref: keywords ignored in schema at path "${n}"`);
}
function vte(e) {
  const { schema: t, opts: n } = e;
  t.default !== void 0 && n.useDefaults && n.strictSchema && (0, wi.checkStrictMode)(e, "default is ignored in the schema root");
}
function bte(e) {
  const t = e.schema[e.opts.schemaId];
  t && (e.baseId = (0, ute.resolveUrl)(e.opts.uriResolver, e.baseId, t));
}
function Ste(e) {
  if (e.schema.$async && !e.schemaEnv.$async)
    throw new Error("async schema in sync schema");
}
function p_({ gen: e, schemaEnv: t, schema: n, errSchemaPath: o, opts: i }) {
  const s = n.$comment;
  if (i.$comment === !0)
    e.code((0, Qe._)`${lt.default.self}.logger.log(${s})`);
  else if (typeof i.$comment == "function") {
    const l = (0, Qe.str)`${o}/$comment`, u = e.scopeValue("root", { ref: t.root });
    e.code((0, Qe._)`${lt.default.self}.opts.$comment(${s}, ${l}, ${u}.schema)`);
  }
}
function wte(e) {
  const { gen: t, schemaEnv: n, validateName: o, ValidationError: i, opts: s } = e;
  n.$async ? t.if((0, Qe._)`${lt.default.errors} === 0`, () => t.return(lt.default.data), () => t.throw((0, Qe._)`new ${i}(${lt.default.vErrors})`)) : (t.assign((0, Qe._)`${o}.errors`, lt.default.vErrors), s.unevaluated && $te(e), t.return((0, Qe._)`${lt.default.errors} === 0`));
}
function $te({ gen: e, evaluated: t, props: n, items: o }) {
  n instanceof Qe.Name && e.assign((0, Qe._)`${t}.props`, n), o instanceof Qe.Name && e.assign((0, Qe._)`${t}.items`, o);
}
function Tw(e, t, n, o) {
  const { gen: i, schema: s, data: l, allErrors: u, opts: c, self: f } = e, { RULES: p } = f;
  if (s.$ref && (c.ignoreKeywordsWithRef || !(0, wi.schemaHasRulesButRef)(s, p))) {
    i.block(() => g_(e, "$ref", p.all.$ref.definition));
    return;
  }
  c.jtd || Cte(e, t), i.block(() => {
    for (const h of p.rules)
      g(h);
    g(p.post);
  });
  function g(h) {
    (0, D0.shouldUseGroup)(s, h) && (h.type ? (i.if((0, Hf.checkDataType)(h.type, l, c.strictNumbers)), Mw(e, h), t.length === 1 && t[0] === h.type && n && (i.else(), (0, Hf.reportTypeError)(e)), i.endIf()) : Mw(e, h), u || i.if((0, Qe._)`${lt.default.errors} === ${o || 0}`));
  }
}
function Mw(e, t) {
  const { gen: n, schema: o, opts: { useDefaults: i } } = e;
  i && (0, lte.assignDefaults)(e, t.type), n.block(() => {
    for (const s of t.rules)
      (0, D0.shouldUseRule)(o, s) && g_(e, s.keyword, s.definition, t.type);
  });
}
function Cte(e, t) {
  e.schemaEnv.meta || !e.opts.strictTypes || (xte(e, t), e.opts.allowUnionTypes || Ete(e, t), _te(e, e.dataTypes));
}
function xte(e, t) {
  if (t.length) {
    if (!e.dataTypes.length) {
      e.dataTypes = t;
      return;
    }
    t.forEach((n) => {
      h_(e.dataTypes, n) || z0(e, `type "${n}" not allowed by context "${e.dataTypes.join(",")}"`);
    }), Pte(e, t);
  }
}
function Ete(e, t) {
  t.length > 1 && !(t.length === 2 && t.includes("null")) && z0(e, "use allowUnionTypes to allow union type keyword");
}
function _te(e, t) {
  const n = e.self.RULES.all;
  for (const o in n) {
    const i = n[o];
    if (typeof i == "object" && (0, D0.shouldUseRule)(e.schema, i)) {
      const { type: s } = i.definition;
      s.length && !s.some((l) => kte(t, l)) && z0(e, `missing type "${s.join(",")}" for keyword "${o}"`);
    }
  }
}
function kte(e, t) {
  return e.includes(t) || t === "number" && e.includes("integer");
}
function h_(e, t) {
  return e.includes(t) || t === "integer" && e.includes("number");
}
function Pte(e, t) {
  const n = [];
  for (const o of e.dataTypes)
    h_(t, o) ? n.push(o) : t.includes("integer") && o === "number" && n.push("integer");
  e.dataTypes = n;
}
function z0(e, t) {
  const n = e.schemaEnv.baseId + e.errSchemaPath;
  t += ` at "${n}" (strictTypes)`, (0, wi.checkStrictMode)(e, t, e.opts.strictTypes);
}
class m_ {
  constructor(t, n, o) {
    if ((0, Gu.validateKeywordUsage)(t, n, o), this.gen = t.gen, this.allErrors = t.allErrors, this.keyword = o, this.data = t.data, this.schema = t.schema[o], this.$data = n.$data && t.opts.$data && this.schema && this.schema.$data, this.schemaValue = (0, wi.schemaRefOrVal)(t, this.schema, o, this.$data), this.schemaType = n.schemaType, this.parentSchema = t.schema, this.params = {}, this.it = t, this.def = n, this.$data)
      this.schemaCode = t.gen.const("vSchema", y_(this.$data, t));
    else if (this.schemaCode = this.schemaValue, !(0, Gu.validSchemaType)(this.schema, n.schemaType, n.allowUndefined))
      throw new Error(`${o} value must be ${JSON.stringify(n.schemaType)}`);
    ("code" in n ? n.trackErrors : n.errors !== !1) && (this.errsCount = t.gen.const("_errs", lt.default.errors));
  }
  result(t, n, o) {
    this.failResult((0, Qe.not)(t), n, o);
  }
  failResult(t, n, o) {
    this.gen.if(t), o ? o() : this.error(), n ? (this.gen.else(), n(), this.allErrors && this.gen.endIf()) : this.allErrors ? this.gen.endIf() : this.gen.else();
  }
  pass(t, n) {
    this.failResult((0, Qe.not)(t), void 0, n);
  }
  fail(t) {
    if (t === void 0) {
      this.error(), this.allErrors || this.gen.if(!1);
      return;
    }
    this.gen.if(t), this.error(), this.allErrors ? this.gen.endIf() : this.gen.else();
  }
  fail$data(t) {
    if (!this.$data)
      return this.fail(t);
    const { schemaCode: n } = this;
    this.fail((0, Qe._)`${n} !== undefined && (${(0, Qe.or)(this.invalid$data(), t)})`);
  }
  error(t, n, o) {
    if (n) {
      this.setParams(n), this._error(t, o), this.setParams({});
      return;
    }
    this._error(t, o);
  }
  _error(t, n) {
    (t ? Iu.reportExtraError : Iu.reportError)(this, this.def.error, n);
  }
  $dataError() {
    (0, Iu.reportError)(this, this.def.$dataError || Iu.keyword$DataError);
  }
  reset() {
    if (this.errsCount === void 0)
      throw new Error('add "trackErrors" to keyword definition');
    (0, Iu.resetErrorsCount)(this.gen, this.errsCount);
  }
  ok(t) {
    this.allErrors || this.gen.if(t);
  }
  setParams(t, n) {
    n ? Object.assign(this.params, t) : this.params = t;
  }
  block$data(t, n, o = Qe.nil) {
    this.gen.block(() => {
      this.check$data(t, o), n();
    });
  }
  check$data(t = Qe.nil, n = Qe.nil) {
    if (!this.$data)
      return;
    const { gen: o, schemaCode: i, schemaType: s, def: l } = this;
    o.if((0, Qe.or)((0, Qe._)`${i} === undefined`, n)), t !== Qe.nil && o.assign(t, !0), (s.length || l.validateSchema) && (o.elseIf(this.invalid$data()), this.$dataError(), t !== Qe.nil && o.assign(t, !1)), o.else();
  }
  invalid$data() {
    const { gen: t, schemaCode: n, schemaType: o, def: i, it: s } = this;
    return (0, Qe.or)(l(), u());
    function l() {
      if (o.length) {
        if (!(n instanceof Qe.Name))
          throw new Error("ajv implementation error");
        const c = Array.isArray(o) ? o : [o];
        return (0, Qe._)`${(0, Hf.checkDataTypes)(c, n, s.opts.strictNumbers, Hf.DataType.Wrong)}`;
      }
      return Qe.nil;
    }
    function u() {
      if (i.validateSchema) {
        const c = t.scopeValue("validate$data", { ref: i.validateSchema });
        return (0, Qe._)`!${c}(${n})`;
      }
      return Qe.nil;
    }
  }
  subschema(t, n) {
    const o = (0, og.getSubschema)(this.it, t);
    (0, og.extendSubschemaData)(o, this.it, t), (0, og.extendSubschemaMode)(o, t);
    const i = { ...this.it, ...o, items: void 0, props: void 0 };
    return mte(i, n), i;
  }
  mergeEvaluated(t, n) {
    const { it: o, gen: i } = this;
    o.opts.unevaluated && (o.props !== !0 && t.props !== void 0 && (o.props = wi.mergeEvaluated.props(i, t.props, o.props, n)), o.items !== !0 && t.items !== void 0 && (o.items = wi.mergeEvaluated.items(i, t.items, o.items, n)));
  }
  mergeValidEvaluated(t, n) {
    const { it: o, gen: i } = this;
    if (o.opts.unevaluated && (o.props !== !0 || o.items !== !0))
      return i.if(n, () => this.mergeEvaluated(t, Qe.Name)), !0;
  }
}
Ao.KeywordCxt = m_;
function g_(e, t, n, o) {
  const i = new m_(e, n, t);
  "code" in n ? n.code(i, o) : i.$data && n.validate ? (0, Gu.funcKeywordCode)(i, n) : "macro" in n ? (0, Gu.macroKeywordCode)(i, n) : (n.compile || n.validate) && (0, Gu.funcKeywordCode)(i, n);
}
const Ote = /^\/(?:[^~]|~0|~1)*$/, Ate = /^([0-9]+)(#|\/(?:[^~]|~0|~1)*)?$/;
function y_(e, { dataLevel: t, dataNames: n, dataPathArr: o }) {
  let i, s;
  if (e === "")
    return lt.default.rootData;
  if (e[0] === "/") {
    if (!Ote.test(e))
      throw new Error(`Invalid JSON-pointer: ${e}`);
    i = e, s = lt.default.rootData;
  } else {
    const f = Ate.exec(e);
    if (!f)
      throw new Error(`Invalid JSON-pointer: ${e}`);
    const p = +f[1];
    if (i = f[2], i === "#") {
      if (p >= t)
        throw new Error(c("property/index", p));
      return o[t - p];
    }
    if (p > t)
      throw new Error(c("data", p));
    if (s = n[t - p], !i)
      return s;
  }
  let l = s;
  const u = i.split("/");
  for (const f of u)
    f && (s = (0, Qe._)`${s}${(0, Qe.getProperty)((0, wi.unescapeJsonPointer)(f))}`, l = (0, Qe._)`${l} && ${s}`);
  return l;
  function c(f, p) {
    return `Cannot access ${f} ${p} levels up, current level is ${t}`;
  }
}
Ao.getData = y_;
var Nc = {};
Object.defineProperty(Nc, "__esModule", { value: !0 });
class Ite extends Error {
  constructor(t) {
    super("validation failed"), this.errors = t, this.ajv = this.validation = !0;
  }
}
Nc.default = Ite;
var Nl = {};
Object.defineProperty(Nl, "__esModule", { value: !0 });
const ig = Xn;
class jte extends Error {
  constructor(t, n, o, i) {
    super(i || `can't resolve reference ${o} from id ${n}`), this.missingRef = (0, ig.resolveUrl)(t, n, o), this.missingSchema = (0, ig.normalizeId)((0, ig.getFullPath)(t, this.missingRef));
  }
}
Nl.default = jte;
var Mr = {};
Object.defineProperty(Mr, "__esModule", { value: !0 });
Mr.resolveSchema = Mr.getCompilingSchema = Mr.resolveRef = Mr.compileSchema = Mr.SchemaEnv = void 0;
const ko = St, Rte = Nc, Ia = ni, Oo = Xn, Nw = ze, Tte = Ao;
class Hp {
  constructor(t) {
    var n;
    this.refs = {}, this.dynamicAnchors = {};
    let o;
    typeof t.schema == "object" && (o = t.schema), this.schema = t.schema, this.schemaId = t.schemaId, this.root = t.root || this, this.baseId = (n = t.baseId) !== null && n !== void 0 ? n : (0, Oo.normalizeId)(o == null ? void 0 : o[t.schemaId || "$id"]), this.schemaPath = t.schemaPath, this.localRefs = t.localRefs, this.meta = t.meta, this.$async = o == null ? void 0 : o.$async, this.refs = {};
  }
}
Mr.SchemaEnv = Hp;
function B0(e) {
  const t = v_.call(this, e);
  if (t)
    return t;
  const n = (0, Oo.getFullPath)(this.opts.uriResolver, e.root.baseId), { es5: o, lines: i } = this.opts.code, { ownProperties: s } = this.opts, l = new ko.CodeGen(this.scope, { es5: o, lines: i, ownProperties: s });
  let u;
  e.$async && (u = l.scopeValue("Error", {
    ref: Rte.default,
    code: (0, ko._)`require("ajv/dist/runtime/validation_error").default`
  }));
  const c = l.scopeName("validate");
  e.validateName = c;
  const f = {
    gen: l,
    allErrors: this.opts.allErrors,
    data: Ia.default.data,
    parentData: Ia.default.parentData,
    parentDataProperty: Ia.default.parentDataProperty,
    dataNames: [Ia.default.data],
    dataPathArr: [ko.nil],
    // TODO can its length be used as dataLevel if nil is removed?
    dataLevel: 0,
    dataTypes: [],
    definedProperties: /* @__PURE__ */ new Set(),
    topSchemaRef: l.scopeValue("schema", this.opts.code.source === !0 ? { ref: e.schema, code: (0, ko.stringify)(e.schema) } : { ref: e.schema }),
    validateName: c,
    ValidationError: u,
    schema: e.schema,
    schemaEnv: e,
    rootId: n,
    baseId: e.baseId || n,
    schemaPath: ko.nil,
    errSchemaPath: e.schemaPath || (this.opts.jtd ? "" : "#"),
    errorPath: (0, ko._)`""`,
    opts: this.opts,
    self: this
  };
  let p;
  try {
    this._compilations.add(e), (0, Tte.validateFunctionCode)(f), l.optimize(this.opts.code.optimize);
    const g = l.toString();
    p = `${l.scopeRefs(Ia.default.scope)}return ${g}`, this.opts.code.process && (p = this.opts.code.process(p, e));
    const y = new Function(`${Ia.default.self}`, `${Ia.default.scope}`, p)(this, this.scope.get());
    if (this.scope.value(c, { ref: y }), y.errors = null, y.schema = e.schema, y.schemaEnv = e, e.$async && (y.$async = !0), this.opts.code.source === !0 && (y.source = { validateName: c, validateCode: g, scopeValues: l._values }), this.opts.unevaluated) {
      const { props: v, items: C } = f;
      y.evaluated = {
        props: v instanceof ko.Name ? void 0 : v,
        items: C instanceof ko.Name ? void 0 : C,
        dynamicProps: v instanceof ko.Name,
        dynamicItems: C instanceof ko.Name
      }, y.source && (y.source.evaluated = (0, ko.stringify)(y.evaluated));
    }
    return e.validate = y, e;
  } catch (g) {
    throw delete e.validate, delete e.validateName, p && this.logger.error("Error compiling schema, function code:", p), g;
  } finally {
    this._compilations.delete(e);
  }
}
Mr.compileSchema = B0;
function Mte(e, t, n) {
  var o;
  n = (0, Oo.resolveUrl)(this.opts.uriResolver, t, n);
  const i = e.refs[n];
  if (i)
    return i;
  let s = Lte.call(this, e, n);
  if (s === void 0) {
    const l = (o = e.localRefs) === null || o === void 0 ? void 0 : o[n], { schemaId: u } = this.opts;
    l && (s = new Hp({ schema: l, schemaId: u, root: e, baseId: t }));
  }
  if (s !== void 0)
    return e.refs[n] = Nte.call(this, s);
}
Mr.resolveRef = Mte;
function Nte(e) {
  return (0, Oo.inlineRef)(e.schema, this.opts.inlineRefs) ? e.schema : e.validate ? e : B0.call(this, e);
}
function v_(e) {
  for (const t of this._compilations)
    if (Fte(t, e))
      return t;
}
Mr.getCompilingSchema = v_;
function Fte(e, t) {
  return e.schema === t.schema && e.root === t.root && e.baseId === t.baseId;
}
function Lte(e, t) {
  let n;
  for (; typeof (n = this.refs[t]) == "string"; )
    t = n;
  return n || this.schemas[t] || Gp.call(this, e, t);
}
function Gp(e, t) {
  const n = this.opts.uriResolver.parse(t), o = (0, Oo._getFullPath)(this.opts.uriResolver, n);
  let i = (0, Oo.getFullPath)(this.opts.uriResolver, e.baseId, void 0);
  if (Object.keys(e.schema).length > 0 && o === i)
    return ag.call(this, n, e);
  const s = (0, Oo.normalizeId)(o), l = this.refs[s] || this.schemas[s];
  if (typeof l == "string") {
    const u = Gp.call(this, e, l);
    return typeof (u == null ? void 0 : u.schema) != "object" ? void 0 : ag.call(this, n, u);
  }
  if (typeof (l == null ? void 0 : l.schema) == "object") {
    if (l.validate || B0.call(this, l), s === (0, Oo.normalizeId)(t)) {
      const { schema: u } = l, { schemaId: c } = this.opts, f = u[c];
      return f && (i = (0, Oo.resolveUrl)(this.opts.uriResolver, i, f)), new Hp({ schema: u, schemaId: c, root: e, baseId: i });
    }
    return ag.call(this, n, l);
  }
}
Mr.resolveSchema = Gp;
const Dte = /* @__PURE__ */ new Set([
  "properties",
  "patternProperties",
  "enum",
  "dependencies",
  "definitions"
]);
function ag(e, { baseId: t, schema: n, root: o }) {
  var i;
  if (((i = e.fragment) === null || i === void 0 ? void 0 : i[0]) !== "/")
    return;
  for (const u of e.fragment.slice(1).split("/")) {
    if (typeof n == "boolean")
      return;
    const c = n[(0, Nw.unescapeFragment)(u)];
    if (c === void 0)
      return;
    n = c;
    const f = typeof n == "object" && n[this.opts.schemaId];
    !Dte.has(u) && f && (t = (0, Oo.resolveUrl)(this.opts.uriResolver, t, f));
  }
  let s;
  if (typeof n != "boolean" && n.$ref && !(0, Nw.schemaHasRulesButRef)(n, this.RULES)) {
    const u = (0, Oo.resolveUrl)(this.opts.uriResolver, t, n.$ref);
    s = Gp.call(this, o, u);
  }
  const { schemaId: l } = this.opts;
  if (s = s || new Hp({ schema: n, schemaId: l, root: o, baseId: t }), s.schema !== s.root.schema)
    return s;
}
const zte = "https://raw.githubusercontent.com/ajv-validator/ajv/master/lib/refs/data.json#", Bte = "Meta-schema for $data reference (JSON AnySchema extension proposal)", Ute = "object", Vte = [
  "$data"
], Wte = {
  $data: {
    type: "string",
    anyOf: [
      {
        format: "relative-json-pointer"
      },
      {
        format: "json-pointer"
      }
    ]
  }
}, qte = !1, Kte = {
  $id: zte,
  description: Bte,
  type: Ute,
  required: Vte,
  properties: Wte,
  additionalProperties: qte
};
var U0 = {}, Jp = { exports: {} };
const Hte = {
  0: 0,
  1: 1,
  2: 2,
  3: 3,
  4: 4,
  5: 5,
  6: 6,
  7: 7,
  8: 8,
  9: 9,
  a: 10,
  A: 10,
  b: 11,
  B: 11,
  c: 12,
  C: 12,
  d: 13,
  D: 13,
  e: 14,
  E: 14,
  f: 15,
  F: 15
};
var Gte = {
  HEX: Hte
};
const { HEX: Jte } = Gte, Yte = /^(?:(?:25[0-5]|2[0-4]\d|1\d{2}|[1-9]\d|\d)\.){3}(?:25[0-5]|2[0-4]\d|1\d{2}|[1-9]\d|\d)$/u;
function b_(e) {
  if (w_(e, ".") < 3)
    return { host: e, isIPV4: !1 };
  const t = e.match(Yte) || [], [n] = t;
  return n ? { host: Xte(n, "."), isIPV4: !0 } : { host: e, isIPV4: !1 };
}
function Kg(e, t = !1) {
  let n = "", o = !0;
  for (const i of e) {
    if (Jte[i] === void 0) return;
    i !== "0" && o === !0 && (o = !1), o || (n += i);
  }
  return t && n.length === 0 && (n = "0"), n;
}
function Qte(e) {
  let t = 0;
  const n = { error: !1, address: "", zone: "" }, o = [], i = [];
  let s = !1, l = !1, u = !1;
  function c() {
    if (i.length) {
      if (s === !1) {
        const f = Kg(i);
        if (f !== void 0)
          o.push(f);
        else
          return n.error = !0, !1;
      }
      i.length = 0;
    }
    return !0;
  }
  for (let f = 0; f < e.length; f++) {
    const p = e[f];
    if (!(p === "[" || p === "]"))
      if (p === ":") {
        if (l === !0 && (u = !0), !c())
          break;
        if (t++, o.push(":"), t > 7) {
          n.error = !0;
          break;
        }
        f - 1 >= 0 && e[f - 1] === ":" && (l = !0);
        continue;
      } else if (p === "%") {
        if (!c())
          break;
        s = !0;
      } else {
        i.push(p);
        continue;
      }
  }
  return i.length && (s ? n.zone = i.join("") : u ? o.push(i.join("")) : o.push(Kg(i))), n.address = o.join(""), n;
}
function S_(e) {
  if (w_(e, ":") < 2)
    return { host: e, isIPV6: !1 };
  const t = Qte(e);
  if (t.error)
    return { host: e, isIPV6: !1 };
  {
    let n = t.address, o = t.address;
    return t.zone && (n += "%" + t.zone, o += "%25" + t.zone), { host: n, escapedHost: o, isIPV6: !0 };
  }
}
function Xte(e, t) {
  let n = "", o = !0;
  const i = e.length;
  for (let s = 0; s < i; s++) {
    const l = e[s];
    l === "0" && o ? (s + 1 <= i && e[s + 1] === t || s + 1 === i) && (n += l, o = !1) : (l === t ? o = !0 : o = !1, n += l);
  }
  return n;
}
function w_(e, t) {
  let n = 0;
  for (let o = 0; o < e.length; o++)
    e[o] === t && n++;
  return n;
}
const Fw = /^\.\.?\//u, Lw = /^\/\.(?:\/|$)/u, Dw = /^\/\.\.(?:\/|$)/u, Zte = /^\/?(?:.|\n)*?(?=\/|$)/u;
function ene(e) {
  const t = [];
  for (; e.length; )
    if (e.match(Fw))
      e = e.replace(Fw, "");
    else if (e.match(Lw))
      e = e.replace(Lw, "/");
    else if (e.match(Dw))
      e = e.replace(Dw, "/"), t.pop();
    else if (e === "." || e === "..")
      e = "";
    else {
      const n = e.match(Zte);
      if (n) {
        const o = n[0];
        e = e.slice(o.length), t.push(o);
      } else
        throw new Error("Unexpected dot segment condition");
    }
  return t.join("");
}
function tne(e, t) {
  const n = t !== !0 ? escape : unescape;
  return e.scheme !== void 0 && (e.scheme = n(e.scheme)), e.userinfo !== void 0 && (e.userinfo = n(e.userinfo)), e.host !== void 0 && (e.host = n(e.host)), e.path !== void 0 && (e.path = n(e.path)), e.query !== void 0 && (e.query = n(e.query)), e.fragment !== void 0 && (e.fragment = n(e.fragment)), e;
}
function nne(e) {
  const t = [];
  if (e.userinfo !== void 0 && (t.push(e.userinfo), t.push("@")), e.host !== void 0) {
    let n = unescape(e.host);
    const o = b_(n);
    if (o.isIPV4)
      n = o.host;
    else {
      const i = S_(o.host);
      i.isIPV6 === !0 ? n = `[${i.escapedHost}]` : n = e.host;
    }
    t.push(n);
  }
  return (typeof e.port == "number" || typeof e.port == "string") && (t.push(":"), t.push(String(e.port))), t.length ? t.join("") : void 0;
}
var rne = {
  recomposeAuthority: nne,
  normalizeComponentEncoding: tne,
  removeDotSegments: ene,
  normalizeIPv4: b_,
  normalizeIPv6: S_,
  stringArrayToHexStripped: Kg
};
const one = /^[\da-f]{8}\b-[\da-f]{4}\b-[\da-f]{4}\b-[\da-f]{4}\b-[\da-f]{12}$/iu, ine = /([\da-z][\d\-a-z]{0,31}):((?:[\w!$'()*+,\-.:;=@]|%[\da-f]{2})+)/iu;
function $_(e) {
  return typeof e.secure == "boolean" ? e.secure : String(e.scheme).toLowerCase() === "wss";
}
function C_(e) {
  return e.host || (e.error = e.error || "HTTP URIs must have a host."), e;
}
function x_(e) {
  const t = String(e.scheme).toLowerCase() === "https";
  return (e.port === (t ? 443 : 80) || e.port === "") && (e.port = void 0), e.path || (e.path = "/"), e;
}
function ane(e) {
  return e.secure = $_(e), e.resourceName = (e.path || "/") + (e.query ? "?" + e.query : ""), e.path = void 0, e.query = void 0, e;
}
function sne(e) {
  if ((e.port === ($_(e) ? 443 : 80) || e.port === "") && (e.port = void 0), typeof e.secure == "boolean" && (e.scheme = e.secure ? "wss" : "ws", e.secure = void 0), e.resourceName) {
    const [t, n] = e.resourceName.split("?");
    e.path = t && t !== "/" ? t : void 0, e.query = n, e.resourceName = void 0;
  }
  return e.fragment = void 0, e;
}
function lne(e, t) {
  if (!e.path)
    return e.error = "URN can not be parsed", e;
  const n = e.path.match(ine);
  if (n) {
    const o = t.scheme || e.scheme || "urn";
    e.nid = n[1].toLowerCase(), e.nss = n[2];
    const i = `${o}:${t.nid || e.nid}`, s = V0[i];
    e.path = void 0, s && (e = s.parse(e, t));
  } else
    e.error = e.error || "URN can not be parsed.";
  return e;
}
function une(e, t) {
  const n = t.scheme || e.scheme || "urn", o = e.nid.toLowerCase(), i = `${n}:${t.nid || o}`, s = V0[i];
  s && (e = s.serialize(e, t));
  const l = e, u = e.nss;
  return l.path = `${o || t.nid}:${u}`, t.skipEscape = !0, l;
}
function cne(e, t) {
  const n = e;
  return n.uuid = n.nss, n.nss = void 0, !t.tolerant && (!n.uuid || !one.test(n.uuid)) && (n.error = n.error || "UUID is not valid."), n;
}
function dne(e) {
  const t = e;
  return t.nss = (e.uuid || "").toLowerCase(), t;
}
const E_ = {
  scheme: "http",
  domainHost: !0,
  parse: C_,
  serialize: x_
}, fne = {
  scheme: "https",
  domainHost: E_.domainHost,
  parse: C_,
  serialize: x_
}, xf = {
  scheme: "ws",
  domainHost: !0,
  parse: ane,
  serialize: sne
}, pne = {
  scheme: "wss",
  domainHost: xf.domainHost,
  parse: xf.parse,
  serialize: xf.serialize
}, hne = {
  scheme: "urn",
  parse: lne,
  serialize: une,
  skipNormalize: !0
}, mne = {
  scheme: "urn:uuid",
  parse: cne,
  serialize: dne,
  skipNormalize: !0
}, V0 = {
  http: E_,
  https: fne,
  ws: xf,
  wss: pne,
  urn: hne,
  "urn:uuid": mne
};
var gne = V0;
const { normalizeIPv6: yne, normalizeIPv4: vne, removeDotSegments: Du, recomposeAuthority: bne, normalizeComponentEncoding: of } = rne, W0 = gne;
function Sne(e, t) {
  return typeof e == "string" ? e = Qo(_i(e, t), t) : typeof e == "object" && (e = _i(Qo(e, t), t)), e;
}
function wne(e, t, n) {
  const o = Object.assign({ scheme: "null" }, n), i = __(_i(e, o), _i(t, o), o, !0);
  return Qo(i, { ...o, skipEscape: !0 });
}
function __(e, t, n, o) {
  const i = {};
  return o || (e = _i(Qo(e, n), n), t = _i(Qo(t, n), n)), n = n || {}, !n.tolerant && t.scheme ? (i.scheme = t.scheme, i.userinfo = t.userinfo, i.host = t.host, i.port = t.port, i.path = Du(t.path || ""), i.query = t.query) : (t.userinfo !== void 0 || t.host !== void 0 || t.port !== void 0 ? (i.userinfo = t.userinfo, i.host = t.host, i.port = t.port, i.path = Du(t.path || ""), i.query = t.query) : (t.path ? (t.path.charAt(0) === "/" ? i.path = Du(t.path) : ((e.userinfo !== void 0 || e.host !== void 0 || e.port !== void 0) && !e.path ? i.path = "/" + t.path : e.path ? i.path = e.path.slice(0, e.path.lastIndexOf("/") + 1) + t.path : i.path = t.path, i.path = Du(i.path)), i.query = t.query) : (i.path = e.path, t.query !== void 0 ? i.query = t.query : i.query = e.query), i.userinfo = e.userinfo, i.host = e.host, i.port = e.port), i.scheme = e.scheme), i.fragment = t.fragment, i;
}
function $ne(e, t, n) {
  return typeof e == "string" ? (e = unescape(e), e = Qo(of(_i(e, n), !0), { ...n, skipEscape: !0 })) : typeof e == "object" && (e = Qo(of(e, !0), { ...n, skipEscape: !0 })), typeof t == "string" ? (t = unescape(t), t = Qo(of(_i(t, n), !0), { ...n, skipEscape: !0 })) : typeof t == "object" && (t = Qo(of(t, !0), { ...n, skipEscape: !0 })), e.toLowerCase() === t.toLowerCase();
}
function Qo(e, t) {
  const n = {
    host: e.host,
    scheme: e.scheme,
    userinfo: e.userinfo,
    port: e.port,
    path: e.path,
    query: e.query,
    nid: e.nid,
    nss: e.nss,
    uuid: e.uuid,
    fragment: e.fragment,
    reference: e.reference,
    resourceName: e.resourceName,
    secure: e.secure,
    error: ""
  }, o = Object.assign({}, t), i = [], s = W0[(o.scheme || n.scheme || "").toLowerCase()];
  s && s.serialize && s.serialize(n, o), n.path !== void 0 && (o.skipEscape ? n.path = unescape(n.path) : (n.path = escape(n.path), n.scheme !== void 0 && (n.path = n.path.split("%3A").join(":")))), o.reference !== "suffix" && n.scheme && i.push(n.scheme, ":");
  const l = bne(n);
  if (l !== void 0 && (o.reference !== "suffix" && i.push("//"), i.push(l), n.path && n.path.charAt(0) !== "/" && i.push("/")), n.path !== void 0) {
    let u = n.path;
    !o.absolutePath && (!s || !s.absolutePath) && (u = Du(u)), l === void 0 && (u = u.replace(/^\/\//u, "/%2F")), i.push(u);
  }
  return n.query !== void 0 && i.push("?", n.query), n.fragment !== void 0 && i.push("#", n.fragment), i.join("");
}
const Cne = Array.from({ length: 127 }, (e, t) => /[^!"$&'()*+,\-.;=_`a-z{}~]/u.test(String.fromCharCode(t)));
function xne(e) {
  let t = 0;
  for (let n = 0, o = e.length; n < o; ++n)
    if (t = e.charCodeAt(n), t > 126 || Cne[t])
      return !0;
  return !1;
}
const Ene = /^(?:([^#/:?]+):)?(?:\/\/((?:([^#/?@]*)@)?(\[[^#/?\]]+\]|[^#/:?]*)(?::(\d*))?))?([^#?]*)(?:\?([^#]*))?(?:#((?:.|[\n\r])*))?/u;
function _i(e, t) {
  const n = Object.assign({}, t), o = {
    scheme: void 0,
    userinfo: void 0,
    host: "",
    port: void 0,
    path: "",
    query: void 0,
    fragment: void 0
  }, i = e.indexOf("%") !== -1;
  let s = !1;
  n.reference === "suffix" && (e = (n.scheme ? n.scheme + ":" : "") + "//" + e);
  const l = e.match(Ene);
  if (l) {
    if (o.scheme = l[1], o.userinfo = l[3], o.host = l[4], o.port = parseInt(l[5], 10), o.path = l[6] || "", o.query = l[7], o.fragment = l[8], isNaN(o.port) && (o.port = l[5]), o.host) {
      const c = vne(o.host);
      if (c.isIPV4 === !1) {
        const f = yne(c.host);
        o.host = f.host.toLowerCase(), s = f.isIPV6;
      } else
        o.host = c.host, s = !0;
    }
    o.scheme === void 0 && o.userinfo === void 0 && o.host === void 0 && o.port === void 0 && !o.path && o.query === void 0 ? o.reference = "same-document" : o.scheme === void 0 ? o.reference = "relative" : o.fragment === void 0 ? o.reference = "absolute" : o.reference = "uri", n.reference && n.reference !== "suffix" && n.reference !== o.reference && (o.error = o.error || "URI is not a " + n.reference + " reference.");
    const u = W0[(n.scheme || o.scheme || "").toLowerCase()];
    if (!n.unicodeSupport && (!u || !u.unicodeSupport) && o.host && (n.domainHost || u && u.domainHost) && s === !1 && xne(o.host))
      try {
        o.host = URL.domainToASCII(o.host.toLowerCase());
      } catch (c) {
        o.error = o.error || "Host's domain name can not be converted to ASCII: " + c;
      }
    (!u || u && !u.skipNormalize) && (i && o.scheme !== void 0 && (o.scheme = unescape(o.scheme)), i && o.host !== void 0 && (o.host = unescape(o.host)), o.path && o.path.length && (o.path = escape(unescape(o.path))), o.fragment && o.fragment.length && (o.fragment = encodeURI(decodeURIComponent(o.fragment)))), u && u.parse && u.parse(o, n);
  } else
    o.error = o.error || "URI can not be parsed.";
  return o;
}
const q0 = {
  SCHEMES: W0,
  normalize: Sne,
  resolve: wne,
  resolveComponents: __,
  equal: $ne,
  serialize: Qo,
  parse: _i
};
Jp.exports = q0;
Jp.exports.default = q0;
Jp.exports.fastUri = q0;
var _ne = Jp.exports;
Object.defineProperty(U0, "__esModule", { value: !0 });
const k_ = _ne;
k_.code = 'require("ajv/dist/runtime/uri").default';
U0.default = k_;
(function(e) {
  Object.defineProperty(e, "__esModule", { value: !0 }), e.CodeGen = e.Name = e.nil = e.stringify = e.str = e._ = e.KeywordCxt = void 0;
  var t = Ao;
  Object.defineProperty(e, "KeywordCxt", { enumerable: !0, get: function() {
    return t.KeywordCxt;
  } });
  var n = St;
  Object.defineProperty(e, "_", { enumerable: !0, get: function() {
    return n._;
  } }), Object.defineProperty(e, "str", { enumerable: !0, get: function() {
    return n.str;
  } }), Object.defineProperty(e, "stringify", { enumerable: !0, get: function() {
    return n.stringify;
  } }), Object.defineProperty(e, "nil", { enumerable: !0, get: function() {
    return n.nil;
  } }), Object.defineProperty(e, "Name", { enumerable: !0, get: function() {
    return n.Name;
  } }), Object.defineProperty(e, "CodeGen", { enumerable: !0, get: function() {
    return n.CodeGen;
  } });
  const o = Nc, i = Nl, s = Wa, l = Mr, u = St, c = Xn, f = Ln, p = ze, g = Kte, h = U0, y = (Y, T) => new RegExp(Y, T);
  y.code = "new RegExp";
  const v = ["removeAdditional", "useDefaults", "coerceTypes"], C = /* @__PURE__ */ new Set([
    "validate",
    "serialize",
    "parse",
    "wrapper",
    "root",
    "schema",
    "keyword",
    "pattern",
    "formats",
    "validate$data",
    "func",
    "obj",
    "Error"
  ]), $ = {
    errorDataPath: "",
    format: "`validateFormats: false` can be used instead.",
    nullable: '"nullable" keyword is supported by default.',
    jsonPointers: "Deprecated jsPropertySyntax can be used instead.",
    extendRefs: "Deprecated ignoreKeywordsWithRef can be used instead.",
    missingRefs: "Pass empty schema with $id that should be ignored to ajv.addSchema.",
    processCode: "Use option `code: {process: (code, schemaEnv: object) => string}`",
    sourceCode: "Use option `code: {source: true}`",
    strictDefaults: "It is default now, see option `strict`.",
    strictKeywords: "It is default now, see option `strict`.",
    uniqueItems: '"uniqueItems" keyword is always validated.',
    unknownFormats: "Disable strict mode or pass `true` to `ajv.addFormat` (or `formats` option).",
    cache: "Map is used as cache, schema object as key.",
    serialize: "Map is used as cache, schema object as key.",
    ajvErrors: "It is default now."
  }, S = {
    ignoreKeywordsWithRef: "",
    jsPropertySyntax: "",
    unicode: '"minLength"/"maxLength" account for unicode characters by default.'
  }, w = 200;
  function _(Y) {
    var T, H, G, N, U, Z, me, ue, Ce, $e, Te, We, et, Xe, ft, Lt, st, pt, Se, gt, yt, Le, cn, ye, Pe;
    const Me = Y.strict, Ye = (T = Y.code) === null || T === void 0 ? void 0 : T.optimize, ot = Ye === !0 || Ye === void 0 ? 1 : Ye || 0, se = (G = (H = Y.code) === null || H === void 0 ? void 0 : H.regExp) !== null && G !== void 0 ? G : y, ae = (N = Y.uriResolver) !== null && N !== void 0 ? N : h.default;
    return {
      strictSchema: (Z = (U = Y.strictSchema) !== null && U !== void 0 ? U : Me) !== null && Z !== void 0 ? Z : !0,
      strictNumbers: (ue = (me = Y.strictNumbers) !== null && me !== void 0 ? me : Me) !== null && ue !== void 0 ? ue : !0,
      strictTypes: ($e = (Ce = Y.strictTypes) !== null && Ce !== void 0 ? Ce : Me) !== null && $e !== void 0 ? $e : "log",
      strictTuples: (We = (Te = Y.strictTuples) !== null && Te !== void 0 ? Te : Me) !== null && We !== void 0 ? We : "log",
      strictRequired: (Xe = (et = Y.strictRequired) !== null && et !== void 0 ? et : Me) !== null && Xe !== void 0 ? Xe : !1,
      code: Y.code ? { ...Y.code, optimize: ot, regExp: se } : { optimize: ot, regExp: se },
      loopRequired: (ft = Y.loopRequired) !== null && ft !== void 0 ? ft : w,
      loopEnum: (Lt = Y.loopEnum) !== null && Lt !== void 0 ? Lt : w,
      meta: (st = Y.meta) !== null && st !== void 0 ? st : !0,
      messages: (pt = Y.messages) !== null && pt !== void 0 ? pt : !0,
      inlineRefs: (Se = Y.inlineRefs) !== null && Se !== void 0 ? Se : !0,
      schemaId: (gt = Y.schemaId) !== null && gt !== void 0 ? gt : "$id",
      addUsedSchema: (yt = Y.addUsedSchema) !== null && yt !== void 0 ? yt : !0,
      validateSchema: (Le = Y.validateSchema) !== null && Le !== void 0 ? Le : !0,
      validateFormats: (cn = Y.validateFormats) !== null && cn !== void 0 ? cn : !0,
      unicodeRegExp: (ye = Y.unicodeRegExp) !== null && ye !== void 0 ? ye : !0,
      int32range: (Pe = Y.int32range) !== null && Pe !== void 0 ? Pe : !0,
      uriResolver: ae
    };
  }
  class b {
    constructor(T = {}) {
      this.schemas = {}, this.refs = {}, this.formats = {}, this._compilations = /* @__PURE__ */ new Set(), this._loading = {}, this._cache = /* @__PURE__ */ new Map(), T = this.opts = { ...T, ..._(T) };
      const { es5: H, lines: G } = this.opts.code;
      this.scope = new u.ValueScope({ scope: {}, prefixes: C, es5: H, lines: G }), this.logger = W(T.logger);
      const N = T.validateFormats;
      T.validateFormats = !1, this.RULES = (0, s.getRules)(), k.call(this, $, T, "NOT SUPPORTED"), k.call(this, S, T, "DEPRECATED", "warn"), this._metaOpts = O.call(this), T.formats && L.call(this), this._addVocabularies(), this._addDefaultMetaSchema(), T.keywords && B.call(this, T.keywords), typeof T.meta == "object" && this.addMetaSchema(T.meta), j.call(this), T.validateFormats = N;
    }
    _addVocabularies() {
      this.addKeyword("$async");
    }
    _addDefaultMetaSchema() {
      const { $data: T, meta: H, schemaId: G } = this.opts;
      let N = g;
      G === "id" && (N = { ...g }, N.id = N.$id, delete N.$id), H && T && this.addMetaSchema(N, N[G], !1);
    }
    defaultMeta() {
      const { meta: T, schemaId: H } = this.opts;
      return this.opts.defaultMeta = typeof T == "object" ? T[H] || T : void 0;
    }
    validate(T, H) {
      let G;
      if (typeof T == "string") {
        if (G = this.getSchema(T), !G)
          throw new Error(`no schema with key or ref "${T}"`);
      } else
        G = this.compile(T);
      const N = G(H);
      return "$async" in G || (this.errors = G.errors), N;
    }
    compile(T, H) {
      const G = this._addSchema(T, H);
      return G.validate || this._compileSchemaEnv(G);
    }
    compileAsync(T, H) {
      if (typeof this.opts.loadSchema != "function")
        throw new Error("options.loadSchema should be a function");
      const { loadSchema: G } = this.opts;
      return N.call(this, T, H);
      async function N($e, Te) {
        await U.call(this, $e.$schema);
        const We = this._addSchema($e, Te);
        return We.validate || Z.call(this, We);
      }
      async function U($e) {
        $e && !this.getSchema($e) && await N.call(this, { $ref: $e }, !0);
      }
      async function Z($e) {
        try {
          return this._compileSchemaEnv($e);
        } catch (Te) {
          if (!(Te instanceof i.default))
            throw Te;
          return me.call(this, Te), await ue.call(this, Te.missingSchema), Z.call(this, $e);
        }
      }
      function me({ missingSchema: $e, missingRef: Te }) {
        if (this.refs[$e])
          throw new Error(`AnySchema ${$e} is loaded but ${Te} cannot be resolved`);
      }
      async function ue($e) {
        const Te = await Ce.call(this, $e);
        this.refs[$e] || await U.call(this, Te.$schema), this.refs[$e] || this.addSchema(Te, $e, H);
      }
      async function Ce($e) {
        const Te = this._loading[$e];
        if (Te)
          return Te;
        try {
          return await (this._loading[$e] = G($e));
        } finally {
          delete this._loading[$e];
        }
      }
    }
    // Adds schema to the instance
    addSchema(T, H, G, N = this.opts.validateSchema) {
      if (Array.isArray(T)) {
        for (const Z of T)
          this.addSchema(Z, void 0, G, N);
        return this;
      }
      let U;
      if (typeof T == "object") {
        const { schemaId: Z } = this.opts;
        if (U = T[Z], U !== void 0 && typeof U != "string")
          throw new Error(`schema ${Z} must be string`);
      }
      return H = (0, c.normalizeId)(H || U), this._checkUnique(H), this.schemas[H] = this._addSchema(T, G, H, N, !0), this;
    }
    // Add schema that will be used to validate other schemas
    // options in META_IGNORE_OPTIONS are alway set to false
    addMetaSchema(T, H, G = this.opts.validateSchema) {
      return this.addSchema(T, H, !0, G), this;
    }
    //  Validate schema against its meta-schema
    validateSchema(T, H) {
      if (typeof T == "boolean")
        return !0;
      let G;
      if (G = T.$schema, G !== void 0 && typeof G != "string")
        throw new Error("$schema must be a string");
      if (G = G || this.opts.defaultMeta || this.defaultMeta(), !G)
        return this.logger.warn("meta-schema not available"), this.errors = null, !0;
      const N = this.validate(G, T);
      if (!N && H) {
        const U = "schema is invalid: " + this.errorsText();
        if (this.opts.validateSchema === "log")
          this.logger.error(U);
        else
          throw new Error(U);
      }
      return N;
    }
    // Get compiled schema by `key` or `ref`.
    // (`key` that was passed to `addSchema` or full schema reference - `schema.$id` or resolved id)
    getSchema(T) {
      let H;
      for (; typeof (H = E.call(this, T)) == "string"; )
        T = H;
      if (H === void 0) {
        const { schemaId: G } = this.opts, N = new l.SchemaEnv({ schema: {}, schemaId: G });
        if (H = l.resolveSchema.call(this, N, T), !H)
          return;
        this.refs[T] = H;
      }
      return H.validate || this._compileSchemaEnv(H);
    }
    // Remove cached schema(s).
    // If no parameter is passed all schemas but meta-schemas are removed.
    // If RegExp is passed all schemas with key/id matching pattern but meta-schemas are removed.
    // Even if schema is referenced by other schemas it still can be removed as other schemas have local references.
    removeSchema(T) {
      if (T instanceof RegExp)
        return this._removeAllSchemas(this.schemas, T), this._removeAllSchemas(this.refs, T), this;
      switch (typeof T) {
        case "undefined":
          return this._removeAllSchemas(this.schemas), this._removeAllSchemas(this.refs), this._cache.clear(), this;
        case "string": {
          const H = E.call(this, T);
          return typeof H == "object" && this._cache.delete(H.schema), delete this.schemas[T], delete this.refs[T], this;
        }
        case "object": {
          const H = T;
          this._cache.delete(H);
          let G = T[this.opts.schemaId];
          return G && (G = (0, c.normalizeId)(G), delete this.schemas[G], delete this.refs[G]), this;
        }
        default:
          throw new Error("ajv.removeSchema: invalid parameter");
      }
    }
    // add "vocabulary" - a collection of keywords
    addVocabulary(T) {
      for (const H of T)
        this.addKeyword(H);
      return this;
    }
    addKeyword(T, H) {
      let G;
      if (typeof T == "string")
        G = T, typeof H == "object" && (this.logger.warn("these parameters are deprecated, see docs for addKeyword"), H.keyword = G);
      else if (typeof T == "object" && H === void 0) {
        if (H = T, G = H.keyword, Array.isArray(G) && !G.length)
          throw new Error("addKeywords: keyword must be string or non-empty array");
      } else
        throw new Error("invalid addKeywords parameters");
      if (Q.call(this, G, H), !H)
        return (0, p.eachItem)(G, (U) => X.call(this, U)), this;
      J.call(this, H);
      const N = {
        ...H,
        type: (0, f.getJSONTypes)(H.type),
        schemaType: (0, f.getJSONTypes)(H.schemaType)
      };
      return (0, p.eachItem)(G, N.type.length === 0 ? (U) => X.call(this, U, N) : (U) => N.type.forEach((Z) => X.call(this, U, N, Z))), this;
    }
    getKeyword(T) {
      const H = this.RULES.all[T];
      return typeof H == "object" ? H.definition : !!H;
    }
    // Remove keyword
    removeKeyword(T) {
      const { RULES: H } = this;
      delete H.keywords[T], delete H.all[T];
      for (const G of H.rules) {
        const N = G.rules.findIndex((U) => U.keyword === T);
        N >= 0 && G.rules.splice(N, 1);
      }
      return this;
    }
    // Add format
    addFormat(T, H) {
      return typeof H == "string" && (H = new RegExp(H)), this.formats[T] = H, this;
    }
    errorsText(T = this.errors, { separator: H = ", ", dataVar: G = "data" } = {}) {
      return !T || T.length === 0 ? "No errors" : T.map((N) => `${G}${N.instancePath} ${N.message}`).reduce((N, U) => N + H + U);
    }
    $dataMetaSchema(T, H) {
      const G = this.RULES.all;
      T = JSON.parse(JSON.stringify(T));
      for (const N of H) {
        const U = N.split("/").slice(1);
        let Z = T;
        for (const me of U)
          Z = Z[me];
        for (const me in G) {
          const ue = G[me];
          if (typeof ue != "object")
            continue;
          const { $data: Ce } = ue.definition, $e = Z[me];
          Ce && $e && (Z[me] = re($e));
        }
      }
      return T;
    }
    _removeAllSchemas(T, H) {
      for (const G in T) {
        const N = T[G];
        (!H || H.test(G)) && (typeof N == "string" ? delete T[G] : N && !N.meta && (this._cache.delete(N.schema), delete T[G]));
      }
    }
    _addSchema(T, H, G, N = this.opts.validateSchema, U = this.opts.addUsedSchema) {
      let Z;
      const { schemaId: me } = this.opts;
      if (typeof T == "object")
        Z = T[me];
      else {
        if (this.opts.jtd)
          throw new Error("schema must be object");
        if (typeof T != "boolean")
          throw new Error("schema must be object or boolean");
      }
      let ue = this._cache.get(T);
      if (ue !== void 0)
        return ue;
      G = (0, c.normalizeId)(Z || G);
      const Ce = c.getSchemaRefs.call(this, T, G);
      return ue = new l.SchemaEnv({ schema: T, schemaId: me, meta: H, baseId: G, localRefs: Ce }), this._cache.set(ue.schema, ue), U && !G.startsWith("#") && (G && this._checkUnique(G), this.refs[G] = ue), N && this.validateSchema(T, !0), ue;
    }
    _checkUnique(T) {
      if (this.schemas[T] || this.refs[T])
        throw new Error(`schema with key or id "${T}" already exists`);
    }
    _compileSchemaEnv(T) {
      if (T.meta ? this._compileMetaSchema(T) : l.compileSchema.call(this, T), !T.validate)
        throw new Error("ajv implementation error");
      return T.validate;
    }
    _compileMetaSchema(T) {
      const H = this.opts;
      this.opts = this._metaOpts;
      try {
        l.compileSchema.call(this, T);
      } finally {
        this.opts = H;
      }
    }
  }
  b.ValidationError = o.default, b.MissingRefError = i.default, e.default = b;
  function k(Y, T, H, G = "error") {
    for (const N in Y) {
      const U = N;
      U in T && this.logger[G](`${H}: option ${N}. ${Y[U]}`);
    }
  }
  function E(Y) {
    return Y = (0, c.normalizeId)(Y), this.schemas[Y] || this.refs[Y];
  }
  function j() {
    const Y = this.opts.schemas;
    if (Y)
      if (Array.isArray(Y))
        this.addSchema(Y);
      else
        for (const T in Y)
          this.addSchema(Y[T], T);
  }
  function L() {
    for (const Y in this.opts.formats) {
      const T = this.opts.formats[Y];
      T && this.addFormat(Y, T);
    }
  }
  function B(Y) {
    if (Array.isArray(Y)) {
      this.addVocabulary(Y);
      return;
    }
    this.logger.warn("keywords option as map is deprecated, pass array");
    for (const T in Y) {
      const H = Y[T];
      H.keyword || (H.keyword = T), this.addKeyword(H);
    }
  }
  function O() {
    const Y = { ...this.opts };
    for (const T of v)
      delete Y[T];
    return Y;
  }
  const V = { log() {
  }, warn() {
  }, error() {
  } };
  function W(Y) {
    if (Y === !1)
      return V;
    if (Y === void 0)
      return console;
    if (Y.log && Y.warn && Y.error)
      return Y;
    throw new Error("logger must implement log, warn and error methods");
  }
  const q = /^[a-z_$][a-z0-9_$:-]*$/i;
  function Q(Y, T) {
    const { RULES: H } = this;
    if ((0, p.eachItem)(Y, (G) => {
      if (H.keywords[G])
        throw new Error(`Keyword ${G} is already defined`);
      if (!q.test(G))
        throw new Error(`Keyword ${G} has invalid name`);
    }), !!T && T.$data && !("code" in T || "validate" in T))
      throw new Error('$data keyword must have "code" or "validate" function');
  }
  function X(Y, T, H) {
    var G;
    const N = T == null ? void 0 : T.post;
    if (H && N)
      throw new Error('keyword with "post" flag cannot have "type"');
    const { RULES: U } = this;
    let Z = N ? U.post : U.rules.find(({ type: ue }) => ue === H);
    if (Z || (Z = { type: H, rules: [] }, U.rules.push(Z)), U.keywords[Y] = !0, !T)
      return;
    const me = {
      keyword: Y,
      definition: {
        ...T,
        type: (0, f.getJSONTypes)(T.type),
        schemaType: (0, f.getJSONTypes)(T.schemaType)
      }
    };
    T.before ? ce.call(this, Z, me, T.before) : Z.rules.push(me), U.all[Y] = me, (G = T.implements) === null || G === void 0 || G.forEach((ue) => this.addKeyword(ue));
  }
  function ce(Y, T, H) {
    const G = Y.rules.findIndex((N) => N.keyword === H);
    G >= 0 ? Y.rules.splice(G, 0, T) : (Y.rules.push(T), this.logger.warn(`rule ${H} is not defined`));
  }
  function J(Y) {
    let { metaSchema: T } = Y;
    T !== void 0 && (Y.$data && this.opts.$data && (T = re(T)), Y.validateSchema = this.compile(T, !0));
  }
  const ie = {
    $ref: "https://raw.githubusercontent.com/ajv-validator/ajv/master/lib/refs/data.json#"
  };
  function re(Y) {
    return { anyOf: [Y, ie] };
  }
})(zE);
var K0 = {}, H0 = {}, G0 = {};
Object.defineProperty(G0, "__esModule", { value: !0 });
const kne = {
  keyword: "id",
  code() {
    throw new Error('NOT SUPPORTED: keyword "id", use "$id" for schema ID');
  }
};
G0.default = kne;
var qa = {};
Object.defineProperty(qa, "__esModule", { value: !0 });
qa.callRef = qa.getValidate = void 0;
const Pne = Nl, zw = Et, jr = St, Bs = ni, Bw = Mr, af = ze, One = {
  keyword: "$ref",
  schemaType: "string",
  code(e) {
    const { gen: t, schema: n, it: o } = e, { baseId: i, schemaEnv: s, validateName: l, opts: u, self: c } = o, { root: f } = s;
    if ((n === "#" || n === "#/") && i === f.baseId)
      return g();
    const p = Bw.resolveRef.call(c, f, i, n);
    if (p === void 0)
      throw new Pne.default(o.opts.uriResolver, i, n);
    if (p instanceof Bw.SchemaEnv)
      return h(p);
    return y(p);
    function g() {
      if (s === f)
        return Ef(e, l, s, s.$async);
      const v = t.scopeValue("root", { ref: f });
      return Ef(e, (0, jr._)`${v}.validate`, f, f.$async);
    }
    function h(v) {
      const C = P_(e, v);
      Ef(e, C, v, v.$async);
    }
    function y(v) {
      const C = t.scopeValue("schema", u.code.source === !0 ? { ref: v, code: (0, jr.stringify)(v) } : { ref: v }), $ = t.name("valid"), S = e.subschema({
        schema: v,
        dataTypes: [],
        schemaPath: jr.nil,
        topSchemaRef: C,
        errSchemaPath: n
      }, $);
      e.mergeEvaluated(S), e.ok($);
    }
  }
};
function P_(e, t) {
  const { gen: n } = e;
  return t.validate ? n.scopeValue("validate", { ref: t.validate }) : (0, jr._)`${n.scopeValue("wrapper", { ref: t })}.validate`;
}
qa.getValidate = P_;
function Ef(e, t, n, o) {
  const { gen: i, it: s } = e, { allErrors: l, schemaEnv: u, opts: c } = s, f = c.passContext ? Bs.default.this : jr.nil;
  o ? p() : g();
  function p() {
    if (!u.$async)
      throw new Error("async schema referenced by sync schema");
    const v = i.let("valid");
    i.try(() => {
      i.code((0, jr._)`await ${(0, zw.callValidateCode)(e, t, f)}`), y(t), l || i.assign(v, !0);
    }, (C) => {
      i.if((0, jr._)`!(${C} instanceof ${s.ValidationError})`, () => i.throw(C)), h(C), l || i.assign(v, !1);
    }), e.ok(v);
  }
  function g() {
    e.result((0, zw.callValidateCode)(e, t, f), () => y(t), () => h(t));
  }
  function h(v) {
    const C = (0, jr._)`${v}.errors`;
    i.assign(Bs.default.vErrors, (0, jr._)`${Bs.default.vErrors} === null ? ${C} : ${Bs.default.vErrors}.concat(${C})`), i.assign(Bs.default.errors, (0, jr._)`${Bs.default.vErrors}.length`);
  }
  function y(v) {
    var C;
    if (!s.opts.unevaluated)
      return;
    const $ = (C = n == null ? void 0 : n.validate) === null || C === void 0 ? void 0 : C.evaluated;
    if (s.props !== !0)
      if ($ && !$.dynamicProps)
        $.props !== void 0 && (s.props = af.mergeEvaluated.props(i, $.props, s.props));
      else {
        const S = i.var("props", (0, jr._)`${v}.evaluated.props`);
        s.props = af.mergeEvaluated.props(i, S, s.props, jr.Name);
      }
    if (s.items !== !0)
      if ($ && !$.dynamicItems)
        $.items !== void 0 && (s.items = af.mergeEvaluated.items(i, $.items, s.items));
      else {
        const S = i.var("items", (0, jr._)`${v}.evaluated.items`);
        s.items = af.mergeEvaluated.items(i, S, s.items, jr.Name);
      }
  }
}
qa.callRef = Ef;
qa.default = One;
Object.defineProperty(H0, "__esModule", { value: !0 });
const Ane = G0, Ine = qa, jne = [
  "$schema",
  "$id",
  "$defs",
  "$vocabulary",
  { keyword: "$comment" },
  "definitions",
  Ane.default,
  Ine.default
];
H0.default = jne;
var J0 = {}, Y0 = {};
Object.defineProperty(Y0, "__esModule", { value: !0 });
const Gf = St, Hi = Gf.operators, Jf = {
  maximum: { okStr: "<=", ok: Hi.LTE, fail: Hi.GT },
  minimum: { okStr: ">=", ok: Hi.GTE, fail: Hi.LT },
  exclusiveMaximum: { okStr: "<", ok: Hi.LT, fail: Hi.GTE },
  exclusiveMinimum: { okStr: ">", ok: Hi.GT, fail: Hi.LTE }
}, Rne = {
  message: ({ keyword: e, schemaCode: t }) => (0, Gf.str)`must be ${Jf[e].okStr} ${t}`,
  params: ({ keyword: e, schemaCode: t }) => (0, Gf._)`{comparison: ${Jf[e].okStr}, limit: ${t}}`
}, Tne = {
  keyword: Object.keys(Jf),
  type: "number",
  schemaType: "number",
  $data: !0,
  error: Rne,
  code(e) {
    const { keyword: t, data: n, schemaCode: o } = e;
    e.fail$data((0, Gf._)`${n} ${Jf[t].fail} ${o} || isNaN(${n})`);
  }
};
Y0.default = Tne;
var Q0 = {};
Object.defineProperty(Q0, "__esModule", { value: !0 });
const Ju = St, Mne = {
  message: ({ schemaCode: e }) => (0, Ju.str)`must be multiple of ${e}`,
  params: ({ schemaCode: e }) => (0, Ju._)`{multipleOf: ${e}}`
}, Nne = {
  keyword: "multipleOf",
  type: "number",
  schemaType: "number",
  $data: !0,
  error: Mne,
  code(e) {
    const { gen: t, data: n, schemaCode: o, it: i } = e, s = i.opts.multipleOfPrecision, l = t.let("res"), u = s ? (0, Ju._)`Math.abs(Math.round(${l}) - ${l}) > 1e-${s}` : (0, Ju._)`${l} !== parseInt(${l})`;
    e.fail$data((0, Ju._)`(${o} === 0 || (${l} = ${n}/${o}, ${u}))`);
  }
};
Q0.default = Nne;
var X0 = {}, Z0 = {};
Object.defineProperty(Z0, "__esModule", { value: !0 });
function O_(e) {
  const t = e.length;
  let n = 0, o = 0, i;
  for (; o < t; )
    n++, i = e.charCodeAt(o++), i >= 55296 && i <= 56319 && o < t && (i = e.charCodeAt(o), (i & 64512) === 56320 && o++);
  return n;
}
Z0.default = O_;
O_.code = 'require("ajv/dist/runtime/ucs2length").default';
Object.defineProperty(X0, "__esModule", { value: !0 });
const La = St, Fne = ze, Lne = Z0, Dne = {
  message({ keyword: e, schemaCode: t }) {
    const n = e === "maxLength" ? "more" : "fewer";
    return (0, La.str)`must NOT have ${n} than ${t} characters`;
  },
  params: ({ schemaCode: e }) => (0, La._)`{limit: ${e}}`
}, zne = {
  keyword: ["maxLength", "minLength"],
  type: "string",
  schemaType: "number",
  $data: !0,
  error: Dne,
  code(e) {
    const { keyword: t, data: n, schemaCode: o, it: i } = e, s = t === "maxLength" ? La.operators.GT : La.operators.LT, l = i.opts.unicode === !1 ? (0, La._)`${n}.length` : (0, La._)`${(0, Fne.useFunc)(e.gen, Lne.default)}(${n})`;
    e.fail$data((0, La._)`${l} ${s} ${o}`);
  }
};
X0.default = zne;
var ev = {};
Object.defineProperty(ev, "__esModule", { value: !0 });
const Bne = Et, Yf = St, Une = {
  message: ({ schemaCode: e }) => (0, Yf.str)`must match pattern "${e}"`,
  params: ({ schemaCode: e }) => (0, Yf._)`{pattern: ${e}}`
}, Vne = {
  keyword: "pattern",
  type: "string",
  schemaType: "string",
  $data: !0,
  error: Une,
  code(e) {
    const { data: t, $data: n, schema: o, schemaCode: i, it: s } = e, l = s.opts.unicodeRegExp ? "u" : "", u = n ? (0, Yf._)`(new RegExp(${i}, ${l}))` : (0, Bne.usePattern)(e, o);
    e.fail$data((0, Yf._)`!${u}.test(${t})`);
  }
};
ev.default = Vne;
var tv = {};
Object.defineProperty(tv, "__esModule", { value: !0 });
const Yu = St, Wne = {
  message({ keyword: e, schemaCode: t }) {
    const n = e === "maxProperties" ? "more" : "fewer";
    return (0, Yu.str)`must NOT have ${n} than ${t} properties`;
  },
  params: ({ schemaCode: e }) => (0, Yu._)`{limit: ${e}}`
}, qne = {
  keyword: ["maxProperties", "minProperties"],
  type: "object",
  schemaType: "number",
  $data: !0,
  error: Wne,
  code(e) {
    const { keyword: t, data: n, schemaCode: o } = e, i = t === "maxProperties" ? Yu.operators.GT : Yu.operators.LT;
    e.fail$data((0, Yu._)`Object.keys(${n}).length ${i} ${o}`);
  }
};
tv.default = qne;
var nv = {};
Object.defineProperty(nv, "__esModule", { value: !0 });
const ju = Et, Qu = St, Kne = ze, Hne = {
  message: ({ params: { missingProperty: e } }) => (0, Qu.str)`must have required property '${e}'`,
  params: ({ params: { missingProperty: e } }) => (0, Qu._)`{missingProperty: ${e}}`
}, Gne = {
  keyword: "required",
  type: "object",
  schemaType: "array",
  $data: !0,
  error: Hne,
  code(e) {
    const { gen: t, schema: n, schemaCode: o, data: i, $data: s, it: l } = e, { opts: u } = l;
    if (!s && n.length === 0)
      return;
    const c = n.length >= u.loopRequired;
    if (l.allErrors ? f() : p(), u.strictRequired) {
      const y = e.parentSchema.properties, { definedProperties: v } = e.it;
      for (const C of n)
        if ((y == null ? void 0 : y[C]) === void 0 && !v.has(C)) {
          const $ = l.schemaEnv.baseId + l.errSchemaPath, S = `required property "${C}" is not defined at "${$}" (strictRequired)`;
          (0, Kne.checkStrictMode)(l, S, l.opts.strictRequired);
        }
    }
    function f() {
      if (c || s)
        e.block$data(Qu.nil, g);
      else
        for (const y of n)
          (0, ju.checkReportMissingProp)(e, y);
    }
    function p() {
      const y = t.let("missing");
      if (c || s) {
        const v = t.let("valid", !0);
        e.block$data(v, () => h(y, v)), e.ok(v);
      } else
        t.if((0, ju.checkMissingProp)(e, n, y)), (0, ju.reportMissingProp)(e, y), t.else();
    }
    function g() {
      t.forOf("prop", o, (y) => {
        e.setParams({ missingProperty: y }), t.if((0, ju.noPropertyInData)(t, i, y, u.ownProperties), () => e.error());
      });
    }
    function h(y, v) {
      e.setParams({ missingProperty: y }), t.forOf(y, o, () => {
        t.assign(v, (0, ju.propertyInData)(t, i, y, u.ownProperties)), t.if((0, Qu.not)(v), () => {
          e.error(), t.break();
        });
      }, Qu.nil);
    }
  }
};
nv.default = Gne;
var rv = {};
Object.defineProperty(rv, "__esModule", { value: !0 });
const Xu = St, Jne = {
  message({ keyword: e, schemaCode: t }) {
    const n = e === "maxItems" ? "more" : "fewer";
    return (0, Xu.str)`must NOT have ${n} than ${t} items`;
  },
  params: ({ schemaCode: e }) => (0, Xu._)`{limit: ${e}}`
}, Yne = {
  keyword: ["maxItems", "minItems"],
  type: "array",
  schemaType: "number",
  $data: !0,
  error: Jne,
  code(e) {
    const { keyword: t, data: n, schemaCode: o } = e, i = t === "maxItems" ? Xu.operators.GT : Xu.operators.LT;
    e.fail$data((0, Xu._)`${n}.length ${i} ${o}`);
  }
};
rv.default = Yne;
var ov = {}, Fc = {};
Object.defineProperty(Fc, "__esModule", { value: !0 });
const A_ = n_;
A_.code = 'require("ajv/dist/runtime/equal").default';
Fc.default = A_;
Object.defineProperty(ov, "__esModule", { value: !0 });
const sg = Ln, Jn = St, Qne = ze, Xne = Fc, Zne = {
  message: ({ params: { i: e, j: t } }) => (0, Jn.str)`must NOT have duplicate items (items ## ${t} and ${e} are identical)`,
  params: ({ params: { i: e, j: t } }) => (0, Jn._)`{i: ${e}, j: ${t}}`
}, ere = {
  keyword: "uniqueItems",
  type: "array",
  schemaType: "boolean",
  $data: !0,
  error: Zne,
  code(e) {
    const { gen: t, data: n, $data: o, schema: i, parentSchema: s, schemaCode: l, it: u } = e;
    if (!o && !i)
      return;
    const c = t.let("valid"), f = s.items ? (0, sg.getSchemaTypes)(s.items) : [];
    e.block$data(c, p, (0, Jn._)`${l} === false`), e.ok(c);
    function p() {
      const v = t.let("i", (0, Jn._)`${n}.length`), C = t.let("j");
      e.setParams({ i: v, j: C }), t.assign(c, !0), t.if((0, Jn._)`${v} > 1`, () => (g() ? h : y)(v, C));
    }
    function g() {
      return f.length > 0 && !f.some((v) => v === "object" || v === "array");
    }
    function h(v, C) {
      const $ = t.name("item"), S = (0, sg.checkDataTypes)(f, $, u.opts.strictNumbers, sg.DataType.Wrong), w = t.const("indices", (0, Jn._)`{}`);
      t.for((0, Jn._)`;${v}--;`, () => {
        t.let($, (0, Jn._)`${n}[${v}]`), t.if(S, (0, Jn._)`continue`), f.length > 1 && t.if((0, Jn._)`typeof ${$} == "string"`, (0, Jn._)`${$} += "_"`), t.if((0, Jn._)`typeof ${w}[${$}] == "number"`, () => {
          t.assign(C, (0, Jn._)`${w}[${$}]`), e.error(), t.assign(c, !1).break();
        }).code((0, Jn._)`${w}[${$}] = ${v}`);
      });
    }
    function y(v, C) {
      const $ = (0, Qne.useFunc)(t, Xne.default), S = t.name("outer");
      t.label(S).for((0, Jn._)`;${v}--;`, () => t.for((0, Jn._)`${C} = ${v}; ${C}--;`, () => t.if((0, Jn._)`${$}(${n}[${v}], ${n}[${C}])`, () => {
        e.error(), t.assign(c, !1).break(S);
      })));
    }
  }
};
ov.default = ere;
var iv = {};
Object.defineProperty(iv, "__esModule", { value: !0 });
const Hg = St, tre = ze, nre = Fc, rre = {
  message: "must be equal to constant",
  params: ({ schemaCode: e }) => (0, Hg._)`{allowedValue: ${e}}`
}, ore = {
  keyword: "const",
  $data: !0,
  error: rre,
  code(e) {
    const { gen: t, data: n, $data: o, schemaCode: i, schema: s } = e;
    o || s && typeof s == "object" ? e.fail$data((0, Hg._)`!${(0, tre.useFunc)(t, nre.default)}(${n}, ${i})`) : e.fail((0, Hg._)`${s} !== ${n}`);
  }
};
iv.default = ore;
var av = {};
Object.defineProperty(av, "__esModule", { value: !0 });
const zu = St, ire = ze, are = Fc, sre = {
  message: "must be equal to one of the allowed values",
  params: ({ schemaCode: e }) => (0, zu._)`{allowedValues: ${e}}`
}, lre = {
  keyword: "enum",
  schemaType: "array",
  $data: !0,
  error: sre,
  code(e) {
    const { gen: t, data: n, $data: o, schema: i, schemaCode: s, it: l } = e;
    if (!o && i.length === 0)
      throw new Error("enum must have non-empty array");
    const u = i.length >= l.opts.loopEnum;
    let c;
    const f = () => c ?? (c = (0, ire.useFunc)(t, are.default));
    let p;
    if (u || o)
      p = t.let("valid"), e.block$data(p, g);
    else {
      if (!Array.isArray(i))
        throw new Error("ajv implementation error");
      const y = t.const("vSchema", s);
      p = (0, zu.or)(...i.map((v, C) => h(y, C)));
    }
    e.pass(p);
    function g() {
      t.assign(p, !1), t.forOf("v", s, (y) => t.if((0, zu._)`${f()}(${n}, ${y})`, () => t.assign(p, !0).break()));
    }
    function h(y, v) {
      const C = i[v];
      return typeof C == "object" && C !== null ? (0, zu._)`${f()}(${n}, ${y}[${v}])` : (0, zu._)`${n} === ${C}`;
    }
  }
};
av.default = lre;
Object.defineProperty(J0, "__esModule", { value: !0 });
const ure = Y0, cre = Q0, dre = X0, fre = ev, pre = tv, hre = nv, mre = rv, gre = ov, yre = iv, vre = av, bre = [
  // number
  ure.default,
  cre.default,
  // string
  dre.default,
  fre.default,
  // object
  pre.default,
  hre.default,
  // array
  mre.default,
  gre.default,
  // any
  { keyword: "type", schemaType: ["string", "array"] },
  { keyword: "nullable", schemaType: "boolean" },
  yre.default,
  vre.default
];
J0.default = bre;
var sv = {}, Fl = {};
Object.defineProperty(Fl, "__esModule", { value: !0 });
Fl.validateAdditionalItems = void 0;
const Da = St, Gg = ze, Sre = {
  message: ({ params: { len: e } }) => (0, Da.str)`must NOT have more than ${e} items`,
  params: ({ params: { len: e } }) => (0, Da._)`{limit: ${e}}`
}, wre = {
  keyword: "additionalItems",
  type: "array",
  schemaType: ["boolean", "object"],
  before: "uniqueItems",
  error: Sre,
  code(e) {
    const { parentSchema: t, it: n } = e, { items: o } = t;
    if (!Array.isArray(o)) {
      (0, Gg.checkStrictMode)(n, '"additionalItems" is ignored when "items" is not an array of schemas');
      return;
    }
    I_(e, o);
  }
};
function I_(e, t) {
  const { gen: n, schema: o, data: i, keyword: s, it: l } = e;
  l.items = !0;
  const u = n.const("len", (0, Da._)`${i}.length`);
  if (o === !1)
    e.setParams({ len: t.length }), e.pass((0, Da._)`${u} <= ${t.length}`);
  else if (typeof o == "object" && !(0, Gg.alwaysValidSchema)(l, o)) {
    const f = n.var("valid", (0, Da._)`${u} <= ${t.length}`);
    n.if((0, Da.not)(f), () => c(f)), e.ok(f);
  }
  function c(f) {
    n.forRange("i", t.length, u, (p) => {
      e.subschema({ keyword: s, dataProp: p, dataPropType: Gg.Type.Num }, f), l.allErrors || n.if((0, Da.not)(f), () => n.break());
    });
  }
}
Fl.validateAdditionalItems = I_;
Fl.default = wre;
var lv = {}, Ll = {};
Object.defineProperty(Ll, "__esModule", { value: !0 });
Ll.validateTuple = void 0;
const Uw = St, _f = ze, $re = Et, Cre = {
  keyword: "items",
  type: "array",
  schemaType: ["object", "array", "boolean"],
  before: "uniqueItems",
  code(e) {
    const { schema: t, it: n } = e;
    if (Array.isArray(t))
      return j_(e, "additionalItems", t);
    n.items = !0, !(0, _f.alwaysValidSchema)(n, t) && e.ok((0, $re.validateArray)(e));
  }
};
function j_(e, t, n = e.schema) {
  const { gen: o, parentSchema: i, data: s, keyword: l, it: u } = e;
  p(i), u.opts.unevaluated && n.length && u.items !== !0 && (u.items = _f.mergeEvaluated.items(o, n.length, u.items));
  const c = o.name("valid"), f = o.const("len", (0, Uw._)`${s}.length`);
  n.forEach((g, h) => {
    (0, _f.alwaysValidSchema)(u, g) || (o.if((0, Uw._)`${f} > ${h}`, () => e.subschema({
      keyword: l,
      schemaProp: h,
      dataProp: h
    }, c)), e.ok(c));
  });
  function p(g) {
    const { opts: h, errSchemaPath: y } = u, v = n.length, C = v === g.minItems && (v === g.maxItems || g[t] === !1);
    if (h.strictTuples && !C) {
      const $ = `"${l}" is ${v}-tuple, but minItems or maxItems/${t} are not specified or different at path "${y}"`;
      (0, _f.checkStrictMode)(u, $, h.strictTuples);
    }
  }
}
Ll.validateTuple = j_;
Ll.default = Cre;
Object.defineProperty(lv, "__esModule", { value: !0 });
const xre = Ll, Ere = {
  keyword: "prefixItems",
  type: "array",
  schemaType: ["array"],
  before: "uniqueItems",
  code: (e) => (0, xre.validateTuple)(e, "items")
};
lv.default = Ere;
var uv = {};
Object.defineProperty(uv, "__esModule", { value: !0 });
const Vw = St, _re = ze, kre = Et, Pre = Fl, Ore = {
  message: ({ params: { len: e } }) => (0, Vw.str)`must NOT have more than ${e} items`,
  params: ({ params: { len: e } }) => (0, Vw._)`{limit: ${e}}`
}, Are = {
  keyword: "items",
  type: "array",
  schemaType: ["object", "boolean"],
  before: "uniqueItems",
  error: Ore,
  code(e) {
    const { schema: t, parentSchema: n, it: o } = e, { prefixItems: i } = n;
    o.items = !0, !(0, _re.alwaysValidSchema)(o, t) && (i ? (0, Pre.validateAdditionalItems)(e, i) : e.ok((0, kre.validateArray)(e)));
  }
};
uv.default = Are;
var cv = {};
Object.defineProperty(cv, "__esModule", { value: !0 });
const io = St, sf = ze, Ire = {
  message: ({ params: { min: e, max: t } }) => t === void 0 ? (0, io.str)`must contain at least ${e} valid item(s)` : (0, io.str)`must contain at least ${e} and no more than ${t} valid item(s)`,
  params: ({ params: { min: e, max: t } }) => t === void 0 ? (0, io._)`{minContains: ${e}}` : (0, io._)`{minContains: ${e}, maxContains: ${t}}`
}, jre = {
  keyword: "contains",
  type: "array",
  schemaType: ["object", "boolean"],
  before: "uniqueItems",
  trackErrors: !0,
  error: Ire,
  code(e) {
    const { gen: t, schema: n, parentSchema: o, data: i, it: s } = e;
    let l, u;
    const { minContains: c, maxContains: f } = o;
    s.opts.next ? (l = c === void 0 ? 1 : c, u = f) : l = 1;
    const p = t.const("len", (0, io._)`${i}.length`);
    if (e.setParams({ min: l, max: u }), u === void 0 && l === 0) {
      (0, sf.checkStrictMode)(s, '"minContains" == 0 without "maxContains": "contains" keyword ignored');
      return;
    }
    if (u !== void 0 && l > u) {
      (0, sf.checkStrictMode)(s, '"minContains" > "maxContains" is always invalid'), e.fail();
      return;
    }
    if ((0, sf.alwaysValidSchema)(s, n)) {
      let C = (0, io._)`${p} >= ${l}`;
      u !== void 0 && (C = (0, io._)`${C} && ${p} <= ${u}`), e.pass(C);
      return;
    }
    s.items = !0;
    const g = t.name("valid");
    u === void 0 && l === 1 ? y(g, () => t.if(g, () => t.break())) : l === 0 ? (t.let(g, !0), u !== void 0 && t.if((0, io._)`${i}.length > 0`, h)) : (t.let(g, !1), h()), e.result(g, () => e.reset());
    function h() {
      const C = t.name("_valid"), $ = t.let("count", 0);
      y(C, () => t.if(C, () => v($)));
    }
    function y(C, $) {
      t.forRange("i", 0, p, (S) => {
        e.subschema({
          keyword: "contains",
          dataProp: S,
          dataPropType: sf.Type.Num,
          compositeRule: !0
        }, C), $();
      });
    }
    function v(C) {
      t.code((0, io._)`${C}++`), u === void 0 ? t.if((0, io._)`${C} >= ${l}`, () => t.assign(g, !0).break()) : (t.if((0, io._)`${C} > ${u}`, () => t.assign(g, !1).break()), l === 1 ? t.assign(g, !0) : t.if((0, io._)`${C} >= ${l}`, () => t.assign(g, !0)));
    }
  }
};
cv.default = jre;
var R_ = {};
(function(e) {
  Object.defineProperty(e, "__esModule", { value: !0 }), e.validateSchemaDeps = e.validatePropertyDeps = e.error = void 0;
  const t = St, n = ze, o = Et;
  e.error = {
    message: ({ params: { property: c, depsCount: f, deps: p } }) => {
      const g = f === 1 ? "property" : "properties";
      return (0, t.str)`must have ${g} ${p} when property ${c} is present`;
    },
    params: ({ params: { property: c, depsCount: f, deps: p, missingProperty: g } }) => (0, t._)`{property: ${c},
    missingProperty: ${g},
    depsCount: ${f},
    deps: ${p}}`
    // TODO change to reference
  };
  const i = {
    keyword: "dependencies",
    type: "object",
    schemaType: "object",
    error: e.error,
    code(c) {
      const [f, p] = s(c);
      l(c, f), u(c, p);
    }
  };
  function s({ schema: c }) {
    const f = {}, p = {};
    for (const g in c) {
      if (g === "__proto__")
        continue;
      const h = Array.isArray(c[g]) ? f : p;
      h[g] = c[g];
    }
    return [f, p];
  }
  function l(c, f = c.schema) {
    const { gen: p, data: g, it: h } = c;
    if (Object.keys(f).length === 0)
      return;
    const y = p.let("missing");
    for (const v in f) {
      const C = f[v];
      if (C.length === 0)
        continue;
      const $ = (0, o.propertyInData)(p, g, v, h.opts.ownProperties);
      c.setParams({
        property: v,
        depsCount: C.length,
        deps: C.join(", ")
      }), h.allErrors ? p.if($, () => {
        for (const S of C)
          (0, o.checkReportMissingProp)(c, S);
      }) : (p.if((0, t._)`${$} && (${(0, o.checkMissingProp)(c, C, y)})`), (0, o.reportMissingProp)(c, y), p.else());
    }
  }
  e.validatePropertyDeps = l;
  function u(c, f = c.schema) {
    const { gen: p, data: g, keyword: h, it: y } = c, v = p.name("valid");
    for (const C in f)
      (0, n.alwaysValidSchema)(y, f[C]) || (p.if(
        (0, o.propertyInData)(p, g, C, y.opts.ownProperties),
        () => {
          const $ = c.subschema({ keyword: h, schemaProp: C }, v);
          c.mergeValidEvaluated($, v);
        },
        () => p.var(v, !0)
        // TODO var
      ), c.ok(v));
  }
  e.validateSchemaDeps = u, e.default = i;
})(R_);
var dv = {};
Object.defineProperty(dv, "__esModule", { value: !0 });
const T_ = St, Rre = ze, Tre = {
  message: "property name must be valid",
  params: ({ params: e }) => (0, T_._)`{propertyName: ${e.propertyName}}`
}, Mre = {
  keyword: "propertyNames",
  type: "object",
  schemaType: ["object", "boolean"],
  error: Tre,
  code(e) {
    const { gen: t, schema: n, data: o, it: i } = e;
    if ((0, Rre.alwaysValidSchema)(i, n))
      return;
    const s = t.name("valid");
    t.forIn("key", o, (l) => {
      e.setParams({ propertyName: l }), e.subschema({
        keyword: "propertyNames",
        data: l,
        dataTypes: ["string"],
        propertyName: l,
        compositeRule: !0
      }, s), t.if((0, T_.not)(s), () => {
        e.error(!0), i.allErrors || t.break();
      });
    }), e.ok(s);
  }
};
dv.default = Mre;
var Yp = {};
Object.defineProperty(Yp, "__esModule", { value: !0 });
const lf = Et, Po = St, Nre = ni, uf = ze, Fre = {
  message: "must NOT have additional properties",
  params: ({ params: e }) => (0, Po._)`{additionalProperty: ${e.additionalProperty}}`
}, Lre = {
  keyword: "additionalProperties",
  type: ["object"],
  schemaType: ["boolean", "object"],
  allowUndefined: !0,
  trackErrors: !0,
  error: Fre,
  code(e) {
    const { gen: t, schema: n, parentSchema: o, data: i, errsCount: s, it: l } = e;
    if (!s)
      throw new Error("ajv implementation error");
    const { allErrors: u, opts: c } = l;
    if (l.props = !0, c.removeAdditional !== "all" && (0, uf.alwaysValidSchema)(l, n))
      return;
    const f = (0, lf.allSchemaProperties)(o.properties), p = (0, lf.allSchemaProperties)(o.patternProperties);
    g(), e.ok((0, Po._)`${s} === ${Nre.default.errors}`);
    function g() {
      t.forIn("key", i, ($) => {
        !f.length && !p.length ? v($) : t.if(h($), () => v($));
      });
    }
    function h($) {
      let S;
      if (f.length > 8) {
        const w = (0, uf.schemaRefOrVal)(l, o.properties, "properties");
        S = (0, lf.isOwnProperty)(t, w, $);
      } else f.length ? S = (0, Po.or)(...f.map((w) => (0, Po._)`${$} === ${w}`)) : S = Po.nil;
      return p.length && (S = (0, Po.or)(S, ...p.map((w) => (0, Po._)`${(0, lf.usePattern)(e, w)}.test(${$})`))), (0, Po.not)(S);
    }
    function y($) {
      t.code((0, Po._)`delete ${i}[${$}]`);
    }
    function v($) {
      if (c.removeAdditional === "all" || c.removeAdditional && n === !1) {
        y($);
        return;
      }
      if (n === !1) {
        e.setParams({ additionalProperty: $ }), e.error(), u || t.break();
        return;
      }
      if (typeof n == "object" && !(0, uf.alwaysValidSchema)(l, n)) {
        const S = t.name("valid");
        c.removeAdditional === "failing" ? (C($, S, !1), t.if((0, Po.not)(S), () => {
          e.reset(), y($);
        })) : (C($, S), u || t.if((0, Po.not)(S), () => t.break()));
      }
    }
    function C($, S, w) {
      const _ = {
        keyword: "additionalProperties",
        dataProp: $,
        dataPropType: uf.Type.Str
      };
      w === !1 && Object.assign(_, {
        compositeRule: !0,
        createErrors: !1,
        allErrors: !1
      }), e.subschema(_, S);
    }
  }
};
Yp.default = Lre;
var fv = {};
Object.defineProperty(fv, "__esModule", { value: !0 });
const Dre = Ao, Ww = Et, lg = ze, qw = Yp, zre = {
  keyword: "properties",
  type: "object",
  schemaType: "object",
  code(e) {
    const { gen: t, schema: n, parentSchema: o, data: i, it: s } = e;
    s.opts.removeAdditional === "all" && o.additionalProperties === void 0 && qw.default.code(new Dre.KeywordCxt(s, qw.default, "additionalProperties"));
    const l = (0, Ww.allSchemaProperties)(n);
    for (const g of l)
      s.definedProperties.add(g);
    s.opts.unevaluated && l.length && s.props !== !0 && (s.props = lg.mergeEvaluated.props(t, (0, lg.toHash)(l), s.props));
    const u = l.filter((g) => !(0, lg.alwaysValidSchema)(s, n[g]));
    if (u.length === 0)
      return;
    const c = t.name("valid");
    for (const g of u)
      f(g) ? p(g) : (t.if((0, Ww.propertyInData)(t, i, g, s.opts.ownProperties)), p(g), s.allErrors || t.else().var(c, !0), t.endIf()), e.it.definedProperties.add(g), e.ok(c);
    function f(g) {
      return s.opts.useDefaults && !s.compositeRule && n[g].default !== void 0;
    }
    function p(g) {
      e.subschema({
        keyword: "properties",
        schemaProp: g,
        dataProp: g
      }, c);
    }
  }
};
fv.default = zre;
var pv = {};
Object.defineProperty(pv, "__esModule", { value: !0 });
const Kw = Et, cf = St, Hw = ze, Gw = ze, Bre = {
  keyword: "patternProperties",
  type: "object",
  schemaType: "object",
  code(e) {
    const { gen: t, schema: n, data: o, parentSchema: i, it: s } = e, { opts: l } = s, u = (0, Kw.allSchemaProperties)(n), c = u.filter((C) => (0, Hw.alwaysValidSchema)(s, n[C]));
    if (u.length === 0 || c.length === u.length && (!s.opts.unevaluated || s.props === !0))
      return;
    const f = l.strictSchema && !l.allowMatchingProperties && i.properties, p = t.name("valid");
    s.props !== !0 && !(s.props instanceof cf.Name) && (s.props = (0, Gw.evaluatedPropsToName)(t, s.props));
    const { props: g } = s;
    h();
    function h() {
      for (const C of u)
        f && y(C), s.allErrors ? v(C) : (t.var(p, !0), v(C), t.if(p));
    }
    function y(C) {
      for (const $ in f)
        new RegExp(C).test($) && (0, Hw.checkStrictMode)(s, `property ${$} matches pattern ${C} (use allowMatchingProperties)`);
    }
    function v(C) {
      t.forIn("key", o, ($) => {
        t.if((0, cf._)`${(0, Kw.usePattern)(e, C)}.test(${$})`, () => {
          const S = c.includes(C);
          S || e.subschema({
            keyword: "patternProperties",
            schemaProp: C,
            dataProp: $,
            dataPropType: Gw.Type.Str
          }, p), s.opts.unevaluated && g !== !0 ? t.assign((0, cf._)`${g}[${$}]`, !0) : !S && !s.allErrors && t.if((0, cf.not)(p), () => t.break());
        });
      });
    }
  }
};
pv.default = Bre;
var hv = {};
Object.defineProperty(hv, "__esModule", { value: !0 });
const Ure = ze, Vre = {
  keyword: "not",
  schemaType: ["object", "boolean"],
  trackErrors: !0,
  code(e) {
    const { gen: t, schema: n, it: o } = e;
    if ((0, Ure.alwaysValidSchema)(o, n)) {
      e.fail();
      return;
    }
    const i = t.name("valid");
    e.subschema({
      keyword: "not",
      compositeRule: !0,
      createErrors: !1,
      allErrors: !1
    }, i), e.failResult(i, () => e.reset(), () => e.error());
  },
  error: { message: "must NOT be valid" }
};
hv.default = Vre;
var mv = {};
Object.defineProperty(mv, "__esModule", { value: !0 });
const Wre = Et, qre = {
  keyword: "anyOf",
  schemaType: "array",
  trackErrors: !0,
  code: Wre.validateUnion,
  error: { message: "must match a schema in anyOf" }
};
mv.default = qre;
var gv = {};
Object.defineProperty(gv, "__esModule", { value: !0 });
const kf = St, Kre = ze, Hre = {
  message: "must match exactly one schema in oneOf",
  params: ({ params: e }) => (0, kf._)`{passingSchemas: ${e.passing}}`
}, Gre = {
  keyword: "oneOf",
  schemaType: "array",
  trackErrors: !0,
  error: Hre,
  code(e) {
    const { gen: t, schema: n, parentSchema: o, it: i } = e;
    if (!Array.isArray(n))
      throw new Error("ajv implementation error");
    if (i.opts.discriminator && o.discriminator)
      return;
    const s = n, l = t.let("valid", !1), u = t.let("passing", null), c = t.name("_valid");
    e.setParams({ passing: u }), t.block(f), e.result(l, () => e.reset(), () => e.error(!0));
    function f() {
      s.forEach((p, g) => {
        let h;
        (0, Kre.alwaysValidSchema)(i, p) ? t.var(c, !0) : h = e.subschema({
          keyword: "oneOf",
          schemaProp: g,
          compositeRule: !0
        }, c), g > 0 && t.if((0, kf._)`${c} && ${l}`).assign(l, !1).assign(u, (0, kf._)`[${u}, ${g}]`).else(), t.if(c, () => {
          t.assign(l, !0), t.assign(u, g), h && e.mergeEvaluated(h, kf.Name);
        });
      });
    }
  }
};
gv.default = Gre;
var yv = {};
Object.defineProperty(yv, "__esModule", { value: !0 });
const Jre = ze, Yre = {
  keyword: "allOf",
  schemaType: "array",
  code(e) {
    const { gen: t, schema: n, it: o } = e;
    if (!Array.isArray(n))
      throw new Error("ajv implementation error");
    const i = t.name("valid");
    n.forEach((s, l) => {
      if ((0, Jre.alwaysValidSchema)(o, s))
        return;
      const u = e.subschema({ keyword: "allOf", schemaProp: l }, i);
      e.ok(i), e.mergeEvaluated(u);
    });
  }
};
yv.default = Yre;
var vv = {};
Object.defineProperty(vv, "__esModule", { value: !0 });
const Qf = St, M_ = ze, Qre = {
  message: ({ params: e }) => (0, Qf.str)`must match "${e.ifClause}" schema`,
  params: ({ params: e }) => (0, Qf._)`{failingKeyword: ${e.ifClause}}`
}, Xre = {
  keyword: "if",
  schemaType: ["object", "boolean"],
  trackErrors: !0,
  error: Qre,
  code(e) {
    const { gen: t, parentSchema: n, it: o } = e;
    n.then === void 0 && n.else === void 0 && (0, M_.checkStrictMode)(o, '"if" without "then" and "else" is ignored');
    const i = Jw(o, "then"), s = Jw(o, "else");
    if (!i && !s)
      return;
    const l = t.let("valid", !0), u = t.name("_valid");
    if (c(), e.reset(), i && s) {
      const p = t.let("ifClause");
      e.setParams({ ifClause: p }), t.if(u, f("then", p), f("else", p));
    } else i ? t.if(u, f("then")) : t.if((0, Qf.not)(u), f("else"));
    e.pass(l, () => e.error(!0));
    function c() {
      const p = e.subschema({
        keyword: "if",
        compositeRule: !0,
        createErrors: !1,
        allErrors: !1
      }, u);
      e.mergeEvaluated(p);
    }
    function f(p, g) {
      return () => {
        const h = e.subschema({ keyword: p }, u);
        t.assign(l, u), e.mergeValidEvaluated(h, l), g ? t.assign(g, (0, Qf._)`${p}`) : e.setParams({ ifClause: p });
      };
    }
  }
};
function Jw(e, t) {
  const n = e.schema[t];
  return n !== void 0 && !(0, M_.alwaysValidSchema)(e, n);
}
vv.default = Xre;
var bv = {};
Object.defineProperty(bv, "__esModule", { value: !0 });
const Zre = ze, eoe = {
  keyword: ["then", "else"],
  schemaType: ["object", "boolean"],
  code({ keyword: e, parentSchema: t, it: n }) {
    t.if === void 0 && (0, Zre.checkStrictMode)(n, `"${e}" without "if" is ignored`);
  }
};
bv.default = eoe;
Object.defineProperty(sv, "__esModule", { value: !0 });
const toe = Fl, noe = lv, roe = Ll, ooe = uv, ioe = cv, aoe = R_, soe = dv, loe = Yp, uoe = fv, coe = pv, doe = hv, foe = mv, poe = gv, hoe = yv, moe = vv, goe = bv;
function yoe(e = !1) {
  const t = [
    // any
    doe.default,
    foe.default,
    poe.default,
    hoe.default,
    moe.default,
    goe.default,
    // object
    soe.default,
    loe.default,
    aoe.default,
    uoe.default,
    coe.default
  ];
  return e ? t.push(noe.default, ooe.default) : t.push(toe.default, roe.default), t.push(ioe.default), t;
}
sv.default = yoe;
var Sv = {}, wv = {};
Object.defineProperty(wv, "__esModule", { value: !0 });
const xn = St, voe = {
  message: ({ schemaCode: e }) => (0, xn.str)`must match format "${e}"`,
  params: ({ schemaCode: e }) => (0, xn._)`{format: ${e}}`
}, boe = {
  keyword: "format",
  type: ["number", "string"],
  schemaType: "string",
  $data: !0,
  error: voe,
  code(e, t) {
    const { gen: n, data: o, $data: i, schema: s, schemaCode: l, it: u } = e, { opts: c, errSchemaPath: f, schemaEnv: p, self: g } = u;
    if (!c.validateFormats)
      return;
    i ? h() : y();
    function h() {
      const v = n.scopeValue("formats", {
        ref: g.formats,
        code: c.code.formats
      }), C = n.const("fDef", (0, xn._)`${v}[${l}]`), $ = n.let("fType"), S = n.let("format");
      n.if((0, xn._)`typeof ${C} == "object" && !(${C} instanceof RegExp)`, () => n.assign($, (0, xn._)`${C}.type || "string"`).assign(S, (0, xn._)`${C}.validate`), () => n.assign($, (0, xn._)`"string"`).assign(S, C)), e.fail$data((0, xn.or)(w(), _()));
      function w() {
        return c.strictSchema === !1 ? xn.nil : (0, xn._)`${l} && !${S}`;
      }
      function _() {
        const b = p.$async ? (0, xn._)`(${C}.async ? await ${S}(${o}) : ${S}(${o}))` : (0, xn._)`${S}(${o})`, k = (0, xn._)`(typeof ${S} == "function" ? ${b} : ${S}.test(${o}))`;
        return (0, xn._)`${S} && ${S} !== true && ${$} === ${t} && !${k}`;
      }
    }
    function y() {
      const v = g.formats[s];
      if (!v) {
        w();
        return;
      }
      if (v === !0)
        return;
      const [C, $, S] = _(v);
      C === t && e.pass(b());
      function w() {
        if (c.strictSchema === !1) {
          g.logger.warn(k());
          return;
        }
        throw new Error(k());
        function k() {
          return `unknown format "${s}" ignored in schema at path "${f}"`;
        }
      }
      function _(k) {
        const E = k instanceof RegExp ? (0, xn.regexpCode)(k) : c.code.formats ? (0, xn._)`${c.code.formats}${(0, xn.getProperty)(s)}` : void 0, j = n.scopeValue("formats", { key: s, ref: k, code: E });
        return typeof k == "object" && !(k instanceof RegExp) ? [k.type || "string", k.validate, (0, xn._)`${j}.validate`] : ["string", k, j];
      }
      function b() {
        if (typeof v == "object" && !(v instanceof RegExp) && v.async) {
          if (!p.$async)
            throw new Error("async format in sync schema");
          return (0, xn._)`await ${S}(${o})`;
        }
        return typeof $ == "function" ? (0, xn._)`${S}(${o})` : (0, xn._)`${S}.test(${o})`;
      }
    }
  }
};
wv.default = boe;
Object.defineProperty(Sv, "__esModule", { value: !0 });
const Soe = wv, woe = [Soe.default];
Sv.default = woe;
var hl = {};
Object.defineProperty(hl, "__esModule", { value: !0 });
hl.contentVocabulary = hl.metadataVocabulary = void 0;
hl.metadataVocabulary = [
  "title",
  "description",
  "default",
  "deprecated",
  "readOnly",
  "writeOnly",
  "examples"
];
hl.contentVocabulary = [
  "contentMediaType",
  "contentEncoding",
  "contentSchema"
];
Object.defineProperty(K0, "__esModule", { value: !0 });
const $oe = H0, Coe = J0, xoe = sv, Eoe = Sv, Yw = hl, _oe = [
  $oe.default,
  Coe.default,
  (0, xoe.default)(),
  Eoe.default,
  Yw.metadataVocabulary,
  Yw.contentVocabulary
];
K0.default = _oe;
var $v = {}, Qp = {};
Object.defineProperty(Qp, "__esModule", { value: !0 });
Qp.DiscrError = void 0;
var Qw;
(function(e) {
  e.Tag = "tag", e.Mapping = "mapping";
})(Qw || (Qp.DiscrError = Qw = {}));
Object.defineProperty($v, "__esModule", { value: !0 });
const qs = St, Jg = Qp, Xw = Mr, koe = Nl, Poe = ze, Ooe = {
  message: ({ params: { discrError: e, tagName: t } }) => e === Jg.DiscrError.Tag ? `tag "${t}" must be string` : `value of tag "${t}" must be in oneOf`,
  params: ({ params: { discrError: e, tag: t, tagName: n } }) => (0, qs._)`{error: ${e}, tag: ${n}, tagValue: ${t}}`
}, Aoe = {
  keyword: "discriminator",
  type: "object",
  schemaType: "object",
  error: Ooe,
  code(e) {
    const { gen: t, data: n, schema: o, parentSchema: i, it: s } = e, { oneOf: l } = i;
    if (!s.opts.discriminator)
      throw new Error("discriminator: requires discriminator option");
    const u = o.propertyName;
    if (typeof u != "string")
      throw new Error("discriminator: requires propertyName");
    if (o.mapping)
      throw new Error("discriminator: mapping is not supported");
    if (!l)
      throw new Error("discriminator: requires oneOf keyword");
    const c = t.let("valid", !1), f = t.const("tag", (0, qs._)`${n}${(0, qs.getProperty)(u)}`);
    t.if((0, qs._)`typeof ${f} == "string"`, () => p(), () => e.error(!1, { discrError: Jg.DiscrError.Tag, tag: f, tagName: u })), e.ok(c);
    function p() {
      const y = h();
      t.if(!1);
      for (const v in y)
        t.elseIf((0, qs._)`${f} === ${v}`), t.assign(c, g(y[v]));
      t.else(), e.error(!1, { discrError: Jg.DiscrError.Mapping, tag: f, tagName: u }), t.endIf();
    }
    function g(y) {
      const v = t.name("valid"), C = e.subschema({ keyword: "oneOf", schemaProp: y }, v);
      return e.mergeEvaluated(C, qs.Name), v;
    }
    function h() {
      var y;
      const v = {}, C = S(i);
      let $ = !0;
      for (let b = 0; b < l.length; b++) {
        let k = l[b];
        if (k != null && k.$ref && !(0, Poe.schemaHasRulesButRef)(k, s.self.RULES)) {
          const j = k.$ref;
          if (k = Xw.resolveRef.call(s.self, s.schemaEnv.root, s.baseId, j), k instanceof Xw.SchemaEnv && (k = k.schema), k === void 0)
            throw new koe.default(s.opts.uriResolver, s.baseId, j);
        }
        const E = (y = k == null ? void 0 : k.properties) === null || y === void 0 ? void 0 : y[u];
        if (typeof E != "object")
          throw new Error(`discriminator: oneOf subschemas (or referenced schemas) must have "properties/${u}"`);
        $ = $ && (C || S(k)), w(E, b);
      }
      if (!$)
        throw new Error(`discriminator: "${u}" must be required`);
      return v;
      function S({ required: b }) {
        return Array.isArray(b) && b.includes(u);
      }
      function w(b, k) {
        if (b.const)
          _(b.const, k);
        else if (b.enum)
          for (const E of b.enum)
            _(E, k);
        else
          throw new Error(`discriminator: "properties/${u}" must have "const" or "enum"`);
      }
      function _(b, k) {
        if (typeof b != "string" || b in v)
          throw new Error(`discriminator: "${u}" values must be unique strings`);
        v[b] = k;
      }
    }
  }
};
$v.default = Aoe;
const Ioe = "http://json-schema.org/draft-07/schema#", joe = "http://json-schema.org/draft-07/schema#", Roe = "Core schema meta-schema", Toe = {
  schemaArray: {
    type: "array",
    minItems: 1,
    items: {
      $ref: "#"
    }
  },
  nonNegativeInteger: {
    type: "integer",
    minimum: 0
  },
  nonNegativeIntegerDefault0: {
    allOf: [
      {
        $ref: "#/definitions/nonNegativeInteger"
      },
      {
        default: 0
      }
    ]
  },
  simpleTypes: {
    enum: [
      "array",
      "boolean",
      "integer",
      "null",
      "number",
      "object",
      "string"
    ]
  },
  stringArray: {
    type: "array",
    items: {
      type: "string"
    },
    uniqueItems: !0,
    default: []
  }
}, Moe = [
  "object",
  "boolean"
], Noe = {
  $id: {
    type: "string",
    format: "uri-reference"
  },
  $schema: {
    type: "string",
    format: "uri"
  },
  $ref: {
    type: "string",
    format: "uri-reference"
  },
  $comment: {
    type: "string"
  },
  title: {
    type: "string"
  },
  description: {
    type: "string"
  },
  default: !0,
  readOnly: {
    type: "boolean",
    default: !1
  },
  examples: {
    type: "array",
    items: !0
  },
  multipleOf: {
    type: "number",
    exclusiveMinimum: 0
  },
  maximum: {
    type: "number"
  },
  exclusiveMaximum: {
    type: "number"
  },
  minimum: {
    type: "number"
  },
  exclusiveMinimum: {
    type: "number"
  },
  maxLength: {
    $ref: "#/definitions/nonNegativeInteger"
  },
  minLength: {
    $ref: "#/definitions/nonNegativeIntegerDefault0"
  },
  pattern: {
    type: "string",
    format: "regex"
  },
  additionalItems: {
    $ref: "#"
  },
  items: {
    anyOf: [
      {
        $ref: "#"
      },
      {
        $ref: "#/definitions/schemaArray"
      }
    ],
    default: !0
  },
  maxItems: {
    $ref: "#/definitions/nonNegativeInteger"
  },
  minItems: {
    $ref: "#/definitions/nonNegativeIntegerDefault0"
  },
  uniqueItems: {
    type: "boolean",
    default: !1
  },
  contains: {
    $ref: "#"
  },
  maxProperties: {
    $ref: "#/definitions/nonNegativeInteger"
  },
  minProperties: {
    $ref: "#/definitions/nonNegativeIntegerDefault0"
  },
  required: {
    $ref: "#/definitions/stringArray"
  },
  additionalProperties: {
    $ref: "#"
  },
  definitions: {
    type: "object",
    additionalProperties: {
      $ref: "#"
    },
    default: {}
  },
  properties: {
    type: "object",
    additionalProperties: {
      $ref: "#"
    },
    default: {}
  },
  patternProperties: {
    type: "object",
    additionalProperties: {
      $ref: "#"
    },
    propertyNames: {
      format: "regex"
    },
    default: {}
  },
  dependencies: {
    type: "object",
    additionalProperties: {
      anyOf: [
        {
          $ref: "#"
        },
        {
          $ref: "#/definitions/stringArray"
        }
      ]
    }
  },
  propertyNames: {
    $ref: "#"
  },
  const: !0,
  enum: {
    type: "array",
    items: !0,
    minItems: 1,
    uniqueItems: !0
  },
  type: {
    anyOf: [
      {
        $ref: "#/definitions/simpleTypes"
      },
      {
        type: "array",
        items: {
          $ref: "#/definitions/simpleTypes"
        },
        minItems: 1,
        uniqueItems: !0
      }
    ]
  },
  format: {
    type: "string"
  },
  contentMediaType: {
    type: "string"
  },
  contentEncoding: {
    type: "string"
  },
  if: {
    $ref: "#"
  },
  then: {
    $ref: "#"
  },
  else: {
    $ref: "#"
  },
  allOf: {
    $ref: "#/definitions/schemaArray"
  },
  anyOf: {
    $ref: "#/definitions/schemaArray"
  },
  oneOf: {
    $ref: "#/definitions/schemaArray"
  },
  not: {
    $ref: "#"
  }
}, Foe = {
  $schema: Ioe,
  $id: joe,
  title: Roe,
  definitions: Toe,
  type: Moe,
  properties: Noe,
  default: !0
};
(function(e, t) {
  Object.defineProperty(t, "__esModule", { value: !0 }), t.MissingRefError = t.ValidationError = t.CodeGen = t.Name = t.nil = t.stringify = t.str = t._ = t.KeywordCxt = t.Ajv = void 0;
  const n = zE, o = K0, i = $v, s = Foe, l = ["/properties"], u = "http://json-schema.org/draft-07/schema";
  class c extends n.default {
    _addVocabularies() {
      super._addVocabularies(), o.default.forEach((v) => this.addVocabulary(v)), this.opts.discriminator && this.addKeyword(i.default);
    }
    _addDefaultMetaSchema() {
      if (super._addDefaultMetaSchema(), !this.opts.meta)
        return;
      const v = this.opts.$data ? this.$dataMetaSchema(s, l) : s;
      this.addMetaSchema(v, u, !1), this.refs["http://json-schema.org/schema"] = u;
    }
    defaultMeta() {
      return this.opts.defaultMeta = super.defaultMeta() || (this.getSchema(u) ? u : void 0);
    }
  }
  t.Ajv = c, e.exports = t = c, e.exports.Ajv = c, Object.defineProperty(t, "__esModule", { value: !0 }), t.default = c;
  var f = Ao;
  Object.defineProperty(t, "KeywordCxt", { enumerable: !0, get: function() {
    return f.KeywordCxt;
  } });
  var p = St;
  Object.defineProperty(t, "_", { enumerable: !0, get: function() {
    return p._;
  } }), Object.defineProperty(t, "str", { enumerable: !0, get: function() {
    return p.str;
  } }), Object.defineProperty(t, "stringify", { enumerable: !0, get: function() {
    return p.stringify;
  } }), Object.defineProperty(t, "nil", { enumerable: !0, get: function() {
    return p.nil;
  } }), Object.defineProperty(t, "Name", { enumerable: !0, get: function() {
    return p.Name;
  } }), Object.defineProperty(t, "CodeGen", { enumerable: !0, get: function() {
    return p.CodeGen;
  } });
  var g = Nc;
  Object.defineProperty(t, "ValidationError", { enumerable: !0, get: function() {
    return g.default;
  } });
  var h = Nl;
  Object.defineProperty(t, "MissingRefError", { enumerable: !0, get: function() {
    return h.default;
  } });
})(Bg, Bg.exports);
var N_ = Bg.exports;
const Loe = /* @__PURE__ */ Ft(N_);
var Yg = { exports: {} }, F_ = {};
(function(e) {
  Object.defineProperty(e, "__esModule", { value: !0 }), e.formatNames = e.fastFormats = e.fullFormats = void 0;
  function t(B, O) {
    return { validate: B, compare: O };
  }
  e.fullFormats = {
    // date: http://tools.ietf.org/html/rfc3339#section-5.6
    date: t(s, l),
    // date-time: http://tools.ietf.org/html/rfc3339#section-5.6
    time: t(c, f),
    "date-time": t(g, h),
    // duration: https://tools.ietf.org/html/rfc3339#appendix-A
    duration: /^P(?!$)((\d+Y)?(\d+M)?(\d+D)?(T(?=\d)(\d+H)?(\d+M)?(\d+S)?)?|(\d+W)?)$/,
    uri: C,
    "uri-reference": /^(?:[a-z][a-z0-9+\-.]*:)?(?:\/?\/(?:(?:[a-z0-9\-._~!$&'()*+,;=:]|%[0-9a-f]{2})*@)?(?:\[(?:(?:(?:(?:[0-9a-f]{1,4}:){6}|::(?:[0-9a-f]{1,4}:){5}|(?:[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){4}|(?:(?:[0-9a-f]{1,4}:){0,1}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){3}|(?:(?:[0-9a-f]{1,4}:){0,2}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){2}|(?:(?:[0-9a-f]{1,4}:){0,3}[0-9a-f]{1,4})?::[0-9a-f]{1,4}:|(?:(?:[0-9a-f]{1,4}:){0,4}[0-9a-f]{1,4})?::)(?:[0-9a-f]{1,4}:[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?))|(?:(?:[0-9a-f]{1,4}:){0,5}[0-9a-f]{1,4})?::[0-9a-f]{1,4}|(?:(?:[0-9a-f]{1,4}:){0,6}[0-9a-f]{1,4})?::)|[Vv][0-9a-f]+\.[a-z0-9\-._~!$&'()*+,;=:]+)\]|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)|(?:[a-z0-9\-._~!$&'"()*+,;=]|%[0-9a-f]{2})*)(?::\d*)?(?:\/(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})*)*|\/(?:(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})*)*)?|(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})*)*)?(?:\?(?:[a-z0-9\-._~!$&'"()*+,;=:@/?]|%[0-9a-f]{2})*)?(?:#(?:[a-z0-9\-._~!$&'"()*+,;=:@/?]|%[0-9a-f]{2})*)?$/i,
    // uri-template: https://tools.ietf.org/html/rfc6570
    "uri-template": /^(?:(?:[^\x00-\x20"'<>%\\^`{|}]|%[0-9a-f]{2})|\{[+#./;?&=,!@|]?(?:[a-z0-9_]|%[0-9a-f]{2})+(?::[1-9][0-9]{0,3}|\*)?(?:,(?:[a-z0-9_]|%[0-9a-f]{2})+(?::[1-9][0-9]{0,3}|\*)?)*\})*$/i,
    // For the source: https://gist.github.com/dperini/729294
    // For test cases: https://mathiasbynens.be/demo/url-regex
    url: /^(?:https?|ftp):\/\/(?:\S+(?::\S*)?@)?(?:(?!(?:10|127)(?:\.\d{1,3}){3})(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z0-9\u{00a1}-\u{ffff}]+-)*[a-z0-9\u{00a1}-\u{ffff}]+)(?:\.(?:[a-z0-9\u{00a1}-\u{ffff}]+-)*[a-z0-9\u{00a1}-\u{ffff}]+)*(?:\.(?:[a-z\u{00a1}-\u{ffff}]{2,})))(?::\d{2,5})?(?:\/[^\s]*)?$/iu,
    email: /^[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?$/i,
    hostname: /^(?=.{1,253}\.?$)[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?(?:\.[a-z0-9](?:[-0-9a-z]{0,61}[0-9a-z])?)*\.?$/i,
    // optimized https://www.safaribooksonline.com/library/view/regular-expressions-cookbook/9780596802837/ch07s16.html
    ipv4: /^(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)$/,
    ipv6: /^((([0-9a-f]{1,4}:){7}([0-9a-f]{1,4}|:))|(([0-9a-f]{1,4}:){6}(:[0-9a-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9a-f]{1,4}:){5}(((:[0-9a-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9a-f]{1,4}:){4}(((:[0-9a-f]{1,4}){1,3})|((:[0-9a-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9a-f]{1,4}:){3}(((:[0-9a-f]{1,4}){1,4})|((:[0-9a-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9a-f]{1,4}:){2}(((:[0-9a-f]{1,4}){1,5})|((:[0-9a-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9a-f]{1,4}:){1}(((:[0-9a-f]{1,4}){1,6})|((:[0-9a-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9a-f]{1,4}){1,7})|((:[0-9a-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))$/i,
    regex: L,
    // uuid: http://tools.ietf.org/html/rfc4122
    uuid: /^(?:urn:uuid:)?[0-9a-f]{8}-(?:[0-9a-f]{4}-){3}[0-9a-f]{12}$/i,
    // JSON-pointer: https://tools.ietf.org/html/rfc6901
    // uri fragment: https://tools.ietf.org/html/rfc3986#appendix-A
    "json-pointer": /^(?:\/(?:[^~/]|~0|~1)*)*$/,
    "json-pointer-uri-fragment": /^#(?:\/(?:[a-z0-9_\-.!$&'()*+,;:=@]|%[0-9a-f]{2}|~0|~1)*)*$/i,
    // relative JSON-pointer: http://tools.ietf.org/html/draft-luff-relative-json-pointer-00
    "relative-json-pointer": /^(?:0|[1-9][0-9]*)(?:#|(?:\/(?:[^~/]|~0|~1)*)*)$/,
    // the following formats are used by the openapi specification: https://spec.openapis.org/oas/v3.0.0#data-types
    // byte: https://github.com/miguelmota/is-base64
    byte: S,
    // signed 32 bit integer
    int32: { type: "number", validate: b },
    // signed 64 bit integer
    int64: { type: "number", validate: k },
    // C-type float
    float: { type: "number", validate: E },
    // C-type double
    double: { type: "number", validate: E },
    // hint to the UI to hide input strings
    password: !0,
    // unchecked string payload
    binary: !0
  }, e.fastFormats = {
    ...e.fullFormats,
    date: t(/^\d\d\d\d-[0-1]\d-[0-3]\d$/, l),
    time: t(/^(?:[0-2]\d:[0-5]\d:[0-5]\d|23:59:60)(?:\.\d+)?(?:z|[+-]\d\d(?::?\d\d)?)?$/i, f),
    "date-time": t(/^\d\d\d\d-[0-1]\d-[0-3]\d[t\s](?:[0-2]\d:[0-5]\d:[0-5]\d|23:59:60)(?:\.\d+)?(?:z|[+-]\d\d(?::?\d\d)?)$/i, h),
    // uri: https://github.com/mafintosh/is-my-json-valid/blob/master/formats.js
    uri: /^(?:[a-z][a-z0-9+\-.]*:)(?:\/?\/)?[^\s]*$/i,
    "uri-reference": /^(?:(?:[a-z][a-z0-9+\-.]*:)?\/?\/)?(?:[^\\\s#][^\s#]*)?(?:#[^\\\s]*)?$/i,
    // email (sources from jsen validator):
    // http://stackoverflow.com/questions/201323/using-a-regular-expression-to-validate-an-email-address#answer-8829363
    // http://www.w3.org/TR/html5/forms.html#valid-e-mail-address (search for 'wilful violation')
    email: /^[a-z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?(?:\.[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?)*$/i
  }, e.formatNames = Object.keys(e.fullFormats);
  function n(B) {
    return B % 4 === 0 && (B % 100 !== 0 || B % 400 === 0);
  }
  const o = /^(\d\d\d\d)-(\d\d)-(\d\d)$/, i = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
  function s(B) {
    const O = o.exec(B);
    if (!O)
      return !1;
    const V = +O[1], W = +O[2], q = +O[3];
    return W >= 1 && W <= 12 && q >= 1 && q <= (W === 2 && n(V) ? 29 : i[W]);
  }
  function l(B, O) {
    if (B && O)
      return B > O ? 1 : B < O ? -1 : 0;
  }
  const u = /^(\d\d):(\d\d):(\d\d)(\.\d+)?(z|[+-]\d\d(?::?\d\d)?)?$/i;
  function c(B, O) {
    const V = u.exec(B);
    if (!V)
      return !1;
    const W = +V[1], q = +V[2], Q = +V[3], X = V[5];
    return (W <= 23 && q <= 59 && Q <= 59 || W === 23 && q === 59 && Q === 60) && (!O || X !== "");
  }
  function f(B, O) {
    if (!(B && O))
      return;
    const V = u.exec(B), W = u.exec(O);
    if (V && W)
      return B = V[1] + V[2] + V[3] + (V[4] || ""), O = W[1] + W[2] + W[3] + (W[4] || ""), B > O ? 1 : B < O ? -1 : 0;
  }
  const p = /t|\s/i;
  function g(B) {
    const O = B.split(p);
    return O.length === 2 && s(O[0]) && c(O[1], !0);
  }
  function h(B, O) {
    if (!(B && O))
      return;
    const [V, W] = B.split(p), [q, Q] = O.split(p), X = l(V, q);
    if (X !== void 0)
      return X || f(W, Q);
  }
  const y = /\/|:/, v = /^(?:[a-z][a-z0-9+\-.]*:)(?:\/?\/(?:(?:[a-z0-9\-._~!$&'()*+,;=:]|%[0-9a-f]{2})*@)?(?:\[(?:(?:(?:(?:[0-9a-f]{1,4}:){6}|::(?:[0-9a-f]{1,4}:){5}|(?:[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){4}|(?:(?:[0-9a-f]{1,4}:){0,1}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){3}|(?:(?:[0-9a-f]{1,4}:){0,2}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){2}|(?:(?:[0-9a-f]{1,4}:){0,3}[0-9a-f]{1,4})?::[0-9a-f]{1,4}:|(?:(?:[0-9a-f]{1,4}:){0,4}[0-9a-f]{1,4})?::)(?:[0-9a-f]{1,4}:[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?))|(?:(?:[0-9a-f]{1,4}:){0,5}[0-9a-f]{1,4})?::[0-9a-f]{1,4}|(?:(?:[0-9a-f]{1,4}:){0,6}[0-9a-f]{1,4})?::)|[Vv][0-9a-f]+\.[a-z0-9\-._~!$&'()*+,;=:]+)\]|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)|(?:[a-z0-9\-._~!$&'()*+,;=]|%[0-9a-f]{2})*)(?::\d*)?(?:\/(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})*)*|\/(?:(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})*)*)?|(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})*)*)(?:\?(?:[a-z0-9\-._~!$&'()*+,;=:@/?]|%[0-9a-f]{2})*)?(?:#(?:[a-z0-9\-._~!$&'()*+,;=:@/?]|%[0-9a-f]{2})*)?$/i;
  function C(B) {
    return y.test(B) && v.test(B);
  }
  const $ = /^(?:[A-Za-z0-9+/]{4})*(?:[A-Za-z0-9+/]{2}==|[A-Za-z0-9+/]{3}=)?$/gm;
  function S(B) {
    return $.lastIndex = 0, $.test(B);
  }
  const w = -2147483648, _ = 2 ** 31 - 1;
  function b(B) {
    return Number.isInteger(B) && B <= _ && B >= w;
  }
  function k(B) {
    return Number.isInteger(B);
  }
  function E() {
    return !0;
  }
  const j = /[^\\]\\Z/;
  function L(B) {
    if (j.test(B))
      return !1;
    try {
      return new RegExp(B), !0;
    } catch {
      return !1;
    }
  }
})(F_);
var L_ = {};
(function(e) {
  Object.defineProperty(e, "__esModule", { value: !0 }), e.formatLimitDefinition = void 0;
  const t = N_, n = St, o = n.operators, i = {
    formatMaximum: { okStr: "<=", ok: o.LTE, fail: o.GT },
    formatMinimum: { okStr: ">=", ok: o.GTE, fail: o.LT },
    formatExclusiveMaximum: { okStr: "<", ok: o.LT, fail: o.GTE },
    formatExclusiveMinimum: { okStr: ">", ok: o.GT, fail: o.LTE }
  }, s = {
    message: ({ keyword: u, schemaCode: c }) => n.str`should be ${i[u].okStr} ${c}`,
    params: ({ keyword: u, schemaCode: c }) => n._`{comparison: ${i[u].okStr}, limit: ${c}}`
  };
  e.formatLimitDefinition = {
    keyword: Object.keys(i),
    type: "string",
    schemaType: "string",
    $data: !0,
    error: s,
    code(u) {
      const { gen: c, data: f, schemaCode: p, keyword: g, it: h } = u, { opts: y, self: v } = h;
      if (!y.validateFormats)
        return;
      const C = new t.KeywordCxt(h, v.RULES.all.format.definition, "format");
      C.$data ? $() : S();
      function $() {
        const _ = c.scopeValue("formats", {
          ref: v.formats,
          code: y.code.formats
        }), b = c.const("fmt", n._`${_}[${C.schemaCode}]`);
        u.fail$data(n.or(n._`typeof ${b} != "object"`, n._`${b} instanceof RegExp`, n._`typeof ${b}.compare != "function"`, w(b)));
      }
      function S() {
        const _ = C.schema, b = v.formats[_];
        if (!b || b === !0)
          return;
        if (typeof b != "object" || b instanceof RegExp || typeof b.compare != "function")
          throw new Error(`"${g}": format "${_}" does not define "compare" function`);
        const k = c.scopeValue("formats", {
          key: _,
          ref: b,
          code: y.code.formats ? n._`${y.code.formats}${n.getProperty(_)}` : void 0
        });
        u.fail$data(w(k));
      }
      function w(_) {
        return n._`${_}.compare(${f}, ${p}) ${i[g].fail} 0`;
      }
    },
    dependencies: ["format"]
  };
  const l = (u) => (u.addKeyword(e.formatLimitDefinition), u);
  e.default = l;
})(L_);
(function(e, t) {
  Object.defineProperty(t, "__esModule", { value: !0 });
  const n = F_, o = L_, i = St, s = new i.Name("fullFormats"), l = new i.Name("fastFormats"), u = (f, p = { keywords: !0 }) => {
    if (Array.isArray(p))
      return c(f, p, n.fullFormats, s), f;
    const [g, h] = p.mode === "fast" ? [n.fastFormats, l] : [n.fullFormats, s], y = p.formats || n.formatNames;
    return c(f, y, g, h), p.keywords && o.default(f), f;
  };
  u.get = (f, p = "full") => {
    const h = (p === "fast" ? n.fastFormats : n.fullFormats)[f];
    if (!h)
      throw new Error(`Unknown format "${f}"`);
    return h;
  };
  function c(f, p, g, h) {
    var y, v;
    (y = (v = f.opts.code).formats) !== null && y !== void 0 || (v.formats = i._`require("ajv-formats/dist/formats").${h}`);
    for (const C of p)
      f.addFormat(C, g[C]);
  }
  e.exports = t = u, Object.defineProperty(t, "__esModule", { value: !0 }), t.default = u;
})(Yg, Yg.exports);
var Doe = Yg.exports;
const Zw = /* @__PURE__ */ Ft(Doe), zoe = {
  allErrors: !0,
  multipleOfPrecision: 8,
  strict: !1,
  verbose: !0,
  discriminator: !1
  // TODO enable this in V6
}, Boe = /^(#?([0-9A-Fa-f]{3}){1,2}\b|aqua|black|blue|fuchsia|gray|green|lime|maroon|navy|olive|orange|purple|red|silver|teal|white|yellow|(rgb\(\s*\b([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])\b\s*,\s*\b([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])\b\s*,\s*\b([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])\b\s*\))|(rgb\(\s*(\d?\d%|100%)+\s*,\s*(\d?\d%|100%)+\s*,\s*(\d?\d%|100%)+\s*\)))$/, Uoe = /^data:([a-z]+\/[a-z0-9-+.]+)?;(?:name=(.*);)?base64,(.*)$/;
function Voe(e, t, n = {}, o, i = Loe) {
  const s = new i({ ...zoe, ...n });
  return o ? Zw(s, o) : o !== !1 && Zw(s), s.addFormat("data-url", Uoe), s.addFormat("color", Boe), s.addKeyword(gl), s.addKeyword(sy), Array.isArray(e) && s.addMetaSchema(e), co(t) && Object.keys(t).forEach((l) => {
    s.addFormat(l, t[l]);
  }), s;
}
function Woe(e = [], t) {
  return e.map((n) => {
    const { instancePath: o, keyword: i, params: s, schemaPath: l, parentSchema: u, ...c } = n;
    let { message: f = "" } = c, p = o.replace(/\//g, "."), g = `${p} ${f}`.trim();
    if ("missingProperty" in s) {
      p = p ? `${p}.${s.missingProperty}` : s.missingProperty;
      const h = s.missingProperty;
      let y = Ot(Je(t, `${p.replace(/^\./, "")}`)).title;
      if (y === void 0) {
        const v = l.replace(/\/properties\//g, "/").split("/").slice(1, -1).concat([h]);
        y = Ot(Je(t, v)).title;
      }
      if (y)
        f = f.replace(`'${h}'`, `'${y}'`);
      else {
        const v = Je(u, [wn, h, "title"]);
        v && (f = f.replace(`'${h}'`, `'${v}'`));
      }
      g = f;
    } else {
      const h = Ot(Je(t, `${p.replace(/^\./, "")}`)).title;
      if (h)
        g = `'${h}' ${f}`.trim();
      else {
        const y = u == null ? void 0 : u.title;
        y && (g = `'${y}' ${f}`.trim());
      }
    }
    return {
      name: i,
      property: p,
      message: f,
      params: s,
      stack: g,
      schemaPath: l
    };
  });
}
function qoe(e, t, n, o, i, s, l) {
  const { validationError: u } = t;
  let c = Woe(t.errors, l);
  u && (c = [...c, { stack: u.message }]), typeof s == "function" && (c = s(c, l));
  let f = SU(c);
  if (u && (f = {
    ...f,
    $schema: {
      __errors: [u.message]
    }
  }), typeof i != "function")
    return { errors: c, errorSchema: f };
  const p = tx(e, o, n, o, !0), g = i(p, dg(p), l), h = px(g);
  return yf({ errors: c, errorSchema: f }, h);
}
class Koe {
  /** Constructs an `AJV8Validator` instance using the `options`
   *
   * @param options - The `CustomValidatorOptionsType` options that are used to create the AJV instance
   * @param [localizer] - If provided, is used to localize a list of Ajv `ErrorObject`s
   */
  constructor(t, n) {
    const { additionalMetaSchemas: o, customFormats: i, ajvOptionsOverrides: s, ajvFormatOptions: l, AjvClass: u } = t;
    this.ajv = Voe(o, i, s, l, u), this.localizer = n;
  }
  /** Resets the internal AJV validator to clear schemas from it. Can be helpful for resetting the validator for tests.
   */
  reset() {
    this.ajv.removeSchema();
  }
  /** Converts an `errorSchema` into a list of `RJSFValidationErrors`
   *
   * @param errorSchema - The `ErrorSchema` instance to convert
   * @param [fieldPath=[]] - The current field path, defaults to [] if not specified
   * @deprecated - Use the `toErrorList()` function provided by `@rjsf/utils` instead. This function will be removed in
   *        the next major release.
   */
  toErrorList(t, n = []) {
    return ic(t, n);
  }
  /** Runs the pure validation of the `schema` and `formData` without any of the RJSF functionality. Provided for use
   * by the playground. Returns the `errors` from the validation
   *
   * @param schema - The schema against which to validate the form data   * @param schema
   * @param formData - The form data to validate
   */
  rawValidation(t, n) {
    var o, i;
    let s, l;
    t[Yi] && (l = this.ajv.getSchema(t[Yi]));
    try {
      l === void 0 && (l = this.ajv.compile(t)), l(n);
    } catch (c) {
      s = c;
    }
    let u;
    return l && (typeof this.localizer == "function" && (((o = l.errors) !== null && o !== void 0 ? o : []).forEach((c) => {
      var f;
      !((f = c.params) === null || f === void 0) && f.missingProperty && (c.params.missingProperty = `'${c.params.missingProperty}'`);
    }), this.localizer(l.errors), ((i = l.errors) !== null && i !== void 0 ? i : []).forEach((c) => {
      var f;
      !((f = c.params) === null || f === void 0) && f.missingProperty && (c.params.missingProperty = c.params.missingProperty.slice(1, -1));
    })), u = l.errors || void 0, l.errors = null), {
      errors: u,
      validationError: s
    };
  }
  /** This function processes the `formData` with an optional user contributed `customValidate` function, which receives
   * the form data and a `errorHandler` function that will be used to add custom validation errors for each field. Also
   * supports a `transformErrors` function that will take the raw AJV validation errors, prior to custom validation and
   * transform them in what ever way it chooses.
   *
   * @param formData - The form data to validate
   * @param schema - The schema against which to validate the form data
   * @param [customValidate] - An optional function that is used to perform custom validation
   * @param [transformErrors] - An optional function that is used to transform errors after AJV validation
   * @param [uiSchema] - An optional uiSchema that is passed to `transformErrors` and `customValidate`
   */
  validateFormData(t, n, o, i, s) {
    const l = this.rawValidation(n, t);
    return qoe(this, l, t, n, o, i, s);
  }
  /**
   * This function checks if a schema needs to be added and if the root schemas don't match it removes the old root schema from the ajv instance and adds the new one.
   * @param rootSchema - The root schema used to provide $ref resolutions
   */
  handleSchemaUpdate(t) {
    var n, o;
    const i = (n = t[Yi]) !== null && n !== void 0 ? n : h$;
    this.ajv.getSchema(i) === void 0 ? this.ajv.addSchema(t, i) : Sr(t, (o = this.ajv.getSchema(i)) === null || o === void 0 ? void 0 : o.schema) || (this.ajv.removeSchema(i), this.ajv.addSchema(t, i));
  }
  /** Validates data against a schema, returning true if the data is valid, or
   * false otherwise. If the schema is invalid, then this function will return
   * false.
   *
   * @param schema - The schema against which to validate the form data
   * @param formData - The form data to validate
   * @param rootSchema - The root schema used to provide $ref resolutions
   */
  isValid(t, n, o) {
    var i;
    try {
      this.handleSchemaUpdate(o);
      const s = Xy(t), l = (i = s[Yi]) !== null && i !== void 0 ? i : iU(s);
      let u;
      return u = this.ajv.getSchema(l), u === void 0 && (u = this.ajv.addSchema(s, l).getSchema(l) || this.ajv.compile(s)), u(n);
    } catch (s) {
      return console.warn("Error encountered compiling schema:", s), !1;
    }
  }
}
function Hoe(e = {}, t) {
  return new Koe(e, t);
}
const Goe = Hoe(), {
  fields: { SchemaField: e$ }
} = Rx(), Joe = (e) => {
  if (e.schema.uiSchema) {
    const t = {
      ...e.uiSchema ? e.uiSchema : {},
      ...e.schema.uiSchema
    };
    return /* @__PURE__ */ A.jsx(e$, { ...e, uiSchema: t });
  }
  return /* @__PURE__ */ A.jsx(e$, { ...e });
}, t$ = (e) => console.log.bind(console, e);
function D_({
  schema: e,
  uiSchema: t,
  fields: n,
  widgets: o,
  onSubmit: i,
  onError: s,
  onChange: l
}) {
  return typeof t > "u" && (t = {}), e.uiSchema && (t = { ...t, ...e.uiSchema }), t.func_name = { "ui:widget": "hidden" }, Object.entries(e.properties).forEach(([u, c]) => {
    c.is_reporter && (t[u] = { "ui:widget": "hidden" });
  }), /* @__PURE__ */ A.jsx(A.Fragment, { children: /* @__PURE__ */ A.jsx(
    HZ,
    {
      schema: e,
      uiSchema: t,
      validator: Goe,
      onSubmit: typeof i < "u" ? i : t$("submitted"),
      onError: typeof s < "u" ? s : t$("errors"),
      fields: { SchemaField: Joe, ...n || {} },
      widgets: o,
      onChange: l
    }
  ) });
}
var z_ = { exports: {} };
(function(e, t) {
  (function(n, o) {
    e.exports = o(z);
  })(Tu, (n) => (() => {
    var o = { 9735: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "apathy", author: "jannik siebert (https://github.com/janniks)", base00: "#031A16", base01: "#0B342D", base02: "#184E45", base03: "#2B685E", base04: "#5F9C92", base05: "#81B5AC", base06: "#A7CEC8", base07: "#D2E7E4", base08: "#3E9688", base09: "#3E7996", base0A: "#3E4C96", base0B: "#883E96", base0C: "#963E4C", base0D: "#96883E", base0E: "#4C963E", base0F: "#3E965B" }, u.exports = c.default;
    }, 294: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "ashes", author: "jannik siebert (https://github.com/janniks)", base00: "#1C2023", base01: "#393F45", base02: "#565E65", base03: "#747C84", base04: "#ADB3BA", base05: "#C7CCD1", base06: "#DFE2E5", base07: "#F3F4F5", base08: "#C7AE95", base09: "#C7C795", base0A: "#AEC795", base0B: "#95C7AE", base0C: "#95AEC7", base0D: "#AE95C7", base0E: "#C795AE", base0F: "#C79595" }, u.exports = c.default;
    }, 1733: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "atelier dune", author: "bram de haan (http://atelierbram.github.io/syntax-highlighting/atelier-schemes/dune)", base00: "#20201d", base01: "#292824", base02: "#6e6b5e", base03: "#7d7a68", base04: "#999580", base05: "#a6a28c", base06: "#e8e4cf", base07: "#fefbec", base08: "#d73737", base09: "#b65611", base0A: "#cfb017", base0B: "#60ac39", base0C: "#1fad83", base0D: "#6684e1", base0E: "#b854d4", base0F: "#d43552" }, u.exports = c.default;
    }, 8974: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "atelier forest", author: "bram de haan (http://atelierbram.github.io/syntax-highlighting/atelier-schemes/forest)", base00: "#1b1918", base01: "#2c2421", base02: "#68615e", base03: "#766e6b", base04: "#9c9491", base05: "#a8a19f", base06: "#e6e2e0", base07: "#f1efee", base08: "#f22c40", base09: "#df5320", base0A: "#d5911a", base0B: "#5ab738", base0C: "#00ad9c", base0D: "#407ee7", base0E: "#6666ea", base0F: "#c33ff3" }, u.exports = c.default;
    }, 6933: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "atelier heath", author: "bram de haan (http://atelierbram.github.io/syntax-highlighting/atelier-schemes/heath)", base00: "#1b181b", base01: "#292329", base02: "#695d69", base03: "#776977", base04: "#9e8f9e", base05: "#ab9bab", base06: "#d8cad8", base07: "#f7f3f7", base08: "#ca402b", base09: "#a65926", base0A: "#bb8a35", base0B: "#379a37", base0C: "#159393", base0D: "#516aec", base0E: "#7b59c0", base0F: "#cc33cc" }, u.exports = c.default;
    }, 523: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "atelier lakeside", author: "bram de haan (http://atelierbram.github.io/syntax-highlighting/atelier-schemes/lakeside/)", base00: "#161b1d", base01: "#1f292e", base02: "#516d7b", base03: "#5a7b8c", base04: "#7195a8", base05: "#7ea2b4", base06: "#c1e4f6", base07: "#ebf8ff", base08: "#d22d72", base09: "#935c25", base0A: "#8a8a0f", base0B: "#568c3b", base0C: "#2d8f6f", base0D: "#257fad", base0E: "#5d5db1", base0F: "#b72dd2" }, u.exports = c.default;
    }, 1223: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "atelier seaside", author: "bram de haan (http://atelierbram.github.io/syntax-highlighting/atelier-schemes/seaside/)", base00: "#131513", base01: "#242924", base02: "#5e6e5e", base03: "#687d68", base04: "#809980", base05: "#8ca68c", base06: "#cfe8cf", base07: "#f0fff0", base08: "#e6193c", base09: "#87711d", base0A: "#c3c322", base0B: "#29a329", base0C: "#1999b3", base0D: "#3d62f5", base0E: "#ad2bee", base0F: "#e619c3" }, u.exports = c.default;
    }, 1233: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "bespin", author: "jan t. sott", base00: "#28211c", base01: "#36312e", base02: "#5e5d5c", base03: "#666666", base04: "#797977", base05: "#8a8986", base06: "#9d9b97", base07: "#baae9e", base08: "#cf6a4c", base09: "#cf7d34", base0A: "#f9ee98", base0B: "#54be0d", base0C: "#afc4db", base0D: "#5ea6ea", base0E: "#9b859d", base0F: "#937121" }, u.exports = c.default;
    }, 2847: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "brewer", author: "timothée poisot (http://github.com/tpoisot)", base00: "#0c0d0e", base01: "#2e2f30", base02: "#515253", base03: "#737475", base04: "#959697", base05: "#b7b8b9", base06: "#dadbdc", base07: "#fcfdfe", base08: "#e31a1c", base09: "#e6550d", base0A: "#dca060", base0B: "#31a354", base0C: "#80b1d3", base0D: "#3182bd", base0E: "#756bb1", base0F: "#b15928" }, u.exports = c.default;
    }, 8120: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "bright", author: "chris kempson (http://chriskempson.com)", base00: "#000000", base01: "#303030", base02: "#505050", base03: "#b0b0b0", base04: "#d0d0d0", base05: "#e0e0e0", base06: "#f5f5f5", base07: "#ffffff", base08: "#fb0120", base09: "#fc6d24", base0A: "#fda331", base0B: "#a1c659", base0C: "#76c7b7", base0D: "#6fb3d2", base0E: "#d381c3", base0F: "#be643c" }, u.exports = c.default;
    }, 6305: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "chalk", author: "chris kempson (http://chriskempson.com)", base00: "#151515", base01: "#202020", base02: "#303030", base03: "#505050", base04: "#b0b0b0", base05: "#d0d0d0", base06: "#e0e0e0", base07: "#f5f5f5", base08: "#fb9fb1", base09: "#eda987", base0A: "#ddb26f", base0B: "#acc267", base0C: "#12cfc0", base0D: "#6fc2ef", base0E: "#e1a3ee", base0F: "#deaf8f" }, u.exports = c.default;
    }, 525: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "codeschool", author: "brettof86", base00: "#232c31", base01: "#1c3657", base02: "#2a343a", base03: "#3f4944", base04: "#84898c", base05: "#9ea7a6", base06: "#a7cfa3", base07: "#b5d8f6", base08: "#2a5491", base09: "#43820d", base0A: "#a03b1e", base0B: "#237986", base0C: "#b02f30", base0D: "#484d79", base0E: "#c59820", base0F: "#c98344" }, u.exports = c.default;
    }, 4124: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "colors", author: "mrmrs (http://clrs.cc)", base00: "#111111", base01: "#333333", base02: "#555555", base03: "#777777", base04: "#999999", base05: "#bbbbbb", base06: "#dddddd", base07: "#ffffff", base08: "#ff4136", base09: "#ff851b", base0A: "#ffdc00", base0B: "#2ecc40", base0C: "#7fdbff", base0D: "#0074d9", base0E: "#b10dc9", base0F: "#85144b" }, u.exports = c.default;
    }, 7167: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "default", author: "chris kempson (http://chriskempson.com)", base00: "#181818", base01: "#282828", base02: "#383838", base03: "#585858", base04: "#b8b8b8", base05: "#d8d8d8", base06: "#e8e8e8", base07: "#f8f8f8", base08: "#ab4642", base09: "#dc9656", base0A: "#f7ca88", base0B: "#a1b56c", base0C: "#86c1b9", base0D: "#7cafc2", base0E: "#ba8baf", base0F: "#a16946" }, u.exports = c.default;
    }, 4582: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "eighties", author: "chris kempson (http://chriskempson.com)", base00: "#2d2d2d", base01: "#393939", base02: "#515151", base03: "#747369", base04: "#a09f93", base05: "#d3d0c8", base06: "#e8e6df", base07: "#f2f0ec", base08: "#f2777a", base09: "#f99157", base0A: "#ffcc66", base0B: "#99cc99", base0C: "#66cccc", base0D: "#6699cc", base0E: "#cc99cc", base0F: "#d27b53" }, u.exports = c.default;
    }, 7096: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "embers", author: "jannik siebert (https://github.com/janniks)", base00: "#16130F", base01: "#2C2620", base02: "#433B32", base03: "#5A5047", base04: "#8A8075", base05: "#A39A90", base06: "#BEB6AE", base07: "#DBD6D1", base08: "#826D57", base09: "#828257", base0A: "#6D8257", base0B: "#57826D", base0C: "#576D82", base0D: "#6D5782", base0E: "#82576D", base0F: "#825757" }, u.exports = c.default;
    }, 9887: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "flat", author: "chris kempson (http://chriskempson.com)", base00: "#2C3E50", base01: "#34495E", base02: "#7F8C8D", base03: "#95A5A6", base04: "#BDC3C7", base05: "#e0e0e0", base06: "#f5f5f5", base07: "#ECF0F1", base08: "#E74C3C", base09: "#E67E22", base0A: "#F1C40F", base0B: "#2ECC71", base0C: "#1ABC9C", base0D: "#3498DB", base0E: "#9B59B6", base0F: "#be643c" }, u.exports = c.default;
    }, 7199: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "google", author: "seth wright (http://sethawright.com)", base00: "#1d1f21", base01: "#282a2e", base02: "#373b41", base03: "#969896", base04: "#b4b7b4", base05: "#c5c8c6", base06: "#e0e0e0", base07: "#ffffff", base08: "#CC342B", base09: "#F96A38", base0A: "#FBA922", base0B: "#198844", base0C: "#3971ED", base0D: "#3971ED", base0E: "#A36AC7", base0F: "#3971ED" }, u.exports = c.default;
    }, 1985: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "grayscale", author: "alexandre gavioli (https://github.com/alexx2/)", base00: "#101010", base01: "#252525", base02: "#464646", base03: "#525252", base04: "#ababab", base05: "#b9b9b9", base06: "#e3e3e3", base07: "#f7f7f7", base08: "#7c7c7c", base09: "#999999", base0A: "#a0a0a0", base0B: "#8e8e8e", base0C: "#868686", base0D: "#686868", base0E: "#747474", base0F: "#5e5e5e" }, u.exports = c.default;
    }, 8093: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "green screen", author: "chris kempson (http://chriskempson.com)", base00: "#001100", base01: "#003300", base02: "#005500", base03: "#007700", base04: "#009900", base05: "#00bb00", base06: "#00dd00", base07: "#00ff00", base08: "#007700", base09: "#009900", base0A: "#007700", base0B: "#00bb00", base0C: "#005500", base0D: "#009900", base0E: "#00bb00", base0F: "#005500" }, u.exports = c.default;
    }, 1615: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "harmonic16", author: "jannik siebert (https://github.com/janniks)", base00: "#0b1c2c", base01: "#223b54", base02: "#405c79", base03: "#627e99", base04: "#aabcce", base05: "#cbd6e2", base06: "#e5ebf1", base07: "#f7f9fb", base08: "#bf8b56", base09: "#bfbf56", base0A: "#8bbf56", base0B: "#56bf8b", base0C: "#568bbf", base0D: "#8b56bf", base0E: "#bf568b", base0F: "#bf5656" }, u.exports = c.default;
    }, 9063: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "hopscotch", author: "jan t. sott", base00: "#322931", base01: "#433b42", base02: "#5c545b", base03: "#797379", base04: "#989498", base05: "#b9b5b8", base06: "#d5d3d5", base07: "#ffffff", base08: "#dd464c", base09: "#fd8b19", base0A: "#fdcc59", base0B: "#8fc13e", base0C: "#149b93", base0D: "#1290bf", base0E: "#c85e7c", base0F: "#b33508" }, u.exports = c.default;
    }, 9446: (u, c, f) => {
      function p(We) {
        return We && We.__esModule ? We.default : We;
      }
      c.__esModule = !0;
      var g = f(1308);
      c.threezerotwofour = p(g);
      var h = f(9735);
      c.apathy = p(h);
      var y = f(294);
      c.ashes = p(y);
      var v = f(1733);
      c.atelierDune = p(v);
      var C = f(8974);
      c.atelierForest = p(C);
      var $ = f(6933);
      c.atelierHeath = p($);
      var S = f(523);
      c.atelierLakeside = p(S);
      var w = f(1223);
      c.atelierSeaside = p(w);
      var _ = f(1233);
      c.bespin = p(_);
      var b = f(2847);
      c.brewer = p(b);
      var k = f(8120);
      c.bright = p(k);
      var E = f(6305);
      c.chalk = p(E);
      var j = f(525);
      c.codeschool = p(j);
      var L = f(4124);
      c.colors = p(L);
      var B = f(7167);
      c.default = p(B);
      var O = f(4582);
      c.eighties = p(O);
      var V = f(7096);
      c.embers = p(V);
      var W = f(9887);
      c.flat = p(W);
      var q = f(7199);
      c.google = p(q);
      var Q = f(1985);
      c.grayscale = p(Q);
      var X = f(8093);
      c.greenscreen = p(X);
      var ce = f(1615);
      c.harmonic = p(ce);
      var J = f(9063);
      c.hopscotch = p(J);
      var ie = f(9033);
      c.isotope = p(ie);
      var re = f(4112);
      c.marrakesh = p(re);
      var Y = f(9600);
      c.mocha = p(Y);
      var T = f(1240);
      c.monokai = p(T);
      var H = f(9768);
      c.ocean = p(H);
      var G = f(8293);
      c.paraiso = p(G);
      var N = f(3093);
      c.pop = p(N);
      var U = f(1951);
      c.railscasts = p(U);
      var Z = f(6368);
      c.shapeshifter = p(Z);
      var me = f(2317);
      c.solarized = p(me);
      var ue = f(1091);
      c.summerfruit = p(ue);
      var Ce = f(6943);
      c.tomorrow = p(Ce);
      var $e = f(5670);
      c.tube = p($e);
      var Te = f(2536);
      c.twilight = p(Te);
    }, 9033: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "isotope", author: "jan t. sott", base00: "#000000", base01: "#404040", base02: "#606060", base03: "#808080", base04: "#c0c0c0", base05: "#d0d0d0", base06: "#e0e0e0", base07: "#ffffff", base08: "#ff0000", base09: "#ff9900", base0A: "#ff0099", base0B: "#33ff00", base0C: "#00ffff", base0D: "#0066ff", base0E: "#cc00ff", base0F: "#3300ff" }, u.exports = c.default;
    }, 4112: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "marrakesh", author: "alexandre gavioli (http://github.com/alexx2/)", base00: "#201602", base01: "#302e00", base02: "#5f5b17", base03: "#6c6823", base04: "#86813b", base05: "#948e48", base06: "#ccc37a", base07: "#faf0a5", base08: "#c35359", base09: "#b36144", base0A: "#a88339", base0B: "#18974e", base0C: "#75a738", base0D: "#477ca1", base0E: "#8868b3", base0F: "#b3588e" }, u.exports = c.default;
    }, 9600: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "mocha", author: "chris kempson (http://chriskempson.com)", base00: "#3B3228", base01: "#534636", base02: "#645240", base03: "#7e705a", base04: "#b8afad", base05: "#d0c8c6", base06: "#e9e1dd", base07: "#f5eeeb", base08: "#cb6077", base09: "#d28b71", base0A: "#f4bc87", base0B: "#beb55b", base0C: "#7bbda4", base0D: "#8ab3b5", base0E: "#a89bb9", base0F: "#bb9584" }, u.exports = c.default;
    }, 1240: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "monokai", author: "wimer hazenberg (http://www.monokai.nl)", base00: "#272822", base01: "#383830", base02: "#49483e", base03: "#75715e", base04: "#a59f85", base05: "#f8f8f2", base06: "#f5f4f1", base07: "#f9f8f5", base08: "#f92672", base09: "#fd971f", base0A: "#f4bf75", base0B: "#a6e22e", base0C: "#a1efe4", base0D: "#66d9ef", base0E: "#ae81ff", base0F: "#cc6633" }, u.exports = c.default;
    }, 9768: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "ocean", author: "chris kempson (http://chriskempson.com)", base00: "#2b303b", base01: "#343d46", base02: "#4f5b66", base03: "#65737e", base04: "#a7adba", base05: "#c0c5ce", base06: "#dfe1e8", base07: "#eff1f5", base08: "#bf616a", base09: "#d08770", base0A: "#ebcb8b", base0B: "#a3be8c", base0C: "#96b5b4", base0D: "#8fa1b3", base0E: "#b48ead", base0F: "#ab7967" }, u.exports = c.default;
    }, 8293: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "paraiso", author: "jan t. sott", base00: "#2f1e2e", base01: "#41323f", base02: "#4f424c", base03: "#776e71", base04: "#8d8687", base05: "#a39e9b", base06: "#b9b6b0", base07: "#e7e9db", base08: "#ef6155", base09: "#f99b15", base0A: "#fec418", base0B: "#48b685", base0C: "#5bc4bf", base0D: "#06b6ef", base0E: "#815ba4", base0F: "#e96ba8" }, u.exports = c.default;
    }, 3093: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "pop", author: "chris kempson (http://chriskempson.com)", base00: "#000000", base01: "#202020", base02: "#303030", base03: "#505050", base04: "#b0b0b0", base05: "#d0d0d0", base06: "#e0e0e0", base07: "#ffffff", base08: "#eb008a", base09: "#f29333", base0A: "#f8ca12", base0B: "#37b349", base0C: "#00aabb", base0D: "#0e5a94", base0E: "#b31e8d", base0F: "#7a2d00" }, u.exports = c.default;
    }, 1951: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "railscasts", author: "ryan bates (http://railscasts.com)", base00: "#2b2b2b", base01: "#272935", base02: "#3a4055", base03: "#5a647e", base04: "#d4cfc9", base05: "#e6e1dc", base06: "#f4f1ed", base07: "#f9f7f3", base08: "#da4939", base09: "#cc7833", base0A: "#ffc66d", base0B: "#a5c261", base0C: "#519f50", base0D: "#6d9cbe", base0E: "#b6b3eb", base0F: "#bc9458" }, u.exports = c.default;
    }, 6368: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "shapeshifter", author: "tyler benziger (http://tybenz.com)", base00: "#000000", base01: "#040404", base02: "#102015", base03: "#343434", base04: "#555555", base05: "#ababab", base06: "#e0e0e0", base07: "#f9f9f9", base08: "#e92f2f", base09: "#e09448", base0A: "#dddd13", base0B: "#0ed839", base0C: "#23edda", base0D: "#3b48e3", base0E: "#f996e2", base0F: "#69542d" }, u.exports = c.default;
    }, 2317: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "solarized", author: "ethan schoonover (http://ethanschoonover.com/solarized)", base00: "#002b36", base01: "#073642", base02: "#586e75", base03: "#657b83", base04: "#839496", base05: "#93a1a1", base06: "#eee8d5", base07: "#fdf6e3", base08: "#dc322f", base09: "#cb4b16", base0A: "#b58900", base0B: "#859900", base0C: "#2aa198", base0D: "#268bd2", base0E: "#6c71c4", base0F: "#d33682" }, u.exports = c.default;
    }, 1091: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "summerfruit", author: "christopher corley (http://cscorley.github.io/)", base00: "#151515", base01: "#202020", base02: "#303030", base03: "#505050", base04: "#B0B0B0", base05: "#D0D0D0", base06: "#E0E0E0", base07: "#FFFFFF", base08: "#FF0086", base09: "#FD8900", base0A: "#ABA800", base0B: "#00C918", base0C: "#1faaaa", base0D: "#3777E6", base0E: "#AD00A1", base0F: "#cc6633" }, u.exports = c.default;
    }, 1308: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "threezerotwofour", author: "jan t. sott (http://github.com/idleberg)", base00: "#090300", base01: "#3a3432", base02: "#4a4543", base03: "#5c5855", base04: "#807d7c", base05: "#a5a2a2", base06: "#d6d5d4", base07: "#f7f7f7", base08: "#db2d20", base09: "#e8bbd0", base0A: "#fded02", base0B: "#01a252", base0C: "#b5e4f4", base0D: "#01a0e4", base0E: "#a16a94", base0F: "#cdab53" }, u.exports = c.default;
    }, 6943: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "tomorrow", author: "chris kempson (http://chriskempson.com)", base00: "#1d1f21", base01: "#282a2e", base02: "#373b41", base03: "#969896", base04: "#b4b7b4", base05: "#c5c8c6", base06: "#e0e0e0", base07: "#ffffff", base08: "#cc6666", base09: "#de935f", base0A: "#f0c674", base0B: "#b5bd68", base0C: "#8abeb7", base0D: "#81a2be", base0E: "#b294bb", base0F: "#a3685a" }, u.exports = c.default;
    }, 5670: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "london tube", author: "jan t. sott", base00: "#231f20", base01: "#1c3f95", base02: "#5a5758", base03: "#737171", base04: "#959ca1", base05: "#d9d8d8", base06: "#e7e7e8", base07: "#ffffff", base08: "#ee2e24", base09: "#f386a1", base0A: "#ffd204", base0B: "#00853e", base0C: "#85cebc", base0D: "#009ddc", base0E: "#98005d", base0F: "#b06110" }, u.exports = c.default;
    }, 2536: (u, c) => {
      c.__esModule = !0, c.default = { scheme: "twilight", author: "david hart (http://hart-dev.com)", base00: "#1e1e1e", base01: "#323537", base02: "#464b50", base03: "#5f5a60", base04: "#838184", base05: "#a7a7a7", base06: "#c3c3c3", base07: "#ffffff", base08: "#cf6a4c", base09: "#cda869", base0A: "#f9ee98", base0B: "#8f9d6a", base0C: "#afc4db", base0D: "#7587a6", base0E: "#9b859d", base0F: "#9b703f" }, u.exports = c.default;
    }, 6481: (u, c, f) => {
      var p = f(4176), g = {};
      for (var h in p) p.hasOwnProperty(h) && (g[p[h]] = h);
      var y = u.exports = { rgb: { channels: 3, labels: "rgb" }, hsl: { channels: 3, labels: "hsl" }, hsv: { channels: 3, labels: "hsv" }, hwb: { channels: 3, labels: "hwb" }, cmyk: { channels: 4, labels: "cmyk" }, xyz: { channels: 3, labels: "xyz" }, lab: { channels: 3, labels: "lab" }, lch: { channels: 3, labels: "lch" }, hex: { channels: 1, labels: ["hex"] }, keyword: { channels: 1, labels: ["keyword"] }, ansi16: { channels: 1, labels: ["ansi16"] }, ansi256: { channels: 1, labels: ["ansi256"] }, hcg: { channels: 3, labels: ["h", "c", "g"] }, apple: { channels: 3, labels: ["r16", "g16", "b16"] }, gray: { channels: 1, labels: ["gray"] } };
      for (var v in y) if (y.hasOwnProperty(v)) {
        if (!("channels" in y[v])) throw new Error("missing channels property: " + v);
        if (!("labels" in y[v])) throw new Error("missing channel labels property: " + v);
        if (y[v].labels.length !== y[v].channels) throw new Error("channel and label counts mismatch: " + v);
        var C = y[v].channels, $ = y[v].labels;
        delete y[v].channels, delete y[v].labels, Object.defineProperty(y[v], "channels", { value: C }), Object.defineProperty(y[v], "labels", { value: $ });
      }
      y.rgb.hsl = function(S) {
        var w, _, b = S[0] / 255, k = S[1] / 255, E = S[2] / 255, j = Math.min(b, k, E), L = Math.max(b, k, E), B = L - j;
        return L === j ? w = 0 : b === L ? w = (k - E) / B : k === L ? w = 2 + (E - b) / B : E === L && (w = 4 + (b - k) / B), (w = Math.min(60 * w, 360)) < 0 && (w += 360), _ = (j + L) / 2, [w, 100 * (L === j ? 0 : _ <= 0.5 ? B / (L + j) : B / (2 - L - j)), 100 * _];
      }, y.rgb.hsv = function(S) {
        var w, _, b, k, E, j = S[0] / 255, L = S[1] / 255, B = S[2] / 255, O = Math.max(j, L, B), V = O - Math.min(j, L, B), W = function(q) {
          return (O - q) / 6 / V + 0.5;
        };
        return V === 0 ? k = E = 0 : (E = V / O, w = W(j), _ = W(L), b = W(B), j === O ? k = b - _ : L === O ? k = 0.3333333333333333 + w - b : B === O && (k = 0.6666666666666666 + _ - w), k < 0 ? k += 1 : k > 1 && (k -= 1)), [360 * k, 100 * E, 100 * O];
      }, y.rgb.hwb = function(S) {
        var w = S[0], _ = S[1], b = S[2];
        return [y.rgb.hsl(S)[0], 100 * (0.00392156862745098 * Math.min(w, Math.min(_, b))), 100 * (b = 1 - 0.00392156862745098 * Math.max(w, Math.max(_, b)))];
      }, y.rgb.cmyk = function(S) {
        var w, _ = S[0] / 255, b = S[1] / 255, k = S[2] / 255;
        return [100 * ((1 - _ - (w = Math.min(1 - _, 1 - b, 1 - k))) / (1 - w) || 0), 100 * ((1 - b - w) / (1 - w) || 0), 100 * ((1 - k - w) / (1 - w) || 0), 100 * w];
      }, y.rgb.keyword = function(S) {
        var w = g[S];
        if (w) return w;
        var _, b, k, E = 1 / 0;
        for (var j in p) if (p.hasOwnProperty(j)) {
          var L = p[j], B = (b = S, k = L, Math.pow(b[0] - k[0], 2) + Math.pow(b[1] - k[1], 2) + Math.pow(b[2] - k[2], 2));
          B < E && (E = B, _ = j);
        }
        return _;
      }, y.keyword.rgb = function(S) {
        return p[S];
      }, y.rgb.xyz = function(S) {
        var w = S[0] / 255, _ = S[1] / 255, b = S[2] / 255;
        return [100 * (0.4124 * (w = w > 0.04045 ? Math.pow((w + 0.055) / 1.055, 2.4) : w / 12.92) + 0.3576 * (_ = _ > 0.04045 ? Math.pow((_ + 0.055) / 1.055, 2.4) : _ / 12.92) + 0.1805 * (b = b > 0.04045 ? Math.pow((b + 0.055) / 1.055, 2.4) : b / 12.92)), 100 * (0.2126 * w + 0.7152 * _ + 0.0722 * b), 100 * (0.0193 * w + 0.1192 * _ + 0.9505 * b)];
      }, y.rgb.lab = function(S) {
        var w = y.rgb.xyz(S), _ = w[0], b = w[1], k = w[2];
        return b /= 100, k /= 108.883, _ = (_ /= 95.047) > 8856e-6 ? Math.pow(_, 0.3333333333333333) : 7.787 * _ + 0.13793103448275862, [116 * (b = b > 8856e-6 ? Math.pow(b, 0.3333333333333333) : 7.787 * b + 0.13793103448275862) - 16, 500 * (_ - b), 200 * (b - (k = k > 8856e-6 ? Math.pow(k, 0.3333333333333333) : 7.787 * k + 0.13793103448275862))];
      }, y.hsl.rgb = function(S) {
        var w, _, b, k, E, j = S[0] / 360, L = S[1] / 100, B = S[2] / 100;
        if (L === 0) return [E = 255 * B, E, E];
        w = 2 * B - (_ = B < 0.5 ? B * (1 + L) : B + L - B * L), k = [0, 0, 0];
        for (var O = 0; O < 3; O++) (b = j + 0.3333333333333333 * -(O - 1)) < 0 && b++, b > 1 && b--, E = 6 * b < 1 ? w + 6 * (_ - w) * b : 2 * b < 1 ? _ : 3 * b < 2 ? w + (_ - w) * (0.6666666666666666 - b) * 6 : w, k[O] = 255 * E;
        return k;
      }, y.hsl.hsv = function(S) {
        var w = S[0], _ = S[1] / 100, b = S[2] / 100, k = _, E = Math.max(b, 0.01);
        return _ *= (b *= 2) <= 1 ? b : 2 - b, k *= E <= 1 ? E : 2 - E, [w, 100 * (b === 0 ? 2 * k / (E + k) : 2 * _ / (b + _)), 100 * ((b + _) / 2)];
      }, y.hsv.rgb = function(S) {
        var w = S[0] / 60, _ = S[1] / 100, b = S[2] / 100, k = Math.floor(w) % 6, E = w - Math.floor(w), j = 255 * b * (1 - _), L = 255 * b * (1 - _ * E), B = 255 * b * (1 - _ * (1 - E));
        switch (b *= 255, k) {
          case 0:
            return [b, B, j];
          case 1:
            return [L, b, j];
          case 2:
            return [j, b, B];
          case 3:
            return [j, L, b];
          case 4:
            return [B, j, b];
          case 5:
            return [b, j, L];
        }
      }, y.hsv.hsl = function(S) {
        var w, _, b, k = S[0], E = S[1] / 100, j = S[2] / 100, L = Math.max(j, 0.01);
        return b = (2 - E) * j, _ = E * L, [k, 100 * (_ = (_ /= (w = (2 - E) * L) <= 1 ? w : 2 - w) || 0), 100 * (b /= 2)];
      }, y.hwb.rgb = function(S) {
        var w, _, b, k, E, j, L, B = S[0] / 360, O = S[1] / 100, V = S[2] / 100, W = O + V;
        switch (W > 1 && (O /= W, V /= W), b = 6 * B - (w = Math.floor(6 * B)), 1 & w && (b = 1 - b), k = O + b * ((_ = 1 - V) - O), w) {
          default:
          case 6:
          case 0:
            E = _, j = k, L = O;
            break;
          case 1:
            E = k, j = _, L = O;
            break;
          case 2:
            E = O, j = _, L = k;
            break;
          case 3:
            E = O, j = k, L = _;
            break;
          case 4:
            E = k, j = O, L = _;
            break;
          case 5:
            E = _, j = O, L = k;
        }
        return [255 * E, 255 * j, 255 * L];
      }, y.cmyk.rgb = function(S) {
        var w = S[0] / 100, _ = S[1] / 100, b = S[2] / 100, k = S[3] / 100;
        return [255 * (1 - Math.min(1, w * (1 - k) + k)), 255 * (1 - Math.min(1, _ * (1 - k) + k)), 255 * (1 - Math.min(1, b * (1 - k) + k))];
      }, y.xyz.rgb = function(S) {
        var w, _, b, k = S[0] / 100, E = S[1] / 100, j = S[2] / 100;
        return _ = -0.9689 * k + 1.8758 * E + 0.0415 * j, b = 0.0557 * k + -0.204 * E + 1.057 * j, w = (w = 3.2406 * k + -1.5372 * E + -0.4986 * j) > 31308e-7 ? 1.055 * Math.pow(w, 0.4166666666666667) - 0.055 : 12.92 * w, _ = _ > 31308e-7 ? 1.055 * Math.pow(_, 0.4166666666666667) - 0.055 : 12.92 * _, b = b > 31308e-7 ? 1.055 * Math.pow(b, 0.4166666666666667) - 0.055 : 12.92 * b, [255 * (w = Math.min(Math.max(0, w), 1)), 255 * (_ = Math.min(Math.max(0, _), 1)), 255 * (b = Math.min(Math.max(0, b), 1))];
      }, y.xyz.lab = function(S) {
        var w = S[0], _ = S[1], b = S[2];
        return _ /= 100, b /= 108.883, w = (w /= 95.047) > 8856e-6 ? Math.pow(w, 0.3333333333333333) : 7.787 * w + 0.13793103448275862, [116 * (_ = _ > 8856e-6 ? Math.pow(_, 0.3333333333333333) : 7.787 * _ + 0.13793103448275862) - 16, 500 * (w - _), 200 * (_ - (b = b > 8856e-6 ? Math.pow(b, 0.3333333333333333) : 7.787 * b + 0.13793103448275862))];
      }, y.lab.xyz = function(S) {
        var w, _, b, k = S[0];
        w = S[1] / 500 + (_ = (k + 16) / 116), b = _ - S[2] / 200;
        var E = Math.pow(_, 3), j = Math.pow(w, 3), L = Math.pow(b, 3);
        return _ = E > 8856e-6 ? E : (_ - 0.13793103448275862) / 7.787, w = j > 8856e-6 ? j : (w - 0.13793103448275862) / 7.787, b = L > 8856e-6 ? L : (b - 0.13793103448275862) / 7.787, [w *= 95.047, _ *= 100, b *= 108.883];
      }, y.lab.lch = function(S) {
        var w, _ = S[0], b = S[1], k = S[2];
        return (w = 360 * Math.atan2(k, b) / 2 / Math.PI) < 0 && (w += 360), [_, Math.sqrt(b * b + k * k), w];
      }, y.lch.lab = function(S) {
        var w, _ = S[0], b = S[1];
        return w = S[2] / 360 * 2 * Math.PI, [_, b * Math.cos(w), b * Math.sin(w)];
      }, y.rgb.ansi16 = function(S) {
        var w = S[0], _ = S[1], b = S[2], k = 1 in arguments ? arguments[1] : y.rgb.hsv(S)[2];
        if ((k = Math.round(k / 50)) === 0) return 30;
        var E = 30 + (Math.round(b / 255) << 2 | Math.round(_ / 255) << 1 | Math.round(w / 255));
        return k === 2 && (E += 60), E;
      }, y.hsv.ansi16 = function(S) {
        return y.rgb.ansi16(y.hsv.rgb(S), S[2]);
      }, y.rgb.ansi256 = function(S) {
        var w = S[0], _ = S[1], b = S[2];
        return w === _ && _ === b ? w < 8 ? 16 : w > 248 ? 231 : Math.round((w - 8) / 247 * 24) + 232 : 16 + 36 * Math.round(w / 255 * 5) + 6 * Math.round(_ / 255 * 5) + Math.round(b / 255 * 5);
      }, y.ansi16.rgb = function(S) {
        var w = S % 10;
        if (w === 0 || w === 7) return S > 50 && (w += 3.5), [w = w / 10.5 * 255, w, w];
        var _ = 0.5 * (1 + ~~(S > 50));
        return [(1 & w) * _ * 255, (w >> 1 & 1) * _ * 255, (w >> 2 & 1) * _ * 255];
      }, y.ansi256.rgb = function(S) {
        if (S >= 232) {
          var w = 10 * (S - 232) + 8;
          return [w, w, w];
        }
        var _;
        return S -= 16, [Math.floor(S / 36) / 5 * 255, Math.floor((_ = S % 36) / 6) / 5 * 255, _ % 6 / 5 * 255];
      }, y.rgb.hex = function(S) {
        var w = (((255 & Math.round(S[0])) << 16) + ((255 & Math.round(S[1])) << 8) + (255 & Math.round(S[2]))).toString(16).toUpperCase();
        return "000000".substring(w.length) + w;
      }, y.hex.rgb = function(S) {
        var w = S.toString(16).match(/[a-f0-9]{6}|[a-f0-9]{3}/i);
        if (!w) return [0, 0, 0];
        var _ = w[0];
        w[0].length === 3 && (_ = _.split("").map(function(k) {
          return k + k;
        }).join(""));
        var b = parseInt(_, 16);
        return [b >> 16 & 255, b >> 8 & 255, 255 & b];
      }, y.rgb.hcg = function(S) {
        var w, _ = S[0] / 255, b = S[1] / 255, k = S[2] / 255, E = Math.max(Math.max(_, b), k), j = Math.min(Math.min(_, b), k), L = E - j;
        return w = L <= 0 ? 0 : E === _ ? (b - k) / L % 6 : E === b ? 2 + (k - _) / L : 4 + (_ - b) / L + 4, w /= 6, [360 * (w %= 1), 100 * L, 100 * (L < 1 ? j / (1 - L) : 0)];
      }, y.hsl.hcg = function(S) {
        var w = S[1] / 100, _ = S[2] / 100, b = 1, k = 0;
        return (b = _ < 0.5 ? 2 * w * _ : 2 * w * (1 - _)) < 1 && (k = (_ - 0.5 * b) / (1 - b)), [S[0], 100 * b, 100 * k];
      }, y.hsv.hcg = function(S) {
        var w = S[1] / 100, _ = S[2] / 100, b = w * _, k = 0;
        return b < 1 && (k = (_ - b) / (1 - b)), [S[0], 100 * b, 100 * k];
      }, y.hcg.rgb = function(S) {
        var w = S[0] / 360, _ = S[1] / 100, b = S[2] / 100;
        if (_ === 0) return [255 * b, 255 * b, 255 * b];
        var k, E = [0, 0, 0], j = w % 1 * 6, L = j % 1, B = 1 - L;
        switch (Math.floor(j)) {
          case 0:
            E[0] = 1, E[1] = L, E[2] = 0;
            break;
          case 1:
            E[0] = B, E[1] = 1, E[2] = 0;
            break;
          case 2:
            E[0] = 0, E[1] = 1, E[2] = L;
            break;
          case 3:
            E[0] = 0, E[1] = B, E[2] = 1;
            break;
          case 4:
            E[0] = L, E[1] = 0, E[2] = 1;
            break;
          default:
            E[0] = 1, E[1] = 0, E[2] = B;
        }
        return k = (1 - _) * b, [255 * (_ * E[0] + k), 255 * (_ * E[1] + k), 255 * (_ * E[2] + k)];
      }, y.hcg.hsv = function(S) {
        var w = S[1] / 100, _ = w + S[2] / 100 * (1 - w), b = 0;
        return _ > 0 && (b = w / _), [S[0], 100 * b, 100 * _];
      }, y.hcg.hsl = function(S) {
        var w = S[1] / 100, _ = S[2] / 100 * (1 - w) + 0.5 * w, b = 0;
        return _ > 0 && _ < 0.5 ? b = w / (2 * _) : _ >= 0.5 && _ < 1 && (b = w / (2 * (1 - _))), [S[0], 100 * b, 100 * _];
      }, y.hcg.hwb = function(S) {
        var w = S[1] / 100, _ = w + S[2] / 100 * (1 - w);
        return [S[0], 100 * (_ - w), 100 * (1 - _)];
      }, y.hwb.hcg = function(S) {
        var w = S[1] / 100, _ = 1 - S[2] / 100, b = _ - w, k = 0;
        return b < 1 && (k = (_ - b) / (1 - b)), [S[0], 100 * b, 100 * k];
      }, y.apple.rgb = function(S) {
        return [S[0] / 65535 * 255, S[1] / 65535 * 255, S[2] / 65535 * 255];
      }, y.rgb.apple = function(S) {
        return [S[0] / 255 * 65535, S[1] / 255 * 65535, S[2] / 255 * 65535];
      }, y.gray.rgb = function(S) {
        return [S[0] / 100 * 255, S[0] / 100 * 255, S[0] / 100 * 255];
      }, y.gray.hsl = y.gray.hsv = function(S) {
        return [0, 0, S[0]];
      }, y.gray.hwb = function(S) {
        return [0, 100, S[0]];
      }, y.gray.cmyk = function(S) {
        return [0, 0, 0, S[0]];
      }, y.gray.lab = function(S) {
        return [S[0], 0, 0];
      }, y.gray.hex = function(S) {
        var w = 255 & Math.round(S[0] / 100 * 255), _ = ((w << 16) + (w << 8) + w).toString(16).toUpperCase();
        return "000000".substring(_.length) + _;
      }, y.rgb.gray = function(S) {
        return [(S[0] + S[1] + S[2]) / 3 / 255 * 100];
      };
    }, 4732: (u, c, f) => {
      var p = f(6481), g = f(1157), h = {};
      Object.keys(p).forEach(function(y) {
        h[y] = {}, Object.defineProperty(h[y], "channels", { value: p[y].channels }), Object.defineProperty(h[y], "labels", { value: p[y].labels });
        var v = g(y);
        Object.keys(v).forEach(function(C) {
          var $ = v[C];
          h[y][C] = function(S) {
            var w = function(_) {
              if (_ == null) return _;
              arguments.length > 1 && (_ = Array.prototype.slice.call(arguments));
              var b = S(_);
              if (typeof b == "object") for (var k = b.length, E = 0; E < k; E++) b[E] = Math.round(b[E]);
              return b;
            };
            return "conversion" in S && (w.conversion = S.conversion), w;
          }($), h[y][C].raw = function(S) {
            var w = function(_) {
              return _ == null ? _ : (arguments.length > 1 && (_ = Array.prototype.slice.call(arguments)), S(_));
            };
            return "conversion" in S && (w.conversion = S.conversion), w;
          }($);
        });
      }), u.exports = h;
    }, 1157: (u, c, f) => {
      var p = f(6481);
      function g(v) {
        var C = function() {
          for (var j = {}, L = Object.keys(p), B = L.length, O = 0; O < B; O++) j[L[O]] = { distance: -1, parent: null };
          return j;
        }(), $ = [v];
        for (C[v].distance = 0; $.length; ) for (var S = $.pop(), w = Object.keys(p[S]), _ = w.length, b = 0; b < _; b++) {
          var k = w[b], E = C[k];
          E.distance === -1 && (E.distance = C[S].distance + 1, E.parent = S, $.unshift(k));
        }
        return C;
      }
      function h(v, C) {
        return function($) {
          return C(v($));
        };
      }
      function y(v, C) {
        for (var $ = [C[v].parent, v], S = p[C[v].parent][v], w = C[v].parent; C[w].parent; ) $.unshift(C[w].parent), S = h(p[C[w].parent][w], S), w = C[w].parent;
        return S.conversion = $, S;
      }
      u.exports = function(v) {
        for (var C = g(v), $ = {}, S = Object.keys(C), w = S.length, _ = 0; _ < w; _++) {
          var b = S[_];
          C[b].parent !== null && ($[b] = y(b, C));
        }
        return $;
      };
    }, 4176: (u) => {
      u.exports = { aliceblue: [240, 248, 255], antiquewhite: [250, 235, 215], aqua: [0, 255, 255], aquamarine: [127, 255, 212], azure: [240, 255, 255], beige: [245, 245, 220], bisque: [255, 228, 196], black: [0, 0, 0], blanchedalmond: [255, 235, 205], blue: [0, 0, 255], blueviolet: [138, 43, 226], brown: [165, 42, 42], burlywood: [222, 184, 135], cadetblue: [95, 158, 160], chartreuse: [127, 255, 0], chocolate: [210, 105, 30], coral: [255, 127, 80], cornflowerblue: [100, 149, 237], cornsilk: [255, 248, 220], crimson: [220, 20, 60], cyan: [0, 255, 255], darkblue: [0, 0, 139], darkcyan: [0, 139, 139], darkgoldenrod: [184, 134, 11], darkgray: [169, 169, 169], darkgreen: [0, 100, 0], darkgrey: [169, 169, 169], darkkhaki: [189, 183, 107], darkmagenta: [139, 0, 139], darkolivegreen: [85, 107, 47], darkorange: [255, 140, 0], darkorchid: [153, 50, 204], darkred: [139, 0, 0], darksalmon: [233, 150, 122], darkseagreen: [143, 188, 143], darkslateblue: [72, 61, 139], darkslategray: [47, 79, 79], darkslategrey: [47, 79, 79], darkturquoise: [0, 206, 209], darkviolet: [148, 0, 211], deeppink: [255, 20, 147], deepskyblue: [0, 191, 255], dimgray: [105, 105, 105], dimgrey: [105, 105, 105], dodgerblue: [30, 144, 255], firebrick: [178, 34, 34], floralwhite: [255, 250, 240], forestgreen: [34, 139, 34], fuchsia: [255, 0, 255], gainsboro: [220, 220, 220], ghostwhite: [248, 248, 255], gold: [255, 215, 0], goldenrod: [218, 165, 32], gray: [128, 128, 128], green: [0, 128, 0], greenyellow: [173, 255, 47], grey: [128, 128, 128], honeydew: [240, 255, 240], hotpink: [255, 105, 180], indianred: [205, 92, 92], indigo: [75, 0, 130], ivory: [255, 255, 240], khaki: [240, 230, 140], lavender: [230, 230, 250], lavenderblush: [255, 240, 245], lawngreen: [124, 252, 0], lemonchiffon: [255, 250, 205], lightblue: [173, 216, 230], lightcoral: [240, 128, 128], lightcyan: [224, 255, 255], lightgoldenrodyellow: [250, 250, 210], lightgray: [211, 211, 211], lightgreen: [144, 238, 144], lightgrey: [211, 211, 211], lightpink: [255, 182, 193], lightsalmon: [255, 160, 122], lightseagreen: [32, 178, 170], lightskyblue: [135, 206, 250], lightslategray: [119, 136, 153], lightslategrey: [119, 136, 153], lightsteelblue: [176, 196, 222], lightyellow: [255, 255, 224], lime: [0, 255, 0], limegreen: [50, 205, 50], linen: [250, 240, 230], magenta: [255, 0, 255], maroon: [128, 0, 0], mediumaquamarine: [102, 205, 170], mediumblue: [0, 0, 205], mediumorchid: [186, 85, 211], mediumpurple: [147, 112, 219], mediumseagreen: [60, 179, 113], mediumslateblue: [123, 104, 238], mediumspringgreen: [0, 250, 154], mediumturquoise: [72, 209, 204], mediumvioletred: [199, 21, 133], midnightblue: [25, 25, 112], mintcream: [245, 255, 250], mistyrose: [255, 228, 225], moccasin: [255, 228, 181], navajowhite: [255, 222, 173], navy: [0, 0, 128], oldlace: [253, 245, 230], olive: [128, 128, 0], olivedrab: [107, 142, 35], orange: [255, 165, 0], orangered: [255, 69, 0], orchid: [218, 112, 214], palegoldenrod: [238, 232, 170], palegreen: [152, 251, 152], paleturquoise: [175, 238, 238], palevioletred: [219, 112, 147], papayawhip: [255, 239, 213], peachpuff: [255, 218, 185], peru: [205, 133, 63], pink: [255, 192, 203], plum: [221, 160, 221], powderblue: [176, 224, 230], purple: [128, 0, 128], rebeccapurple: [102, 51, 153], red: [255, 0, 0], rosybrown: [188, 143, 143], royalblue: [65, 105, 225], saddlebrown: [139, 69, 19], salmon: [250, 128, 114], sandybrown: [244, 164, 96], seagreen: [46, 139, 87], seashell: [255, 245, 238], sienna: [160, 82, 45], silver: [192, 192, 192], skyblue: [135, 206, 235], slateblue: [106, 90, 205], slategray: [112, 128, 144], slategrey: [112, 128, 144], snow: [255, 250, 250], springgreen: [0, 255, 127], steelblue: [70, 130, 180], tan: [210, 180, 140], teal: [0, 128, 128], thistle: [216, 191, 216], tomato: [255, 99, 71], turquoise: [64, 224, 208], violet: [238, 130, 238], wheat: [245, 222, 179], white: [255, 255, 255], whitesmoke: [245, 245, 245], yellow: [255, 255, 0], yellowgreen: [154, 205, 50] };
    }, 4877: (u) => {
      u.exports = { aliceblue: [240, 248, 255], antiquewhite: [250, 235, 215], aqua: [0, 255, 255], aquamarine: [127, 255, 212], azure: [240, 255, 255], beige: [245, 245, 220], bisque: [255, 228, 196], black: [0, 0, 0], blanchedalmond: [255, 235, 205], blue: [0, 0, 255], blueviolet: [138, 43, 226], brown: [165, 42, 42], burlywood: [222, 184, 135], cadetblue: [95, 158, 160], chartreuse: [127, 255, 0], chocolate: [210, 105, 30], coral: [255, 127, 80], cornflowerblue: [100, 149, 237], cornsilk: [255, 248, 220], crimson: [220, 20, 60], cyan: [0, 255, 255], darkblue: [0, 0, 139], darkcyan: [0, 139, 139], darkgoldenrod: [184, 134, 11], darkgray: [169, 169, 169], darkgreen: [0, 100, 0], darkgrey: [169, 169, 169], darkkhaki: [189, 183, 107], darkmagenta: [139, 0, 139], darkolivegreen: [85, 107, 47], darkorange: [255, 140, 0], darkorchid: [153, 50, 204], darkred: [139, 0, 0], darksalmon: [233, 150, 122], darkseagreen: [143, 188, 143], darkslateblue: [72, 61, 139], darkslategray: [47, 79, 79], darkslategrey: [47, 79, 79], darkturquoise: [0, 206, 209], darkviolet: [148, 0, 211], deeppink: [255, 20, 147], deepskyblue: [0, 191, 255], dimgray: [105, 105, 105], dimgrey: [105, 105, 105], dodgerblue: [30, 144, 255], firebrick: [178, 34, 34], floralwhite: [255, 250, 240], forestgreen: [34, 139, 34], fuchsia: [255, 0, 255], gainsboro: [220, 220, 220], ghostwhite: [248, 248, 255], gold: [255, 215, 0], goldenrod: [218, 165, 32], gray: [128, 128, 128], green: [0, 128, 0], greenyellow: [173, 255, 47], grey: [128, 128, 128], honeydew: [240, 255, 240], hotpink: [255, 105, 180], indianred: [205, 92, 92], indigo: [75, 0, 130], ivory: [255, 255, 240], khaki: [240, 230, 140], lavender: [230, 230, 250], lavenderblush: [255, 240, 245], lawngreen: [124, 252, 0], lemonchiffon: [255, 250, 205], lightblue: [173, 216, 230], lightcoral: [240, 128, 128], lightcyan: [224, 255, 255], lightgoldenrodyellow: [250, 250, 210], lightgray: [211, 211, 211], lightgreen: [144, 238, 144], lightgrey: [211, 211, 211], lightpink: [255, 182, 193], lightsalmon: [255, 160, 122], lightseagreen: [32, 178, 170], lightskyblue: [135, 206, 250], lightslategray: [119, 136, 153], lightslategrey: [119, 136, 153], lightsteelblue: [176, 196, 222], lightyellow: [255, 255, 224], lime: [0, 255, 0], limegreen: [50, 205, 50], linen: [250, 240, 230], magenta: [255, 0, 255], maroon: [128, 0, 0], mediumaquamarine: [102, 205, 170], mediumblue: [0, 0, 205], mediumorchid: [186, 85, 211], mediumpurple: [147, 112, 219], mediumseagreen: [60, 179, 113], mediumslateblue: [123, 104, 238], mediumspringgreen: [0, 250, 154], mediumturquoise: [72, 209, 204], mediumvioletred: [199, 21, 133], midnightblue: [25, 25, 112], mintcream: [245, 255, 250], mistyrose: [255, 228, 225], moccasin: [255, 228, 181], navajowhite: [255, 222, 173], navy: [0, 0, 128], oldlace: [253, 245, 230], olive: [128, 128, 0], olivedrab: [107, 142, 35], orange: [255, 165, 0], orangered: [255, 69, 0], orchid: [218, 112, 214], palegoldenrod: [238, 232, 170], palegreen: [152, 251, 152], paleturquoise: [175, 238, 238], palevioletred: [219, 112, 147], papayawhip: [255, 239, 213], peachpuff: [255, 218, 185], peru: [205, 133, 63], pink: [255, 192, 203], plum: [221, 160, 221], powderblue: [176, 224, 230], purple: [128, 0, 128], rebeccapurple: [102, 51, 153], red: [255, 0, 0], rosybrown: [188, 143, 143], royalblue: [65, 105, 225], saddlebrown: [139, 69, 19], salmon: [250, 128, 114], sandybrown: [244, 164, 96], seagreen: [46, 139, 87], seashell: [255, 245, 238], sienna: [160, 82, 45], silver: [192, 192, 192], skyblue: [135, 206, 235], slateblue: [106, 90, 205], slategray: [112, 128, 144], slategrey: [112, 128, 144], snow: [255, 250, 250], springgreen: [0, 255, 127], steelblue: [70, 130, 180], tan: [210, 180, 140], teal: [0, 128, 128], thistle: [216, 191, 216], tomato: [255, 99, 71], turquoise: [64, 224, 208], violet: [238, 130, 238], wheat: [245, 222, 179], white: [255, 255, 255], whitesmoke: [245, 245, 245], yellow: [255, 255, 0], yellowgreen: [154, 205, 50] };
    }, 6138: (u, c, f) => {
      var p = f(4877), g = f(301), h = Object.hasOwnProperty, y = /* @__PURE__ */ Object.create(null);
      for (var v in p) h.call(p, v) && (y[p[v]] = v);
      var C = u.exports = { to: {}, get: {} };
      function $(w, _, b) {
        return Math.min(Math.max(_, w), b);
      }
      function S(w) {
        var _ = Math.round(w).toString(16).toUpperCase();
        return _.length < 2 ? "0" + _ : _;
      }
      C.get = function(w) {
        var _, b;
        switch (w.substring(0, 3).toLowerCase()) {
          case "hsl":
            _ = C.get.hsl(w), b = "hsl";
            break;
          case "hwb":
            _ = C.get.hwb(w), b = "hwb";
            break;
          default:
            _ = C.get.rgb(w), b = "rgb";
        }
        return _ ? { model: b, value: _ } : null;
      }, C.get.rgb = function(w) {
        if (!w) return null;
        var _, b, k, E = [0, 0, 0, 1];
        if (_ = w.match(/^#([a-f0-9]{6})([a-f0-9]{2})?$/i)) {
          for (k = _[2], _ = _[1], b = 0; b < 3; b++) {
            var j = 2 * b;
            E[b] = parseInt(_.slice(j, j + 2), 16);
          }
          k && (E[3] = parseInt(k, 16) / 255);
        } else if (_ = w.match(/^#([a-f0-9]{3,4})$/i)) {
          for (k = (_ = _[1])[3], b = 0; b < 3; b++) E[b] = parseInt(_[b] + _[b], 16);
          k && (E[3] = parseInt(k + k, 16) / 255);
        } else if (_ = w.match(/^rgba?\(\s*([+-]?\d+)(?=[\s,])\s*(?:,\s*)?([+-]?\d+)(?=[\s,])\s*(?:,\s*)?([+-]?\d+)\s*(?:[,|\/]\s*([+-]?[\d\.]+)(%?)\s*)?\)$/)) {
          for (b = 0; b < 3; b++) E[b] = parseInt(_[b + 1], 0);
          _[4] && (_[5] ? E[3] = 0.01 * parseFloat(_[4]) : E[3] = parseFloat(_[4]));
        } else {
          if (!(_ = w.match(/^rgba?\(\s*([+-]?[\d\.]+)\%\s*,?\s*([+-]?[\d\.]+)\%\s*,?\s*([+-]?[\d\.]+)\%\s*(?:[,|\/]\s*([+-]?[\d\.]+)(%?)\s*)?\)$/))) return (_ = w.match(/^(\w+)$/)) ? _[1] === "transparent" ? [0, 0, 0, 0] : h.call(p, _[1]) ? ((E = p[_[1]])[3] = 1, E) : null : null;
          for (b = 0; b < 3; b++) E[b] = Math.round(2.55 * parseFloat(_[b + 1]));
          _[4] && (_[5] ? E[3] = 0.01 * parseFloat(_[4]) : E[3] = parseFloat(_[4]));
        }
        for (b = 0; b < 3; b++) E[b] = $(E[b], 0, 255);
        return E[3] = $(E[3], 0, 1), E;
      }, C.get.hsl = function(w) {
        if (!w) return null;
        var _ = w.match(/^hsla?\(\s*([+-]?(?:\d{0,3}\.)?\d+)(?:deg)?\s*,?\s*([+-]?[\d\.]+)%\s*,?\s*([+-]?[\d\.]+)%\s*(?:[,|\/]\s*([+-]?(?=\.\d|\d)(?:0|[1-9]\d*)?(?:\.\d*)?(?:[eE][+-]?\d+)?)\s*)?\)$/);
        if (_) {
          var b = parseFloat(_[4]);
          return [(parseFloat(_[1]) % 360 + 360) % 360, $(parseFloat(_[2]), 0, 100), $(parseFloat(_[3]), 0, 100), $(isNaN(b) ? 1 : b, 0, 1)];
        }
        return null;
      }, C.get.hwb = function(w) {
        if (!w) return null;
        var _ = w.match(/^hwb\(\s*([+-]?\d{0,3}(?:\.\d+)?)(?:deg)?\s*,\s*([+-]?[\d\.]+)%\s*,\s*([+-]?[\d\.]+)%\s*(?:,\s*([+-]?(?=\.\d|\d)(?:0|[1-9]\d*)?(?:\.\d*)?(?:[eE][+-]?\d+)?)\s*)?\)$/);
        if (_) {
          var b = parseFloat(_[4]);
          return [(parseFloat(_[1]) % 360 + 360) % 360, $(parseFloat(_[2]), 0, 100), $(parseFloat(_[3]), 0, 100), $(isNaN(b) ? 1 : b, 0, 1)];
        }
        return null;
      }, C.to.hex = function() {
        var w = g(arguments);
        return "#" + S(w[0]) + S(w[1]) + S(w[2]) + (w[3] < 1 ? S(Math.round(255 * w[3])) : "");
      }, C.to.rgb = function() {
        var w = g(arguments);
        return w.length < 4 || w[3] === 1 ? "rgb(" + Math.round(w[0]) + ", " + Math.round(w[1]) + ", " + Math.round(w[2]) + ")" : "rgba(" + Math.round(w[0]) + ", " + Math.round(w[1]) + ", " + Math.round(w[2]) + ", " + w[3] + ")";
      }, C.to.rgb.percent = function() {
        var w = g(arguments), _ = Math.round(w[0] / 255 * 100), b = Math.round(w[1] / 255 * 100), k = Math.round(w[2] / 255 * 100);
        return w.length < 4 || w[3] === 1 ? "rgb(" + _ + "%, " + b + "%, " + k + "%)" : "rgba(" + _ + "%, " + b + "%, " + k + "%, " + w[3] + ")";
      }, C.to.hsl = function() {
        var w = g(arguments);
        return w.length < 4 || w[3] === 1 ? "hsl(" + w[0] + ", " + w[1] + "%, " + w[2] + "%)" : "hsla(" + w[0] + ", " + w[1] + "%, " + w[2] + "%, " + w[3] + ")";
      }, C.to.hwb = function() {
        var w = g(arguments), _ = "";
        return w.length >= 4 && w[3] !== 1 && (_ = ", " + w[3]), "hwb(" + w[0] + ", " + w[1] + "%, " + w[2] + "%" + _ + ")";
      }, C.to.keyword = function(w) {
        return y[w.slice(0, 3)];
      };
    }, 3639: (u, c, f) => {
      var p = f(6138), g = f(4732), h = [].slice, y = ["keyword", "gray", "hex"], v = {};
      Object.keys(g).forEach(function(b) {
        v[h.call(g[b].labels).sort().join("")] = b;
      });
      var C = {};
      function $(b, k) {
        if (!(this instanceof $)) return new $(b, k);
        if (k && k in y && (k = null), k && !(k in g)) throw new Error("Unknown model: " + k);
        var E, j;
        if (b == null) this.model = "rgb", this.color = [0, 0, 0], this.valpha = 1;
        else if (b instanceof $) this.model = b.model, this.color = b.color.slice(), this.valpha = b.valpha;
        else if (typeof b == "string") {
          var L = p.get(b);
          if (L === null) throw new Error("Unable to parse color from string: " + b);
          this.model = L.model, j = g[this.model].channels, this.color = L.value.slice(0, j), this.valpha = typeof L.value[j] == "number" ? L.value[j] : 1;
        } else if (b.length) {
          this.model = k || "rgb", j = g[this.model].channels;
          var B = h.call(b, 0, j);
          this.color = _(B, j), this.valpha = typeof b[j] == "number" ? b[j] : 1;
        } else if (typeof b == "number") b &= 16777215, this.model = "rgb", this.color = [b >> 16 & 255, b >> 8 & 255, 255 & b], this.valpha = 1;
        else {
          this.valpha = 1;
          var O = Object.keys(b);
          "alpha" in b && (O.splice(O.indexOf("alpha"), 1), this.valpha = typeof b.alpha == "number" ? b.alpha : 0);
          var V = O.sort().join("");
          if (!(V in v)) throw new Error("Unable to parse color from object: " + JSON.stringify(b));
          this.model = v[V];
          var W = g[this.model].labels, q = [];
          for (E = 0; E < W.length; E++) q.push(b[W[E]]);
          this.color = _(q);
        }
        if (C[this.model]) for (j = g[this.model].channels, E = 0; E < j; E++) {
          var Q = C[this.model][E];
          Q && (this.color[E] = Q(this.color[E]));
        }
        this.valpha = Math.max(0, Math.min(1, this.valpha)), Object.freeze && Object.freeze(this);
      }
      function S(b, k, E) {
        return (b = Array.isArray(b) ? b : [b]).forEach(function(j) {
          (C[j] || (C[j] = []))[k] = E;
        }), b = b[0], function(j) {
          var L;
          return arguments.length ? (E && (j = E(j)), (L = this[b]()).color[k] = j, L) : (L = this[b]().color[k], E && (L = E(L)), L);
        };
      }
      function w(b) {
        return function(k) {
          return Math.max(0, Math.min(b, k));
        };
      }
      function _(b, k) {
        for (var E = 0; E < k; E++) typeof b[E] != "number" && (b[E] = 0);
        return b;
      }
      $.prototype = { toString: function() {
        return this.string();
      }, toJSON: function() {
        return this[this.model]();
      }, string: function(b) {
        var k = this.model in p.to ? this : this.rgb(), E = (k = k.round(typeof b == "number" ? b : 1)).valpha === 1 ? k.color : k.color.concat(this.valpha);
        return p.to[k.model](E);
      }, percentString: function(b) {
        var k = this.rgb().round(typeof b == "number" ? b : 1), E = k.valpha === 1 ? k.color : k.color.concat(this.valpha);
        return p.to.rgb.percent(E);
      }, array: function() {
        return this.valpha === 1 ? this.color.slice() : this.color.concat(this.valpha);
      }, object: function() {
        for (var b = {}, k = g[this.model].channels, E = g[this.model].labels, j = 0; j < k; j++) b[E[j]] = this.color[j];
        return this.valpha !== 1 && (b.alpha = this.valpha), b;
      }, unitArray: function() {
        var b = this.rgb().color;
        return b[0] /= 255, b[1] /= 255, b[2] /= 255, this.valpha !== 1 && b.push(this.valpha), b;
      }, unitObject: function() {
        var b = this.rgb().object();
        return b.r /= 255, b.g /= 255, b.b /= 255, this.valpha !== 1 && (b.alpha = this.valpha), b;
      }, round: function(b) {
        return b = Math.max(b || 0, 0), new $(this.color.map(/* @__PURE__ */ function(k) {
          return function(E) {
            return function(j, L) {
              return Number(j.toFixed(L));
            }(E, k);
          };
        }(b)).concat(this.valpha), this.model);
      }, alpha: function(b) {
        return arguments.length ? new $(this.color.concat(Math.max(0, Math.min(1, b))), this.model) : this.valpha;
      }, red: S("rgb", 0, w(255)), green: S("rgb", 1, w(255)), blue: S("rgb", 2, w(255)), hue: S(["hsl", "hsv", "hsl", "hwb", "hcg"], 0, function(b) {
        return (b % 360 + 360) % 360;
      }), saturationl: S("hsl", 1, w(100)), lightness: S("hsl", 2, w(100)), saturationv: S("hsv", 1, w(100)), value: S("hsv", 2, w(100)), chroma: S("hcg", 1, w(100)), gray: S("hcg", 2, w(100)), white: S("hwb", 1, w(100)), wblack: S("hwb", 2, w(100)), cyan: S("cmyk", 0, w(100)), magenta: S("cmyk", 1, w(100)), yellow: S("cmyk", 2, w(100)), black: S("cmyk", 3, w(100)), x: S("xyz", 0, w(100)), y: S("xyz", 1, w(100)), z: S("xyz", 2, w(100)), l: S("lab", 0, w(100)), a: S("lab", 1), b: S("lab", 2), keyword: function(b) {
        return arguments.length ? new $(b) : g[this.model].keyword(this.color);
      }, hex: function(b) {
        return arguments.length ? new $(b) : p.to.hex(this.rgb().round().color);
      }, rgbNumber: function() {
        var b = this.rgb().color;
        return (255 & b[0]) << 16 | (255 & b[1]) << 8 | 255 & b[2];
      }, luminosity: function() {
        for (var b = this.rgb().color, k = [], E = 0; E < b.length; E++) {
          var j = b[E] / 255;
          k[E] = j <= 0.03928 ? j / 12.92 : Math.pow((j + 0.055) / 1.055, 2.4);
        }
        return 0.2126 * k[0] + 0.7152 * k[1] + 0.0722 * k[2];
      }, contrast: function(b) {
        var k = this.luminosity(), E = b.luminosity();
        return k > E ? (k + 0.05) / (E + 0.05) : (E + 0.05) / (k + 0.05);
      }, level: function(b) {
        var k = this.contrast(b);
        return k >= 7.1 ? "AAA" : k >= 4.5 ? "AA" : "";
      }, isDark: function() {
        var b = this.rgb().color;
        return (299 * b[0] + 587 * b[1] + 114 * b[2]) / 1e3 < 128;
      }, isLight: function() {
        return !this.isDark();
      }, negate: function() {
        for (var b = this.rgb(), k = 0; k < 3; k++) b.color[k] = 255 - b.color[k];
        return b;
      }, lighten: function(b) {
        var k = this.hsl();
        return k.color[2] += k.color[2] * b, k;
      }, darken: function(b) {
        var k = this.hsl();
        return k.color[2] -= k.color[2] * b, k;
      }, saturate: function(b) {
        var k = this.hsl();
        return k.color[1] += k.color[1] * b, k;
      }, desaturate: function(b) {
        var k = this.hsl();
        return k.color[1] -= k.color[1] * b, k;
      }, whiten: function(b) {
        var k = this.hwb();
        return k.color[1] += k.color[1] * b, k;
      }, blacken: function(b) {
        var k = this.hwb();
        return k.color[2] += k.color[2] * b, k;
      }, grayscale: function() {
        var b = this.rgb().color, k = 0.3 * b[0] + 0.59 * b[1] + 0.11 * b[2];
        return $.rgb(k, k, k);
      }, fade: function(b) {
        return this.alpha(this.valpha - this.valpha * b);
      }, opaquer: function(b) {
        return this.alpha(this.valpha + this.valpha * b);
      }, rotate: function(b) {
        var k = this.hsl(), E = k.color[0];
        return E = (E = (E + b) % 360) < 0 ? 360 + E : E, k.color[0] = E, k;
      }, mix: function(b, k) {
        if (!b || !b.rgb) throw new Error('Argument to "mix" was not a Color instance, but rather an instance of ' + typeof b);
        var E = b.rgb(), j = this.rgb(), L = k === void 0 ? 0.5 : k, B = 2 * L - 1, O = E.alpha() - j.alpha(), V = ((B * O == -1 ? B : (B + O) / (1 + B * O)) + 1) / 2, W = 1 - V;
        return $.rgb(V * E.red() + W * j.red(), V * E.green() + W * j.green(), V * E.blue() + W * j.blue(), E.alpha() * L + j.alpha() * (1 - L));
      } }, Object.keys(g).forEach(function(b) {
        if (y.indexOf(b) === -1) {
          var k = g[b].channels;
          $.prototype[b] = function() {
            if (this.model === b) return new $(this);
            if (arguments.length) return new $(arguments, b);
            var E, j = typeof arguments[k] == "number" ? k : this.valpha;
            return new $((E = g[this.model][b].raw(this.color), Array.isArray(E) ? E : [E]).concat(j), b);
          }, $[b] = function(E) {
            return typeof E == "number" && (E = _(h.call(arguments), k)), new $(E, b);
          };
        }
      }), u.exports = $;
    }, 9784: (u) => {
      var c, f = typeof Reflect == "object" ? Reflect : null, p = f && typeof f.apply == "function" ? f.apply : function(j, L, B) {
        return Function.prototype.apply.call(j, L, B);
      };
      c = f && typeof f.ownKeys == "function" ? f.ownKeys : Object.getOwnPropertySymbols ? function(j) {
        return Object.getOwnPropertyNames(j).concat(Object.getOwnPropertySymbols(j));
      } : function(j) {
        return Object.getOwnPropertyNames(j);
      };
      var g = Number.isNaN || function(j) {
        return j != j;
      };
      function h() {
        h.init.call(this);
      }
      u.exports = h, u.exports.once = function(j, L) {
        return new Promise(function(B, O) {
          function V(q) {
            j.removeListener(L, W), O(q);
          }
          function W() {
            typeof j.removeListener == "function" && j.removeListener("error", V), B([].slice.call(arguments));
          }
          E(j, L, W, { once: !0 }), L !== "error" && function(q, Q, X) {
            typeof q.on == "function" && E(q, "error", Q, X);
          }(j, V, { once: !0 });
        });
      }, h.EventEmitter = h, h.prototype._events = void 0, h.prototype._eventsCount = 0, h.prototype._maxListeners = void 0;
      var y = 10;
      function v(j) {
        if (typeof j != "function") throw new TypeError('The "listener" argument must be of type Function. Received type ' + typeof j);
      }
      function C(j) {
        return j._maxListeners === void 0 ? h.defaultMaxListeners : j._maxListeners;
      }
      function $(j, L, B, O) {
        var V, W, q, Q;
        if (v(B), (W = j._events) === void 0 ? (W = j._events = /* @__PURE__ */ Object.create(null), j._eventsCount = 0) : (W.newListener !== void 0 && (j.emit("newListener", L, B.listener ? B.listener : B), W = j._events), q = W[L]), q === void 0) q = W[L] = B, ++j._eventsCount;
        else if (typeof q == "function" ? q = W[L] = O ? [B, q] : [q, B] : O ? q.unshift(B) : q.push(B), (V = C(j)) > 0 && q.length > V && !q.warned) {
          q.warned = !0;
          var X = new Error("Possible EventEmitter memory leak detected. " + q.length + " " + String(L) + " listeners added. Use emitter.setMaxListeners() to increase limit");
          X.name = "MaxListenersExceededWarning", X.emitter = j, X.type = L, X.count = q.length, Q = X, console && console.warn && console.warn(Q);
        }
        return j;
      }
      function S() {
        if (!this.fired) return this.target.removeListener(this.type, this.wrapFn), this.fired = !0, arguments.length === 0 ? this.listener.call(this.target) : this.listener.apply(this.target, arguments);
      }
      function w(j, L, B) {
        var O = { fired: !1, wrapFn: void 0, target: j, type: L, listener: B }, V = S.bind(O);
        return V.listener = B, O.wrapFn = V, V;
      }
      function _(j, L, B) {
        var O = j._events;
        if (O === void 0) return [];
        var V = O[L];
        return V === void 0 ? [] : typeof V == "function" ? B ? [V.listener || V] : [V] : B ? function(W) {
          for (var q = new Array(W.length), Q = 0; Q < q.length; ++Q) q[Q] = W[Q].listener || W[Q];
          return q;
        }(V) : k(V, V.length);
      }
      function b(j) {
        var L = this._events;
        if (L !== void 0) {
          var B = L[j];
          if (typeof B == "function") return 1;
          if (B !== void 0) return B.length;
        }
        return 0;
      }
      function k(j, L) {
        for (var B = new Array(L), O = 0; O < L; ++O) B[O] = j[O];
        return B;
      }
      function E(j, L, B, O) {
        if (typeof j.on == "function") O.once ? j.once(L, B) : j.on(L, B);
        else {
          if (typeof j.addEventListener != "function") throw new TypeError('The "emitter" argument must be of type EventEmitter. Received type ' + typeof j);
          j.addEventListener(L, function V(W) {
            O.once && j.removeEventListener(L, V), B(W);
          });
        }
      }
      Object.defineProperty(h, "defaultMaxListeners", { enumerable: !0, get: function() {
        return y;
      }, set: function(j) {
        if (typeof j != "number" || j < 0 || g(j)) throw new RangeError('The value of "defaultMaxListeners" is out of range. It must be a non-negative number. Received ' + j + ".");
        y = j;
      } }), h.init = function() {
        this._events !== void 0 && this._events !== Object.getPrototypeOf(this)._events || (this._events = /* @__PURE__ */ Object.create(null), this._eventsCount = 0), this._maxListeners = this._maxListeners || void 0;
      }, h.prototype.setMaxListeners = function(j) {
        if (typeof j != "number" || j < 0 || g(j)) throw new RangeError('The value of "n" is out of range. It must be a non-negative number. Received ' + j + ".");
        return this._maxListeners = j, this;
      }, h.prototype.getMaxListeners = function() {
        return C(this);
      }, h.prototype.emit = function(j) {
        for (var L = [], B = 1; B < arguments.length; B++) L.push(arguments[B]);
        var O = j === "error", V = this._events;
        if (V !== void 0) O = O && V.error === void 0;
        else if (!O) return !1;
        if (O) {
          var W;
          if (L.length > 0 && (W = L[0]), W instanceof Error) throw W;
          var q = new Error("Unhandled error." + (W ? " (" + W.message + ")" : ""));
          throw q.context = W, q;
        }
        var Q = V[j];
        if (Q === void 0) return !1;
        if (typeof Q == "function") p(Q, this, L);
        else {
          var X = Q.length, ce = k(Q, X);
          for (B = 0; B < X; ++B) p(ce[B], this, L);
        }
        return !0;
      }, h.prototype.addListener = function(j, L) {
        return $(this, j, L, !1);
      }, h.prototype.on = h.prototype.addListener, h.prototype.prependListener = function(j, L) {
        return $(this, j, L, !0);
      }, h.prototype.once = function(j, L) {
        return v(L), this.on(j, w(this, j, L)), this;
      }, h.prototype.prependOnceListener = function(j, L) {
        return v(L), this.prependListener(j, w(this, j, L)), this;
      }, h.prototype.removeListener = function(j, L) {
        var B, O, V, W, q;
        if (v(L), (O = this._events) === void 0) return this;
        if ((B = O[j]) === void 0) return this;
        if (B === L || B.listener === L) --this._eventsCount == 0 ? this._events = /* @__PURE__ */ Object.create(null) : (delete O[j], O.removeListener && this.emit("removeListener", j, B.listener || L));
        else if (typeof B != "function") {
          for (V = -1, W = B.length - 1; W >= 0; W--) if (B[W] === L || B[W].listener === L) {
            q = B[W].listener, V = W;
            break;
          }
          if (V < 0) return this;
          V === 0 ? B.shift() : function(Q, X) {
            for (; X + 1 < Q.length; X++) Q[X] = Q[X + 1];
            Q.pop();
          }(B, V), B.length === 1 && (O[j] = B[0]), O.removeListener !== void 0 && this.emit("removeListener", j, q || L);
        }
        return this;
      }, h.prototype.off = h.prototype.removeListener, h.prototype.removeAllListeners = function(j) {
        var L, B, O;
        if ((B = this._events) === void 0) return this;
        if (B.removeListener === void 0) return arguments.length === 0 ? (this._events = /* @__PURE__ */ Object.create(null), this._eventsCount = 0) : B[j] !== void 0 && (--this._eventsCount == 0 ? this._events = /* @__PURE__ */ Object.create(null) : delete B[j]), this;
        if (arguments.length === 0) {
          var V, W = Object.keys(B);
          for (O = 0; O < W.length; ++O) (V = W[O]) !== "removeListener" && this.removeAllListeners(V);
          return this.removeAllListeners("removeListener"), this._events = /* @__PURE__ */ Object.create(null), this._eventsCount = 0, this;
        }
        if (typeof (L = B[j]) == "function") this.removeListener(j, L);
        else if (L !== void 0) for (O = L.length - 1; O >= 0; O--) this.removeListener(j, L[O]);
        return this;
      }, h.prototype.listeners = function(j) {
        return _(this, j, !0);
      }, h.prototype.rawListeners = function(j) {
        return _(this, j, !1);
      }, h.listenerCount = function(j, L) {
        return typeof j.listenerCount == "function" ? j.listenerCount(L) : b.call(j, L);
      }, h.prototype.listenerCount = b, h.prototype.eventNames = function() {
        return this._eventsCount > 0 ? c(this._events) : [];
      };
    }, 8336: (u) => {
      u.exports = function(c) {
        return !(!c || typeof c == "string") && (c instanceof Array || Array.isArray(c) || c.length >= 0 && (c.splice instanceof Function || Object.getOwnPropertyDescriptor(c, c.length - 1) && c.constructor.name !== "String"));
      };
    }, 3989: (u, c, f) => {
      var p = "__lodash_placeholder__", g = 32, h = 1 / 0, y = NaN, v = [["ary", 128], ["bind", 1], ["bindKey", 2], ["curry", 8], ["curryRight", 16], ["flip", 512], ["partial", g], ["partialRight", 64], ["rearg", 256]], C = "[object Function]", $ = "[object GeneratorFunction]", S = /^\s+|\s+$/g, w = /\{(?:\n\/\* \[wrapped with .+\] \*\/)?\n?/, _ = /\{\n\/\* \[wrapped with (.+)\] \*/, b = /,? & /, k = /^[-+]0x[0-9a-f]+$/i, E = /^0b[01]+$/i, j = /^\[object .+?Constructor\]$/, L = /^0o[0-7]+$/i, B = /^(?:0|[1-9]\d*)$/, O = parseInt, V = typeof f.g == "object" && f.g && f.g.Object === Object && f.g, W = typeof self == "object" && self && self.Object === Object && self, q = V || W || Function("return this")();
      function Q(se, ae, be) {
        switch (be.length) {
          case 0:
            return se.call(ae);
          case 1:
            return se.call(ae, be[0]);
          case 2:
            return se.call(ae, be[0], be[1]);
          case 3:
            return se.call(ae, be[0], be[1], be[2]);
        }
        return se.apply(ae, be);
      }
      function X(se, ae) {
        return !!(se && se.length) && function(be, Re, Ge) {
          if (Re != Re) return function(tt, wt, Gt, dn) {
            for (var jn = tt.length, qt = Gt + -1; ++qt < jn; ) if (wt(tt[qt], qt, tt)) return qt;
            return -1;
          }(be, ce, Ge);
          for (var Rt = Ge - 1, Fe = be.length; ++Rt < Fe; ) if (be[Rt] === Re) return Rt;
          return -1;
        }(se, ae, 0) > -1;
      }
      function ce(se) {
        return se != se;
      }
      function J(se, ae) {
        for (var be = -1, Re = se.length, Ge = 0, Rt = []; ++be < Re; ) {
          var Fe = se[be];
          Fe !== ae && Fe !== p || (se[be] = p, Rt[Ge++] = be);
        }
        return Rt;
      }
      var ie, re, Y, T = Function.prototype, H = Object.prototype, G = q["__core-js_shared__"], N = (ie = /[^.]+$/.exec(G && G.keys && G.keys.IE_PROTO || "")) ? "Symbol(src)_1." + ie : "", U = T.toString, Z = H.hasOwnProperty, me = H.toString, ue = RegExp("^" + U.call(Z).replace(/[\\^$.*+?()[\]{}|]/g, "\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g, "$1.*?") + "$"), Ce = Object.create, $e = Math.max, Te = Math.min, We = (re = Se(Object, "defineProperty"), (Y = Se.name) && Y.length > 2 ? re : void 0);
      function et(se) {
        if (!Me(se) || function(be) {
          return !!N && N in be;
        }(se)) return !1;
        var ae = function(be) {
          var Re = Me(be) ? me.call(be) : "";
          return Re == C || Re == $;
        }(se) || function(be) {
          var Re = !1;
          if (be != null && typeof be.toString != "function") try {
            Re = !!(be + "");
          } catch {
          }
          return Re;
        }(se) ? ue : j;
        return ae.test(function(be) {
          if (be != null) {
            try {
              return U.call(be);
            } catch {
            }
            try {
              return be + "";
            } catch {
            }
          }
          return "";
        }(se));
      }
      function Xe(se) {
        return function() {
          var ae = arguments;
          switch (ae.length) {
            case 0:
              return new se();
            case 1:
              return new se(ae[0]);
            case 2:
              return new se(ae[0], ae[1]);
            case 3:
              return new se(ae[0], ae[1], ae[2]);
            case 4:
              return new se(ae[0], ae[1], ae[2], ae[3]);
            case 5:
              return new se(ae[0], ae[1], ae[2], ae[3], ae[4]);
            case 6:
              return new se(ae[0], ae[1], ae[2], ae[3], ae[4], ae[5]);
            case 7:
              return new se(ae[0], ae[1], ae[2], ae[3], ae[4], ae[5], ae[6]);
          }
          var be, Re = Me(be = se.prototype) ? Ce(be) : {}, Ge = se.apply(Re, ae);
          return Me(Ge) ? Ge : Re;
        };
      }
      function ft(se, ae, be, Re, Ge, Rt, Fe, tt, wt, Gt) {
        var dn = 128 & ae, jn = 1 & ae, qt = 2 & ae, $t = 24 & ae, At = 512 & ae, Jt = qt ? void 0 : Xe(se);
        return function $n() {
          for (var Ut = arguments.length, Yt = Array(Ut), On = Ut; On--; ) Yt[On] = arguments[On];
          if ($t) var Zn = pt($n), cr = function(Qt, Rn) {
            for (var zn = Qt.length, er = 0; zn--; ) Qt[zn] === Rn && er++;
            return er;
          }(Yt, Zn);
          if (Re && (Yt = function(Qt, Rn, zn, er) {
            for (var Bn = -1, Jr = Qt.length, Yr = zn.length, yn = -1, fr = Rn.length, Lr = $e(Jr - Yr, 0), yo = Array(fr + Lr), vo = !er; ++yn < fr; ) yo[yn] = Rn[yn];
            for (; ++Bn < Yr; ) (vo || Bn < Jr) && (yo[zn[Bn]] = Qt[Bn]);
            for (; Lr--; ) yo[yn++] = Qt[Bn++];
            return yo;
          }(Yt, Re, Ge, $t)), Rt && (Yt = function(Qt, Rn, zn, er) {
            for (var Bn = -1, Jr = Qt.length, Yr = -1, yn = zn.length, fr = -1, Lr = Rn.length, yo = $e(Jr - yn, 0), vo = Array(yo + Lr), ns = !er; ++Bn < yo; ) vo[Bn] = Qt[Bn];
            for (var da = Bn; ++fr < Lr; ) vo[da + fr] = Rn[fr];
            for (; ++Yr < yn; ) (ns || Bn < Jr) && (vo[da + zn[Yr]] = Qt[Bn++]);
            return vo;
          }(Yt, Rt, Fe, $t)), Ut -= cr, $t && Ut < Gt) {
            var dr = J(Yt, Zn);
            return Lt(se, ae, ft, $n.placeholder, be, Yt, dr, tt, wt, Gt - Ut);
          }
          var Mt = jn ? be : this, Ro = qt ? Mt[se] : se;
          return Ut = Yt.length, tt ? Yt = function(Qt, Rn) {
            for (var zn = Qt.length, er = Te(Rn.length, zn), Bn = function(Yr, yn) {
              var fr = -1, Lr = Yr.length;
              for (yn || (yn = Array(Lr)); ++fr < Lr; ) yn[fr] = Yr[fr];
              return yn;
            }(Qt); er--; ) {
              var Jr = Rn[er];
              Qt[er] = Le(Jr, zn) ? Bn[Jr] : void 0;
            }
            return Qt;
          }(Yt, tt) : At && Ut > 1 && Yt.reverse(), dn && wt < Ut && (Yt.length = wt), this && this !== q && this instanceof $n && (Ro = Jt || Xe(Ro)), Ro.apply(Mt, Yt);
        };
      }
      function Lt(se, ae, be, Re, Ge, Rt, Fe, tt, wt, Gt) {
        var dn = 8 & ae;
        ae |= dn ? g : 64, 4 & (ae &= ~(dn ? 64 : g)) || (ae &= -4);
        var jn = be(se, ae, Ge, dn ? Rt : void 0, dn ? Fe : void 0, dn ? void 0 : Rt, dn ? void 0 : Fe, tt, wt, Gt);
        return jn.placeholder = Re, cn(jn, se, ae);
      }
      function st(se, ae, be, Re, Ge, Rt, Fe, tt) {
        var wt = 2 & ae;
        if (!wt && typeof se != "function") throw new TypeError("Expected a function");
        var Gt = Re ? Re.length : 0;
        if (Gt || (ae &= -97, Re = Ge = void 0), Fe = Fe === void 0 ? Fe : $e(ot(Fe), 0), tt = tt === void 0 ? tt : ot(tt), Gt -= Ge ? Ge.length : 0, 64 & ae) {
          var dn = Re, jn = Ge;
          Re = Ge = void 0;
        }
        var qt = [se, ae, be, Re, Ge, dn, jn, Rt, Fe, tt];
        if (se = qt[0], ae = qt[1], be = qt[2], Re = qt[3], Ge = qt[4], !(tt = qt[9] = qt[9] == null ? wt ? 0 : se.length : $e(qt[9] - Gt, 0)) && 24 & ae && (ae &= -25), ae && ae != 1) $t = ae == 8 || ae == 16 ? function(At, Jt, $n) {
          var Ut = Xe(At);
          return function Yt() {
            for (var On = arguments.length, Zn = Array(On), cr = On, dr = pt(Yt); cr--; ) Zn[cr] = arguments[cr];
            var Mt = On < 3 && Zn[0] !== dr && Zn[On - 1] !== dr ? [] : J(Zn, dr);
            return (On -= Mt.length) < $n ? Lt(At, Jt, ft, Yt.placeholder, void 0, Zn, Mt, void 0, void 0, $n - On) : Q(this && this !== q && this instanceof Yt ? Ut : At, this, Zn);
          };
        }(se, ae, tt) : ae != g && ae != 33 || Ge.length ? ft.apply(void 0, qt) : function(At, Jt, $n, Ut) {
          var Yt = 1 & Jt, On = Xe(At);
          return function Zn() {
            for (var cr = -1, dr = arguments.length, Mt = -1, Ro = Ut.length, Qt = Array(Ro + dr), Rn = this && this !== q && this instanceof Zn ? On : At; ++Mt < Ro; ) Qt[Mt] = Ut[Mt];
            for (; dr--; ) Qt[Mt++] = arguments[++cr];
            return Q(Rn, Yt ? $n : this, Qt);
          };
        }(se, ae, be, Re);
        else var $t = function(At, Jt, $n) {
          var Ut = 1 & Jt, Yt = Xe(At);
          return function On() {
            return (this && this !== q && this instanceof On ? Yt : At).apply(Ut ? $n : this, arguments);
          };
        }(se, ae, be);
        return cn($t, se, ae);
      }
      function pt(se) {
        return se.placeholder;
      }
      function Se(se, ae) {
        var be = function(Re, Ge) {
          return Re == null ? void 0 : Re[Ge];
        }(se, ae);
        return et(be) ? be : void 0;
      }
      function gt(se) {
        var ae = se.match(_);
        return ae ? ae[1].split(b) : [];
      }
      function yt(se, ae) {
        var be = ae.length, Re = be - 1;
        return ae[Re] = (be > 1 ? "& " : "") + ae[Re], ae = ae.join(be > 2 ? ", " : " "), se.replace(w, `{
/* [wrapped with ` + ae + `] */
`);
      }
      function Le(se, ae) {
        return !!(ae = ae ?? 9007199254740991) && (typeof se == "number" || B.test(se)) && se > -1 && se % 1 == 0 && se < ae;
      }
      var cn = We ? function(se, ae, be) {
        var Re, Ge = ae + "";
        return We(se, "toString", { configurable: !0, enumerable: !1, value: (Re = yt(Ge, ye(gt(Ge), be)), function() {
          return Re;
        }) });
      } : function(se) {
        return se;
      };
      function ye(se, ae) {
        return function(be, Re) {
          for (var Ge = -1, Rt = be.length; ++Ge < Rt && Re(be[Ge], Ge, be) !== !1; ) ;
        }(v, function(be) {
          var Re = "_." + be[0];
          ae & be[1] && !X(se, Re) && se.push(Re);
        }), se.sort();
      }
      function Pe(se, ae, be) {
        var Re = st(se, 8, void 0, void 0, void 0, void 0, void 0, ae = be ? void 0 : ae);
        return Re.placeholder = Pe.placeholder, Re;
      }
      function Me(se) {
        var ae = typeof se;
        return !!se && (ae == "object" || ae == "function");
      }
      function Ye(se) {
        return se ? (se = function(ae) {
          if (typeof ae == "number") return ae;
          if (function(Ge) {
            return typeof Ge == "symbol" || /* @__PURE__ */ function(Rt) {
              return !!Rt && typeof Rt == "object";
            }(Ge) && me.call(Ge) == "[object Symbol]";
          }(ae)) return y;
          if (Me(ae)) {
            var be = typeof ae.valueOf == "function" ? ae.valueOf() : ae;
            ae = Me(be) ? be + "" : be;
          }
          if (typeof ae != "string") return ae === 0 ? ae : +ae;
          ae = ae.replace(S, "");
          var Re = E.test(ae);
          return Re || L.test(ae) ? O(ae.slice(2), Re ? 2 : 8) : k.test(ae) ? y : +ae;
        }(se)) === h || se === -1 / 0 ? 17976931348623157e292 * (se < 0 ? -1 : 1) : se == se ? se : 0 : se === 0 ? se : 0;
      }
      function ot(se) {
        var ae = Ye(se), be = ae % 1;
        return ae == ae ? be ? ae - be : ae : 0;
      }
      Pe.placeholder = {}, u.exports = Pe;
    }, 301: (u, c, f) => {
      var p = f(8336), g = Array.prototype.concat, h = Array.prototype.slice, y = u.exports = function(v) {
        for (var C = [], $ = 0, S = v.length; $ < S; $++) {
          var w = v[$];
          p(w) ? C = g.call(C, h.call(w)) : C.push(w);
        }
        return C;
      };
      y.wrap = function(v) {
        return function() {
          return v(y(arguments));
        };
      };
    }, 4119: (u) => {
      u.exports = n;
    } }, i = {};
    function s(u) {
      var c = i[u];
      if (c !== void 0) return c.exports;
      var f = i[u] = { exports: {} };
      return o[u](f, f.exports, s), f.exports;
    }
    s.n = (u) => {
      var c = u && u.__esModule ? () => u.default : () => u;
      return s.d(c, { a: c }), c;
    }, s.d = (u, c) => {
      for (var f in c) s.o(c, f) && !s.o(u, f) && Object.defineProperty(u, f, { enumerable: !0, get: c[f] });
    }, s.g = function() {
      if (typeof globalThis == "object") return globalThis;
      try {
        return this || new Function("return this")();
      } catch {
        if (typeof window == "object") return window;
      }
    }(), s.o = (u, c) => Object.prototype.hasOwnProperty.call(u, c), s.r = (u) => {
      typeof Symbol < "u" && Symbol.toStringTag && Object.defineProperty(u, Symbol.toStringTag, { value: "Module" }), Object.defineProperty(u, "__esModule", { value: !0 });
    };
    var l = {};
    return (() => {
      function u(D) {
        return u = typeof Symbol == "function" && typeof Symbol.iterator == "symbol" ? function(I) {
          return typeof I;
        } : function(I) {
          return I && typeof Symbol == "function" && I.constructor === Symbol && I !== Symbol.prototype ? "symbol" : typeof I;
        }, u(D);
      }
      function c(D) {
        var I = function(R, F) {
          if (u(R) != "object" || !R) return R;
          var ee = R[Symbol.toPrimitive];
          if (ee !== void 0) {
            var pe = ee.call(R, F || "default");
            if (u(pe) != "object") return pe;
            throw new TypeError("@@toPrimitive must return a primitive value.");
          }
          return (F === "string" ? String : Number)(R);
        }(D, "string");
        return u(I) == "symbol" ? I : I + "";
      }
      function f(D, I, R) {
        return (I = c(I)) in D ? Object.defineProperty(D, I, { value: R, enumerable: !0, configurable: !0, writable: !0 }) : D[I] = R, D;
      }
      function p(D, I) {
        var R = Object.keys(D);
        if (Object.getOwnPropertySymbols) {
          var F = Object.getOwnPropertySymbols(D);
          I && (F = F.filter(function(ee) {
            return Object.getOwnPropertyDescriptor(D, ee).enumerable;
          })), R.push.apply(R, F);
        }
        return R;
      }
      function g(D) {
        for (var I = 1; I < arguments.length; I++) {
          var R = arguments[I] != null ? arguments[I] : {};
          I % 2 ? p(Object(R), !0).forEach(function(F) {
            f(D, F, R[F]);
          }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(D, Object.getOwnPropertyDescriptors(R)) : p(Object(R)).forEach(function(F) {
            Object.defineProperty(D, F, Object.getOwnPropertyDescriptor(R, F));
          });
        }
        return D;
      }
      function h(D, I) {
        if (!(D instanceof I)) throw new TypeError("Cannot call a class as a function");
      }
      function y(D, I) {
        for (var R = 0; R < I.length; R++) {
          var F = I[R];
          F.enumerable = F.enumerable || !1, F.configurable = !0, "value" in F && (F.writable = !0), Object.defineProperty(D, c(F.key), F);
        }
      }
      function v(D, I, R) {
        return I && y(D.prototype, I), R && y(D, R), Object.defineProperty(D, "prototype", { writable: !1 }), D;
      }
      function C(D) {
        return C = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function(I) {
          return I.__proto__ || Object.getPrototypeOf(I);
        }, C(D);
      }
      function $() {
        try {
          var D = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function() {
          }));
        } catch {
        }
        return ($ = function() {
          return !!D;
        })();
      }
      function S(D, I) {
        if (I && (u(I) == "object" || typeof I == "function")) return I;
        if (I !== void 0) throw new TypeError("Derived constructors may only return object or undefined");
        return function(R) {
          if (R === void 0) throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
          return R;
        }(D);
      }
      function w(D, I, R) {
        return I = C(I), S(D, $() ? Reflect.construct(I, R || [], C(D).constructor) : I.apply(D, R));
      }
      function _(D, I) {
        return _ = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function(R, F) {
          return R.__proto__ = F, R;
        }, _(D, I);
      }
      function b(D, I) {
        if (typeof I != "function" && I !== null) throw new TypeError("Super expression must either be null or a function");
        D.prototype = Object.create(I && I.prototype, { constructor: { value: D, writable: !0, configurable: !0 } }), Object.defineProperty(D, "prototype", { writable: !1 }), I && _(D, I);
      }
      s.r(l), s.d(l, { default: () => nh });
      var k = s(4119), E = s.n(k);
      function j() {
        var D = this.constructor.getDerivedStateFromProps(this.props, this.state);
        D != null && this.setState(D);
      }
      function L(D) {
        this.setState((function(I) {
          var R = this.constructor.getDerivedStateFromProps(D, I);
          return R ?? null;
        }).bind(this));
      }
      function B(D, I) {
        try {
          var R = this.props, F = this.state;
          this.props = D, this.state = I, this.__reactInternalSnapshotFlag = !0, this.__reactInternalSnapshot = this.getSnapshotBeforeUpdate(R, F);
        } finally {
          this.props = R, this.state = F;
        }
      }
      function O(D) {
        var I = D.prototype;
        if (!I || !I.isReactComponent) throw new Error("Can only polyfill class components");
        if (typeof D.getDerivedStateFromProps != "function" && typeof I.getSnapshotBeforeUpdate != "function") return D;
        var R = null, F = null, ee = null;
        if (typeof I.componentWillMount == "function" ? R = "componentWillMount" : typeof I.UNSAFE_componentWillMount == "function" && (R = "UNSAFE_componentWillMount"), typeof I.componentWillReceiveProps == "function" ? F = "componentWillReceiveProps" : typeof I.UNSAFE_componentWillReceiveProps == "function" && (F = "UNSAFE_componentWillReceiveProps"), typeof I.componentWillUpdate == "function" ? ee = "componentWillUpdate" : typeof I.UNSAFE_componentWillUpdate == "function" && (ee = "UNSAFE_componentWillUpdate"), R !== null || F !== null || ee !== null) {
          var pe = D.displayName || D.name, de = typeof D.getDerivedStateFromProps == "function" ? "getDerivedStateFromProps()" : "getSnapshotBeforeUpdate()";
          throw Error(`Unsafe legacy lifecycles will not be called for components using new component APIs.

` + pe + " uses " + de + " but also contains the following legacy lifecycles:" + (R !== null ? `
  ` + R : "") + (F !== null ? `
  ` + F : "") + (ee !== null ? `
  ` + ee : "") + `

The above lifecycles should be removed. Learn more about this warning here:
https://fb.me/react-async-component-lifecycle-hooks`);
        }
        if (typeof D.getDerivedStateFromProps == "function" && (I.componentWillMount = j, I.componentWillReceiveProps = L), typeof I.getSnapshotBeforeUpdate == "function") {
          if (typeof I.componentDidUpdate != "function") throw new Error("Cannot polyfill getSnapshotBeforeUpdate() for components that do not define componentDidUpdate() on the prototype");
          I.componentWillUpdate = B;
          var ne = I.componentDidUpdate;
          I.componentDidUpdate = function(ve, we, Ae) {
            var je = this.__reactInternalSnapshotFlag ? this.__reactInternalSnapshot : Ae;
            ne.call(this, ve, we, je);
          };
        }
        return D;
      }
      function V(D, I) {
        if (D == null) return {};
        var R = {};
        for (var F in D) if ({}.hasOwnProperty.call(D, F)) {
          if (I.includes(F)) continue;
          R[F] = D[F];
        }
        return R;
      }
      function W(D, I) {
        if (D == null) return {};
        var R, F, ee = V(D, I);
        if (Object.getOwnPropertySymbols) {
          var pe = Object.getOwnPropertySymbols(D);
          for (F = 0; F < pe.length; F++) R = pe[F], I.includes(R) || {}.propertyIsEnumerable.call(D, R) && (ee[R] = D[R]);
        }
        return ee;
      }
      function q(D, I) {
        var R;
        if (I && (D == null || (R = D.constructor) === null || R === void 0 ? void 0 : R.name) === (I == null ? void 0 : I.name)) return "bigNumber";
        var F = function(ee) {
          return {}.toString.call(ee).match(/\s([a-zA-Z]+)/)[1].toLowerCase();
        }(D);
        return F === "number" && (F = isNaN(D) ? "nan" : (0 | D) != D ? "float" : "integer"), F;
      }
      function Q(D) {
        return D.replace(/\\/g, "\\\\").replace(/\n/g, "\\n").replace(/\t/g, "\\t").replace(/\r/g, "\\r").replace(/\f/g, "\\f");
      }
      j.__suppressDeprecationWarning = !0, L.__suppressDeprecationWarning = !0, B.__suppressDeprecationWarning = !0;
      var X = { scheme: "rjv-default", author: "mac gainor", base00: "rgba(0, 0, 0, 0)", base01: "rgb(245, 245, 245)", base02: "rgb(235, 235, 235)", base03: "#93a1a1", base04: "rgba(0, 0, 0, 0.3)", base05: "#586e75", base06: "#073642", base07: "#002b36", base08: "#d33682", base09: "#cb4b16", base0A: "#dc322f", base0B: "#859900", base0C: "#6c71c4", base0D: "#586e75", base0E: "#2aa198", base0F: "#268bd2" }, ce = { scheme: "rjv-grey", author: "mac gainor", base00: "rgba(1, 1, 1, 0)", base01: "rgba(1, 1, 1, 0.1)", base02: "rgba(0, 0, 0, 0.2)", base03: "rgba(1, 1, 1, 0.3)", base04: "rgba(0, 0, 0, 0.4)", base05: "rgba(1, 1, 1, 0.5)", base06: "rgba(1, 1, 1, 0.6)", base07: "rgba(1, 1, 1, 0.7)", base08: "rgba(1, 1, 1, 0.8)", base09: "rgba(1, 1, 1, 0.8)", base0A: "rgba(1, 1, 1, 0.8)", base0B: "rgba(1, 1, 1, 0.8)", base0C: "rgba(1, 1, 1, 0.8)", base0D: "rgba(1, 1, 1, 0.8)", base0E: "rgba(1, 1, 1, 0.8)", base0F: "rgba(1, 1, 1, 0.8)" };
      const J = { white: "#fff", black: "#000", transparent: "rgba(1, 1, 1, 0)", globalFontFamily: "monospace", globalCursor: "default", indentBlockWidth: "5px", braceFontWeight: "bold", braceCursor: "pointer", ellipsisFontSize: "18px", ellipsisLineHeight: "10px", ellipsisCursor: "pointer", keyMargin: "0px 5px", keyLetterSpacing: "0.5px", keyFontStyle: "none", keyBorderRadius: "3px", keyColonWeight: "bold", keyVerticalAlign: "top", keyOpacity: "0.85", keyOpacityHover: "1", keyValPaddingTop: "3px", keyValPaddingBottom: "3px", keyValPaddingRight: "5px", keyValBorderLeft: "1px solid", keyValBorderHover: "2px solid", keyValPaddingHover: "3px 5px 3px 4px", pushedContentMarginLeft: "6px", variableValuePaddingRight: "6px", nullFontSize: "11px", nullFontWeight: "bold", nullPadding: "1px 2px", nullBorderRadius: "3px", nanFontSize: "11px", nanFontWeight: "bold", nanPadding: "1px 2px", nanBorderRadius: "3px", undefinedFontSize: "11px", undefinedFontWeight: "bold", undefinedPadding: "1px 2px", undefinedBorderRadius: "3px", dataTypeFontSize: "11px", dataTypeMarginRight: "4px", datatypeOpacity: "0.8", objectSizeBorderRadius: "3px", objectSizeFontStyle: "italic", objectSizeMargin: "0px 6px 0px 0px", clipboardCursor: "pointer", clipboardCheckMarginLeft: "-12px", metaDataPadding: "0px 0px 0px 10px", arrayGroupMetaPadding: "0px 0px 0px 4px", iconContainerWidth: "17px", tooltipPadding: "4px", editInputMinWidth: "130px", editInputBorderRadius: "2px", editInputPadding: "5px", editInputMarginRight: "4px", editInputFontFamily: "monospace", iconCursor: "pointer", iconFontSize: "15px", iconPaddingRight: "1px", dateValueMarginLeft: "2px", iconMarginRight: "3px", detectedRowPaddingTop: "3px", addKeyCoverBackground: "rgba(255, 255, 255, 0.3)", addKeyCoverPosition: "absolute", addKeyCoverPositionPx: "0px", addKeyModalWidth: "200px", addKeyModalMargin: "auto", addKeyModalPadding: "10px", addKeyModalRadius: "3px" };
      function ie(D, I) {
        (I == null || I > D.length) && (I = D.length);
        for (var R = 0, F = Array(I); R < I; R++) F[R] = D[R];
        return F;
      }
      function re(D, I) {
        if (D) {
          if (typeof D == "string") return ie(D, I);
          var R = {}.toString.call(D).slice(8, -1);
          return R === "Object" && D.constructor && (R = D.constructor.name), R === "Map" || R === "Set" ? Array.from(D) : R === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(R) ? ie(D, I) : void 0;
        }
      }
      function Y(D, I) {
        return function(R) {
          if (Array.isArray(R)) return R;
        }(D) || function(R, F) {
          var ee = R == null ? null : typeof Symbol < "u" && R[Symbol.iterator] || R["@@iterator"];
          if (ee != null) {
            var pe, de, ne, ve, we = [], Ae = !0, je = !1;
            try {
              if (ne = (ee = ee.call(R)).next, F === 0) {
                if (Object(ee) !== ee) return;
                Ae = !1;
              } else for (; !(Ae = (pe = ne.call(ee)).done) && (we.push(pe.value), we.length !== F); Ae = !0) ;
            } catch (Be) {
              je = !0, de = Be;
            } finally {
              try {
                if (!Ae && ee.return != null && (ve = ee.return(), Object(ve) !== ve)) return;
              } finally {
                if (je) throw de;
              }
            }
            return we;
          }
        }(D, I) || re(D, I) || function() {
          throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`);
        }();
      }
      var T = s(9446), H = s(3639), G = s.n(H), N = s(3989), U = s.n(N);
      function Z(D, I) {
        var R = Object.keys(D);
        if (Object.getOwnPropertySymbols) {
          var F = Object.getOwnPropertySymbols(D);
          I && (F = F.filter(function(ee) {
            return Object.getOwnPropertyDescriptor(D, ee).enumerable;
          })), R.push.apply(R, F);
        }
        return R;
      }
      function me(D) {
        for (var I = 1; I < arguments.length; I++) {
          var R = arguments[I] != null ? arguments[I] : {};
          I % 2 ? Z(Object(R), !0).forEach(function(F) {
            f(D, F, R[F]);
          }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(D, Object.getOwnPropertyDescriptors(R)) : Z(Object(R)).forEach(function(F) {
            Object.defineProperty(D, F, Object.getOwnPropertyDescriptor(R, F));
          });
        }
        return D;
      }
      var ue = T.default, Ce = Object.keys(ue), $e = function(D) {
        var I, R = function(ve) {
          var we = ve[0] / 255, Ae = ve[1] / 255, je = ve[2] / 255;
          return [0.299 * we + 0.587 * Ae + 0.114 * je, -0.14713 * we + -0.28886 * Ae + 0.436 * je, 0.615 * we + -0.51499 * Ae + -0.10001 * je];
        }(G()(D).array()), F = Y(R, 3), ee = F[0], pe = F[1], de = F[2], ne = function(ve) {
          var we, Ae, je, Be = ve[0], Ie = ve[1], ut = ve[2];
          return we = 1 * Be + 0 * Ie + 1.13983 * ut, Ae = 1 * Be + -0.39465 * Ie + -0.5806 * ut, je = 1 * Be + 2.02311 * Ie + 0 * ut, [255 * (we = Math.min(Math.max(0, we), 1)), 255 * (Ae = Math.min(Math.max(0, Ae), 1)), 255 * (je = Math.min(Math.max(0, je), 1))];
        }([(I = ee, I < 0.25 ? 1 : I < 0.5 ? 0.9 - I : 1.1 - I), pe, de]);
        return G().rgb(ne).hex();
      }, Te = function(D) {
        return function(I) {
          return { className: [I.className, D.className].filter(Boolean).join(" "), style: me(me({}, I.style || {}), D.style || {}) };
        };
      }, We = function(D, I) {
        var R = Object.keys(I);
        for (var F in D) R.indexOf(F) === -1 && R.push(F);
        return R.reduce(function(ee, pe) {
          return ee[pe] = function(de, ne) {
            if (de === void 0) return ne;
            if (ne === void 0) return de;
            var ve = u(de), we = u(ne);
            switch (ve) {
              case "string":
                switch (we) {
                  case "string":
                    return [ne, de].filter(Boolean).join(" ");
                  case "object":
                    return Te({ className: de, style: ne });
                  case "function":
                    return function(Ae) {
                      for (var je = arguments.length, Be = new Array(je > 1 ? je - 1 : 0), Ie = 1; Ie < je; Ie++) Be[Ie - 1] = arguments[Ie];
                      return Te({ className: de })(ne.apply(void 0, [Ae].concat(Be)));
                    };
                }
                break;
              case "object":
                switch (we) {
                  case "string":
                    return Te({ className: ne, style: de });
                  case "object":
                    return me(me({}, ne), de);
                  case "function":
                    return function(Ae) {
                      for (var je = arguments.length, Be = new Array(je > 1 ? je - 1 : 0), Ie = 1; Ie < je; Ie++) Be[Ie - 1] = arguments[Ie];
                      return Te({ style: de })(ne.apply(void 0, [Ae].concat(Be)));
                    };
                }
                break;
              case "function":
                switch (we) {
                  case "string":
                    return function(Ae) {
                      for (var je = arguments.length, Be = new Array(je > 1 ? je - 1 : 0), Ie = 1; Ie < je; Ie++) Be[Ie - 1] = arguments[Ie];
                      return de.apply(void 0, [Te(Ae)({ className: ne })].concat(Be));
                    };
                  case "object":
                    return function(Ae) {
                      for (var je = arguments.length, Be = new Array(je > 1 ? je - 1 : 0), Ie = 1; Ie < je; Ie++) Be[Ie - 1] = arguments[Ie];
                      return de.apply(void 0, [Te(Ae)({ style: ne })].concat(Be));
                    };
                  case "function":
                    return function(Ae) {
                      for (var je = arguments.length, Be = new Array(je > 1 ? je - 1 : 0), Ie = 1; Ie < je; Ie++) Be[Ie - 1] = arguments[Ie];
                      return de.apply(void 0, [ne.apply(void 0, [Ae].concat(Be))].concat(Be));
                    };
                }
            }
          }(D[pe], I[pe]), ee;
        }, {});
      }, et = function(D, I) {
        for (var R = arguments.length, F = new Array(R > 2 ? R - 2 : 0), ee = 2; ee < R; ee++) F[ee - 2] = arguments[ee];
        if (I === null) return D;
        Array.isArray(I) || (I = [I]);
        var pe = I.map(function(de) {
          return D[de];
        }).filter(Boolean).reduce(function(de, ne) {
          return typeof ne == "string" ? de.className = [de.className, ne].filter(Boolean).join(" ") : u(ne) === "object" ? de.style = me(me({}, de.style), ne) : typeof ne == "function" && (de = me(me({}, de), ne.apply(void 0, [de].concat(F)))), de;
        }, { className: "", style: {} });
        return pe.className || delete pe.className, Object.keys(pe.style).length === 0 && delete pe.style, pe;
      }, Xe = function(D) {
        return Object.keys(D).reduce(function(I, R) {
          return I[R] = /^base/.test(R) ? $e(D[R]) : R === "scheme" ? D[R] + ":inverted" : D[R], I;
        }, {});
      }, ft = U()(function(D) {
        var I = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {}, R = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : {}, F = I.defaultBase16, ee = F === void 0 ? ue : F, pe = I.base16Themes, de = st(R, pe === void 0 ? null : pe);
        de && (R = me(me({}, de), R));
        for (var ne = Ce.reduce(function(ut, Ct) {
          return ut[Ct] = R[Ct] || ee[Ct], ut;
        }, {}), ve = Object.keys(R).reduce(function(ut, Ct) {
          return Ce.indexOf(Ct) === -1 && (ut[Ct] = R[Ct]), ut;
        }, {}), we = D(ne), Ae = We(ve, we), je = arguments.length, Be = new Array(je > 3 ? je - 3 : 0), Ie = 3; Ie < je; Ie++) Be[Ie - 3] = arguments[Ie];
        return U()(et, 2).apply(void 0, [Ae].concat(Be));
      }, 3), Lt = function(D) {
        return !!D.extend;
      }, st = function(D, I) {
        if (D && Lt(D) && D.extend && (D = D.extend), typeof D == "string") {
          var R = Y(D.split(":"), 2), F = R[0], ee = R[1];
          D = I ? I[F] : T[F], ee === "inverted" && (D = Xe(D));
        }
        return D && Object.prototype.hasOwnProperty.call(D, "base00") ? D : void 0;
      }, pt = function(D) {
        var I = function(R) {
          return { backgroundColor: R.base00, ellipsisColor: R.base09, braceColor: R.base07, expandedIcon: R.base0D, collapsedIcon: R.base0E, keyColor: R.base07, arrayKeyColor: R.base0C, objectSize: R.base04, copyToClipboard: R.base0F, copyToClipboardCheck: R.base0D, objectBorder: R.base02, dataTypes: { boolean: R.base0E, date: R.base0D, float: R.base0B, function: R.base0D, integer: R.base0F, string: R.base09, nan: R.base08, null: R.base0A, undefined: R.base05, regexp: R.base0A, background: R.base02, bigNumber: R.base09 }, editVariable: { editIcon: R.base0E, cancelIcon: R.base09, removeIcon: R.base09, addIcon: R.base0E, checkIcon: R.base0E, background: R.base01, color: R.base0A, border: R.base07 }, addKeyModal: { background: R.base05, border: R.base04, color: R.base0A, labelColor: R.base01 }, validationFailure: { background: R.base09, iconColor: R.base01, fontColor: R.base01 } };
        }(D);
        return { "app-container": { fontFamily: J.globalFontFamily, cursor: J.globalCursor, backgroundColor: I.backgroundColor, position: "relative" }, ellipsis: { display: "inline-block", color: I.ellipsisColor, fontSize: J.ellipsisFontSize, lineHeight: J.ellipsisLineHeight, cursor: J.ellipsisCursor }, "brace-row": { display: "inline-block", cursor: "pointer" }, brace: { display: "inline-block", cursor: J.braceCursor, fontWeight: J.braceFontWeight, color: I.braceColor }, "expanded-icon": { color: I.expandedIcon }, "collapsed-icon": { color: I.collapsedIcon }, colon: { display: "inline-block", margin: J.keyMargin, color: I.keyColor, verticalAlign: "top" }, objectKeyVal: function(R, F) {
          return { style: g({ paddingTop: J.keyValPaddingTop, paddingRight: J.keyValPaddingRight, paddingBottom: J.keyValPaddingBottom, borderLeft: J.keyValBorderLeft + " " + I.objectBorder, ":hover": { paddingLeft: F.paddingLeft - 1 + "px", borderLeft: J.keyValBorderHover + " " + I.objectBorder } }, F) };
        }, "object-key-val-no-border": { padding: J.keyValPadding }, "pushed-content": { marginLeft: J.pushedContentMarginLeft }, variableValue: function(R, F) {
          return { style: g({ display: "inline-block", paddingRight: J.variableValuePaddingRight, position: "relative" }, F) };
        }, "object-name": { display: "inline-block", color: I.keyColor, letterSpacing: J.keyLetterSpacing, fontStyle: J.keyFontStyle, verticalAlign: J.keyVerticalAlign, opacity: J.keyOpacity, ":hover": { opacity: J.keyOpacityHover } }, "array-key": { display: "inline-block", color: I.arrayKeyColor, letterSpacing: J.keyLetterSpacing, fontStyle: J.keyFontStyle, verticalAlign: J.keyVerticalAlign, opacity: J.keyOpacity, ":hover": { opacity: J.keyOpacityHover } }, "object-size": { color: I.objectSize, borderRadius: J.objectSizeBorderRadius, fontStyle: J.objectSizeFontStyle, margin: J.objectSizeMargin, cursor: "default" }, "data-type-label": { fontSize: J.dataTypeFontSize, marginRight: J.dataTypeMarginRight, opacity: J.datatypeOpacity }, boolean: { display: "inline-block", color: I.dataTypes.boolean }, date: { display: "inline-block", color: I.dataTypes.date }, "date-value": { marginLeft: J.dateValueMarginLeft }, float: { display: "inline-block", color: I.dataTypes.float }, function: { display: "inline-block", color: I.dataTypes.function, cursor: "pointer", whiteSpace: "pre-line" }, "function-value": { fontStyle: "italic" }, integer: { display: "inline-block", color: I.dataTypes.integer }, bigNumber: { display: "inline-block", color: I.dataTypes.bigNumber }, string: { display: "inline-block", color: I.dataTypes.string }, nan: { display: "inline-block", color: I.dataTypes.nan, fontSize: J.nanFontSize, fontWeight: J.nanFontWeight, backgroundColor: I.dataTypes.background, padding: J.nanPadding, borderRadius: J.nanBorderRadius }, null: { display: "inline-block", color: I.dataTypes.null, fontSize: J.nullFontSize, fontWeight: J.nullFontWeight, backgroundColor: I.dataTypes.background, padding: J.nullPadding, borderRadius: J.nullBorderRadius }, undefined: { display: "inline-block", color: I.dataTypes.undefined, fontSize: J.undefinedFontSize, padding: J.undefinedPadding, borderRadius: J.undefinedBorderRadius, backgroundColor: I.dataTypes.background }, regexp: { display: "inline-block", color: I.dataTypes.regexp }, "copy-to-clipboard": { cursor: J.clipboardCursor }, "copy-icon": { color: I.copyToClipboard, fontSize: J.iconFontSize, marginRight: J.iconMarginRight, verticalAlign: "top" }, "copy-icon-copied": { color: I.copyToClipboardCheck, marginLeft: J.clipboardCheckMarginLeft }, "array-group-meta-data": { display: "inline-block", padding: J.arrayGroupMetaPadding }, "object-meta-data": { display: "inline-block", padding: J.metaDataPadding }, "icon-container": { display: "inline-block", width: J.iconContainerWidth }, tooltip: { padding: J.tooltipPadding }, removeVarIcon: { verticalAlign: "top", display: "inline-block", color: I.editVariable.removeIcon, cursor: J.iconCursor, fontSize: J.iconFontSize, marginRight: J.iconMarginRight }, addVarIcon: { verticalAlign: "top", display: "inline-block", color: I.editVariable.addIcon, cursor: J.iconCursor, fontSize: J.iconFontSize, marginRight: J.iconMarginRight }, editVarIcon: { verticalAlign: "top", display: "inline-block", color: I.editVariable.editIcon, cursor: J.iconCursor, fontSize: J.iconFontSize, marginRight: J.iconMarginRight }, "edit-icon-container": { display: "inline-block", verticalAlign: "top" }, "check-icon": { display: "inline-block", cursor: J.iconCursor, color: I.editVariable.checkIcon, fontSize: J.iconFontSize, paddingRight: J.iconPaddingRight }, "cancel-icon": { display: "inline-block", cursor: J.iconCursor, color: I.editVariable.cancelIcon, fontSize: J.iconFontSize, paddingRight: J.iconPaddingRight }, "edit-input": { display: "inline-block", minWidth: J.editInputMinWidth, borderRadius: J.editInputBorderRadius, backgroundColor: I.editVariable.background, color: I.editVariable.color, padding: J.editInputPadding, marginRight: J.editInputMarginRight, fontFamily: J.editInputFontFamily }, "detected-row": { paddingTop: J.detectedRowPaddingTop }, "key-modal-request": { position: J.addKeyCoverPosition, top: J.addKeyCoverPositionPx, left: J.addKeyCoverPositionPx, right: J.addKeyCoverPositionPx, bottom: J.addKeyCoverPositionPx, backgroundColor: J.addKeyCoverBackground }, "key-modal": { width: J.addKeyModalWidth, backgroundColor: I.addKeyModal.background, marginLeft: J.addKeyModalMargin, marginRight: J.addKeyModalMargin, padding: J.addKeyModalPadding, borderRadius: J.addKeyModalRadius, marginTop: "15px", position: "relative" }, "key-modal-label": { color: I.addKeyModal.labelColor, marginLeft: "2px", marginBottom: "5px", fontSize: "11px" }, "key-modal-input-container": { overflow: "hidden" }, "key-modal-input": { width: "100%", padding: "3px 6px", fontFamily: "monospace", color: I.addKeyModal.color, border: "none", boxSizing: "border-box", borderRadius: "2px" }, "key-modal-cancel": { backgroundColor: I.editVariable.removeIcon, position: "absolute", top: "0px", right: "0px", borderRadius: "0px 3px 0px 3px", cursor: "pointer" }, "key-modal-cancel-icon": { color: I.addKeyModal.labelColor, fontSize: J.iconFontSize, transform: "rotate(45deg)" }, "key-modal-submit": { color: I.editVariable.addIcon, fontSize: J.iconFontSize, position: "absolute", right: "2px", top: "3px", cursor: "pointer" }, "function-ellipsis": { display: "inline-block", color: I.ellipsisColor, fontSize: J.ellipsisFontSize, lineHeight: J.ellipsisLineHeight, cursor: J.ellipsisCursor }, "validation-failure": { float: "right", padding: "3px 6px", borderRadius: "2px", cursor: "pointer", color: I.validationFailure.fontColor, backgroundColor: I.validationFailure.background }, "validation-failure-label": { marginRight: "6px" }, "validation-failure-clear": { position: "relative", verticalAlign: "top", cursor: "pointer", color: I.validationFailure.iconColor, fontSize: J.iconFontSize, transform: "rotate(45deg)" } };
      };
      function Se(D, I, R) {
        return D || console.error("theme has not been set"), function(F) {
          var ee = X;
          return F !== !1 && F !== "none" || (ee = ce), ft(pt, { defaultBase16: ee })(F);
        }(D)(I, R);
      }
      var gt = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = (R.rjvId, R.type_name), ee = R.displayDataTypes, pe = R.theme;
          return ee ? E().createElement("span", Object.assign({ className: "data-type-label" }, Se(pe, "data-type-label")), F) : null;
        } }]);
      }(E().PureComponent), yt = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props;
          return E().createElement("div", Se(R.theme, "boolean"), E().createElement(gt, Object.assign({ type_name: "bool" }, R)), R.value ? "true" : "false");
        } }]);
      }(E().PureComponent), Le = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props;
          return E().createElement("div", Se(R.theme, "date"), E().createElement(gt, Object.assign({ type_name: "date" }, R)), E().createElement("span", Object.assign({ className: "date-value" }, Se(R.theme, "date-value")), R.value.toLocaleTimeString("en-us", { weekday: "short", year: "numeric", month: "short", day: "numeric", hour: "2-digit", minute: "2-digit" })));
        } }]);
      }(E().PureComponent), cn = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props;
          return E().createElement("div", Se(R.theme, "float"), E().createElement(gt, Object.assign({ type_name: "float" }, R)), this.props.value);
        } }]);
      }(E().PureComponent);
      function ye(D) {
        return function(I) {
          if (Array.isArray(I)) return ie(I);
        }(D) || function(I) {
          if (typeof Symbol < "u" && I[Symbol.iterator] != null || I["@@iterator"] != null) return Array.from(I);
        }(D) || re(D) || function() {
          throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`);
        }();
      }
      var Pe = s(9784), Me = function() {
        return v(function D() {
          h(this, D), this.handler = function() {
          };
        }, [{ key: "register", value: function(D) {
          this.handler = D;
        } }, { key: "dispatch", value: function(D) {
          var I;
          (I = this.handler) === null || I === void 0 || I.call(this, D);
        } }]);
      }();
      globalThis.__globalDispatcherInstance || (globalThis.__globalDispatcherInstance = new Me());
      const Ye = globalThis.__globalDispatcherInstance;
      var ot = new (function(D) {
        function I() {
          var R;
          h(this, I);
          for (var F = arguments.length, ee = new Array(F), pe = 0; pe < F; pe++) ee[pe] = arguments[pe];
          return (R = w(this, I, [].concat(ee))).objects = {}, R.set = function(de, ne, ve, we) {
            R.objects[de] === void 0 && (R.objects[de] = {}), R.objects[de][ne] === void 0 && (R.objects[de][ne] = {}), R.objects[de][ne][ve] = we;
          }, R.get = function(de, ne, ve, we) {
            return R.objects[de] === void 0 || R.objects[de][ne] === void 0 || R.objects[de][ne][ve] == null ? we : R.objects[de][ne][ve];
          }, R.handleAction = function(de) {
            var ne = de.rjvId, ve = de.data;
            switch (de.name) {
              case "RESET":
                R.emit("reset-" + ne);
                break;
              case "VARIABLE_UPDATED":
                de.data.updated_src = R.updateSrc(ne, ve), R.set(ne, "action", "variable-update", g(g({}, ve), {}, { type: "variable-edited" })), R.emit("variable-update-" + ne);
                break;
              case "VARIABLE_REMOVED":
                de.data.updated_src = R.updateSrc(ne, ve), R.set(ne, "action", "variable-update", g(g({}, ve), {}, { type: "variable-removed" })), R.emit("variable-update-" + ne);
                break;
              case "VARIABLE_ADDED":
                de.data.updated_src = R.updateSrc(ne, ve), R.set(ne, "action", "variable-update", g(g({}, ve), {}, { type: "variable-added" })), R.emit("variable-update-" + ne);
                break;
              case "ADD_VARIABLE_KEY_REQUEST":
                R.set(ne, "action", "new-key-request", ve), R.emit("add-key-request-" + ne);
            }
          }, R.updateSrc = function(de, ne) {
            var ve = ne.name, we = ne.namespace, Ae = ne.new_value, je = (ne.existing_value, ne.variable_removed);
            we.shift();
            var Be, Ie = R.get(de, "global", "src"), ut = R.deepCopy(Ie, ye(we)), Ct = ut, Pt = function(Kt, hr) {
              var Tn = typeof Symbol < "u" && Kt[Symbol.iterator] || Kt["@@iterator"];
              if (!Tn) {
                if (Array.isArray(Kt) || (Tn = re(Kt)) || hr) {
                  Tn && (Kt = Tn);
                  var ha = 0, Er = function() {
                  };
                  return { s: Er, n: function() {
                    return ha >= Kt.length ? { done: !0 } : { done: !1, value: Kt[ha++] };
                  }, e: function(Mn) {
                    throw Mn;
                  }, f: Er };
                }
                throw new TypeError(`Invalid attempt to iterate non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`);
              }
              var Mo, bo = !0, on = !1;
              return { s: function() {
                Tn = Tn.call(Kt);
              }, n: function() {
                var Mn = Tn.next();
                return bo = Mn.done, Mn;
              }, e: function(Mn) {
                on = !0, Mo = Mn;
              }, f: function() {
                try {
                  bo || Tn.return == null || Tn.return();
                } finally {
                  if (on) throw Mo;
                }
              } };
            }(we);
            try {
              for (Pt.s(); !(Be = Pt.n()).done; )
                Ct = Ct[Be.value];
            } catch (Kt) {
              Pt.e(Kt);
            } finally {
              Pt.f();
            }
            return je ? q(Ct) == "array" ? Ct.splice(ve, 1) : delete Ct[ve] : ve !== null ? Ct[ve] = Ae : ut = Ae, R.set(de, "global", "src", ut), ut;
          }, R.deepCopy = function(de, ne) {
            var ve, we = q(de), Ae = ne.shift();
            return we == "array" ? ve = ye(de) : we == "object" && (ve = g({}, de)), Ae !== void 0 && (ve[Ae] = R.deepCopy(de[Ae], ne)), ve;
          }, R;
        }
        return b(I, D), v(I);
      }(Pe.EventEmitter))();
      Ye.register(ot.handleAction.bind(ot));
      const se = ot;
      var ae = function(D) {
        function I(R) {
          var F;
          return h(this, I), (F = w(this, I, [R])).toggleCollapsed = function() {
            F.setState({ collapsed: !F.state.collapsed }, function() {
              se.set(F.props.rjvId, F.props.namespace, "collapsed", F.state.collapsed);
            });
          }, F.getFunctionDisplay = function(ee) {
            var pe = F.props;
            return ee ? E().createElement("span", null, F.props.value.toString().slice(9, -1).replace(/\{[\s\S]+/, ""), E().createElement("span", { className: "function-collapsed", style: { fontWeight: "bold" } }, E().createElement("span", null, "{"), E().createElement("span", Se(pe.theme, "ellipsis"), "..."), E().createElement("span", null, "}"))) : F.props.value.toString().slice(9, -1);
          }, F.state = { collapsed: se.get(R.rjvId, R.namespace, "collapsed", !0) }, F;
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = this.state.collapsed;
          return E().createElement("div", Se(R.theme, "function"), E().createElement(gt, Object.assign({ type_name: "function" }, R)), E().createElement("span", Object.assign({}, Se(R.theme, "function-value"), { className: "rjv-function-container", onClick: this.toggleCollapsed }), this.getFunctionDisplay(F)));
        } }]);
      }(E().PureComponent), be = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          return E().createElement("div", Se(this.props.theme, "nan"), "NaN");
        } }]);
      }(E().PureComponent), Re = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          return E().createElement("div", Se(this.props.theme, "null"), "NULL");
        } }]);
      }(E().PureComponent), Ge = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props;
          return E().createElement("div", Se(R.theme, "integer"), E().createElement(gt, Object.assign({ type_name: "int" }, R)), this.props.value);
        } }]);
      }(E().PureComponent), Rt = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props;
          return E().createElement("div", Se(R.theme, "regexp"), E().createElement(gt, Object.assign({ type_name: "regexp" }, R)), this.props.value.toString());
        } }]);
      }(E().PureComponent), Fe = function(D) {
        function I(R) {
          var F;
          return h(this, I), (F = w(this, I, [R])).toggleCollapsed = function() {
            F.setState({ collapsed: !F.state.collapsed }, function() {
              se.set(F.props.rjvId, F.props.namespace, "collapsed", F.state.collapsed);
            });
          }, F.state = { collapsed: se.get(R.rjvId, R.namespace, "collapsed", !0) }, F;
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.state.collapsed, F = this.props, ee = F.collapseStringsAfterLength, pe = F.theme, de = F.escapeStrings, ne = F.value, ve = q(ee) === "integer", we = { style: { cursor: "default" } };
          return de && (ne = Q(ne)), ve && ne.length > ee && (we.style.cursor = "pointer", R && (ne = E().createElement("span", null, ne.substring(0, ee), E().createElement("span", Se(pe, "ellipsis"), " ...")))), E().createElement("div", Se(pe, "string"), E().createElement(gt, Object.assign({ type_name: "string" }, F)), E().createElement("span", Object.assign({ className: "string-value" }, we, { onClick: this.toggleCollapsed }), '"', ne, '"'));
        } }]);
      }(E().PureComponent), tt = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          return E().createElement("div", Se(this.props.theme, "undefined"), "undefined");
        } }]);
      }(E().PureComponent), wt = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props;
          return E().createElement("div", Se(R.theme, "bigNumber"), E().createElement(gt, Object.assign({ type_name: "bigNumber" }, R)), this.props.value.toString());
        } }]);
      }(E().PureComponent);
      function Gt() {
        return Gt = Object.assign ? Object.assign.bind() : function(D) {
          for (var I = 1; I < arguments.length; I++) {
            var R = arguments[I];
            for (var F in R) ({}).hasOwnProperty.call(R, F) && (D[F] = R[F]);
          }
          return D;
        }, Gt.apply(null, arguments);
      }
      const dn = k.useLayoutEffect;
      var jn = function(D, I) {
        typeof D != "function" ? D.current = I : D(I);
      };
      const qt = function(D, I) {
        var R = (0, k.useRef)();
        return (0, k.useCallback)(function(F) {
          D.current = F, R.current && jn(R.current, null), R.current = I, I && jn(I, F);
        }, [I]);
      };
      var $t = { "min-height": "0", "max-height": "none", height: "0", visibility: "hidden", overflow: "hidden", position: "absolute", "z-index": "-1000", top: "0", right: "0" }, At = function(D) {
        Object.keys($t).forEach(function(I) {
          D.style.setProperty(I, $t[I], "important");
        });
      }, Jt = null, $n = function() {
      }, Ut = ["borderBottomWidth", "borderLeftWidth", "borderRightWidth", "borderTopWidth", "boxSizing", "fontFamily", "fontSize", "fontStyle", "fontWeight", "letterSpacing", "lineHeight", "paddingBottom", "paddingLeft", "paddingRight", "paddingTop", "tabSize", "textIndent", "textRendering", "textTransform", "width", "wordBreak"], Yt = !!document.documentElement.currentStyle, On = function(D) {
        var I, R, F = (I = D, R = k.useRef(I), dn(function() {
          R.current = I;
        }), R);
        (0, k.useLayoutEffect)(function() {
          var ee = function(pe) {
            F.current(pe);
          };
          return window.addEventListener("resize", ee), function() {
            window.removeEventListener("resize", ee);
          };
        }, []);
      }, Zn = function(D, I) {
        var R = D.cacheMeasurements, F = D.maxRows, ee = D.minRows, pe = D.onChange, de = pe === void 0 ? $n : pe, ne = D.onHeightChange, ve = ne === void 0 ? $n : ne, we = V(D, ["cacheMeasurements", "maxRows", "minRows", "onChange", "onHeightChange"]), Ae = we.value !== void 0, je = (0, k.useRef)(null), Be = qt(je, I), Ie = (0, k.useRef)(0), ut = (0, k.useRef)(), Ct = function() {
          var Pt = je.current, Kt = R && ut.current ? ut.current : function(Er) {
            var Mo = window.getComputedStyle(Er);
            if (Mo === null) return null;
            var bo, on = (bo = Mo, Ut.reduce(function(Xr, So) {
              return Xr[So] = bo[So], Xr;
            }, {})), Mn = on.boxSizing;
            return Mn === "" ? null : (Yt && Mn === "border-box" && (on.width = parseFloat(on.width) + parseFloat(on.borderRightWidth) + parseFloat(on.borderLeftWidth) + parseFloat(on.paddingRight) + parseFloat(on.paddingLeft) + "px"), { sizingStyle: on, paddingSize: parseFloat(on.paddingBottom) + parseFloat(on.paddingTop), borderSize: parseFloat(on.borderBottomWidth) + parseFloat(on.borderTopWidth) });
          }(Pt);
          if (Kt) {
            ut.current = Kt;
            var hr = function(Er, Mo, bo, on) {
              bo === void 0 && (bo = 1), on === void 0 && (on = 1 / 0), Jt || ((Jt = document.createElement("textarea")).setAttribute("tabindex", "-1"), Jt.setAttribute("aria-hidden", "true"), At(Jt)), Jt.parentNode === null && document.body.appendChild(Jt);
              var Mn = Er.paddingSize, Xr = Er.borderSize, So = Er.sizingStyle, Vc = So.boxSizing;
              Object.keys(So).forEach(function(ls) {
                var mr = ls;
                Jt.style[mr] = So[mr];
              }), At(Jt), Jt.value = Mo;
              var ss = function(ls, mr) {
                var us = ls.scrollHeight;
                return mr.sizingStyle.boxSizing === "border-box" ? us + mr.borderSize : us - mr.paddingSize;
              }(Jt, Er);
              Jt.value = "x";
              var ma = Jt.scrollHeight - Mn, Oi = ma * bo;
              Vc === "border-box" && (Oi = Oi + Mn + Xr), ss = Math.max(Oi, ss);
              var ga = ma * on;
              return Vc === "border-box" && (ga = ga + Mn + Xr), [ss = Math.min(ga, ss), ma];
            }(Kt, Pt.value || Pt.placeholder || "x", ee, F), Tn = hr[0], ha = hr[1];
            Ie.current !== Tn && (Ie.current = Tn, Pt.style.setProperty("height", Tn + "px", "important"), ve(Tn, { rowHeight: ha }));
          }
        };
        return (0, k.useLayoutEffect)(Ct), On(Ct), (0, k.createElement)("textarea", Gt({}, we, { onChange: function(Pt) {
          Ae || Ct(), de(Pt);
        }, ref: Be }));
      };
      const cr = (0, k.forwardRef)(Zn);
      function dr(D, I) {
        D = D.trim();
        try {
          if ((D = structuredClone(D))[0] === "[") return Mt("array", JSON.parse(D));
          if (D[0] === "{") return Mt("object", JSON.parse(D));
          if (D.match(/\-?\d+\.\d+/) && D.match(/\-?\d+\.\d+/)[0] === D) return I && parseFloat(D).toString() !== D ? Mt("bigNumber", D) : Mt("float", parseFloat(D));
          if (D.match(/\-?\d+e-\d+/) && D.match(/\-?\d+e-\d+/)[0] === D) return Mt("float", Number(D));
          if (D.match(/\-?\d+/) && D.match(/\-?\d+/)[0] === D) return I && parseInt(D).toString() !== D ? Mt("bigNumber", D) : Mt("integer", parseInt(D));
          if (D.match(/\-?\d+e\+\d+/) && D.match(/\-?\d+e\+\d+/)[0] === D) return Mt("integer", Number(D));
        } catch {
        }
        switch (D = D.toLowerCase()) {
          case "undefined":
            return Mt("undefined", void 0);
          case "nan":
            return Mt("nan", NaN);
          case "null":
            return Mt("null", null);
          case "true":
            return Mt("boolean", !0);
          case "false":
            return Mt("boolean", !1);
          default:
            if (D = Date.parse(D)) return Mt("date", new Date(D));
        }
        return Mt(!1, null);
      }
      function Mt(D, I) {
        return { type: D, value: I };
      }
      var Ro = ["style"], Qt = ["style"], Rn = ["style"], zn = ["style"], er = ["style"], Bn = ["style"], Jr = ["style"], Yr = ["style"], yn = ["style"], fr = ["style"], Lr = ["style"], yo = ["style"], vo = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = R.style, ee = W(R, Ro);
          return E().createElement("span", ee, E().createElement("svg", Object.assign({}, pr(F), { viewBox: "0 0 24 24", fill: "currentColor", preserveAspectRatio: "xMidYMid meet" }), E().createElement("path", { d: "M12,20C7.59,20 4,16.41 4,12C4,7.59 7.59,4 12,4C16.41,4 20,7.59 20,12C20,16.41 16.41,20 12,20M12,2A10,10 0 0,0 2,12A10,10 0 0,0 12,22A10,10 0 0,0 22,12A10,10 0 0,0 12,2M7,13H17V11H7" })));
        } }]);
      }(E().PureComponent), ns = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = R.style, ee = W(R, Qt);
          return E().createElement("span", ee, E().createElement("svg", Object.assign({}, pr(F), { viewBox: "0 0 24 24", fill: "currentColor", preserveAspectRatio: "xMidYMid meet" }), E().createElement("path", { d: "M12,20C7.59,20 4,16.41 4,12C4,7.59 7.59,4 12,4C16.41,4 20,7.59 20,12C20,16.41 16.41,20 12,20M12,2A10,10 0 0,0 2,12A10,10 0 0,0 12,22A10,10 0 0,0 22,12A10,10 0 0,0 12,2M13,7H11V11H7V13H11V17H13V13H17V11H13V7Z" })));
        } }]);
      }(E().PureComponent), da = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = R.style, ee = W(R, Rn), pe = pr(F).style;
          return E().createElement("span", ee, E().createElement("svg", { fill: pe.color, width: pe.height, height: pe.width, style: pe, viewBox: "0 0 1792 1792" }, E().createElement("path", { d: "M1344 800v64q0 14-9 23t-23 9h-832q-14 0-23-9t-9-23v-64q0-14 9-23t23-9h832q14 0 23 9t9 23zm128 448v-832q0-66-47-113t-113-47h-832q-66 0-113 47t-47 113v832q0 66 47 113t113 47h832q66 0 113-47t47-113zm128-832v832q0 119-84.5 203.5t-203.5 84.5h-832q-119 0-203.5-84.5t-84.5-203.5v-832q0-119 84.5-203.5t203.5-84.5h832q119 0 203.5 84.5t84.5 203.5z" })));
        } }]);
      }(E().PureComponent), Lc = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = R.style, ee = W(R, zn), pe = pr(F).style;
          return E().createElement("span", ee, E().createElement("svg", { fill: pe.color, width: pe.height, height: pe.width, style: pe, viewBox: "0 0 1792 1792" }, E().createElement("path", { d: "M1344 800v64q0 14-9 23t-23 9h-352v352q0 14-9 23t-23 9h-64q-14 0-23-9t-9-23v-352h-352q-14 0-23-9t-9-23v-64q0-14 9-23t23-9h352v-352q0-14 9-23t23-9h64q14 0 23 9t9 23v352h352q14 0 23 9t9 23zm128 448v-832q0-66-47-113t-113-47h-832q-66 0-113 47t-47 113v832q0 66 47 113t113 47h832q66 0 113-47t47-113zm128-832v832q0 119-84.5 203.5t-203.5 84.5h-832q-119 0-203.5-84.5t-84.5-203.5v-832q0-119 84.5-203.5t203.5-84.5h832q119 0 203.5 84.5t84.5 203.5z" })));
        } }]);
      }(E().PureComponent), Xp = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = R.style, ee = W(R, er);
          return E().createElement("span", ee, E().createElement("svg", { style: g(g({}, pr(F).style), {}, { paddingLeft: "2px", verticalAlign: "top" }), viewBox: "0 0 15 15", fill: "currentColor" }, E().createElement("path", { d: "M0 14l6-6-6-6z" })));
        } }]);
      }(E().PureComponent), Zp = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = R.style, ee = W(R, Bn);
          return E().createElement("span", ee, E().createElement("svg", { style: g(g({}, pr(F).style), {}, { paddingLeft: "2px", verticalAlign: "top" }), viewBox: "0 0 15 15", fill: "currentColor" }, E().createElement("path", { d: "M0 5l6 6 6-6z" })));
        } }]);
      }(E().PureComponent), fn = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = R.style, ee = W(R, Jr);
          return E().createElement("span", ee, E().createElement("svg", Object.assign({}, pr(F), { viewBox: "0 0 40 40", fill: "currentColor", preserveAspectRatio: "xMidYMid meet" }), E().createElement("g", null, E().createElement("path", { d: "m30 35h-25v-22.5h25v7.5h2.5v-12.5c0-1.4-1.1-2.5-2.5-2.5h-7.5c0-2.8-2.2-5-5-5s-5 2.2-5 5h-7.5c-1.4 0-2.5 1.1-2.5 2.5v27.5c0 1.4 1.1 2.5 2.5 2.5h25c1.4 0 2.5-1.1 2.5-2.5v-5h-2.5v5z m-20-27.5h2.5s2.5-1.1 2.5-2.5 1.1-2.5 2.5-2.5 2.5 1.1 2.5 2.5 1.3 2.5 2.5 2.5h2.5s2.5 1.1 2.5 2.5h-20c0-1.5 1.1-2.5 2.5-2.5z m-2.5 20h5v-2.5h-5v2.5z m17.5-5v-5l-10 7.5 10 7.5v-5h12.5v-5h-12.5z m-17.5 10h7.5v-2.5h-7.5v2.5z m12.5-17.5h-12.5v2.5h12.5v-2.5z m-7.5 5h-5v2.5h5v-2.5z" }))));
        } }]);
      }(E().PureComponent), Dl = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = R.style, ee = W(R, Yr);
          return E().createElement("span", ee, E().createElement("svg", Object.assign({}, pr(F), { viewBox: "0 0 40 40", fill: "currentColor", preserveAspectRatio: "xMidYMid meet" }), E().createElement("g", null, E().createElement("path", { d: "m28.6 25q0-0.5-0.4-1l-4-4 4-4q0.4-0.5 0.4-1 0-0.6-0.4-1.1l-2-2q-0.4-0.4-1-0.4-0.6 0-1 0.4l-4.1 4.1-4-4.1q-0.4-0.4-1-0.4-0.6 0-1 0.4l-2 2q-0.5 0.5-0.5 1.1 0 0.5 0.5 1l4 4-4 4q-0.5 0.5-0.5 1 0 0.7 0.5 1.1l2 2q0.4 0.4 1 0.4 0.6 0 1-0.4l4-4.1 4.1 4.1q0.4 0.4 1 0.4 0.6 0 1-0.4l2-2q0.4-0.4 0.4-1z m8.7-5q0 4.7-2.3 8.6t-6.3 6.2-8.6 2.3-8.6-2.3-6.2-6.2-2.3-8.6 2.3-8.6 6.2-6.2 8.6-2.3 8.6 2.3 6.3 6.2 2.3 8.6z" }))));
        } }]);
      }(E().PureComponent), zl = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = R.style, ee = W(R, yn);
          return E().createElement("span", ee, E().createElement("svg", Object.assign({}, pr(F), { viewBox: "0 0 40 40", fill: "currentColor", preserveAspectRatio: "xMidYMid meet" }), E().createElement("g", null, E().createElement("path", { d: "m30.1 21.4v-2.8q0-0.6-0.4-1t-1-0.5h-5.7v-5.7q0-0.6-0.4-1t-1-0.4h-2.9q-0.6 0-1 0.4t-0.4 1v5.7h-5.7q-0.6 0-1 0.5t-0.5 1v2.8q0 0.6 0.5 1t1 0.5h5.7v5.7q0 0.5 0.4 1t1 0.4h2.9q0.6 0 1-0.4t0.4-1v-5.7h5.7q0.6 0 1-0.5t0.4-1z m7.2-1.4q0 4.7-2.3 8.6t-6.3 6.2-8.6 2.3-8.6-2.3-6.2-6.2-2.3-8.6 2.3-8.6 6.2-6.2 8.6-2.3 8.6 2.3 6.3 6.2 2.3 8.6z" }))));
        } }]);
      }(E().PureComponent), Bl = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = R.style, ee = W(R, fr);
          return E().createElement("span", ee, E().createElement("svg", Object.assign({}, pr(F), { viewBox: "0 0 40 40", fill: "currentColor", preserveAspectRatio: "xMidYMid meet" }), E().createElement("g", null, E().createElement("path", { d: "m31.6 21.6h-10v10h-3.2v-10h-10v-3.2h10v-10h3.2v10h10v3.2z" }))));
        } }]);
      }(E().PureComponent), rs = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = R.style, ee = W(R, Lr);
          return E().createElement("span", ee, E().createElement("svg", Object.assign({}, pr(F), { viewBox: "0 0 40 40", fill: "currentColor", preserveAspectRatio: "xMidYMid meet" }), E().createElement("g", null, E().createElement("path", { d: "m19.8 26.4l2.6-2.6-3.4-3.4-2.6 2.6v1.3h2.2v2.1h1.2z m9.8-16q-0.3-0.4-0.7 0l-7.8 7.8q-0.4 0.4 0 0.7t0.7 0l7.8-7.8q0.4-0.4 0-0.7z m1.8 13.2v4.3q0 2.6-1.9 4.5t-4.5 1.9h-18.6q-2.6 0-4.5-1.9t-1.9-4.5v-18.6q0-2.7 1.9-4.6t4.5-1.8h18.6q1.4 0 2.6 0.5 0.3 0.2 0.4 0.5 0.1 0.4-0.2 0.7l-1.1 1.1q-0.3 0.3-0.7 0.1-0.5-0.1-1-0.1h-18.6q-1.4 0-2.5 1.1t-1 2.5v18.6q0 1.4 1 2.5t2.5 1h18.6q1.5 0 2.5-1t1.1-2.5v-2.9q0-0.2 0.2-0.4l1.4-1.5q0.3-0.3 0.8-0.1t0.4 0.6z m-2.1-16.5l6.4 6.5-15 15h-6.4v-6.5z m9.9 3l-2.1 2-6.4-6.4 2.1-2q0.6-0.7 1.5-0.7t1.5 0.7l3.4 3.4q0.6 0.6 0.6 1.5t-0.6 1.5z" }))));
        } }]);
      }(E().PureComponent), Ul = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = R.style, ee = W(R, yo);
          return E().createElement("span", ee, E().createElement("svg", Object.assign({}, pr(F), { viewBox: "0 0 40 40", fill: "currentColor", preserveAspectRatio: "xMidYMid meet" }), E().createElement("g", null, E().createElement("path", { d: "m31.7 16.4q0-0.6-0.4-1l-2.1-2.1q-0.4-0.4-1-0.4t-1 0.4l-9.1 9.1-5-5q-0.5-0.4-1-0.4t-1 0.4l-2.1 2q-0.4 0.4-0.4 1 0 0.6 0.4 1l8.1 8.1q0.4 0.4 1 0.4 0.6 0 1-0.4l12.2-12.1q0.4-0.4 0.4-1z m5.6 3.6q0 4.7-2.3 8.6t-6.3 6.2-8.6 2.3-8.6-2.3-6.2-6.2-2.3-8.6 2.3-8.6 6.2-6.2 8.6-2.3 8.6 2.3 6.3 6.2 2.3 8.6z" }))));
        } }]);
      }(E().PureComponent);
      function pr(D) {
        return D || (D = {}), { style: g(g({ verticalAlign: "middle" }, D), {}, { color: D.color ? D.color : "#000000", height: "1em", width: "1em" }) };
      }
      var fa = function(D) {
        function I(R) {
          var F;
          return h(this, I), (F = w(this, I, [R])).copiedTimer = null, F.copyToClipboardFallback = function(ee) {
            var pe = document.createElement("textarea");
            pe.value = ee, document.body.appendChild(pe), pe.select(), document.execCommand("copy"), document.body.removeChild(pe);
          }, F.handleCopy = function() {
            var ee = F.props, pe = ee.clickCallback, de = ee.src, ne = ee.namespace, ve = JSON.stringify(F.clipboardValue(de), null, "  ");
            navigator.clipboard ? navigator.clipboard.writeText(ve).catch(function() {
              F.copyToClipboardFallback(ve);
            }) : F.copyToClipboardFallback(ve), F.copiedTimer = setTimeout(function() {
              F.setState({ copied: !1 });
            }, 5500), F.setState({ copied: !0 }, function() {
              typeof pe == "function" && pe({ src: de, namespace: ne, name: ne[ne.length - 1] });
            });
          }, F.getClippyIcon = function() {
            var ee = F.props.theme;
            return F.state.copied ? E().createElement("span", null, E().createElement(fn, Object.assign({ className: "copy-icon" }, Se(ee, "copy-icon"))), E().createElement("span", Se(ee, "copy-icon-copied"), "✔")) : E().createElement(fn, Object.assign({ className: "copy-icon" }, Se(ee, "copy-icon")));
          }, F.clipboardValue = function(ee) {
            switch (q(ee)) {
              case "function":
              case "regexp":
                return ee.toString();
              default:
                return ee;
            }
          }, F.state = { copied: !1 }, F;
        }
        return b(I, D), v(I, [{ key: "componentWillUnmount", value: function() {
          this.copiedTimer && (clearTimeout(this.copiedTimer), this.copiedTimer = null);
        } }, { key: "render", value: function() {
          var R = this.props, F = (R.src, R.theme), ee = R.hidden, pe = R.rowHovered, de = Se(F, "copy-to-clipboard").style, ne = "inline";
          return ee && (ne = "none"), E().createElement("span", { className: "copy-to-clipboard-container", title: "Copy to clipboard", style: { verticalAlign: "top", display: pe ? "inline-block" : "none" } }, E().createElement("span", { style: g(g({}, de), {}, { display: ne }), onClick: this.handleCopy }, this.getClippyIcon()));
        } }]);
      }(E().PureComponent);
      const Qr = function(D) {
        function I(R) {
          var F;
          return h(this, I), (F = w(this, I, [R])).getEditIcon = function() {
            var ee = F.props, pe = ee.variable, de = ee.theme;
            return E().createElement("div", { className: "click-to-edit", style: { verticalAlign: "top", display: F.state.hovered ? "inline-block" : "none" } }, E().createElement(rs, Object.assign({ className: "click-to-edit-icon" }, Se(de, "editVarIcon"), { onClick: function() {
              F.prepopInput(pe);
            } })));
          }, F.prepopInput = function(ee) {
            if (F.props.onEdit !== !1) {
              var pe = function(ne, ve) {
                var we;
                switch (q(ne, ve)) {
                  case "undefined":
                    we = "undefined";
                    break;
                  case "nan":
                    we = "NaN";
                    break;
                  case "string":
                    we = ne;
                    break;
                  case "bigNumber":
                  case "date":
                  case "function":
                  case "regexp":
                    we = ne.toString();
                    break;
                  default:
                    try {
                      we = JSON.stringify(ne, null, "  ");
                    } catch {
                      we = "";
                    }
                }
                return we;
              }(ee.value, F.props.bigNumber), de = dr(pe, F.props.bigNumber);
              F.setState({ editMode: !0, editValue: pe, parsedInput: { type: de.type, value: de.value } });
            }
          }, F.getRemoveIcon = function() {
            var ee = F.props, pe = ee.variable, de = ee.namespace, ne = ee.theme, ve = ee.rjvId;
            return E().createElement("div", { className: "click-to-remove", style: { verticalAlign: "top", display: F.state.hovered ? "inline-block" : "none" } }, E().createElement(Dl, Object.assign({ className: "click-to-remove-icon" }, Se(ne, "removeVarIcon"), { onClick: function() {
              Ye.dispatch({ name: "VARIABLE_REMOVED", rjvId: ve, data: { name: pe.name, namespace: de, existing_value: pe.value, variable_removed: !0 } });
            } })));
          }, F.getValue = function(ee, pe) {
            var de = !pe && ee.type, ne = F.props;
            switch (de) {
              case !1:
                return F.getEditInput();
              case "string":
                return E().createElement(Fe, Object.assign({ value: ee.value }, ne));
              case "integer":
                return E().createElement(Ge, Object.assign({ value: ee.value }, ne));
              case "float":
                return E().createElement(cn, Object.assign({ value: ee.value }, ne));
              case "boolean":
                return E().createElement(yt, Object.assign({ value: ee.value }, ne));
              case "function":
                return E().createElement(ae, Object.assign({ value: ee.value }, ne));
              case "null":
                return E().createElement(Re, ne);
              case "nan":
                return E().createElement(be, ne);
              case "undefined":
                return E().createElement(tt, ne);
              case "date":
                return E().createElement(Le, Object.assign({ value: ee.value }, ne));
              case "regexp":
                return E().createElement(Rt, Object.assign({ value: ee.value }, ne));
              case "bigNumber":
                return E().createElement(wt, Object.assign({ value: ee.value }, ne));
              default:
                return E().createElement("div", { className: "object-value" }, JSON.stringify(ee.value));
            }
          }, F.getEditInput = function() {
            var ee = F.props, pe = ee.keyModifier, de = ee.selectOnFocus, ne = ee.theme, ve = F.state.editValue;
            return E().createElement("div", null, E().createElement(cr, Object.assign({ type: "text", ref: function(we) {
              we && we[de ? "select" : "focus"]();
            }, value: ve, className: "variable-editor", onChange: function(we) {
              var Ae = we.target.value, je = dr(Ae, F.props.bigNumber);
              F.setState({ editValue: Ae, parsedInput: { type: je.type, value: je.value } });
            }, onKeyDown: function(we) {
              switch (we.key) {
                case "Escape":
                  F.setState({ editMode: !1, editValue: "" });
                  break;
                case "Enter":
                  pe(we, "submit") && F.submitEdit(!0);
              }
              we.stopPropagation();
            }, placeholder: "update this value", minRows: 2 }, Se(ne, "edit-input"))), E().createElement("div", Se(ne, "edit-icon-container"), E().createElement(Dl, Object.assign({ className: "edit-cancel" }, Se(ne, "cancel-icon"), { onClick: function(we) {
              we && we.stopPropagation(), F.setState({ editMode: !1, editValue: "" });
            } })), E().createElement(Ul, Object.assign({ className: "edit-check string-value" }, Se(ne, "check-icon"), { onClick: function(we) {
              we && we.stopPropagation(), F.submitEdit();
            } })), E().createElement("div", null, F.showDetected())));
          }, F.submitEdit = function(ee) {
            var pe = F.props, de = pe.variable, ne = pe.namespace, ve = pe.rjvId, we = pe.bigNumber, Ae = F.state, je = Ae.editValue, Be = Ae.parsedInput, Ie = je;
            ee && Be.type && (Ie = Be.value, we && Be.type === "bigNumber" && (Ie = new we(Ie))), F.setState({ editMode: !1 }), Ye.dispatch({ name: "VARIABLE_UPDATED", rjvId: ve, data: { name: de.name, namespace: ne, existing_value: de.value, new_value: Ie, variable_removed: !1 } });
          }, F.showDetected = function() {
            var ee = F.props, pe = ee.theme, de = (ee.variable, ee.namespace, ee.rjvId, F.state.parsedInput), ne = (de.type, de.value, F.getDetectedInput());
            if (ne) return E().createElement("div", null, E().createElement("div", Se(pe, "detected-row"), ne, E().createElement(Ul, { className: "edit-check detected", style: g({ verticalAlign: "top", paddingLeft: "3px" }, Se(pe, "check-icon").style), onClick: function(ve) {
              ve && ve.stopPropagation(), F.submitEdit(!0);
            } })));
          }, F.getDetectedInput = function() {
            var ee = F.state.parsedInput, pe = ee.type, de = ee.value, ne = F.props, ve = ne.theme;
            if (pe !== !1) switch (pe.toLowerCase()) {
              case "object":
                return E().createElement("span", null, E().createElement("span", { style: g(g({}, Se(ve, "brace").style), {}, { cursor: "default" }) }, "{"), E().createElement("span", { style: g(g({}, Se(ve, "ellipsis").style), {}, { cursor: "default" }) }, "..."), E().createElement("span", { style: g(g({}, Se(ve, "brace").style), {}, { cursor: "default" }) }, "}"));
              case "array":
                return E().createElement("span", null, E().createElement("span", { style: g(g({}, Se(ve, "brace").style), {}, { cursor: "default" }) }, "["), E().createElement("span", { style: g(g({}, Se(ve, "ellipsis").style), {}, { cursor: "default" }) }, "..."), E().createElement("span", { style: g(g({}, Se(ve, "brace").style), {}, { cursor: "default" }) }, "]"));
              case "string":
                return E().createElement(Fe, Object.assign({ value: de }, ne));
              case "integer":
                return E().createElement(Ge, Object.assign({ value: de }, ne));
              case "float":
                return E().createElement(cn, Object.assign({ value: de }, ne));
              case "boolean":
                return E().createElement(yt, Object.assign({ value: de }, ne));
              case "function":
                return E().createElement(ae, Object.assign({ value: de }, ne));
              case "null":
                return E().createElement(Re, ne);
              case "nan":
                return E().createElement(be, ne);
              case "undefined":
                return E().createElement(tt, ne);
              case "date":
                return E().createElement(Le, Object.assign({ value: new Date(de) }, ne));
              case "bignumber":
                return E().createElement(wt, Object.assign({ value: de }, ne));
            }
          }, F.state = { editMode: !1, editValue: "", hovered: !1, renameKey: !1, parsedInput: { type: !1, value: null } }, F;
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this, F = this.props, ee = F.variable, pe = F.singleIndent, de = F.type, ne = F.theme, ve = F.namespace, we = F.indentWidth, Ae = F.enableClipboard, je = F.onEdit, Be = F.onDelete, Ie = F.onSelect, ut = F.displayArrayKey, Ct = F.quotesOnKeys, Pt = F.keyModifier, Kt = this.state.editMode;
          return E().createElement("div", Object.assign({}, Se(ne, "objectKeyVal", { paddingLeft: we * pe }), { onMouseEnter: function() {
            return R.setState(g(g({}, R.state), {}, { hovered: !0 }));
          }, onMouseLeave: function() {
            return R.setState(g(g({}, R.state), {}, { hovered: !1 }));
          }, className: "variable-row", key: ee.name }), de == "array" ? ut ? E().createElement("span", Object.assign({}, Se(ne, "array-key"), { key: ee.name + "_" + ve }), ee.name, E().createElement("div", Se(ne, "colon"), ":")) : null : E().createElement("span", null, E().createElement("span", Object.assign({}, Se(ne, "object-name"), { className: "object-key", key: ee.name + "_" + ve }), !!Ct && E().createElement("span", { style: { verticalAlign: "top" } }, '"'), E().createElement("span", { style: { display: "inline-block" } }, Q(ee.name)), !!Ct && E().createElement("span", { style: { verticalAlign: "top" } }, '"')), E().createElement("span", Se(ne, "colon"), ":")), E().createElement("div", Object.assign({ className: "variable-value", onClick: Ie === !1 && je === !1 ? null : function(hr) {
            var Tn = ye(ve);
            Pt(hr, "edit") && je !== !1 ? R.prepopInput(ee) : Ie !== !1 && (Tn.shift(), Ie(g(g({}, ee), {}, { namespace: Tn })));
          } }, Se(ne, "variableValue", { cursor: Ie === !1 ? "default" : "pointer" })), this.getValue(ee, Kt)), Ae ? E().createElement(fa, { rowHovered: this.state.hovered, hidden: Kt, src: ee.value, clickCallback: Ae, theme: ne, namespace: [].concat(ye(ve), [ee.name]) }) : null, je !== !1 && Kt == 0 ? this.getEditIcon() : null, Be !== !1 && Kt == 0 ? this.getRemoveIcon() : null);
        } }]);
      }(E().PureComponent);
      var Dc = function(D) {
        function I() {
          var R;
          h(this, I);
          for (var F = arguments.length, ee = new Array(F), pe = 0; pe < F; pe++) ee[pe] = arguments[pe];
          return (R = w(this, I, [].concat(ee))).getObjectSize = function() {
            var de = R.props, ne = de.size, ve = de.theme;
            if (de.displayObjectSize) return E().createElement("span", Object.assign({ className: "object-size" }, Se(ve, "object-size")), ne, " item", ne === 1 ? "" : "s");
          }, R.getAddAttribute = function(de) {
            var ne = R.props, ve = ne.theme, we = ne.namespace, Ae = ne.name, je = ne.src, Be = ne.rjvId, Ie = ne.depth;
            return E().createElement("span", { className: "click-to-add", style: { verticalAlign: "top", display: de ? "inline-block" : "none" } }, E().createElement(zl, Object.assign({ className: "click-to-add-icon" }, Se(ve, "addVarIcon"), { onClick: function() {
              var ut = { name: Ie > 0 ? Ae : null, namespace: we.splice(0, we.length - 1), existing_value: je, variable_removed: !1, key_name: null };
              q(je) === "object" ? Ye.dispatch({ name: "ADD_VARIABLE_KEY_REQUEST", rjvId: Be, data: ut }) : Ye.dispatch({ name: "VARIABLE_ADDED", rjvId: Be, data: g(g({}, ut), {}, { new_value: [].concat(ye(je), [null]) }) });
            } })));
          }, R.getRemoveObject = function(de) {
            var ne = R.props, ve = ne.theme, we = (ne.hover, ne.namespace), Ae = ne.name, je = ne.src, Be = ne.rjvId;
            if (we.length !== 1) return E().createElement("span", { className: "click-to-remove", style: { display: de ? "inline-block" : "none" } }, E().createElement(Dl, Object.assign({ className: "click-to-remove-icon" }, Se(ve, "removeVarIcon"), { onClick: function() {
              Ye.dispatch({ name: "VARIABLE_REMOVED", rjvId: Be, data: { name: Ae, namespace: we.splice(0, we.length - 1), existing_value: je, variable_removed: !0 } });
            } })));
          }, R.render = function() {
            var de = R.props, ne = de.theme, ve = de.onDelete, we = de.onAdd, Ae = de.enableClipboard, je = de.src, Be = de.namespace, Ie = de.rowHovered;
            return E().createElement("div", Object.assign({}, Se(ne, "object-meta-data"), { className: "object-meta-data", onClick: function(ut) {
              ut.stopPropagation();
            } }), R.getObjectSize(), Ae ? E().createElement(fa, { rowHovered: Ie, clickCallback: Ae, src: je, theme: ne, namespace: Be }) : null, we !== !1 ? R.getAddAttribute(Ie) : null, ve !== !1 ? R.getRemoveObject(Ie) : null);
          }, R;
        }
        return b(I, D), v(I);
      }(E().PureComponent);
      function xr(D) {
        var I = D.parent_type, R = D.namespace, F = D.quotesOnKeys, ee = D.theme, pe = D.jsvRoot, de = D.name, ne = D.displayArrayKey, ve = D.name ? D.name : "";
        return !pe || de !== !1 && de !== null ? I == "array" ? ne ? E().createElement("span", Object.assign({}, Se(ee, "array-key"), { key: R }), E().createElement("span", { className: "array-key" }, ve), E().createElement("span", Se(ee, "colon"), ":")) : E().createElement("span", null) : E().createElement("span", Object.assign({}, Se(ee, "object-name"), { key: R }), E().createElement("span", { className: "object-key" }, F && E().createElement("span", { style: { verticalAlign: "top" } }, '"'), E().createElement("span", null, ve), F && E().createElement("span", { style: { verticalAlign: "top" } }, '"')), E().createElement("span", Se(ee, "colon"), ":")) : E().createElement("span", null);
      }
      function zc(D) {
        var I = D.theme;
        switch (D.iconStyle) {
          case "triangle":
            return E().createElement(Zp, Object.assign({}, Se(I, "expanded-icon"), { className: "expanded-icon" }));
          case "square":
            return E().createElement(da, Object.assign({}, Se(I, "expanded-icon"), { className: "expanded-icon" }));
          default:
            return E().createElement(vo, Object.assign({}, Se(I, "expanded-icon"), { className: "expanded-icon" }));
        }
      }
      function Bc(D) {
        var I = D.theme;
        switch (D.iconStyle) {
          case "triangle":
            return E().createElement(Xp, Object.assign({}, Se(I, "collapsed-icon"), { className: "collapsed-icon" }));
          case "square":
            return E().createElement(Lc, Object.assign({}, Se(I, "collapsed-icon"), { className: "collapsed-icon" }));
          default:
            return E().createElement(ns, Object.assign({}, Se(I, "collapsed-icon"), { className: "collapsed-icon" }));
        }
      }
      var eh = ["src", "groupArraysAfterLength", "depth", "name", "theme", "jsvRoot", "namespace", "parent_type"], pa = function(D) {
        function I(R) {
          var F;
          return h(this, I), (F = w(this, I, [R])).toggleCollapsed = function(ee) {
            var pe = [];
            for (var de in F.state.expanded) pe.push(F.state.expanded[de]);
            pe[ee] = !pe[ee], F.setState({ expanded: pe });
          }, F.state = { expanded: [] }, F;
        }
        return b(I, D), v(I, [{ key: "getExpandedIcon", value: function(R) {
          var F = this.props, ee = F.theme, pe = F.iconStyle;
          return this.state.expanded[R] ? E().createElement(zc, { theme: ee, iconStyle: pe }) : E().createElement(Bc, { theme: ee, iconStyle: pe });
        } }, { key: "render", value: function() {
          var R = this, F = this.props, ee = F.src, pe = F.groupArraysAfterLength, de = (F.depth, F.name), ne = F.theme, ve = F.jsvRoot, we = F.namespace, Ae = (F.parent_type, W(F, eh)), je = 0, Be = 5 * this.props.indentWidth;
          ve || (je = 5 * this.props.indentWidth);
          var Ie = pe, ut = Math.ceil(ee.length / Ie);
          return E().createElement("div", Object.assign({ className: "object-key-val" }, Se(ne, ve ? "jsv-root" : "objectKeyVal", { paddingLeft: je })), E().createElement(xr, this.props), E().createElement("span", null, E().createElement(Dc, Object.assign({ size: ee.length }, this.props))), ye(Array(ut)).map(function(Ct, Pt) {
            return E().createElement("div", Object.assign({ key: Pt, className: "object-key-val array-group" }, Se(ne, "objectKeyVal", { marginLeft: 6, paddingLeft: Be })), E().createElement("span", Se(ne, "brace-row"), E().createElement("div", Object.assign({ className: "icon-container" }, Se(ne, "icon-container"), { onClick: function(Kt) {
              R.toggleCollapsed(Pt);
            } }), R.getExpandedIcon(Pt)), R.state.expanded[Pt] ? E().createElement(as, Object.assign({ key: de + Pt, depth: 0, name: !1, collapsed: !1, groupArraysAfterLength: Ie, index_offset: Pt * Ie, src: ee.slice(Pt * Ie, Pt * Ie + Ie), namespace: we, type: "array", parent_type: "array_group", theme: ne }, Ae)) : E().createElement("span", Object.assign({}, Se(ne, "brace"), { onClick: function(Kt) {
              R.toggleCollapsed(Pt);
            }, className: "array-group-brace" }), "[", E().createElement("div", Object.assign({}, Se(ne, "array-group-meta-data"), { className: "array-group-meta-data" }), E().createElement("span", Object.assign({ className: "object-size" }, Se(ne, "object-size")), Pt * Ie, " - ", Pt * Ie + Ie > ee.length ? ee.length : Pt * Ie + Ie)), "]")));
          }));
        } }]);
      }(E().PureComponent), os = ["depth", "src", "namespace", "name", "type", "parent_type", "theme", "jsvRoot", "iconStyle"], ri = function(D) {
        function I(R) {
          var F;
          h(this, I), (F = w(this, I, [R])).toggleCollapsed = function() {
            F.setState({ expanded: !F.state.expanded }, function() {
              se.set(F.props.rjvId, F.props.namespace, "expanded", F.state.expanded);
            });
          }, F.getObjectContent = function(pe, de, ne) {
            return E().createElement("div", { className: "pushed-content object-container" }, E().createElement("div", Object.assign({ className: "object-content" }, Se(F.props.theme, "pushed-content")), F.renderObjectContents(de, ne)));
          }, F.getEllipsis = function() {
            return F.state.size === 0 ? null : E().createElement("div", Object.assign({}, Se(F.props.theme, "ellipsis"), { className: "node-ellipsis", onClick: F.toggleCollapsed }), "...");
          }, F.getObjectMetaData = function(pe) {
            var de = F.props, ne = (de.rjvId, de.theme, F.state), ve = ne.size, we = ne.hovered;
            return E().createElement(Dc, Object.assign({ rowHovered: we, size: ve }, F.props));
          }, F.renderObjectContents = function(pe, de) {
            var ne, ve = F.props, we = ve.depth, Ae = ve.parent_type, je = ve.index_offset, Be = ve.groupArraysAfterLength, Ie = ve.namespace, ut = F.state.object_type, Ct = [], Pt = Object.keys(pe || {});
            return F.props.sortKeys && ut !== "array" && (Pt = Pt.sort()), Pt.forEach(function(Kt) {
              if (ne = new is(Kt, pe[Kt], de.bigNumber), Ae === "array_group" && je && (ne.name = parseInt(ne.name) + je), Object.prototype.hasOwnProperty.call(pe, Kt)) if (ne.type === "object") Ct.push(E().createElement(as, Object.assign({ key: ne.name, depth: we + 1, name: ne.name, src: ne.value, namespace: Ie.concat(ne.name), parent_type: ut }, de)));
              else if (ne.type === "array") {
                var hr = as;
                Be && ne.value.length > Be && (hr = pa), Ct.push(E().createElement(hr, Object.assign({ key: ne.name, depth: we + 1, name: ne.name, src: ne.value, namespace: Ie.concat(ne.name), type: "array", parent_type: ut }, de)));
              } else Ct.push(E().createElement(Qr, Object.assign({ key: ne.name + "_" + Ie, variable: ne, singleIndent: 5, namespace: Ie, type: F.props.type }, de)));
            }), Ct;
          };
          var ee = I.getState(R);
          return F.state = g(g({}, ee), {}, { prevProps: {} }), F;
        }
        return b(I, D), v(I, [{ key: "getBraceStart", value: function(R, F) {
          var ee = this, pe = this.props, de = pe.src, ne = pe.theme, ve = pe.iconStyle;
          if (pe.parent_type === "array_group") return E().createElement("span", null, E().createElement("span", Se(ne, "brace"), R === "array" ? "[" : "{"), F ? this.getObjectMetaData(de) : null);
          var we = F ? zc : Bc;
          return E().createElement("span", null, E().createElement("span", Object.assign({ onClick: function(Ae) {
            ee.toggleCollapsed();
          } }, Se(ne, "brace-row")), E().createElement("div", Object.assign({ className: "icon-container" }, Se(ne, "icon-container")), E().createElement(we, { theme: ne, iconStyle: ve })), E().createElement(xr, this.props), E().createElement("span", Se(ne, "brace"), R === "array" ? "[" : "{")), F ? this.getObjectMetaData(de) : null);
        } }, { key: "render", value: function() {
          var R = this, F = this.props, ee = F.depth, pe = F.src, de = (F.namespace, F.name, F.type, F.parent_type), ne = F.theme, ve = F.jsvRoot, we = F.iconStyle, Ae = W(F, os), je = this.state, Be = je.object_type, Ie = je.expanded, ut = {};
          return ve || de === "array_group" ? de === "array_group" && (ut.borderLeft = 0, ut.display = "inline") : ut.paddingLeft = 5 * this.props.indentWidth, E().createElement("div", Object.assign({ className: "object-key-val", onMouseEnter: function() {
            return R.setState(g(g({}, R.state), {}, { hovered: !0 }));
          }, onMouseLeave: function() {
            return R.setState(g(g({}, R.state), {}, { hovered: !1 }));
          } }, Se(ne, ve ? "jsv-root" : "objectKeyVal", ut)), this.getBraceStart(Be, Ie), Ie ? this.getObjectContent(ee, pe, g({ theme: ne, iconStyle: we }, Ae)) : this.getEllipsis(), E().createElement("span", { className: "brace-row" }, E().createElement("span", { style: g(g({}, Se(ne, "brace").style), {}, { paddingLeft: Ie ? "3px" : "0px" }) }, Be === "array" ? "]" : "}"), Ie ? null : this.getObjectMetaData(pe)));
        } }], [{ key: "getDerivedStateFromProps", value: function(R, F) {
          var ee = F.prevProps;
          return R.src !== ee.src || R.collapsed !== ee.collapsed || R.name !== ee.name || R.namespace !== ee.namespace || R.rjvId !== ee.rjvId ? g(g({}, I.getState(R)), {}, { prevProps: R }) : null;
        } }]);
      }(E().PureComponent);
      ri.getState = function(D) {
        var I = Object.keys(D.src).length, R = (D.collapsed === !1 || D.collapsed !== !0 && D.collapsed > D.depth) && (!D.shouldCollapse || D.shouldCollapse({ name: D.name, src: D.src, type: q(D.src), namespace: D.namespace }) === !1) && I !== 0;
        return { expanded: se.get(D.rjvId, D.namespace, "expanded", R), object_type: D.type === "array" ? "array" : "object", parent_type: D.type === "array" ? "array" : "object", size: I, hovered: !1 };
      };
      var is = v(function D(I, R, F) {
        h(this, D), this.name = I, this.value = R, this.type = q(R, F);
      });
      O(ri);
      const as = ri;
      var th = function(D) {
        function I() {
          var R;
          h(this, I);
          for (var F = arguments.length, ee = new Array(F), pe = 0; pe < F; pe++) ee[pe] = arguments[pe];
          return (R = w(this, I, [].concat(ee))).render = function() {
            var de, ne, ve, we, Ae = R.props, je = [Ae.name], Be = as;
            return typeof Ae.name != "object" || Array.isArray(Ae.name) || (je = [((de = Ae.name) === null || de === void 0 ? void 0 : de.displayName) || ((ne = Ae.name) === null || ne === void 0 ? void 0 : ne.name) || ((ve = Ae.name) === null || ve === void 0 || (we = ve.type) === null || we === void 0 ? void 0 : we.name) || "Anonymous"]), Array.isArray(Ae.src) && Ae.groupArraysAfterLength && Ae.src.length > Ae.groupArraysAfterLength && (Be = pa), E().createElement("div", { className: "pretty-json-container object-container" }, E().createElement("div", { className: "object-content" }, E().createElement(Be, Object.assign({ namespace: je, depth: 0, jsvRoot: !0 }, Ae))));
          }, R;
        }
        return b(I, D), v(I);
      }(E().PureComponent), Vl = function(D) {
        function I(R) {
          var F;
          return h(this, I), (F = w(this, I, [R])).closeModal = function() {
            Ye.dispatch({ rjvId: F.props.rjvId, name: "RESET" });
          }, F.submit = function() {
            F.props.submit(F.state.input);
          }, F.state = { input: R.input ? R.input : "" }, F;
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this, F = this.props, ee = F.theme, pe = F.rjvId, de = F.isValid, ne = this.state.input, ve = de(ne);
          return E().createElement("div", Object.assign({ className: "key-modal-request" }, Se(ee, "key-modal-request"), { onClick: this.closeModal }), E().createElement("div", Object.assign({}, Se(ee, "key-modal"), { onClick: function(we) {
            we.stopPropagation();
          } }), E().createElement("div", Se(ee, "key-modal-label"), "Key Name:"), E().createElement("div", { style: { position: "relative" } }, E().createElement("input", Object.assign({}, Se(ee, "key-modal-input"), { className: "key-modal-input", ref: function(we) {
            return we && we.focus();
          }, spellCheck: !1, value: ne, placeholder: "...", onChange: function(we) {
            R.setState({ input: we.target.value });
          }, onKeyPress: function(we) {
            ve && we.key === "Enter" ? R.submit() : we.key === "Escape" && R.closeModal();
          } })), ve ? E().createElement(Ul, Object.assign({}, Se(ee, "key-modal-submit"), { className: "key-modal-submit", onClick: function(we) {
            return R.submit();
          } })) : null), E().createElement("span", Se(ee, "key-modal-cancel"), E().createElement(Bl, Object.assign({}, Se(ee, "key-modal-cancel-icon"), { className: "key-modal-cancel", onClick: function() {
            Ye.dispatch({ rjvId: pe, name: "RESET" });
          } })))));
        } }]);
      }(E().PureComponent), Uc = function(D) {
        function I() {
          var R;
          h(this, I);
          for (var F = arguments.length, ee = new Array(F), pe = 0; pe < F; pe++) ee[pe] = arguments[pe];
          return (R = w(this, I, [].concat(ee))).isValid = function(de) {
            var ne = R.props.rjvId, ve = se.get(ne, "action", "new-key-request");
            return de != "" && Object.keys(ve.existing_value).indexOf(de) === -1;
          }, R.submit = function(de) {
            var ne = R.props.rjvId, ve = se.get(ne, "action", "new-key-request");
            ve.new_value = g({}, ve.existing_value), ve.new_value[de] = R.props.defaultValue, Ye.dispatch({ name: "VARIABLE_ADDED", rjvId: ne, data: ve });
          }, R;
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = R.active, ee = R.theme, pe = R.rjvId;
          return F ? E().createElement(Vl, { rjvId: pe, theme: ee, isValid: this.isValid, submit: this.submit }) : null;
        } }]);
      }(E().PureComponent), Wl = function(D) {
        function I() {
          return h(this, I), w(this, I, arguments);
        }
        return b(I, D), v(I, [{ key: "render", value: function() {
          var R = this.props, F = R.message, ee = R.active, pe = R.theme, de = R.rjvId;
          return ee ? E().createElement("div", Object.assign({ className: "validation-failure" }, Se(pe, "validation-failure"), { onClick: function() {
            Ye.dispatch({ rjvId: de, name: "RESET" });
          } }), E().createElement("span", Se(pe, "validation-failure-label"), F), E().createElement(Bl, Se(pe, "validation-failure-clear"))) : null;
        } }]);
      }(E().PureComponent), To = function(D) {
        function I(R) {
          var F;
          return h(this, I), (F = w(this, I, [R])).rjvId = Date.now().toString() + Math.random().toString(36).slice(2), F.getListeners = function() {
            return { reset: F.resetState, "variable-update": F.updateSrc, "add-key-request": F.addKeyRequest };
          }, F.updateSrc = function() {
            var ee, pe = se.get(F.rjvId, "action", "variable-update"), de = pe.name, ne = pe.namespace, ve = pe.new_value, we = pe.existing_value, Ae = pe.updated_src, je = pe.type, Be = F.props, Ie = Be.onEdit, ut = Be.onDelete, Ct = Be.onAdd, Pt = { existing_src: F.state.src, new_value: ve, updated_src: Ae, name: de, namespace: ne, existing_value: we };
            switch (je) {
              case "variable-added":
                ee = Ct(Pt);
                break;
              case "variable-edited":
                ee = Ie(Pt);
                break;
              case "variable-removed":
                ee = ut(Pt);
            }
            ee !== !1 ? (se.set(F.rjvId, "global", "src", Ae), F.setState({ src: Ae })) : F.setState({ validationFailure: !0 });
          }, F.addKeyRequest = function() {
            F.setState({ addKeyRequest: !0 });
          }, F.resetState = function() {
            F.setState({ validationFailure: !1, addKeyRequest: !1 });
          }, F.state = { addKeyRequest: !1, editKeyRequest: !1, validationFailure: !1, src: I.defaultProps.src, name: I.defaultProps.name, theme: I.defaultProps.theme, validationMessage: I.defaultProps.validationMessage, prevSrc: I.defaultProps.src, prevName: I.defaultProps.name, prevTheme: I.defaultProps.theme }, F;
        }
        return b(I, D), v(I, [{ key: "componentDidMount", value: function() {
          se.set(this.rjvId, "global", "src", this.state.src);
          var R = this.getListeners();
          for (var F in R) se.on(F + "-" + this.rjvId, R[F]);
          this.setState({ addKeyRequest: !1, editKeyRequest: !1 });
        } }, { key: "componentDidUpdate", value: function(R, F) {
          F.addKeyRequest !== !1 && this.setState({ addKeyRequest: !1 }), F.editKeyRequest !== !1 && this.setState({ editKeyRequest: !1 }), R.src !== this.state.src && se.set(this.rjvId, "global", "src", this.state.src);
        } }, { key: "componentWillUnmount", value: function() {
          var R = this.getListeners();
          for (var F in R) se.removeListener(F + "-" + this.rjvId, R[F]);
        } }, { key: "render", value: function() {
          var R = this.state, F = R.validationFailure, ee = R.validationMessage, pe = R.addKeyRequest, de = R.theme, ne = R.src, ve = R.name, we = this.props, Ae = we.style, je = we.defaultValue;
          return E().createElement("div", { className: "react-json-view", style: g(g({}, Se(de, "app-container").style), Ae) }, E().createElement(Wl, { message: ee, active: F, theme: de, rjvId: this.rjvId }), E().createElement(th, Object.assign({}, this.props, { src: ne, name: ve, theme: de, type: q(ne), rjvId: this.rjvId })), E().createElement(Uc, { active: pe, theme: de, rjvId: this.rjvId, defaultValue: je }));
        } }], [{ key: "getDerivedStateFromProps", value: function(R, F) {
          if (R.src !== F.prevSrc || R.name !== F.prevName || R.theme !== F.prevTheme) {
            var ee = { src: R.src, name: R.name, theme: R.theme, validationMessage: R.validationMessage, prevSrc: R.src, prevName: R.name, prevTheme: R.theme };
            return I.validateState(ee);
          }
          return null;
        } }]);
      }(E().PureComponent);
      To.defaultProps = { src: {}, name: "root", theme: "rjv-default", collapsed: !1, collapseStringsAfterLength: !1, shouldCollapse: !1, sortKeys: !1, quotesOnKeys: !0, groupArraysAfterLength: 100, indentWidth: 4, enableClipboard: !0, escapeStrings: !0, displayObjectSize: !0, displayDataTypes: !0, onEdit: !1, onDelete: !1, onAdd: !1, onSelect: !1, iconStyle: "triangle", style: {}, validationMessage: "Validation Error", defaultValue: null, displayArrayKey: !0, selectOnFocus: !1, keyModifier: function(D) {
        return D.metaKey || D.ctrlKey;
      }, bigNumber: null }, To.validateState = function(D) {
        var I = {};
        return q(D.theme) !== "object" || function(R) {
          var F = ["base00", "base01", "base02", "base03", "base04", "base05", "base06", "base07", "base08", "base09", "base0A", "base0B", "base0C", "base0D", "base0E", "base0F"];
          if (q(R) === "object") {
            for (var ee = 0; ee < F.length; ee++) if (!(F[ee] in R)) return !1;
            return !0;
          }
          return !1;
        }(D.theme) || (console.error("react-json-view error:", "theme prop must be a theme name or valid base-16 theme object.", 'defaulting to "rjv-default" theme'), I.theme = "rjv-default"), q(D.src) !== "object" && q(D.src) !== "array" && (console.error("react-json-view error:", "src property must be a valid json object"), I.name = "ERROR", I.src = { message: "src property must be a valid json object" }), g(g({}, D), I);
      }, O(To);
      const nh = To;
    })(), l;
  })());
})(z_);
var Yoe = z_.exports;
const Qoe = /* @__PURE__ */ Ft(Yoe);
function Xoe({
  content: e,
  description: t,
  renderStringAsHtml: n = !1
}) {
  const o = n ? ["string", "object"] : ["object"], i = (s) => !!(typeof s == "string" && (s.startsWith("http://") || s.startsWith("https://")) && URL.canParse(s) && !/\s/g.test(s));
  return /* @__PURE__ */ A.jsxs(A.Fragment, { children: [
    typeof e == "string" && n && /* @__PURE__ */ A.jsx("div", { dangerouslySetInnerHTML: { __html: e } }),
    typeof e == "object" && e != null && /* @__PURE__ */ A.jsx(Qoe, { src: e }),
    e != null && !o.includes(typeof e) && /* @__PURE__ */ A.jsxs(A.Fragment, { children: [
      /* @__PURE__ */ A.jsx("div", { children: t || "Output" }),
      i(e) && /* @__PURE__ */ A.jsx("a", { href: e, target: "_blank", children: e }) || /* @__PURE__ */ A.jsx("pre", { children: JSON.stringify(e, null, "    ") })
    ] })
  ] });
}
function Zoe(e) {
  return ht("MuiLinearProgress", e);
}
dt("MuiLinearProgress", ["root", "colorPrimary", "colorSecondary", "determinate", "indeterminate", "buffer", "query", "dashed", "dashedColorPrimary", "dashedColorSecondary", "bar", "barColorPrimary", "barColorSecondary", "bar1Indeterminate", "bar1Determinate", "bar1Buffer", "bar2Indeterminate", "bar2Buffer"]);
const Qg = 4, Xg = ua`
  0% {
    left: -35%;
    right: 100%;
  }

  60% {
    left: 100%;
    right: -90%;
  }

  100% {
    left: 100%;
    right: -90%;
  }
`, eie = typeof Xg != "string" ? jl`
        animation: ${Xg} 2.1s cubic-bezier(0.65, 0.815, 0.735, 0.395) infinite;
      ` : null, Zg = ua`
  0% {
    left: -200%;
    right: 100%;
  }

  60% {
    left: 107%;
    right: -8%;
  }

  100% {
    left: 107%;
    right: -8%;
  }
`, tie = typeof Zg != "string" ? jl`
        animation: ${Zg} 2.1s cubic-bezier(0.165, 0.84, 0.44, 1) 1.15s infinite;
      ` : null, ey = ua`
  0% {
    opacity: 1;
    background-position: 0 -23px;
  }

  60% {
    opacity: 0;
    background-position: 0 -23px;
  }

  100% {
    opacity: 1;
    background-position: -200px -23px;
  }
`, nie = typeof ey != "string" ? jl`
        animation: ${ey} 3s infinite linear;
      ` : null, rie = (e) => {
  const {
    classes: t,
    variant: n,
    color: o
  } = e, i = {
    root: ["root", `color${Ne(o)}`, n],
    dashed: ["dashed", `dashedColor${Ne(o)}`],
    bar1: ["bar", `barColor${Ne(o)}`, (n === "indeterminate" || n === "query") && "bar1Indeterminate", n === "determinate" && "bar1Determinate", n === "buffer" && "bar1Buffer"],
    bar2: ["bar", n !== "buffer" && `barColor${Ne(o)}`, n === "buffer" && `color${Ne(o)}`, (n === "indeterminate" || n === "query") && "bar2Indeterminate", n === "buffer" && "bar2Buffer"]
  };
  return mt(i, Zoe, t);
}, Cv = (e, t) => e.vars ? e.vars.palette.LinearProgress[`${t}Bg`] : e.palette.mode === "light" ? cl(e.palette[t].main, 0.62) : ul(e.palette[t].main, 0.5), oie = Oe("span", {
  name: "MuiLinearProgress",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, t[`color${Ne(n.color)}`], t[n.variant]];
  }
})(ct(({
  theme: e
}) => ({
  position: "relative",
  overflow: "hidden",
  display: "block",
  height: 4,
  // Fix Safari's bug during composition of different paint.
  zIndex: 0,
  "@media print": {
    colorAdjust: "exact"
  },
  variants: [...Object.entries(e.palette).filter(kn()).map(([t]) => ({
    props: {
      color: t
    },
    style: {
      backgroundColor: Cv(e, t)
    }
  })), {
    props: ({
      ownerState: t
    }) => t.color === "inherit" && t.variant !== "buffer",
    style: {
      "&::before": {
        content: '""',
        position: "absolute",
        left: 0,
        top: 0,
        right: 0,
        bottom: 0,
        backgroundColor: "currentColor",
        opacity: 0.3
      }
    }
  }, {
    props: {
      variant: "buffer"
    },
    style: {
      backgroundColor: "transparent"
    }
  }, {
    props: {
      variant: "query"
    },
    style: {
      transform: "rotate(180deg)"
    }
  }]
}))), iie = Oe("span", {
  name: "MuiLinearProgress",
  slot: "Dashed",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.dashed, t[`dashedColor${Ne(n.color)}`]];
  }
})(ct(({
  theme: e
}) => ({
  position: "absolute",
  marginTop: 0,
  height: "100%",
  width: "100%",
  backgroundSize: "10px 10px",
  backgroundPosition: "0 -23px",
  variants: [{
    props: {
      color: "inherit"
    },
    style: {
      opacity: 0.3,
      backgroundImage: "radial-gradient(currentColor 0%, currentColor 16%, transparent 42%)"
    }
  }, ...Object.entries(e.palette).filter(kn()).map(([t]) => {
    const n = Cv(e, t);
    return {
      props: {
        color: t
      },
      style: {
        backgroundImage: `radial-gradient(${n} 0%, ${n} 16%, transparent 42%)`
      }
    };
  })]
})), nie || {
  // At runtime for Pigment CSS, `bufferAnimation` will be null and the generated keyframe will be used.
  animation: `${ey} 3s infinite linear`
}), aie = Oe("span", {
  name: "MuiLinearProgress",
  slot: "Bar1",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.bar, t[`barColor${Ne(n.color)}`], (n.variant === "indeterminate" || n.variant === "query") && t.bar1Indeterminate, n.variant === "determinate" && t.bar1Determinate, n.variant === "buffer" && t.bar1Buffer];
  }
})(ct(({
  theme: e
}) => ({
  width: "100%",
  position: "absolute",
  left: 0,
  bottom: 0,
  top: 0,
  transition: "transform 0.2s linear",
  transformOrigin: "left",
  variants: [{
    props: {
      color: "inherit"
    },
    style: {
      backgroundColor: "currentColor"
    }
  }, ...Object.entries(e.palette).filter(kn()).map(([t]) => ({
    props: {
      color: t
    },
    style: {
      backgroundColor: (e.vars || e).palette[t].main
    }
  })), {
    props: {
      variant: "determinate"
    },
    style: {
      transition: `transform .${Qg}s linear`
    }
  }, {
    props: {
      variant: "buffer"
    },
    style: {
      zIndex: 1,
      transition: `transform .${Qg}s linear`
    }
  }, {
    props: ({
      ownerState: t
    }) => t.variant === "indeterminate" || t.variant === "query",
    style: {
      width: "auto"
    }
  }, {
    props: ({
      ownerState: t
    }) => t.variant === "indeterminate" || t.variant === "query",
    style: eie || {
      animation: `${Xg} 2.1s cubic-bezier(0.65, 0.815, 0.735, 0.395) infinite`
    }
  }]
}))), sie = Oe("span", {
  name: "MuiLinearProgress",
  slot: "Bar2",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.bar, t[`barColor${Ne(n.color)}`], (n.variant === "indeterminate" || n.variant === "query") && t.bar2Indeterminate, n.variant === "buffer" && t.bar2Buffer];
  }
})(ct(({
  theme: e
}) => ({
  width: "100%",
  position: "absolute",
  left: 0,
  bottom: 0,
  top: 0,
  transition: "transform 0.2s linear",
  transformOrigin: "left",
  variants: [...Object.entries(e.palette).filter(kn()).map(([t]) => ({
    props: {
      color: t
    },
    style: {
      "--LinearProgressBar2-barColor": (e.vars || e).palette[t].main
    }
  })), {
    props: ({
      ownerState: t
    }) => t.variant !== "buffer" && t.color !== "inherit",
    style: {
      backgroundColor: "var(--LinearProgressBar2-barColor, currentColor)"
    }
  }, {
    props: ({
      ownerState: t
    }) => t.variant !== "buffer" && t.color === "inherit",
    style: {
      backgroundColor: "currentColor"
    }
  }, {
    props: {
      color: "inherit"
    },
    style: {
      opacity: 0.3
    }
  }, ...Object.entries(e.palette).filter(kn()).map(([t]) => ({
    props: {
      color: t,
      variant: "buffer"
    },
    style: {
      backgroundColor: Cv(e, t),
      transition: `transform .${Qg}s linear`
    }
  })), {
    props: ({
      ownerState: t
    }) => t.variant === "indeterminate" || t.variant === "query",
    style: {
      width: "auto"
    }
  }, {
    props: ({
      ownerState: t
    }) => t.variant === "indeterminate" || t.variant === "query",
    style: tie || {
      animation: `${Zg} 2.1s cubic-bezier(0.165, 0.84, 0.44, 1) 1.15s infinite`
    }
  }]
}))), B_ = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiLinearProgress"
  }), {
    className: i,
    color: s = "primary",
    value: l,
    valueBuffer: u,
    variant: c = "indeterminate",
    ...f
  } = o, p = {
    ...o,
    color: s,
    variant: c
  }, g = rie(p), h = g0(), y = {}, v = {
    bar1: {},
    bar2: {}
  };
  if ((c === "determinate" || c === "buffer") && l !== void 0) {
    y["aria-valuenow"] = Math.round(l), y["aria-valuemin"] = 0, y["aria-valuemax"] = 100;
    let C = l - 100;
    h && (C = -C), v.bar1.transform = `translateX(${C}%)`;
  }
  if (c === "buffer" && u !== void 0) {
    let C = (u || 0) - 100;
    h && (C = -C), v.bar2.transform = `translateX(${C}%)`;
  }
  return /* @__PURE__ */ A.jsxs(oie, {
    className: Ve(g.root, i),
    ownerState: p,
    role: "progressbar",
    ...y,
    ref: n,
    ...f,
    children: [c === "buffer" ? /* @__PURE__ */ A.jsx(iie, {
      className: g.dashed,
      ownerState: p
    }) : null, /* @__PURE__ */ A.jsx(aie, {
      className: g.bar1,
      ownerState: p,
      style: v.bar1
    }), c === "determinate" ? null : /* @__PURE__ */ A.jsx(sie, {
      className: g.bar2,
      ownerState: p,
      style: v.bar2
    })]
  });
});
function lie(e) {
  return ht("MuiCircularProgress", e);
}
dt("MuiCircularProgress", ["root", "determinate", "indeterminate", "colorPrimary", "colorSecondary", "svg", "circle", "circleDeterminate", "circleIndeterminate", "circleDisableShrink"]);
const Gi = 44, ty = ua`
  0% {
    transform: rotate(0deg);
  }

  100% {
    transform: rotate(360deg);
  }
`, ny = ua`
  0% {
    stroke-dasharray: 1px, 200px;
    stroke-dashoffset: 0;
  }

  50% {
    stroke-dasharray: 100px, 200px;
    stroke-dashoffset: -15px;
  }

  100% {
    stroke-dasharray: 100px, 200px;
    stroke-dashoffset: -125px;
  }
`, uie = typeof ty != "string" ? jl`
        animation: ${ty} 1.4s linear infinite;
      ` : null, cie = typeof ny != "string" ? jl`
        animation: ${ny} 1.4s ease-in-out infinite;
      ` : null, die = (e) => {
  const {
    classes: t,
    variant: n,
    color: o,
    disableShrink: i
  } = e, s = {
    root: ["root", n, `color${Ne(o)}`],
    svg: ["svg"],
    circle: ["circle", `circle${Ne(n)}`, i && "circleDisableShrink"]
  };
  return mt(s, lie, t);
}, fie = Oe("span", {
  name: "MuiCircularProgress",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, t[n.variant], t[`color${Ne(n.color)}`]];
  }
})(ct(({
  theme: e
}) => ({
  display: "inline-block",
  variants: [{
    props: {
      variant: "determinate"
    },
    style: {
      transition: e.transitions.create("transform")
    }
  }, {
    props: {
      variant: "indeterminate"
    },
    style: uie || {
      animation: `${ty} 1.4s linear infinite`
    }
  }, ...Object.entries(e.palette).filter(kn()).map(([t]) => ({
    props: {
      color: t
    },
    style: {
      color: (e.vars || e).palette[t].main
    }
  }))]
}))), pie = Oe("svg", {
  name: "MuiCircularProgress",
  slot: "Svg",
  overridesResolver: (e, t) => t.svg
})({
  display: "block"
  // Keeps the progress centered
}), hie = Oe("circle", {
  name: "MuiCircularProgress",
  slot: "Circle",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.circle, t[`circle${Ne(n.variant)}`], n.disableShrink && t.circleDisableShrink];
  }
})(ct(({
  theme: e
}) => ({
  stroke: "currentColor",
  variants: [{
    props: {
      variant: "determinate"
    },
    style: {
      transition: e.transitions.create("stroke-dashoffset")
    }
  }, {
    props: {
      variant: "indeterminate"
    },
    style: {
      // Some default value that looks fine waiting for the animation to kicks in.
      strokeDasharray: "80px, 200px",
      strokeDashoffset: 0
      // Add the unit to fix a Edge 16 and below bug.
    }
  }, {
    props: ({
      ownerState: t
    }) => t.variant === "indeterminate" && !t.disableShrink,
    style: cie || {
      // At runtime for Pigment CSS, `bufferAnimation` will be null and the generated keyframe will be used.
      animation: `${ny} 1.4s ease-in-out infinite`
    }
  }]
}))), U_ = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiCircularProgress"
  }), {
    className: i,
    color: s = "primary",
    disableShrink: l = !1,
    size: u = 40,
    style: c,
    thickness: f = 3.6,
    value: p = 0,
    variant: g = "indeterminate",
    ...h
  } = o, y = {
    ...o,
    color: s,
    disableShrink: l,
    size: u,
    thickness: f,
    value: p,
    variant: g
  }, v = die(y), C = {}, $ = {}, S = {};
  if (g === "determinate") {
    const w = 2 * Math.PI * ((Gi - f) / 2);
    C.strokeDasharray = w.toFixed(3), S["aria-valuenow"] = Math.round(p), C.strokeDashoffset = `${((100 - p) / 100 * w).toFixed(3)}px`, $.transform = "rotate(-90deg)";
  }
  return /* @__PURE__ */ A.jsx(fie, {
    className: Ve(v.root, i),
    style: {
      width: u,
      height: u,
      ...$,
      ...c
    },
    ownerState: y,
    ref: n,
    role: "progressbar",
    ...S,
    ...h,
    children: /* @__PURE__ */ A.jsx(pie, {
      className: v.svg,
      ownerState: y,
      viewBox: `${Gi / 2} ${Gi / 2} ${Gi} ${Gi}`,
      children: /* @__PURE__ */ A.jsx(hie, {
        className: v.circle,
        style: C,
        ownerState: y,
        cx: Gi,
        cy: Gi,
        r: (Gi - f) / 2,
        fill: "none",
        strokeWidth: f
      })
    })
  });
});
var mie = Object.defineProperty, gie = (e, t, n) => t in e ? mie(e, t, { enumerable: !0, configurable: !0, writable: !0, value: n }) : e[t] = n, Ze = (e, t, n) => gie(e, typeof t != "symbol" ? t + "" : t, n), Uo = {}, df = {}, n$ = {}, r$;
function V_() {
  return r$ || (r$ = 1, function(e) {
    Object.defineProperty(e, "__esModule", { value: !0 }), function(t) {
      t[t.DISABLE = 0] = "DISABLE", t[t.ENABLE = 1] = "ENABLE", t[t.LOGGING = 2] = "LOGGING";
    }(e.OperationMode || (e.OperationMode = {})), function(t) {
      t[t.CASE_STRICT = 1] = "CASE_STRICT", t[t.CASE_INSENSITIVE = 2] = "CASE_INSENSITIVE";
    }(e.PropertyMatchingRule || (e.PropertyMatchingRule = {})), function(t) {
      t[t.ALLOW_NULL = 1] = "ALLOW_NULL", t[t.ALLOW_OBJECT_NULL = 2] = "ALLOW_OBJECT_NULL", t[t.DISALLOW_NULL = 3] = "DISALLOW_NULL";
    }(e.ValueCheckingMode || (e.ValueCheckingMode = {}));
  }(n$)), n$;
}
var Ru = {}, o$;
function W_() {
  if (o$) return Ru;
  o$ = 1, Object.defineProperty(Ru, "__esModule", { value: !0 });
  var e = (
    /** @class */
    function() {
      function n() {
      }
      return n.MAPPING_PROPERTY = "__jsonconvert__mapping__", n.MAPPER_PROPERTY = "__jsonconvert__mapper__", n.CLASS_IDENTIFIER = "__jsonconvert__class_identifier__", n;
    }()
  );
  Ru.Settings = e;
  var t = (
    /** @class */
    /* @__PURE__ */ function() {
      function n() {
        this.classPropertyName = "", this.jsonPropertyName = "", this.expectedJsonType = void 0, this.isOptional = !1, this.customConverter = null;
      }
      return n;
    }()
  );
  return Ru.MappingOptions = t, Ru;
}
var ff = {}, i$;
function xv() {
  if (i$) return ff;
  i$ = 1, Object.defineProperty(ff, "__esModule", { value: !0 });
  var e = (
    /** @class */
    /* @__PURE__ */ function() {
      function t() {
      }
      return t;
    }()
  );
  return ff.Any = e, ff;
}
var a$;
function yie() {
  if (a$) return df;
  a$ = 1, Object.defineProperty(df, "__esModule", { value: !0 });
  var e = V_(), t = W_(), n = xv(), o = (
    /** @class */
    function() {
      function i(s, l, u, c) {
        this._operationMode = e.OperationMode.ENABLE, this._valueCheckingMode = e.ValueCheckingMode.ALLOW_OBJECT_NULL, this._ignorePrimitiveChecks = !1, this._propertyMatchingRule = e.PropertyMatchingRule.CASE_STRICT, this._ignoreRequiredCheck = !1, s !== void 0 && s in e.OperationMode && (this.operationMode = s), l !== void 0 && l in e.ValueCheckingMode && (this.valueCheckingMode = l), u !== void 0 && (this.ignorePrimitiveChecks = u), c !== void 0 && (this.propertyMatchingRule = c);
      }
      return Object.defineProperty(i.prototype, "operationMode", {
        /**
         * Determines how the JsonConvert class instance should operate.
         *
         * You may assign three different values:
         * - OperationMode.DISABLE: json2typescript will be disabled, no type checking or mapping is done
         * - OperationMode.ENABLE: json2typescript is enabled, but only errors are logged
         * - OperationMode.LOGGING: json2typescript is enabled and detailed information is logged
         *
         * @see https://www.npmjs.com/package/json2typescript full documentation
         */
        get: function() {
          return this._operationMode;
        },
        /**
         * Determines how the JsonConvert class instance should operate.
         *
         * You may assign three different values:
         * - OperationMode.DISABLE: json2typescript will be disabled, no type checking or mapping is done
         * - OperationMode.ENABLE: json2typescript is enabled, but only errors are logged
         * - OperationMode.LOGGING: json2typescript is enabled and detailed information is logged
         *
         * @see https://www.npmjs.com/package/json2typescript full documentation
         */
        set: function(s) {
          s in e.OperationMode && (this._operationMode = s);
        },
        enumerable: !0,
        configurable: !0
      }), Object.defineProperty(i.prototype, "valueCheckingMode", {
        /**
         * Determines which types are allowed to be null.
         *
         * You may assign three different values:
         * - ValueCheckingMode.ALLOW_NULL: all given values in the JSON are allowed to be null
         * - ValueCheckingMode.ALLOW_OBJECT_NULL: objects in the JSON are allowed to be null, primitive types are not allowed to be null
         * - ValueCheckingMode.DISALLOW_NULL: no null values are tolerated in the JSON
         *
         * @see https://www.npmjs.com/package/json2typescript full documentation
         */
        get: function() {
          return this._valueCheckingMode;
        },
        /**
         * Determines which types are allowed to be null.
         *
         * You may assign three different values:
         * - ValueCheckingMode.ALLOW_NULL: all given values in the JSON are allowed to be null
         * - ValueCheckingMode.ALLOW_OBJECT_NULL: objects in the JSON are allowed to be null, primitive types are not allowed to be null
         * - ValueCheckingMode.DISALLOW_NULL: no null values are tolerated in the JSON
         *
         * @see https://www.npmjs.com/package/json2typescript full documentation
         */
        set: function(s) {
          s in e.ValueCheckingMode && (this._valueCheckingMode = s);
        },
        enumerable: !0,
        configurable: !0
      }), Object.defineProperty(i.prototype, "ignorePrimitiveChecks", {
        /**
         * Determines whether primitive types should be checked.
         * If true, it will be allowed to assign primitive to other primitive types.
         *
         * @see https://www.npmjs.com/package/json2typescript full documentation
         */
        get: function() {
          return this._ignorePrimitiveChecks;
        },
        /**
         * Determines whether primitive types should be checked.
         * If true, it will be allowed to assign primitive to other primitive types.
         *
         * @see https://www.npmjs.com/package/json2typescript full documentation
         */
        set: function(s) {
          this._ignorePrimitiveChecks = s;
        },
        enumerable: !0,
        configurable: !0
      }), Object.defineProperty(i.prototype, "propertyMatchingRule", {
        /**
         * Determines the rule of how JSON properties shall be matched with class properties during deserialization.
         *
         * You may assign the following values:
         * - PropertyMatchingRule.CASE_STRICT: JSON properties need to match exactly the names in the decorators
         * - PropertyMatchingRule.CASE_INSENSITIVE: JSON properties need to match names in the decorators, but names they are not case sensitive
         *
         * @see https://www.npmjs.com/package/json2typescript full documentation
         */
        get: function() {
          return this._propertyMatchingRule;
        },
        /**
         * Determines the rule of how JSON properties shall be matched with class properties during deserialization.
         *
         * You may assign the following values:
         * - PropertyMatchingRule.CASE_STRICT: JSON properties need to match exactly the names in the decorators
         * - PropertyMatchingRule.CASE_INSENSITIVE: JSON properties need to match names in the decorators, but names they are not case sensitive
         *
         * @see https://www.npmjs.com/package/json2typescript full documentation
         */
        set: function(s) {
          s in e.PropertyMatchingRule && (this._propertyMatchingRule = s);
        },
        enumerable: !0,
        configurable: !0
      }), Object.defineProperty(i.prototype, "ignoreRequiredCheck", {
        /**
         * Determines whether the check for "required" properties should be ignored, making all
         * mapped values optional, whether or not the isOptional property mapping parameter is set.
         * If true, any missing properties (undefined) when serializing or deserializing will be
         * ignored, as if they were marked optional.
         *
         * @see https://www.npmjs.com/package/json2typescript full documentation
         */
        get: function() {
          return this._ignoreRequiredCheck;
        },
        /**
         * Determines whether the check for "required" properties should be ignored, making all
         * mapped values optional, whether or not the isOptional property mapping parameter is set.
         * If true, any missing properties (undefined) when serializing or deserializing will be
         * ignored, as if they were marked optional.
         *
         * @see https://www.npmjs.com/package/json2typescript full documentation
         */
        set: function(s) {
          this._ignoreRequiredCheck = s;
        },
        enumerable: !0,
        configurable: !0
      }), i.prototype.serialize = function(s, l) {
        if (this.operationMode === e.OperationMode.DISABLE)
          return s;
        if (s instanceof Array)
          return this.serializeArray(s, l);
        if (typeof s == "object")
          return this.serializeObject(s, l);
        throw new Error(`Fatal error in JsonConvert. Passed parameter data in JsonConvert.serialize() is not in valid format (object or array).
`);
      }, i.prototype.serializeObject = function(s, l) {
        if (this.operationMode === e.OperationMode.DISABLE)
          return s;
        if (s === void 0)
          throw new Error(`Fatal error in JsonConvert. Passed parameter instance in JsonConvert.serializeObject() is undefined. This is not a valid JSON format.
`);
        if (s === null) {
          if (this.valueCheckingMode === e.ValueCheckingMode.DISALLOW_NULL)
            throw new Error(`Fatal error in JsonConvert. Passed parameter instance in JsonConvert.serializeObject() is undefined. You have specified to disallow null values.
`);
          return s;
        } else if (typeof s != "object" || s instanceof Array)
          throw new Error(`Fatal error in JsonConvert. Passed parameter instance in JsonConvert.serializeObject() is not of type object.
`);
        this.operationMode === e.OperationMode.LOGGING && (console.log("----------"), console.log("Receiving JavaScript instance:"), console.log(s));
        var u = {}, c;
        l ? c = new l() : c = s;
        for (var f = 0, p = Object.keys(c); f < p.length; f++) {
          var g = p[f];
          try {
            this.serializeObject_loopProperty(s, c, g, u);
          } catch (h) {
            throw this.operationMode === e.OperationMode.LOGGING && (console.log("Failed to serialize property:"), console.log(h), console.log("----------")), h;
          }
        }
        return this.operationMode === e.OperationMode.LOGGING && (console.log("Returning JSON object:"), console.log(u), console.log("----------")), u;
      }, i.prototype.serializeArray = function(s, l) {
        if (this.operationMode === e.OperationMode.DISABLE)
          return s;
        if (s === void 0)
          throw new Error(`Fatal error in JsonConvert. Passed parameter instanceArray in JsonConvert.serializeArray() is undefined. This is not a valid JSON format.
`);
        if (s === null) {
          if (this.valueCheckingMode === e.ValueCheckingMode.DISALLOW_NULL)
            throw new Error(`Fatal error in JsonConvert. Passed parameter instanceArray in JsonConvert.serializeArray() is undefined. You have specified to disallow null values.
`);
          return s;
        } else if (typeof s != "object" || !(s instanceof Array))
          throw new Error(`Fatal error in JsonConvert. Passed parameter instanceArray in JsonConvert.serializeArray() is not of type array.
`);
        this.operationMode === e.OperationMode.LOGGING && (console.log("----------"), console.log("Receiving JavaScript array:"), console.log(s));
        for (var u = [], c = 0, f = s; c < f.length; c++) {
          var p = f[c];
          u.push(this.serializeObject(p, l));
        }
        return this.operationMode === e.OperationMode.LOGGING && (console.log("Returning JSON array:"), console.log(u), console.log("----------")), u;
      }, i.prototype.deserialize = function(s, l) {
        if (this.operationMode === e.OperationMode.DISABLE)
          return s;
        if (s instanceof Array)
          return this.deserializeArray(s, l);
        if (typeof s == "object")
          return this.deserializeObject(s, l);
        throw new Error(`Fatal error in JsonConvert. Passed parameter json in JsonConvert.deserialize() is not in valid JSON format (object or array).
`);
      }, i.prototype.deserializeObject = function(s, l) {
        if (this.operationMode === e.OperationMode.DISABLE)
          return s;
        if (s === void 0)
          throw new Error(`Fatal error in JsonConvert. Passed parameter jsonObject in JsonConvert.deserializeObject() is undefined. This is not a valid JSON format.
`);
        if (s === null) {
          if (this.valueCheckingMode === e.ValueCheckingMode.DISALLOW_NULL)
            throw new Error(`Fatal error in JsonConvert. Passed parameter jsonObject in JsonConvert.deserializeObject() is undefined. You have specified to disallow null values.
`);
          return s;
        } else if (typeof s != "object" || s instanceof Array)
          throw new Error(`Fatal error in JsonConvert. Passed parameter jsonObject in JsonConvert.deserializeObject() is not of type object.
`);
        this.operationMode === e.OperationMode.LOGGING && (console.log("----------"), console.log("Receiving JSON object:"), console.log(s));
        for (var u = new l(), c = 0, f = Object.keys(u); c < f.length; c++) {
          var p = f[c];
          try {
            this.deserializeObject_loopProperty(u, p, s);
          } catch (g) {
            throw this.operationMode === e.OperationMode.LOGGING && (console.log("Failed to deserialize property:"), console.log(g), console.log("----------")), g;
          }
        }
        return this.operationMode === e.OperationMode.LOGGING && (console.log("Returning CLASS instance:"), console.log(u), console.log("----------")), u;
      }, i.prototype.deserializeArray = function(s, l) {
        if (this.operationMode === e.OperationMode.DISABLE)
          return s;
        if (s === void 0)
          throw new Error(`Fatal error in JsonConvert. Passed parameter jsonArray in JsonConvert.deserializeObject() is undefined. This is not a valid JSON format.
`);
        if (s === null) {
          if (this.valueCheckingMode === e.ValueCheckingMode.DISALLOW_NULL)
            throw new Error(`Fatal error in JsonConvert. Passed parameter jsonArray in JsonConvert.deserializeObject() is undefined. You have specified to disallow null values.
`);
          return s;
        } else if (typeof s != "object" || !(s instanceof Array))
          throw new Error(`Fatal error in JsonConvert. Passed parameter jsonArray in JsonConvert.deserializeArray() is not of type array.
`);
        if (this.operationMode === e.OperationMode.DISABLE)
          return s;
        this.operationMode === e.OperationMode.LOGGING && (console.log("----------"), console.log("Receiving JSON array:"), console.log(s));
        for (var u = [], c = 0, f = s; c < f.length; c++) {
          var p = f[c];
          u.push(this.deserializeObject(p, l));
        }
        return this.operationMode === e.OperationMode.LOGGING && (console.log("Returning array of CLASS instances:"), console.log(u), console.log("----------")), u;
      }, i.prototype.serializeObject_loopProperty = function(s, l, u, c) {
        var f = this.getClassPropertyMappingOptions(l, u);
        if (f !== null) {
          var p = f.jsonPropertyName, g = f.expectedJsonType, h = f.isOptional, y = f.customConverter, v = s[u];
          if (typeof v > "u") {
            if (h || this._ignoreRequiredCheck)
              return;
            throw new Error('Fatal error in JsonConvert. Failed to map the JavaScript instance of class "' + l[t.Settings.CLASS_IDENTIFIER] + '" to JSON because the defined class property "' + u + `" does not exist or is not defined:

	Class property: 
		` + u + `

	JSON property: 
		` + p + `

`);
          }
          if (!(h && v === null))
            try {
              c[p] = y !== null ? y.serialize(v) : this.verifyProperty(g, v, !0);
            } catch (C) {
              throw new Error('Fatal error in JsonConvert. Failed to map the JavaScript instance of class "' + l[t.Settings.CLASS_IDENTIFIER] + `" to JSON because of a type error.

	Class property: 
		` + u + `

	Class property value: 
		` + v + `

	Expected type: 
		` + this.getExpectedType(g) + `

	Runtime type: 
		` + this.getTrueType(v) + `

	JSON property: 
		` + p + `

` + C.message + `
`);
            }
        }
      }, i.prototype.deserializeObject_loopProperty = function(s, l, u) {
        var c = this.getClassPropertyMappingOptions(s, l);
        if (c !== null) {
          var f = c.jsonPropertyName, p = c.expectedJsonType, g = c.isOptional, h = c.customConverter, y = void 0;
          try {
            y = this.getObjectValue(u, f);
          } catch {
          }
          if (typeof y > "u") {
            if (g || this._ignoreRequiredCheck)
              return;
            throw new Error('Fatal error in JsonConvert. Failed to map the JSON object to the class "' + s[t.Settings.CLASS_IDENTIFIER] + '" because the defined JSON property "' + f + `" does not exist:

	Class property: 
		` + l + `

	JSON property: 
		` + f + `

`);
          }
          if (!(g && y === null))
            try {
              s[l] = h !== null ? h.deserialize(y) : this.verifyProperty(p, y);
            } catch (v) {
              throw new Error('Fatal error in JsonConvert. Failed to map the JSON object to the class "' + s[t.Settings.CLASS_IDENTIFIER] + `" because of a type error.

	Class property: 
		` + l + `

	Expected type: 
		` + this.getExpectedType(p) + `

	JSON property: 
		` + f + `

	JSON type: 
		` + this.getJsonType(y) + `

	JSON value: 
		` + JSON.stringify(y) + `

` + v.message + `
`);
            }
        }
      }, i.prototype.getClassPropertyMappingOptions = function(s, l) {
        var u = s[t.Settings.MAPPING_PROPERTY];
        if (typeof u > "u")
          return null;
        for (var c = Object.getPrototypeOf(s); c != null; ) {
          var f = c[t.Settings.CLASS_IDENTIFIER];
          if (f) {
            var p = f + "." + l;
            if (typeof u[p] < "u")
              return u[p];
          }
          c = Object.getPrototypeOf(c);
        }
        return null;
      }, i.prototype.verifyProperty = function(s, l, u) {
        if (s === n.Any || s === null || s === Object)
          return l;
        if (!(s instanceof Array) && !(l instanceof Array))
          if (typeof s < "u" && s.prototype.hasOwnProperty(t.Settings.CLASS_IDENTIFIER)) {
            if (l === null) {
              if (this.valueCheckingMode !== e.ValueCheckingMode.DISALLOW_NULL)
                return null;
              throw new Error("	Reason: Given value is null.");
            }
            return u ? this.serializeObject(l, s) : this.deserializeObject(l, s);
          } else if (s === n.Any || s === null || s === Object) {
            if (l === null) {
              if (this.valueCheckingMode !== e.ValueCheckingMode.DISALLOW_NULL)
                return null;
              throw new Error("	Reason: Given value is null.");
            }
            return l;
          } else if (s === String || s === Number || s === Boolean) {
            if (l === null) {
              if (this.valueCheckingMode === e.ValueCheckingMode.ALLOW_NULL)
                return null;
              throw new Error("	Reason: Given value is null.");
            }
            if (
              // primitive types match
              s === String && typeof l == "string" || s === Number && typeof l == "number" || s === Boolean && typeof l == "boolean" || this.ignorePrimitiveChecks
            )
              return l;
            throw new Error("	Reason: Given object does not match the expected primitive type.");
          } else
            throw new Error(`	Reason: Expected type is unknown. There might be multiple reasons for this:
	- You are missing the decorator @JsonObject (for object mapping)
	- You are missing the decorator @JsonConverter (for custom mapping) before your class definition
	- Your given class is undefined in the decorator because of circular dependencies`);
        if (s instanceof Array && l instanceof Array) {
          var c = [];
          if (l.length === 0)
            return c;
          if (s.length === 0)
            return l;
          for (var f = s.length < l.length, p = 0; p < l.length; p++)
            f && p >= s.length && (s[p] = s[p - 1]), c[p] = this.verifyProperty(s[p], l[p], u);
          return c;
        }
        if (s instanceof Array && l instanceof Object) {
          var c = [];
          if (l.length === 0)
            return c;
          if (s.length === 0)
            return l;
          var f = s.length < Object.keys(l).length, p = 0;
          for (var g in l)
            f && p >= s.length && (s[p] = s[p - 1]), c[g] = this.verifyProperty(s[p], l[g]), p++;
          return c;
        }
        if (s instanceof Array) {
          if (l === null) {
            if (this.valueCheckingMode !== e.ValueCheckingMode.DISALLOW_NULL)
              return null;
            throw new Error("	Reason: Given value is null.");
          }
          throw new Error("	Reason: Expected type is array, but given value is non-array.");
        }
        throw l instanceof Array ? new Error("	Reason: Given value is array, but expected a non-array type.") : new Error("	Reason: Mapping failed because of an unknown error.");
      }, i.prototype.getObjectValue = function(s, l) {
        if (this.propertyMatchingRule === e.PropertyMatchingRule.CASE_INSENSITIVE) {
          var u = Object.keys(s).reduce(function(c, f) {
            return c[f.toLowerCase()] = f, c;
          }, {});
          l = u[l.toLowerCase()];
        }
        if (!(l in s))
          throw new Error();
        return s[l];
      }, i.prototype.getExpectedType = function(s) {
        var l = "";
        if (s instanceof Array) {
          l = "[";
          for (var u = 0; u < s.length; u++)
            u > 0 && (l += ","), l += this.getExpectedType(s[u]);
          return l += "]", l;
        } else
          return s === n.Any || s === null || s === Object ? "any" : s === String || s === Boolean || s === Number ? new s().constructor.name.toLowerCase() : typeof s == "function" ? new s().constructor.name : s === void 0 ? "undefined" : "?????";
      }, i.prototype.getJsonType = function(s) {
        if (s === null)
          return "null";
        var l = "";
        if (s instanceof Array) {
          l = "[";
          for (var u = 0; u < s.length; u++)
            u > 0 && (l += ","), l += this.getJsonType(s[u]);
          return l += "]", l;
        } else
          return typeof s;
      }, i.prototype.getTrueType = function(s) {
        return typeof s;
      }, i;
    }()
  );
  return df.JsonConvert = o, df;
}
var Us = {}, s$;
function vie() {
  if (s$) return Us;
  s$ = 1, Object.defineProperty(Us, "__esModule", { value: !0 });
  var e = W_(), t = xv();
  function n(s) {
    s[e.Settings.MAPPER_PROPERTY] = "";
  }
  Us.JsonConverter = n;
  function o(s) {
    var l = "", u = function(f) {
      f.prototype[e.Settings.CLASS_IDENTIFIER] = l.length > 0 ? l : f.name;
      var p = f.prototype[e.Settings.MAPPING_PROPERTY];
      if (p)
        for (var g = Object.keys(p).filter(function(C) {
          return C.indexOf(e.Settings.CLASS_IDENTIFIER + ".") === 0;
        }), h = 0, y = g; h < y.length; h++) {
          var v = y[h];
          p[v.replace(e.Settings.CLASS_IDENTIFIER, f.prototype[e.Settings.CLASS_IDENTIFIER])] = p[v], delete p[v];
        }
    }, c = typeof s;
    switch (c) {
      case "string":
        return l = s, u;
      case "function":
      case "undefined":
      default:
        throw new Error(`Fatal error in JsonConvert. It is mandatory to pass a string as parameter in the @JsonObject decorator.

Use @JsonObject(classId) where classId is a string.

`);
    }
  }
  Us.JsonObject = o;
  function i() {
    for (var s = [], l = 0; l < arguments.length; l++)
      s[l] = arguments[l];
    return function(u, c) {
      var f = c, p = t.Any, g = !1;
      switch (s.length) {
        case 1:
          if (s[0] === void 0)
            throw new Error(`Fatal error in JsonConvert. It is not allowed to explicitly pass "undefined" as first parameter in the @JsonProperty decorator.

	Class property: 
		` + c + `

Leave the decorator parameters empty if you do not wish to pass the first parameter.

`);
          f = s[0];
          break;
        case 2:
          if (s[0] === void 0)
            throw new Error(`Fatal error in JsonConvert. It is not allowed to explicitly pass "undefined" as first parameter in the @JsonProperty decorator.

	Class property: 
		` + c + `

Leave the decorator parameters empty if you do not wish to pass the first parameter.

`);
          if (s[1] === void 0)
            throw new Error(`Fatal error in JsonConvert. It is not allowed to explicitly pass "undefined" as second parameter in the @JsonProperty decorator.

	Class property: 
		` + c + `

Use "Any" to allow any type. You can import this class from "json2typescript".

`);
          f = s[0], p = s[1];
          break;
        case 3:
          if (s[0] === void 0)
            throw new Error(`Fatal error in JsonConvert. It is not allowed to explicitly pass "undefined" as first parameter in the @JsonProperty decorator.

	Class property: 
		` + c + `

Leave the decorator parameters empty if you do not wish to pass the first parameter.

`);
          if (s[1] === void 0)
            throw new Error(`Fatal error in JsonConvert. It is not allowed to explicitly pass "undefined" as second parameter in the @JsonProperty decorator.

	Class property: 
		` + c + `

Use "Any" to allow any type. You can import this class from "json2typescript".

`);
          f = s[0], p = s[1], g = s[2];
          break;
      }
      typeof u[e.Settings.MAPPING_PROPERTY] > "u" && (u[e.Settings.MAPPING_PROPERTY] = []);
      var h = new e.MappingOptions();
      if (h.classPropertyName = c, h.jsonPropertyName = f, h.isOptional = g || !1, typeof p < "u" && p !== null && typeof p[e.Settings.MAPPER_PROPERTY] < "u" ? h.customConverter = new p() : h.expectedJsonType = p, typeof u[e.Settings.MAPPING_PROPERTY][e.Settings.CLASS_IDENTIFIER + "." + c] > "u")
        u[e.Settings.MAPPING_PROPERTY][e.Settings.CLASS_IDENTIFIER + "." + c] = h;
      else
        throw new Error(`Fatal error in JsonConvert. It is not allowed to add multiple decorators for the same property.

	Class property: 
		` + c + `

`);
    };
  }
  return Us.JsonProperty = i, Us;
}
var l$;
function bie() {
  if (l$) return Uo;
  l$ = 1, Object.defineProperty(Uo, "__esModule", { value: !0 });
  var e = yie();
  Uo.JsonConvert = e.JsonConvert;
  var t = V_();
  Uo.ValueCheckingMode = t.ValueCheckingMode, Uo.OperationMode = t.OperationMode;
  var n = vie();
  Uo.JsonObject = n.JsonObject, Uo.JsonProperty = n.JsonProperty, Uo.JsonConverter = n.JsonConverter;
  var o = xv();
  return Uo.Any = o.Any, Uo;
}
var kt = bie(), Sie = Object.defineProperty, wie = Object.getOwnPropertyDescriptor, It = (e, t, n, o) => {
  for (var i = o > 1 ? void 0 : o ? wie(t, n) : t, s = e.length - 1, l; s >= 0; s--)
    (l = e[s]) && (i = (o ? l(t, n, i) : l(i)) || i);
  return o && i && Sie(t, n, i), i;
}, Ks = /* @__PURE__ */ ((e) => (e.Ping = "ping", e.Pong = "pong", e.Info = "info", e.ApiInfo = "api_info", e.Request = "request", e.Response = "response", e.Log = "log", e.Progress = "progress", e))(Ks || {}), ry = /* @__PURE__ */ ((e) => (e.ResponseTopic = "response_topic", e.RequestContext = "requestContext", e.RequestMessageId = "requestMessageId", e.MessageType = "messageType", e.SourceTopic = "source_topic", e.Fragment = "fragment", e.NumFragments = "num_fragments", e.Status = "status", e))(ry || {}), tl = /* @__PURE__ */ ((e) => (e.Success = "success", e.Error = "error", e.Running = "running", e))(tl || {});
let Rr = class {
  constructor() {
    Ze(this, "payload", ""), Ze(this, "context", ""), Ze(this, "properties");
  }
  static createMessage(e) {
    const t = new Rr();
    return t.properties = {}, t.properties.messageType = e, t;
  }
  /** Shortcut to create a request message
   *
   * This static method creates a :class:`DASFModuleRequest` with the
   * :ref:`MessageType` `MessageType.Request`
   *
   * @param data - Optional data as javascript object that will be
   * json-serialized and added as :attr:`payload` to the request.
   *
   * @returns The :class:`DASFModuleRequest` of type `MessageType.Request`
   */
  static createRequestMessage(e) {
    const t = Rr.createMessage(
      "request"
      /* Request */
    );
    return typeof e < "u" && (t.payload = btoa(JSON.stringify(e))), t;
  }
  /** Shortcut to create a Ping message
   *
   * This static method creates a :class:`DASFModuleRequest` with the
   * :ref:`MessageType` `MessageType.Ping`
   *
   * @returns The :class:`DASFModuleRequest` of type `MessageType.Ping`
   */
  static createPingMessage() {
    return Rr.createMessage(
      "ping"
      /* Ping */
    );
  }
  /** Shortcut to create a Pong message
   *
   * This static method creates a :class:`DASFModuleRequest` with the
   * :ref:`MessageType` `MessageType.Pong`
   *
   * @returns The :class:`DASFModuleRequest` of type `MessageType.Pong`
   */
  static createPongMessage() {
    return Rr.createMessage(
      "pong"
      /* Pong */
    );
  }
  /** Shortcut to create an Info message
   *
   * This static method creates a :class:`DASFModuleRequest` with the
   * :ref:`MessageType` `MessageType.Info` to get the information on the
   * backend module, see also :meth:`DASFConnection.getModuleInfo`
   *
   * @returns The :class:`DASFModuleRequest` of type `MessageType.Info`
   */
  static createInfoMessage() {
    return Rr.createMessage(
      "info"
      /* Info */
    );
  }
  /** Shortcut to create an Info message
   *
   * This static method creates a :class:`DASFModuleRequest` with the
   * :ref:`MessageType` `MessageType.Info` to get the information on the
   * backend module, see also :meth:`DASFConnection.getApiInfo`
   *
   * @returns The :class:`DASFModuleRequest` of type `MessageType.ApiInfo`
   */
  static createApiInfoMessage() {
    return Rr.createMessage(
      "api_info"
      /* ApiInfo */
    );
  }
  /** Shortcut to the the messagetype of the properties */
  getMessageType() {
    if (this.properties)
      return this.properties.messageType;
    throw new Error("Missing or empty message type property");
  }
};
It([
  kt.JsonProperty("payload", String)
], Rr.prototype, "payload", 2);
It([
  kt.JsonProperty("properties", kt.Any)
], Rr.prototype, "properties", 2);
Rr = It([
  kt.JsonObject("DASFModuleRequest")
], Rr);
let sa = class {
  constructor() {
    Ze(this, "result", ""), Ze(this, "messageId", ""), Ze(this, "errorMsg", ""), Ze(this, "context", "");
  }
  isOk() {
    return this.result == "ok";
  }
};
It([
  kt.JsonProperty("result", String)
], sa.prototype, "result", 2);
It([
  kt.JsonProperty("messageId", String, !0)
], sa.prototype, "messageId", 2);
It([
  kt.JsonProperty("errorMsg", String, !0)
], sa.prototype, "errorMsg", 2);
It([
  kt.JsonProperty("context", String, !0)
], sa.prototype, "context", 2);
sa = It([
  kt.JsonObject("DASFModuleRequestReceipt")
], sa);
let Xf = class {
  constructor(e) {
    Ze(this, "messageId", ""), e && (this.messageId = e);
  }
};
It([
  kt.JsonProperty("messageId", String)
], Xf.prototype, "messageId", 2);
Xf = It([
  kt.JsonObject("DASFAcknowledgment")
], Xf);
let Ka = class {
  constructor() {
    Ze(this, "messageId", ""), Ze(this, "payload", ""), Ze(this, "publishTime", ""), Ze(this, "properties");
  }
  /** Shortcut to the the messagetype of the properties */
  getMessageType() {
    if (this.properties) {
      const e = this.properties.messageType;
      if (e && e.trim().length > 0)
        return e;
    }
    throw new Error("Missing or empty message type property");
  }
  /** Shortcut to the the request context of the properties */
  getRequestContext() {
    if (this.properties) {
      const e = this.properties.requestContext;
      if (e)
        return e;
    }
    throw new Error("Missing or empty request context property");
  }
  /** Get the original id of the request */
  getRequestMessageId() {
    if (this.properties) {
      const e = this.properties.requestMessageId;
      if (e)
        return e;
    }
    throw new Error("Missing or empty request messageId property");
  }
  /** Check if the response is fragmented or not. */
  isFragmented() {
    return this.properties != null && Object.prototype.hasOwnProperty.call(
      this.properties,
      "num_fragments"
      /* NumFragments */
    ) && Object.prototype.hasOwnProperty.call(
      this.properties,
      "fragment"
      /* Fragment */
    );
  }
  /** Get the id of this fragment. */
  getFragmentId() {
    if (this.properties && this.isFragmented())
      return this.properties.fragment;
    throw new Error(
      "Unable to request fragment id from unfragmented response."
    );
  }
  /** Get the total number of fragments */
  getNumberOfFragments() {
    if (this.properties && this.isFragmented())
      return this.properties.num_fragments;
    throw new Error(
      "Unable to request number of fragments from unfragmented response."
    );
  }
};
It([
  kt.JsonProperty("messageId", String)
], Ka.prototype, "messageId", 2);
It([
  kt.JsonProperty("payload", String)
], Ka.prototype, "payload", 2);
It([
  kt.JsonProperty("publishTime", String)
], Ka.prototype, "publishTime", 2);
It([
  kt.JsonProperty("properties")
], Ka.prototype, "properties", 2);
Ka = It([
  kt.JsonObject("DASFModuleResponse")
], Ka);
let Zo = class {
  constructor() {
    Ze(this, "report_type", ""), Ze(this, "report_id", ""), Ze(this, "step_message", ""), Ze(this, "steps", 0), Ze(this, "status", "running"), Ze(this, "children");
  }
  hasError() {
    return this.status == "error";
  }
  hasSuccess() {
    return this.status == "success";
  }
  isRunning() {
    return this.status == "running";
  }
  isComplete() {
    return !this.isRunning();
  }
};
It([
  kt.JsonProperty("report_type", String)
], Zo.prototype, "report_type", 2);
It([
  kt.JsonProperty("report_id", String)
], Zo.prototype, "report_id", 2);
It([
  kt.JsonProperty("step_message", String)
], Zo.prototype, "step_message", 2);
It([
  kt.JsonProperty("steps", Number)
], Zo.prototype, "steps", 2);
It([
  kt.JsonProperty("status", String)
], Zo.prototype, "status", 2);
It([
  kt.JsonProperty("children", [Zo])
], Zo.prototype, "children", 2);
Zo = It([
  kt.JsonObject("DASFProgressReport")
], Zo);
let ml = class {
  // dummy JsonSchemaConverter to make sure the json2typescript library does
  // not complain
  serialize(e) {
    return JSON.stringify(e);
  }
  deserialize(e) {
    return e;
  }
};
ml = It([
  kt.JsonConverter
], ml);
let Ha = class {
  constructor() {
    Ze(this, "name", ""), Ze(this, "rpcSchema", {}), Ze(this, "returnSchema", {});
  }
};
It([
  kt.JsonProperty("name", String)
], Ha.prototype, "name", 2);
It([
  kt.JsonProperty("rpc_schema", ml)
], Ha.prototype, "rpcSchema", 2);
It([
  kt.JsonProperty("return_schema", ml)
], Ha.prototype, "returnSchema", 2);
Ha = It([
  kt.JsonObject("FunctionApiInfo")
], Ha);
let Ga = class {
  constructor() {
    Ze(this, "name", ""), Ze(this, "rpcSchema", {}), Ze(this, "methods", []);
  }
};
It([
  kt.JsonProperty("name", String)
], Ga.prototype, "name", 2);
It([
  kt.JsonProperty("rpc_schema", ml)
], Ga.prototype, "rpcSchema", 2);
It([
  kt.JsonProperty("methods", [Ha])
], Ga.prototype, "methods", 2);
Ga = It([
  kt.JsonObject("ClassApiInfo")
], Ga);
let Ja = class {
  constructor() {
    Ze(this, "classes", []), Ze(this, "functions", []), Ze(this, "rpcSchema", {});
  }
};
It([
  kt.JsonProperty("classes", [Ga])
], Ja.prototype, "classes", 2);
It([
  kt.JsonProperty("functions", [Ha])
], Ja.prototype, "functions", 2);
It([
  kt.JsonProperty("rpc_schema", ml)
], Ja.prototype, "rpcSchema", 2);
Ja = It([
  kt.JsonObject("ModuleApiInfo")
], Ja);
class $ie {
  /**
   * @param urlBuilder - A builder for the consumer and producer websocket url
   * @param onConnectionError - A callback for the case when no connection could be established
   */
  constructor(t, n) {
    Ze(this, "outgoingChannel"), Ze(this, "incomingChannel"), Ze(this, "DASFProducerURL"), Ze(this, "DASFConsumerURL"), Ze(this, "consumeTopic"), Ze(this, "jsonCoder"), Ze(this, "contextCounter", 0), Ze(this, "responseResolvers", /* @__PURE__ */ new Map()), Ze(this, "progressCallbacks", /* @__PURE__ */ new Map()), Ze(this, "responseRejectHandler", /* @__PURE__ */ new Map()), Ze(this, "finishedContexts", /* @__PURE__ */ new Set()), Ze(this, "contextResponseFragments", /* @__PURE__ */ new Map()), Ze(this, "pendingRequests", []), Ze(this, "waitingForPong", !1), Ze(this, "connectionTimeout", 1e4), Ze(this, "connectionError", ""), Ze(this, "onConnectionError"), Ze(this, "startedReconnectOutgoing", !0), Ze(this, "startedReconnectIncoming", !0), this.onConnectionError = n, this.consumeTopic = t.consumeTopic, this.DASFProducerURL = t.DASFProducerURL, this.DASFConsumerURL = t.DASFConsumerURL, this.jsonCoder = new kt.JsonConvert(), this.connect(), setInterval(() => this.checkConnection(), this.connectionTimeout);
  }
  connect() {
    this.startedReconnectIncoming = !0, this.startedReconnectOutgoing = !0, this.outgoingChannel = new WebSocket(this.DASFProducerURL), this.incomingChannel = new WebSocket(this.DASFConsumerURL), this.registerListener();
  }
  isConnected() {
    return this.outgoingChannel && this.incomingChannel && this.outgoingChannel.readyState == WebSocket.OPEN && this.incomingChannel.readyState == WebSocket.OPEN;
  }
  isConnecting() {
    return this.outgoingChannel && this.incomingChannel && this.outgoingChannel.readyState != WebSocket.OPEN && this.incomingChannel.readyState != WebSocket.OPEN;
  }
  /**
   * Do something as soon as the websocket connection has been established
   *
   * This property returns a promise that resolves with the established
   * connection. You can use this to do something right after the connection has
   * been created.
   *
   * ```
   * import { DASFConnection, WebsocketUrlBuilder } from '@dasf/dasf-messaging'
   * const connection = new DASFConnection(
   *     new WebsocketUrlBuilder(
   *         'ws://localhost:8080/ws',
   *         'some-topic'
   *     )
   * )
   *
   * // log to the console when the connection has been connected
   * connection.connected.then(
   *     (connection: DASFConnection) => console.log("Websocket connection has been established.")
   * ).catch(
   *     (error: Error) => console.log(error)
   * )
   * ```
   *
   *
   */
  get connected() {
    const t = (n, o) => {
      this.hasConnectionError() ? o(new Error(this.connectionError)) : this.assureConnected() ? n(this) : setTimeout(() => t(n, o), 100);
    };
    return new Promise(t);
  }
  assureConnected() {
    return this.isConnected() ? !0 : (this.startedReconnectIncoming || this.startedReconnectOutgoing || this.isConnecting() || this.connect(), !1);
  }
  hasPendingRequests() {
    return this.pendingRequests.length > 0;
  }
  sendPendingRequests() {
    if (this.hasPendingRequests()) {
      const t = this.pendingRequests.shift();
      this.outgoingChannel.send(JSON.stringify(t)), setTimeout(() => this.sendPendingRequests(), 200);
    }
  }
  registerListener() {
    const t = (n) => {
      this.startedReconnectOutgoing = !1, this.startedReconnectIncoming = !1, !this.hasConnectionError() && this.isConnecting() && (this.connectionError = "Connection to " + n.target.url + " failed.", this.onConnectionError && this.onConnectionError(this.connectionError));
    };
    this.outgoingChannel.addEventListener("error", t), this.incomingChannel.addEventListener("error", t), this.outgoingChannel.addEventListener("open", () => {
      this.startedReconnectOutgoing = !1, this.connectionError = "", this.outgoingChannel.addEventListener("message", (n) => {
        const o = this.jsonCoder.deserializeObject(
          JSON.parse(n.data),
          sa
        );
        if (o.isOk())
          this.responseRejectHandler.delete(o.context);
        else {
          console.error(o), this.responseResolvers.delete(o.context);
          const i = this.responseRejectHandler.get(
            o.context
          );
          i ? (this.responseRejectHandler.delete(o.context), i(o)) : console.error(
            "received error receipt for an unknwon context: " + o.context
          );
        }
      });
    }), this.incomingChannel.addEventListener("open", () => {
      this.startedReconnectIncoming = !1, this.connectionError = "", this.incomingChannel.addEventListener("message", (n) => {
        try {
          const o = JSON.parse(n.data), i = this.jsonCoder.deserializeObject(
            o,
            Ka
          );
          this.acknowledgeMessage(i);
          const s = i.getRequestContext();
          if (this.finishedContexts.has(s))
            return;
          if (i.getMessageType() == Ks.Progress) {
            const l = this.jsonCoder.deserializeObject(
              JSON.parse(atob(o.payload)),
              Zo
            ), u = this.progressCallbacks.get(s);
            u && u({
              message: l,
              props: i.properties
            });
          } else {
            if (i.isFragmented())
              if (this.contextResponseFragments.has(s)) {
                let u = this.contextResponseFragments.get(s);
                if (u)
                  if (u.push(i), u.length == i.getNumberOfFragments()) {
                    u = u.sort(
                      (f, p) => f.getFragmentId() - p.getFragmentId()
                    );
                    let c = "";
                    for (const f of u)
                      c += f.payload;
                    i.payload = c, this.contextResponseFragments.delete(s);
                  } else
                    return;
              } else {
                this.contextResponseFragments.set(s, [i]);
                return;
              }
            const l = this.responseResolvers.get(s);
            l ? (this.responseResolvers.delete(s), this.progressCallbacks.delete(s), this.responseRejectHandler.delete(s), this.finishedContexts.add(s), l(i)) : console.warn(
              "received a response for an unknown context: " + s,
              i
            );
          }
        } catch (o) {
          console.error(o);
        }
      });
    });
  }
  checkConnection() {
    if (this.waitingForPong && console.warn("Ping to backend module timed out."), this.startedReconnectOutgoing || this.startedReconnectIncoming)
      return;
    this.waitingForPong = !0;
    const t = Rr.createPingMessage();
    this.sendMessage(t).then(() => {
      this.waitingForPong = !1, this.sendPendingRequests();
    }).catch((n) => {
      console.warn(n.errorMsg);
    });
  }
  /** Send a request to the backend module and retrieve the response
   *
   * This high-level implementation of :meth:`sendMessage` takes an object and
   * sends it as a :class:`DASFModuleRequest` of type
   * :attr:`MessageType.Request` to the backend module (as soon as
   * the websocket connection has been established). The result
   * is a promise that resolves to the response of the backend module.
   *
   * **Example**
   *
   * ```
   * import { DASFConnection, WebsocketUrlBuilder } from '@dasf/dasf-messaging'
   * const connection = new DASFConnection(
   *     new WebsocketUrlBuilder(
   *         'ws://localhost:8080/ws',
   *         'some-topic'
   *     )
   * )
   *
   * // send request via the connection and log the response
   * connection.sendRequest({func_name: 'version_info'}).then(
   *     (response) => console.log(response)
   * )
   * ```
   *
   * @param data - The object that is sent as payload of the request to the
   *   backend module
   * @param onProgress - A callback to use when we receive progress reports
   *   while processing the request in the backend module.
   *
   * @returns The promise resolving to the deserialized response of the backend
   *   module
   */
  async sendRequest(t, n) {
    const o = Rr.createRequestMessage(t);
    return this.sendMessage(o, n).then(
      (i) => i.properties && i.properties.status == "error" ? "Error: " + atob(i.payload) : JSON.parse(atob(i.payload))
    );
  }
  /** Send a generic message to the backend module
   *
   * This low-level implementation takes a generic :class:`DASFModuleRequest`
   * and sends it to the backend module (as soon as the websocket connection
   * has been established). The result is a promise that resolves to the
   * response of the backend module.
   *
   * @param requestMessage - The message you want to send
   * @param onProgress - A callback to use when we receive progress reports
   *   while processing the request in the backend module.
   *
   * @returns The promise resolving to the response of the backend module
   */
  async sendMessage(t, n) {
    if (t.context = String(++this.contextCounter), !t.properties)
      throw new Error("missing request properties");
    t.properties[ry.ResponseTopic] = this.consumeTopic, t.properties[ry.RequestContext] = t.context;
    const {
      promise: o,
      resolve: i,
      reject: s
    } = Promise.withResolvers();
    if (!this.hasConnectionError())
      this.responseResolvers.set(t.context, i), this.responseRejectHandler.set(t.context, s), typeof n < "u" && this.progressCallbacks.set(t.context, n), this.connected.then((l) => {
        const u = JSON.stringify(t);
        l.outgoingChannel.send(u);
      });
    else {
      const l = new sa();
      l.context = t.context, l.errorMsg = this.connectionError, s(l);
    }
    return o;
  }
  acknowledgeMessage(t) {
    const n = this.jsonCoder.serializeObject(
      new Xf(t.messageId)
    );
    this.incomingChannel.send(JSON.stringify(n));
  }
  /** Close all connections
   *
   * This method closes the connections to the outgoing and incoming websocket.
   *
   * **Example**
   *
   * ```
   * import { DASFConnection, WebsocketUrlBuilder } from '@dasf/dasf-messaging'
   * const connection = new DASFConnection(
   *     new WebsocketUrlBuilder(
   *         'ws://localhost:8080/ws',
   *         'some-topic'
   *     )
   * )
   *
   * // send request via the connection and log the response
   * connection.sendRequest({func_name: 'version_info'}).then(
   *     (response) => {
   *       console.log(response)
   *       // close the connection upon receival
   *       connection.close()
   *     }
   * )
   *
   * ```
   */
  close() {
    this.outgoingChannel.close(), this.incomingChannel.close();
  }
  /** Get information on requests for this module
   *
   * This method retrieves the JSONSchema representation for requests to this
   * module. It is one large schema object that can be used to validate a
   * request for the :func:`sendRequest` method.
   *
   * @returns A promise that resolves to the JSONSchema for a request to this topic
   */
  getModuleInfo() {
    return this.sendInfoRequest(
      Rr.createInfoMessage(),
      Ks.Info
    );
  }
  /** Get api information for this backend module
   *
   * This method retrieves the api info, namely the individual JSONSchema
   * representations for the functions and classes in the backend module.
   * Different from :func:`getModuleInfo`, this method does not only provide a
   * single JSONSchema, but rather one JSONSchema for each method/function in
   * the backend module.
   *
   * @returns an object representing the api of the backend module.
   */
  getApiInfo() {
    return this.sendInfoRequest(
      Rr.createApiInfoMessage(),
      Ks.ApiInfo
    );
  }
  sendInfoRequest(t, n = Ks.Info) {
    const { promise: o, resolve: i, reject: s } = Promise.withResolvers();
    return this.sendMessage(t).then((l) => {
      if (l.properties)
        try {
          const u = JSON.parse(l.properties[n]);
          n == Ks.ApiInfo ? i(this.jsonCoder.deserializeObject(u, Ja)) : i(u);
        } catch (u) {
          console.warn(
            `Unable to retrieve backend module ${n}, got: `,
            l.properties[n]
          ), s(u);
        }
    }), o;
  }
  hasConnectionError() {
    return this.connectionError.length > 0;
  }
}
class Cie {
  constructor(t, n, o = "", i = "") {
    Ze(this, "topic"), Ze(this, "consumeTopic"), Ze(this, "DASFConsumerURL"), Ze(this, "DASFProducerURL"), this.topic = n, this.consumeTopic = this.generateConsumeTopic(n), this.DASFConsumerURL = (i || this.buildUrl(t, this.consumeTopic)) + "/" + this.generateSubscriptionPrefix(), this.DASFProducerURL = o || this.buildUrl(t, this.topic);
  }
  buildUrl(t, n) {
    return t + "/" + n;
  }
  generateConsumeTopic(t) {
    return t + "_" + this.generateRequestToken();
  }
  generateSubscriptionPrefix() {
    return "web-frontend-" + (/* @__PURE__ */ new Date()).toISOString();
  }
  generateRequestToken() {
    return Math.random().toString(16).slice(2);
  }
}
const xie = class q_ {
  constructor(t, n, o, i, s = "ws") {
    Ze(this, "topic"), Ze(this, "consumeTopic"), Ze(this, "DASFConsumerURL"), Ze(this, "DASFProducerURL"), this.topic = i, this.consumeTopic = this.generateConsumeTopic(i), this.DASFConsumerURL = this.buildUrl(
      t,
      n,
      o,
      "consumer",
      this.consumeTopic,
      s
    ), this.DASFProducerURL = this.buildUrl(
      t,
      n,
      o,
      "producer",
      this.topic,
      s
    );
  }
  buildUrl(t, n, o, i, s, l) {
    let u = q_.BASE_DASF_URL.replace("%host%", t).replace("%port%", n).replace("%namespace%", o).replace("%type%", i).replace("%topic%", s).replace("%protocol%", l);
    return i == "consumer" && (u += this.generateSubscriptionPrefix() + "?subscriptionType=Exclusive"), u;
  }
  generateConsumeTopic(t) {
    return t + "_" + this.generateRequestToken();
  }
  generateSubscriptionPrefix() {
    return "web-frontend-" + (/* @__PURE__ */ new Date()).toISOString();
  }
  generateRequestToken() {
    return Math.random().toString(16).slice(2);
  }
};
Ze(xie, "BASE_DASF_URL", "%protocol%://%host%:%port%/ws/v2/%type%/non-persistent/public/%namespace%/%topic%/");
function Eie({ report: e }) {
  const t = typeof e.children > "u" ? 0 : e.children.filter(
    (n) => n.status == tl.Success
  ).length;
  return /* @__PURE__ */ A.jsxs(A.Fragment, { children: [
    /* @__PURE__ */ A.jsxs(wr, { sx: { display: "flex", alignItems: "center" }, children: [
      /* @__PURE__ */ A.jsx(wr, { sx: { minWidth: 100 }, children: /* @__PURE__ */ A.jsxs(sr, { variant: "body2", sx: { color: "text.secondary" }, children: [
        "Step ",
        t,
        " of ",
        e.steps
      ] }) }),
      /* @__PURE__ */ A.jsx(wr, { sx: { width: "100%", ml: 1 }, children: /* @__PURE__ */ A.jsx(
        B_,
        {
          variant: "determinate",
          value: t / e.steps * 100
        }
      ) })
    ] }),
    typeof e.children < "u" && e.children.length > 0 && /* @__PURE__ */ A.jsx(wr, { sx: { width: "100%", ml: 1, m: 1 }, children: /* @__PURE__ */ A.jsxs(sr, { variant: "body2", sx: { color: "text.secondary" }, children: [
      e.children[e.children.length - 1].status == tl.Running && /* @__PURE__ */ A.jsx(U_, { size: "1rem", sx: { mr: 2 } }),
      e.children[e.children.length - 1].step_message
    ] }) })
  ] });
}
const _ie = Cr(/* @__PURE__ */ A.jsx("path", {
  d: "M9 16.2 4.8 12l-1.4 1.4L9 19 21 7l-1.4-1.4z"
}), "Done"), kie = {
  50: "#fce4ec",
  100: "#f8bbd0",
  200: "#f48fb1",
  300: "#f06292",
  400: "#ec407a",
  500: "#e91e63",
  600: "#d81b60",
  700: "#c2185b",
  800: "#ad1457",
  900: "#880e4f",
  A100: "#ff80ab",
  A200: "#ff4081",
  A400: "#f50057",
  A700: "#c51162"
}, Pie = Cr(/* @__PURE__ */ A.jsx("path", {
  d: "M1 21h22L12 2zm12-3h-2v-2h2zm0-4h-2v-4h2z"
}), "ReportProblem");
function Oie({ status: e, message: t }) {
  return /* @__PURE__ */ A.jsxs("span", { children: [
    e == tl.Running && /* @__PURE__ */ A.jsx(U_, { size: "1rem", sx: { mr: 2 } }),
    e == tl.Success && /* @__PURE__ */ A.jsx(_ie, { color: "success", sx: { mr: 1 } }),
    e == tl.Error && /* @__PURE__ */ A.jsx(Pie, { sx: { color: kie[500], mr: 1 } }),
    t
  ] });
}
function K_({ reportItem: e }) {
  return /* @__PURE__ */ A.jsxs(wr, { sx: { ml: 2, my: 1 }, children: [
    /* @__PURE__ */ A.jsx(
      Oie,
      {
        status: e.status,
        message: e.step_message
      }
    ),
    typeof e.children < "u" && e.children.map((t, n) => /* @__PURE__ */ A.jsx(K_, { reportItem: t }, n))
  ] });
}
function Aie({ reportItem: e }) {
  return /* @__PURE__ */ A.jsx(wr, { sx: { my: 2 }, children: /* @__PURE__ */ A.jsx(Ml, { elevation: 3, children: /* @__PURE__ */ A.jsx(wr, { sx: { p: 2 }, children: /* @__PURE__ */ A.jsx(K_, { reportItem: e }) }) }) });
}
function Iie({ children: e }) {
  return /* @__PURE__ */ A.jsx(A.Fragment, { children: e.map((t, n) => /* @__PURE__ */ A.jsx(Aie, { reportItem: t }, n)) });
}
function jie(e) {
  return ht("MuiCollapse", e);
}
dt("MuiCollapse", ["root", "horizontal", "vertical", "entered", "hidden", "wrapper", "wrapperInner"]);
const Rie = (e) => {
  const {
    orientation: t,
    classes: n
  } = e, o = {
    root: ["root", `${t}`],
    entered: ["entered"],
    hidden: ["hidden"],
    wrapper: ["wrapper", `${t}`],
    wrapperInner: ["wrapperInner", `${t}`]
  };
  return mt(o, jie, n);
}, Tie = Oe("div", {
  name: "MuiCollapse",
  slot: "Root",
  overridesResolver: (e, t) => {
    const {
      ownerState: n
    } = e;
    return [t.root, t[n.orientation], n.state === "entered" && t.entered, n.state === "exited" && !n.in && n.collapsedSize === "0px" && t.hidden];
  }
})(ct(({
  theme: e
}) => ({
  height: 0,
  overflow: "hidden",
  transition: e.transitions.create("height"),
  variants: [{
    props: {
      orientation: "horizontal"
    },
    style: {
      height: "auto",
      width: 0,
      transition: e.transitions.create("width")
    }
  }, {
    props: {
      state: "entered"
    },
    style: {
      height: "auto",
      overflow: "visible"
    }
  }, {
    props: {
      state: "entered",
      orientation: "horizontal"
    },
    style: {
      width: "auto"
    }
  }, {
    props: ({
      ownerState: t
    }) => t.state === "exited" && !t.in && t.collapsedSize === "0px",
    style: {
      visibility: "hidden"
    }
  }]
}))), Mie = Oe("div", {
  name: "MuiCollapse",
  slot: "Wrapper",
  overridesResolver: (e, t) => t.wrapper
})({
  // Hack to get children with a negative margin to not falsify the height computation.
  display: "flex",
  width: "100%",
  variants: [{
    props: {
      orientation: "horizontal"
    },
    style: {
      width: "auto",
      height: "100%"
    }
  }]
}), Nie = Oe("div", {
  name: "MuiCollapse",
  slot: "WrapperInner",
  overridesResolver: (e, t) => t.wrapperInner
})({
  width: "100%",
  variants: [{
    props: {
      orientation: "horizontal"
    },
    style: {
      width: "auto",
      height: "100%"
    }
  }]
}), na = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiCollapse"
  }), {
    addEndListener: i,
    children: s,
    className: l,
    collapsedSize: u = "0px",
    component: c,
    easing: f,
    in: p,
    onEnter: g,
    onEntered: h,
    onEntering: y,
    onExit: v,
    onExited: C,
    onExiting: $,
    orientation: S = "vertical",
    style: w,
    timeout: _ = mE.standard,
    // eslint-disable-next-line react/prop-types
    TransitionComponent: b = jo,
    ...k
  } = o, E = {
    ...o,
    orientation: S,
    collapsedSize: u
  }, j = Rie(E), L = Tl(), B = r0(), O = z.useRef(null), V = z.useRef(), W = typeof u == "number" ? `${u}px` : u, q = S === "horizontal", Q = q ? "width" : "height", X = z.useRef(null), ce = _n(n, X), J = (Z) => (me) => {
    if (Z) {
      const ue = X.current;
      me === void 0 ? Z(ue) : Z(ue, me);
    }
  }, ie = () => O.current ? O.current[q ? "clientWidth" : "clientHeight"] : 0, re = J((Z, me) => {
    O.current && q && (O.current.style.position = "absolute"), Z.style[Q] = W, g && g(Z, me);
  }), Y = J((Z, me) => {
    const ue = ie();
    O.current && q && (O.current.style.position = "");
    const {
      duration: Ce,
      easing: $e
    } = fl({
      style: w,
      timeout: _,
      easing: f
    }, {
      mode: "enter"
    });
    if (_ === "auto") {
      const Te = L.transitions.getAutoHeightDuration(ue);
      Z.style.transitionDuration = `${Te}ms`, V.current = Te;
    } else
      Z.style.transitionDuration = typeof Ce == "string" ? Ce : `${Ce}ms`;
    Z.style[Q] = `${ue}px`, Z.style.transitionTimingFunction = $e, y && y(Z, me);
  }), T = J((Z, me) => {
    Z.style[Q] = "auto", h && h(Z, me);
  }), H = J((Z) => {
    Z.style[Q] = `${ie()}px`, v && v(Z);
  }), G = J(C), N = J((Z) => {
    const me = ie(), {
      duration: ue,
      easing: Ce
    } = fl({
      style: w,
      timeout: _,
      easing: f
    }, {
      mode: "exit"
    });
    if (_ === "auto") {
      const $e = L.transitions.getAutoHeightDuration(me);
      Z.style.transitionDuration = `${$e}ms`, V.current = $e;
    } else
      Z.style.transitionDuration = typeof ue == "string" ? ue : `${ue}ms`;
    Z.style[Q] = W, Z.style.transitionTimingFunction = Ce, $ && $(Z);
  }), U = (Z) => {
    _ === "auto" && B.start(V.current || 0, Z), i && i(X.current, Z);
  };
  return /* @__PURE__ */ A.jsx(b, {
    in: p,
    onEnter: re,
    onEntered: T,
    onEntering: Y,
    onExit: H,
    onExited: G,
    onExiting: N,
    addEndListener: U,
    nodeRef: X,
    timeout: _ === "auto" ? null : _,
    ...k,
    children: (Z, {
      ownerState: me,
      ...ue
    }) => /* @__PURE__ */ A.jsx(Tie, {
      as: c,
      className: Ve(j.root, l, {
        entered: j.entered,
        exited: !p && W === "0px" && j.hidden
      }[Z]),
      style: {
        [q ? "minWidth" : "minHeight"]: W,
        ...w
      },
      ref: ce,
      ownerState: {
        ...E,
        state: Z
      },
      ...ue,
      children: /* @__PURE__ */ A.jsx(Mie, {
        ownerState: {
          ...E,
          state: Z
        },
        className: j.wrapper,
        ref: O,
        children: /* @__PURE__ */ A.jsx(Nie, {
          ownerState: {
            ...E,
            state: Z
          },
          className: j.wrapperInner,
          children: s
        })
      })
    })
  });
});
na && (na.muiSupportAuto = !0);
function Fie({ report: e }) {
  const [t, n] = z.useState(!1);
  return /* @__PURE__ */ A.jsxs(A.Fragment, { children: [
    /* @__PURE__ */ A.jsx(Eie, { report: e }),
    typeof e.children < "u" && /* @__PURE__ */ A.jsxs(A.Fragment, { children: [
      /* @__PURE__ */ A.jsx(hc, { variant: "text", onClick: () => n(!t), children: "Show details" }),
      /* @__PURE__ */ A.jsx(na, { in: t, timeout: "auto", unmountOnExit: !0, children: /* @__PURE__ */ A.jsx(Iie, { children: e.children }) })
    ] })
  ] });
}
function Ev({
  connection: e,
  websocketUrl: t,
  topic: n
}) {
  return typeof e > "u" ? new $ie(
    // @ts-expect-error:next-line
    new Cie(t, n)
  ) : typeof e == "function" ? e() : e;
}
const Ba = sG({
  createStyledComponent: Oe("div", {
    name: "MuiGrid2",
    slot: "Root",
    overridesResolver: (e, t) => {
      const {
        ownerState: n
      } = e;
      return [t.root, n.container && t.container];
    }
  }),
  componentName: "MuiGrid2",
  useThemeProps: (e) => bt({
    props: e,
    name: "MuiGrid2"
  }),
  useTheme: Tl
});
var ra = /* @__PURE__ */ ((e) => (e.Rows = "rows", e.Columns = "columns", e))(ra || {});
function _v({
  schemaElement: e,
  schema: t,
  uiSchema: n,
  outputDescription: o,
  connection: i,
  websocketUrl: s,
  topic: l,
  functionName: u,
  onResponse: c,
  skipDefaultResponseHandler: f = !1,
  skipDefaultResponseHandlerCheck: p,
  onError: g,
  skipDefaultErrorHandler: h = !1,
  skipDefaultErrorHandlerCheck: y,
  onProgress: v,
  skipDefaultProgressHandler: C = !1,
  skipDefaultProgressHandlerCheck: $,
  fields: S,
  widgets: w,
  constructorData: _,
  renderStringAsHtml: b = !1,
  layout: k = ra.Columns
}) {
  var me;
  typeof e < "u" && (t = JSON.parse(
    // @ts-expect-error:next-line
    (me = document.getElementById(e)) == null ? void 0 : me.textContent
  ));
  const [E, j] = z.useState(null), [L, B] = z.useState(null), [O, V] = z.useState(!1), [W, q] = z.useState(t), [Q, X] = z.useState(o), ce = z.useRef(null);
  function J(ue) {
    return typeof p > "u" ? f : p(ue);
  }
  function ie(ue) {
    return typeof $ > "u" ? C : $(ue);
  }
  function re(ue) {
    return typeof y > "u" ? h : y(ue);
  }
  const Y = (ue) => {
    j(ue);
  }, T = (ue) => {
    B(ue.message);
  }, H = (ue) => {
    typeof v > "u" ? T(ue) : (v(ue), ie(ue) || T(ue));
  }, G = (ue) => {
    V(!1), ce.current.disabled = !1, ce.current.classList.remove("Mui-disabled"), typeof c > "u" ? Y(ue) : (c(ue), J(ue) || Y(ue));
  }, N = (ue) => {
    X("Request failed!"), typeof g > "u" ? console.error(ue) : (g(ue), re(ue) || console.error("miao!", ue));
  }, U = Ev({
    connection: i,
    websocketUrl: s,
    topic: l
  }), Z = ({ formData: ue }, Ce) => {
    V(!0), j(null), X(o), ce.current.disabled = !0, ce.current.classList.add("Mui-disabled"), typeof _ < "u" && (ue = { ..._, function: ue }), U.sendRequest(ue, H).then(G).catch(N);
  };
  return typeof n > "u" && (n = {}), n["ui:submitButtonOptions"] = {
    props: { ref: ce }
    // don't use the disabled property here because this would trigger a rerender on every submit
  }, typeof W > "u" && U.getApiInfo().then((ue) => {
    q(typeof u > "u" ? ue.functions[0].rpcSchema : ue.functions.filter((Ce) => Ce.name == u)[0].rpcSchema);
  }), /* @__PURE__ */ A.jsx(A.Fragment, { children: /* @__PURE__ */ A.jsxs(Ba, { container: !0, spacing: 2, children: [
    /* @__PURE__ */ A.jsxs(
      Ba,
      {
        size: {
          xs: 12,
          md: k == ra.Rows ? 12 : L || E ? 6 : 12
        },
        children: [
          W && /* @__PURE__ */ A.jsx(
            D_,
            {
              schema: W,
              uiSchema: n,
              onSubmit: Z,
              fields: S,
              widgets: w
            }
          ),
          O && /* @__PURE__ */ A.jsx(wr, { sx: { mt: 2 }, children: /* @__PURE__ */ A.jsx(B_, {}) })
        ]
      }
    ),
    (L || E != null) && /* @__PURE__ */ A.jsxs(Ba, { size: { xs: 12, md: k == ra.Rows ? 12 : 6 }, children: [
      L && /* @__PURE__ */ A.jsx(Fie, { report: L }),
      E != null && /* @__PURE__ */ A.jsx(
        Xoe,
        {
          content: E,
          description: Q,
          renderStringAsHtml: b
        }
      )
    ] })
  ] }) });
}
function Lie(e) {
  return ht("MuiCard", e);
}
dt("MuiCard", ["root"]);
const Die = (e) => {
  const {
    classes: t
  } = e;
  return mt({
    root: ["root"]
  }, Lie, t);
}, zie = Oe(Ml, {
  name: "MuiCard",
  slot: "Root",
  overridesResolver: (e, t) => t.root
})({
  overflow: "hidden"
}), H_ = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiCard"
  }), {
    className: i,
    raised: s = !1,
    ...l
  } = o, u = {
    ...o,
    raised: s
  }, c = Die(u);
  return /* @__PURE__ */ A.jsx(zie, {
    className: Ve(c.root, i),
    elevation: s ? 8 : void 0,
    ref: n,
    ownerState: u,
    ...l
  });
});
function Bie(e) {
  return ht("MuiCardActionArea", e);
}
const ug = dt("MuiCardActionArea", ["root", "focusVisible", "focusHighlight"]), Uie = (e) => {
  const {
    classes: t
  } = e;
  return mt({
    root: ["root"],
    focusHighlight: ["focusHighlight"]
  }, Bie, t);
}, Vie = Oe(Tc, {
  name: "MuiCardActionArea",
  slot: "Root",
  overridesResolver: (e, t) => t.root
})(ct(({
  theme: e
}) => ({
  display: "block",
  textAlign: "inherit",
  borderRadius: "inherit",
  // for Safari to work https://github.com/mui/material-ui/issues/36285.
  width: "100%",
  [`&:hover .${ug.focusHighlight}`]: {
    opacity: (e.vars || e).palette.action.hoverOpacity,
    "@media (hover: none)": {
      opacity: 0
    }
  },
  [`&.${ug.focusVisible} .${ug.focusHighlight}`]: {
    opacity: (e.vars || e).palette.action.focusOpacity
  }
}))), Wie = Oe("span", {
  name: "MuiCardActionArea",
  slot: "FocusHighlight",
  overridesResolver: (e, t) => t.focusHighlight
})(ct(({
  theme: e
}) => ({
  overflow: "hidden",
  pointerEvents: "none",
  position: "absolute",
  top: 0,
  right: 0,
  bottom: 0,
  left: 0,
  borderRadius: "inherit",
  opacity: 0,
  backgroundColor: "currentcolor",
  transition: e.transitions.create("opacity", {
    duration: e.transitions.duration.short
  })
}))), G_ = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiCardActionArea"
  }), {
    children: i,
    className: s,
    focusVisibleClassName: l,
    ...u
  } = o, c = o, f = Uie(c);
  return /* @__PURE__ */ A.jsxs(Vie, {
    className: Ve(f.root, s),
    focusVisibleClassName: Ve(l, f.focusVisible),
    ref: n,
    ownerState: c,
    ...u,
    children: [i, /* @__PURE__ */ A.jsx(Wie, {
      className: f.focusHighlight,
      ownerState: c
    })]
  });
});
function qie(e) {
  return ht("MuiCardContent", e);
}
dt("MuiCardContent", ["root"]);
const Kie = (e) => {
  const {
    classes: t
  } = e;
  return mt({
    root: ["root"]
  }, qie, t);
}, Hie = Oe("div", {
  name: "MuiCardContent",
  slot: "Root",
  overridesResolver: (e, t) => t.root
})({
  padding: 16,
  "&:last-child": {
    paddingBottom: 24
  }
}), J_ = /* @__PURE__ */ z.forwardRef(function(t, n) {
  const o = bt({
    props: t,
    name: "MuiCardContent"
  }), {
    className: i,
    component: s = "div",
    ...l
  } = o, u = {
    ...o,
    component: s
  }, c = Kie(u);
  return /* @__PURE__ */ A.jsx(Hie, {
    as: s,
    className: Ve(c.root, i),
    ownerState: u,
    ref: n,
    ...l
  });
}), oy = Cr([/* @__PURE__ */ A.jsx("path", {
  d: "M17.59 18 19 16.59 14.42 12 19 7.41 17.59 6l-6 6z"
}, "0"), /* @__PURE__ */ A.jsx("path", {
  d: "m11 18 1.41-1.41L7.83 12l4.58-4.59L11 6l-6 6z"
}, "1")], "KeyboardDoubleArrowLeft");
function Y_({
  apiInfoElement: e,
  apiInfo: t,
  uiSchema: n,
  connection: o,
  websocketUrl: i,
  topic: s,
  className: l,
  fields: u,
  widgets: c,
  layout: f = ra.Columns,
  ...p
}) {
  var b;
  const [g, h] = z.useState(-1), [y, v] = z.useState({}), C = Ev({
    connection: o,
    websocketUrl: i,
    topic: s
  });
  typeof e < "u" && (t = C.jsonCoder.deserializeObject(
    JSON.parse(
      // @ts-expect-error:next-line
      (b = document.getElementById(e)) == null ? void 0 : b.textContent
    ),
    Ga
  ));
  const [$, S] = z.useState(
    t
  );
  typeof n > "u" && (n = {});
  const w = {
    ...n,
    "ui:submitButtonOptions": { norender: !0 },
    class_name: { "ui:widget": "hidden" }
  };
  typeof $ > "u" && C.getApiInfo().then((k) => {
    S(typeof l > "u" ? k.classes[0] : k.classes.filter((E) => E.name == l)[0]);
  });
  const _ = ({ formData: k }) => {
    v(k);
  };
  return /* @__PURE__ */ A.jsx(A.Fragment, { children: $ && /* @__PURE__ */ A.jsxs(A.Fragment, { children: [
    /* @__PURE__ */ A.jsx(
      D_,
      {
        schema: $.rpcSchema,
        uiSchema: w,
        onChange: _,
        fields: u,
        widgets: c
      }
    ),
    /* @__PURE__ */ A.jsx(na, { in: g == -1, timeout: "auto", unmountOnExit: !0, children: /* @__PURE__ */ A.jsx(Ba, { container: !0, spacing: { xs: 2, md: 3 }, children: $.methods.map(
      (k, E) => /* @__PURE__ */ A.jsx(
        Ba,
        {
          size: { xs: 12, md: f == ra.Rows ? 12 : 6 },
          children: /* @__PURE__ */ A.jsx(H_, { variant: "outlined", children: /* @__PURE__ */ A.jsx(
            G_,
            {
              onClick: () => h(E),
              "data-active": void 0,
              sx: {
                height: "100%",
                "&[data-active]": {
                  backgroundColor: "action.selected",
                  "&:hover": {
                    backgroundColor: "action.selectedHover"
                  }
                }
              },
              children: /* @__PURE__ */ A.jsxs(J_, { children: [
                /* @__PURE__ */ A.jsx(
                  sr,
                  {
                    variant: "h5",
                    component: "div",
                    sx: { mb: 2 },
                    children: k.name
                  }
                ),
                k.rpcSchema.description && /* @__PURE__ */ A.jsx(
                  sr,
                  {
                    variant: "body2",
                    sx: { color: "text.secondary" },
                    children: k.rpcSchema.description
                  }
                )
              ] })
            }
          ) })
        },
        k.name
      )
    ) }) }),
    $.methods.map(
      (k, E) => /* @__PURE__ */ A.jsxs(
        na,
        {
          in: E == g,
          timeout: "auto",
          unmountOnExit: !0,
          children: [
            /* @__PURE__ */ A.jsx(
              hc,
              {
                variant: "text",
                onClick: () => h(-1),
                startIcon: /* @__PURE__ */ A.jsx(oy, {}),
                children: "All functions"
              }
            ),
            /* @__PURE__ */ A.jsx(
              _v,
              {
                schema: k.rpcSchema,
                uiSchema: n,
                outputDescription: k.returnSchema.description,
                connection: C,
                constructorData: y,
                fields: u,
                widgets: c,
                layout: f,
                ...p
              },
              k.name
            )
          ]
        },
        k.name
      )
    )
  ] }) });
}
function Gie(e) {
  const t = z.useRef();
  return z.useEffect(() => {
    t.current = e;
  }, [e]), t.current;
}
function Jie({
  apiInfoElement: e,
  apiInfo: t,
  uiSchema: n,
  member: o,
  connection: i,
  websocketUrl: s,
  topic: l,
  layout: u = ra.Columns,
  updateUrl: c = !1,
  ...f
}) {
  var $;
  const [p, g] = z.useState(
    typeof o > "u" ? "" : o
  ), h = Gie(p), y = Ev({
    connection: i,
    websocketUrl: s,
    topic: l
  });
  typeof e < "u" && (t = y.jsonCoder.deserializeObject(
    JSON.parse(
      // @ts-expect-error:next-line
      ($ = document.getElementById(e)) == null ? void 0 : $.textContent
    ),
    Ja
  ));
  const [v, C] = z.useState(
    t
  );
  return typeof v > "u" && y.getApiInfo().then((S) => {
    C(S);
  }), z.useEffect(() => {
    if (c && h != p && typeof h < "u") {
      const S = new URL(
        (h ? "../" : "") + p + (p ? "/" : ""),
        window.location.href
      );
      history.replaceState(null, "", S);
    }
  }, [p, h, c]), /* @__PURE__ */ A.jsxs(A.Fragment, { children: [
    v && /* @__PURE__ */ A.jsx(na, { in: !p, timeout: "auto", unmountOnExit: !0, children: /* @__PURE__ */ A.jsx(Ba, { container: !0, spacing: { xs: 2, md: 3 }, children: v.functions.concat(v.classes).map((S) => /* @__PURE__ */ A.jsx(
      Ba,
      {
        size: { xs: 12, md: u == ra.Rows ? 12 : 6 },
        children: /* @__PURE__ */ A.jsx(H_, { variant: "outlined", children: /* @__PURE__ */ A.jsx(
          G_,
          {
            onClick: () => g(S.name),
            "data-active": void 0,
            sx: {
              height: "100%",
              "&[data-active]": {
                backgroundColor: "action.selected",
                "&:hover": {
                  backgroundColor: "action.selectedHover"
                }
              }
            },
            children: /* @__PURE__ */ A.jsxs(J_, { children: [
              /* @__PURE__ */ A.jsx(sr, { variant: "h5", component: "div", sx: { mb: 2 }, children: S.name }),
              S.rpcSchema.description && /* @__PURE__ */ A.jsx(
                sr,
                {
                  variant: "body2",
                  sx: { color: "text.secondary" },
                  children: S.rpcSchema.description
                }
              )
            ] })
          }
        ) })
      },
      S.name
    )) }) }),
    v && v.functions.map((S) => /* @__PURE__ */ A.jsxs(
      na,
      {
        in: p == S.name,
        timeout: "auto",
        unmountOnExit: !0,
        children: [
          /* @__PURE__ */ A.jsx(
            hc,
            {
              variant: "text",
              onClick: () => g(""),
              startIcon: /* @__PURE__ */ A.jsx(oy, {}),
              children: "All functions"
            }
          ),
          /* @__PURE__ */ A.jsx(
            _v,
            {
              schema: S.rpcSchema,
              uiSchema: n,
              outputDescription: S.returnSchema.description,
              connection: y,
              layout: u,
              ...f
            },
            S.name
          )
        ]
      },
      S.name
    )),
    v && v.classes.map((S) => /* @__PURE__ */ A.jsxs(
      na,
      {
        in: S.name == p,
        timeout: "auto",
        unmountOnExit: !0,
        children: [
          /* @__PURE__ */ A.jsx(
            hc,
            {
              variant: "text",
              onClick: () => g(""),
              startIcon: /* @__PURE__ */ A.jsx(oy, {}),
              children: "All functions"
            }
          ),
          /* @__PURE__ */ A.jsx(
            Y_,
            {
              apiInfo: S,
              uiSchema: n,
              connection: y,
              layout: u,
              ...f
            },
            S.name
          )
        ]
      },
      S.name
    ))
  ] });
}
const Yie = ay(_v, {
  props: {
    schemaElement: "string",
    schema: "json",
    uiSchema: "json",
    connection: "function",
    websocketUrl: "string",
    topic: "string",
    outputDescription: "string",
    functionName: "string",
    skipDefaultResponseHandler: "boolean",
    skipDefaultResponseHandlerCheck: "function",
    skipDefaultErrorHandler: "boolean",
    skipDefaultErrorHandlerCheck: "function",
    skipDefaultProgressHandler: "boolean",
    skipDefaultProgressHandlerCheck: "function",
    constructorData: "json",
    renderStringAsHtml: "boolean",
    layout: "string"
  },
  events: {
    onResponse: {},
    onError: {},
    onProgress: {}
  }
}), Qie = ay(Y_, {
  props: {
    apiInfoElement: "string",
    apiInfo: "json",
    uiSchema: "json",
    connection: "function",
    websocketUrl: "string",
    topic: "string",
    className: "string",
    skipDefaultResponseHandler: "boolean",
    skipDefaultResponseHandlerCheck: "function",
    skipDefaultErrorHandler: "boolean",
    skipDefaultErrorHandlerCheck: "function",
    skipDefaultProgressHandler: "boolean",
    skipDefaultProgressHandlerCheck: "function",
    renderStringAsHtml: "boolean",
    layout: "string"
  },
  events: {
    onResponse: {},
    onError: {},
    onProgress: {}
  }
}), Xie = ay(Jie, {
  props: {
    apiInfoElement: "string",
    apiInfo: "json",
    member: "string",
    connection: "function",
    websocketUrl: "string",
    topic: "string",
    skipDefaultResponseHandler: "boolean",
    skipDefaultResponseHandlerCheck: "function",
    skipDefaultErrorHandler: "boolean",
    skipDefaultErrorHandlerCheck: "function",
    skipDefaultProgressHandler: "boolean",
    skipDefaultProgressHandlerCheck: "function",
    renderStringAsHtml: "boolean",
    layout: "string",
    updateUrl: "boolean"
  },
  events: {
    onResponse: {},
    onError: {},
    onProgress: {}
  }
});
customElements.define("dasf-function", Yie);
customElements.define("dasf-class", Qie);
customElements.define("dasf-module", Xie);
